VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Backup"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 1998-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Private oParent As Object

'''

Const FUNC_SAVE = "Save"
Const FUNC_SAVEREPLACE = "SaveReplace"
Const FUNC_SAVEIGNORE = "SaveIgnore"
Const FUNC_SAVECANCEL = "SaveCancel"

Const FUNC_RESTORE = "Restore"
Const FUNC_RESTOREREPLACE = "RestoreReplace"
Const FUNC_RESTOREIGNORE = "RestoreIgnore"
Const FUNC_RESTORECANCEL = "RestoreCancel"

Const FUNC_CHECK = "Check"
Const FUNC_CHECKREPLACE = "CheckReplace"
Const FUNC_CHECKIGNORE = "CheckIgnore"
Const FUNC_CHECKCANCEL = "CheckCancel"

'''

Const RC_OK = 0
Const RC_NOLOG = -123
Const RC_IOERROR = -903
Const RC_WRONGVOL = -3004
Const RC_SKIPPED = -7075
Const RC_VOLEND = -7076
Const RC_WRONGLOG = -8003
Const RC_NEXTVOL = -8020

'''

Private oDatabase As DBMLib.Database

Private oServerObject As DBMSOB.ServerObject

Private oState As DBMApp.BackupState

'''

Private oMedia As Object
Private oMedium As Object

Private lBackupType As DBMBackupTypeConstants

Private lSaveFor As DBMBackupForConstants

Private lFileNumber As Long
Private bUseUntilDate As Boolean
Private dUntilDate As Date

'''

Private bDBMConnected As Boolean
Private bUTLConnected As Boolean
Private bSRVConnected As Boolean

'''

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Public Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As DBMLib.Database
    Set Database = oDatabase
End Property
Public Property Set Database(NewVal As DBMLib.Database)
    Set oDatabase = NewVal
End Property

'''

'
' Save
'
Public Function Save(Media As Object, _
                     ByVal SaveType As DBMBackupTypeConstants, _
                     Optional ByVal SaveFor As DBMBackupForConstants, _
                     Optional ByVal Asynchron As Boolean) As Boolean
    #If DBG Then
    T "--> Save()"
    #End If
                     
    Save = False
    
    If oDatabase Is Nothing Then Exit Function
    
    Set oMedia = Media
    lBackupType = SaveType
    lSaveFor = SaveFor
    
    If Not Asynchron Then
        Save = Execute(FUNC_SAVE)
    Else
        modBackup.Execute Me, FUNC_SAVE
        Save = True
    End If
    
    #If DBG Then
    T "<-- Save() = " & Save
    #End If
End Function

Public Function SaveReplace(Medium As DBMLib.Medium, _
                            Optional ByVal Asynchron As Boolean = False) As Boolean
    #If DBG Then
    T "--> SaveReplace()"
    #End If
    
    SaveReplace = False
    
    If oDatabase Is Nothing Then Exit Function
    If Not bDBMConnected Then Exit Function
    If Not bUTLConnected Then Exit Function
    
    Set oMedium = Medium
    
    If Not Asynchron Then
        SaveReplace = Execute(FUNC_SAVEREPLACE)
    Else
        modBackup.Execute Me, FUNC_SAVEREPLACE
        SaveReplace = True
    End If
    
    #If DBG Then
    T "<-- SaveReplace() = " & SaveReplace
    #End If
End Function

Public Function SaveIgnore(Medium As DBMLib.Medium, _
                           Optional ByVal Asynchron As Boolean) As Boolean
    #If DBG Then
    T "--> SaveIgnore()"
    #End If
   
    SaveIgnore = False
    
    If oDatabase Is Nothing Then Exit Function
    If Not bDBMConnected Then Exit Function
    If Not bUTLConnected Then Exit Function
    
    Set oMedium = Medium
    
    If Not Asynchron Then
        SaveIgnore = Execute(FUNC_SAVEIGNORE)
    Else
        modBackup.Execute Me, FUNC_SAVEIGNORE
        SaveIgnore = True
    End If
    
    #If DBG Then
    T "<-- SaveIgnore() = " & SaveIgnore
    #End If
End Function

Public Function SaveCancel(Optional ByVal Asynchron As Boolean) As Boolean
    #If DBG Then
    T "--> SaveCancel()"
    #End If
    
    SaveCancel = False
    
    If oDatabase Is Nothing Then Exit Function
    If Not bDBMConnected Then Exit Function
    If Not bUTLConnected Then Exit Function
    
    If Not Asynchron Then
        SaveCancel = Execute(FUNC_SAVECANCEL)
    Else
        modBackup.Execute Me, FUNC_SAVECANCEL
        SaveCancel = True
    End If
    
    #If DBG Then
    T "<-- SaveCancel() = " & SaveCancel
    #End If
End Function

'''

'
' Restore
'
Public Function Restore(Media As Object, _
                        ByVal RestoreType As DBMLib.DBMBackupTypeConstants, _
                        Optional ByVal FileNumber As Long = 0, _
                        Optional ByVal UseUntilDate As Boolean = False, _
                        Optional ByVal UntilDate As Date = Empty, _
                        Optional ByVal Asynchron As Boolean = False) As Boolean
    #If DBG Then
    T "--> Restore()"
    #End If
                        
    Restore = False
    
    If oDatabase Is Nothing Then Exit Function
    
    Set oMedia = Media
    lBackupType = RestoreType
    lFileNumber = FileNumber
    bUseUntilDate = UseUntilDate
    dUntilDate = UntilDate
    
    If Not Asynchron Then
        Restore = Execute(FUNC_RESTORE)
    Else
        modBackup.Execute Me, FUNC_RESTORE
        Restore = True
    End If
    
    #If DBG Then
    T "<-- Restore() = " & Restore
    #End If
End Function

Public Function RestoreReplace(Medium As DBMLib.Medium, _
                               Optional ByVal FileNumber As Long = 0, _
                               Optional ByVal Asynchron As Boolean = False) As Boolean
    #If DBG Then
    T "--> RestoreReplace()"
    #End If
    
    RestoreReplace = False
    
    If oDatabase Is Nothing Then Exit Function
    If Not bDBMConnected Then Exit Function
    If Not bUTLConnected Then Exit Function
    
    Set oMedium = Medium
    lFileNumber = FileNumber
    
    If Not Asynchron Then
        RestoreReplace = Execute(FUNC_RESTOREREPLACE)
    Else
        modBackup.Execute Me, FUNC_RESTOREREPLACE
        RestoreReplace = True
    End If
    
    #If DBG Then
    T "<-- RestoreReplace() = " & RestoreReplace
    #End If
End Function

Public Function RestoreIgnore(Medium As DBMLib.Medium, _
                              Optional ByVal Asynchron As Boolean = False) As Boolean
    #If DBG Then
    T "--> RestoreIgnore()"
    #End If
    
    RestoreIgnore = False
    
    If oDatabase Is Nothing Then Exit Function
    If Not bDBMConnected Then Exit Function
    If Not bUTLConnected Then Exit Function
    
    Set oMedium = Medium
    
    If Not Asynchron Then
        RestoreIgnore = Execute(FUNC_RESTOREIGNORE)
    Else
        modBackup.Execute Me, FUNC_RESTOREIGNORE
        RestoreIgnore = True
    End If
    
    #If DBG Then
    T "<-- RestoreIgnore() = " & RestoreIgnore
    #End If
End Function

Public Function RestoreCancel(Optional ByVal Asynchron As Boolean = False) As Boolean
    #If DBG Then
    T "--> RestoreCancel()"
    #End If
    
    RestoreCancel = False
    
    If oDatabase Is Nothing Then Exit Function
'    If Not bDBMConnected Then Exit Function
'    If Not bUTLConnected Then Exit Function
    
    If Not Asynchron Then
        RestoreCancel = Execute(FUNC_RESTORECANCEL)
    Else
        modBackup.Execute Me, FUNC_RESTORECANCEL
        RestoreCancel = True
    End If
    
    #If DBG Then
    T "<-- RestoreCancel() = " & RestoreCancel
    #End If
End Function
'
' Check
'
Public Function Check(Media As Object, _
                      ByVal CheckType As DBMLib.DBMBackupTypeConstants, _
                      Optional ByVal FileNumber As Long, _
                      Optional ByVal Asynchron As Boolean = False) As Boolean
    #If DBG Then
    T "--> Check()"
    #End If
    
    Check = False
    
    If oDatabase Is Nothing Then Exit Function
    
    Set oMedia = Media
    lBackupType = CheckType
    lFileNumber = FileNumber
    
    If Not Asynchron Then
        Check = Execute(FUNC_CHECK)
    Else
        modBackup.Execute Me, FUNC_CHECK
        Check = True
    End If
    
    #If DBG Then
    T "<-- Check() = " & Check
    #End If
End Function

Public Function CheckReplace(Medium As DBMLib.Medium, _
                             Optional ByVal FileNumber As Long = 0, _
                             Optional ByVal Asynchron As Boolean = False) As Boolean
    #If DBG Then
    T "--> CheckReplace()"
    #End If
    
    CheckReplace = False
    
    If oDatabase Is Nothing Then Exit Function
    If Not bDBMConnected Then Exit Function
    If Not bSRVConnected Then Exit Function
    
    Set oMedium = Medium
    lFileNumber = FileNumber
    
    If Not Asynchron Then
        CheckReplace = Execute(FUNC_CHECKREPLACE)
    Else
        modBackup.Execute Me, FUNC_CHECKREPLACE
        CheckReplace = True
    End If
    
    #If DBG Then
    T "<-- CheckReplace() = " & CheckReplace
    #End If
End Function

Public Function CheckIgnore(Medium As DBMLib.Medium, _
                            Optional ByVal Asynchron As Boolean = False) As Boolean
    #If DBG Then
    T "--> CheckIgnore()"
    #End If
    
    CheckIgnore = False
    
    If oDatabase Is Nothing Then Exit Function
    If Not bDBMConnected Then Exit Function
    If Not bSRVConnected Then Exit Function
    
    Set oMedium = Medium
    
    If Not Asynchron Then
        CheckIgnore = Execute(FUNC_CHECKIGNORE)
    Else
        modBackup.Execute Me, FUNC_CHECKIGNORE
        CheckIgnore = True
    End If
    
    #If DBG Then
    T "<-- CheckIgnore() = " & CheckIgnore
    #End If
End Function

Public Function CheckCancel(Optional ByVal Asynchron As Boolean = False) As Boolean
    #If DBG Then
    T "--> CheckCancel()"
    #End If
    
    CheckCancel = False
    
    If oDatabase Is Nothing Then Exit Function
    If Not bDBMConnected Then Exit Function
    If Not bSRVConnected Then Exit Function
    
    If Not Asynchron Then
        CheckCancel = Execute(FUNC_CHECKCANCEL)
    Else
        modBackup.Execute Me, FUNC_CHECKCANCEL
        CheckCancel = True
    End If
    
    #If DBG Then
    T "<-- CheckCancel() = " & CheckCancel
    #End If
End Function

'''

Public Sub Cancel()
    oServerObject.Cancel
End Sub

'''

Public Function Execute(Optional ByVal FunctionName As String) As Boolean
    #If DBG Then
    T "--> Execute(" & FunctionName & ")"
    #End If
    
    Dim bRC As Boolean
    
    bRC = False
    
    Select Case FunctionName
        Case FUNC_SAVE:             bRC = ExecuteSave()
        Case FUNC_SAVEREPLACE:      bRC = ExecuteSaveReplace()
        Case FUNC_SAVEIGNORE:       bRC = ExecuteSaveIgnore()
        Case FUNC_SAVECANCEL:       bRC = ExecuteSaveCancel()
        
        Case FUNC_RESTORE:          bRC = ExecuteRestore()
        Case FUNC_RESTOREREPLACE:   bRC = ExecuteRestoreReplace()
        Case FUNC_RESTOREIGNORE:    bRC = ExecuteRestoreIgnore()
        Case FUNC_RESTORECANCEL:    bRC = ExecuteRestoreCancel()
    
        Case FUNC_CHECK:            bRC = ExecuteCheck()
        Case FUNC_CHECKREPLACE:     bRC = ExecuteCheckReplace()
        Case FUNC_CHECKIGNORE:      bRC = ExecuteCheckIgnore()
        Case FUNC_CHECKCANCEL:      bRC = ExecuteCheckCancel()
    End Select
    
    Execute = bRC
    
    #If DBG Then
    T "<-- Execute(" & FunctionName & ") = " & bRC
    #End If
End Function

'''

Public Sub CleanUp()
    modBackup.CleanUp
    
    If Not oState Is Nothing Then oState.CleanUp

    Set oState = Nothing
    
    Set oDatabase = Nothing
    Set oMedia = Nothing
    Set oMedium = Nothing
    
    Set oParent = Nothing
End Sub

'''

Private Sub Class_Initialize()
    Set oServerObject = New DBMSOB.ServerObject
    
    bDBMConnected = False
    bUTLConnected = False
End Sub

'''

'
' Save
'
Public Function ExecuteSave() As Boolean
    Dim bRelease As Boolean
    
    ExecuteSave = False
    
    bRelease = False
    
    With oDatabase
        .EventBackupStart
        
        If Not bDBMConnected Then
            If oServerObject.OpenSession(.ServerName, .DatabaseName, .UserName, .Password) Then
                bDBMConnected = True
            End If
        End If
            
        If bDBMConnected Then
            If Not bUTLConnected Then
                If DBMSRVUtil.Connect(oServerObject) Then
                    bUTLConnected = True
                End If
            End If
        End If
            
        If bDBMConnected And bUTLConnected Then
            StateStart
            If DBMSRVBackup.Save(.Backup, oServerObject, oMedia, lBackupType, lSaveFor) Then
                StateStop
                ExecuteSave = SaveResult()
            Else
                StateStop
                bRelease = True
            End If
        End If
    End With
    
    DBMSrvError dbmBASave, bRelease
End Function

Public Function ExecuteSaveReplace() As Boolean
    ExecuteSaveReplace = False
    
    oDatabase.EventBackupStart
    
    StateStart
    
    If DBMSRVBackup.SaveReplace(oServerObject, oMedium) Then
        StateStop
        oMedium.ReplaceCount = oMedium.ReplaceCount + 1
        ExecuteSaveReplace = SaveResult()
    Else
        StateStop
    End If
    
    DBMSrvError dbmBASave
End Function

Public Function ExecuteSaveIgnore() As Boolean
    ExecuteSaveIgnore = False
        
    oDatabase.EventBackupStart
    
    StateStart
    
    If DBMSRVBackup.SaveIgnore(oServerObject, oDatabase) Then
        StateStop
        oMedium.Ignored = True
        ExecuteSaveIgnore = SaveResult()
    Else
        StateStop
    End If
    
    DBMSrvError dbmBASave
End Function

Public Function ExecuteSaveCancel() As Boolean
    ExecuteSaveCancel = False
        
    oDatabase.EventBackupStart
    
    If DBMSRVBackup.SaveCancel(oServerObject, oDatabase) Then
        oDatabase.EventBackupCanceled
        ExecuteSaveCancel = True
    End If
                
    If bDBMConnected Then
        If bUTLConnected Then
            DBMSRVUtil.Release oServerObject
            bUTLConnected = False
        End If
        oServerObject.CloseSession
        bDBMConnected = False
    End If
    
    DBMSrvError dbmBASave
End Function

Private Function SaveResult() As Boolean
    Dim bRelease As Boolean
    Dim oMedium As Object
    Dim bReturn As Boolean
    
    #If DBG Then
    T "--> SaveResult() ReturnCode = " + CStr(oDatabase.Backup.BackupResult.ReturnCode)
    #End If
    
    bReturn = False
    
    bRelease = True

    Set oMedium = Nothing
    
    Select Case oDatabase.Backup.BackupResult.ReturnCode
        Case RC_OK:
            oDatabase.EventBackupReady
            bReturn = True
        
        Case RC_NEXTVOL:
            Select Case oMedia.ObjectType
                Case dbmOTMediaGroup:
                    Set oMedium = oMedia.Media.ItemByLocation(oDatabase.Backup.BackupResult.Location)
                    If oMedium Is Nothing Then
                        Set oMedium = oMedia.Media.ItemByNewLocation(oDatabase.Backup.BackupResult.Location)
                    End If
                Case dbmOTMedium:
                    Set oMedium = oMedia
            End Select
            oDatabase.EventBackupVolumeNext oMedium
            
            bRelease = False
            bReturn = True
            
        Case Else:
            With oDatabase.Backup
                .BackupError.Number = .BackupResult.ReturnCode
                .BackupError.Description = .BackupResult.ErrorText
            End With
            oDatabase.EventBackupError
            bReturn = False
            
    End Select

    If bRelease Then
        If bDBMConnected Then
            If bUTLConnected Then
                DBMSRVUtil.Release oServerObject
                bUTLConnected = False
            End If
            oServerObject.CloseSession
            bDBMConnected = False
        End If
    End If
    
    SaveResult = bReturn
    
    #If DBG Then
    T "<-- SaveResult() ReturnCode = " + CStr(oDatabase.Backup.BackupResult.ReturnCode)
    #End If
End Function
'
' Restore
'
Public Function ExecuteRestore() As Boolean
    Dim bRC As Boolean
    Dim bRelease As Boolean
    
    ' check label
    Dim oMedium As DBMLib.Medium
    Dim sLabelHistory As String
    Dim sLabelMedium As String
    Dim bLabelEqual As Boolean
    Dim bLabelRead As Boolean
    Dim sMsg As String
    '
    
    bRC = False
    
    ' check label
    bLabelEqual = True
    bLabelRead = False
    If Not oMedia Is Nothing Then
        If oMedia.ObjectType = dbmOTMedium Then
            Set oMedium = oMedia
        Else
            If oMedia.ObjectType = dbmOTMediaGroup Then
                If oMedia.Media.Count > 0 Then
                    Set oMedium = oMedia.Media(1)
                End If
            End If
        End If
        
        If Not oMedium Is Nothing Then
            If oMedium.ExternalID Is Nothing Then
                sLabelHistory = ""
                sLabelMedium = ""
                If Not oMedium.RestoreItem Is Nothing Then
                    If Not oMedium.RestoreItem.HistoryItem Is Nothing Then
                        If oMedium.RestoreItem.HistoryItem.BackupType = dbmBTData Or _
                           oMedium.RestoreItem.HistoryItem.BackupType = dbmBTPages _
                        Then
                            sLabelHistory = UCase$(Trim$(oMedium.RestoreItem.HistoryItem.Label))
                            If Len(sLabelHistory) > 0 Then
                                bLabelEqual = False
                                If oMedium.ReadLabel() Then
                                    bLabelRead = True
                                    sLabelMedium = UCase$(Trim$(oMedium.BackupResult.Label))
                                    If sLabelHistory = sLabelMedium Then
                                        bLabelEqual = True
                                    End If
                                End If
                            End If
                        End If
                    End If
                End If
            End If
        End If
    
    End If
        
    If bLabelEqual Then
    
        bRelease = False
    
        With oDatabase
            .EventRestoreStart
            
            If Not bDBMConnected Then
                If oServerObject.OpenSession(.ServerName, .DatabaseName, .UserName, .Password) Then
                    bDBMConnected = True
                End If
            End If
                
            If bDBMConnected Then
                If Not bUTLConnected Then
                    If DBMSRVUtil.Connect(oServerObject) Then
                        bUTLConnected = True
                    End If
                End If
            End If
                
            If bDBMConnected And bUTLConnected Then
                StateStart dbmBackupStateRestore
                
                If DBMSRVBackup.Restore(.Backup, oServerObject, oMedia, lBackupType, lFileNumber, bUseUntilDate, dUntilDate) Then
                    StateStop
                    bRC = RestoreResult()
                Else
                    StateStop
                    bRelease = True
                End If
            End If
        End With
    
        DBMSrvError dbmBARestore, bRelease
        
    Else
    
        If bLabelRead Then
            sMsg = "Incorrect label! The current medium has the label " + sLabelMedium
            sMsg = sMsg + ". A medium with the label " + sLabelHistory + " was expected."
            oDatabase.RaiseError -24999, sMsg
        
            With oDatabase.Backup.BackupError
              .Number = -24999
              .Description = sMsg
            End With
        
            oDatabase.EventRestoreError
        Else
            DBMSrvError dbmBARestore, True
        End If
        
    End If
    
    ExecuteRestore = bRC
End Function

Public Function ExecuteRestoreReplace() As Boolean
    ExecuteRestoreReplace = False
        
    oDatabase.EventRestoreStart
    
    StateStart dbmBackupStateRestore
    
    If DBMSRVBackup.RestoreReplace(oServerObject, oMedium, lFileNumber) Then
        StateStop
        oMedium.ReplaceCount = oMedium.ReplaceCount + 1
        ExecuteRestoreReplace = RestoreResult()
    Else
        StateStop
    End If
    
    DBMSrvError dbmBARestore
End Function

Public Function ExecuteRestoreIgnore() As Boolean
    ExecuteRestoreIgnore = False
        
    oDatabase.EventRestoreStart
    
    StateStart dbmBackupStateRestore
    
    If DBMSRVBackup.RestoreIgnore(oServerObject, oDatabase) Then
        StateStop
        If Not oMedium Is Nothing Then
            oMedium.Ignored = True
        End If
        ExecuteRestoreIgnore = RestoreResult()
    Else
        StateStop
    End If
    
    DBMSrvError dbmBARestore
End Function

Public Function ExecuteRestoreCancel() As Boolean
    ExecuteRestoreCancel = False
        
    oDatabase.EventRestoreStart
    
    If bDBMConnected And bUTLConnected Then
        If DBMSRVBackup.RestoreCancel(oServerObject, oDatabase) Then
            ExecuteRestoreCancel = True
        End If
    End If
                
    oDatabase.EventRestoreCanceled
                
    If bDBMConnected Then
        If bUTLConnected Then
            DBMSRVUtil.Release oServerObject
            bUTLConnected = False
        End If
        oServerObject.CloseSession
        bDBMConnected = False
    End If
    
    DBMSrvError dbmBARestore
End Function

Private Function RestoreResult() As Boolean
    Dim bRelease As Boolean
    Dim oMedium As Object
    Dim bReturn As Boolean

    #If DBG Then
    T "--> RestoreResult() ReturnCode = " + CStr(oDatabase.Backup.BackupResult.ReturnCode)
    #End If
    
    bReturn = False

    bRelease = True

    Set oMedium = Nothing

    Select Case oDatabase.Backup.BackupResult.ReturnCode
        Case RC_OK:
            If lBackupType = dbmBTLog Or _
               lBackupType = dbmBTAuto _
            Then
                oDatabase.EventRestoreReady
                oDatabase.State.Refresh
            Else
                If DBMSRVBackup.RestartInfo(oServerObject, oDatabase.RestartInfo) Then
                End If
                oDatabase.EventRestoreReady
            End If
            bReturn = True
            
        Case RC_NEXTVOL:
            Select Case oMedia.ObjectType
                Case dbmOTMediaGroup:
                    Set oMedium = oMedia.Media.ItemByLocation(oDatabase.Backup.BackupResult.Location)
                    If oMedium Is Nothing Then
                        Set oMedium = oMedia.Media.ItemByNewLocation(oDatabase.Backup.BackupResult.Location)
                    End If
                Case dbmOTMedium:
                    Set oMedium = oMedia
            End Select
            oDatabase.EventRestoreVolumeNext oMedium
            bRelease = False
            bReturn = True
            
        Case RC_SKIPPED:
            oDatabase.EventRestoreVolumeSkipped
            bRelease = True
            bReturn = True

        Case RC_VOLEND:
            oDatabase.EventRestoreVolumeEnd
            bRelease = False
            bReturn = True

        Case Else:
            With oDatabase.Backup
                .BackupError.Number = .BackupResult.ReturnCode
                .BackupError.Description = .BackupResult.ErrorText
            End With
            oDatabase.EventRestoreError
            bRelease = False
            bReturn = False

    End Select

    If bRelease Then
        If bDBMConnected Then
            If bUTLConnected Then
                DBMSRVUtil.Release oServerObject
                bUTLConnected = False
            End If
            oServerObject.CloseSession
            bDBMConnected = False
        End If
    End If

    RestoreResult = bReturn
    
    #If DBG Then
    T "<-- RestoreResult() ReturnCode = " + CStr(oDatabase.Backup.BackupResult.ReturnCode)
    #End If
End Function
'
' Check
'
Public Function ExecuteCheck() As Boolean
    Dim bRelease As Boolean
    
    ExecuteCheck = False
        
    bRelease = False
    
    With oDatabase
        .EventBackupCheckStart
        
        If Not bDBMConnected Then
            If oServerObject.OpenSession(.ServerName, .DatabaseName, .UserName, .Password) Then
                bDBMConnected = True
            End If
        End If
        
        If bDBMConnected Then
            If Not bSRVConnected Then
                If DBMSRVService.Connect(oServerObject) Then
                    bSRVConnected = True
                End If
            End If
        End If
                
        If bDBMConnected And bSRVConnected Then
            StateStart dbmBackupStateCheck
            
            If DBMSRVBackup.Check(.Backup, oServerObject, oMedia, lBackupType, lFileNumber) Then
                StateStop
                ExecuteCheck = CheckResult()
            Else
                StateStop
                bRelease = True
            End If
        End If
    End With
    
    DBMSrvError dbmBACheck, bRelease
End Function

Public Function ExecuteCheckReplace() As Boolean
    ExecuteCheckReplace = False
        
    oDatabase.EventBackupCheckStart
    
    StateStart dbmBackupStateCheck
    
    If DBMSRVBackup.CheckReplace(oServerObject, oMedium, lFileNumber) Then
        StateStop
        oMedium.ReplaceCount = oMedium.ReplaceCount + 1
        ExecuteCheckReplace = CheckResult()
    Else
        StateStop
    End If
    
    DBMSrvError dbmBACheck
End Function

Public Function ExecuteCheckIgnore() As Boolean
    ExecuteCheckIgnore = False
        
    oDatabase.EventBackupCheckStart
    
    StateStart dbmBackupStateCheck
    
    If DBMSRVBackup.CheckIgnore(oServerObject, oDatabase) Then
        StateStop
        If Not oMedium Is Nothing Then
            oMedium.Ignored = True
        End If
        ExecuteCheckIgnore = CheckResult()
    Else
        StateStop
    End If
    
    DBMSrvError dbmBACheck
End Function

Public Function ExecuteCheckCancel() As Boolean
    ExecuteCheckCancel = False
        
    oDatabase.EventBackupCheckStart
    
    If DBMSRVBackup.CheckCancel(oServerObject, oDatabase) Then
        oDatabase.EventBackupCheckCanceled
        ExecuteCheckCancel = True
    End If
                
    If bDBMConnected Then
        If bSRVConnected Then
            DBMSRVService.Release oServerObject
            bSRVConnected = False
        End If
        oServerObject.CloseSession
        bDBMConnected = False
    End If
    
    DBMSrvError dbmBACheck
End Function

Private Function CheckResult() As Boolean
    Dim bRelease As Boolean
    Dim oMedium As Object
    Dim bReturn As Boolean

    #If DBG Then
    T "--> CheckResult() ReturnCode = " + CStr(oDatabase.Backup.BackupResult.ReturnCode)
    #End If
    
    bReturn = False

    bRelease = True

    Set oMedium = Nothing

    Select Case oDatabase.Backup.BackupResult.ReturnCode
        Case RC_OK:
            oDatabase.EventBackupCheckReady
            bReturn = True
            
        Case RC_NEXTVOL:
            Select Case oMedia.ObjectType
                Case dbmOTMediaGroup:
                    Set oMedium = oMedia.Media.ItemByLocation(oDatabase.Backup.BackupResult.Location)
                    If oMedium Is Nothing Then
                        Set oMedium = oMedia.Media.ItemByNewLocation(oDatabase.Backup.BackupResult.Location)
                    End If
                Case dbmOTMedium:
                    Set oMedium = oMedia
            End Select
            oDatabase.EventBackupCheckVolumeNext oMedium
            bRelease = False
            bReturn = True

        Case RC_VOLEND:
            oDatabase.EventBackupCheckVolumeEnd
            bRelease = False
            bReturn = True

        Case Else:
            With oDatabase.Backup
                .BackupError.Number = .BackupResult.ReturnCode
                .BackupError.Description = .BackupResult.ErrorText
            End With
            oDatabase.EventBackupCheckError
            bRelease = False
            bReturn = False

    End Select

    If bRelease Then
        If bDBMConnected Then
            If bSRVConnected Then
                DBMSRVService.Release oServerObject
                bSRVConnected = False
            End If
            oServerObject.CloseSession
            bDBMConnected = False
        End If
    End If

    CheckResult = bReturn
    
    #If DBG Then
    T "<-- CheckResult() ReturnCode = " + CStr(oDatabase.Backup.BackupResult.ReturnCode)
    #End If
End Function

'''

Private Sub DBMSrvError(ByVal lBackupAction As DBMBackupActionConstants, _
                        Optional ByVal bRelease As Boolean = False)
    If oServerObject.ReturnCode = DBMSRV_RC_OK Then Exit Sub
    If oDatabase Is Nothing Then Exit Sub
    
    oDatabase.RaiseError oServerObject.ReturnCode, oServerObject.ErrorText
    
    With oDatabase.Backup.BackupError
        If oServerObject.SQLErrorNumber <> 0 Then
            .Number = oServerObject.SQLErrorNumber
            .Description = oServerObject.SQLErrorText
        Else
            .Number = oServerObject.ReturnCode
            .Description = oServerObject.ErrorText
        End If
        .Canceled = oServerObject.Canceled
    End With
    
    Select Case lBackupAction
        Case dbmBASave:     oDatabase.EventBackupError
        Case dbmBARestore:  oDatabase.EventRestoreError
        Case dbmBACheck:    oDatabase.EventBackupCheckError
    End Select
    
    If bRelease Then
        If bDBMConnected Then
            If bSRVConnected Then
                DBMSRVService.Release oServerObject
                bSRVConnected = False
            End If
            If bUTLConnected Then
                DBMSRVUtil.Release oServerObject
                bUTLConnected = False
            End If
            oServerObject.CloseSession
            bDBMConnected = False
        End If
    End If
End Sub

'''

Private Sub StateStart(Optional ByVal lStateType As DBMBackupStateTypeConstants = dbmBackupStateSave)
    #If DBG Then
    T "<-> StateStart()"
    #End If
    
    If Not oDatabase Is Nothing Then
        If oState Is Nothing Then
            Set oState = New DBMApp.BackupState
        End If
        
        Set oState.Parent = Me
        Set oState.Database = oDatabase
        
        oState.StateStart lStateType
    End If
End Sub

Private Sub StateStop()
    #If DBG Then
    T "<-> StateStop()"
    #End If
    
    If Not oState Is Nothing Then
        oState.StateStop
    End If
End Sub

'''

#If DBG Then

Public Property Get DebugTrace() As Object
    Set DebugTrace = oDbgTrace
End Property
Public Property Set DebugTrace(NewVal As Object)
    Set oDbgTrace = NewVal
    If Not oServerObject Is Nothing Then
        Set oServerObject.DebugTrace = oDbgTrace
    End If
End Property

Private Sub T(ByVal sText As String)
    If Not oDbgTrace Is Nothing Then
        oDbgTrace.Text "DBMAPP(x0" + Hex(App.ThreadID) + ").Backup: " + sText
    End If
End Sub

#End If
