VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "BackupState"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 1998-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Private Declare Function GetTickCount Lib "kernel32.dll" () As Long

'''

Private oParent As Object

Private oDatabase As DBMLib.Database

Private lInterval As Long

Private oServerObject As DBMSOB.ServerObject

Private bConnected As Boolean

Public Enum DBMBackupStateTypeConstants
    dbmBackupStateSave
    dbmBackupStateRestore
    dbmBackupStateCheck
End Enum

Private lStateType As DBMBackupStateTypeConstants

Private lStart As Long

'''

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Public Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As DBMLib.Database
    Set Database = oDatabase
End Property
Public Property Set Database(NewVal As DBMLib.Database)
    Set oDatabase = NewVal
End Property

Public Property Get Interval() As Long
    Interval = lInterval
End Property
Public Property Let Interval(ByVal NewVal As Long)
    lInterval = NewVal
End Property

'''

Public Function StateStart(Optional ByVal StateType As DBMBackupStateTypeConstants = dbmBackupStateSave) As Boolean
    StateStart = False
    
    Set oServerObject = New DBMSOB.ServerObject
    
    bConnected = False
    
    With oDatabase
        If oServerObject.OpenSession(.ServerName, .DatabaseName, .UserName, .Password) Then
            bConnected = True
        End If
    End With
    
    If Not bConnected Then Exit Function
    
    lStateType = StateType
    lStart = GetTickCount()
    
    Select Case lStateType
        Case dbmBackupStateSave:    oDatabase.EventBackupState dbmBSStart
        Case dbmBackupStateRestore: oDatabase.EventRestoreState dbmBSStart
        Case dbmBackupStateCheck:   oDatabase.EventBackupCheckState dbmBSStart
    End Select
    
    modBackupState.StartTimer Me, 1000 'lInterval
    
    StateStart = True
End Function

Public Sub StateStop()
    modBackupState.StopTimer
    
    If bConnected Then
        If Not oServerObject Is Nothing Then
            oServerObject.CloseSession
        End If
        bConnected = False
    End If
    
    modBackupState.CleanUp
    
    If Not oDatabase Is Nothing Then
        Select Case lStateType
            Case dbmBackupStateSave:    oDatabase.EventBackupState dbmBSStop
            Case dbmBackupStateRestore: oDatabase.EventRestoreState dbmBSStop
            Case dbmBackupStateCheck:   oDatabase.EventBackupCheckState dbmBSStop
        End Select
    End If
    
    Set oServerObject = Nothing
End Sub

Public Function Execute() As Boolean
    Dim lTicks As Long
    Dim bOK As Boolean
    
    Execute = False
        
    If oServerObject Is Nothing Then Exit Function
    
    Select Case lStateType
        Case dbmBackupStateSave:    bOK = DBMSRVBackup.SaveState(oServerObject, oDatabase.Backup.BackupState)
        Case dbmBackupStateRestore: bOK = DBMSRVBackup.RestoreState(oServerObject, oDatabase.Backup.BackupState)
        Case dbmBackupStateCheck:   bOK = DBMSRVBackup.CheckState(oServerObject, oDatabase.Backup.BackupState)
    End Select
    
    If bOK Then
        If Not oDatabase Is Nothing Then
            Select Case lStateType
                Case dbmBackupStateSave:    oDatabase.EventBackupState dbmBSRefresh
                Case dbmBackupStateRestore: oDatabase.EventRestoreState dbmBSRefresh
                Case dbmBackupStateCheck:   oDatabase.EventBackupCheckState dbmBSRefresh
            End Select
        End If
        Execute = True
    End If
    
    lTicks = GetTicks(lStart, GetTickCount())
    lInterval = lTicks \ 12
    If lInterval < 1000 Then lInterval = 1000
    If lInterval > 300000 Then lInterval = 300000
    
    modBackupState.ChangeInterval lInterval
End Function

Public Sub CleanUp()
    StateStop
    
    Set oDatabase = Nothing
    Set oParent = Nothing
End Sub

'''

Private Sub Class_Initialize()
    lInterval = 1000
    bConnected = False
End Sub
Private Sub Class_Terminate()
    StateStop
End Sub

'''

Private Function GetTicks(ByVal t1 As Long, ByVal t2 As Long) As Long
    On Error Resume Next
    
    GetTicks = &H7FFFFFFF
    
    If t1 >= 0 Then
        If t2 >= 0 Then
            GetTicks = t2 - t1
        Else
            GetTicks = (&H7FFFFFFF - t1) - (&H80000000 - t2) + 1
        End If
    Else
        If t2 >= 0 Then
            GetTicks = (&HFFFFFFFF - t1) + t2 + 1
        Else
            GetTicks = t2 - t1
        End If
    End If
End Function
