VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "DatabaseEvents"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 1998-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const DBMSRV_CMD_EVENT_WAIT = "event_wait"
Const DBMSRV_CMD_EVENT_RELEASE = "event_release"

Const EVENT_PROP_NAME = "NAME"
Const EVENT_PROP_PRIORITY = "PRIORITY"
Const EVENT_PROP_DATE = "DATE"
Const EVENT_PROP_TIME = "TIME"
Const EVENT_PROP_VAL1 = "VALUE1"
Const EVENT_PROP_VAL2 = "VALUE2"
Const EVENT_PROP_TEXT = "TEXT"
Const EVENT_PROP_COUNT = "COUNT"
Const EVENT_PROP_DESC = "DESCRIPTION"

'''

Private oParent As Object

Private oDatabase As DBMLib.Database
    
Private oServerObject As DBMSOB.ServerObject

Private bCancel As Boolean

Private bRunning As Boolean

'''

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Public Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As DBMLib.Database
    Set Database = oDatabase
End Property
Public Property Set Database(NewVal As DBMLib.Database)
    Set oDatabase = NewVal
End Property

Public Sub EventsOn()
    #If DBG Then
    T "<-> EventsOn"
    #End If
    modDatabaseEvents.Execute Me
End Sub

Public Sub EventsOff()
    #If DBG Then
    T "<-> EventsOff"
    #End If
    bCancel = True
    If Not oServerObject Is Nothing Then
        oServerObject.Cancel
    End If
End Sub

'''

Public Sub Execute(Optional ByVal FunctionName As String)
    On Error GoTo OnError_Msg_Execute
    
    Dim sData As String
    Dim sLine As String
    Dim sProp As String
    Dim sValue As String
    
    Dim sName As String
    Dim sPriority As String
    Dim sDate As String
    Dim sTime As String
    Dim lCount As Long
    Dim sValue1 As String
    Dim sValue2 As String
    Dim sText As String
    Dim sDescription As String
    Dim lPos1 As Long
    Dim d As Variant
    Dim dOccurrence As Date
    
    If oDatabase Is Nothing Then Exit Sub
    
    Set oServerObject = New DBMSOB.ServerObject
    
    #If DBG Then
    Set oServerObject.DebugTrace = oDbgTrace
    #End If
    
    #If DBG Then
    T "--> Execute()"
    #End If

    bCancel = False
    
    With oDatabase
        If oServerObject.OpenSession(.ServerName, .DatabaseName, .UserName, .Password) Then
            
            bRunning = True
            
            Do While Not bCancel
                If oServerObject.Command(DBMSRV_CMD_EVENT_WAIT, sData) Then
                    sName = ""
                    sPriority = ""
                    sDate = ""
                    sTime = ""
                    lCount = 0
                    sValue1 = ""
                    sValue2 = ""
                    sText = ""
                    sDescription = ""
                    lPos1 = 1
                    Do While lPos1 < Len(sData)
                        If Not oServerObject Is Nothing Then
                            GetNameValue Trim$(oServerObject.GetLine(sData, lPos1)), sProp, sValue
                            Select Case sProp
                                Case EVENT_PROP_NAME:       sName = sValue
                                Case EVENT_PROP_PRIORITY:   sPriority = sValue
                                Case EVENT_PROP_DATE:       sDate = sValue
                                Case EVENT_PROP_TIME:       sTime = sValue
                                Case EVENT_PROP_COUNT:      lCount = Val(sValue)
                                Case EVENT_PROP_VAL1:       sValue1 = sValue
                                Case EVENT_PROP_VAL2:       sValue2 = sValue
                                Case EVENT_PROP_TEXT:       sText = sValue
                                Case EVENT_PROP_DESC:       sDescription = sValue
                            End Select
                        Else
                            bCancel = True
                            Exit Do
                        End If
                    Loop
                    If Not bCancel Then
                        If Len(sDate) > 0 And Len(sTime) > 0 Then
                            dOccurrence = vbEmpty
                            d = GetDateFromString(sDate)
                            If Not IsEmpty(d) Then
                                d = d + GetTimeFromString(sTime)
                            End If
                            dOccurrence = d
                        End If
                        oDatabase.EventDatabaseEvent sName, sPriority, dOccurrence, lCount, sValue1, sValue2, sText, sDescription
                        #If DBG Then
                        T "--- Event = " + sName + " " + CStr(dOccurrence)
                        #End If
                    End If
                Else
                    If bCancel Then
                        #If DBG Then
                        T "--- Event Canceled!"
                        #End If
                    Else
                        bCancel = True
                        
                        #If DBG Then
                        T "--- Event Error!"
                        #End If
                    End If
                End If
            Loop

            bRunning = False
            
            If Not oServerObject Is Nothing Then
                If oServerObject.Command(DBMSRV_CMD_EVENT_RELEASE, sData) Then
                End If
                oServerObject.CloseSession
            End If
        End If

    End With
    
OnError_Exit_Execute:
    
    bRunning = False
    
    Set oServerObject = Nothing

    #If DBG Then
    T "<-- Execute()"
    #End If
    
    Exit Sub
OnError_Msg_Execute:
    OnError.Error Err.Number, Err.Description, "DatabaseEvents\Execute", 1, 0
    Resume OnError_Exit_Execute
End Sub

'''

Public Sub CleanUp()
    Enabled = False
    
    modDatabaseEvents.CleanUp
    
    Set oDatabase = Nothing
    Set oParent = Nothing
End Sub

'''

Private Sub Class_Initialize()
    bRunning = False
End Sub
Private Sub Class_Terminate()
    CleanUp
End Sub

'''

Private Sub GetNameValue(sLine As String, _
                         sName As String, _
                         sValue As String)
    Dim lPos As Long
    lPos = InStr(1, sLine, "=")
    If lPos > 0 Then
        sName = Trim$(Left$(sLine, lPos - 1))
        sValue = Trim$(Mid$(sLine, lPos + 1))
    Else
        sName = Trim$(sLine)
        sValue = ""
    End If
End Sub

Private Function GetDateFromString(sDate As String) As Variant
    If Len(sDate) > 0 Then
        GetDateFromString = DateSerial(Val(Left$(sDate, 4)), Val(Mid$(sDate, 5, 2)), Val(Mid$(sDate, 7, 2)))
    Else
        GetDateFromString = Empty
    End If
End Function

Private Function GetTimeFromString(sTime As String) As Variant
    If Len(sTime) > 0 Then
        GetTimeFromString = TimeSerial(Val(Mid$(sTime, 3, 2)), Val(Mid$(sTime, 5, 2)), Val(Mid$(sTime, 7, 2)))
    Else
        GetTimeFromString = Empty
    End If
End Function

#If DBG Then

Public Property Get DebugTrace() As Object
    Set DebugTrace = oDbgTrace
End Property
Public Property Set DebugTrace(NewVal As Object)
    Set oDbgTrace = NewVal
End Property

Private Sub T(ByVal sText As String)
    If Not oDbgTrace Is Nothing Then
        oDbgTrace.Text "DBMAPP(x0" + Hex(App.ThreadID) + ").Events: " + sText
    End If
End Sub

#End If

