VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Documents"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' --- GPL ---
'
' Copyright (C) 1998-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Public Enum DBMDocumentCloseModeConstants
    dbmCloseModeDefault
    dbmCloseModeWrongState
    dbmCloseModeAppServer
    dbmCloseModeWindows
    dbmCloseModeTaskManager
End Enum

Const DOC_MENU_POS = 4

Public Enum DBMDocumentTypeConstants
    dbmDDatabase
    dbmDInformation
    dbmDBackup
    dbmDRestore
    dbmDMedia
    dbmDHistory
    dbmDBackupCheck
    dbmDKernelTrace
    dbmDFiles
    dbmDShows
    dbmDCommand
    dbmDEventMonitor
    dbmDIndexUse
    dbmDIndexRecovery
    dbmDUpdStat
    dbmDVerify
    dbmDBadDevspaces
    dbmDParameters
    dbmDUsers
    dbmDDevSpaceData
    dbmDDevSpaceLog
    dbmDLogMode
    dbmDTermcharSets
    dbmDMapcharSets
    dbmDLoadSysTabs
    dbmDEvents
    dbmDRestartUntil
End Enum

Const FIRST_DOC = dbmDDatabase
Const LAST_DOC = dbmDRestartUntil

Const MAX_DEFAULT_CONTROLS = 7

Private bControl(FIRST_DOC To LAST_DOC, 0 To MAX_DEFAULT_CONTROLS) As Boolean
Private iControlMax As Integer

Private oItems As New Collection

Private oActDoc As Document

Public Property Get Count() As Integer
    Count = oItems.Count
End Property

Public Property Get Item(Index As Integer) As Document
Attribute Item.VB_UserMemId = 0
    Set Item = oItems(Index)
End Property

Public Property Get ItemByDocType(lDoctype As DBMDocumentTypeConstants, _
                                  iControlIndex As Integer, _
                                  iItemIndex As Integer) As Document
    Dim oDocument As Document
    Dim iItem As Integer
    
    Set oDocument = Nothing
    iItemIndex = 0
    
    For iItem = 1 To oItems.Count
        Set oDocument = oItems(iItem)
        
        If oDocument.DocType = lDoctype And _
           oDocument.Control.Index = iControlIndex _
        Then
            iItemIndex = iItem
            Exit For
        End If
    Next
    
    Set ItemByDocType = oDocument
End Property

Public Property Get NewEnum()
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    Set NewEnum = oItems.[_NewEnum]
End Property

Public Property Get ActDoc() As Document
    Set ActDoc = oActDoc
End Property

Public Sub Show(lDoctype As DBMDocumentTypeConstants, _
                oDatabase As DBMLib.Database)
    Dim oDocument As Document
    Dim bExists As Boolean
    Dim bRefresh As Boolean
    
    If Not oActDoc Is Nothing Then
        If oActDoc.DocType = lDoctype And _
           oActDoc.Database.Name = oDatabase.Name _
        Then
            Select Case oActDoc.DocType
                Case dbmDFiles:
                    oActDoc.Control.ViewList
                Case dbmDShows:
                    oActDoc.Control.ViewList
            End Select
            
            Exit Sub
        End If
        
        oActDoc.Control.HideMenu
    End If
    
    Set oDocument = GetDoc(lDoctype, oDatabase, bExists, bRefresh)
        
    ShowDoc oDocument, oDatabase, bRefresh
End Sub

Public Sub ShowInfo(oDatabase As DBMLib.Database, _
                    lInfo As DBMLib.DBMInfoTypeConstants)
    Dim oDocument As Document
    Dim bExists As Boolean
    Dim bRefresh As Boolean
    
    If Not oActDoc Is Nothing Then
        If oActDoc.DocType = dbmDInformation And _
           oActDoc.Database.Name = oDatabase.Name _
        Then
            If oActDoc.Control.InfoType <> lInfo Then
                oActDoc.Control.InfoType = lInfo
                oActDoc.Control.Refresh
            End If
            Exit Sub
        End If
        oActDoc.Control.HideMenu
    End If
    
    Set oDocument = GetDoc(dbmDInformation, oDatabase, bExists, bRefresh)
       
    With oDocument.Control
        If bExists And (.InfoType <> lInfo) Then
            bRefresh = True
        End If
        .InfoType = lInfo
    End With
        
    ShowDoc oDocument, oDatabase, bRefresh
End Sub

Public Sub ShowBackup(oDatabase As DBMLib.Database, _
                      lMode As DBMBackupModeConstants)
    Dim oDocument As Document
    Dim bExists As Boolean
    Dim bRefresh As Boolean
    Dim iItem As Integer
    
    ' PTS 1107054 >>>
    Dim bCancel As Boolean
    If Exists(dbmDRestore, oDatabase, iItem) Then
        Set oDocument = oItems(iItem)
        
        Hide dbmDRestore, oDocument.Control.Index, bCancel, dbmCloseModeDefault
        If bCancel Then Exit Sub
        
        Set oDocument = Nothing
    End If
    ' <<< PTS 1107054
    
    If Not oActDoc Is Nothing Then
        If oActDoc.DocType = dbmDBackup And _
           oActDoc.Database.Name = oDatabase.Name _
        Then
            If oActDoc.Control.Mode <> lMode Then
                oActDoc.Control.Mode = lMode
                oActDoc.Control.Refresh
            End If
            oActDoc.Control.SetFocus
            Exit Sub
        End If
        
        oActDoc.Control.HideMenu
    End If
    
    Set oDocument = GetDoc(dbmDBackup, oDatabase, bExists, bRefresh)
    
    With oDocument.Control
        If bExists And (.Mode <> lMode) Then
            bRefresh = True
        End If
        .Mode = lMode
    End With
    
    ShowDoc oDocument, oDatabase, bRefresh
End Sub

Public Sub ShowNext()
    If oItems.Count > 0 Then
        If Not oActDoc Is Nothing Then
            If oActDoc.Index < oItems.Count Then
                ShowByIndex oActDoc.Index + 1
            Else
                ShowByIndex 1
            End If
        End If
    End If
End Sub

Public Sub ShowPrev()
    If oItems.Count > 0 Then
        If Not oActDoc Is Nothing Then
            If oActDoc.Index > 1 Then
                ShowByIndex oActDoc.Index - 1
            Else
                ShowByIndex oItems.Count
            End If
        End If
    End If
End Sub

Private Function GetDoc(lDoctype As DBMDocumentTypeConstants, oDatabase As DBMLib.Database, bExists As Boolean, bRefresh As Boolean) As Document
    Dim oDocument As Document
    Dim iItem As Integer
    Dim oFont As StdFont
    
    bExists = False
    bRefresh = False
    
    iItem = 0
    
    If Exists(lDoctype, oDatabase, iItem) Then
        Set oDocument = oItems(iItem)
        bExists = True
    Else
        Set oDocument = Add(lDoctype, oDatabase)
        bRefresh = True
    End If
    
    If lDoctype = dbmDCommand Or _
       lDoctype = dbmDFiles Or _
       lDoctype = dbmDShows Or _
       lDoctype = dbmDLoadSysTabs _
    Then
        Set oFont = New StdFont
        With oFont
            .Name = REG.Settings.ProtocolFontName
            .Size = REG.Settings.ProtocolFontSize
            .Bold = False
            .Underline = False
            .Italic = False
            .Strikethrough = False
        End With
        
        Set oDocument.Control.ProtocolFont = oFont
    End If

    Set GetDoc = oDocument
End Function

Private Sub ShowDoc(oDocument As Document, oDatabase As DBMLib.Database, ByVal bRefresh As Boolean)
    With oDocument.Control
        SetStatusBarStatus "Open '" + .Caption + "'."
        
        .Move 0, 0, .Container.ScaleWidth, .Container.ScaleHeight
        
        .Visible = True
        .ZOrder 0
        
        Set FRM.TIB.Picture = .Picture
        FRM.TIB.Enabled = Not .Running
        FRM.TIB.At = .At
        
        Set .TitleBar = FRM.TIB
        Set .Database = oDatabase
        
        FRM.TIB.Visible = True
        
        DoEvents
        
        If bRefresh Then
            Select Case oDocument.DocType
                Case dbmDParameters: .Groups = 0
            End Select
            .Refresh
        End If
        
        .ShowMenu FRM.MBR, DOC_MENU_POS
        
        .SetFocus
    End With
    
    If Not oActDoc Is Nothing Then
        oActDoc.Control.Visible = False
    End If
    
    SetStatusBarStatus ""
    
    Set oActDoc = oDocument
    
    If Not oActDoc Is Nothing Then
        FRM.DatabaseSetActByDB oActDoc.Database
    End If
End Sub

Public Sub ShowByIndex(ByVal Index As Integer)
    Dim oDocument As Document
    Dim bExists As Boolean
    Dim bRefresh As Boolean
    
    If Index < 1 Or Index > oItems.Count Then
        Exit Sub
    End If
    
    Set oDocument = oItems(Index)
    
    If Not oActDoc Is Nothing Then
        If oActDoc.DocType = oDocument.DocType And _
           oActDoc.Database.Name = oDocument.Database.Name _
        Then
            Exit Sub
        End If
        
        oActDoc.Control.HideMenu
    End If
    
    With oDocument.Control
        .Move 0, 0, .Container.ScaleWidth, .Container.ScaleHeight
        
        .Visible = True
        .ZOrder 0
        
        Set .TitleBar = FRM.TIB
        
        Set FRM.TIB.Picture = .Picture
        FRM.TIB.Caption = .Caption
        FRM.TIB.Enabled = Not .Running
        FRM.TIB.At = .At
        FRM.TIB.Visible = True
        
        DoEvents
        
        .ShowMenu FRM.MBR, DOC_MENU_POS
    End With
    
    If Not oActDoc Is Nothing Then
        oActDoc.Control.Visible = False
    End If
    
    Set oActDoc = oDocument
End Sub

Public Sub GotFocus(DocType As DBMDocumentTypeConstants, _
                    Control As Integer)
    Dim oDocument As Document
    Dim iItem As Integer
        
    Set oDocument = ItemByDocType(DocType, Control, iItem)
    
    If Not oDocument Is Nothing Then
        FRM.DatabaseSetActByDB oDocument.Database
    End If
End Sub

Public Sub HideWrongDBState(DocType As DBMDocumentTypeConstants, _
                            Control As Integer)
    Dim bCancel As Boolean
    Hide DocType, Control, bCancel, dbmCloseModeWrongState
End Sub

Public Sub Hide(DocType As DBMDocumentTypeConstants, _
                Control As Integer, _
                Cancel As Boolean, _
                CloseMode As DBMDocumentCloseModeConstants)
                
    Dim oDocument As Document
    Dim iItem As Integer
        
    Set oDocument = ItemByDocType(DocType, Control, iItem)
    
    If Not oDocument Is Nothing Then
        HideDoc oDocument, Cancel, CloseMode
        If Not Cancel Then
            oItems.Remove iItem
            RenumberIndex
            ShowByIndex 1
            If oItems.Count = 0 Then FRM.TIB.Visible = False
        End If
    End If
End Sub

Public Sub HideActDoc(Cancel As Boolean, CloseMode As DBMDocumentCloseModeConstants)
    If Not oActDoc Is Nothing Then
        Hide oActDoc.DocType, oActDoc.Control.Index, Cancel, CloseMode
    End If
End Sub

Public Sub HideByDB(Database As DBMLib.Database, _
                    Cancel As Boolean, _
                    CloseMode As DBMDocumentCloseModeConstants)
                    
    Dim oDocument As Document
    Dim iItem As Integer
    
    Dim bDeleted As Boolean

    For iItem = 1 To oItems.Count
        Set oDocument = oItems(iItem)

        If oDocument.Database.Name = Database.Name Then
            HideDoc oDocument, Cancel, CloseMode
            If Cancel Then Exit For
        End If
    Next

    RemoveUnusedItems
    
    ShowByIndex 1
    
    If oItems.Count = 0 Then FRM.TIB.Visible = False
End Sub

Public Sub HideAll(Cancel As Boolean, _
                   CloseMode As DBMDocumentCloseModeConstants)
                   
    Dim oDocument As Document
    Dim iItem As Integer
    
    For iItem = 1 To oItems.Count
        Set oDocument = oItems(iItem)
        
        HideDoc oDocument, Cancel, CloseMode
        If Cancel Then Exit For
    Next

    RemoveUnusedItems
    
    If Cancel Then Exit Sub
    
    FRM.TIB.Visible = False
End Sub

Public Sub HideByFlag(Cancel As Boolean, _
                      CloseMode As DBMDocumentCloseModeConstants)
                      
    Dim oDocument As Document
    Dim iItem As Integer

    For iItem = 1 To oItems.Count
        Set oDocument = oItems(iItem)

        If oDocument.HideFlag = True Then
            HideDoc oDocument, Cancel, CloseMode
            If Cancel Then Exit For
        End If
    Next

    RemoveUnusedItems
    
    If Cancel Then Exit Sub
    
    ShowByIndex 1
    
    If oItems.Count = 0 Then FRM.TIB.Visible = False
End Sub

'''

Private Sub Class_Initialize()
    Dim iDocument As Integer
    Dim iControl As Integer
    
    iControlMax = MAX_DEFAULT_CONTROLS
    
    For iDocument = FIRST_DOC To LAST_DOC
        For iControl = 0 To iControlMax
            bControl(iDocument, iControl) = False
        Next
    Next
    
    Set oActDoc = Nothing
End Sub

'''

Private Function Add(DocType As DBMDocumentTypeConstants, _
                     Database As DBMLib.Database) As Document
    Dim oDocument As New Document
    Dim oControl As Object
    Dim iControl As Integer
    
    LoadControl DocType, oControl, iControl
    
    With oDocument
        .DocType = DocType
        Set .Database = Database
        Set .Control = oControl
    End With
    
    oDocument.Control.Init
    
    oItems.Add oDocument
    
    oDocument.Index = oItems.Count
    
    Set Add = oDocument
End Function

Public Function Exists(lDoctype As DBMDocumentTypeConstants, _
                       oDatabase As DBMLib.Database, _
                       iFound As Integer) As Boolean
    Dim oDocument As Document
    Dim iItem As Integer
        
    iFound = 0
    
    For iItem = 1 To oItems.Count
        Set oDocument = oItems(iItem)
        If oDocument.DocType = lDoctype Then
            If Not oDocument.Database Is Nothing Then
                If oDocument.Database.Name = oDatabase.Name Then
                    iFound = iItem
                    Exit For
                End If
            End If
        End If
    Next
    
    Exists = (iFound > 0)
End Function

Private Sub HideDoc(oDocument As Document, _
                    bCancel As Boolean, _
                    lClosemode As DBMDocumentCloseModeConstants)
    With oDocument
        bCancel = False
        
        .Control.QueryClose lClosemode, bCancel
        
        If Not bCancel Then
            SetStatusBarStatus "Close '" + .Control.Caption + "'."
        
            If Not oActDoc Is Nothing Then
                If oActDoc.DocType = .DocType Then
                    If Not oActDoc.Database Is Nothing Then
                        If oActDoc.Database.Name = .Database.Name Then
                            oActDoc.Control.HideMenu
                            Set oActDoc = Nothing
                        End If
                    End If
                End If
            End If
            
            Select Case .DocType
                Case dbmDParameters: .Control.CloseIt
            End Select
               
            UnloadControl .DocType, .Control.Index
        
            Set .Control = Nothing
            Set .Database = Nothing
            
            SetStatusBarStatus ""
        End If
    End With
End Sub

Private Sub RemoveUnusedItems()
    Dim oDocument As Document
    Dim bDeleted As Boolean
    Dim iItem As Integer
    
    Do
        bDeleted = False
        For iItem = 1 To oItems.Count
            Set oDocument = oItems(iItem)

            If oDocument.Database Is Nothing Then
                oItems.Remove iItem
                bDeleted = True
                Exit For
            End If
        Next
    Loop Until bDeleted = False
    
    RenumberIndex
End Sub

Private Function FreeControl(lDoctype As DBMDocumentTypeConstants) As Integer
    Dim iControl As Integer
    Dim iFree As Integer
    
    iFree = -1
    
    For iControl = 0 To iControlMax
        If bControl(lDoctype, iControl) = False Then
            iFree = iControl
            Exit For
        End If
    Next
    
    FreeControl = iFree
End Function

Private Sub LoadControl(lDoctype As DBMDocumentTypeConstants, _
                        oControl As Object, _
                        iControl As Integer)
    iControl = FreeControl(lDoctype)
    
    If iControl > 0 Then
        With FRM
            Select Case lDoctype
                Case dbmDDatabase:              Load .docDatabase(iControl)
                Case dbmDInformation:           Load .docInformation(iControl)
                Case dbmDBackup:                Load .docBackup(iControl)
                Case dbmDRestore:               Load .docRestore(iControl)
                Case dbmDMedia:                 Load .docMedia(iControl)
                Case dbmDHistory:               Load .docHistory(iControl)
                Case dbmDBackupCheck:           Load .docBackupCheck(iControl)
                Case dbmDKernelTrace:           Load .docKernelTrace(iControl)
                Case dbmDFiles:                 Load .docFiles(iControl)
                Case dbmDShows:                 Load .docShows(iControl)
                Case dbmDParameters:            Load .docParameters(iControl)
                Case dbmDUsers:                 Load .docUsers(iControl)
                Case dbmDDevSpaceData:          Load .docDevSpaceData(iControl)
                Case dbmDDevSpaceLog:           Load .docDevSpaceLog(iControl)
                Case dbmDLogMode:               Load .docLogMode(iControl)
                Case dbmDTermcharSets:          Load .docTermcharSets(iControl)
                Case dbmDMapcharSets:           Load .docMapcharSets(iControl)
                Case dbmDLoadSysTabs:           Load .docLoadSysTabs(iControl)
                Case dbmDEvents:                Load .docEvents(iControl)
                Case dbmDCommand:               Load .docCommand(iControl)
                Case dbmDEventMonitor:          Load .docEventMonitor(iControl)
                Case dbmDIndexUse:              Load .docIndexUse(iControl)
                Case dbmDIndexRecovery:         Load .docIndexRecovery(iControl)
                Case dbmDUpdStat:               Load .docUpdateStatistics(iControl)
                Case dbmDVerify:                Load .docVerify(iControl)
                Case dbmDBadDevspaces:          Load .docBadDevSpaces(iControl)
                Case dbmDRestartUntil:          Load .docRestartUntil(iControl)
            End Select
        End With
    End If
    
    Set oControl = GetControl(lDoctype, iControl)
    
    bControl(lDoctype, iControl) = True
End Sub

Private Sub UnloadControl(lDoctype As DBMDocumentTypeConstants, _
                          iControl As Integer)
    Dim oControl As Object
    
    Set oControl = GetControl(lDoctype, iControl)
    
    oControl.Reset
    
    If iControl > 0 Then
        oControl.CleanUp
        Unload oControl
    Else
        oControl.Visible = False
    End If
    
    bControl(lDoctype, iControl) = False
End Sub

Private Function GetControl(lDoctype As DBMDocumentTypeConstants, _
                            iControl As Integer) As Object
    Dim oControl As Object
                          
    With FRM
        Select Case lDoctype
            Case dbmDDatabase:               Set oControl = .docDatabase(iControl)
            
            Case dbmDInformation:            Set oControl = .docInformation(iControl)
            
            Case dbmDBackup:                 Set oControl = .docBackup(iControl)
            Case dbmDRestore:                Set oControl = .docRestore(iControl)
            Case dbmDMedia:                  Set oControl = .docMedia(iControl)
            Case dbmDHistory:                Set oControl = .docHistory(iControl)
            Case dbmDBackupCheck:            Set oControl = .docBackupCheck(iControl)
                                            
            Case dbmDKernelTrace:            Set oControl = .docKernelTrace(iControl)
            Case dbmDFiles:                  Set oControl = .docFiles(iControl)
            Case dbmDShows:                  Set oControl = .docShows(iControl)
            Case dbmDEventMonitor:           Set oControl = .docEventMonitor(iControl)
            Case dbmDIndexUse:               Set oControl = .docIndexUse(iControl)
            Case dbmDIndexRecovery:          Set oControl = .docIndexRecovery(iControl)
            Case dbmDUpdStat:                Set oControl = .docUpdateStatistics(iControl)
            Case dbmDVerify:                 Set oControl = .docVerify(iControl)
            Case dbmDBadDevspaces:           Set oControl = .docBadDevSpaces(iControl)
                                            
            Case dbmDParameters:             Set oControl = .docParameters(iControl)
            Case dbmDUsers:                  Set oControl = .docUsers(iControl)
            Case dbmDDevSpaceData:           Set oControl = .docDevSpaceData(iControl)
            Case dbmDDevSpaceLog:            Set oControl = .docDevSpaceLog(iControl)
            Case dbmDLogMode:                Set oControl = .docLogMode(iControl)
            Case dbmDTermcharSets:           Set oControl = .docTermcharSets(iControl)
            Case dbmDMapcharSets:            Set oControl = .docMapcharSets(iControl)
            Case dbmDLoadSysTabs:            Set oControl = .docLoadSysTabs(iControl)
            Case dbmDEvents:                 Set oControl = .docEvents(iControl)
            Case dbmDCommand:                Set oControl = .docCommand(iControl)
            Case dbmDRestartUntil:           Set oControl = .docRestartUntil(iControl)
            
        End Select
    End With
    
    Set GetControl = oControl
End Function

Public Function CommandEnabled(lDoctype As DBMDocumentTypeConstants) As Boolean
    Dim bEnabled As Boolean
    With FRM.CML.CommandItems
        Select Case lDoctype
            Case dbmDDatabase:               bEnabled = .Item(cmdIViewDetails).Enabled
            
            Case dbmDInformation:            bEnabled = .Item(cmdIIVersions).Enabled
            
            Case dbmDBackup:                 bEnabled = .Item(cmdIBComplete).Enabled Or _
                                                        .Item(cmdIBIncremental).Enabled Or _
                                                        .Item(cmdIBLog).Enabled Or _
                                                        .Item(cmdIBAutoLog).Enabled
                                                        
            Case dbmDRestore:                bEnabled = .Item(cmdIRDatabase).Enabled
            Case dbmDMedia:                  bEnabled = .Item(cmdICMedia).Enabled
            Case dbmDHistory:                bEnabled = .Item(cmdIIHistory).Enabled
            Case dbmDBackupCheck:            bEnabled = .Item(cmdICBackup).Enabled
                                            
            Case dbmDKernelTrace:            bEnabled = .Item(cmdICKernelTrace).Enabled
            Case dbmDFiles:                  bEnabled = .Item(cmdICFiles).Enabled
            Case dbmDShows:                  bEnabled = .Item(cmdICServer).Enabled
            Case dbmDEventMonitor:           bEnabled = .Item(cmdICEventMonitor).Enabled
            Case dbmDIndexUse:               bEnabled = .Item(cmdITIndexUse).Enabled
            Case dbmDIndexRecovery:          bEnabled = .Item(cmdIRIndex).Enabled
            Case dbmDUpdStat:                bEnabled = .Item(cmdITOptStat).Enabled
            Case dbmDVerify:                 bEnabled = .Item(cmdICDatabase).Enabled
            Case dbmDBadDevspaces:           bEnabled = .Item(cmdIRDevspaces).Enabled
                                            
            Case dbmDParameters:             bEnabled = .Item(cmdICParameters).Enabled
            Case dbmDUsers:                  bEnabled = .Item(cmdICUsers).Enabled
            Case dbmDDevSpaceData:           bEnabled = .Item(cmdICDevSpaceData).Enabled
            Case dbmDDevSpaceLog:            bEnabled = .Item(cmdICDevSpaceLog).Enabled
            Case dbmDLogMode:                bEnabled = .Item(cmdICLogMode).Enabled
            Case dbmDTermcharSets:           bEnabled = .Item(cmdICTermcharSets).Enabled
            Case dbmDMapcharSets:            bEnabled = .Item(cmdICMapcharSets).Enabled
            Case dbmDLoadSysTabs:            bEnabled = .Item(cmdICLoadSysTabs).Enabled
            Case dbmDEvents:                 bEnabled = .Item(cmdICEvents).Enabled
            Case dbmDCommand:                bEnabled = .Item(cmdICommandLine).Enabled
            Case dbmDRestartUntil:           bEnabled = .Item(cmdIRestartUntil).Enabled
        End Select
    End With
    
    CommandEnabled = bEnabled
End Function

Private Sub SetStatusBarStatus(sStatus As String)
    FRM.SBR.Panels.Item("Status").Text = sStatus
End Sub

Private Sub RenumberIndex()
    Dim oDocument As Document
    Dim iItem As Integer
    
    For iItem = 1 To oItems.Count
        Set oDocument = oItems(iItem)
        oDocument.Index = iItem
    Next
End Sub


