VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "RegSettings"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' --- GPL ---
'
' Copyright (C) 1998-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const MAX_SERVER = 10

Const KEY_SETTINGS = "Settings"

Const VAL_WINLEFT = "WindowLeft"
Const VAL_WINTOP = "WindowTop"
Const VAL_WINWIDTH = "WindowWidth"
Const VAL_WINHEIGHT = "WindowHeight"
Const VAL_WINSTATE = "WindowState"

Const VAL_SPLITTERSIZE = "SplitterSize"

Const VAL_PANPERCENT = "PanelPercent"
Const VAL_SDBPANPERC = "SingleDBPanelPercent"

Const VAL_VIEWTBR = "ViewToolbar"
Const VAL_VIEWSBR = "ViewStatusbar"
Const VAL_VIEWTRE = "ViewTree"
Const VAL_VIEWMNU = "ViewMenu"

Const VAL_DBVIEW = "DatabaseView"

Const VAL_SERVER = "Server"

Const VAL_AUTOCON = "AutoConnect"
Const VAL_SUBFLDS = "SubFolders"

Const VAL_COLORDER = "ColOrder"
Const VAL_COLWIDTH = "ColWidth"

Const VAL_PROTFONTNAME = "ProtocolFontName"
Const VAL_PROTFONTSIZE = "ProtocolFontSize"

Public WindowLeft As Long
Public WindowTop As Long
Public WindowWidth As Long
Public WindowHeight As Long
Public WindowState As Long
Public SplitterSize As Long

Public PanelPercent As String
Public PanelHidden As String

Public SingleDBPanelPercent As String

Public ViewToolbar As Boolean
Public ViewStatusbar As Boolean
Public ViewTree As Boolean
Public ViewMenu As Boolean

Public DatabaseView As Long

Private sServer(1 To MAX_SERVER) As String

Public AutoConnect As Boolean
Public SubFolders As Boolean

Public ColOrder As String
Public ColWidth As String

Public ProtocolFontName As String
Public ProtocolFontSize As String

'

Private sRoot As String
Private oReg As DBMReg.Registry
Private oKey As DBMReg.Key

'''

Public Property Let Root(NewVal As String)
    sRoot = NewVal
End Property

Public Property Set Registry(NewVal As DBMReg.Registry)
    Set oReg = NewVal
End Property

Public Property Get Key() As DBMReg.Key
    Set Key = oKey
End Property

Public Property Get Server(Index As Integer) As String
    If Index > 0 And Index <= MAX_SERVER Then
        Server = sServer(Index)
    End If
End Property

Public Property Let Server(Index As Integer, NewVal As String)
    If Index > 0 And Index <= MAX_SERVER Then
        sServer(Index) = NewVal
    End If
End Property

'''

Public Sub Init()
    Dim oValue As DBMReg.Value
    Dim iServer As Integer
    
    SetDefaults
    
    Set oKey = oReg.Keys.Add(HKEY_CURRENT_USER, sRoot & "\" & KEY_SETTINGS)
    
    With oKey
        If Not .Check(False, False) Then
            If Not .Create(False, False) Then
                Exit Sub
            End If
        End If
        
        .Values.Add VAL_WINLEFT, WindowLeft, REG_DWORD
        .Values.Add VAL_WINTOP, WindowTop, REG_DWORD
        .Values.Add VAL_WINWIDTH, WindowWidth, REG_DWORD
        .Values.Add VAL_WINHEIGHT, WindowHeight, REG_DWORD
        .Values.Add VAL_WINSTATE, WindowState, REG_DWORD
        .Values.Add VAL_SPLITTERSIZE, SplitterSize, REG_DWORD
        
        .Values.Add VAL_PANPERCENT, PanelPercent, REG_SZ
        .Values.Add VAL_SDBPANPERC, SingleDBPanelPercent, REG_SZ
        
        .Values.Add VAL_VIEWTBR, ViewToolbar, REG_DWORD
        .Values.Add VAL_VIEWSBR, ViewStatusbar, REG_DWORD
        .Values.Add VAL_VIEWTRE, ViewTree, REG_DWORD
        .Values.Add VAL_VIEWMNU, ViewMenu, REG_DWORD
    
        .Values.Add VAL_DBVIEW, DatabaseView, REG_DWORD
        
        For iServer = 1 To MAX_SERVER
            .Values.Add VAL_SERVER & CStr(iServer), Server(iServer), REG_SZ
        Next
    
        .Values.Add VAL_AUTOCON, AutoConnect, REG_DWORD
        .Values.Add VAL_SUBFLDS, SubFolders, REG_DWORD
        
        .Values.Add VAL_COLORDER, ColOrder, REG_SZ
        .Values.Add VAL_COLWIDTH, ColWidth, REG_SZ
        
        .Values.Add VAL_PROTFONTNAME, ProtocolFontName, REG_SZ
        .Values.Add VAL_PROTFONTSIZE, ProtocolFontSize, REG_SZ
        
        If .OpenKey() Then
            For Each oValue In oKey.Values
                If Not oValue.CheckValue(.Key) Then
                    If Not oValue.CreateValue(.Key) Then
                        Exit Sub
                    End If
                Else
                    If Not oValue.QueryValue(.Key) Then
                        Exit Sub
                    End If
                End If
            Next
            .CloseKey
        End If
    End With
        
    GetValues
End Sub

'''

Private Sub Class_Terminate()
    SetValues
    
    If Not oKey.Check(False, True) Then
        oKey.Create False, True
    Else
        oKey.SetValues False
    End If
End Sub

Private Sub SetDefaults()
    Dim lWidth As Long
    Dim lHeight As Long
    Dim iServer As Integer
    
    lWidth = GetSystemMetrics(SM_CXSCREEN)
    lHeight = GetSystemMetrics(SM_CYSCREEN)
        
    WindowWidth = lWidth * 0.7
    WindowHeight = lHeight * 0.7
    WindowLeft = (lWidth - WindowWidth) \ 2
    WindowTop = (lHeight - WindowHeight) \ 2
    WindowState = vbNormal
    
    SplitterSize = 4
    
    PanelPercent = "100020080085015015085015085"
    SingleDBPanelPercent = "100085015020080020080"
    
    ViewToolbar = True
    ViewStatusbar = True
    ViewTree = True
    ViewMenu = True
    
    DatabaseView = 0
    
    For iServer = 1 To MAX_SERVER
        Server(iServer) = ""
    Next
    
    AutoConnect = True
    SubFolders = False
    
    ColOrder = ""
    ColWidth = ""
    
    ProtocolFontName = "Courier New"
    ProtocolFontSize = "10"
End Sub

Private Sub GetValues()
    Dim iServer As Integer
    
    With oKey.Values
        WindowLeft = .Item(VAL_WINLEFT).Value
        WindowTop = .Item(VAL_WINTOP).Value
        WindowWidth = .Item(VAL_WINWIDTH).Value
        WindowHeight = .Item(VAL_WINHEIGHT).Value
        WindowState = .Item(VAL_WINSTATE).Value
        SplitterSize = .Item(VAL_SPLITTERSIZE).Value
        
        PanelPercent = .Item(VAL_PANPERCENT).Value
        SingleDBPanelPercent = .Item(VAL_SDBPANPERC).Value
        
        ViewToolbar = .Item(VAL_VIEWTBR).Value
        ViewStatusbar = .Item(VAL_VIEWSBR).Value
        ViewTree = .Item(VAL_VIEWTRE).Value
        ViewMenu = .Item(VAL_VIEWMNU).Value
        
        DatabaseView = .Item(VAL_DBVIEW).Value
        
        For iServer = 1 To MAX_SERVER
            Server(iServer) = .Item(VAL_SERVER & CStr(iServer)).Value
        Next
        
        AutoConnect = .Item(VAL_AUTOCON).Value
        SubFolders = .Item(VAL_SUBFLDS).Value
        
        ColOrder = .Item(VAL_COLORDER).Value
        ColWidth = .Item(VAL_COLWIDTH).Value
        
        ProtocolFontName = .Item(VAL_PROTFONTNAME).Value
        ProtocolFontSize = .Item(VAL_PROTFONTSIZE).Value
    End With
End Sub

Private Sub SetValues()
    Dim iServer As Integer
    
    With oKey.Values
        .Item(VAL_WINLEFT).Value = WindowLeft
        .Item(VAL_WINTOP).Value = WindowTop
        .Item(VAL_WINWIDTH).Value = WindowWidth
        .Item(VAL_WINHEIGHT).Value = WindowHeight
        .Item(VAL_WINSTATE).Value = WindowState
        .Item(VAL_SPLITTERSIZE).Value = SplitterSize
        
        .Item(VAL_PANPERCENT).Value = PanelPercent
        .Item(VAL_SDBPANPERC).Value = SingleDBPanelPercent
        
        .Item(VAL_VIEWTBR).Value = ViewToolbar
        .Item(VAL_VIEWSBR).Value = ViewStatusbar
        .Item(VAL_VIEWTRE).Value = ViewTree
        .Item(VAL_VIEWMNU).Value = ViewMenu
        
        .Item(VAL_DBVIEW).Value = DatabaseView
        
        For iServer = 1 To MAX_SERVER
            .Item(VAL_SERVER & CStr(iServer)).Value = Server(iServer)
        Next
    
        .Item(VAL_AUTOCON).Value = AutoConnect
        .Item(VAL_SUBFLDS).Value = SubFolders
        
        .Item(VAL_COLORDER).Value = ColOrder
        .Item(VAL_COLWIDTH).Value = ColWidth
        
        .Item(VAL_PROTFONTNAME).Value = ProtocolFontName
        .Item(VAL_PROTFONTSIZE).Value = ProtocolFontSize
    End With
End Sub

