VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsCommandLine"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' --- GPL ---
'
' Copyright (C) 1999-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const CML_PREFIX_SLASH = "/"
Const CML_PREFIX_MINUS = "-"
Const CML_PREFIX_PLUS = "+"

Public Enum CommandLinePrefixConstants
    cmlPrefixSLASH
    cmlPrefixMinus
    cmlPrefixPlus
End Enum

Const QUOTE1 = """"
Const QUOTE2 = "'"

Private sCommandLine As String

Private oItems As New Collection

Public Property Get Text() As String
    Text = sCommandLine
End Property
Public Property Let Text(ByVal NewVal As String)
    sCommandLine = Trim(NewVal)
    GetCmdLineArgs
End Property

Public Property Get Count() As Long
    Count = oItems.Count
End Property

Public Property Get Item(ByVal Index As Variant) As clsCommandLineArgument
Attribute Item.VB_UserMemId = 0
    Set Item = oItems(Index)
End Property

Public Property Get NewEnum()
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    Set NewEnum = oItems.[_NewEnum]
End Property
'
'
'
Public Sub Add(Prefix As CommandLinePrefixConstants, _
               Name As String, _
               IsOptional As Boolean, _
               ValueDesc As String, _
               Description As String)

    Dim oCommandLineArg As New clsCommandLineArgument
    
    With oCommandLineArg
        .Prefix = Prefix
        .Name = UCase(Name)
        .IsOptional = IsOptional
        .ValueDesc = ValueDesc
        .Description = Description
        .Exists = False
    End With
    
    oItems.Add oCommandLineArg, oCommandLineArg.Name
End Sub

Public Function Exists(ByVal Index As Variant) As Boolean
    Dim oCommandLineArg As clsCommandLineArgument
    
    On Error Resume Next
    Set oCommandLineArg = oItems(Index)
    Exists = (Err.Number = 0)
End Function

Public Sub Remove(ByVal Index As Variant)
    oItems.Remove Index
End Sub

Public Sub Clear()
    Dim l As Long
    For l = 1 To oItems.Count
        oItems.Remove 1
    Next
End Sub
'
' Private Sub/Function
'
Private Sub GetCmdLineArgs()
    Dim oCmdLineArg As clsCommandLineArgument
    Dim lLen As Long
    Dim lPos As Long
    Dim lPrefix As CommandLinePrefixConstants
    Dim sName As String
    
    lLen = Len(sCommandLine)
    
    If lLen = 0 Then Exit Sub

    lPos = 1
    Do While lPos < lLen
        If IsPrefix(lPos, lPrefix) Then
            If lPos <= lLen Then
                sName = GetName(lPos, lLen)
                If Exists(sName) Then
                    Set oCmdLineArg = oItems(sName)
                Else
                    Set oCmdLineArg = New clsCommandLineArgument
                    oItems.Add oCmdLineArg, sName
                End If
                
                With oCmdLineArg
                    .Prefix = lPrefix
                    .Name = sName
                    If lPos <= lLen Then
                        .Value = GetValue(lPos, lLen)
                    End If
                    .Exists = True
                End With
                
                Set oCmdLineArg = Nothing
                
            End If
        Else
            lPos = lPos + 1
        End If
    Loop
End Sub

Private Function IsPrefix(lPos As Long, lPrefix As CommandLinePrefixConstants) As Boolean
    Dim sPrefix As String
    
    sPrefix = Mid$(sCommandLine, lPos, 1)
    
    IsPrefix = True
    
    Select Case sPrefix
        Case CML_PREFIX_SLASH: lPrefix = cmlPrefixSLASH
        Case CML_PREFIX_MINUS: lPrefix = cmlPrefixMinus
        Case CML_PREFIX_PLUS:  lPrefix = cmlPrefixPlus
        Case Else:             IsPrefix = False
    End Select
    
    If IsPrefix Then
        lPos = lPos + 1
    End If
End Function

Private Function IsQuote(lPos As Long) As Boolean
    If Mid$(sCommandLine, lPos, 1) = QUOTE1 Or Mid(sCommandLine, lPos, 1) = QUOTE2 Then
        lPos = lPos + 1
        IsQuote = True
    Else
        IsQuote = False
    End If
End Function

Private Function GetName(lPos As Long, lLen As Long) As String
    Dim lPos1 As Long
    Dim lPos2 As Long
    
    Blanks lPos, lLen

    lPos1 = lPos
    Do While lPos <= lLen
        If Mid$(sCommandLine, lPos, 1) <> " " Then
            lPos = lPos + 1
        Else
            Exit Do
        End If
    Loop
    
    lPos2 = lPos
    
    GetName = Mid$(sCommandLine, lPos1, lPos2 - lPos1)
End Function

Private Function GetValue(lPos As Long, lLen As Long) As String
    Dim sValue As String
    Dim bInQuote As Boolean
    Dim lPos1 As Long
    Dim lPos2 As Long
    Dim lPrefix As CommandLinePrefixConstants
    
    sValue = ""
    bInQuote = False
    
    Blanks lPos, lLen
    
    lPos1 = lPos
    
    Do While lPos <= lLen
        If IsQuote(lPos) Then
            bInQuote = Not bInQuote
        End If
        
        If IsPrefix(lPos, lPrefix) Then
            lPos = lPos - 1
            Exit Do
        End If
        
        If Mid$(sCommandLine, lPos, 1) <> " " Then
            lPos = lPos + 1
        Else
            If bInQuote Then
                lPos = lPos + 1
            Else
                Blanks lPos, lLen
            End If
        End If
    Loop
    
    lPos2 = lPos
    
    sValue = Trim$(Mid$(sCommandLine, lPos1, lPos2 - lPos1))
    
    If Left$(sValue, 1) = QUOTE1 Or Left$(sValue, 1) = QUOTE2 Then sValue = Mid$(sValue, 2)
    If Right$(sValue, 1) = QUOTE1 Or Right$(sValue, 1) = QUOTE2 Then sValue = Left$(sValue, Len(sValue) - 1)
    
    GetValue = sValue
End Function

Private Sub Blanks(lPos As Long, lLen As Long)
    Do While lPos <= lLen
        If Mid$(sCommandLine, lPos, 1) = " " Then
            lPos = lPos + 1
        Else
            Exit Do
        End If
    Loop
End Sub

