Attribute VB_Name = "DBMSRVExternalIDs"
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Public Const EID_COL_STATE = 1
Public Const EID_COL_ID = 2
Public Const EID_COL_ACTION = 3
Public Const EID_COL_STAMP = 4

Const EID_COL_MAX = EID_COL_STAMP

Public EIDColPos(1 To EID_COL_MAX) As Integer
Public EIDColLen(1 To EID_COL_MAX) As Integer

'''

Const DBMSRV_CMD_EXTIDS_OPEN = "backup_ext_ids_get"
Const DBMSRV_CMD_EXTIDS_LIST = "backup_ext_ids_list"
Const DBMSRV_CMD_EXTIDS_LISTNEXT = "backup_ext_ids_listnext"
Const DBMSRV_CMD_EXTIDS_CLOSE = "backup_ext_ids_forget"

Const DBMSRV_OPT_EXTIDS_CON = "CONTINUE"
Const DBMSRV_OPT_EXTIDS_END = "END"

Public Function ExternalIDsOpen(oDatabase As Database, _
                                ByVal sMediumName As String, _
                                Optional ByVal sDatabase As String = "", _
                                Optional ByVal sServer As String = "") As Boolean
    Dim sCommand As String
    Dim sData As String

    ExternalIDsOpen = False

    sCommand = DBMSRV_CMD_EXTIDS_OPEN + " " + sMediumName
    If Len(sDatabase) > 0 Then
        sCommand = sCommand + " " + sDatabase
        If Len(sServer) > 0 Then
            sCommand = sCommand + " " + sServer
        End If
    End If
    
    If oDatabase.ServerObject.Command(sCommand, sData) Then
        ExternalIDsOpen = True
    End If
End Function

Public Function ExternalIDsGet(oDatabase As Database, _
                               oExternalIDs As ExternalIDs) As Boolean

    Dim sData As String
    Dim sLine As String
    Dim lPos1 As Long
    Dim bContinue As Boolean
    Dim sCommand As String
    Dim bFirstLine As Boolean
    Dim oExternalID As ExternalID

    ExternalIDsGet = False

    bContinue = True
    sCommand = DBMSRV_CMD_EXTIDS_LIST

    bFirstLine = True

    Do While bContinue
        If oDatabase.ServerObject.Command(sCommand, sData) Then
            sCommand = DBMSRV_CMD_EXTIDS_LISTNEXT
            ExternalIDsGet = True
            lPos1 = 1
            sLine = Trim(oDatabase.ServerObject.GetLine(sData, lPos1))
            bContinue = (UCase(sLine) = DBMSRV_OPT_EXTIDS_CON)
            Do While lPos1 < Len(sData)
                sLine = RTrim(oDatabase.ServerObject.GetLine(sData, lPos1))
                
                If bFirstLine Then
                    ExternalIdsInitColumns sLine
                    bFirstLine = False
                End If
                
                Set oExternalID = Nothing
                Set oExternalID = New ExternalID
                oExternalID.SetPropertiesFromLine sLine
                oExternalIDs.Add oExternalID
            Loop
        Else
            ExternalIDsGet = False
            Exit Do
        End If
    Loop
End Function

Public Function ExternalIDsClose(oDatabase As Database) As Boolean
    Dim sData As String
    
    ExternalIDsClose = False
    
    If oDatabase.ServerObject.Command(DBMSRV_CMD_EXTIDS_CLOSE, sData) Then
        ExternalIDsClose = True
    End If
End Function

'''

Private Sub ExternalIdsInitColumns(sLine As String)
    Dim iCol As Integer
    Dim iPos1 As Integer
    Dim iPos2 As Integer
    Dim iLen As Integer
    
    iCol = 1
    iPos1 = 1
    iLen = Len(sLine)
    Do
        iPos2 = InStr(iPos1, sLine, "|")
        If iPos2 > 0 Then
            EIDColPos(iCol) = iPos1
            EIDColLen(iCol) = iPos2 - iPos1
            iPos1 = iPos2 + 1
        Else
            iPos2 = Len(sLine) + 1
            EIDColPos(iCol) = iPos1
            EIDColLen(iCol) = iPos2 - iPos1
        End If
        iCol = iCol + 1
    Loop While iPos2 < iLen And iCol <= EID_COL_MAX
End Sub

