Attribute VB_Name = "DBMSRVParam"
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Public Const DBMSRV_CMD_PARAM_STARTSESSION = "param_startsession"
Public Const DBMSRV_CMD_PARAM_ABORTSESSION = "param_abortsession"
Public Const DBMSRV_CMD_PARAM_COMMITSESSION = "param_commitsession"

Const DBMSRV_CMD_PARAM_INIT = "param_init"
Const DBMSRV_CMD_PARAM_COPY = "param_copy"

Const DBMSRV_CMD_PARAM_DIRECTDEL = "param_directdel"
Const DBMSRV_CMD_PARAM_DIRECTGET = "param_directget"
Const DBMSRV_CMD_PARAM_DIRECTGETALL = "param_directgetall"
Const DBMSRV_CMD_PARAM_DIRECTPUT = "param_directput"

Const DBMSRV_CMD_PARAM_EXTGET = "param_extget"
Const DBMSRV_CMD_PARAM_EXTGETALL = "param_extgetall"

Const DBMSRV_CMD_PARAM_GETFULL = "param_getfull"
Const DBMSRV_CMD_PARAM_GETTYPE = "param_gettype"
Const DBMSRV_CMD_PARAM_GETDEFAULT = "param_getdefault"
Public Const DBMSRV_CMD_PARAM_GETVALUE = "param_getvalue"
Const DBMSRV_CMD_PARAM_GETPROPERTIES = "param_getproperties"
Const DBMSRV_CMD_PARAM_GETHELP = "param_gethelp"
Const DBMSRV_CMD_PARAM_GETEXPLAIN = "param_getexplain"

Public Const DBMSRV_CMD_PARAM_PUT = "param_put"
Const DBMSRV_CMD_PARAM_PUTCONFIRM = "param_putconfirm"

Public Const DBMSRV_CMD_PARAM_CHECKALL = "param_checkall"

Const DBMSRV_CMD_PARAM_VERSIONS = "param_versions"
Const DBMSRV_CMD_PARAM_RESTORE = "param_restore"
Const DBMSRV_CMD_PARAM_RMFILE = "param_rmfile"

Public Const DBMSRV_CMD_PARAM_ADDDEVSPACE = "param_adddevspace"
Const DBMSRV_CMD_PARAM_GETDEVSPACE = "param_getdevspace"
Public Const DBMSRV_CMD_PARAM_DELDEVSPACE = "param_deldevspace"

Const CHECK_ERR_REQUEST = "REQUEST"
Const CHECK_ERR_CONSTRAINT = "CONSTRAINT"
Const CHECK_ERR_MANDATORY = "MANDATORY"
Const CHECK_ERR_UNKNOWN = "UNKNOWN"

'''

Public Function StartSession(oServerObject As DBMSOB.ServerObject) As Boolean
    Dim sData As String
    
    StartSession = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_STARTSESSION, sData) Then
        StartSession = True
    End If
End Function

Public Function AbortSession(oServerObject As DBMSOB.ServerObject) As Boolean
    Dim sData As String
    
    AbortSession = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_ABORTSESSION, sData) Then
        AbortSession = True
    End If
End Function

Public Function CommitSession(oServerObject As DBMSOB.ServerObject, _
                              lStatus As DBMParameterStatusConstants, _
                              sBadParam As String, _
                              sOwnValue As String, _
                              sCompValue As String, _
                              Optional ByVal bNoClose As Boolean = False) As Boolean
    Dim sData As String
    Dim sCmd As String
    
    CommitSession = False
    
    sCmd = DBMSRV_CMD_PARAM_COMMITSESSION
    If bNoClose Then
        sCmd = sCmd + " NOCLOSE"
    End If
    
    If oServerObject.Command(sCmd, sData) Then
        CommitSession = True
    Else
        EvaluateCheckError oServerObject, lStatus, sBadParam, sOwnValue, sCompValue
    End If
End Function

'''

Public Function Init(oServerObject As DBMSOB.ServerObject, _
                     sInstanceType As String) As Boolean
    Dim sData As String
    Dim sCmd As String
    
    Init = False
    
    If Len(sInstanceType) > 0 Then
        sCmd = DBMSRV_CMD_PARAM_INIT + " " + sInstanceType
    Else
        sCmd = DBMSRV_CMD_PARAM_INIT
    End If
    
    If oServerObject.Command(sCmd, sData) Then
        Init = True
    End If
End Function

Public Function Copy(oServerObject As DBMSOB.ServerObject, _
                     sSourceDatabase As String) As Boolean
    Dim sData As String
    
    Copy = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_COPY + " " + sSourceDatabase, sData) Then
        Copy = True
    End If
End Function

'''

Public Function DirectGet(oServerObject As DBMSOB.ServerObject, _
                          sName As String, _
                          sValue As String) As Boolean
    Dim sData As String
    Dim lPos As Long
    Dim sLine As String
    
    DirectGet = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_DIRECTGET + " " + sName, sData) Then
        lPos = 1
        sLine = Trim$(oServerObject.GetLine(sData, lPos))
        lPos = 1
        sName = Trim$(oServerObject.GetField(sLine, vbTab, lPos))
        sValue = Trim$(oServerObject.GetField(sLine, vbTab, lPos))
        DirectGet = True
    End If
End Function

Public Function DirectGetAll(oServerObject As DBMSOB.ServerObject, _
                             sData As String) As Boolean
    DirectGetAll = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_DIRECTGET, sData) Then
        DirectGetAll = True
    End If
End Function

Public Function DirectPut(oServerObject As DBMSOB.ServerObject, _
                          sName As String, _
                          sValue As String) As Boolean
    Dim sData As String
    
    DirectPut = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_DIRECTPUT + " " + sName + " " + sValue, sData) Then
        DirectPut = True
    End If
End Function

Public Function DirectDelete(oServerObject As DBMSOB.ServerObject, _
                             sName As String) As Boolean
    Dim sData As String
    
    DirectDelete = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_DIRECTDEL + " " + sName, sData) Then
        DirectDelete = True
    End If
End Function

'''

Public Function ExtGet(oServerObject As DBMSOB.ServerObject, _
                       sName As String, _
                       sData As String) As Boolean
    ExtGet = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_EXTGET + " " + sName, sData) Then
        ExtGet = True
    End If
End Function

Public Function ExtGetAll(oServerObject As DBMSOB.ServerObject, _
                          sSelection As String, _
                          sData As String) As Boolean
    ExtGetAll = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_EXTGETALL + " " + sSelection, sData) Then
        ExtGetAll = True
    End If
End Function

'''

Public Function GetFullProp(oServerObject As DBMSOB.ServerObject, _
                            sName As String, _
                            sValue As String, _
                            sType As String, _
                            sDefault As String, _
                            sHelp As String, _
                            sExplain As String, _
                            sProperties As String) As Boolean
    Dim sData As String
    Dim lLen As Long
    Dim DataPos As Long
    Dim linePos As Long
    Dim sLine As String
    Dim sLineFeed As String
    Dim bNext As Boolean
    
    GetFullProp = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_GETFULL + " " + sName, sData) Then
        DataPos = 1
        lLen = Len(sData)
        
        ' Type
        sLine = Trim$(oServerObject.GetLine(sData, DataPos))
        linePos = 1
        sType = Trim$(oServerObject.GetField(sLine, vbTab, linePos))
        
        ' Default
        sLine = Trim$(oServerObject.GetLine(sData, DataPos))
        linePos = 1
        sDefault = Trim$(oServerObject.GetField(sLine, vbTab, linePos))
        
        ' Value
        sLine = Trim$(oServerObject.GetLine(sData, DataPos))
        linePos = 1
        sValue = Trim$(oServerObject.GetField(sLine, vbTab, linePos))
        
        ' Properties
        bNext = False
        sProperties = ""
        While (DataPos < lLen And Not bNext)
            sLine = Trim$(oServerObject.GetLine(sData, DataPos))
            If (sLine <> "HELP") Then
                sProperties = sProperties + sLine + vbLf
            Else
                bNext = True
            End If
        Wend
        
        ' Help
        bNext = False
        sHelp = ""
        sLineFeed = ""
        While (DataPos < lLen And Not bNext)
            sLine = oServerObject.GetLine(sData, DataPos)
            If (sLine <> "EXPLAIN") Then
                sHelp = sHelp & sLineFeed & sLine
                sLineFeed = vbCrLf
            Else
                bNext = True
            End If
        Wend
        
        ' Explain
        sExplain = ""
        sLineFeed = ""
        While (DataPos < lLen)
            sLine = oServerObject.GetLine(sData, DataPos)
            sExplain = sExplain & sLineFeed & sLine
            sLineFeed = vbCrLf
        Wend
        
        GetFullProp = True
    End If
End Function

Public Function GetFullData(oServerObject As DBMSOB.ServerObject, _
                            sName As String, _
                            sData As String) As Boolean
    GetFullData = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_GETFULL + " " + sName, sData) Then
        GetFullData = True
    End If
End Function

Public Function GetType(oServerObject As DBMSOB.ServerObject, _
                        sName As String, _
                        sData As String) As Boolean
    GetType = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_GETTYPE + " " + sName, sData) Then
        GetType = True
    End If
End Function

Public Function GetDefault(oServerObject As DBMSOB.ServerObject, _
                           sName As String, _
                           sData As String) As Boolean
    GetDefault = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_GETDEFAULT + " " + sName, sData) Then
        GetDefault = True
    End If
End Function

Public Function GetValue(oServerObject As DBMSOB.ServerObject, _
                         sName As String, _
                         sValue As String) As Boolean
    Dim sData As String
    Dim lPos As Long
    
    GetValue = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_GETVALUE + " " + sName, sData) Then
        lPos = 1
        sValue = Trim$(oServerObject.GetLine(sData, lPos))
        GetValue = True
    End If
End Function

Public Function GetProperties(oServerObject As DBMSOB.ServerObject, _
                              sName As String, _
                              sData As String) As Boolean
    GetProperties = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_GETPROPERTIES + " " + sName, sData) Then
        GetProperties = True
    End If
End Function

Public Function GetHelp(oServerObject As DBMSOB.ServerObject, _
                        sName As String, _
                        sData As String) As Boolean
    GetHelp = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_GETHELP + " " + sName, sData) Then
        GetHelp = True
    End If
End Function

Public Function GetExplain(oServerObject As DBMSOB.ServerObject, _
                           sName As String, _
                           sData As String) As Boolean
    GetExplain = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_GETEXPLAIN + " " + sName, sData) Then
        GetExplain = True
    End If
End Function

'''

Public Function PutOnly(oServerObject As DBMSOB.ServerObject, _
                        sName As String, _
                        sValue As String) As Boolean
    Dim sData As String
    
    PutOnly = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_PUT + " " + sName + " " + sValue, sData) Then
        PutOnly = True
    End If
End Function

Public Function PutConfirm(oServerObject As DBMSOB.ServerObject, _
                           sName As String, _
                           sValue As String) As Boolean
    Dim sData As String
    
    PutConfirm = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_PUTCONFIRM + " " + sName + " " + sValue, sData) Then
        PutConfirm = True
    End If
End Function

'''

Public Function CheckAll(oServerObject As DBMSOB.ServerObject, _
                         sMode As String, _
                         lStatus As DBMParameterStatusConstants, _
                         sBadParam As String, _
                         sOwnValue As String, _
                         sCompValue As String) As Boolean
    Dim sData As String
    
    CheckAll = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_CHECKALL & " " & sMode, sData) Then
        CheckAll = True
    Else
        EvaluateCheckError oServerObject, lStatus, sBadParam, sOwnValue, sCompValue
    End If
End Function

'''

Public Function Versions(oServerObject As DBMSOB.ServerObject, _
                         sData As String) As Boolean
    Versions = False

    If oServerObject.Command(DBMSRV_CMD_PARAM_VERSIONS, sData) Then
        Versions = True
    End If
End Function


Public Function Restore(oServerObject As DBMSOB.ServerObject, _
                        lVersion As Long) As Boolean
    Dim sData As String
    
    Restore = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_RESTORE + " " + CStr(lVersion), sData) Then
        Restore = True
    End If
End Function

Public Function RemoveFile(oServerObject As DBMSOB.ServerObject) As Boolean
    Dim sData As String
  
    RemoveFile = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_RMFILE, sData) Then
        RemoveFile = True
    End If
End Function

Public Function FileExists(oServerObject As DBMSOB.ServerObject, _
                           Optional ByVal lVersion As Long = 0) As Boolean
    Dim sData As String
    Dim sLine As String
    Dim lPos1 As Long
    Dim bFound As Boolean
    
    FileExists = False
    
    If Versions(oServerObject, sData) Then
        lPos1 = 1
        bFound = False
        Do While (lPos1 < Len(sData)) And Not bFound
            sLine = Trim(oServerObject.GetLine(sData, lPos1))
            If Len(sLine) > 0 Then
                If lVersion = 0 Then
                    bFound = True
                Else
                    If Mid$(sLine, Len(sLine) - 2, 1) = "." Then
                        If Right$(sLine, 2) = Format$(lVersion, "00") Then
                            bFound = True
                        End If
                    End If
                End If
            End If
        Loop
        
        FileExists = bFound
    End If
End Function

'''

Public Function AddDevspace(oServerObject As DBMSOB.ServerObject, _
                            lNumber As Long, _
                            sClass As String, _
                            sName As String, _
                            sType As String, _
                            sSize As String) As Boolean
    Dim sData As String
  
    AddDevspace = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_ADDDEVSPACE + " " + CStr(lNumber) + " " + sClass + " " + sName + " " + sType + " " + sSize, sData) Then
        AddDevspace = True
    End If
End Function

Public Function GetDevspace(oServerObject As DBMSOB.ServerObject, _
                            lNumber As Long, _
                            sClass As String, _
                            sName As String, _
                            sType As String, _
                            sSize As String) As Boolean
    Dim sData As String
    Dim lPos As Long
    Dim sLine As String
  
    GetDevspace = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_GETDEVSPACE + " " + CStr(lNumber) + " " + sClass, sData) Then
        lPos = 1
        sName = Trim$(oServerObject.GetLine(sData, lPos))
        sType = Trim$(oServerObject.GetLine(sData, lPos))
        sSize = Trim$(oServerObject.GetLine(sData, lPos))
        GetDevspace = True
    End If
End Function

Public Function DeleteDevspace(oServerObject As DBMSOB.ServerObject, _
                                lNumber As Long, _
                                sClass As String) As Boolean
    
    Dim sData As String
  
    DeleteDevspace = False
    
    If oServerObject.Command(DBMSRV_CMD_PARAM_DELDEVSPACE + " " + CStr(lNumber) + " " + sClass, sData) Then
        DeleteDevspace = True
    End If
End Function

'''

Private Sub EvaluateCheckError(oServerObject As DBMSOB.ServerObject, _
                               lStatus As DBMParameterStatusConstants, _
                               sBadParam As String, _
                               sOwnValue As String, _
                               sCompValue As String)
    Dim sData As String
    Dim nLinePos As Long
    Dim nDataPos As Long
    Dim sLine As String
    Dim sStatus As String
    
    If (oServerObject.ReturnCode = DBMSRV_ERR_XPCHECK) Then
        sData = oServerObject.ErrorData
    
        nDataPos = 1
        sLine = Trim$(oServerObject.GetLine(sData, nDataPos))
        
        nLinePos = 1
        sBadParam = Trim$(oServerObject.GetField(sLine, vbTab, nLinePos))
        sStatus = Trim$(oServerObject.GetField(sLine, vbTab, nLinePos))
        Select Case UCase$(sStatus)
            Case CHECK_ERR_REQUEST:     lStatus = dbmPSRequest
            Case CHECK_ERR_CONSTRAINT:  lStatus = dbmPSConstraint
            Case CHECK_ERR_MANDATORY:   lStatus = dbmPSMandatory
            Case CHECK_ERR_UNKNOWN:     lStatus = dbmPSUnknown
        End Select
        
        sOwnValue = Trim$(oServerObject.GetLine(sData, nDataPos))
        sCompValue = Trim$(oServerObject.GetLine(sData, nDataPos))
    Else
        sStatus = ""
        sBadParam = ""
        sOwnValue = ""
        sCompValue = ""
    End If
End Sub

