Attribute VB_Name = "DBMSRVSQL"
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Public Const DBMSRV_SQL_ROWNOTFOUND = 100

Const DBMSRV_SQL_CONTINUE = "CONTINUE"
Const DBMSRV_SQL_END = "END"

Const DBMSRV_SQL_ERROR = -24988

Public Function Connect(oServerObject As DBMSOB.ServerObject, _
                        Optional sUsername As String = "", _
                        Optional sPassword As String = "") As Boolean
    Dim sCmd As String
    Dim sData As String
    
    Connect = False
    
    sCmd = DBMSRV_CMD_SQL_CONNECT
    
    If Len(sUsername) > 0 And Len(sPassword) > 0 Then
        sCmd = sCmd + " " + sUsername + "," + sPassword
    End If
    
    If oServerObject.Command(sCmd, sData) Then
        Connect = True
    End If
End Function

Public Function Release(oServerObject As DBMSOB.ServerObject) As Boolean
    Dim sData As String
    
    Release = False
    
    If oServerObject.Command(DBMSRV_CMD_SQL_RELEASE, sData) Then
        Release = True
    End If
End Function

Public Function Execute(oServerObject As DBMSOB.ServerObject, _
                        sStatement As String, _
                        Optional sResult As String = "", _
                        Optional bContinue As Boolean = False) As Boolean
    Dim sData As String
    
    Execute = False
    
    If oServerObject.Command(DBMSRV_CMD_SQL_EXECUTE & " " & sStatement, sData) Then
        Call ReadResult(oServerObject, sData, sResult, bContinue)
        Execute = True
    End If
End Function

Public Function Info(ByVal oServerObject As DBMSOB.ServerObject, _
                     ByVal sStatement As String, _
                     sInfo As String) As Boolean
    Dim sData As String
    Dim lLinePos As Long
    
    Info = False
    
    If oServerObject.Command(DBMSRV_CMD_SQL_INFO & " " & sStatement, sData) Then
        Info = True
        lLinePos = 0
        sInfo = Trim(oServerObject.GetLine(sData, lLinePos))
    End If
End Function

Public Function Fetch(ByVal oServerObject As DBMSOB.ServerObject, _
                      sResult As String, _
                      bContinue As Boolean) As Boolean
    Dim sData As String
    
    Fetch = False
    
    If oServerObject.Command(DBMSRV_CMD_SQL_FETCH, sData) Then
        Call ReadResult(oServerObject, sData, sResult, bContinue)
        Fetch = True
    End If
End Function

Public Function RecreateIndex(oServerObject As DBMSOB.ServerObject, _
                              Optional sSchema As String = "", _
                              Optional sTableName As String = "", _
                              Optional sIndexname As String = "") As Boolean
    Dim sData As String
    Dim sCmd As String
    
    RecreateIndex = False
    
    sCmd = DBMSRV_CMD_SQL_RECREATEINDEX
    
    If Len(sSchema) > 0 And _
       Len(sTableName) > 0 And _
       Len(sIndexname) > 0 _
    Then
        sCmd = sCmd + " """ + sSchema + """.""" + sTableName + """.""" + sIndexname + """"
    End If
    
    If oServerObject.Command(sCmd, sData) Then
        RecreateIndex = True
    End If
End Function

Public Function UpdateStat(oServerObject As DBMSOB.ServerObject, _
                           Optional sSpec As String = "") As Boolean
    Dim sData As String
    Dim sCmd As String
    
    UpdateStat = False
    
    sCmd = DBMSRV_CMD_SQL_UPDATESTAT
    
    If Len(sSpec) > 0 Then
        sCmd = sCmd + " " + sSpec
    End If
    
    If oServerObject.Command(sCmd, sData) Then
        UpdateStat = True
    End If
End Function

Public Function NextRow(sResult As String) As String
    Dim nPos As Long
    Dim sRow As String
    
    sRow = ""
    
    nPos = InStr(1, sResult, vbCrLf)
    If nPos > 0 Then
        sRow = Left(sResult, nPos - 1)
        sResult = Mid(sResult, nPos + 2)
    Else
        sRow = sResult
        sResult = ""
    End If
    
    NextRow = Trim(sRow)
End Function

Public Function NextField(sRow As String) As String
    Dim nPos As Long
    Dim sField As String
    
    sField = ""
    
    nPos = InStr(1, sRow, ";")
    If nPos > 0 Then
        sField = Left(sRow, nPos - 1)
        sRow = Mid(sRow, nPos + 1)
    Else
        sField = sRow
        sRow = ""
    End If
    
    sField = Trim(sField)
    
    If (Left(sField, 1) = "'") Then
        sField = Mid(sField, 2, Len(sField) - 2)
    End If
    
    NextField = sField
    
End Function

Public Function LastError(ByVal oServerObject As DBMSOB.ServerObject) As Long

    Dim sError As String
    Dim nError As Long

    If (oServerObject.ReturnCode = DBMSRV_SQL_ERROR) Then

        sError = Left(oServerObject.ErrorData, InStr(1, oServerObject.ErrorData, ",") - 1)
        nError = Val(sError)

    End If

    LastError = nError

End Function

'''

Private Sub ReadResult(ByVal oServerObject As DBMSOB.ServerObject, _
                       ByVal sData As String, _
                       sResult As String, _
                       bContinue As Boolean)
    Dim sContinue As String
    Dim lDataPos As Long
    Dim lDataLen As Long
    Dim sLine As String
    Dim sLineFeed As String
    
    bContinue = False
    sResult = ""
    lDataPos = 1
    lDataLen = Len(sData)
    
    If (lDataLen > 0) Then
    
        ' get continue mark
        sContinue = Trim(oServerObject.GetLine(sData, lDataPos))
        
        ' check continue mark
        If (sContinue = DBMSRV_SQL_CONTINUE Or sContinue = DBMSRV_SQL_END) Then
        ' set continue flag
            bContinue = (sContinue = DBMSRV_SQL_CONTINUE)
            
            ' get records
            While lDataPos < lDataLen
                sLine = oServerObject.GetLine(sData, lDataPos)
                sResult = sResult & sLineFeed & sLine
                sLineFeed = vbCrLf
            Wend
        
        End If
    
    End If

End Sub

