Attribute VB_Name = "DBMSRVUser"
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const KEY_PASSWORD = "PASSWORD"
Const KEY_SERVERRIGHTS = "SERVERRIGHTS"
Const KEY_GUIRIGHTS = "GUIRIGHTS"
Const KEY_SECONDPWD = "SECONDPASSWORD"
Const KEY_DBMUSER = "DBMUSER"
Const KEY_SQLUSER = "SQLUSER"
Const KEY_SQLUSERMODE = "SQLUSERMODE"
Const KEY_DISABLED = "DISABLED"
Const KEY_SYSTEMNAME = "SYSTEMNAME"
Const KEY_SYSTEMPWD = "SYSTEMPASSWORD"
Const KEY_COMMENT = "COMMENT"

Const KEY_YES = "YES"
Const KEY_NO = "NO"

Public Function GetUsers(ByVal oServerObject As DBMSOB.ServerObject, _
                         ByVal oUsers As Users) As Boolean
    Dim bOK As Boolean
    Dim sData As String
    Dim sLine As String
    Dim lPos1 As Long
    Dim sDBM As String
    Dim sDBA As String
    Dim sName As String
    Dim oUser As User
        
    bOK = False
    
    If oServerObject.Command(DBMSRV_CMD_DBM_CONFIGGET + " " + DBMSRV_CONFIGGET_KEYDBM, sData) Then
        lPos1 = 1
        sDBM = Trim(oServerObject.GetLine(sData, lPos1))
    End If
    If oServerObject.Command(DBMSRV_CMD_DBM_CONFIGGET + " " + DBMSRV_CONFIGGET_KEYDBA, sData) Then
        lPos1 = 1
        sDBA = Trim(oServerObject.GetLine(sData, lPos1))
    End If
    
    If oServerObject.Command(DBMSRV_CMD_USER_GETALL, sData) Then
        bOK = True
        lPos1 = 1
        Do While lPos1 < Len(sData) And bOK
            sName = Trim(oServerObject.GetLine(sData, lPos1))
            Set oUser = New User
            oUser.Name = sName
            oUser.UserType = dbmUTDefault
            If UCase(sName) = UCase(sDBM) Then oUser.UserType = dbmUTDBM
            If UCase(sName) = UCase(sDBA) Then oUser.UserType = dbmUTDBA
            bOK = GetUser(oServerObject, oUser)
            If bOK Then
                oUsers.Add oUser, oUser.Name
            End If
        Loop
    End If
    
    GetUsers = bOK
End Function

Public Function GetUser(ByVal oServerObject As DBMSOB.ServerObject, _
                        ByVal oUser As User) As Boolean
    Dim bOK As Boolean
    Dim sData As String
    Dim sLine As String
    Dim lPos1 As Long
    Dim lPos2 As Long
    Dim sKey As String
    Dim sValue As String
    
    bOK = False
    
    If oServerObject.Command(DBMSRV_CMD_USER_GET + " " + oUser.Name, sData) Then
        bOK = True
        lPos1 = 1
        Do While lPos1 < Len(sData) And bOK
            sLine = Trim$(oServerObject.GetLine(sData, lPos1))
            lPos2 = InStr(1, sLine, "=")
            If lPos2 > 0 Then
                sKey = Left$(sLine, lPos2 - 1)
                sValue = Mid$(sLine, lPos2 + 1)
            Else
                sKey = sLine
                sValue = ""
            End If
        
            Select Case sKey
                Case KEY_SERVERRIGHTS:
                Case KEY_GUIRIGHTS:
                Case KEY_SECONDPWD:     oUser.SecondPwdEnabled = (UCase(sValue) = KEY_YES)
                Case KEY_DBMUSER:       oUser.DBMUser = (UCase(sValue) = KEY_YES)
                Case KEY_SQLUSER:       oUser.SQLUser = (UCase(sValue) = KEY_YES)
                Case KEY_SQLUSERMODE:   oUser.SQLUserModeString = sValue
                Case KEY_DISABLED:      oUser.Disabled = (UCase(sValue) = KEY_YES)
                Case KEY_SYSTEMNAME:    oUser.SystemName = sValue
                Case KEY_COMMENT:       oUser.Comment = sValue
            End Select
            
            Select Case oUser.UserType
                Case dbmUTDBM: oUser.Comment = "First Database Manager Operator"
                Case dbmUTDBA: oUser.Comment = "Database System Administrator (SYSDBA)"
            End Select
        Loop
        bOK = GetUserRights(oServerObject, oUser)
    End If
    
    GetUser = bOK
End Function

Public Function GetUserRights(ByVal oServerObject As DBMSOB.ServerObject, _
                              ByVal oUser As User) As Boolean
    Dim bOK As Boolean
    Dim sData As String
    Dim sLine As String
    Dim lPos1 As Long
    Dim lPos2 As Long
    Dim sName As String
    Dim bValue As String
    Dim sDesc As String
    
    Dim oUserRight As UserRight
    
    bOK = False
    
    If oServerObject.Command(DBMSRV_CMD_USER_GETRIGHTS + " " + oUser.Name + " " + DBMSRV_OPT_USERRIGHTS_SERVER, sData) Then
        bOK = True
        
        oUser.ServerRights.Clear
        
        lPos1 = 1
        Do While lPos1 < Len(sData)
            sLine = Trim$(oServerObject.GetLine(sData, lPos1))
            lPos2 = 1
            Set oUserRight = New UserRight
            With oUserRight
                .Name = Trim$(oServerObject.GetField(sLine, vbTab, lPos2))
                .Value = (Trim$(oServerObject.GetField(sLine, vbTab, lPos2)) = "+")
                .Description = Trim$(oServerObject.GetField(sLine, vbTab, lPos2))
            End With
            oUser.ServerRights.Add oUserRight, oUserRight.Name
        Loop
    End If
    
    GetUserRights = bOK
End Function

Public Function PutUsers(ByVal oServerObject As DBMSOB.ServerObject, _
                         ByVal oUsers As Users) As Boolean
    Dim oUser As User
                        
    PutUsers = True
    
    For Each oUser In oUsers
        If Not PutUser(oServerObject, oUser) Then
            PutUsers = False
            Exit For
        End If
    Next
End Function

Public Function PutUser(ByVal oServerObject As DBMSOB.ServerObject, _
                        ByVal oUser As User, _
                        Optional ByVal bSecondPwd As Boolean = False, _
                        Optional ByVal bSystemPwd As Boolean = False) As Boolean
    Dim sDef As String
    Dim sData As String
    Dim sRights As String
    
    PutUser = False
    
    With oUser
        sDef = .Name
        
        sRights = .ServerRightString
        If Len(sRights) > 0 Then
            sDef = sDef + " " + KEY_SERVERRIGHTS + "=" + .ServerRightString
        End If
        
        sRights = .GUIRightString
        If Len(sRights) > 0 Then
            sDef = sDef + " " + KEY_GUIRIGHTS + "=" + .GUIRightString
        End If
        
        sDef = sDef + " " + KEY_DBMUSER + "="
        If .DBMUser Then sDef = sDef + KEY_YES Else sDef = sDef + KEY_NO
        
        sDef = sDef + " " + KEY_SQLUSER + "="
        If .SQLUser Then sDef = sDef + KEY_YES Else sDef = sDef + KEY_NO
        
        sDef = sDef + " " + KEY_SQLUSERMODE + "=" + .SQLUserModeString
        
        sDef = sDef + " " + KEY_DISABLED + "="
        If .Disabled Then sDef = sDef + KEY_YES Else sDef = sDef + KEY_NO + .SecondPwd
        
        sDef = sDef + " " + KEY_SYSTEMNAME + "=" + .SystemName
        
        sDef = sDef + " " + KEY_COMMENT + "=""" + .Comment + """"
        
        If bSecondPwd Then
            sDef = sDef + " " + KEY_SECONDPWD + "=" + .SecondPwd
        End If
        
        If bSystemPwd Then
            sDef = sDef + " " + KEY_SYSTEMPWD + "=" + .SystemPwd
        End If
    End With
    
    If oServerObject.Command(DBMSRV_CMD_USER_PUT + " " + sDef, sData) Then
        PutUser = True
    End If
End Function

Public Function PutUserPassword(ByVal oServerObject As DBMSOB.ServerObject, _
                                ByVal oUser As User) As Boolean
    Dim sDef As String
    Dim sData As String
    
    PutUserPassword = False
    
    With oUser
        sDef = .Name
        sDef = sDef + " " + KEY_PASSWORD + "=" + .Password
    End With
    
    If oServerObject.Command(DBMSRV_CMD_USER_PUT + " " + sDef, sData) Then
        PutUserPassword = True
    End If
End Function

Public Function PutUserSecondPassword(ByVal oServerObject As DBMSOB.ServerObject, _
                                      ByVal oUser As User) As Boolean
    Dim sDef As String
    Dim sData As String
    
    PutUserSecondPassword = False
    
    With oUser
        sDef = .Name
        sDef = sDef + " " + KEY_SECONDPWD + "=" + .SecondPwd
    End With
    
    If oServerObject.Command(DBMSRV_CMD_USER_PUT + " " + sDef, sData) Then
        PutUserSecondPassword = True
    End If
End Function

Public Function PutUserSystemPassword(ByVal oServerObject As DBMSOB.ServerObject, _
                                      ByVal oUser As User) As Boolean
    Dim sDef As String
    Dim sData As String
    
    PutUserSystemPassword = False
    
    With oUser
        sDef = .Name
        sDef = sDef + " " + KEY_SYSTEMPWD + "=" + .SystemPwd
    End With
    
    If oServerObject.Command(DBMSRV_CMD_USER_PUT + " " + sDef, sData) Then
        PutUserSystemPassword = True
    End If
End Function

Public Function CreateUser(ByVal oServerObject As DBMSOB.ServerObject, _
                           ByVal oUser As User) As Boolean
    Dim sData As String
    
    CreateUser = False
    
    If oServerObject.Command(DBMSRV_CMD_USER_CREATE + " " + oUser.Name + "," + oUser.Password, sData) Then
        CreateUser = True
    End If
End Function

Public Function DeleteUser(ByVal oServerObject As DBMSOB.ServerObject, _
                           ByVal oUser As User) As Boolean
    Dim sData As String
    
    DeleteUser = False
    
    If oServerObject.Command(DBMSRV_CMD_USER_DELETE + " " + oUser.Name, sData) Then
        DeleteUser = True
    End If
End Function

