VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "DatabaseEventArray"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const MAX_ITEMS = 100

Private oParent As Object

Private sName As String

Private lState(1 To MAX_ITEMS)  As DBMDatabaseEventStateConstants
Private lPriority(1 To MAX_ITEMS) As DBMDatabaseEventPrioConstants
Private lValue1(1 To MAX_ITEMS) As Long
Private lValue2(1 To MAX_ITEMS) As Long
Private lCount As Long

Private sDescription As String

'''

Public Property Get ObjectType() As DBMObjectTypeConstants
    ObjectType = dbmOTDatabaseEventArray
End Property

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Friend Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As Database
    Set Database = oParent.Database
End Property

'''

Public Property Get Name() As String
    Name = sName
End Property
Friend Property Let Name(NewVal As String)
    sName = NewVal
End Property

Public Property Get State(ByVal Index As Long) As DBMDatabaseEventStateConstants
    If Index >= 1 And Index <= MAX_ITEMS Then
        State = lState(Index)
    End If
End Property

Public Property Get StateString(ByVal Index As Long) As String
    If Index >= 1 And Index <= MAX_ITEMS Then
        StateString = Database.DatabaseEvents.GetStateString(lState(Index))
    End If
End Property

Public Property Get Priority(ByVal Index As Long) As DBMDatabaseEventPrioConstants
    If Index >= 1 And Index <= MAX_ITEMS Then
        Priority = lPriority(Index)
    End If
End Property

Public Property Get PriorityString(ByVal Index As Long) As String
    If Index >= 1 And Index <= MAX_ITEMS Then
        PriorityString = Database.DatabaseEvents.GetPriorityString(lPriority(Index))
    End If
End Property

Public Property Get Value1(ByVal Index As Long) As Long
    If Index >= 1 And Index <= MAX_ITEMS Then
        Value1 = lValue1(Index)
    End If
End Property

Public Property Get Value2(ByVal Index As Long) As Long
    If Index >= 1 And Index <= MAX_ITEMS Then
        Value2 = lValue2(Index)
    End If
End Property

Public Property Get Description() As String
    Description = sDescription
End Property
Friend Property Let Description(NewVal As String)
    sDescription = NewVal
End Property

Public Property Get Count() As Long
    Count = lCount
End Property

Public Property Get MaxItems() As Long
    MaxItems = MAX_ITEMS
End Property

'''

Friend Function Reset()
    lCount = 0
End Function

Friend Function AddItem(State As String, _
                        Priority As String, _
                        Value1 As Long, _
                        Value2 As Long)
                        
    If lCount < MAX_ITEMS Then
        lCount = lCount + 1
        lState(lCount) = Database.DatabaseEvents.GetStateValue(State)
        lPriority(lCount) = Database.DatabaseEvents.GetPriorityValue(Priority)
        lValue1(lCount) = Value1
        lValue2(lCount) = Value2
    End If
End Function

'''

Public Function EventsOn(ByVal FirstValue As Long, Optional ByVal Below As Boolean = False) As Boolean
    Dim bOK As Boolean
    Dim lValue As Long
    
    bOK = True
    
    lValue = FirstValue
    
    With Database
        If .UtilConnect Then
            If EventOn(lValue) Then
                Do While GetNextAboveValue(lValue)
                    If Not EventOn(lValue) Then
                        bOK = False
                        Exit Do
                    End If
                Loop
            Else
                bOK = False
            End If
            .UtilRelease
        End If
    End With
        
    EventsOn = bOK
End Function

Private Function EventOn(ByVal lValue As Long) As Boolean
    Dim bOK As Boolean
    Dim sCmd As String
    Dim sData As String
    
    bOK = False
    
    With Database
        sCmd = DBMSRV_CMD_EVENT_SET
        sCmd = sCmd + " " + sName
        sCmd = sCmd + " " + .DatabaseEvents.GetPriorityString(dbmDBEventPrioHigh)
        sCmd = sCmd + " " + CStr(lValue)
        
        If .ServerObject.Command(sCmd, sData) Then
            bOK = True
        End If
    End With

    EventOn = bOK
End Function

Public Function EventsOff() As Boolean
    Dim bOK As Boolean
    Dim lItem As Long
    Dim sCmd As String
    Dim sData As String
    
    bOK = False
    
    With Database
        If lCount > 0 Then
            If .UtilConnect Then
                For lItem = 1 To lCount
                    If lState(lItem) = dbmDBEventStateOn Then
                    
                        sCmd = DBMSRV_CMD_EVENT_DELETE
                        sCmd = sCmd + " " + sName
                        sCmd = sCmd + " " + CStr(lValue1(lItem))

                        If .ServerObject.Command(sCmd, sData) Then
                            bOK = True
                            lState(lItem) = dbmDBEventStateOff
                        Else
                            bOK = False
                        End If
                    Else
                        bOK = True
                    End If
                    If Not bOK Then Exit For
                Next
                .UtilRelease
            End If
        End If
    End With
    
    If bOK Then Reset
        
    EventsOff = bOK
End Function

'''

Public Sub CleanUp()
    Set oParent = Nothing
End Sub

'''

Private Sub Class_Initialize()
    InitObject ObjectType
End Sub
Private Sub Class_Terminate()
    TermObject ObjectType
End Sub

'''

Private Function GetNextAboveValue(lValue As Long) As Boolean
    Dim lNew As Long
    Dim bOK  As Boolean
    
    bOK = True
    
    If lValue < 50 Then
        lNew = (lValue \ 10) * 10
        lNew = lNew + 10
    Else
        If lValue < 90 Then
            lNew = (lValue \ 5) * 5
            lNew = lNew + 5
        Else
            If lValue < 99 Then
                lNew = lValue + 1
            Else
                bOK = False
            End If
        End If
    End If
    
    lValue = lNew
    
    GetNextAboveValue = bOK
End Function
