VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "DatabaseEvents"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Public Enum DBMDatabaseEventStateConstants
    dbmDBEventStateOff
    dbmDBEventStateOn
End Enum

Const EVENT_STATE_OFF = "OFF"
Const EVENT_STATE_ON = "ON"

Public Enum DBMDatabaseEventPrioConstants
    dbmDBEventPrioNIL
    dbmDBEventPrioLow
    dbmDBEventPrioMedium
    dbmDBEventPrioHigh
End Enum
    
Const EVENT_PRIO_NIL = "NIL"
Const EVENT_PRIO_LOW = "LOW"
Const EVENT_PRIO_MED = "MEDIUM"
Const EVENT_PRIO_HIG = "HIGH"

Private oParent As Object

Private oItems As New Collection
Private oEventDataAbove As DatabaseEventArray
Private oEventDataBelow As DatabaseEventArray
Private oEventLogAbove As DatabaseEventArray

'''

Public Property Get ObjectType() As DBMObjectTypeConstants
    ObjectType = dbmOTDatabaseEvents
End Property

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Friend Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As Database
    Set Database = oParent.Database
End Property

Public Property Get Count() As Long
    Count = oItems.Count
End Property

Public Property Get Item(Index As Variant) As DatabaseEvent
Attribute Item.VB_UserMemId = 0
    Set Item = oItems(Index)
End Property

Public Property Get ItemByType(ByVal EventType As DBMDatabaseEventTypeConstants) As DatabaseEvent
    Dim lItem As Long
    Dim lCount As Long
    
    lCount = oItems.Count
        
    For lItem = 1 To lCount
        If oItems(lItem).EventType = EventType Then
            Set ItemByType = oItems(lItem)
            Exit Property
        End If
    Next
    
    Set ItemByType = Nothing
End Property

Public Property Get NewEnum()
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    Set NewEnum = oItems.[_NewEnum]
End Property

Public Property Get EventDataAbove() As DatabaseEventArray
    If oEventDataAbove Is Nothing Then
        Set oEventDataAbove = New DatabaseEventArray
        With oEventDataAbove
            Set .Parent = Me
            .Name = EVENT_DB_ABOVE_LIMIT
            .Description = "Database above limit"
        End With
    End If
    Set EventDataAbove = oEventDataAbove
End Property

Public Property Get EventDataBelow() As DatabaseEventArray
    If oEventDataBelow Is Nothing Then
        Set oEventDataBelow = New DatabaseEventArray
        With oEventDataBelow
            Set .Parent = Me
            .Name = EVENT_DB_BELOW_LIMIT
            .Description = "Database below limit"
        End With
    End If
    Set EventDataBelow = oEventDataBelow
End Property

Public Property Get EventLogAbove() As DatabaseEventArray
    If oEventLogAbove Is Nothing Then
        Set oEventLogAbove = New DatabaseEventArray
        With oEventLogAbove
            Set .Parent = Me
            .Name = EVENT_LOG_ABOVE_LIMIT
            .Description = "Log above limit"
        End With
    End If
    Set EventLogAbove = oEventLogAbove
End Property

'''

Friend Sub Add(DatabaseEvent As DatabaseEvent)
    oItems.Add DatabaseEvent, DatabaseEvent.Key
    DatabaseEvent.Index = oItems.Count
End Sub

Public Sub Clear()
    Dim lCount As Long
    Dim l As Long
    
    lCount = oItems.Count
    
    For l = 1 To lCount
        oItems(1).CleanUp
        oItems.Remove 1
    Next
End Sub

Public Sub CleanUp()
    Clear
    
    If Not oEventDataAbove Is Nothing Then oEventDataAbove.CleanUp
    If Not oEventDataBelow Is Nothing Then oEventDataBelow.CleanUp
    If Not oEventLogAbove Is Nothing Then oEventLogAbove.CleanUp
    
    Set oEventDataAbove = Nothing
    Set oEventDataBelow = Nothing
    Set oEventLogAbove = Nothing
    
    Set oParent = Nothing
End Sub

Public Sub Remove(Index As Variant)
    Dim lStart As Long
    
    lStart = oItems(Index).Index
    
    oItems(Index).CleanUp
    oItems.Remove Index
    
    SetIndex lStart
End Sub

Public Function Exists(Index As Variant) As Boolean
    Dim oDatabaseEvent As DatabaseEvent
    
    On Error Resume Next

    Set oDatabaseEvent = oItems(Index)
    
    If Err.Number Then
        Exists = False
    Else
        Exists = True
    End If
End Function

Public Function OpenDatabaseEvents() As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    Clear
    
    EventDataAbove.Reset
    EventDataBelow.Reset
    EventLogAbove.Reset
    
    If Database.Check() Then
        With Database
            If DBMSRVEve.EventList(.ServerObject, Me) Then
                bOK = True
            End If
        End With
    End If
    
    OpenDatabaseEvents = bOK
End Function

Public Function GetStateString(ByVal lState As DBMDatabaseEventStateConstants) As String
    Select Case lState
        Case dbmDBEventStateOff: GetStateString = EVENT_STATE_OFF
        Case dbmDBEventStateOn:  GetStateString = EVENT_STATE_ON
    End Select
End Function
Public Function GetStateValue(sStateString As String) As DBMDatabaseEventStateConstants
    Select Case UCase$(Trim$(sStateString))
        Case EVENT_STATE_OFF: GetStateValue = dbmDBEventStateOff
        Case EVENT_STATE_ON:  GetStateValue = dbmDBEventStateOn
    End Select
End Function

Public Function GetPriorityString(ByVal lPriority As DBMDatabaseEventPrioConstants) As String
    Select Case lPriority
        Case dbmDBEventPrioNIL:     GetPriorityString = EVENT_PRIO_NIL
        Case dbmDBEventPrioLow:     GetPriorityString = EVENT_PRIO_LOW
        Case dbmDBEventPrioMedium:  GetPriorityString = EVENT_PRIO_MED
        Case dbmDBEventPrioHigh:    GetPriorityString = EVENT_PRIO_HIG
    End Select
End Function
Public Function GetPriorityValue(sPriorityString As String) As DBMDatabaseEventPrioConstants
    Select Case UCase$(Trim$(sPriorityString))
        Case EVENT_PRIO_NIL: GetPriorityValue = dbmDBEventPrioNIL
        Case EVENT_PRIO_LOW: GetPriorityValue = dbmDBEventPrioLow
        Case EVENT_PRIO_MED: GetPriorityValue = dbmDBEventPrioMedium
        Case EVENT_PRIO_HIG: GetPriorityValue = dbmDBEventPrioHigh
    End Select
End Function

'''

Private Sub Class_Initialize()
    InitObject ObjectType
End Sub
Private Sub Class_Terminate()
    TermObject ObjectType
End Sub

Private Sub SetIndex(ByVal lStart As Long)
    Dim oDatabaseEvent As DatabaseEvent
    Dim lCount As Long
    Dim l As Long
    
    lCount = oItems.Count
    
    For l = lStart To lCount
        Set oDatabaseEvent = oItems(l)
        oDatabaseEvent.Index = l
    Next
End Sub


