VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "File"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const FUNC_OPENFILE = "OpenFile"
Const FUNC_READLINE = "ReadLine"
Const FUNC_READTEXT = "ReadText"
Const FUNC_CLOSEFILE = "CloseFile"

Public Enum DBMFileTypeConstants
    dbmFTAscii
    dbmFTBinary
End Enum

Const FT_ASC = "ASCII"
Const FT_BIN = "BINARY"

Private oParent As Object

Private lIndex As Long

Private dAt As Date

Private sName As String
Private lFileType As DBMFileTypeConstants
Private lSize As Long
Private dModified As Date
Private sDescription As String
Private sFilename As String

Private sLine As String
Private sText As String

'''

Public Property Get ObjectType() As DBMObjectTypeConstants
    ObjectType = dbmOTFile
End Property

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Friend Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As Database
    Set Database = oParent.Database
End Property

Public Property Get Index() As Long
    Index = lIndex
End Property
Friend Property Let Index(ByVal NewVal As Long)
    lIndex = NewVal
End Property

'''

Public Property Get At() As Date
    At = dAt
End Property

Public Property Get Name() As String
    Name = sName
End Property
Friend Property Let Name(NewVal As String)
    sName = NewVal
End Property

Public Property Let FileType(ByVal NewVal As DBMFileTypeConstants)
    lFileType = NewVal
End Property
Friend Property Get FileType() As DBMFileTypeConstants
    FileType = lFileType
End Property

Public Property Get FileTypeString() As String
    Select Case lFileType
        Case dbmFTAscii:    FileTypeString = FT_ASC
        Case dbmFTBinary:   FileTypeString = FT_BIN
    End Select
End Property
Public Property Let FileTypeString(NewVal As String)
    Select Case UCase$(Trim$(NewVal))
        Case FT_ASC:    lFileType = dbmFTAscii
        Case FT_BIN:    lFileType = dbmFTBinary
    End Select
End Property

Public Property Get Size() As Long
    Size = lSize
End Property
Friend Property Let Size(ByVal NewVal As Long)
    lSize = NewVal
End Property

Public Property Get Modified() As Date
    Modified = dModified
End Property
Friend Property Let Modified(ByVal NewVal As Date)
    dModified = NewVal
End Property

Public Property Get Description() As String
    Description = sDescription
End Property
Friend Property Let Description(NewVal As String)
    sDescription = NewVal
End Property

Public Property Get Filename() As String
    Filename = sFilename
End Property
Friend Property Let Filename(NewVal As String)
    sFilename = NewVal
End Property

Public Property Get Line() As String
    Line = sLine
End Property

Public Property Get Text() As String
    Text = sText
End Property

'''

Public Function OpenFile(Optional ByVal Asynchron As Boolean = False) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Not Asynchron Then
            bOK = Execute(FUNC_OPENFILE, False)
        Else
            QUEUE.QueueItems.Add Me, FUNC_OPENFILE
            bOK = True
        End If
    End If
    
    OpenFile = bOK
End Function

Public Function ReadLine(Optional ByVal Asynchron As Boolean = False) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Not Asynchron Then
            bOK = Execute(FUNC_READLINE, False)
        Else
            QUEUE.QueueItems.Add Me, FUNC_READLINE
            bOK = True
        End If
    End If
    
    ReadLine = bOK
End Function

Public Function ReadText(Optional ByVal Asynchron As Boolean = False) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Not Asynchron Then
            bOK = Execute(FUNC_READTEXT, False)
        Else
            QUEUE.QueueItems.Add Me, FUNC_READTEXT
            bOK = True
        End If
    End If
    
    ReadText = bOK
End Function

Public Function CloseFile(Optional ByVal Asynchron As Boolean = False) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Not Asynchron Then
            bOK = Execute(FUNC_CLOSEFILE, False)
        Else
            QUEUE.QueueItems.Add Me, FUNC_CLOSEFILE
            bOK = True
        End If
    End If
    
    CloseFile = bOK
End Function

Public Function Execute(ByVal FunctionName As String, ByVal Asynchron As Boolean, Optional ByVal SenderID As Long) As Boolean
    Dim bOK As Boolean
    
    Database.Busy = True
    
    bOK = False
    
    Select Case FunctionName
        Case FUNC_OPENFILE:     bOK = ExecuteOpenFile()
        Case FUNC_READLINE:     bOK = ExecuteReadLine()
        Case FUNC_READTEXT:     bOK = ExecuteReadText()
        Case FUNC_CLOSEFILE:    bOK = ExecuteCloseFile()
    End Select
            
    Database.Busy = False
    
    Execute = bOK
End Function

Public Function EOF() As Boolean
    Database.Busy = True
    EOF = DBMSRVFile.FileReadEOF()
    Database.Busy = False
End Function

'''

Public Sub CleanUp()
    Set oParent = Nothing
End Sub

'''

Private Sub Class_Initialize()
    InitObject ObjectType
End Sub
Private Sub Class_Terminate()
    TermObject ObjectType
End Sub

'''

Private Function ExecuteOpenFile() As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    With Database
        If .Check() Then
            If DBMSRVFile.FileReadOpen(.ServerObject, sName, True) Then
                dAt = Now()
                bOK = True
            Else
                .LastError.SetByServerObject .ServerObject
            End If
        End If
        
        .EventFileAction Not bOK, dbmFileActionOpenFile, Me
    End With
    
    ExecuteOpenFile = bOK
End Function

Private Function ExecuteReadLine() As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    sLine = ""
            
    With Database
        If .Check() Then
            If Not DBMSRVFile.FileReadEOF() Then
                If DBMSRVFile.FileReadLine(Database.ServerObject, sLine) Then
                    bOK = True
                Else
                    .LastError.SetByServerObject .ServerObject
                End If
            Else
                .LastError.SetByServerObject .ServerObject
            End If
        End If
        
        .EventFileAction Not bOK, dbmFileActionReadLine, Me
    End With
    
    ExecuteReadLine = bOK
End Function

Private Function ExecuteReadText() As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    sText = ""
            
    With Database
        If .Check() Then
            If Not DBMSRVFile.FileReadEOF() Then
                If DBMSRVFile.FileReadText(Database.ServerObject, sText) Then
                    bOK = True
                Else
                    .LastError.SetByServerObject .ServerObject
                End If
            Else
                .LastError.SetByServerObject .ServerObject
            End If
        End If
        
        .EventFileAction Not bOK, dbmFileActionReadText, Me
    End With
    
    ExecuteReadText = bOK
End Function

Private Function ExecuteCloseFile() As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    sText = ""
    
    With Database
        If .Check() Then
            If DBMSRVFile.FileReadClose(Database.ServerObject) Then
                bOK = True
            Else
                .LastError.SetByServerObject .ServerObject
            End If
        End If
        
        .EventFileAction Not bOK, dbmFileActionCloseFile, Me
    End With
    
    ExecuteCloseFile = bOK
End Function

