VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "MapcharSet"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Ext_KEY = "RVB_UniqueId" ,"33F01F180222"
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Private oParent As Object

Private Const MAPCHAR_SQL_STATE1 = "SELECT DISTINCT UPPER(ASCII(CODE)) FROM MAPCHARSETS WHERE MAPCHARSETNAME = UPPER('"
Private Const MAPCHAR_SQL_STATE2 = "')"
Private Const MAPCHAR_SQL_VALUES1 = "SELECT HEX(INTERN), HEX(MAP_CODE) FROM MAPCHARSETS WHERE MAPCHARSETNAME = UPPER('"
Private Const MAPCHAR_SQL_VALUES2 = "')"

Private Const MAPCHAR_UTL_DROP = "DROP MAPCHAR SET "
Private Const MAPCHAR_UTL_ALTER = "ALTER MAPCHAR SET "
Private Const MAPCHAR_UTL_CREATE = "CREATE MAPCHAR SET "

Private Const MAPCHAR_UTL_AS = " AS "
Private Const MAPCHAR_UTL_WITH = " WITH "

Private Const MAPCHAR_UTL_DIGIT1 = " X'"
Private Const MAPCHAR_UTL_DIGIT2 = "' TO X'"
Private Const MAPCHAR_UTL_DIGIT3 = "'"

Private Const MAPCHAR_UTL_COMMA = ","

Private Const MAPCHAR_COUNT_MAX = 64
Private Const MAPCHAR_INTERN_LEN = 2
Private Const MAPCHAR_MAPCODE_LEN = 4

Private Const MAPCHAR_TYPE_ASCII = "ASCII"
Private Const MAPCHAR_TYPE_EBCDIC = "EBCDIC"

Public Enum MapcharCodeTypeConstants
    dbmMapcharCodeTypeASCII
    dbmMapcharCodeTypeEBCDIC
End Enum

Private Type MapcharElement
    sIntern As String * MAPCHAR_INTERN_LEN
    sMapCode As String * MAPCHAR_MAPCODE_LEN
End Type

Private m_sName As String
Private m_sCode As String
Private m_bExistInDb As Boolean
Private m_bToBeDeleted As Boolean

Private aCodeList(1 To MAPCHAR_COUNT_MAX) As MapcharElement

'''

Public Property Get ObjectType() As DBMObjectTypeConstants
    ObjectType = dbmOTMapcharSet
End Property

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Friend Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As Database
    Set Database = oParent.Database
End Property

Public Property Get CodeType() As MapcharCodeTypeConstants
    If (m_sCode = MAPCHAR_TYPE_EBCDIC) Then
        CodeType = dbmMapcharCodeTypeEBCDIC
    Else
        CodeType = dbmMapcharCodeTypeASCII
    End If
End Property

Public Property Let CodeType(nValue As MapcharCodeTypeConstants)
    If (nValue = dbmMapcharCodeTypeEBCDIC) Then
        m_sCode = MAPCHAR_TYPE_EBCDIC
    Else
        m_sCode = MAPCHAR_TYPE_ASCII
    End If
End Property

'''

Public Property Get CodeTypeStr() As String
    CodeTypeStr = m_sCode
End Property
Public Property Let CodeTypeStr(NewVal As String)
    Select Case UCase(NewVal)
        Case MAPCHAR_TYPE_EBCDIC:   m_sCode = NewVal
        Case MAPCHAR_TYPE_ASCII:    m_sCode = NewVal
        Case Else:                  m_sCode = MAPCHAR_TYPE_ASCII
    End Select
End Property

'''

Public Property Let ToDelete(bValue As Boolean)
    m_bToBeDeleted = bValue
End Property

Public Property Get MaxCode() As Integer
    MaxCode = MAPCHAR_COUNT_MAX
End Property

Public Property Get Name() As String
    Name = m_sName
End Property
Public Property Let Name(ByVal sValue As String)
    m_sName = sValue
End Property

Public Function ReQuery(sError As String) As Boolean
    Call Clear
    
    ReQuery = GetValuesFromDB(sError)
End Function

Public Sub Clear()
    Dim nIndex As Integer
    
    For nIndex = 1 To MAPCHAR_COUNT_MAX
        aCodeList(nIndex).sIntern = ""
        aCodeList(nIndex).sMapCode = ""
    Next
End Sub

Public Function SetCode(ByVal nIndex As Long, _
                        ByVal sIntern As String, _
                        ByVal sMapCode As String) As Boolean
  If (nIndex >= 0 And _
      nIndex <= MAPCHAR_COUNT_MAX And _
      Len(sIntern) <= MAPCHAR_INTERN_LEN And _
      Len(sMapCode) <= MAPCHAR_MAPCODE_LEN) Then
  
    aCodeList(nIndex).sIntern = sIntern
    aCodeList(nIndex).sMapCode = sMapCode
    
    SetCode = True
    
  End If
    
End Function

Public Function GetCode(ByVal nIndex As Long, _
                        sIntern As String, _
                        sMapCode As String) As Boolean
  If (nIndex >= 0 And _
      nIndex <= MAPCHAR_COUNT_MAX) Then
  
    sIntern = Trim(aCodeList(nIndex).sIntern)
    sMapCode = Trim(aCodeList(nIndex).sMapCode)
    
    GetCode = True
  End If
    
End Function

'''

Public Function Save() As Boolean
  If (m_bExistInDb) Then
    If (m_bToBeDeleted) Then
      Save = Drop()
    Else
      Save = Alter()
    End If
  Else
    If (Not m_bToBeDeleted) Then
      Save = Create()
    End If
  End If
  m_bExistInDb = Save

End Function

'''

Private Function Create() As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    With Database
        If .UtilConnect() Then
            If .UtilExecute(MAPCHAR_UTL_CREATE & Name & _
                            MAPCHAR_UTL_AS & IIf((CodeType = dbmMapcharCodeTypeEBCDIC), MAPCHAR_TYPE_EBCDIC, MAPCHAR_TYPE_ASCII) & _
                            MAPCHAR_UTL_WITH & MapcharList) _
            Then
                bOK = True
            Else
                .LastError.SetByServerObject .ServerObject
            End If
            .UtilRelease
        End If
    End With
    
    Create = bOK
End Function

Private Function Alter() As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    With Database
        If .UtilConnect() Then
            If .UtilExecute(MAPCHAR_UTL_ALTER & Name & _
                            MAPCHAR_UTL_AS & IIf((CodeType = dbmMapcharCodeTypeEBCDIC), MAPCHAR_TYPE_EBCDIC, MAPCHAR_TYPE_ASCII) & _
                            MAPCHAR_UTL_WITH & MapcharList) _
            Then
                bOK = True
            Else
                .LastError.SetByServerObject .ServerObject
            End If
            .UtilRelease
        End If
    End With
    
    Alter = bOK
End Function

Private Function Drop() As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    With Database
        If .UtilConnect() Then
            If .UtilExecute(MAPCHAR_UTL_DROP & Name) _
            Then
                bOK = True
            Else
                .LastError.SetByServerObject .ServerObject
            End If
            .UtilRelease
        End If
    End With
    
    Drop = bOK
End Function

Private Function GetValuesFromDB(sError As String) As Boolean
    Dim bSQLCon As Boolean
    Dim sResult As String
    Dim sRow As String
    Dim sIntern As String
    Dim sMapCode As String
    Dim nIndex As Long
    Dim bStatus As Boolean
    Dim bContinue As Boolean
    
    bSQLCon = False
    bStatus = False
        
    With Database
        ' Connect SQL
        If (DBMSRVSQL.Connect(.ServerObject)) Then
            bSQLCon = True
            ' Retrieve set codetype (ascii/ebcdic) and state (enabled/disabled)
            If (DBMSRVSQL.Execute(.ServerObject, MAPCHAR_SQL_STATE1 & Name & MAPCHAR_SQL_STATE2, sResult)) Then
                sRow = DBMSRVSQL.NextRow(sResult)
                If (Len(sRow) > 0) Then
                    CodeType = IIf((DBMSRVSQL.NextField(sRow) = MAPCHAR_TYPE_EBCDIC), dbmMapcharCodeTypeEBCDIC, dbmMapcharCodeTypeASCII)
                    bStatus = True
                End If
            Else
                sError = .ServerObject.ErrorText & vbCrLf & .ServerObject.ErrorData
            End If
        Else
            sError = .ServerObject.ErrorText & vbCrLf & .ServerObject.ErrorData
        End If
        
        ' retrieve codes
        If (bStatus And DBMSRVSQL.Execute(.ServerObject, MAPCHAR_SQL_VALUES1 & Name & MAPCHAR_SQL_VALUES2, sResult, bContinue)) Then
            sRow = DBMSRVSQL.NextRow(sResult)
            nIndex = 1
        
            While (Len(sRow) > 0 And bStatus)
                sIntern = DBMSRVSQL.NextField(sRow)
                sMapCode = DBMSRVSQL.NextField(sRow)
                bStatus = SetCode(nIndex, sIntern, sMapCode)
                sRow = DBMSRVSQL.NextRow(sResult)
                nIndex = nIndex + 1
            Wend
        
            While (bStatus And bContinue)
                bStatus = DBMSRVSQL.Fetch(.ServerObject, sResult, bContinue)
        
                If (bStatus) Then
                    sRow = DBMSRVSQL.NextRow(sResult)
                    nIndex = 1
        
                    While (Len(sRow) > 0 And bStatus)
                        sIntern = DBMSRVSQL.NextField(sRow)
                        sMapCode = DBMSRVSQL.NextField(sRow)
                        bStatus = SetCode(nIndex, sIntern, sMapCode)
                        sRow = DBMSRVSQL.NextRow(sResult)
                        nIndex = nIndex + 1
                    Wend
                Else
                    sError = .ServerObject.ErrorText & vbCrLf & .ServerObject.ErrorData
                End If
        
            Wend
        Else
            sError = .ServerObject.ErrorText & vbCrLf & .ServerObject.ErrorData
            bStatus = False
        End If
        
        If bSQLCon Then DBMSRVSQL.Release .ServerObject
    End With
        
    m_bExistInDb = bStatus
    
    GetValuesFromDB = bStatus

End Function

Private Function MapcharList() As String
  Dim sList As String
  Dim nIndex As Integer
    
  sList = ""
  For nIndex = 1 To MAPCHAR_COUNT_MAX
        
    If Trim(aCodeList(nIndex).sIntern) <> "" And Trim(aCodeList(nIndex).sMapCode) <> "" Then
        
      sList = sList & IIf(nIndex > 1, MAPCHAR_UTL_COMMA, "")
      sList = sList & _
              MAPCHAR_UTL_DIGIT1 & UCase(aCodeList(nIndex).sIntern) & _
              MAPCHAR_UTL_DIGIT2 & UCase(aCodeList(nIndex).sMapCode) & _
              MAPCHAR_UTL_DIGIT3
    End If
  Next
    
  MapcharList = sList

End Function

Public Sub CleanUp()
    Set oParent = Nothing
End Sub

Private Sub Class_Initialize()
    InitObject ObjectType
    
    m_sName = ""
    m_sCode = MAPCHAR_TYPE_ASCII
    m_bExistInDb = False
    m_bToBeDeleted = False
End Sub

Private Sub Class_Terminate()
  TermObject ObjectType
End Sub

