VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "MediaGroup"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Ext_KEY = "RVB_UniqueId" ,"33F01F440077"
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const FUNC_SAVE = "MediaGroupSave"
Const FUNC_DELETE = "MediaGroupDelete"
Const FUNC_RENAME = "MediaGroupRename"

'''

Private oParent As Object

Private lIndex As Long
Private sKey As String

Private sName As String

Private sOldName As String

Private oMedia As Media

Private oRestoreItem As RestoreItem

'''

Private sNewName As String

'''

Public Property Get ObjectType() As DBMObjectTypeConstants
    ObjectType = dbmOTMediaGroup
End Property

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Friend Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As Database
    Set Database = oParent.Database
End Property

Public Property Get Index() As Long
    Index = lIndex
End Property
Friend Property Let Index(ByVal NewVal As Long)
    lIndex = NewVal
End Property

Public Property Get Key() As String
    Key = sKey
End Property

'''

Public Property Get Media() As Media
    Set Media = oMedia
End Property

Public Property Get Name() As String
    Name = sName
End Property
Public Property Let Name(NewVal As String)
    sName = NewVal
End Property

Public Property Get OldName() As String
    OldName = sOldName
End Property
Public Property Let OldName(NewVal As String)
    sOldName = NewVal
End Property

Public Property Get SaveType() As DBMBackupTypeConstants
    SaveType = dbmBTUnknown
    If Not oMedia Is Nothing Then
        If oMedia.Count > 0 Then
            SaveType = oMedia(1).SaveType
        End If
    End If
End Property
Public Property Let SaveType(ByVal NewVal As DBMBackupTypeConstants)
    Dim oMedium As Medium
    
    If Not oMedia Is Nothing Then
        For Each oMedium In oMedia
            oMedium.SaveType = NewVal
        Next
    End If
End Property

Public Property Get SaveTypeString() As String
    SaveTypeString = Database.Backup.BackupTypeString(dbmBTUnknown)
    If Not oMedia Is Nothing Then
        If oMedia.Count > 0 Then
            SaveTypeString = oMedia(1).SaveTypeString
        End If
    End If
End Property
Public Property Let SaveTypeString(NewVal As String)
    Dim oMedium As Medium
    
    If Not oMedia Is Nothing Then
        For Each oMedium In oMedia
            oMedium.SaveTypeString = NewVal
        Next
    End If
End Property

Public Property Get RestoreItem() As RestoreItem
    Set RestoreItem = oRestoreItem
End Property
Public Property Set RestoreItem(ByVal NewVal As RestoreItem)
    Set oRestoreItem = NewVal
End Property

'''

Public Function CreateMedium(ByVal Name As String, _
                             ByVal Location As String, _
                             ByVal DeviceType As DBMDeviceTypeConstants, _
                             ByVal SaveType As DBMBackupTypeConstants)
    Dim oMedium As New Medium
    
    With oMedium
        Set .Parent = Me
        .InGroup = True
        .Name = Name
        .Location = Location
        .DeviceType = DeviceType
        .SaveType = SaveType
    End With
    
    oMedia.Add oMedium
    
    Set CreateMedium = oMedium
End Function

Public Function Save(Optional ByVal Asynchron As Boolean = False, _
                     Optional ByVal SenderID As Long = 0) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Not Asynchron Then
            bOK = Execute(FUNC_SAVE, False)
        Else
            QUEUE.QueueItems.Add Me, FUNC_SAVE, SenderID
            bOK = True
        End If
    End If
    
    Save = bOK
End Function

Public Function Delete(Optional ByVal Asynchron As Boolean = False, _
                       Optional ByVal SenderID As Long = 0) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Not Asynchron Then
            bOK = Execute(FUNC_DELETE, False)
        Else
            QUEUE.QueueItems.Add Me, FUNC_DELETE, SenderID
            bOK = True
        End If
    End If
    
    Delete = bOK
End Function

Public Function Rename(NewName As String, _
                       Optional ByVal Asynchron As Boolean = False, _
                       Optional ByVal SenderID As Long = 0) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
    
        sNewName = NewName
    
        If Not Asynchron Then
            bOK = Execute(FUNC_RENAME, False)
        Else
            QUEUE.QueueItems.Add Me, FUNC_RENAME, SenderID
            bOK = True
        End If
    End If
    
    Rename = bOK
End Function

Public Sub CleanUp()
    If Not oMedia Is Nothing Then oMedia.CleanUp
    
    Set oMedia = Nothing
    Set oRestoreItem = Nothing
    
    Set oParent = Nothing
End Sub

'''

Private Sub Class_Initialize()
    InitObject ObjectType
    
    sKey = GetUniqueKeyStr()
    
    sName = ""
    
    sOldName = False
    
    Set oMedia = New Media
End Sub
Private Sub Class_Terminate()
    TermObject ObjectType
End Sub

'''

Public Function Execute(ByVal FunctionName As String, ByVal Asynchron As Boolean, Optional ByVal SenderID As Long) As Boolean
    Dim bOK As Boolean
    Dim oDB As DBMLib.Database
    
    Set oDB = Database
    
    oDB.Busy = True
    
    bOK = False
    
    Select Case FunctionName
        Case FUNC_SAVE:         bOK = ExecuteSave(Asynchron, SenderID)
        Case FUNC_DELETE:       bOK = ExecuteDelete(Asynchron, SenderID)
        Case FUNC_RENAME:       bOK = ExecuteRename(Asynchron, SenderID)
    End Select
            
    oDB.Busy = False
    
    Execute = bOK
End Function

Private Function ExecuteSave(ByVal bAsynchron As Boolean, ByVal lSenderID As Long) As Boolean
    Dim oMedium As Medium
    Dim bOK As Boolean
    
    bOK = False
    
    With Database
        If .Check() Then
            For Each oMedium In oMedia
                bOK = oMedium.Save()
                If Not bOK Then Exit For
            Next
    
            If Not bOK Then
                .LastError.SetByServerObject .ServerObject
            End If
        End If

        If bAsynchron Then
            .EventMediaGroupAction lSenderID, Not bOK, dbmMediaGroupActionSave, Me
        End If
    End With
    
    ExecuteSave = bOK
End Function

Private Function ExecuteDelete(ByVal bAsynchron As Boolean, ByVal lSenderID As Long) As Boolean
    Dim oMedium As Medium
    Dim oDB As Database
    Dim bOK As Boolean
    
    Set oDB = Database
    
    bOK = False
    
    With oDB
        If .Check() Then
            If oMedia.Count > 0 Then
                For Each oMedium In oMedia
                    bOK = oMedium.Delete()
                    If Not bOK Then Exit For
                Next
            Else
                bOK = True
            End If
            
            If bOK Then
                oParent.MediaGroups.Remove lIndex
            Else
                .LastError.SetByServerObject .ServerObject
            End If
        End If
    
        If bAsynchron Then
            .EventMediaGroupAction lSenderID, Not bOK, dbmMediaGroupActionDelete, Me
        End If
    End With
    
    ExecuteDelete = bOK
End Function

Private Function ExecuteRename(ByVal bAsynchron As Boolean, ByVal lSenderID As Long) As Boolean
    Dim oMedium As Medium
    Dim bOK As Boolean
    
    bOK = False
    
    With Database
        If .Check() Then
            If oMedia.Count > 0 Then
                For Each oMedium In oMedia
                    bOK = DBMSRVMedia.DeleteMedium(.ServerObject, oMedium)
                    If Not bOK Then Exit For
                Next
            Else
                bOK = True
            End If
            
            If bOK Then
                bOK = False
            
                sName = sNewName
                
                If oMedia.Count > 0 Then
                    For Each oMedium In oMedia
                        If DBMSRVMedia.PutMedium(.ServerObject, oMedium) Then
                            If DBMSRVMedia.GetMedium(.ServerObject, oMedium) Then
                                bOK = True
                            End If
                        End If
                        If Not bOK Then Exit For
                    Next
                Else
                    bOK = True
                End If
            End If
            
            If Not bOK Then
                .LastError.SetByServerObject .ServerObject
            End If
        End If
        
        If bAsynchron Then
            .EventMediaGroupAction lSenderID, Not bOK, dbmMediaGroupActionRename, Me
        End If
    End With

    ExecuteRename = bOK
End Function

