VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "RestartInfo"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const FUNC_REFRESH = "RefreshRestartInfo"

Const RESTARTINFO_FIRSTLOGPAGE = "First LOG Page"
Const RESTARTINFO_USEDLOGPAGE = "Used LOG Page"
Const RESTARTINFO_RESTARTABLE = "Restartable"

Const MAX_LONG = &H7FFFFFFF

Private oParent As Object
    
Private lFirstLogPage As Long
Private lUsedLogPage As Long
Private bRestartable As Boolean
Private bInitConfig As Boolean

'''

Public Property Get ObjectType() As DBMObjectTypeConstants
    ObjectType = dbmOTRestartInfo
End Property

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Friend Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As Database
    Set Database = oParent.Database
End Property

'''

Public Property Get FirstLogPage() As Long
    FirstLogPage = lFirstLogPage
End Property

Public Property Get UsedLogPage() As Long
    UsedLogPage = lUsedLogPage
End Property

Public Property Get LogEmpty() As Boolean
    LogEmpty = lUsedLogPage < lFirstLogPage
End Property

Public Property Get InitConfig() As Boolean
    InitConfig = bInitConfig
End Property

Public Property Get Restartable() As Boolean
    Restartable = bRestartable
End Property

Public Property Let Data(sData As String)
    Dim lPos1 As Long
    Dim lPos2 As Long
    Dim sLine As String
    Dim sName As String
    Dim sValue As String
    
    With Database
        lPos1 = 1
        Do While lPos1 < Len(sData)
            sLine = Trim$(.ServerObject.GetLine(sData, lPos1))
            If Len(sLine) > 0 Then
                lPos2 = 1
                sName = Trim$(.ServerObject.GetField(sLine, vbTab, lPos2))
                sValue = Trim$(.ServerObject.GetField(sLine, vbTab, lPos2))
                Select Case sName
                
                    Case RESTARTINFO_FIRSTLOGPAGE:
                        If IsNumeric(sValue) Then
                            lFirstLogPage = CLng(sValue)
                        Else
                            lFirstLogPage = MAX_LONG
                        End If
                        If lFirstLogPage = MAX_LONG Then
                            bInitConfig = True
                        End If
                        
                    Case RESTARTINFO_USEDLOGPAGE:
                        If IsNumeric(sValue) Then
                            lUsedLogPage = CLng(sValue)
                        Else
                            lUsedLogPage = 0
                        End If
                        
                    Case RESTARTINFO_RESTARTABLE:
                        bRestartable = (sValue = "1")
                End Select
            End If
        Loop
    End With
End Property

'''

Public Function Refresh(Optional ByVal Asynchron As Boolean = False, _
                        Optional ByVal SenderID As Long = 0) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Not Asynchron Then
            bOK = Execute(FUNC_REFRESH, False, SenderID)
        Else
            QUEUE.QueueItems.Add Me, FUNC_REFRESH, SenderID
            bOK = True
        End If
    End If
    
    Refresh = bOK
End Function

Public Sub CleanUp()
    Set oParent = Nothing
End Sub

'''

Private Sub Class_Initialize()
    InitObject ObjectType
End Sub
Private Sub Class_Terminate()
    TermObject ObjectType
End Sub

'''

Public Function Execute(ByVal FunctionName As String, ByVal Asynchron As Boolean, Optional ByVal SenderID As Long) As Boolean
    Dim bOK As Boolean
    Dim oDB As DBMLib.Database
    
    Set oDB = Database
    
    oDB.Busy = True
    
    bOK = False
    
    Select Case FunctionName
        Case FUNC_REFRESH:  bOK = ExecuteRefresh(Asynchron, SenderID)
    End Select
            
    oDB.Busy = False
    
    Execute = bOK
End Function

Private Function ExecuteRefresh(ByVal bAsynchron As Boolean, ByVal lSenderID As Long) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    lFirstLogPage = CStr(MAX_LONG)
    lUsedLogPage = "0"
    bRestartable = True
    bInitConfig = False
    
    With Database
        If .Check() Then
            If .UtilConnect Then
                If DBMSRVBackup.RestartInfo(.ServerObject, Me) Then
                    bOK = True
                Else
                    .LastError.SetByServerObject .ServerObject
                End If
                .UtilRelease
            End If
        End If
        
        If bAsynchron Then
            .EventRestartInfoRefreshed lSenderID, Not bOK
        End If
    End With
    
    ExecuteRefresh = bOK
End Function

