VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "RestoreManager"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Private oParent As Object

Private oRestoreHistory As History
Private oRestoreItems As RestoreItems
Private oRestoreMedia As RestoreMedia

Private lFirstPage As Long
Private lLastPage As Long

Private bUseUntilDate As Boolean
Private dUntilDate As Date

Private lActRestoreItem As Long
Private lActRestoreMedium As Long

'''

Public Property Get ObjectType() As DBMObjectTypeConstants
    ObjectType = dbmOTRestoreManager
End Property

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Friend Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As Database
    Set Database = oParent.Database
End Property

Public Property Get HistoryItems() As HistoryItems
    Set HistoryItems = RestoreHistory.HistoryItems
End Property

Public Property Get RestoreHistory() As History
    If oRestoreHistory Is Nothing Then
        Set oRestoreHistory = New History
        Set oRestoreHistory.Parent = Me
    End If
    Set RestoreHistory = oRestoreHistory
End Property

Public Property Get RestoreItems() As RestoreItems
    If oRestoreItems Is Nothing Then
        Set oRestoreItems = New RestoreItems
    End If
    Set RestoreItems = oRestoreItems
End Property

Public Property Get RestoreMedia() As RestoreMedia
    If oRestoreMedia Is Nothing Then
        Set oRestoreMedia = New RestoreMedia
    End If
    Set RestoreMedia = oRestoreMedia
End Property

Public Property Get FirstPage() As Long
    FirstPage = lFirstPage
End Property

Public Property Get LastPage() As Long
    LastPage = lLastPage
End Property

Public Property Get UseUntildate() As Boolean
    UseUntildate = bUseUntilDate
End Property

Public Property Get UntilDate() As Date
    UntilDate = dUntilDate
End Property

Public Property Get ActRestoreItem() As RestoreItem
    If lActRestoreItem > 0 And RestoreItems.Count > 0 Then
        Set ActRestoreItem = RestoreItems(lActRestoreItem)
    Else
        Set ActRestoreItem = Nothing
    End If
End Property

Public Property Get ActRestoreItemIndex() As Long
    ActRestoreItemIndex = lActRestoreItem
End Property

Public Property Get ActRestoreMedium() As Medium
    If lActRestoreMedium > 0 And RestoreMedia.Count > 0 Then
        Set ActRestoreMedium = RestoreMedia(lActRestoreMedium)
    Else
        Set ActRestoreMedium = Nothing
    End If
End Property

Public Property Get ActRestoreMediumIndex() As Long
    ActRestoreMediumIndex = lActRestoreMedium
End Property

'''
'
' LastSaveData:
'
Public Function LastSaveData(Optional ByVal UseUntildate As Boolean = False, _
                             Optional ByVal UntilDate As Date = Empty) As HistoryItem
    Dim oHistoryItem As HistoryItem
    Dim lItem As Long
    Dim lCount As Long
    
    Set LastSaveData = Nothing
    
    lCount = HistoryItems.Count
    
    For lItem = lCount To 1 Step -1
        Set oHistoryItem = HistoryItems.Item(lItem)
        
        If oHistoryItem.ReturnCode = 0 And _
           oHistoryItem.ActionType = dbmBASave And _
           oHistoryItem.BackupType = dbmBTData _
        Then
            If UseUntildate Then
                If oHistoryItem.DBStamp1 <= UntilDate Then
                    Set LastSaveData = oHistoryItem
                    Exit For
                End If
            Else
                Set LastSaveData = oHistoryItem
                Exit For
            End If
        End If
    Next
End Function
'
' PrepareForRestoreLastSave:
'
Public Sub PrepareForRestoreLastSave(Optional ByVal SavePages As Boolean = True, _
                                     Optional ByVal UseUntildate As Boolean = False, _
                                     Optional ByVal UntilDate As Date = Empty)
    Dim oHistoryItem As HistoryItem
    Set oHistoryItem = LastSaveData(UseUntildate, UntilDate)
    PrepareForRestoreSelectedSave oHistoryItem, SavePages, UseUntildate, UntilDate
End Sub
'
' PrepareForRestoreSelectedSave:
'
Public Sub PrepareForRestoreSelectedSave(HistoryItem As HistoryItem, _
                                         Optional ByVal SavePages As Boolean = True, _
                                         Optional ByVal UseUntildate As Boolean = False, _
                                         Optional ByVal UntilDate As Date = Empty)
    Dim oHistoryItem As HistoryItem
    Dim lItem As Long
    Dim lSaveData As Long
    Dim lFirstItem As Long
    
    Reset
    
    If HistoryItem Is Nothing Then Exit Sub
    
    lSaveData = 0
    
    ' check entry, must be a SAVE DATA
    lItem = HistoryItem.Index
    Set oHistoryItem = HistoryItem
    
    If oHistoryItem.ReturnCode = 0 And _
       oHistoryItem.ActionType = dbmBASave And _
       oHistoryItem.BackupType = dbmBTData Then
       
        AddRestoreHistoryItem oHistoryItem, True
        lSaveData = lItem
    End If
    
    ' no valid entry
    If lSaveData = 0 Then Exit Sub
    
    ' log before save data
    If oHistoryItem.LogPage < Database.RestartInfo.FirstLogPage Or _
       oHistoryItem.LogPage >= Database.RestartInfo.UsedLogPage _
    Then
        lFirstPage = oHistoryItem.LogPage
        lLastPage = Database.RestartInfo.FirstLogPage
        
        ' log before save data
        lFirstItem = 1
        For lItem = lSaveData To 1 Step -1
            Set oHistoryItem = HistoryItems.Item(lItem)
            If oHistoryItem.ActionType = dbmBAHistLost Then
                lFirstItem = lItem + 1
                Exit For
            End If
        Next
        
        GetItems lSaveData, lFirstItem, lFirstPage, lLastPage, SavePages, UseUntildate, UntilDate
    End If
End Sub
'
' PrepareForRestoreContinue:
'
Public Sub PrepareForRestoreContinue(Optional ByVal SavePages As Boolean = True, _
                                     Optional ByVal UseUntildate As Boolean = False, _
                                     Optional ByVal UntilDate As Date = Empty)
    Dim oHistoryItem As HistoryItem
    Dim lItem As Long
    Dim lCount As Long
    Dim lFirstItem As Long
    
    Reset
    
    lFirstPage = Database.RestartInfo.UsedLogPage
    lLastPage = Database.RestartInfo.FirstLogPage
    
    lCount = HistoryItems.Count
    
    lFirstItem = 1
    
    ' PTS 1107058 >>>
    ' get last HISTLOST
    For lItem = lCount To 1 Step -1
        Set oHistoryItem = HistoryItems.Item(lItem)
        
        If oHistoryItem.ActionType = dbmBAHistLost Then
            lFirstItem = lItem + 1
            Exit For
        End If
    Next
    ' <<< PTS 1107058
    
    GetItems 0, lFirstItem, lFirstPage, lLastPage, SavePages, UseUntildate, UntilDate
    
End Sub

'''

Private Sub GetItems(ByVal lSaveData As Long, _
                     ByVal lFirstItem As Long, _
                     ByVal lFirstLogPage As Long, _
                     ByVal lLastLogPage As Long, _
                     Optional ByVal SavePages As Boolean = True, _
                     Optional ByVal UseUntildate As Boolean = False, _
                     Optional ByVal UntilDate As Date = Empty)
                     
    Dim oHistoryItem As HistoryItem
    
    Dim lFirst As Long
    Dim lLast As Long
    Dim lCount As Long
    Dim lItem As Long
    
    Dim lFirstPAG As Long
    Dim lLastPAG As Long
    Dim lFirstLOG As Long
    Dim lLastLOG As Long
    
    Dim bPages As Boolean
    
    Dim bGet As Boolean
    Dim bAdd As Boolean
        
    Dim lActLogPage As Long
        
    bUseUntilDate = UseUntildate
    dUntilDate = UntilDate
    
    lFirst = 0
    lLast = 0
    lCount = HistoryItems.Count
    
    lFirstPAG = 0
    lLastPAG = 0
    lFirstLOG = 0
    lLastLOG = 0
    
    bPages = True
    
    ' find all possible items
    For lItem = lFirstItem To lCount
        
        Set oHistoryItem = HistoryItems.Item(lItem)
        
        If oHistoryItem.ActionType = dbmBAHistLost Then
            Exit For
        Else
            If oHistoryItem.ReturnCode = 0 And _
               oHistoryItem.ActionType = dbmBASave Then
               
                bGet = False
               
                Select Case oHistoryItem.BackupType
                    Case dbmBTData:
                        If lSaveData > 0 Then
                            If oHistoryItem.Index > lSaveData Then
                                bPages = False
                            End If
                        Else
                            bPages = False
                        End If
                    
                    Case dbmBTPages:
                        If SavePages And bPages Then
                            If oHistoryItem.LogPage > lFirstLogPage Then
                                If bUseUntilDate Then
                                    If oHistoryItem.DBStamp1 < dUntilDate Then
                                        bGet = True
                                    End If
                                Else
                                    bGet = True
                                End If
                            End If
                            If bGet Then
                                If lFirstPAG = 0 Then lFirstPAG = lItem
                                lLastPAG = lItem
                                
                                If lFirst = 0 Then lFirst = lItem
                                lLast = lItem
                            End If
                        End If
                        
                    Case dbmBTLog:
                        If (oHistoryItem.FromPage < lFirstLogPage And oHistoryItem.ToPage >= lFirstLogPage) Or _
                           (oHistoryItem.FromPage < lLastLogPage And oHistoryItem.ToPage >= lLastLogPage) Or _
                           (oHistoryItem.FromPage >= lFirstLogPage And oHistoryItem.ToPage < lLastLogPage) _
                        Then
                            If bUseUntilDate Then
                                If oHistoryItem.DBStamp1 < dUntilDate Then
                                    bGet = True
                                End If
                            Else
                                bGet = True
                            End If
                            If bGet Then
                                If lFirstLOG = 0 Then lFirstLOG = lItem
                                lLastLOG = lItem
                            
                                If lFirst = 0 Then lFirst = lItem
                                lLast = lItem
                            End If
                        End If
                End Select
            End If
        End If
    Next
    
    ' found nothing
    If lFirst = 0 Then Exit Sub
    
    ' get all possible items
    bPages = True
    
    lActLogPage = lFirstLogPage
    
    For lItem = lFirst To lLast
        
        Set oHistoryItem = HistoryItems.Item(lItem)
        
        If oHistoryItem.ReturnCode = 0 And _
           oHistoryItem.ActionType = dbmBASave _
        Then
        
            bAdd = False
            
            Select Case oHistoryItem.BackupType
                Case dbmBTData:
                    ' no more save pages after a save data
                    If lSaveData > 0 Then
                        If oHistoryItem.Index > lSaveData Then
                            bPages = False
                        End If
                    Else
                        bPages = False
                    End If
                
                Case dbmBTPages:
                    If SavePages And bPages Then
                        If bUseUntilDate Then
                            If oHistoryItem.DBStamp1 < dUntilDate Then
                                bAdd = True
                            End If
                        Else
                            bAdd = True
                        End If
                        If bAdd Then
                            If Database.Version.Number >= dbmversion74 Then
                                If lItem = lLastPAG Then
                                    AddRestoreHistoryItem oHistoryItem, True
                                Else
                                    AddRestoreHistoryItem oHistoryItem, False
                                End If
                            Else
                                AddRestoreHistoryItem oHistoryItem, True
                            End If
                            lActLogPage = oHistoryItem.LogPage
                        End If
                    End If
                    
                Case dbmBTLog:
                    If lItem <= lLastLOG Then
                        If bUseUntilDate Then
                            If oHistoryItem.DBStamp1 < dUntilDate Then
                                bAdd = True
                            End If
                        Else
                            bAdd = True
                        End If
                        If bAdd Then
                            AddRestoreHistoryItem oHistoryItem, False
                        End If
                    End If
            End Select
        End If
    Next
    
    ' select log entries
    
    SelectTheLog lActLogPage
End Sub
'
' PrepareAddRestoreMediaGroup:
'
Public Sub PrepareAddRestoreMediaGroup(MediaGroup As MediaGroup)
    AddRestoreMediaGroup MediaGroup, True
End Sub
'
' AddRestoreMedium:
'
Public Sub PrepareAddRestoreMedium(Medium As Medium)
    AddRestoreMedium Medium, True
End Sub
'
' RestoreItemSelected:
'
Public Sub RestoreItemSelected(ByVal Index As Long, ByVal Selected As Boolean)
    Dim oRestoreItem As RestoreItem
    Dim lRestoreItemCount As Long
    Dim lRestoreItem As Long
    
    Dim oHistoryItem As HistoryItem
    
    Dim lActPage As Long
            
    lRestoreItemCount = RestoreItems.Count
    
    ' nothing to do
    If lRestoreItemCount = 0 Then Exit Sub
    If Selected = RestoreItems(Index).Selected Then Exit Sub
    
    If RestoreItems(1).HistoryItem.BackupType = dbmBTLog Then
        lActPage = 0
    Else
        lActPage = RestoreItems(1).HistoryItem.LogPage
    End If
    
    Select Case RestoreItems(Index).ItemType
                        
        Case dbmRITHistory:
            
            If RestoreItems(1).HistoryItem.BackupType = dbmBTData Then
                RestoreItems(1).Selected = True
            End If
    
            If Selected Then
                Set oHistoryItem = RestoreItems(Index).HistoryItem
                
                Select Case oHistoryItem.BackupType
                
                    Case dbmBTData:
                        RestoreItems(Index).Selected = True
                        
                    Case dbmBTPages:
                        If Database.Version.Number >= dbmversion74 Then
                            ' select only one save pages by index
                            For lRestoreItem = 2 To lRestoreItemCount
                                Set oRestoreItem = RestoreItems(lRestoreItem)
                            
                                If oRestoreItem.HistoryItem.BackupType = dbmBTPages Then
                                    If lRestoreItem = Index Then
                                        oRestoreItem.Selected = True
                                        lActPage = oRestoreItem.HistoryItem.LogPage
                                    Else
                                        oRestoreItem.Selected = False
                                    End If
                                End If
                            Next
                        Else
                            ' select all save pages until index
                            For lRestoreItem = 2 To Index
                                Set oRestoreItem = RestoreItems(lRestoreItem)
                            
                                If oRestoreItem.HistoryItem.BackupType = dbmBTPages Then
                                    oRestoreItem.Selected = True
                                    lActPage = oRestoreItem.HistoryItem.LogPage
                                End If
                            Next
                        End If
                        
                        SelectTheLog lActPage
                        
                    Case dbmBTLog:
                        For lRestoreItem = 2 To lRestoreItemCount
                            Set oRestoreItem = RestoreItems(lRestoreItem)
                        
                            If oRestoreItem.HistoryItem.BackupType = dbmBTPages Then
                                If oRestoreItem.HistoryItem.LogPage > oHistoryItem.ToPage Then
                                    oRestoreItem.Selected = False
                                End If
                            End If
                        Next
                        
                        For lRestoreItem = lRestoreItemCount To 2 Step -1
                            Set oRestoreItem = RestoreItems(lRestoreItem)
                            If oRestoreItem.HistoryItem.BackupType = dbmBTPages Then
                                If oRestoreItem.Selected Then
                                    lActPage = oRestoreItem.HistoryItem.LogPage
                                    Exit For
                                End If
                            End If
                        Next
                        
                        SelectTheLog lActPage
                        
                End Select
                
            Else
                
                Set oHistoryItem = RestoreItems(Index).HistoryItem
                
                Select Case oHistoryItem.BackupType
                    Case dbmBTData:
                        RestoreItems(Index).Selected = True
                    
                    Case dbmBTPages:
                        For lRestoreItem = Index To lRestoreItemCount
                            Set oRestoreItem = RestoreItems(lRestoreItem)
                        
                            If oRestoreItem.HistoryItem.BackupType = dbmBTPages Then
                                oRestoreItem.Selected = False
                            End If
                        Next
                    
                        For lRestoreItem = lRestoreItemCount To 2 Step -1
                            Set oRestoreItem = RestoreItems(lRestoreItem)
                            If oRestoreItem.HistoryItem.BackupType = dbmBTPages Then
                                If oRestoreItem.Selected Then
                                    lActPage = oRestoreItem.HistoryItem.LogPage
                                    Exit For
                                End If
                            End If
                        Next
                    
                        SelectTheLog lActPage
                        
                    Case dbmBTLog:
                        RestoreItems(Index).Selected = True
                    
                End Select
            End If
            
        Case dbmRITMedia:
            RestoreItems(Index).Selected = Selected
        
        Case dbmRITMedium:
            RestoreItems(Index).Selected = Selected
        
    End Select
End Sub

Public Sub PrepareRestoreMedia()
    RestoreMedia.Clear
    
    PrepareRestoreMediaBackupType dbmBTData
    PrepareRestoreMediaBackupType dbmBTPages
    PrepareRestoreMediaBackupType dbmBTLog
End Sub

Private Sub PrepareRestoreMediaBackupType(ByVal lBackupType As DBMBackupTypeConstants)
    Dim oRestoreItem As RestoreItem
    Dim oMedium As Medium
    Dim oMediaGroup As MediaGroup
    Dim lFirstReplace As Long
    Dim lMedium As Long
    Dim sLocation As String
    Dim sFirstName As String
    Dim sExternalID As String
    Dim sEIDOption As String
    Dim oExternalID As ExternalID
    
    For Each oRestoreItem In RestoreItems
        If oRestoreItem.ItemType = dbmRITHistory And _
           oRestoreItem.Selected _
        Then
            If oRestoreItem.HistoryItem.BackupType = lBackupType Then
                
                lFirstReplace = 2
                
                With Database.Backup
                    If .MediaGroups.ExistsByName(oRestoreItem.HistoryItem.MediaName) Then
                        
                        Set oMediaGroup = .MediaGroups.ItemByName(oRestoreItem.HistoryItem.MediaName)
                        
                        lFirstReplace = oMediaGroup.Media.Count + 1
                        
                        sLocation = ""
                        sExternalID = ""
                        sEIDOption = ""
                        
                        For lMedium = 1 To oMediaGroup.Media.Count
                            Set oMedium = oRestoreItem.HistoryItem.Media(lMedium)
                            sLocation = sLocation + oMedium.Location + " | "
                            If Not oMedium.ExternalID Is Nothing Then
                                sExternalID = sExternalID + oMedium.ExternalID.Text + " , "
                                sEIDOption = sEIDOption + oMedium.ExternalID.Text + ","
                            End If
                        Next
                            
                        If Len(sLocation) > 0 Then
                            sLocation = Left$(sLocation, Len(sLocation) - 3)
                        End If
                        oRestoreItem.FirstLocation = sLocation
                        
                        If Len(sExternalID) > 0 Then
                            sExternalID = Left$(sExternalID, Len(sExternalID) - 3)
                        End If
                        oRestoreItem.FirstExternalID = sExternalID
                        
                        If Len(sEIDOption) > 0 Then
                            sEIDOption = Left$(sEIDOption, Len(sEIDOption) - 1)
                        End If
                        Set oMedium = oRestoreItem.HistoryItem.Media(1)
                        If Not oMedium.ExternalID Is Nothing Then
                            oMedium.ExternalID.OptionText = sEIDOption
                        End If
                        
                        sFirstName = oMediaGroup.Media(1).Name
                        
                    Else
                        oRestoreItem.FirstLocation = ""
                        oRestoreItem.FirstExternalID = ""
                        
                        Set oMedium = oRestoreItem.HistoryItem.Media(1)
                        If Not oMedium.ExternalID Is Nothing Then
                            oMedium.ExternalID.OptionText = oMedium.ExternalID.Text
                        End If
                        
                        sFirstName = ""
                    End If
                End With

                Set oMedium = oRestoreItem.HistoryItem.Media(1)
                Set oMedium.RestoreItem = oRestoreItem
                oMedium.First = True
                oMedium.FirstName = sFirstName
                RestoreMedia.AddMedium oMedium
                
                For lMedium = lFirstReplace To oRestoreItem.HistoryItem.Media.Count
                    Set oMedium = oRestoreItem.HistoryItem.Media(lMedium)
                    
                    If Not oMedium.ExternalID Is Nothing Then
                        oMedium.ExternalID.OptionText = oMedium.ExternalID.Text
                    End If
                    
                    Set oMedium.RestoreItem = oRestoreItem
                    oMedium.First = False
                    RestoreMedia.AddMedium oMedium
                Next
            End If
        Else
            If oRestoreItem.ItemType = dbmRITMedia Then
                If oRestoreItem.MediaGroup.SaveType = lBackupType Then
                
                    lMedium = 1
                    For Each oExternalID In Database.Backup.ExternalIDs
                        If oExternalID.Selected Then
                            If lMedium <= oRestoreItem.MediaGroup.Media.Count Then
                                Set oMedium = oRestoreItem.MediaGroup.Media(lMedium)
                                Set oMedium.ExternalID = oExternalID
                                Set oExternalID.Medium = oMedium
                                oMedium.ExternalID.OptionText = oMedium.ExternalID.Text
                                lMedium = lMedium + 1
                            End If
                        End If
                    Next
                
                    sLocation = ""
                    sExternalID = ""
                    For Each oMedium In oRestoreItem.MediaGroup.Media
                        sLocation = sLocation + oMedium.Location + " | "
                        If Not oMedium.ExternalID Is Nothing Then
                            sExternalID = sExternalID + oMedium.ExternalID.Text + " | "
                        End If
                    Next
                    
                    If Len(sLocation) > 0 Then
                        sLocation = Left$(sLocation, Len(sLocation) - 3)
                    End If
                    oRestoreItem.FirstLocation = sLocation
                    
                    If Len(sExternalID) > 0 Then
                        sExternalID = Left$(sExternalID, Len(sExternalID) - 3)
                    End If
                    oRestoreItem.FirstExternalID = sExternalID
                End If
                
            ElseIf oRestoreItem.ItemType = dbmRITMedium Then
            
                If oRestoreItem.Medium.SaveType = lBackupType Then
                    Set oMedium = oRestoreItem.Medium
                
                    For Each oExternalID In Database.Backup.ExternalIDs
                        If oExternalID.Selected Then
                            Set oMedium.ExternalID = oExternalID
                            Set oExternalID.Medium = oMedium
                            Exit For
                        End If
                    Next
                    
                    If Not oMedium.ExternalID Is Nothing Then
                        oMedium.ExternalID.OptionText = oMedium.ExternalID.Text
                    End If
                End If
            End If
        End If
    Next
End Sub

Public Sub SetNextToRestore()
    Dim oMedium As Medium
    
    If lActRestoreMedium < RestoreMedia.Count Then
        lActRestoreMedium = lActRestoreMedium + 1
        Set oMedium = oRestoreMedia(lActRestoreMedium)
        lActRestoreItem = oMedium.RestoreItem.Index
    Else
        If lActRestoreItem < RestoreItems.Count Then
            lActRestoreItem = lActRestoreItem + 1
        End If
    End If
End Sub

Public Sub Reset()
    ResetRestoreItems
    ResetRestoreMedia
End Sub

Public Sub ResetRestoreItems()
    If Not oRestoreItems Is Nothing Then oRestoreItems.Clear
    
    Set oRestoreItems = Nothing

    lFirstPage = 0
    lLastPage = 0
    
    bUseUntilDate = False
    dUntilDate = Empty

    lActRestoreItem = 0
End Sub

Public Sub ResetRestoreMedia()
    If Not oRestoreMedia Is Nothing Then oRestoreMedia.Clear
    
    Set oRestoreMedia = Nothing

    lActRestoreMedium = 0
End Sub

Public Sub CleanUp()
    If Not oRestoreHistory Is Nothing Then oRestoreHistory.CleanUp
    If Not oRestoreItems Is Nothing Then oRestoreItems.CleanUp
    If Not oRestoreMedia Is Nothing Then oRestoreMedia.CleanUp

    Set oRestoreHistory = Nothing
    Set oRestoreItems = Nothing
    Set oRestoreMedia = Nothing

    Set oParent = Nothing
End Sub

'''

Private Sub Class_Initialize()
    InitObject ObjectType
    Reset
End Sub
Private Sub Class_Terminate()
    TermObject ObjectType
End Sub

'''

Private Sub GetMinMaxPage(PageMin As Long, _
                          PageMax As Long)
                          
    Dim oEntry As HistoryItem

    PageMin = &H7FFFFFFF
    PageMax = 0
    
    For Each oEntry In HistoryItems
        With oEntry
            If .ActionType = dbmBASave Then
                If .FromPage <> "" Then
                    If .FromPage < PageMin Then PageMin = .FromPage
                    If .FromPage > PageMax Then PageMax = .FromPage
                End If
                If .ToPage <> "" Then
                    If .ToPage < PageMin Then PageMin = .ToPage
                    If .ToPage > PageMax Then PageMax = .ToPage
                End If
            End If
        End With
    Next
End Sub

'''

Private Sub AddRestoreHistoryItem(ByVal oHistoryItem As HistoryItem, _
                                  ByVal bSelected As Boolean)
    Dim oRestoreItem As RestoreItem
    
    Set oRestoreItem = New RestoreItem
    
    With oRestoreItem
        .ItemType = dbmRITHistory
        Set .HistoryItem = oHistoryItem
        .Selected = bSelected
    End With
    
    RestoreItems.Add oRestoreItem
    Set oRestoreItem = Nothing
End Sub

Private Sub AddRestoreMediaGroup(ByVal oMediaGroup As MediaGroup, _
                                 ByVal bSelected As Boolean)
                           
    Dim oRestoreItem As New RestoreItem
    
    With oRestoreItem
        .ItemType = dbmRITMedia
        Set .MediaGroup = oMediaGroup
        .Selected = bSelected
    End With
    
    Set oMediaGroup.RestoreItem = oRestoreItem
    
    RestoreItems.Add oRestoreItem
    
    Set oRestoreItem = Nothing
End Sub

Private Sub AddRestoreMedium(ByVal oMedium As Medium, _
                             ByVal bSelected As Boolean)
                           
    Dim oRestoreItem As New RestoreItem
    
    With oRestoreItem
        .ItemType = dbmRITMedium
        Set .Medium = oMedium
        .Selected = bSelected
    End With
    
    Set oMedium.RestoreItem = oRestoreItem
    
    RestoreItems.Add oRestoreItem
    
    Set oRestoreItem = Nothing
End Sub

'''

Private Sub SelectTheLog(ByVal lActPage As Long)
    Dim oRestoreItem As RestoreItem

    For Each oRestoreItem In RestoreItems
        With oRestoreItem
            If .HistoryItem.BackupType = dbmBTLog Then
                If (.HistoryItem.FromPage < lActPage And .HistoryItem.ToPage >= lActPage) Or _
                   (.HistoryItem.FromPage >= lActPage) _
                Then
                    .Selected = True
                Else
                    .Selected = False
                End If
            End If
        End With
    Next
End Sub
