VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Users"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const FUNC_OPEN = "UsersOpen"
Const FUNC_SAVE = "UsersSave"

Private oParent As Object

Private oItems As New Collection

'''

Private dAt As Date

'''

Public Property Get ObjectType() As DBMObjectTypeConstants
    ObjectType = dbmOTUsers
End Property

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Friend Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As Database
    Set Database = oParent.Database
End Property

Public Property Get Count() As Integer
    Count = oItems.Count
End Property

Public Property Get Item(ByVal Index As Variant) As User
Attribute Item.VB_UserMemId = 0
Attribute Item.VB_MemberFlags = "200"
    Set Item = oItems(Index)
End Property

Public Property Get NewEnum()
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    Set NewEnum = oItems.[_NewEnum]
End Property

'''

Public Property Get At() As Date
    At = dAt
End Property

'''

Friend Sub Add(ByVal User As User, _
               Optional ByVal Key As Variant)
    Set User.Parent = Me
               
    If IsMissing(Key) Then
        oItems.Add User
    Else
        oItems.Add User, Key
    End If
    
    User.Index = oItems.Count
End Sub

Public Sub Remove(ByVal Index As Variant)
    Dim lStart As Long
    
    lStart = oItems(Index).Index
    
    oItems(Index).CleanUp
    oItems.Remove Index
    
    SetIndex lStart
End Sub

Public Sub Clear()
    Dim lCount As Long
    Dim l As Long
    
    lCount = oItems.Count
    
    For l = 1 To lCount
        oItems(1).CleanUp
        oItems.Remove 1
    Next
End Sub

Public Sub CleanUp()
    Clear
    Set oParent = Nothing
End Sub

Public Function Exists(ByVal Index As Variant) As Boolean
    Dim oUser As User
    
    On Error Resume Next

    Set oUser = oItems(Index)
    
    If Err.Number Then
        Exists = False
    Else
        Exists = True
    End If
End Function

Public Function NewUser(ByVal Name As String) As User
    Dim oUser As New User
    
    With oUser
        .Name = Name
    End With
    
    Add oUser, oUser.Name
    
    Set NewUser = oUser
End Function

'''

Public Function OpenUsers(Optional ByVal Asynchron As Boolean = False) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Not Asynchron Then
            bOK = Execute(FUNC_OPEN, False)
        Else
            QUEUE.QueueItems.Add Me, FUNC_OPEN
            bOK = True
        End If
    End If
    
    OpenUsers = bOK
End Function

Public Function SaveUsers(Optional ByVal Asynchron As Boolean = False) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Not Asynchron Then
            bOK = Execute(FUNC_SAVE, False)
        Else
            QUEUE.QueueItems.Add Me, FUNC_SAVE
            bOK = True
        End If
    End If
    
    SaveUsers = bOK
End Function

'''

Private Sub Class_Initialize()
    InitObject ObjectType
End Sub
Private Sub Class_Terminate()
    TermObject ObjectType
End Sub

'''

Public Function Execute(ByVal FunctionName As String, ByVal Asynchron As Boolean, Optional ByVal SenderID As Long) As Boolean
    Dim bOK As Boolean
    
    Database.Busy = True
    
    bOK = False
    
    Select Case FunctionName
        Case FUNC_OPEN: bOK = ExecuteOpenUsers(Asynchron)
        Case FUNC_SAVE: bOK = ExecuteSaveUsers(Asynchron)
    End Select
            
    Database.Busy = False
    
    Execute = bOK
End Function

Private Function ExecuteOpenUsers(ByVal bAsynchron As Boolean) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    Clear
    
    With Database
        If .Check() Then
            If DBMSRVUser.GetUsers(.ServerObject, Me) Then
                bOK = True
            Else
                .LastError.SetByServerObject .ServerObject
            End If
        End If
        
        If bOK Then dAt = Now()
        
        If bAsynchron Then
            .EventUsersAction Not bOK, dbmUsersActionOpen
        End If
    End With
    
    ExecuteOpenUsers = bOK
End Function

Private Function ExecuteSaveUsers(ByVal bAsynchron As Boolean) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    With Database
        If .Check() Then
            If DBMSRVUser.PutUsers(.ServerObject, Me) Then
                bOK = True
            Else
                .LastError.SetByServerObject .ServerObject
            End If
        End If
    
        If bAsynchron Then
            .EventUsersAction Not bOK, dbmUsersActionSave
        End If
    End With
    
    ExecuteSaveUsers = bOK
End Function

'''

Private Sub SetIndex(ByVal lStart As Long)
    Dim oUser As User
    Dim lCount As Long
    Dim l As Long
    
    lCount = oItems.Count
    
    For l = lStart To lCount
        Set oUser = oItems.Item(l)
        oUser.Index = l
    Next
End Sub




