VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "GenericMsgBox"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
'========================================================================================
'
' module:      GenericMsgBox.cls
'
'========================================================================================
'
' responsible:    RalfS
'
' special area:   Common
'
' description:    Class that provides the public functions for using the generic message
'                 box dialog.
'
'========================================================================================

Option Explicit

Const MODULE_NAME = "SQLStudio.GenericMsgBoxClass"

'==============================================================================
' Private member variables
'==============================================================================
Private mDlg    As New GenMsgBox

'=========================================================================================
'    property:      Button (Let)
'    description:   Sets the text of the button with the given index.
'    arguments:     [in] index - index of the button
'                   [in] value - new value of the label
'    return value:  -
'=========================================================================================
Public Property Let button(index As Integer, value As String)
    On Error GoTo errP_Let_Button

    Const FUNCTION_NAME = "P_LET_Button"

    mDlg.button(index) = value

    Exit Property

errP_Let_Button:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property

'=========================================================================================
'    property:      Button (Get)
'    description:   Gets the text of the button with the given index.
'    arguments:     [in] index - index of the button
'    return value:  String
'=========================================================================================
Public Property Get button(index As Integer) As String
    On Error GoTo errP_Get_Button

    Const FUNCTION_NAME = "P_GET_Button"

    button = mDlg.button(index)
    
    Exit Property

errP_Get_Button:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property

'=========================================================================================
'    property:      Caption (Let)
'    description:   Sets the caption of the dialog.
'    arguments:     [in] value - new value of the dialogs caption
'    return value:  -
'=========================================================================================
Public Property Let caption(value As String)
    On Error GoTo errP_Let_Caption

    Const FUNCTION_NAME = "P_LET_Caption"
    
    mDlg.caption = value
    
    Exit Property
    
errP_Let_Caption:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property

'=========================================================================================
'    property:      Caption (Get)
'    description:   Gets the caption of the dialog.
'    arguments:     -
'    return value:  String
'=========================================================================================
Public Property Get caption() As String
    On Error GoTo errP_Get_Caption

    Const FUNCTION_NAME = "P_GET_Caption"
    
    caption = mDlg.caption
    
    Exit Property
    
errP_Get_Caption:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property

'=========================================================================================
'    property:      DefaultButton (Let)
'    description:   Sets the default button of the dialog.
'    arguments:     [in] index - index of the button
'    return value:  -
'=========================================================================================
Public Property Let defaultButton(index As Integer)
    On Error GoTo errP_Let_DefaultButton

    Const FUNCTION_NAME = "P_LET_DefaultButton"

    mDlg.defaultButton = index
    
    Exit Property

errP_Let_DefaultButton:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property

'=========================================================================================
'    property:      EscapeButton (Let)
'    description:   Sets the escape button of the dialog.
'    arguments:     [in] index - index of the button
'    return value:  -
'=========================================================================================
Public Property Let escapeButton(index As Integer)
    On Error GoTo errP_Let_EscapeButton

    Const FUNCTION_NAME = "P_LET_EscapeButton"

    mDlg.escapeButton = index
    
    Exit Property

errP_Let_EscapeButton:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      Text (Let)
'    description:   Sets the message text of the dialog.
'    arguments:     [in] value - new value of the dialogs message
'    return value:  -
'=========================================================================================
Public Property Let text(value As String)
    On Error GoTo errP_Let_Text

    Const FUNCTION_NAME = "P_LET_Text"

    mDlg.text = value

    Exit Property

errP_Let_Text:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property

'=========================================================================================
'    property:      Text (Get)
'    description:   Gets the message text of the dialog.
'    arguments:     -
'    return value:  String
'=========================================================================================
Public Property Get text() As String
    On Error GoTo errP_Get_Text

    Const FUNCTION_NAME = "P_GET_Text"

    text = mDlg.text

    Exit Property

errP_Get_Text:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property

'=========================================================================================
'    function:      Show
'    description:   Displays the dialog.
'    arguments:     [in] parent - Parent control of the dialog (optional)
'    return value:  Integer
'=========================================================================================
Public Function show(Optional Parent As Object) As Integer
    On Error GoTo errShow

    Const FUNCTION_NAME = "show"

    If IsMissing(Parent) = True Then
        mDlg.show vbModal
    Else
        mDlg.show vbModal, Parent
    End If
    
    show = mDlg.selection

    Exit Function

errShow:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

'=========================================================================================
'    function:      Class_Initialize
'    description:   Initialize the class.
'    arguments:     -
'    return value:  -
'=========================================================================================
Private Sub Class_Initialize()
    On Error GoTo ErrClass_Initialize

    Const FUNCTION_NAME = "Class_Initialize"
    
    Load mDlg
    
    Exit Sub
    
ErrClass_Initialize:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub

'=========================================================================================
'    function:      Class_Terminate
'    description:   Clean up the class.
'    arguments:     -
'    return value:  -
'=========================================================================================
Private Sub Class_Terminate()
    On Error GoTo ErrClass_Terminate

    Const FUNCTION_NAME = "Class_Terminate"
    
    Unload mDlg
    
    Exit Sub
    
ErrClass_Terminate:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
