VERSION 5.00
Begin VB.UserControl PicButton 
   AutoRedraw      =   -1  'True
   ClientHeight    =   525
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   540
   ScaleHeight     =   35
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   36
   ToolboxBitmap   =   "PicButton.ctx":0000
End
Attribute VB_Name = "PicButton"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Enum PicButtonBorderConstants
    pbtBorderNone = 0
    pbtBorder3D = 1
    pbtBorder3DSoft = 2
End Enum
    
Enum PicButtonStateConstants
    pbtStateUp = 0
    pbtStateDown = 1
    pbtStateDisabled = 2
End Enum
    
Private lBorder As Long
Private bEnabled As Boolean
Private lState As Long
Private bStretch As Boolean
Private oPicUp As Picture
Private oPicDown As Picture
Private oPicDisabled As Picture
Private oPicMove As Picture
'
' Events
'
Public Event Click()
Public Event MouseDown(button As Integer, Shift As Integer, x As Single, y As Single)
Public Event MouseMove(button As Integer, Shift As Integer, x As Single, y As Single)
Public Event MouseUp(button As Integer, Shift As Integer, x As Single, y As Single)
'
' Properties
'
Public Property Get border() As PicButtonBorderConstants
    border = lBorder
End Property
Public Property Let border(ByVal NewValue As PicButtonBorderConstants)
    lBorder = NewValue
    PropertyChanged "Border"
    draw lState
End Property

Public Property Get enabled() As Boolean
    enabled = bEnabled
End Property
Public Property Let enabled(ByVal bNewValue As Boolean)
    bEnabled = bNewValue
    If bEnabled Then
        draw pbtStateUp
    Else
        draw pbtStateDisabled
    End If
    PropertyChanged "Enabled"
End Property

Public Property Get state() As PicButtonStateConstants
    state = lState
End Property
Public Property Let state(ByVal NewValue As PicButtonStateConstants)
    lState = NewValue
    PropertyChanged "State"
    draw lState
End Property

Public Property Get stretch() As Boolean
    stretch = bStretch
End Property
Public Property Let stretch(ByVal bNewValue As Boolean)
    bStretch = bNewValue
    PropertyChanged "Stretch"
    draw lState
End Property

Public Property Get pictureUp() As Picture
    Set pictureUp = oPicUp
End Property
Public Property Set pictureUp(ByVal oPicture As Picture)
    Set oPicUp = oPicture
    If oPicDown Is Nothing Then
        Set oPicDown = oPicture
        PropertyChanged "PictureDown"
    End If
    If oPicDisabled Is Nothing Then
        Set oPicDisabled = oPicture
        PropertyChanged "PictureDisabled"
    End If
    PropertyChanged "PictureUp"
    draw pbtStateUp
End Property

Public Property Get pictureDown() As Picture
    Set pictureDown = oPicDown
End Property
Public Property Set pictureDown(ByVal oPicture As Picture)
    Set oPicDown = oPicture
    PropertyChanged "PictureDown"
End Property

Public Property Get pictureDisabled() As Picture
    Set pictureDisabled = oPicDisabled
End Property
Public Property Set pictureDisabled(ByVal oPicture As Picture)
    Set oPicDisabled = oPicture
    PropertyChanged "PictureDisable"
End Property
'
' Events UserControl
'
Private Sub UserControl_Initialize()
    lBorder = pbtBorder3D
    bEnabled = True
    lState = pbtStateUp
    bStretch = False
    Set oPicUp = Nothing
    Set oPicDown = Nothing
    Set oPicDisabled = Nothing
End Sub

Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
    On Error Resume Next
    With PropBag
        border = .ReadProperty("Border", pbtBorder3D)
        enabled = .ReadProperty("Enabled", True)
        state = .ReadProperty("State", pbtStateUp)
        stretch = .ReadProperty("Stretch", False)
        Set pictureUp = .ReadProperty("PictureUp", Nothing)
        Set pictureDown = .ReadProperty("PictureDown", Nothing)
        Set pictureDisabled = .ReadProperty("PictureDisabled", Nothing)
    End With
End Sub

Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
    On Error Resume Next
    With PropBag
        .WriteProperty "Border", border, pbtBorder3D
        .WriteProperty "Enabled", enabled, True
        .WriteProperty "State", state, pbtStateUp
        .WriteProperty "Stretch", stretch, False
        .WriteProperty "PictureUp", pictureUp, Nothing
        .WriteProperty "PictureDown", pictureDown, Nothing
        .WriteProperty "PictureDisabled", pictureDisabled, Nothing
    End With
End Sub

Private Sub UserControl_Resize()
    draw lState
End Sub
'
'
'
Private Sub UserControl_Click()
    If bEnabled Then
        RaiseEvent Click
    End If
End Sub

Private Sub UserControl_MouseDown(button As Integer, Shift As Integer, x As Single, y As Single)
    If bEnabled Then
        If (button And vbLeftButton) = vbLeftButton Then
            draw pbtStateDown
        End If
        RaiseEvent MouseDown(button, Shift, x, y)
    End If
End Sub

Private Sub UserControl_MouseMove(button As Integer, Shift As Integer, x As Single, y As Single)
    If bEnabled Then
        RaiseEvent MouseMove(button, Shift, x, y)
    End If
End Sub

Private Sub UserControl_MouseUp(button As Integer, Shift As Integer, x As Single, y As Single)
    If bEnabled Then
        If (button And vbLeftButton) = vbLeftButton Then
            draw pbtStateUp
        End If
        RaiseEvent MouseUp(button, Shift, x, y)
    End If
End Sub
'
' Private Sub/Function
'
Private Function draw(ByVal lState As PicButtonStateConstants)
    Dim r As RECT
    Dim lEdge As Long
    Dim lBF As Long
    Dim x As Integer
    Dim y As Integer
    Dim w As Integer
    Dim h As Integer
    Dim iDown As Integer
    
    With r
        .Left = 0
        .Top = 0
        .Right = UserControl.ScaleWidth '- 1
        .Bottom = UserControl.ScaleHeight ' - 1
    End With
    
    UserControl.Cls
    
    If lBorder > pbtBorderNone Then
        Select Case lState
            Case pbtStateUp, pbtStateDisabled:  lEdge = EDGE_RAISED
            Case pbtStateDown:                  lEdge = EDGE_SUNKEN
        End Select
        
        lBF = BF_RECT
        Select Case lBorder
            Case pbtBorder3D:
            Case pbtBorder3DSoft: lBF = lBF + BF_SOFT
        End Select
        
        DrawEdge UserControl.hDC, r, lEdge, lBF
    End If
        
    Select Case lBorder
        Case pbtBorderNone:     iDown = 1
        Case pbtBorder3D:       iDown = 1
        Case pbtBorder3DSoft:   iDown = 1
    End Select
    
    Select Case lState
        Case pbtStateUp:
            If Not oPicUp Is Nothing Then
                If Not bStretch Then
                    x = (UserControl.ScaleWidth - ScaleX(oPicUp.Width, vbHimetric, vbPixels)) \ 2
                    y = (UserControl.ScaleHeight - ScaleY(oPicUp.Height, vbHimetric, vbPixels)) \ 2
                    UserControl.PaintPicture oPicUp, x, y
                Else
                    x = 3
                    y = 3
                    w = UserControl.ScaleWidth - 7
                    h = UserControl.ScaleHeight - 7
                    UserControl.PaintPicture oPicUp, x, y, w, h
                End If
            End If
        Case pbtStateDown:
            If Not oPicDown Is Nothing Then
                If Not bStretch Then
                    x = (UserControl.ScaleWidth - ScaleX(oPicDown.Width, vbHimetric, vbPixels)) \ 2 + iDown
                    y = (UserControl.ScaleHeight - ScaleY(oPicDown.Height, vbHimetric, vbPixels)) \ 2 + iDown
                    UserControl.PaintPicture oPicDown, x, y
                Else
                    x = 3 + iDown
                    y = 3 + iDown
                    w = UserControl.ScaleWidth - 7
                    h = UserControl.ScaleHeight - 7
                    UserControl.PaintPicture oPicUp, x, y, w, h
                End If
            End If
        Case pbtStateDisabled:
            If Not oPicDisabled Is Nothing Then
                If Not bStretch Then
                    x = (UserControl.ScaleWidth - ScaleX(oPicDisabled.Width, vbHimetric, vbPixels)) \ 2
                    y = (UserControl.ScaleHeight - ScaleY(oPicDisabled.Height, vbHimetric, vbPixels)) \ 2
                    UserControl.PaintPicture oPicDisabled, x, y
                Else
                    x = 3
                    y = 3
                    w = UserControl.ScaleWidth - 7
                    h = UserControl.ScaleHeight - 7
                    UserControl.PaintPicture oPicUp, x, y, w, h
                End If
            End If
    End Select
        
End Function
'
' Methodes
'
Public Function refresh()
    UserControl.refresh
End Function


