Attribute VB_Name = "Common"
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
'  Module:     Common -  Controls Common Functions
''
'-----------------------------------------------------------------------
'
' responsible:  WolfgangA
'
' special area: SQLStudio
'
' description:  Catalog view
'
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option Explicit

'module name
Const MODULE_NAME = "Catalogmanagement.Common"


'global variables
Public m_oComFunc                As New SQLStudioUtil.Functions


Public Const TABLESTAG = "TABLES"
Public Const INDEXESTAG = "INDEXES"
Public Const SEQUENCESTAG = "SEQUENCES"
Public Const FAVORITESTAG = "FAVORITES"
Public Const OWNEDUSERSTAG = "OWNEDUSERS"
Public Const DBPROCTAG = "DBPROC"
Public Const DBPROCTAGEXT = "DBPROCEXT"
Public Const DBPROCTAGINT = "DBPROCINT"
Public Const OWNEDUSERS = "OWNEDUSERS"

Public Const INDEXES = "Indexes"
Public Const SEQUENCES = "Sequences"
Public Const INDEPENDENT = "Independent"

Public Const FAVORITESOFUSERTAG = "Favorites of user tag"

Public Const ITEMOWNERTAG = "Item Owner"
Public Const LOCALSTORAGE = "Local Folder"
Public Const SOTABLETAG = "WA_MY_"

Public Const SOMAIN = "Stored Object Main"
Public Const SOFOLDERTAG = "Stored Object Folder"
Public Const SOITEMTAG = "Stored Object Item"

Public Const STOREDDSQL = "StoredSQL"
Public Const STOREDQBE = "StoredQBE"
Public Const STOREDVIQ = "StoredVIQ"
Public Const STOREDDRAFT = "CreateTable"

Public Const LINKVIQ = "LINKVIQ"
Public Const LINKQBE = "LINKQBE"
Public Const LINKDSQL = "LINKDSQL"
Public Const LINKSTOREDDRAFT = "LINKCREATETABLE"

Public Const LINKVIQCUT = "LINKVIQCUT"
Public Const LINKQBECUT = "LINKQBECUT"
Public Const LINKDSQLCUT = "LINKDSQLCUT"
Public Const LINKSTOREDDRAFTCUT = "LINKCREATETABLECUT"


Public Const DSQLCUT = "StoredSQLCut"
Public Const VIQCUT = "VIQCUT"
Public Const QBECUT = "QBECUT"
Public Const STOREDDRAFTCUT = "CREATETABLECUT"

Public Const FOLDERCLOSEDIMAGE = "FolderClosed"
Public Const FOLDEROPENIMAGE = "FolderOpen"
Public Const FOLDEROPENBROKEIMAGE = "FolderOpenBroke"
Public Const FOLDERCLOSEDBROKEIMAGE = "FolderClosedBroke"

Public Const FOLDERCLOSEDCUT = "FolderClosedCut"
Public Const FOLDEROPENCUT = "FolderOpenCut"

'=========================================================================================
'    function:      internalError
'    description:   shows message box with error
'    arguments:     [in] CurrentError, vb error object
'                   [in] Modulname, name of source modul
'                   [in] FunctionName, name of used function
'    return value:  /
'
'=========================================================================================
Public Sub internalError(ByVal CurrentError As ErrObject, ModulName As String, FunctionName As String)
    MsgBox "Module:" + Chr$(9) + ModulName + Chr$(13) + Chr$(10) + "Function:" + Chr$(9) + FunctionName + Chr$(13) + Chr$(10) + "Error No:" + Chr$(9) + Trim$(Str$(CurrentError.Number)) + Chr$(13) + Chr$(10) + "Error:" + Chr$(9) + CurrentError.Description, vbExclamation, "Unexpected Error:"
End Sub
'=========================================================================================
'    function:      commonResultOperation
'    description:   do a  bundle op operations with a db result object
'    arguments:     [in] oGlobalCom, oGlobalCom object
'                   [in] oResult, db result object
'                   [in] sStatement, sql statement to execute
'    return value:  /
'
'=========================================================================================
Public Function commonResultOperation(ByRef oGlobalCom As globalInfo, _
                                      ByRef oResult As SQLStudioDBLayer.Result, _
                                      ByVal sStatement As String) As Boolean
    On Error GoTo errCommonResultOperation
    
    Const FUNCTION_NAME = "commonResultOperation"
    
    'rc
    commonResultOperation = False
    
    oResult.limitForLong = -1
    If Not oResult.openResult(sStatement, Internal) Then
    
        If oResult.connection.Error.nativeError = "-807" Or _
           oResult.connection.Error.nativeError = "-708" Then

            oResult.showError
            If MsgBox(LoadResString(115), vbYesNo) = vbNo Then
                            
                oGlobalCom.Hourglass.State = qpArrowHourglassOff
                Exit Function
            Else
                If oGlobalCom.conMan.reconnect() = False Then
                    
                    MsgBox LoadResString(116), vbCritical
                    oGlobalCom.UserProfile.confirmation = False
                End If
                
                oGlobalCom.Hourglass.State = qpArrowHourglassOff
                Exit Function
            End If
         Else
            If timeOutReconnect(oResult.connection.Error) Then
            
                oResult.closeResult
                commonResultOperation = commonResultOperation(oGlobalCom, oResult, sStatement)
            Else
                oResult.connection.Error.showError 'show error
            End If
         End If
         
         oGlobalCom.Hourglass.State = qpArrowHourglassOff
         Exit Function
    End If
    
    ' get next row
    If Not oResult.fetch Then Exit Function
    
    'rc
    commonResultOperation = True
    
    Exit Function
    
errCommonResultOperation:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      isSynonymPublic
'    description:   is a synonym public
'    arguments:     [in] oConnection, a sap db connection object object
'                   [in] sOwner, owner of synonym
'                   [in] sName, name of symonym
'    return value:  /
'
'=========================================================================================
Public Function isSynonymPublic(oConnection As SQLStudioDBLayer.connection, _
                                ByVal sOwner As String, _
                                ByVal sName As String) As Boolean
    On Error GoTo errIsSynonymPublic

    Const FUNCTION_NAME = "isSynonymPublic"

    Dim sStatement            As String
    Dim oResult               As New SQLStudioDBLayer.Result
    
    'rc
    isSynonymPublic = False
    
    Set oResult.connection = oConnection
    
    sStatement = "SELECT PUBLIC FROM DOMAIN.SYNONYMS WHERE OWNER  = '" + _
                 sOwner + "' AND SYNONYMNAME = '" + sName + "'"
    If Not oResult.openResult(sStatement, Internal) Then Exit Function
    
    'get next row
    If Not oResult.fetch Then Exit Function
       
    'get data
    If Not oResult.noRows Then
    
        oResult.limitForLong = -1
        If oResult.GetData() Then
        
            If oResult.Columns(1).colValue = "YES" Then _
                isSynonymPublic = True
        End If
     End If
    
    Set oResult = Nothing
    
    Exit Function

errIsSynonymPublic:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      showDBFSError
'    description:   show a messagebox with a dbfs error
'    arguments:     [in]  lDBFS, handle to dbfs
'
'    return value:  /
'
'=========================================================================================
Public Function showDBFSError(lDBFS As Long)
    On Error GoTo errShowDBFSError
    
    Const FUNCTION_NAME = "showDBFSError"
    
    MsgBox getErrorMessage(lDBFS), vbCritical
    
    Exit Function
    
errShowDBFSError:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getStringFromUtf8
'    description:   get an vb string from a utf8 buffer
'    arguments:     [in]  bStartUtf8, utf8 buffer
'
'    return value:  string
'
'=========================================================================================
Public Function getStringFromUtf8(bStartUtf8() As Byte) As String
    On Error GoTo errGetStringFromUtf8
    
    Const FUNCTION_NAME = "getStringFromUtf8"

    Dim sBuffer      As String
    Dim bStartUcs2() As Byte
    Dim lUsed        As Long
    Dim lRet         As Long
    Dim lLen         As Long
    
    If Len(bStartUtf8(0)) < 1 Then Exit Function
    
    lLen = InStrB(bStartUtf8(), Chr$(0)) - 1
    'if there is no null termination take length of buffer
    If lLen < 0 Then
        lLen = Len(bStartUtf8(0)) + 1
    End If
    
    '=convert from utf8 to ucs2==============================================
    ReDim bStartUcs2((lLen * 2))
    lRet = convertUTF8ToVB_UCS2(bStartUtf8(0), _
                             lLen, _
                             bStartUcs2(0), _
                             lLen * 2, _
                             lUsed)
                             
    sBuffer = Left(bStartUcs2, lUsed \ 2)
    '=========================================================================
        
    getStringFromUtf8 = sBuffer
    
    Exit Function
    
errGetStringFromUtf8:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getUtf8FromString
'    description:   get an utf8 buffer from a vb String
'    arguments:     [in]  sData as string
'
'    return value:  utf8 byte array
'
'=========================================================================================
Public Function getUtf8FromString(sData As String) As Byte()
    On Error GoTo errGetUtf8FromString
    
    Const FUNCTION_NAME = "getUtf8FromString"
    
    '=variables=====================================================
    Dim bStartUcs2() As Byte
    Dim bStartUtf8() As Byte
    Dim lUsed        As Long
    Dim lRet         As Long
    '===============================================================
    
    bStartUcs2 = sData + vbNullChar
    ReDim bStartUtf8(Len(sData) * 6)
    
    lRet = convertVB_UCS2ToUTF8(bStartUcs2(0), _
                         Len(sData) * 2, _
                         bStartUtf8(0), _
                         Len(sData) * 6, _
                         lUsed)
        
    getUtf8FromString = bStartUtf8()
    
    Exit Function
    
errGetUtf8FromString:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

'=========================================================================================
'    function:      getErrorMessage
'    description:   get an error message from the dbfs
'    arguments:     [in]  lDBFS, handle to dbfs
'
'    return value:  error message as string
'
'=========================================================================================
Public Function getErrorMessage(lDBFS As Long) As String
    On Error GoTo errGetErrorMessage
    
    Const FUNCTION_NAME = "getErrorMessage"

    
    Dim bStartUtf8() As Byte
    ReDim bStartUtf8(2048)
    
    wd101CopyErrorMsg lDBFS, bStartUtf8(0), 2048
    getErrorMessage = getStringFromUtf8(bStartUtf8())
    
    Exit Function
    
errGetErrorMessage:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      makeDir
'    description:   make a directory
'    arguments:     [in] lDBFS, pointer to, database file system
'                   [in] sLocalRoot, root folder of local folder
'                   [in] sName, name of directory
'                   [in] bLocal, flag if action is local (on disk)
'    return value:  /
'
'=========================================================================================
Public Function makeDir(lDBFS As Long, _
                        sLocalRoot As String, _
                        sName As String, _
                        bLocal As Boolean) As Boolean
    On Error GoTo errMakeDir
    
    '=variables=====================================================
    Dim bStartUtf8() As Byte
    '===============================================================
        
    Const FUNCTION_NAME = "makeDir"
        
    makeDir = False
    
    If bLocal Then
    
        MkDir (sLocalRoot + sName)
    Else
        bStartUtf8() = getUtf8FromString(sName)
        If wd101MkDir(lDBFS, bStartUtf8(0)) = 0 Then
        
            showDBFSError (lDBFS)
            Exit Function
        End If
    End If
    
    makeDir = True
    
    Exit Function
    
errMakeDir:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      readFile
'    description:   reads a file
'    arguments:     [in] lDBFS a valid file system handle
'                   [in] sLocalRoot,root path of local directory
'                   [in] bLocal, flag if action is local (on disk)
'                   [in] sFileName, name of item
'                   [out] sBuffer, buffer for data
'
'    return value:  true if successfully
'
'=========================================================================================
Public Function readFile(lDBFS As Long, _
                         sLocalRoot As String, _
                         bLocal As Boolean, _
                         ByVal sFileName As String, _
                         ByRef sBuffer As String) As Boolean
    On Error GoTo errReadFile
   
    Const FUNCTION_NAME = "readFile"
    
    '=variables=====================================================
    Dim nFreeFile       As Long
    Dim lFileLen        As Long
    Dim lFile           As Long
    Dim lBytes          As Long
    Dim bBuf()          As Byte
    ReDim bBuf(SOFILE_LENGTH)
    Dim bStartUtf8FileName() As Byte
    '===============================================================
    
    'rc
    readFile = False
     
    If bLocal Then
    
        'first open file 'normal' to check if it exits
        'open in binary mode creates a file with 0k if it does no exits
        nFreeFile = FreeFile
        Open sLocalRoot + sFileName For Input Access Read As nFreeFile
        Close nFreeFile
        
        'now open for binary access
        nFreeFile = FreeFile
        Open sLocalRoot + sFileName For Binary Access Read As nFreeFile
        lFileLen = LOF(nFreeFile)
        sBuffer = Input(lFileLen, nFreeFile)
        Close nFreeFile
    Else
        'check file system handle
        If lDBFS = 0 Then Exit Function
        '=convert ucs2 to utf8 ================================
        bStartUtf8FileName() = getUtf8FromString(sFileName)
        '======================================================
        lFile = wd101Open(lDBFS, bStartUtf8FileName(0), 1)
        If lFile = -1 Then
    
            If timeOutReconnectDBFS(lDBFS) Then
                'call function again
                readFile lDBFS, sLocalRoot, bLocal, sFileName, sBuffer
                Exit Function
            End If
        End If
       
        lBytes = wd101Read(lDBFS, lFile, bBuf(0), SOFILE_LENGTH)
        If lBytes < 1 Then
        
            If wd101GetErrorId(lDBFS) <> 0 Then showDBFSError (lDBFS)
                        
            If MsgBox(LoadResString(350) + " " + Chr(34) + sFileName + Chr(34) + " " + LoadResString(351) + vbCrLf + LoadResString(352), vbYesNo) = vbYes Then
            
                If wd101Close(lDBFS, lFile) = 0 Then
        
                    showDBFSError (lDBFS)
                    Exit Function
                End If
                If wd101Delete(lDBFS, bStartUtf8FileName(0)) = 0 Then showDBFSError (lDBFS)
                Exit Function
            End If
        End If
        
        sBuffer = getStringFromUtf8(bBuf())
        If wd101Close(lDBFS, lFile) = 0 Then
        
            showDBFSError (lDBFS)
            Exit Function
        End If
        If lBytes < 0 Then Exit Function
    End If
    
    'rc is true id data in in buffer
    If Len(sBuffer) > 0 Then readFile = True
    
    Exit Function
 
errReadFile:
    Close nFreeFile
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getFileItemAttributes
'    description:
'    arguments:     [in] emItemType, item type
'                   [in] Info, file info
'                   [in] sTag, tag to get
'                   [in] sImage, image to get
'
'    return value:  see description
'
'=========================================================================================
Public Function getFileItemAttributes(emItemType As cvItemType, _
                                      Info As st_db_stat, _
                                      sTag As String, _
                                      sImage As String)
    On Error GoTo errGetFileItemAttributes
    
    Const FUNCTION_NAME = "getFileItemAttributes"
    
    Select Case emItemType
    
        Case cmStoredDSQL:
            If Info.mode = DB_FIL Then
            
                sImage = STOREDDSQL
                sTag = SOITEMTAG
            End If
        Case cmStoredQBE:
            If Info.mode = DB_FIL Then
            
                sImage = STOREDQBE
                sTag = SOITEMTAG
            End If
        Case cmStoredVIQ:
            If Info.mode = DB_FIL Then
            
                sImage = STOREDVIQ
                sTag = SOITEMTAG
            End If
        Case cmStoredDraft:
            
            If Info.mode = DB_FIL Then
            
                sImage = STOREDDRAFT
                sTag = SOITEMTAG
            End If
        Case Else:
            Exit Function
    End Select
    
    Exit Function
    
errGetFileItemAttributes:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      isItemLocal
'    description:   get if file or folder is a local item
'    arguments:     [in] nodeItem, node of item
'
'    return value:  see description
'
'=========================================================================================
Public Function isItemLocal(nodeItem As Node) As Boolean
    On Error GoTo errIsItemLocal
    
    Const FUNCTION_NAME = "isItemLocal"

    If Left(nodeItem.Tag, Len((SOFOLDERTAG + LOCALSTORAGE))) = (SOFOLDERTAG + LOCALSTORAGE) Then
    
        isItemLocal = True
    Else
        If Left(nodeItem.Tag, Len((SOITEMTAG + LOCALSTORAGE))) = (SOITEMTAG + LOCALSTORAGE) Then
        
            isItemLocal = True
        Else
            isItemLocal = False
        End If
    End If
    
    Exit Function
 
errIsItemLocal:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      isItemLocalByFullPath
'    description:   get if file or folder is a local item
'    arguments:     [in] sFullPath, full path of item
'
'    return value:  see description
'
'=========================================================================================
Public Function isItemLocalByFullPath(sFullPath As String) As Boolean
    On Error GoTo errIsItemLocalByFullPath
    
    Const FUNCTION_NAME = "IsItemLocalByFullPath"

    If Left(sFullPath, Len("/" + LOCALSTORAGE)) = "/" + LOCALSTORAGE Then
    
        isItemLocalByFullPath = True
    Else
        isItemLocalByFullPath = False
    End If
    
    Exit Function
 
errIsItemLocalByFullPath:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      openDBDir
'    description:   open and read a directory
'    arguments:     [in] lDBFS a valid file system handle
'                   [in] sName,name of directory to open
'                   [in] sOut, dir content
'
'    return value:  file number or -1 if no success
'
'=========================================================================================
Public Function openDBDir(lDBFS As Long, _
                          sName As String, _
                          sOut As String) As Long
    On Error GoTo ErrOpenDBDir

    Const FUNCTION_NAME = "openDBDir"

    '=local variables====================
    Dim lfno            As Long
    Dim lLen            As Long
    Dim lRet            As Long
    
    Dim bStartUtf8Name() As Byte
    Dim bStartUtf8Buffer(2048) As Byte
    '====================================
    
    'rc
    openDBDir = -1
    
    If lDBFS < 1 Then Exit Function
    
    '=convert from ucs2 to utf8 ============================
    bStartUtf8Name() = getUtf8FromString(sName)
    '======================================================
    lfno = wd101OpenDir(lDBFS, bStartUtf8Name(0), 6)
    If lfno = -1 Then
    
       If timeOutReconnectDBFS(lDBFS) Then
            'call function again
            openDBDir lDBFS, sName, sOut
            Exit Function
       End If
    End If
      
    lRet = wd101GetDirEntry(lDBFS, lfno, bStartUtf8Buffer(0), UBound(bStartUtf8Buffer()) - 1, 0)
    sOut$ = getStringFromUtf8(bStartUtf8Buffer())
    If lRet = -1 Then
    
       If wd101GetErrorId(lDBFS) <> WA_ERR_NO_MORE_ENTRIES Then
       
            showDBFSError (lDBFS)
            Exit Function
        End If
    End If
    
    'true
    openDBDir = lfno
      
    Exit Function
 
ErrOpenDBDir:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getDBDirEntry
'    description:   get the entries of a directory
'    arguments:     [in] lDBFS a valid file system handle
'                   [in] lfno, file number
'                   [in] sOut, dir content
'
'    return value:  true if successfully
'
'=========================================================================================
Public Function getDBDirEntry(lDBFS As Long, _
                              lfno As Long, _
                              sOut As String) As Long
    On Error GoTo errGetDBDirEntry

    Const FUNCTION_NAME = "getDBDirEntry"

    '=local variables====================
    Dim lLen            As Long
    Dim lRet            As Long
    Dim bStartUtf8Buffer(2048) As Byte
    '====================================
    
    'rc
    getDBDirEntry = -1
    lRet = wd101GetDirEntry(lDBFS, _
                            lfno, _
                            bStartUtf8Buffer(0), _
                            UBound(bStartUtf8Buffer()) - 1, _
                            0)
                            
    '=convert from utf8 to ucs2============================
    sOut$ = getStringFromUtf8(bStartUtf8Buffer())
    '======================================================
       
    'true
    getDBDirEntry = lRet
      
    Exit Function
 
errGetDBDirEntry:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      isItemAnyDBFSFolder
'    description:   determine if itemtype is a dbfs folder
'    arguments:     [in] itemType, item to check
'
'    return value:  true if it item is a dbfile system folder
'
'=========================================================================================
Public Function isItemAnyDBFSFolder(itemType As cvItemType) As Boolean
    On Error GoTo errIsItemAnyDBFSFolder
   
    Const FUNCTION_NAME = "isItemAnyDBFSFolder"

    'rc
    isItemAnyDBFSFolder = False
    
    If itemType = cmSOFolder Then isItemAnyDBFSFolder = True
        
    Exit Function
 
errIsItemAnyDBFSFolder:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      isItemAnyDBFSFolder
'    description:   determine if itemtype is any dbfs object type
'    arguments:     [in] itemType, item to check
'
'    return value:  true if it item is any dbfs object type
'
'=========================================================================================
Public Function isItemAnyDBFSObject(itemType As cvItemType) As Boolean
    On Error GoTo errisItemAnyDBFSObject
   
    Const FUNCTION_NAME = "isItemAnyDBFSObject"

    'rc
    isItemAnyDBFSObject = False
    
    If itemType = cmSOFolder Or _
       itemType = cmStoredDraft Or _
       itemType = cmStoredDSQL Or _
       itemType = cmStoredQBE Or _
       itemType = cmStoredVIQ Or _
       itemType = cmSequence _
    Then
        isItemAnyDBFSObject = True
    End If
    
    Exit Function
 
errisItemAnyDBFSObject:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      isItemSOFile
'    description:   determine if itemtype is a stored object file
'    arguments:     [in] itemType, item to check
'
'    return value:  true if it item is a stored object file
'
'=========================================================================================
Public Function isItemSOFile(itemType As cvItemType) As Boolean
    On Error GoTo errIsItemSOFile
   
    Const FUNCTION_NAME = "isItemSOFile"

    'rc
    isItemSOFile = False
    
    If itemType = cmStoredDraft Or itemType = cmStoredDSQL Or _
       itemType = cmStoredQBE Or itemType = cmStoredVIQ _
    Then
        isItemSOFile = True
    End If
    
    Exit Function
 
errIsItemSOFile:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      isItemSOLink
'    description:   determine if itemtype is a link to a stored object file
'    arguments:     [in] itemType, item to check
'
'    return value:  true if it item is a link to a stored object file
'
'=========================================================================================
Public Function isItemSOLink(itemType As cvItemType) As Boolean
    On Error GoTo errIsItemSOLink
   
    Const FUNCTION_NAME = "isItemSOLink"

    'rc
    isItemSOLink = False
    
    If itemType = cmLinkDraft Or itemType = cmLinkDSQL Or _
       itemType = cmLinkQBE Or itemType = cmLinkVIQ _
    Then
        isItemSOLink = True
    End If
    
    Exit Function
 
errIsItemSOLink:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      isItemDBObject
'    description:   determine if itemtype is a dbfs folder
'    arguments:     [in] itemType, item to check
'
'    return value:  true if it item is a dbfile system folder
'
'=========================================================================================
Public Function isItemDBObject(itemType As cvItemType) As Boolean
    On Error GoTo errIsItemDBObject
   
    Const FUNCTION_NAME = "isItemDBObject"

    'rc
    isItemDBObject = False
    
    If itemType = cmTable Or itemType = cmView Or _
       itemType = cmSystem Or itemType = cmSynonym Or _
       itemType = cmResult Or itemType = cmSnapshot Or _
       itemType = cmSequence Or itemType = cmIndex Or _
       itemType = cmDBProcInt Or itemType = cmDBProcExt Or _
       itemType = cmUser Or itemType = cmUserGroup _
    Then
        isItemDBObject = True
    End If
    
    Exit Function
 
errIsItemDBObject:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      showDBFSError
'    description:   show a messagebox with a dbfs error
'    arguments:     [in]  sItemType, identifies the tabletype in the database
'                   [in] sItemName, identifies the itemname in the database
'                   [out] itemType, enumtype for tabletype
'                   [out] sImageKey, key for image of tree item
'                   [out] sItemText, text of tree item
'
'    return value:  /
'
'=========================================================================================
Public Function switchTableType(sItemType As String, _
                                sItemName As String, _
                                itemType As cvItemType, _
                                sImageKey As String, _
                                sItemText As String)
    On Error GoTo errSwitchTableType
    
                    
    
    Const FUNCTION_NAME = "switchTableType"
    
    'init
    itemType = cmUnknown
    sImageKey = "TABLE"

    Select Case sItemType
    
        Case "VIEW"
            itemType = cmView
            sImageKey = "VIEW"
            sItemText = sItemName
        Case "TABLE"
            itemType = cmTable
            sItemText = sItemName
        Case "SYSTEM"
            itemType = cmSystem
            sItemText = sItemName
            sImageKey = "SYSTEMTABLE"
        Case "RESULT"
            itemType = cmResult
            sItemText = sItemName
        Case "SYNONYM"
            itemType = cmSynonym
            sImageKey = "Synonym"
            sItemText = sItemName
        Case "SNAPSHOT"
            itemType = cmSnapshot
            sItemText = sItemName
    End Select

    Exit Function
    
errSwitchTableType:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      timeOutReconnectDBFS
'    description:   check if there was a timeout error
'                   if there is another error show a messagebox
'                   with a dbfs error
'    arguments:     [in]  lDBFS, handle to dbfs
'
'    return value:  true if timeout occured
'
'=========================================================================================
Public Function timeOutReconnectDBFS(lDBFS As Long) As Boolean
    On Error GoTo errtimeOutReconnectDBFS
   
    Const FUNCTION_NAME = "timeOutReconnectDBFS"

    'rc
    timeOutReconnectDBFS = False
    
    Dim sBuffer As String
        
    sBuffer = getErrorMessage(lDBFS)
    If (InStr(sBuffer, "-22005") <> 0 And InStr(UCase(sBuffer), "TIMEOUT") <> 0) Or (InStr(UCase(sBuffer), "TIMEOUT EXPIRED")) Then
        timeOutReconnectDBFS = True
    Else
        MsgBox sBuffer, vbCritical
    End If
    
    Exit Function
 
errtimeOutReconnectDBFS:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function


'=========================================================================================
'    function:      timeOutReconnect
'    description:   check if there was a timeout error
'                   if there is another error show a messagebox
'                   with a dbfs error
'    arguments:     [in]  lDBFS, handle to dbfs
'
'    return value:  true if timeout occured
'
'=========================================================================================
Public Function timeOutReconnect(oError As SQLStudioDBLayer.Error) As Boolean
    On Error GoTo errtimeOutReconnect
   
    Const FUNCTION_NAME = "timeOutReconnect"

    'rc
    timeOutReconnect = False
    
    If (InStr(oError.Text, "-22005") <> 0 And InStr(UCase(oError.Text), "TIMEOUT") <> 0) Or (InStr(UCase(oError.Text), "TIMEOUT EXPIRED")) Then
        timeOutReconnect = True
    End If
    
    Exit Function
 
errtimeOutReconnect:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

