Attribute VB_Name = "Core"
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
'  Module:     Core
'
'-----------------------------------------------------------------------
'
' responsible:  WolfgangA
'
' special area: SQLStudio
'
' description:  Catalog view
'
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option Explicit

'module name
Const MODULE_NAME = "CatalogManagement.Core"

Global Const SOFILE_LENGTH = 32768

' Database file modes
Public Const DB_DIR = 1
Public Const DB_FIL = 2
Public Const DB_LIN = 3

Type st_db_stat

    mode As Integer
    size As Long
End Type

Global Const WA_ERR_CANT_GET_ODBC_HANDLES = 110
Global Const WA_ERR_CANT_GET_DBCONNECTION = 111
Global Const WA_ERR_INVALID_FILE_DESC = 112
Global Const WA_ERR_FILE_NOT_FOUND = 113
Global Const WA_ERR_FILE_OPEN = 114
Global Const WA_ERR_WRITE_ERROR = 115
Global Const WA_ERR_DIR_NOT_FOUND = 116
Global Const WA_ERR_NOT_DIR = 117
Global Const WA_ERR_NOT_FILE = 118
Global Const WA_ERR_DIR_EXISTS = 119
Global Const WA_ERR_DIR_NOT_EMPTY = 120
Global Const WA_ERR_FILE_OR_DIR_NOT_FOUND = 121
Global Const WA_ERR_FILE_OR_DIR_EXISTS = 122
Global Const WA_ERR_INVALID_FILENAME = 123
Global Const WA_ERR_FILENAME_TOO_LONG = 124
Global Const WA_ERR_DIFFERENT_PATH_NAMES = 125
Global Const WA_ERR_COMPRESS = 126
Global Const WA_ERR_UNCOMPRESS = 127
Global Const WA_ERR_SUBDIR_NOT_FOUND = 128
Global Const WA_ERR_NO_MORE_ENTRIES = 131

Global Const ERR_CANT_GET_ODBC_HANDLES_WD26 = WA_ERR_CANT_GET_ODBC_HANDLES
Global Const ERR_CANT_GET_DBCONNECTION_WD26 = WA_ERR_CANT_GET_DBCONNECTION
Global Const ERR_INVALID_FILE_DESC_WD26 = WA_ERR_INVALID_FILE_DESC
Global Const ERR_FILE_NOT_FOUND_WD26 = WA_ERR_FILE_NOT_FOUND
Global Const ERR_FILE_OPEN_WD26 = WA_ERR_FILE_OPEN
Global Const ERR_WRITE_ERROR_WD26 = WA_ERR_WRITE_ERROR
Global Const ERR_DIR_NOT_FOUND_WD26 = WA_ERR_DIR_NOT_FOUND
Global Const ERR_NOT_DIR_WD26 = WA_ERR_NOT_DIR
Global Const ERR_NOT_FILE_WD26 = WA_ERR_NOT_FILE
Global Const ERR_DIR_EXISTS_WD26 = WA_ERR_DIR_EXISTS
Global Const ERR_DIR_NOT_EMPTY_WD26 = WA_ERR_DIR_NOT_EMPTY
Global Const ERR_FILE_OR_DIR_NOT_FOUND_WD26 = WA_ERR_FILE_OR_DIR_NOT_FOUND
Global Const ERR_FILE_OR_DIR_EXISTS_WD26 = WA_ERR_FILE_OR_DIR_EXISTS
Global Const ERR_INVALID_FILENAME_WD26 = WA_ERR_INVALID_FILENAME
Global Const ERR_FILENAME_TOO_LONG_WD26 = WA_ERR_FILENAME_TOO_LONG
Global Const ERR_DIFFERENT_PATH_NAMES_WD26 = WA_ERR_DIFFERENT_PATH_NAMES
Global Const ERR_COMPRESS_WD26 = WA_ERR_COMPRESS
Global Const ERR_UNCOMPRESS_WD26 = WA_ERR_UNCOMPRESS
Global Const ERR_SUBDIR_NOT_FOUND_WD26 = WA_ERR_SUBDIR_NOT_FOUND


Public Enum en_access

 no_access = 0
 access_read = 1
 access_write = 2
 access_write_compressed = 3
 access_readwrite = 4
End Enum

Public Enum dragdrop_mode

  dd_none = 0
  dd_move = 1
  dd_copy = 2
  dd_link = 3
End Enum


                                    
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'  Function: wd101CreateDbcDBFS
'
'  description: Create a new DBFS object that uses connections from the
'               WADBCPoolP.
'
'arguments:
'      dbcPool    [in] The dbcPool that is used by the fs to get database
'                      connections.
'      datasource [in] The name of the datasource that is used to connect
'                      with dbcPool.
'      driver     [in] Instead of a datasource you can specify the
'                      drivername to be used.
'      serverNode [in] The hostname of the database server.
'      serverDb   [in] The name of the server DB.
'      user       [in] The user for the DBC connection.
'      password   [in] The password of user.
'      sqltrace   [in] An optional filename. If this is not NULL and not
'                      emty it specifies the filename of the precompiler
'                      SQL-trace. To prevent trace-file-open errors a
'                      unique id is appended to this name for every
'                      new connection.
'      tabPrefix  [in] The prefix for the INODE and CONTAINER tables. This
'                      allows us to define multiple filesystems for each
'                      user.
'
'  return value: New twd101DBFSP or NULL if creation failed.
'
'  note: The pool must not be used for other dbfs-objects with a different
'        tabPrefix, because the sql-statements are parsed for specific
'        table-names!!!
'
'twd101DBFSP WINAPI wd101CreateDbcDBFS( WADBCPoolP      dbcPool,
'                                       const char     *datasource,
'                                       const char     *driver,
'                                       const char     *serverNode,
'                                       const char     *serverDb,
'                                       const char     *user,
'                                       const char     *password,
'                                       const char     *sqltrace,
'                                       const char     *tabPrefix );
                                    
Declare Function wd101CreateDbcDBFS Lib "dbfsapi.dll" _
                                    (ByVal pool As Long, _
                                    ByRef datasource As Byte, _
                                    ByRef driver As Byte, _
                                    ByRef ServerNode As Byte, _
                                    ByRef ServerDB As Byte, _
                                    ByRef user As Byte, _
                                    ByRef password As Byte, _
                                    ByRef sqltrace As Byte, _
                                    ByVal pID As Long) As Long
                                    
Declare Function wd101DestroyDBFS Lib "dbfsapi.dll" (ByVal dbfs As Long) As Byte

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'wd101OpenDir
'twd101DBFile wd101OpenDir( twd101DBFSP dbfs, const char *path );
'description:
'Open a directory for reading.
'arguments:
'dbfs [In]
'DBFS used.
'path [In]
'The name of the directory to be opened.
'return value:
'The handle to the opened directory or -1 for failure. If the function fails wd101CmdGetErrorId and wd101CmdGetErrorMsg can be used to retrieve error information.


'Switch (OrderFlag)
'        {
'        Case name_asc:
'            strcat( SelectStr, "ORDER BY Name ASC");
'            break;
'        Case name_desc:
'            strcat( SelectStr, "ORDER BY Name DESC");
'            break;
'        Case size_asc:
'            strcat( SelectStr, "ORDER BY Size ASC");
'            break;
'        Case size_desc:
'            strcat( SelectStr, "ORDER BY Size DESC");
'            break;
'        Case type_asc:
'            strcat( SelectStr, "ORDER BY Type ASC");
'            break;
'        Case type_desc:
'            strcat( SelectStr, "ORDER BY Type DESC");
'            break;
'        Case not_def:
'default:
'            break;
'        }
        
'typedef enum en_order_flag {
'    name_asc,
'    name_desc,
'    size_asc,
'    size_desc,
'    type_asc,
'    type_desc,
'   not_def
'} twd101OrderFlag;

Declare Function wd101OpenDir Lib "dbfsapi.dll" _
                (ByVal dbfs As Long, ByRef Path As Byte, ByVal lOrderFlag As Long) As Long
                                    

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'wd101CloseDir
'tsp00_Bool wd101CloseDir( twd101DBFSP dbfs, twd101DBFile fno );
'description:
'Close a directory that was opened with wd0101OpenDir.
'arguments:
'dbfs [In]
'The DBFS object used.
'fno [In]
'The handle of the directory to be closed.
'return value:
'true for success, false for failure. The functions wd101GetErrorId and wd101GetErrorMsg can be used to retrieve the reason for a failure.
Declare Function wd101CloseDir Lib "dbfsapi.dll" _
        (ByVal dbfs As Long, ByVal fno As Long) As Byte


                                    
'tsp00_Int4 wd101GetDirEntry( twd101DBFSP  dbfs,
'                             twd101DBFile fno,
'                             char        *buf,
'                             tsp00_Int4   bufLen,
'                             tsp00_Bool   details );
'description:
'Return a formatted line of a directory listing.
'precondition:
'The directory was opened with wd101OpenDir.
'arguments:
'dbfs [In]
'DBFS used.
'fno [In]
'Handle for the open directory.
'buf [out]
'Buffer to return the formatted line.
'bufLen [In]
'The size of buf.
'details [In]
'If this is true the dir-entry is returned with detailed information (file-date etc.). The line is formatted in the UNIX 'ls -l' style. If this is false only the file or directory name is returned in buf.
'return value:
'The number of bytes written to buf or -1 for failure. If the function fails wd101CmdGetErrorId and wd101CmdGetErrorMsg can be used to retrieve error information
Declare Function wd101GetDirEntry Lib "dbfsapi.dll" _
                            (ByVal dbfs As Long, _
                             ByVal fno As Long, _
                             ByRef buf As Byte, _
                             ByVal buflen As Long, _
                             ByVal details As Boolean) As Long



'wd101Stat
'tsp00_Bool wd101Stat( twd101DBFSP        dbfs,
'                      const char        *path,
'                      struct st_db_stat *buf );
'description:
'Get information for a file.
'arguments:
'dbfs [In]
'DBFS, to be used.
'path [In]
'Filename with absolute path.
'buf [out]
'Buffer used to return information.
'return value:
'true for success, false for failure. If the function fails wd101CmdGetErrorId and wd101CmdGetErrorMsg can be used to retrieve error information.

Declare Function wd101Stat Lib "dbfsapi.dll" _
                        (ByVal dbfs As Long, _
                         ByRef Path As Byte, _
                         ByRef buf As st_db_stat) As Byte



'wd101MkDir
'tsp00_Bool wd101MkDir( twd101DBFSP dbfs, const char  *path );
'description:
'Create a new directory. If a directory with the given name already eixts the function will fail.
'arguments:
'dbfs [In]
'DBFS, to be used.
'path [In]
'The name of the new directory with absolute path.
'return value:
'true for success, false for failure. If the function fails wd101CmdGetErrorId and wd101CmdGetErrorMsg can be used to retrieve error information.

Declare Function wd101MkDir Lib "dbfsapi.dll" _
                (ByVal dbfs As Long, _
                 ByRef Path As Byte) As Byte



'void WINAPI wd101CopyErrorMsg( twd101DBFSP     dbfs,
'                               char           *buf,
'                               tsp00_Uint4     bufLen )

Declare Function wd101CopyErrorMsg Lib "dbfsapi.dll" _
                (ByVal dbfs As Long, _
                 ByRef buf As Byte, _
                 ByVal buflen As Long) As Byte



'tsp00_Uint4 WINAPI wd101GetErrorId( twd101DBFSP dbfs );

Declare Function wd101GetErrorId Lib "dbfsapi.dll" _
                (ByVal dbfs As Long) As Long

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'wd101RmDir
'tsp00_Bool WINAPI wd101RmDir( twd101DBFSP  dbfs,
'                              const char  *path,
'                              tsp00_Bool   recursive );

'description:
'Remove a directory in the dbfs.
'precondition:
'wd101InitDBFS has been called.
'arguments:
'path [In]
'recursive  [in]  Indicates whether the contents of the directory
'                 should also be removed. If this is true all files
'                  and subdirectories in this directory are deleted.
'
'The directory to be deleted.
'return value:
'true for success, false for failure. If the function fails wd101CmdGetErrorId and wd101CmdGetErrorMsg can be used to retrieve error information.
'note:
'This function can only be used by single-threaded applications.
Declare Function wd101RmDir Lib "dbfsapi.dll" _
                (ByVal dbfs As Long, ByRef Path As Byte, ByVal bRecursive As Boolean) As Byte

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'wd101Rename
'tsp00_Bool WINAPI wd101Rename( twd101DBFSP  dbfs,
'                               const char  *oldName,
'                               const char  *newName );
'description:
'Rename a file or directory.
'arguments:
'dbfs [In]
'DBFS used.
'oldName [In]
'The file or directory that should be renamed.
'newName [In]
'The new name.
'return value:
'true for success or false for failure. If the function fails wd101CmdGetErrorId and wd101CmdGetErrorMsg can be used to retrieve error information.

Declare Function wd101Rename Lib "dbfsapi.dll" _
                (ByVal dbfs As Long, _
                 ByRef sOldName As Byte, _
                 ByRef sNewName As Byte) As Byte

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'wd101Create
'twd101DBFile WINAPI wd101Create( twd101DBFSP     dbfs,
'                                 const char     *path,
'                                 twd101OpenFlag  oflag );
'description:
'Create a new file in the DBFS. If a file with the given name already exists it is deleted.
'arguments:
'dbfs [In]
'DBFS, that is used.
'path [In]
'The name of the new file with absolute path.
'oflag [In]
'access_write or access_write_compressed.
'return value:
'A handle for the new file or FILE_ERROR_WD101 if the creation failed. The functions wd101GetErrorId and wd101GetErrorMsg can be used to retrieve the reason for a failure.

'typedef enum en_access {
' no_access = 0,
' access_read,
' access_write,
' access_write_compressed,
' access_readwrite
'} twd101OpenFlag

Declare Function wd101Create Lib "dbfsapi.dll" _
                                 (ByVal dbfs As Long, _
                                  Path As Byte, _
                                  ByVal oflag As Long) As Long
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'wd101Delete
'tsp00_Bool WINAPI wd101Delete( twd101DBFSP  dbfs, const char  *Path );
'description:
'Delete a file in the DBFS.
'arguments:
'dbfs [In]
'DBFS, that is used.
'path [In]
'The name of the file to delete with absolute path.
'return value:
'true for success, false for failure. The functions wd101GetErrorId and wd101GetErrorMsg can be used to retrieve the reason for a failure.


Declare Function wd101Delete Lib "dbfsapi.dll" _
        (ByVal dbfs As Long, ByRef Path As Byte) As Byte

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'wd101Open
'twd101DBFile WINAPI wd101Open( twd101DBFSP     dbfs,
'                               const char     *path,
'                               twd101OpenFlag  oflag );
'description:
'Open a file for reading or writing.
'arguments:
'dbfs [In]
'DBFS used.
'Path [In]
'The name of the file to be opened.
'oflag [In]
'ignored.
'return value:
'The handle to the opened file or -1 for failure. If the function fails wd101CmdGetErrorId and wd101CmdGetErrorMsg can be used to retrieve error information.


Declare Function wd101Open Lib "dbfsapi.dll" _
                                (ByVal dbfs As Long, _
                                 Path As Byte, _
                                 ByVal oflag As Long) As Long

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'wd101Close
'tsp00_Bool WINAPI wd101Close( twd101DBFSP dbfs, twd101DBFile fno );
'description:
'Close a file that was opened with wd101Create or wd0101Open.
'arguments:
'dbfs [In]
'The DBFS object used.
'fno [In]
'The handle of the file to be closed.
'return value:
'true for success, false for failure. The functions wd101GetErrorId and wd101GetErrorMsg can be used to retrieve the reason for a failure.

Declare Function wd101Close Lib "dbfsapi.dll" _
        (ByVal dbfs As Long, ByVal fno As Long) As Byte



'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'wd101Read
'tsp00_Int4 WINAPI wd101Read( twd101DBFSP   dbfs,
'                             twd101DBFile  fno,
'                             void         *buf,
'                             tsp00_Uint4   nbyte );
'description:
'Read data from an opened file.
'arguments:
'dbfs [In]
'DBFS used.
'Path [In]
'File to read from.
'buf [out]
'Buffer in which the data is stored.
'nByte [In]
'Size of buf.
'return value:
'The number of bytes read or -1 for failure. If the function fails wd101CmdGetErrorId and wd101CmdGetErrorMsg can be used to retrieve error information.
Declare Function wd101Read Lib "dbfsapi.dll" _
                                (ByVal dbfs As Long, _
                                 ByVal fno As Long, _
                                 buf As Byte, _
                                 ByVal lbufsize As Long) As Long




'wd101Write
'
'tsp00_Int4 WINAPI wd101Write(twd101DBFSP   dbfs,
'                             twd101DBFile  fno,
'                             void         *buf,
'                             tsp00_Uint4   nbyte );
'description:
'Write data to an open file. The file must have been opened with wd101Create. It is not possible to overwrite or append data to a file opened with wd101Open.
'arguments:
'dbfs [In]
'DBFS, to be used.
'fno [In]
'Filehandle.
'buf [In]
'Buffer with data to write.
'nByte [In]
'Number of bytes to write.
'return value:
'The number of bytes written or -1 for failure. If the function fails wd101CmdGetErrorId and wd101CmdGetErrorMsg can be used to retrieve error information.

Declare Function wd101Write Lib "dbfsapi.dll" _
                                (ByVal dbfs As Long, _
                                 ByVal fno As Long, _
                                 buf As Byte, _
                                 ByVal lbufsize As Long) As Long



'/*!**********************************************************************
'  Function: wd101Mv
'
'  description: Move files or directory trees from one location to another.
'
'arguments:
'    dbfs     [in]  DBFS, to be used.
'    oldPath  [in]  The name of the old file or directory with absolute
'                   path.
'    newPath  [in]  The name of the new file or directory.
'
'  return value: true for success, false for failure. If the function fails
'                wd101CmdGetErrorId and wd101CmdGetErrorMsg can be used to
'                retrieve error information.
'
'  remarks: oldPath must be an existing file or directory. If oldPath is a
'           file the following rules apply:
'           <UL>
'           <LI> If newPath is an existing directory then the file is
'                moved to this directory with its original name.
'           <LI> If newPath does not exist then a file newPath is created
'                with the content of the file oldPath. If the parent
'                directory of newPath does not exist you get the error
'                ERR_DIR_NOT_FOUND_WD26.
'           <LI> If newPath is an existing file wd101Mv returns false with
'                the error ERR_FILE_OR_DIR_EXISTS_WD26.
'           </UL>
'
'           If oldPath is a directory the following rules apply:
'           <UL>
'           <LI> If newPath is an existing directory then the directory and
'                all its subdirectories and files are moved to newPath.
'           <LI> If newPath does not exist then a directory newPath is created
'                and all subdirectories and files of oldPath are moved to
'                newPath. If the parent directory of newPath does not exist
'                you get the error ERR_DIR_NOT_FOUND_WD26.
'           <LI> If newPath is an existing file wd101Mv returns false with
'                the error ERR_FILE_OR_DIR_EXISTS_WD26.
'           </UL>
'  Error -codes:
'  <UL>
'  <LI> ERR_FILE_OR_DIR_NOT_FOUND_WD26: sourcePath does not exist.
'  <LI> ERR_FILE_OR_DIR_EXISTS_WD26: destPath is an existing file.
'  <LI> ERR_DIR_NOT_FOUND_WD26: sourcePath is a file and destPath
'       specifies a non-existing file in a non-existing path or sourcePath
'       is a directory and destPath is not a directory.
'  </UL>
'*/
'tsp00_Bool WINAPI wd101Mv( twd101DBFSP  dbfs,
'                           const char  *oldPath,
'                           const char  *newPath );
'

Declare Function wd101Mv Lib "dbfsapi.dll" _
                                (ByVal dbfs As Long, _
                                 ByRef oldPath As Byte, _
                                 ByRef newPath As Byte _
                                 ) As Byte



'/*!**********************************************************************
'  Function: wd101Copy
'
'  description: Copy a file or directory tree.
'
'arguments:
'    dbfs        [in]  DBFS, that is used.
'    sourcePath  [in]  The name of the source file or directory with
'                      absolute path.
'    destPath    [in]  The name of the new file or directory.
'
'  return value: true for success, false for failure. If the function fails
'                wd101CmdGetErrorId and wd101CmdGetErrorMsg can be used to
'                retrieve error information.
'
'  remarks: If source path is a directoy then destPath must also be a
'           directory. If sourcePath is a file then destPath may be an
'           existing directory or a non-existing filename.
'
'  Error -codes:
'  <UL>
'  <LI> ERR_FILE_OR_DIR_NOT_FOUND_WD26: sourcePath does not exist.
'  <LI> ERR_FILE_OR_DIR_EXISTS_WD26: destPath is an existing file.
'  <LI> ERR_DIR_NOT_FOUND_WD26: sourcePath is a file and destPath
'       specifies a non-existing file in a non-existing path or sourcePath
'       is a directory and the subdirectory of destPath does not exist.
'  </UL>
'*/
'tsp00_Bool WINAPI wd101Copy( twd101DBFSP     dbfs,
'                             const char     *sourcePath,
'                             const char     *destPath );


Declare Function wd101Copy Lib "dbfsapi.dll" _
                                (ByVal dbfs As Long, _
                                 ByRef sourcePath As Byte, _
                                 ByRef destPath As Byte _
                                 ) As Byte


'/*!**********************************************************************
'  Function: wd101CopyWithLinks
'
'  description: Copy a file or directory tree. Files are not copied. Only
'               links to the original file are generated.
'
'arguments:
'    dbfs        [in]  DBFS, that is used.
'    sourcePath  [in]  The name of the source file or directory with
'                      absolute path.
'    destPath    [in]  The name of the new file or directory.
'
'  return value: true for success, false for failure. If the function fails
'                wd101CmdGetErrorId and wd101CmdGetErrorMsg can be used to
'                retrieve error information.
'
'  remarks: If source path is a directoy then destPath must also be a
'           directory. If sourcePath is a file then destPath may be an
'           existing directory or a non-existing filename.
'
'  Error -codes:
'  <UL>
'  <LI> ERR_FILE_OR_DIR_NOT_FOUND_WD26: sourcePath does not exist.
'  <LI> ERR_FILE_OR_DIR_EXISTS_WD26: destPath is an existing file.
'  <LI> ERR_DIR_NOT_FOUND_WD26: sourcePath is a file and destPath
'       specifies a non-existing file in a non-existing path or sourcePath
'       is a directory and the subdirectory of destPath does not exist.
'  </UL>
'*/
'tsp00_Bool WINAPI wd101CopyWithLinks( twd101DBFSP     dbfs,
'                                      const char     *sourcePath,
'                                      const char     *destPath );
Declare Function wd101CopyWithLinks Lib "dbfsapi.dll" _
                                (ByVal dbfs As Long, _
                                 ByRef sourcePath As Byte, _
                                 ByRef destPath As Byte _
                                 ) As Byte
'/*!**********************************************************************
'  Function: wd101CreateLink
'
'  description: Create a link in the DBFS. If a file or link with the given
'               name already exists the function fails.
'
'arguments:
'    dbfs       [in]  DBFS, that is used.
'    path       [in]  The name of the new link with absolute path.
'    targetPath [in]  The name of the target file or directory of the link.
'
'  return value: true for success, false for failure.
'                The functions wd101GetErrorId and
'                wd101GetErrorMsg can be used to get the reason
'                for a failure.
'*/
Declare Function wd101CreateLink Lib "dbfsapi.dll" _
                                (ByVal dbfs As Long, _
                                 ByRef sourcePath As Byte, _
                                 ByRef targetPath As Byte _
                                 ) As Byte




'tsp00_Bool WINAPI wd101LinkStat( twd101DBFSP        dbfs,
'                                 const char        *path,
'                                 struct st_db_stat *buf,
'                                 char              *targetBuf,
'                                 tsp00_Int4         bufLen )

Declare Function wd101LinkStat Lib "dbfsapi.dll" _
                                (ByVal dbfs As Long, _
                                 ByRef sourcePath As Byte, _
                                 ByRef stat As st_db_stat, _
                                 ByRef targetPath As Byte, _
                                 ByVal lBufLen As Long _
                                 ) As Byte


Declare Function convertVB_UCS2ToUTF8 Lib "StudioUtil.dll" _
        (srcBeg As Byte, _
        ByVal srcSize As Long, _
        destBeg As Byte, _
        ByVal destSize As Long, _
        destUsed As Long) As Byte


Declare Function convertUTF8ToVB_UCS2 Lib "StudioUtil.dll" _
        (srcBeg As Byte, _
        ByVal srcSize As Long, _
        destBeg As Byte, _
        ByVal destSize As Long, _
        destUsed As Long) As Byte
        

