VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Column"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' Class:      Column - SAPDB Connection Layer Column
'
'-----------------------------------------------------------------------
'
' responsible:  WolfgangA
'
' special area: SQL Studio DB Layer
'
' description:  column class for a sapdb result
'
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option Explicit

' Module name
Const MODULE_NAME = "Column"

' Privat property variables
Private m_sColName              As String
Private m_emODBCColType         As odbcColumnType
Private m_emSAPDBColType        As SAPDBColumnType
Private m_emSAPDBColCodeType    As SAPDBColumnCodeType
Private m_lColLength            As Long
Private m_lColDecimal           As Integer
Private m_bColNullable          As Boolean
Private m_vntColValue           As Variant
Private m_bColIsNull            As Boolean
Private m_bColIsKey             As Boolean
Private m_bColIsUpdatable       As Boolean

'if column is visualized, variable contains value of width
Private m_lColDisplayWidth             As Long
'if column is visualized, flag inidcates if column is displayed
Private m_bColDisplayVisible           As Boolean

Private m_sDecimalSeparator           As String
Private m_sThousandSeparator          As String
Private m_srepresentationOfNullValue    As String

'=========================================================================================
'    property:      colName
'    description:   set the name of the db column
'    arguments:     [in] newColName, new name of column
'
'    return value:  /
'
'=========================================================================================
Public Property Let colName(newColName As String)
    On Error GoTo errP_Let_ColName

    Const FUNCTION_NAME = "P_LET ColName"
    
    m_sColName = newColName
    
    Exit Property
    
errP_Let_ColName:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      colName
'    description:   Get the name of the db column
'    arguments:     /
'
'    return value:  name of column
'
'=========================================================================================
Public Property Get colName() As String
    On Error GoTo errP_Get_ColName

    Const FUNCTION_NAME = "P_Get colName"
    
    colName = m_sColName
    
    Exit Property
    
errP_Get_ColName:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      odbcColType
'    description:   set the columntype
'    arguments:     [in] NewODBCColType, type of column
'
'    return value:  /
'
'=========================================================================================
Public Property Let odbcColType(NewODBCColType As odbcColumnType)
    On Error GoTo errP_Let_ODBCColType

    Const FUNCTION_NAME = "P_Let odbcColType"

    m_emODBCColType = NewODBCColType

    Exit Property

errP_Let_ODBCColType:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      odbcColType
'    description:   Get the columntype
'    arguments:     /
'
'    return value:  type of column
'
'=========================================================================================
Public Property Get odbcColType() As odbcColumnType
    On Error GoTo errP_Get_ODBCColType

    Const FUNCTION_NAME = "P_GET ODBCColType"

    odbcColType = m_emODBCColType

    Exit Property

errP_Get_ODBCColType:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      colDisplayVisible
'    description:   let if column should be displayed
'    arguments:     [in] newColDisplayVisible, see description
'
'    return value:  /
'
'=========================================================================================
Public Property Let colDisplayVisible(newColDisplayVisible As Boolean)
    On Error GoTo errP_Let_ColDisplayVisible

    Const FUNCTION_NAME = "P_LET colDisplayVisible"
    
    m_bColDisplayVisible = newColDisplayVisible
    
    Exit Property
    
errP_Let_ColDisplayVisible:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      colDisplayVisible
'    description:   get if column should be displayed
'    arguments:     /
'
'    return value:  see description
'
'=========================================================================================
Public Property Get colDisplayVisible() As Boolean
    On Error GoTo errP_Get_ColDisplayVisible

    Const FUNCTION_NAME = "P_Get colDisplayVisible"
    
    colDisplayVisible = m_bColDisplayVisible
    
    Exit Property
    
errP_Get_ColDisplayVisible:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      colDisplayWidth
'    description:   let width of column if displayed
'    arguments:     [in] newColDisplayVisible, see description
'
'    return value:  /
'
'=========================================================================================
Public Property Let colDisplayWidth(newColDisplayWidth As Long)
    On Error GoTo errP_Let_colDisplayWidth

    Const FUNCTION_NAME = "P_LET colDisplayWidth"
    
    m_lColDisplayWidth = newColDisplayWidth
    
    Exit Property
    
errP_Let_colDisplayWidth:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      colDisplayWidth
'    description:   get width of column if displayed
'    arguments:     /
'
'    return value:  see description
'
'=========================================================================================
Public Property Get colDisplayWidth() As Long
    On Error GoTo errP_Get_colDisplayWidth

    Const FUNCTION_NAME = "P_Get colDisplayWidth"
    
    colDisplayWidth = m_lColDisplayWidth
    
    Exit Property
    
errP_Get_colDisplayWidth:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      colLength
'    description:   set (database) length of column
'    arguments:     [in] newColLength, set (database) length of column
'
'    return value:  /
'
'=========================================================================================
Public Property Let colLength(newColLength As Long)
    On Error GoTo errP_Let_ColLength

    Const FUNCTION_NAME = "P_LET ColLength"
    
    m_lColLength = newColLength
    
    Exit Property
    
errP_Let_ColLength:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      colLength
'    description:   get (database) length of column
'    arguments:     /
'
'    return value:  see description
'
'=========================================================================================
Public Property Get colLength() As Long
    On Error GoTo errP_Get_ColLength

    Const FUNCTION_NAME = "P_Get colLength"
    
    colLength = m_lColLength
    
    Exit Property
    
errP_Get_ColLength:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      colDecimal
'    description:   Let decimal property of column
'    arguments:     [in] newColDecimal, decimal property of column
'
'    return value:  /
'
'=========================================================================================
Public Property Let colDecimal(newColDecimal As Integer)
    On Error GoTo errP_Let_ColDecimal

    Const FUNCTION_NAME = "P_LET ColDecimal"
    
    m_lColDecimal = newColDecimal
    
    Exit Property
    
errP_Let_ColDecimal:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      colDecimal
'    description:   get decimal property of column
'    arguments:     /
'
'    return value:  see description
'
'=========================================================================================
Public Property Get colDecimal() As Integer
    On Error GoTo errP_Get_ColDecimal

    Const FUNCTION_NAME = "P_Get colDecimal"
    
    colDecimal = m_lColDecimal
    
    Exit Property
    
errP_Get_ColDecimal:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      colNullable
'    description:   set if column is nullable
'    arguments:     [in] newColNullable, new column is nullable
'
'    return value:  /
'
'=========================================================================================
Public Property Let colNullable(newColNullable As Integer)
    On Error GoTo errP_Let_ColNullable

    Const FUNCTION_NAME = "P_LET colNullable"
    
    m_bColNullable = newColNullable
    
    Exit Property
    
errP_Let_ColNullable:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      colNullable
'    description:   get if column is nullable
'    arguments:     /
'
'    return value:  see description
'
'=========================================================================================
Public Property Get colNullable() As Integer
    On Error GoTo errP_Get_ColNullable

    Const FUNCTION_NAME = "P_Get colNullable"
    
    colNullable = m_bColNullable
    
    Exit Property
    
errP_Get_ColNullable:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      colValue
'    description:   set the value of column
'    arguments:     [in] newColValue, new column value
'
'    return value:  /
'
'=========================================================================================
Public Property Let colValue(newColValue As Variant)
    On Error GoTo errP_Let_ColValue

    Const FUNCTION_NAME = "P_LET ColValue"
    
    m_vntColValue = newColValue
    
    Exit Property
    
errP_Let_ColValue:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      colValue
'    description:   Get the value of column
'    arguments:     /
'
'    return value:  see description
'
'=========================================================================================
Public Property Get colValue() As Variant
    On Error GoTo errP_Get_ColValue

    Const FUNCTION_NAME = "P_Get colValue"
    
    colValue = m_vntColValue
    
    Exit Property
    
errP_Get_ColValue:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      colIsNull
'    description:   let if column has null value
'    arguments:     [in] newColIsNull, new flag if column has null value
'
'    return value:  /
'
'=========================================================================================
Friend Property Let colIsNull(newColIsNull As Boolean)
    On Error GoTo errP_Let_ColIsNull

    Const FUNCTION_NAME = "P_LET colIsNull"
    
    m_bColIsNull = newColIsNull
    
    Exit Property
    
errP_Let_ColIsNull:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      colIsNull
'    description:   get if column has null value
'    arguments:     /
'
'    return value:  see description
'
'=========================================================================================
Public Property Get colIsNull() As Boolean
    On Error GoTo errP_Get_ColIsNull

    Const FUNCTION_NAME = "P_Get colIsNull"
    
    colIsNull = m_bColIsNull
    
    Exit Property
    
errP_Get_ColIsNull:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    function:      class_Initialize
'    description:   Occurs when an application creates an instance of a Form, MDIForm, User control, Property Page, Webclass, DHTML Page Designer, or class.
'
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Private Sub Class_Initialize()
    On Error GoTo ErrClass_Initialize

    Const FUNCTION_NAME = "Class_Initialize"
    
    m_sColName = ""
    m_emODBCColType = 0
    m_lColLength = 0
    m_lColDecimal = 0
    m_bColNullable = False
    m_bColIsNull = False
    
    m_lColDisplayWidth = 0
    m_bColDisplayVisible = True
    
    m_srepresentationOfNullValue = cnstQUERY_NULL
    
    Exit Sub
    
ErrClass_Initialize:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    property:      sapDBColCodeType
'    description:   [in] NewSAPDBColCodeType, set column code type
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Property Let sapDBColCodeType(NewSAPDBColCodeType As SAPDBColumnCodeType)
    On Error GoTo errP_Let_SAPDBColCodeType

    Const FUNCTION_NAME = "P_Let sAPDBColCodeType"

    m_emSAPDBColCodeType = NewSAPDBColCodeType

    Exit Property

errP_Let_SAPDBColCodeType:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      sapDBColCodeType
'    description:   Get column code type
'    arguments:     /
'
'    return value:  see description
'
'=========================================================================================
Public Property Get sapDBColCodeType() As SAPDBColumnCodeType
    On Error GoTo errP_Get_SAPDBColCodeType

    Const FUNCTION_NAME = "P_Get sapDBColCodeType"

    sapDBColCodeType = m_emSAPDBColCodeType

    Exit Property

errP_Get_SAPDBColCodeType:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      sapDBColCodeType
'    description:   [in] NewSAPDBColType, set column type
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Property Let sapDBColType(NewSAPDBColType As SAPDBColumnType)
    On Error GoTo errP_Let_SAPDBColType

    Const FUNCTION_NAME = "P_Let sAPDBColType"

    m_emSAPDBColType = NewSAPDBColType

    Exit Property

errP_Let_SAPDBColType:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      sapDBColType
'    description:   Get column type
'    arguments:     /
'
'    return value:  see description
'
'=========================================================================================
Public Property Get sapDBColType() As SAPDBColumnType
    On Error GoTo errP_Get_SAPDBColType

    Const FUNCTION_NAME = "P_Get sapDBColType"

    sapDBColType = m_emSAPDBColType

    Exit Property

errP_Get_SAPDBColType:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      colIsKey
'    description:   [in] newColIsKey, set if column is key
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Property Let colIsKey(newColIsKey As Boolean)
    On Error GoTo errP_Let_ColIsKey

    Const FUNCTION_NAME = "P_LET ColIsKey"
    
    m_bColIsKey = newColIsKey
    
    Exit Property
    
errP_Let_ColIsKey:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      colIsKey
'    description:   get if column is key
'    arguments:     /
'
'    return value:  see description
'
'=========================================================================================
Public Property Get colIsKey() As Boolean
    On Error GoTo errP_Get_ColIsKey

    Const FUNCTION_NAME = "P_Get colIsKey"
    
    colIsKey = m_bColIsKey
    
    Exit Property
    
errP_Get_ColIsKey:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      colIsUpdatable
'    description:   set if column is updatable
'    arguments:     [in] newColIsUpdatable, set if column is updatable
'
'    return value:  /
'
'=========================================================================================
Public Property Let colIsUpdatable(newColIsUpdatable As Boolean)
    On Error GoTo errP_Let_ColIsUpdatable

    Const FUNCTION_NAME = "P_LET ColIsUpdatable"
    
    m_bColIsUpdatable = newColIsUpdatable
    
    Exit Property
    
errP_Let_ColIsUpdatable:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      colIsUpdatable
'    description:   get if column is updatable
'    arguments:     /
'
'    return value:  see description
'
'=========================================================================================
Public Property Get colIsUpdatable() As Boolean
    On Error GoTo errP_Get_ColIsUpdatable

    Const FUNCTION_NAME = "P_Get colIsKey"
    
    colIsUpdatable = m_bColIsUpdatable
    
    Exit Property
    
errP_Get_ColIsUpdatable:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      sapDBColTypeAsString
'    description:   Get column type as string
'    arguments:     /
'
'    return value:  see description
'
'=========================================================================================
Public Property Get sapDBColTypeAsString() As String
    On Error GoTo errP_Get_SAPDBColTypeAsString

    Const FUNCTION_NAME = "P_Get sapDBColTypeAsString"
    
    sapDBColTypeAsString = "Unkown"
    
    Select Case m_emSAPDBColType
    
        Case sapdbColTypeUnknown
            sapDBColTypeAsString = "Unkown"
        Case sapdbColTypeChar
            sapDBColTypeAsString = "Char"
        Case sapdbColTypeVarChar
            sapDBColTypeAsString = "VarChar"
        Case sapdbColTypeLong
            sapDBColTypeAsString = "Long"
        Case sapdbColTypeBoolean
            sapDBColTypeAsString = "Boolean"
        Case sapdbColTypeFixed
            sapDBColTypeAsString = "Fixed"
        Case sapdbColTypeFloat
            sapDBColTypeAsString = "Float"
        Case sapdbColTypeInt
            sapDBColTypeAsString = "Int"
        Case sapdbColTypeSmallInt
            sapDBColTypeAsString = "SmallInt"
        Case sapdbColTypeDate
            sapDBColTypeAsString = "Date"
        Case sapdbColTypeTime
            sapDBColTypeAsString = "Time"
        Case sapdbColTypeTimeStamp
            sapDBColTypeAsString = "TimeStamp"
        Case sapdbColTypeLongFile
            sapDBColTypeAsString = "LongFile"
    End Select
    
    Exit Property
    
errP_Get_SAPDBColTypeAsString:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      representationOfNullValue
'    description:   Get the representation for a null value
'    arguments:     /
'
'    return value:  see description
'
'=========================================================================================
Public Property Get representationOfNullValue() As String
    On Error GoTo ErrGETrepresentationOfNullValue
    
    Const FUNCTION_NAME = "Get representationOfNullValue"
    
    representationOfNullValue = m_srepresentationOfNullValue
    
    Exit Property
    
ErrGETrepresentationOfNullValue:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      representationOfNullValue
'    description:   set the representation for a null value
'    arguments:     [in] newRepresentationOfNullValue, new representation for a null value
'
'    return value:  /
'
'=========================================================================================
Public Property Let representationOfNullValue(newRepresentationOfNullValue As String)
    On Error GoTo errLETrepresentationOfNullValue
    
    Const FUNCTION_NAME = "Let representationOfNullValue"
    
    m_srepresentationOfNullValue = newRepresentationOfNullValue
    
    Exit Property
    
errLETrepresentationOfNullValue:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    function:      getFormattedValue
'    description:   Get the formated value of column depending its column type
'    arguments:     [in] Value, value to format
'
'    return value:  formated value of column depending its column type
'
'=========================================================================================
Public Function getFormattedValue(ByVal Value As String) As String
    On Error GoTo ErrGetFormattedValue

    Const FUNCTION_NAME = "GetFormattedValue"
    
    'inits
    getFormattedValue = ""
    
    If Value = m_srepresentationOfNullValue Or Len(Value) < 1 Then
        'format value
        Select Case m_emSAPDBColType
            Case sapdbColTypeChar, sapdbColTypeVarChar:
                
                If Len(Value) < 1 Then
                
                    getFormattedValue = "''"
                Else
                    getFormattedValue = getCodeTypePrefix + "'" + convertStringForCharCol(Value, True) + "'"
                End If
            Case Else
                getFormattedValue = "NULL"
        End Select
    Else
        'format value
        Select Case m_emSAPDBColType
            Case sapdbColTypeChar, sapdbColTypeVarChar:
                getFormattedValue = getCodeTypePrefix + "'" + convertStringForCharCol(Value, True) + "'"
            Case sapdbColTypeDate:
                getFormattedValue = "'" + getISODate(Value) + "'"
            Case sapdbColTypeTime:
                getFormattedValue = "'" + getISOTime(Value) + "'"
            Case sapdbColTypeTimeStamp:
                getFormattedValue = "'" + getISOTimeStamp(Value) + "'"
            Case sapdbColTypeFixed, _
                 sapdbColTypeFloat, _
                 sapdbColTypeSmallInt, _
                 sapdbColTypeInt:
                 getFormattedValue = getISONumber(Value)
            Case sapdbColTypeLong, sapdbColTypeLongFile:
                getFormattedValue = "'" + Value + "'"
            Case Else
                getFormattedValue = Value
        End Select
    End If
    
    Exit Function

ErrGetFormattedValue:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getCodeTypePrefix
'    description:   Get the code prefix of a column
'    arguments:     /
'
'    return value:  see description
'
'=========================================================================================
Public Function getCodeTypePrefix() As String
    On Error GoTo ErrGetCodeTypePrefix

    Const FUNCTION_NAME = "GetCodeTypePrefix"

    'inits
    getCodeTypePrefix = ""

    If m_emSAPDBColCodeType = sapdbColCodeTypeBYTE _
       Then getCodeTypePrefix = "X"
    
    Exit Function

ErrGetCodeTypePrefix:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getISODate
'    description:   Get the current date as string
'    arguments:     [in] CurrentDate, current date to be formated
'
'    return value:  see description
'
'=========================================================================================
Public Function getISODate(ByVal CurrentDate As String) As String
    On Error GoTo ErrGetISODate

    Const FUNCTION_NAME = "GetISODate"

    'format date
    If CurrentDate <> "" Then
        If IsDate(CurrentDate) Then
            getISODate = Format$(CVDate(CurrentDate), "yyyy-mm-dd")
        Else
            getISODate = CurrentDate
        End If
    Else
        getISODate = CurrentDate
    End If

    Exit Function

ErrGetISODate:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getISONumber
'    description:   Get the current number as string
'    arguments:     [in] CurrentNumber, current number to be formated
'
'    return value:  see description
'
'=========================================================================================
Public Function getISONumber(ByVal CurrentNumber As String) As String
    On Error GoTo ErrGetISONumber

    Const FUNCTION_NAME = "GetISONumber"

    Dim NumberStr       As String
    Dim NumberLen       As Integer
    Dim ReplacePos      As Integer
    Dim i               As Integer

    ' Inits
    NumberLen = Len(CurrentNumber)
    ReplacePos = 0
    NumberStr = ""

    'Get the replace position for the separator
    ReplacePos = InStr(CurrentNumber, m_sDecimalSeparator)
    If ReplacePos <> 0 Then
        ' Replace the separator
        NumberStr = left$(CurrentNumber, ReplacePos - 1) + "." + right$(CurrentNumber, NumberLen - ReplacePos)
    Else
        NumberStr = CurrentNumber
    End If

    ' Search the position to begin setting of thousands
    If ReplacePos = 0 Then
        ReplacePos = NumberLen
    Else
        ReplacePos = ReplacePos - 1
    End If

    ' Remove the thousands separator
    For i = ReplacePos - 3 To 1 Step -3
        If Mid$(NumberStr, i, 1) = m_sThousandSeparator Then
            NumberStr = left$(NumberStr, i - 1) + right$(NumberStr, NumberLen - i)
            NumberLen = NumberLen - 1
            i = i - 1
        Else
            Exit For
        End If
    Next

    getISONumber = NumberStr

    Exit Function

ErrGetISONumber:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getISOTime
'    description:   Get the current time as string
'    arguments:     [in] CurrentTime, current time to be formated
'
'    return value:  see description
'
'=========================================================================================
Public Function getISOTime(ByVal CurrentTime As String) As String
    On Error GoTo ErrGetISOTime

    Const FUNCTION_NAME = "GetISOTime"

    'format date
    getISOTime = Format$(CurrentTime, "hh:mm:ss")

    Exit Function

ErrGetISOTime:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getISOTimeStamp
'    description:   Get the current time stamp as string
'    arguments:     [in] CurrentTimeStamp, current time stamp to be formated
'
'    return value:  see description
'
'=========================================================================================
Public Function getISOTimeStamp(ByVal CurrentTimeStamp As String) As String
    On Error GoTo ErrGetISOTimeStamp

    Const FUNCTION_NAME = "GetISOTimeStamp"

    'format date
    getISOTimeStamp = Format$(CurrentTimeStamp, "yyyy-mm-dd hh:mm:ss")

    Exit Function

ErrGetISOTimeStamp:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      convertStringForCharCol
'    description:   convert a string to be used with a character column
'    arguments:     [in] StringToConvert, data to be converted
'                   [in] bConvertWholeString, flag if whole string has to be converted
'
'    return value:  see description
'
'=========================================================================================
Private Function convertStringForCharCol(ByVal StringToConvert As String, _
                                         ByVal bConvertWholeString As Boolean) As String
    On Error GoTo errConvertStringForCharCol

    Const FUNCTION_NAME = "convertStringForCharCol"
    
    '=local variables================
    Dim i           As Integer
    Dim ch          As String
    Dim StrLen      As Integer
    Dim newCharCol  As String
    '================================

    convertStringForCharCol = ""

    If Len(StringToConvert) = 0 Or InStr(StringToConvert, "'") = 0 Then '= 0
    
        convertStringForCharCol = StringToConvert
        Exit Function
    End If

    StrLen = Len(StringToConvert)
    For i = 1 To StrLen
        ch = Mid$(StringToConvert, i, 1)
        If (ch = "'" And i > 1 And i < StrLen) Or (ch = "'" And bConvertWholeString) Then
            newCharCol = newCharCol + "''"
        ElseIf ch <> "'" Then
            newCharCol = newCharCol + ch
        End If
    Next
    
    convertStringForCharCol = newCharCol

    Exit Function

errConvertStringForCharCol:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    property:      thousandSeparator
'    description:   set the thousand operator
'    arguments:     [in] NewTSeperator, new thousand operator
'
'    return value:  /
'
'=========================================================================================
Public Property Let thousandSeparator(NewTSeperator As String)
    On Error GoTo errP_Let_ThousandSeparator

    Const FUNCTION_NAME = "P_Let thousandSeparator"
    
    m_sThousandSeparator = NewTSeperator
    
    Exit Property
    
errP_Let_ThousandSeparator:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      decimalSeparator
'    description:   set the decimal operator
'    arguments:     [in] NewDecimalSeperator, new decimal operator
'
'    return value:  /
'
'=========================================================================================
Public Property Let decimalSeparator(NewDecimalSeparator As String)
    On Error GoTo errP_Let_DecimalSeparator

    Const FUNCTION_NAME = "P_Let decimalSeparator"
    
    m_sDecimalSeparator = NewDecimalSeparator
    
    Exit Property
    
errP_Let_DecimalSeparator:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    function:      setSapDBColumnType
'    description:   set the column type
'    arguments:     [in] SAPDBColumnType, new column type
'
'    return value:  /
'
'=========================================================================================
Public Function setSapDBColumnType(ByVal SAPDBColumnType As String)
    On Error GoTo errSetSapDBColumnType

    Const FUNCTION_NAME = "setSapDBColumnType"
    
    ' Select column type
    Select Case UCase(SAPDBColumnType)
        Case "CHAR":
            m_emSAPDBColType = sapdbColTypeChar
        Case "VARCHAR":
            m_emSAPDBColType = sapdbColTypeVarChar
        Case "LONG":
            m_emSAPDBColType = sapdbColTypeLong
        Case "BOOLEAN":
            m_emSAPDBColType = sapdbColTypeBoolean
        Case "FIXED":
            m_emSAPDBColType = sapdbColTypeFixed
        Case "FLOAT", "DECIMAL", "REAL", "NUMERIC":
            m_emSAPDBColType = sapdbColTypeFloat
        Case "INT", "INTEGER":
            m_emSAPDBColType = sapdbColTypeInt
        Case "SMALLINT", "SMALLINTEGER":
            m_emSAPDBColType = sapdbColTypeSmallInt
        Case "DATE":
            m_emSAPDBColType = sapdbColTypeDate
        Case "TIME":
            m_emSAPDBColType = sapdbColTypeTime
        Case "TIMESTAMP":
            m_emSAPDBColType = sapdbColTypeTimeStamp
        Case "LONGFILE":
            m_emSAPDBColType = sapdbColTypeLongFile
        Case Else
            m_emSAPDBColType = sapdbColTypeUnknown
    End Select
    
    Exit Function
    
errSetSapDBColumnType:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      setSapDBColumnCodeType
'    description:   set the column code type
'    arguments:     [in] SAPDBColumnCodeType, new column code type
'
'    return value:  /
'
'=========================================================================================
Public Function setSapDBColumnCodeType(ByVal SAPDBColumnCodeType As String)
    On Error GoTo errSetSapDBColumnCodeType

    Const FUNCTION_NAME = "SetSapDBColumnCodeType"
    
    ' Select column code type
    Select Case SAPDBColumnCodeType
        Case "ASCII":
            m_emSAPDBColCodeType = sapdbColCodeTypeASCII
        Case "BYTE":
            m_emSAPDBColCodeType = sapdbColCodeTypeBYTE
        Case "EBCDIC":
            m_emSAPDBColCodeType = sapdbColCodeTypeEBCDIC
        Case "UNICODE":
            m_emSAPDBColCodeType = sapdbColCodeTypeUNICODE
        Case Else
            m_emSAPDBColCodeType = sapdbColCodeTypeUnknown
    End Select
    
    Exit Function
    
errSetSapDBColumnCodeType:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getSapDBColumnCodeTypeAsString
'    description:   set the column code type
'    arguments:     /
'
'    return value:  column code type as string
'
'=========================================================================================
Public Function getSapDBColumnCodeTypeASString() As String
    On Error GoTo errGetSapDBColumnCodeType

    Const FUNCTION_NAME = "getSapDBColumnCodeTypeASString"
    
    ' Select column code type
    Select Case m_emSAPDBColCodeType
        Case sapdbColCodeTypeASCII:
            getSapDBColumnCodeTypeASString = "Ascii"
        Case sapdbColCodeTypeBYTE:
            getSapDBColumnCodeTypeASString = "Byte"
        Case sapdbColCodeTypeEBCDIC:
            getSapDBColumnCodeTypeASString = "Ebcdic"
        Case sapdbColCodeTypeUNICODE:
            getSapDBColumnCodeTypeASString = "Unicode"
        Case Else
            getSapDBColumnCodeTypeASString = ""
    End Select
    
    Exit Function
    
errGetSapDBColumnCodeType:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
Public Function initODBCColTypeFromSAPDBTypes() As Boolean
    On Error GoTo errInitODBCColTypeFromSAPDBTypes

    Const FUNCTION_NAME = "initODBCColTypeFromSAPDBTypes"
    
    initODBCColTypeFromSAPDBTypes = False
    
    Select Case m_emSAPDBColType
    
        Case sapdbColTypeUnknown
            Exit Function
        Case sapdbColTypeChar
        
            'select column code type
            Select Case m_emSAPDBColCodeType
                Case sapdbColCodeTypeASCII:
                    m_emODBCColType = odbcColTypeChar
                Case sapdbColCodeTypeBYTE:
                    m_emODBCColType = odbcColTypeBinary
                Case sapdbColCodeTypeEBCDIC:
                    m_emODBCColType = odbcColTypeBinary
                Case sapdbColCodeTypeUNICODE:
                    m_emODBCColType = odbcColTypeWChar
                Case Else
                    Exit Function
            End Select
            
        Case sapdbColTypeVarChar
            'select column code type
            Select Case m_emSAPDBColCodeType
                Case sapdbColCodeTypeASCII:
                    m_emODBCColType = odbcColTypeVarChar
                Case sapdbColCodeTypeBYTE:
                    m_emODBCColType = odbcColTypeVarBinary
                Case sapdbColCodeTypeEBCDIC:
                    m_emODBCColType = odbcColTypeVarBinary
                Case sapdbColCodeTypeUNICODE:
                    m_emODBCColType = odbcColTypeWVarChar
                Case Else
                    Exit Function
            End Select
            
        Case sapdbColTypeLong
        
            'select column code type
            Select Case m_emSAPDBColCodeType
                Case sapdbColCodeTypeASCII:
                    m_emODBCColType = odbcColTypeLongVarChar
                Case sapdbColCodeTypeBYTE:
                    m_emODBCColType = odbcColTypeLongVarBinary
                Case sapdbColCodeTypeEBCDIC:
                    m_emODBCColType = odbcColTypeLongVarBinary
                Case sapdbColCodeTypeUNICODE:
                    m_emODBCColType = odbcColTypeWLongVarChar
                Case Else
                    Exit Function
            End Select
            
        Case sapdbColTypeBoolean
            m_emODBCColType = odbcColTypeBit
        Case sapdbColTypeFixed
            m_emODBCColType = odbcColTypeDecimal
        Case sapdbColTypeFloat
            m_emODBCColType = odbcColTypeFloat
        Case sapdbColTypeInt
            m_emODBCColType = odbcColTypeInteger
        Case sapdbColTypeSmallInt
            m_emODBCColType = odbcColTypeSmallInt
        Case sapdbColTypeDate
            m_emODBCColType = odbcColTypeDate
        Case sapdbColTypeTime
            m_emODBCColType = odbcColTypeTime
        Case sapdbColTypeTimeStamp
            m_emODBCColType = odbcColTypeTimeStamp
        Case sapdbColTypeLongFile
            m_emODBCColType = odbcColTypeLongVarBinary
    End Select
    
    initODBCColTypeFromSAPDBTypes = True
    
    Exit Function
    
errInitODBCColTypeFromSAPDBTypes:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function

