Attribute VB_Name = "Common"
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' Module:      Common
'
'
'-----------------------------------------------------------------------
'
' responsible:  WolfgangA
'
' special area: SQL Studio DB Layer
'
' description:  contains some functions
'
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''


Option Explicit

'--------------------------------------------------------
'Module Name
'--------------------------------------------------------
Const MODULE_NAME = "Common"


'=public global objects========================================================================================
Public m_oComFunc           As New SQLStudioUtil.Functions
Public m_oIntSettings       As New SQLStudioUtil.International
'=========================================================================================

'=========================================================================================
'    function:      internalError
'    description:   shows message box with error
'    arguments:     [in] CurrentError, vb error object
'                   [in] Modulname, name of source modul
'                   [in] FunctionName, name of used function
'    return value:  /
'
'=========================================================================================
Public Function internalError(ByVal CurrentError As ErrObject, ModulName As String, FunctionName As String)
    MsgBox "Module:" + Chr$(9) + ModulName + Chr$(13) + Chr$(10) + "Function:" + Chr$(9) + FunctionName + Chr$(13) + Chr$(10) + "error No:" + Chr$(9) + Trim$(Str$(CurrentError.Number)) + Chr$(13) + Chr$(10) + "error:" + Chr$(9) + CurrentError.Description, vbExclamation, "Unexpected Error:"
End Function

'=========================================================================================
'    function       commonFreeStmt
'    description:   free statement handle
'    arguments:     [in] hStmt, statement handle
'
'    return value:  true if success
'
'=========================================================================================
Public Function commonFreeStmt(hStmt As Long)
    On Error GoTo errCommonFreeStmt

    Const FUNCTION_NAME = "commonFreeStmt"

    Dim lRc  As Integer

    'free statement handle
    If hStmt <> SQL_NULL_HSTMT Then
    
        lRc = SQLFreeStmt(hStmt, SQL_DROP)
        hStmt = SQL_NULL_HSTMT
    End If

    Exit Function

errCommonFreeStmt:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function

'=========================================================================================
'    function:      getUID
'    description:   gets the user ID out of a odbc connection string
'    arguments:     [in] ConnectString
'                   [out] sUID
'
'    return value:  /
'
'=========================================================================================
Public Function getUID(ByVal ConnectString As String, ByRef sUID As String)
    On Error GoTo errGetUID

    Const FUNCTION_NAME = "getUID"
    
    '=varibles============================================================================
    Dim BeginPos    As Integer
    Dim EndPos      As Integer
    '=====================================================================================

    'get first position
    BeginPos = InStr(ConnectString, ";UID=")
    If BeginPos > 0 Then
    
        If Len(ConnectString) > BeginPos + 5 Then
        
            EndPos = InStr(BeginPos, ConnectString, ";PWD=")
            If EndPos > 0 Then
            
                sUID = Mid$(ConnectString, BeginPos + 5, EndPos - BeginPos - 5)
                
                If left$(sUID, 1) = """" And right$(sUID, 1) = """" And Len(sUID) > 2 Then
                    sUID = Mid$(sUID, 2, Len(sUID) - 2)
                End If
            End If
        End If
    Else
        'in a new version  odbc changed output of connect string
        'see PTS error 1108230
        BeginPos = InStr(ConnectString, "UID=")
        If Len(ConnectString) > BeginPos + 4 Then
        
            EndPos = InStr(BeginPos, ConnectString, ";PWD=")
            If EndPos > 0 Then
            
                sUID = Mid$(ConnectString, BeginPos + 4, EndPos - BeginPos - 4)
                
                If left$(sUID, 1) = """" And right$(sUID, 1) = """" And Len(sUID) > 2 Then
                    sUID = Mid$(sUID, 2, Len(sUID) - 2)
                End If
            End If
        End If
    End If
    
    Exit Function

errGetUID:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getPWD
'    description:   gets the password out of a odbc connection string
'    arguments:     [in] ConnectString
'                   [out] sPWD
'
'    return value:  /
'
'=========================================================================================
Public Function getPWD(ByVal ConnectString As String, ByRef sPWD As String)
    On Error GoTo errGetPWD

    Const FUNCTION_NAME = "getPWD"
    
    '=varibles============================================================================
    Dim BeginPos    As Integer
    Dim EndPos      As Integer
    '======================================================================================

    'get first position
    BeginPos = InStr(ConnectString, ";PWD=")
    If BeginPos > 0 Then
        If Len(ConnectString) > BeginPos + 5 Then
            EndPos = InStr(BeginPos, ConnectString, ";SERVERDB=")
            If EndPos > 0 Then
                sPWD = Mid$(ConnectString, BeginPos + 5, EndPos - BeginPos - 5)
                
                If left$(sPWD, 1) = """" And right$(sPWD, 1) = """" And Len(sPWD) > 2 Then
                    sPWD = Mid$(sPWD, 2, Len(sPWD) - 2)
                End If
            End If
        End If
    End If
    
    Exit Function

errGetPWD:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getServerDB
'    description:   gets the server DB out of a odbc connection string
'    arguments:     [in] ConnectString
'                   [out] sServerDB
'
'    return value:  /
'
'=========================================================================================
Public Function getServerDB(ByVal ConnectString As String, ByRef sServerDB As String)
    On Error GoTo errGetServerDB

    Const FUNCTION_NAME = "getServerDB"
    
    '=varibles============================================================================
    Dim BeginPos    As Integer
    Dim EndPos      As Integer
    '======================================================================================

    'get first position
    BeginPos = InStr(ConnectString, ";SERVERDB=")
    If BeginPos > 0 Then
        If Len(ConnectString) > BeginPos + 10 Then
            EndPos = InStr(BeginPos, ConnectString, ";SERVERNODE=")
            If EndPos > 0 Then
            
                sServerDB = Mid$(ConnectString, BeginPos + 10, EndPos - BeginPos - 10)
            End If
        End If
    End If
    
    Exit Function

errGetServerDB:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getServerNode
'    description:   gets the server Node out of a odbc connection string
'    arguments:     [in] ConnectString
'                   [out] sServerNode
'
'    return value:  /
'
'=========================================================================================
Public Function getServerNode(ByVal ConnectString As String, ByRef sServerNode As String)
    On Error GoTo errGetServerNode

    Const FUNCTION_NAME = "getServerNode"
    
    '=varibles============================================================================
    Dim BeginPos    As Integer
    Dim EndPos      As Integer
    '======================================================================================

    'get first position
    BeginPos = InStr(ConnectString, ";SERVERNODE=")
    If BeginPos > 0 Then
    
        If Len(ConnectString) > BeginPos + 12 Then
        
            EndPos = InStr(BeginPos + 1, ConnectString, ";")
            If EndPos > 0 Then
            
                sServerNode = Mid$(ConnectString, BeginPos + 12, EndPos - BeginPos - 12)
            End If
        End If
    End If
    
    Exit Function

errGetServerNode:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function

