VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Connection"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Description = "Represents a  SAPDB-Connection"
Attribute VB_Ext_KEY = "SavedWithClassBuilder" ,"Yes"
Attribute VB_Ext_KEY = "Member0" ,"Connection"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' Class:      Connection
'
'
'-----------------------------------------------------------------------
'
' responsible:  WolfgangA
'
' special area: SQL Studio DB Layer
'
' description:  class for representation one connection
'
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option Explicit

'Module name
Const MODULE_NAME = "Connection"


'=varibles============================================================================


Private m_emIsoLevel          As isoLevel
Private m_emSQLMode           As sqlMode
Private m_emAutoCommit        As autoCommit

Private m_sUserGroup          As String

'object for error handling
Private m_oError              As New error

'flag if connection running in ni mode
Private m_bNI                 As Boolean

Private m_bUnicodeDatabase    As Boolean

'========================================================================
Private m_lWaDBCHandle         As Long
'handle for WA Connection pool
Private m_lPool                As Long

'========================================================================

'public definitions
Public Enum isoLevel

    UnCommitted = SQL_TXN_READ_UNCOMMITTED
    Committed = SQL_TXN_READ_COMMITTED
    Repeatable = SQL_TXN_REPEATABLE_READ
    Serializable = SQL_TXN_SERIALIZABLE
    Versionizing = SQL_TXN_VERSIONING
   
End Enum

Public Enum sqlMode

        Internal = CPR_KIND_ADABAS
        DB2 = CPR_KIND_DB2
        Ansi = CPR_KIND_ANSI
        Oracle = CPR_KIND_ORACLE
        SAPR3 = CPR_KIND_SAPR3
End Enum

Public Enum UserMode

    umUnkown
    umSTANDARD
    umRESOURCE
    umDBA
    umSYSDBA
    umCOLDUSER
End Enum

Public Enum autoCommit

    AutoCommitOff = SQL_AUTOCOMMIT_OFF
    AutoCommitOn = SQL_AUTOCOMMIT_ON
End Enum

'=========================================================================================
'    property:      error
'    description:   return pointer to error object of connection
'    arguments:     /
'
'    return value:  pointer to error object of connection
'
'=========================================================================================
Public Property Get error() As error
    On Error GoTo errP_Get_error

    Const FUNCTION_NAME = "P_GET error"
    
    Set error = m_oError
    
    Exit Property
    
errP_Get_error:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      isoLevel
'    description:   get iso level of the connection object
'    arguments:     /
'
'    return value:  isoLevel as em
'
'=========================================================================================
Public Property Get isoLevel() As isoLevel
    On Error GoTo errP_Get_isoLevel

    Const FUNCTION_NAME = "P_Get isoLevel"
    
    isoLevel = m_emIsoLevel
    
    Exit Property
    
errP_Get_isoLevel:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      isoLevel
'    description:   set iso level of the connection object
'    arguments:     [in] newIsoLevel as em
'
'    return value:  /
'
'=========================================================================================
Friend Property Let isoLevel(newIsoLevel As isoLevel)
    On Error GoTo errP_Let_isoLevel

    Const FUNCTION_NAME = "P_Let isoLevel"
    
    m_emIsoLevel = newIsoLevel
    
    Exit Property
    
errP_Let_isoLevel:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      sqlMode
'    description:   get sql mode of the connection object
'    arguments:     /
'
'    return value:  sql mode of the connection object as em
'
'=========================================================================================
Public Property Get sqlMode() As sqlMode
    On Error GoTo errP_Get_SQLMode

    Const FUNCTION_NAME = "P_Get sqlMode"
    
    sqlMode = m_emSQLMode
    
    Exit Property
    
errP_Get_SQLMode:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      sqlMode
'    description:   let sql mode of the connection object
'    arguments:     [in] newSQLMode, sql mode of the connection object as em
'
'    return value:
'
'=========================================================================================
Friend Property Let sqlMode(NewSQLMode As sqlMode)
    On Error GoTo errP_Let_SQLMode

    Const FUNCTION_NAME = "P_Let sqlMode"
    
    m_emSQLMode = NewSQLMode
    
    Exit Property
    
errP_Let_SQLMode:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      autoCommit
'    description:   get auto commit of the connection object
'    arguments:     /
'
'    return value:  auto commit of the connection object as em
'
'=========================================================================================
Public Property Get autoCommit() As autoCommit
    On Error GoTo errP_Get_autoCommit
    
    Const FUNCTION_NAME = "P_GET autoCommit"
    
    autoCommit = m_emAutoCommit
    
    Exit Property
    
errP_Get_autoCommit:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      autoCommit
'    description:   get auto commit of the connection object
'    arguments:     [in] newAutoCommit, auto commit of the connection object as em
'
'    return value:  /
'
'=========================================================================================
Friend Property Let autoCommit(newAutoCommit As autoCommit)
    On Error GoTo errP_Get_autoCommit
    
    Const FUNCTION_NAME = "P_GET autoCommit"
    
    m_emAutoCommit = newAutoCommit
    
    Exit Property
    
errP_Get_autoCommit:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    function:      rollBack
'    description:   roll back all actions on a connection
'
'    arguments:     /
'
'    return value:  true if success
'
'=========================================================================================
Private Function rollBack() As Boolean
    On Error GoTo errRollBack

    Const FUNCTION_NAME = "rollBack"
    
    '=local varibles===================
    Dim lRc            As Integer
    Dim hStmt          As Long
    Dim bStatement()   As Byte
    Dim lStringLength  As Long
    '==================================
        
    'rc
    rollBack = False
    
    'init
    lStringLength = 0
    hStmt = SQL_NULL_HSTMT
    
    'allocate storage for the statement
    lRc = SQLAllocStmt(hDbc, hStmt)
    If lRc <> SQL_SUCCESS Then
    
        If Not m_oError.checkRC(lRc, hStmt) Then Exit Function
    End If
    
    'lRc = SQLSetStmtOption(hStmt, SQL_SQLMODE, Internal)
    lRc = SQLSetStmtAttrLong(hStmt, SQL_SQLMODE, Internal, lStringLength)
    If lRc <> SQL_SUCCESS Then
        If Not m_oError.checkRC(lRc, hStmt) Then
            
            If hStmt <> SQL_NULL_HSTMT Then
            
                SQLFreeStmt hStmt, SQL_DROP 'free statement handle
                hStmt = SQL_NULL_HSTMT
            End If
            
            Exit Function
        End If
    End If

    m_oError.statement = "RollBack"
    'execute command
    
    bStatement = "RollBack" + vbNullChar
    'lRc = SQLExecDirectW(hStmt, bStatement(0), SQL_NTS)
    'Unicode functions that always return or take strings or
    'length arguments are passed as count-of-characters
    lRc = SQLExecDirectW(hStmt, bStatement(0), 8)
    If lRc <> SQL_SUCCESS Then
        If Not m_oError.checkRC(lRc, hStmt) Then
        
            If m_oError.rc <> SQL_NEED_DATA Then
                
                If hStmt <> SQL_NULL_HSTMT Then
                
                    SQLFreeStmt hStmt, SQL_DROP 'free statement handle
                    hStmt = SQL_NULL_HSTMT
                End If
                Exit Function
            End If
        End If
    End If

    'free statement handle
    If hStmt <> SQL_NULL_HSTMT Then
    
        SQLFreeStmt hStmt, SQL_DROP
        hStmt = SQL_NULL_HSTMT
    End If

    'Set rc
    rollBack = True

    Exit Function

errRollBack:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      initConnectProperties
'    description:   initialize some members concerning the connect parameters
'
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Private Function initConnectProperties()
    On Error GoTo errInitConnectProperties

    Const FUNCTION_NAME = "initConnectProperties"
    
    m_emIsoLevel = Committed
    m_emSQLMode = Internal
    m_emAutoCommit = AutoCommitOn
     
    Exit Function

errInitConnectProperties:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      disconnect
'    description:   decrement the reference count of the connection object
'                   if pooling mode is using webagent pool
'                   call sapdbwa_DBCDisconnect to decrement ref counter
'
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function disconnect()
    On Error GoTo ErrDisconnect

    Const FUNCTION_NAME = "disconnect"
    
    '=variables===========================================================
    Dim lError            As Long
    Dim sByteBuffer(2048) As Byte
    '=====================================================================
    
    If m_lWaDBCHandle > 0 Then
    
        If m_emAutoCommit = AutoCommitOff Then rollBack
            
        If sapdbwa_DBCDisconnect(m_lPool, m_lWaDBCHandle) = False Then

            lError = sapdbwa_DBCGetErr(m_lWaDBCHandle)
            If sapdbwa_GetErrId(lError) <> WA_ERR_NO_ERROR Then

                sapdbwa_CopyErrMsg lError, sByteBuffer(0), UBound(sByteBuffer())
                MsgBox m_oComFunc.getStringFromUtf8(sByteBuffer()), vbCritical
            End If
            sapdbwa_DestroyDBC m_lWaDBCHandle
            m_lWaDBCHandle = 0
        Else
            lError = sapdbwa_DBCGetErr(m_lWaDBCHandle)
            If sapdbwa_GetErrId(lError) <> WA_ERR_NO_ERROR Then

                sapdbwa_CopyErrMsg lError, sByteBuffer(0), UBound(sByteBuffer())
                MsgBox m_oComFunc.getStringFromUtf8(sByteBuffer()), vbCritical
                m_lWaDBCHandle = 0
            Else
                sapdbwa_DestroyDBC m_lWaDBCHandle
                m_lWaDBCHandle = 0
            End If
        End If
    End If
                
    Exit Function

ErrDisconnect:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      showLastDBError
'    description:   show dialog with last db error
'
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function showLastDBError()
    On Error GoTo errShowLastDBError
    
    Const FUNCTION_NAME = "showLastDBError"
    
    If m_oError.lastIsError Then
    
        Set frmShowDBError.connection = Me
        frmShowDBError.Show vbModal
    End If
    
    Exit Function
    
errShowLastDBError:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
Private Sub Class_Terminate()
On Error GoTo ErrClass_Terminate

    Const FUNCTION_NAME = "Class_Terminate"
    
    disconnect
    
    Exit Sub
    
ErrClass_Terminate:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      class_Initialize
'    description:   Occurs when an application creates an instance of a Form, MDIForm, User control, Property Page, Webclass, DHTML Page Designer, or class.
'
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Private Sub Class_Initialize()
    On Error GoTo ErrClass_Initialize

    Const FUNCTION_NAME = "Class_Initialize"
    
    m_emIsoLevel = Committed
    m_emSQLMode = Internal
    m_emAutoCommit = AutoCommitOn
    
    m_lPool = 0
    m_lWaDBCHandle = 0
        
    m_bUnicodeDatabase = False
    
    Exit Sub
    
ErrClass_Initialize:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Sub


'=========================================================================================
'    property:      hDbc
'    description:   get database handle of connection
'    arguments:     /
'
'    return value:  dbc handle as long
'
'=========================================================================================
Public Property Get hDbc() As Long
    On Error GoTo errP_Get_hDBC

    Const FUNCTION_NAME = "P_GET hDBC"
        
    If m_lWaDBCHandle > 0 Then
        hDbc = sapdbwa_DBCHandle(m_lWaDBCHandle)
    Else
        hDbc = 0
    End If
        
    Exit Property
    
errP_Get_hDBC:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      hEnv
'    description:   get enviroment handle of connection object
'    arguments:     /
'
'    return value:  env handle as long
'
'=========================================================================================
Public Property Get hEnv() As Long
    On Error GoTo errP_Get_hEnv

    Const FUNCTION_NAME = "P_GET hEnv"
    
    If m_lWaDBCHandle > 0 Then
        hEnv = sapdbwa_DBCEnv(m_lWaDBCHandle)
    Else
        hEnv = 0
    End If
    
    Exit Property
    
errP_Get_hEnv:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property

'=========================================================================================
'    property:      waDBC
'    description:   handle of database class in webagent pool
'    arguments:     [in] newWADBC, handle
'
'    return value:  /
'
'=========================================================================================
Public Property Let waDBC(newWADBC As Long)
    On Error GoTo errP_Let_WADBC

    Const FUNCTION_NAME = "P_LET waDBC"
    
    m_lWaDBCHandle = newWADBC
    
    Exit Property
    
errP_Let_WADBC:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      waDBCPool
'    description:   let pool handle of webagent pool
'    arguments:     [in] newPool handle as long
'
'    return value:  /
'
'=========================================================================================
Public Property Let waDBCPool(newPool As Long)
    On Error GoTo errP_Let_WADBCPool

    Const FUNCTION_NAME = "P_LET waDBCPool"
    
    m_lPool = newPool
    
    Exit Property
    
errP_Let_WADBCPool:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      UserGroup
'    description:   set a new usergroup
'    arguments:     [in] sNewUserGroup, new user group
'
'    return value:  /
'
'=========================================================================================
Friend Property Let userGroup(sNewUserGroup As String)
    On Error GoTo errP_Let_UserGroup

    Const FUNCTION_NAME = "P_LET UserGroup"
    
    m_sUserGroup = sNewUserGroup
    
    Exit Property
    
errP_Let_UserGroup:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      userGroup
'    description:   return usergroup of connected user
'    arguments:     /
'
'    return value:  usergroup as string
'
'=========================================================================================
Public Property Get userGroup() As String
    On Error GoTo errP_Get_userGroup

    Const FUNCTION_NAME = "P_GET userGroup"
    
    userGroup = m_sUserGroup
    
    Exit Property
    
errP_Get_userGroup:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property

'=========================================================================================
'    property:      isNIConnection
'    description:   return if connection object is running in NI Mode
'    arguments:     /
'
'    return value:  true is connection object is running in NI Mode
'
'=========================================================================================
Public Property Get isNIConnection() As Boolean
    On Error GoTo errP_Get_isNIConnection
    
    Const FUNCTION_NAME = "P_Get isNIConnection"
    
    isNIConnection = m_bNI
    
    Exit Property
    
errP_Get_isNIConnection:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      isNIConnection
'    description:   set if connection object is running in NI Mode
'    arguments:     [in] NewisNIConnection,flag to indicate if connection object is running in NI Mode
'
'    return value:  /
'
'=========================================================================================
Friend Property Let isNIConnection(NewisNIConnection As Boolean)
    On Error GoTo errP_Let_isNIConnection
    
    Const FUNCTION_NAME = "P_Let isNIConnection"
    
    m_bNI = NewisNIConnection
    
    Exit Property
    
errP_Let_isNIConnection:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      unicodeDatabase
'    description:   return if connected database is unicode
'    arguments:     /
'
'    return value:  true is connections object is running in NI Mode
'
'=========================================================================================
Public Property Get unicodeDatabase() As Boolean
    On Error GoTo errP_Get_UnicodeDatabase

    Const FUNCTION_NAME = "P_GET unicodeDatabase"

    unicodeDatabase = m_bUnicodeDatabase

    Exit Property

errP_Get_UnicodeDatabase:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      unicodeDatabase
'    description:   return if connected database is unicode
'    arguments:     /
'
'    return value:  true is connections object is running in NI Mode
'
'=========================================================================================
Public Property Let unicodeDatabase(newUnicodeDB As Boolean)
    On Error GoTo errP_Let_UnicodeDatabase

    Const FUNCTION_NAME = "P_LET unicodeDatabase"

     m_bUnicodeDatabase = newUnicodeDB

    Exit Property

errP_Let_UnicodeDatabase:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property



''''=========================================================================================
''''    property:      dbServer
''''    description:   return server database of connection object
''''    arguments:     /
''''
''''    return value:  dbServer as string
''''
''''=========================================================================================
'''Public Property Get dbServer() As String
'''    On Error GoTo errP_Get_dbServer
'''
'''    Const FUNCTION_NAME = "P_GET dbServer"
'''
'''    dbServer = m_sDBServer
'''
'''    Exit Property
'''
'''errP_Get_dbServer:
'''    internalError err, MODULE_NAME, FUNCTION_NAME
'''End Property
'''
''''=========================================================================================
''''    property:      dbServer
''''    description:   set server data base of connection object
''''    arguments:     [in] sNewDBServer, new server data base
''''
''''    return value:  /
''''
''''=========================================================================================
'''Friend Property Let dbServer(sNewDBServer As String)
'''    On Error GoTo errP_Let_dbServer
'''
'''    Const FUNCTION_NAME = "P_Let dbServer"
'''
'''    m_sDBServer = sNewDBServer
'''
'''    Exit Property
'''
'''errP_Let_dbServer:
'''    internalError err, MODULE_NAME, FUNCTION_NAME
'''End Property
'''
''''=========================================================================================
''''    property:      uid
''''    description:   set user ID of connection object
''''    arguments:     [in] UserID ID as string
''''
''''    return value:  /
''''
''''=========================================================================================
'''Friend Property Let uid(UserID As String)
'''    On Error GoTo errP_Let_uid
'''
'''    Const FUNCTION_NAME = "P_LET uid"
'''
'''    m_sUID = UserID
'''
'''    Exit Property
'''
'''errP_Let_uid:
'''    internalError err, MODULE_NAME, FUNCTION_NAME
'''End Property
''''=========================================================================================
''''    property:      uid
''''    description:   return user ID of connection object
''''    arguments:     /
''''
''''    return value:  user ID as string
''''
''''=========================================================================================
'''Public Property Get uid() As String
'''    On Error GoTo errP_Get_uid
'''
'''    Const FUNCTION_NAME = "P_GET uid"
'''
'''    uid = m_sUID
'''
'''    Exit Property
'''
'''errP_Get_uid:
'''    internalError err, MODULE_NAME, FUNCTION_NAME
'''End Property
'''
