VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Error"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Description = "Handles ADABAS D errors"
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' Class:      error
'
'
'-----------------------------------------------------------------------
'
' responsible:  WolfgangA
'
' special area: SQL Studio DB Layer
'
' description:  error class for handling a errors on connection
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Option Explicit

'--------------------------------------------------------
'Module Name
'--------------------------------------------------------
Const MODULE_NAME = "error"

'=private constants============================================================================

'Const MX_SQL_STATE = 16
Const MX_ERROR_MSG = 256
Const SQLSTATE_LEN = 5
Const NATIVEERROR_SUCCESS = 0

'=varibles============================================================================

'Property variables
Private m_lEnv            As Long
Private m_lDbc            As Long
Private m_bIsError        As Boolean
Private m_bIsWarning      As Boolean
Private m_sRC             As String
Private m_sNativeError    As String
Private m_sText           As String
Private m_sSQLState       As String
Private m_sPos            As String
Private m_sTime           As String
Private m_sStatement      As String
Private m_sSQLStateErrorNumber  As String

'if you want a SQLState to be ignored set member
Private m_sNoSQLState     As String

'if you want a NativeError to be ignored set member
Private m_sNoNativeError  As String

'=remember last error==================================================
Private m_bLastIsError        As Boolean
Private m_sLastIsWarning      As Boolean
Private m_sLastNativeError    As String
Private m_sLastText           As String
Private m_sLastSQLState       As String
Private m_sLastPos            As String
Private m_sLastTime           As String
Private m_sLastStatement      As String
Private m_sLastNoSQLState     As String
Private m_sLastNoNativeError  As String
Private m_sLastRC             As String
Private m_sLastSQLStateErrNo  As String
'======================================================================

'=========================================================================================
'    property:      hEnv
'    description:   set enviroment handle of error object
'    arguments:     [in] handleEnv, env handle as long
'
'    return value:  /
'
'=========================================================================================
Public Property Let hEnv(handleEnv As Long)
    On Error GoTo errP_Let_hEnv

    Const FUNCTION_NAME = "P_LET hEnv"
    
    m_lEnv = handleEnv
    
    Exit Property
    
errP_Let_hEnv:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      hEnv
'    description:   get enviroment handle of error object
'    arguments:     /
'
'    return value:  env handle as long
'
'=========================================================================================
Private Property Get hEnv() As Long
    On Error GoTo errP_Get_hEnv

    Const FUNCTION_NAME = "P_GET hEnv"
    
    hEnv = m_lEnv
    
    Exit Property
    
errP_Get_hEnv:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      hDbc
'    description:   set database handle of error object
'    arguments:     [in] handleDBC, dbc handle as long
'
'    return value:  /
'
'=========================================================================================
Public Property Let hDbc(handleDBC As Long)
    On Error GoTo errP_Let_hDBC

    Const FUNCTION_NAME = "P_LET hDBC"
    
    m_lDbc = handleDBC
    
    Exit Property
    
errP_Let_hDBC:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      hDbc
'    description:   get database handle of error object
'    arguments:     /
'
'    return value:  dbc handle as long
'
'=========================================================================================
Public Property Get hDbc() As Long
    On Error GoTo errP_Get_hDBC

    Const FUNCTION_NAME = "P_GET hDBC"
    
    hDbc = m_lDbc
    
    Exit Property
    
errP_Get_hDBC:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      isError
'    description:   get if error object contains an error
'    arguments:     /
'
'    return value:  boolean
'
'=========================================================================================
Public Property Get isError() As Boolean
    On Error GoTo errP_Get_IsError

    Const FUNCTION_NAME = "P_Get isError"
    
    isError = m_bIsError
    
    Exit Property
    
errP_Get_IsError:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      isError
'    description:   set if error object contains an error
'    arguments:     newIsError, new value of property
'
'    return value:  /
'
'=========================================================================================
Friend Property Let isError(newIsError As Boolean)
    On Error GoTo errP_Let_IsError

    Const FUNCTION_NAME = "P_Let isError"
    
    m_bIsError = newIsError
    
    Exit Property
    
errP_Let_IsError:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      isWarning
'    description:   get if error object contains a warning
'    arguments:     /
'
'    return value:  boolean
'
'=========================================================================================
Public Property Get isWarning() As Boolean
    On Error GoTo errP_Get_IsWarning

    Const FUNCTION_NAME = "P_Get isWarning"
    
    isWarning = m_bIsWarning
    
    Exit Property
    
errP_Get_IsWarning:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      RC
'    description:   get error return code as string
'    arguments:     /
'
'    return value:  m_sRC as string
'
'=========================================================================================
Public Property Get rc() As String
    On Error GoTo errP_Get_RC

    Const FUNCTION_NAME = "P_GET RC"
    
    rc = m_sRC
    
    Exit Property
    
errP_Get_RC:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      nativeError
'    description:   get native error as string
'    arguments:     /
'
'    return value:  m_sNativeError as string
'
'=========================================================================================
Public Property Get nativeError() As String
   On Error GoTo errP_Get_NativeError

    Const FUNCTION_NAME = "P_Get nativeError"
    
    nativeError = m_sNativeError
    
    Exit Property
    
errP_Get_NativeError:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      text
'    description:   get error text
'    arguments:     /
'
'    return value:  m_sText as string
'
'=========================================================================================
Public Property Get text() As String
    On Error GoTo errP_Get_Text

    Const FUNCTION_NAME = "P_Get text"
    
    text = m_sText
    
    Exit Property
    
errP_Get_Text:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      text
'    description:   set error text
'    arguments:     newText, new error text
'
'    return value:  /
'
'=========================================================================================
Friend Property Let text(snewText As String)
    On Error GoTo errP_Let_Text

    Const FUNCTION_NAME = "P_Let text"
    
    m_sText = snewText
    
    Exit Property
    
errP_Let_Text:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      sqlState
'    description:   get sql state of actual error
'    arguments:     /
'
'    return value:  string
'
'=========================================================================================
Public Property Get sqlState() As String
    On Error GoTo errP_Get_SQLState

    Const FUNCTION_NAME = "P_Get sqlState"
    
    sqlState = m_sSQLState
    
    Exit Property
    
errP_Get_SQLState:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      sqlStateErrorNumber
'    description:   get sql state error number of actual error
'    arguments:     /
'
'    return value:  string
'
'=========================================================================================
Public Property Get sqlStateErrorNumber() As String
    On Error GoTo errP_Get_SQLStateErrorNumber

    Const FUNCTION_NAME = "P_Get sqlStateErrorNumber"
    
    sqlStateErrorNumber = m_sSQLStateErrorNumber
    
    Exit Property
    
errP_Get_SQLStateErrorNumber:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      pos
'    description:   get position of error in sql command as string
'    arguments:     /
'
'    return value:  string
'
'=========================================================================================
Public Property Get pos() As String
    On Error GoTo errP_Get_Pos

    Const FUNCTION_NAME = "P_Get pos"
    
    pos = m_sPos
    
    Exit Property
    
errP_Get_Pos:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      time
'    description:   get examination time of error as string
'    arguments:     /
'
'    return value:  string
'
'=========================================================================================
Public Property Get time() As String
    On Error GoTo errP_Get_Time

    Const FUNCTION_NAME = "P_Get time"
    
    time = m_sTime
    
    Exit Property
    
errP_Get_Time:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      statement
'    description:   set sql statement of error object
'    arguments:     newStatement, new sql statement
'
'    return value:  /
'
'=========================================================================================
Public Property Let statement(newStatement As String)
    On Error GoTo errP_Let_Statement

    Const FUNCTION_NAME = "P_Let statement"
    
    m_sStatement = newStatement
    
    'replaceNonChars m_sStatement
    
    Exit Property
    
errP_Let_Statement:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      statement
'    description:   get sql statement of error object as string
'    arguments:     /
'
'    return value:  string
'
'=========================================================================================
Public Property Get statement() As String
    On Error GoTo errP_Get_Statement

    Const FUNCTION_NAME = "P_Get statement"
    
    statement = m_sStatement
    
    Exit Property
    
errP_Get_Statement:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      noSQLState
'    description:   set sql state to be ignored in CheckRC
'    arguments:     sNoSQLState
'
'    return value:  /
'
'=========================================================================================
Public Property Let noSQLState(sNoSQLState As String)
    On Error GoTo errP_Let_NoSQLState

    Const FUNCTION_NAME = "P_Let noSQLState"
    
    m_sNoSQLState = sNoSQLState
    
    Exit Property
    
errP_Let_NoSQLState:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      noSQLState
'    description:   get sql state to be ignored in CheckRC
'    arguments:     /
'
'    return value:  string
'
'=========================================================================================
Public Property Get noSQLState() As String
    On Error GoTo errP_Get_NoSQLState

    Const FUNCTION_NAME = "P_Get noSQLState"
    
    noSQLState = m_sNoSQLState
    
    Exit Property
    
errP_Get_NoSQLState:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      noSQLState
'    description:   set native error to be ignored in CheckRC
'    arguments:     sNoNativeError
'
'    return value:  /
'
'=========================================================================================
Public Property Let noNativeError(sNoNativeError As String)
    On Error GoTo errP_Let_NoNativeError

    Const FUNCTION_NAME = "P_Let noNativeError"
    
    m_sNoNativeError = sNoNativeError
    
    Exit Property
    
errP_Let_NoNativeError:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      noNativeError
'    description:   get native error to be ignored in CheckRC
'    arguments:     /
'
'    return value:  m_NoNativeError as string
'
'=========================================================================================
Public Property Get noNativeError() As String
    On Error GoTo errP_Get_NoNativeError

    Const FUNCTION_NAME = "P_Get noNativeError"
    
    noNativeError = m_sNoNativeError
    
    Exit Property
    
errP_Get_NoNativeError:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    function:      clear
'    description:   clear all members
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Private Function clear()
    On Error GoTo ErrClear

    Const FUNCTION_NAME = "Clear"
    
    m_bIsError = False
    m_bIsWarning = False
    m_sRC = ""
    m_sNativeError = ""
    m_sText = ""
    m_sSQLState = ""
    m_sPos = ""
    m_sTime = ""
    m_sNoNativeError = ""
    m_sNoSQLState = ""
    m_sSQLStateErrorNumber = ""
    
    m_bLastIsError = False
    m_sLastIsWarning = False
    m_sLastNativeError = ""
    m_sLastText = ""
    m_sLastSQLState = ""
    m_sLastPos = ""
    m_sLastTime = ""
    m_sLastStatement = ""
    m_sLastNoSQLState = ""
    m_sLastNoNativeError = ""
    m_sLastRC = ""
    m_sLastSQLStateErrNo = ""
    
    Exit Function
    
ErrClear:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      reset
'    description:   reset all members
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function reset()
    On Error GoTo errReset

    Const FUNCTION_NAME = "Reset"
  
    'clear all members
    clear
    
    m_sText = LoadResString(109)
    
    Exit Function
    
errReset:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      checkRC
'    description:   connects to database
'    arguments:     [in] ReturnCode, err number of database
'                   [in] hStmt, handle of statement
'
'    return value:  true if successfully
'
'=========================================================================================
Public Function checkRC(ByVal ReturnCode As Integer, ByVal hStmt As Long) As Boolean
    On Error GoTo errCheckRC

    Const FUNCTION_NAME = "checkRC"

    Dim SQLErrorRC         As Integer
    Dim sSQLState          As String
    Dim bByteSQLState()    As Byte
    Dim nativeError        As Long
    Dim sErrorMsg          As String
    Dim bByteErrorMsg()    As Byte
    Dim nErrorMsgLen       As Integer
    
    'inits
    checkRC = True
    m_bIsError = False
    m_bIsWarning = False
    m_sRC = Trim$(Str$(ReturnCode))
    m_sNativeError = ""
    'initial value
    m_sText = LoadResString(109)
    m_sSQLState = ""
    m_sSQLStateErrorNumber = ""
    m_sPos = ""
    m_sTime = ""

    'Everything ok?
    If ReturnCode = SQL_SUCCESS Then Exit Function
    
    'Invalid handle?
    If ReturnCode = SQL_INVALID_HANDLE Then
        
        m_bIsError = True
        m_sText = LoadResString(108)
        checkRC = False
        setLastError
        Exit Function
    End If

    'get more error information
    If (ReturnCode = SQL_ERROR) Or (ReturnCode = SQL_SUCCESS_WITH_INFO) Then
    
        'buffer for unicode function
        ReDim bByteSQLState(SQLSTATE_LEN * 2)
        ReDim bByteErrorMsg(MX_ERROR_MSG * 2)
        
        'SQLError argument description
        'sSQLState: The value for the SQLSTATE represented by the error.
        'NativeError: A nonzero value indicates a ODBC Driver Native error Message.
        '             A value of zero indicates the error has been detected
        '             by the driver and mapped to the appropriate ODBC error Code.
        'sErrorMsg: The text for the native error or ODBC error.
        'nErrorMsgLen: The length of the message text plus the length of the identifiers

        SQLErrorRC = SQLErrorW(m_lEnv, _
                               m_lDbc, _
                               hStmt, _
                               bByteSQLState(0), _
                               nativeError, _
                               bByteErrorMsg(0), _
                               MX_ERROR_MSG, _
                               nErrorMsgLen)
        sSQLState = left$(bByteSQLState, SQLSTATE_LEN)
        sErrorMsg = left$(bByteErrorMsg, nErrorMsgLen)
        If (SQLErrorRC = SQL_SUCCESS) Or (SQLErrorRC = SQL_SUCCESS_WITH_INFO) Then
        
        
            'Store SQL state and native error
            m_sSQLStateErrorNumber = Trim$(left$(sSQLState, SQLSTATE_LEN))
    
            'If no SQL state then return
            If sSQLState <> SQLSTATE_SUCCESS Then
            
                If sSQLState = m_sNoSQLState Then
                
                    m_sRC = SQL_SUCCESS
                    m_bIsError = False
                    checkRC = True
                    Exit Function
                End If
            End If

            'If no native error then return
            If nativeError <> NATIVEERROR_SUCCESS Then
            
                If nativeError = Val(m_sNoNativeError) Then
                
                    m_sRC = SQL_SUCCESS
                    checkRC = True
                    m_bIsError = False
                    Exit Function
                End If
            End If
            
            'SQLState 1008 Operation canceled is not a error
            If ReturnCode = SQL_SUCCESS_WITH_INFO Or sSQLState = "1008" Then
            
                
                m_sSQLState = Trim$(sSQLState)
                m_sText = sErrorMsg
                m_sRC = SQL_SUCCESS
                
                'a time out error is be handled as an error not as a warning
                If nativeError = -22005 Then
                
                    m_bIsWarning = False
                    m_bIsError = True
                    checkRC = False
                Else
                    m_bIsWarning = True
                    m_bIsError = False
                    checkRC = True
                End If
                
                Exit Function
            End If

            'show error message
            If nErrorMsgLen = 0 Or ReturnCode = SQL_SUCCESS_WITH_INFO Then
            
                m_sText = LoadResString(103)
                MsgBox LoadResString(103), vbOKOnly + vbExclamation, "error"
            Else
                m_sText = sErrorMsg
                scanErrorText left$(sErrorMsg, nErrorMsgLen), sSQLState, nativeError
            End If
            
            m_bIsError = True
            setLastError
            checkRC = False
            Exit Function
        Else
            m_bIsError = True
            m_sText = LoadResString(103)
            MsgBox LoadResString(103), vbOKOnly + vbExclamation, "error"
            setLastError
            checkRC = False
            Exit Function
        End If
    Else
        If ReturnCode = SQL_NO_DATA_FOUND Then
        
            m_sText = LoadResString(106)
            checkRC = True
        End If
        
        Exit Function
    End If
   
    
    Exit Function

errCheckRC:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      showError
'    description:   display actual error
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function showError()
    On Error GoTo errShowError

    Const FUNCTION_NAME = "ShowError"
    
    Dim ErrorText   As String
    
    If m_bIsError Then
    
        If Len(m_sNativeError) > 0 Then
        
            ErrorText = "Database error occured:" + Chr$(13) + Chr$(10) + Chr$(13) + Chr$(10)
            ErrorText = ErrorText + "Native error:" + Chr$(9) + m_sNativeError + Chr$(13) + Chr$(10)
            ErrorText = ErrorText + "Text:" + Chr$(9) + Chr$(9) + m_sText + Chr$(13) + Chr$(10)
        Else
            ErrorText = "error occured:" + Chr$(13) + Chr$(10) + Chr$(13) + Chr$(10)
            If Len(m_sText) > 0 Then
            
                ErrorText = ErrorText + "Text:" + Chr$(9) + m_sText + Chr$(13) + Chr$(10)
            Else
                ErrorText = ErrorText + "Text: Can't analyze !" + Chr$(13) + Chr$(10)
            End If
        End If
        
        setLastError
        MsgBox ErrorText, vbOKOnly + vbExclamation, "error"
    End If
    
    Exit Function
    
errShowError:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      showWarning
'    description:   display actual warning
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function showWarning()
    On Error GoTo errShowWarning

    Const FUNCTION_NAME = "ShowWarning"
    
    Dim ErrorText   As String
    
    If m_bIsWarning Then
    
        ErrorText = "Warning:" + Chr$(13) + Chr$(10) + Chr$(13) + Chr$(10)
        ErrorText = ErrorText + "Text:" + Chr$(9) + Chr$(9) + m_sText + Chr$(13) + Chr$(10)
    End If
    
    MsgBox ErrorText, vbOKOnly + vbExclamation, "Warning"
        
    Exit Function
    
errShowWarning:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      scanErrorText
'    description:   scan error text to set some error object members
'    arguments:     [in] sErrorText
'                   [in] sSQLState
'                   [in] sNativeError
'
'    return value:  /
'
'=========================================================================================
Private Function scanErrorText(ByVal sErrorText As String, _
                          ByVal sSQLState As String, _
                          ByVal sNativeError As String)
    On Error GoTo errScanErrorText

    Const FUNCTION_NAME = "scanErrorText"

    Dim i           As Integer
    Dim ch          As String
    Dim Pos1        As Integer
    Dim Pos2        As Integer
    Dim Pos3        As Integer
    Dim ErrNo       As String
    Dim ErrMsg      As String
    Dim errState    As String
    Dim errPos      As String

    'Inits
    ErrNo = ""
    errPos = ""
    errState = ""
    ErrMsg = ""

    'Search the SQL state indicator
    If sSQLState <> SQLSTATE_SUCCESS Then
    
        Pos3 = 0
        Do
            Pos1 = InStr(Pos3 + 1, sErrorText, "]")
            Pos2 = InStr(Pos1 + 1, sErrorText, ";")
            Pos3 = InStr(Pos1 + 1, sErrorText, "]")
        Loop While Pos3 > Pos1 And Pos3 < Pos2
    End If

    'If a native error exists then split the error message into state and native error message
    If sNativeError <> NATIVEERROR_SUCCESS Then
        If Pos2 > 0 Then
            errState = Mid$(sErrorText, Pos1 + 1, Pos2 - Pos1 - 1)
            ErrMsg = Mid$(sErrorText, Pos2 + 1, Len(sErrorText) - Pos2)

            'get error no
            For i = 1 To Len(ErrMsg)
                ch = Mid$(ErrMsg, i, 1)
                If ch <> " " Then
                
                    ErrNo = ErrNo + ch
                Else
                    ErrMsg = right$(ErrMsg, Len(ErrMsg) - i)
                    Exit For
                End If
            Next

            'get error position
            Pos1 = InStr(1, ErrMsg, "POS(")
            If Pos1 > 0 Then
            
                Pos2 = InStr(Pos1, ErrMsg, ") ")
                If Pos2 > 0 Then
                
                    errPos = Mid$(ErrMsg, Pos1 + 4, Pos2 - Pos1 - 4)
                    ErrMsg = right$(ErrMsg, Len(ErrMsg) - Pos2 - 1)
                End If
            End If
        End If
    Else
        'Get the the state error message
        errState = Mid$(sErrorText, Pos1 + 1, Len(sErrorText) - Pos1)
    End If
    
    'Set error properties
    m_sNativeError = ErrNo
    m_sSQLState = "[ODBC] [" + Trim$(sSQLState) + "] " + Trim$(errState)
    m_sPos = Trim$(Str$(Val(errPos)))
    m_sTime = Now
    
    Exit Function

errScanErrorText:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      class_Initialize
'    description:   Occurs when an application creates an instance of a Form, MDIForm, User control, Property Page, Webclass, DHTML Page Designer, or class.
'
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Private Sub Class_Initialize()
    On Error GoTo ErrClass_Initialize
    
    Const FUNCTION_NAME = "Class_Initialize"
    
    m_lEnv = SQL_NULL_HENV
    m_lDbc = SQL_NULL_HDBC
    
    clear
    
    Exit Sub

ErrClass_Initialize:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    property:      setLastError
'    description:   set memebers of last error
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Private Function setLastError()
    On Error GoTo errSetLastError
    
    Const FUNCTION_NAME = "setLastError"
    
    m_bLastIsError = m_bIsError
    m_sLastIsWarning = m_bIsWarning
    m_sLastNativeError = m_sNativeError
    m_sLastText = m_sText
    m_sLastSQLState = m_sSQLState
    m_sLastPos = m_sPos
    m_sLastTime = m_sTime
    m_sLastStatement = m_sStatement
    m_sLastNoSQLState = m_sNoSQLState
    m_sLastNoNativeError = m_sNoNativeError
    m_sLastRC = m_sRC
    m_sLastSQLStateErrNo = m_sSQLStateErrorNumber
     
    Exit Function

errSetLastError:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    property:      lastIsError
'    description:   get if error object contained an error before
'    arguments:     /
'
'    return value:  boolean
'
'=========================================================================================
Public Property Get lastIsError() As Boolean
    On Error GoTo errP_Get_LastIsError

    Const FUNCTION_NAME = "P_Get lastIsError"
    
    lastIsError = m_bLastIsError
    
    Exit Property
    
errP_Get_LastIsError:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      lastIsWarning
'    description:   get if error object contained a warning before
'    arguments:     /
'
'    return value:  boolean
'
'=========================================================================================
Public Property Get lastIsWarning() As Boolean
    On Error GoTo errP_Get_LastIsWarning

    Const FUNCTION_NAME = "P_Get lastIsWarning"
    
    lastIsWarning = m_sLastIsWarning
    
    Exit Property
    
errP_Get_LastIsWarning:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      lastRC
'    description:   Get last error return code as string
'    arguments:     /
'
'    return value:  m_sLastRC as string
'
'=========================================================================================
Public Property Get lastRC() As String
    On Error GoTo errP_Get_LastRC

    Const FUNCTION_NAME = "P_Get lastRC"
    
    lastRC = m_sLastRC
    
    Exit Property
    
errP_Get_LastRC:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      lastNativeError
'    description:   get native error as string
'    arguments:     /
'
'    return value:  m_sLastNativeError as string
'
'=========================================================================================
Public Property Get lastNativeError() As String
    On Error GoTo errP_Get_LastNativeError

    Const FUNCTION_NAME = "P_Get lastNativeError"
    
    lastNativeError = m_sLastNativeError
    
    Exit Property
    
errP_Get_LastNativeError:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      lastText
'    description:   Get last error text
'    arguments:     /
'
'    return value:  m_sLastText as string
'
'=========================================================================================
Public Property Get lastText() As String
    On Error GoTo errP_Get_LastText

    Const FUNCTION_NAME = "P_Get lastText"
    
    lastText = m_sLastText
    
    Exit Property
    
errP_Get_LastText:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      lastSQLState
'    description:   Get last sql state of actual error
'    arguments:     /
'
'    return value:  string
'
'=========================================================================================
Public Property Get lastSQLState() As String
    On Error GoTo errP_Get_LastSQLState

    Const FUNCTION_NAME = "P_Get lastSQLState"
    
    lastSQLState = m_sLastSQLState
    
    Exit Property
    
errP_Get_LastSQLState:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      lastSQLStateErrorNumber
'    description:   Get last sql state error number of actual error
'    arguments:     /
'
'    return value:  string
'
'=========================================================================================
Public Property Get lastSQLStateErrorNumber() As String
    On Error GoTo errP_Get_LastSQLStateErrorNumber

    Const FUNCTION_NAME = "P_Get lastSQLStateErrorNumber"
    
    lastSQLStateErrorNumber = m_sLastSQLStateErrNo
    
    Exit Property
    
errP_Get_LastSQLStateErrorNumber:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      lastPos
'    description:   Get last position of error in sql command as string
'    arguments:     /
'
'    return value:  string
'
'=========================================================================================
Public Property Get lastPos() As String
    On Error GoTo errP_Get_LastPos

    Const FUNCTION_NAME = "P_Get lastPos"
    
    lastPos = m_sLastPos
    
    Exit Property
    
errP_Get_LastPos:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      lastTime
'    description:   Get last examination time of error as string
'    arguments:     /
'
'    return value:  string
'
'=========================================================================================
Public Property Get lastTime() As String
    On Error GoTo errP_Get_LastTime

    Const FUNCTION_NAME = "P_Get lastTime"
    
    lastTime = m_sLastTime
    
    Exit Property
    
errP_Get_LastTime:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      lastStatement
'    description:   Get last sql statement of error object as string
'    arguments:     /
'
'    return value:  string
'
'=========================================================================================
Public Property Get lastStatement() As String
    On Error GoTo errP_Get_LastStatement

    Const FUNCTION_NAME = "P_Get lastStatement"
    
    lastStatement = m_sLastStatement
    
    Exit Property
    
errP_Get_LastStatement:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      lastNoSQLState
'    description:   Get last sql state to be ignored in CheckRC
'    arguments:     /
'
'    return value:  string
'
'=========================================================================================
Public Property Get lastNoSQLState() As String
    On Error GoTo errP_Get_LastNoSQLState

    Const FUNCTION_NAME = "P_Get lastNoSQLState"
    
    lastNoSQLState = m_sLastNoSQLState
    
    Exit Property
    
errP_Get_LastNoSQLState:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      lastNoNativeError
'    description:   get native error to be ignored in CheckRC
'    arguments:     /
'
'    return value:  m_LastNoNativeError as string
'
'=========================================================================================
Public Property Get lastNoNativeError() As String
    On Error GoTo errP_Get_LastNoNativeError

    Const FUNCTION_NAME = "P_Get noNativeError"
    
    lastNoNativeError = m_sLastNoNativeError
    
    Exit Property
    
errP_Get_LastNoNativeError:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      replaceNonChars
'    description:   replace non chars in sText
'    arguments:     [in] sText, string to be cleaned
'                   property should be private,
'                   because of binary compability it is kept public
'
'    return value:  /
'
'=========================================================================================
Public Function replaceNonChars(sText As String)
    On Error GoTo errReplaceZero

    Const FUNCTION_NAME = "ReplaceZero"
    
    Dim i   As Integer
    
    ' 9 = Replace tabulators
    For i = 0 To 9
        Call replace(sText, Chr$(i), " ")
    Next i
    
    ' Replace tabulators
    Call replace(sText, Chr$(11), " ")
    Call replace(sText, Chr$(12), " ")
    
    For i = 14 To 31
        Call replace(sText, Chr$(i), " ")
    Next i
    
    ' Replace CR/LF
    Call replace(sText, Chr$(13) + Chr$(10), " ")
    
    ' Replace LF/CR
    Call replace(sText, Chr$(10) + Chr$(13), " ")
    
    ' Replace LF
    Call replace(sText, Chr$(10), " ")
    
    ' Replace CR
    Call replace(sText, Chr$(13), " ")
        
    Exit Function

errReplaceZero:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    property:      replace
'    description:   replace non chars in sText
'    arguments:     [in] sText, string to be cleaned
'                   [in] sReplace, partial string to be replaced
'                   [in] sReplaceTo, partial string to be set
'
'    return value:  /
'
'=========================================================================================
Private Function replace(sText As String, _
                    ByVal sReplace As String, _
                    ByVal sReplaceTo As String)
    On Error GoTo errReplace

    Const FUNCTION_NAME = "Replace"

    Dim ReplacePos      As Long
    Dim ReplaceOfLen    As Long
    Dim ReplaceToLen    As Long
    Dim TextLen         As Long
    
    ReplacePos = InStr(sText, sReplace)
    If ReplacePos = 0 Then Exit Function
    
    ReplaceOfLen = Len(sReplace)
    TextLen = Len(sText)

    While ReplacePos > 0
        Mid$(sText, ReplacePos, ReplaceOfLen) = sReplaceTo
        ReplaceToLen = Len(sReplaceTo)
        TextLen = TextLen + ReplaceToLen - ReplaceOfLen
        If ReplacePos + ReplaceToLen <= TextLen Then
            ReplacePos = InStr(ReplacePos + ReplaceToLen, sText, sReplace)
        Else
            Exit Function
        End If
    Wend

    Exit Function

errReplace:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function

