VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Connections"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Description = "Manages  SAPDB Connections"
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' Class:      Connections
'
'
'-----------------------------------------------------------------------
'
' responsible:  WolfgangA
'
' special area: SQL Studio DB Layer
'
' description:  class for handling a connection pool
'
'
'
'  How to receive a database connection from the connections object ?
'
'  1. create a instance of saodbconnections
'     set oconman = new sapdbxconnections
'
'  2. create a con object
'     set ocon = sapdbcon
'
'  3. set all available connect params
'     oconman.uid = "user"
'     oconman.pwd = "pwd"
'     oconman.driver = "SAP DB DRIVER"
'
'  4. set poolingmode
'
'  5. call oconman.getconnection(autoCommit,....,ocon)
'
'     the first call of getconnection will always Get the attributes of the driver
'     execption: sql mode oracle is set, e.g. for support mode with not exclusive user
'
'  6. check returncode and if valid error object of ocon object
'
'  6. call conman.disconnect to close all open database connections
'
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option Explicit

'Module name
Const MODULE_NAME = "Connections"


'WA_DBC_EXCL: The returned connection is currently unused and won't be returned for another Connect until Disconnect is called.
'WA_DBC_EXCL_autoCommit_OFF: Only autoCommit off connections are exclusiv. autoCommit on connections are shared.
'WA_DBC_SHARED: The connection may be returned by several Connects. Only if #Connects-#Disconntecs is 0 the connection can be used exclusive.

Public Enum poolingMode

    WA_DBC_EXCL = 0
    WA_DBC_EXCL_AUTOCOMMIT_OFF = 1
    WA_DBC_SHARED = 2
End Enum

'=variables for WA Pooling=======================================================================
Private m_lWaDBCHandle         As Long

'=varibles============================================================================

'handle of calling window
Private m_lWnd            As Long

'=connection user properties===================
Private m_bNotExclusive   As Boolean
Private m_bIsResourceUser As Boolean
Private m_bIsConnected    As Boolean

Private m_sPWD            As String
Private m_sUID            As String
Private m_sDriver         As String
Private m_sDataBase       As String
Private m_sDBServer       As String
Private m_sUserGroup      As String

Private m_sTraceFile      As String
'==============================================

'database is a unicode database
Private m_bUnicodeDatabase      As Boolean

'isolevel of odbc driver
Private m_emDriverIsoLevel       As isoLevel

'sqlMode of odbc driver
Private m_emDriverSQLMode        As sqlMode

'pooling mode for connections
'values: {WA_NONE,WA_DBC_EXCL,WA_DBC_EXCL_AUTOCOMMIT_OFF,WA_DBC_SHARED}
Private m_lPoolingMode             As poolingMode

'handle for WA Connection Pool
Private m_lPool                    As Long

'flaf if connections run in NI Mode
Private m_bNI                       As Boolean

'em varible to store the type of user which owns the connection
'values: {umUnkown,umSTANDARD,umRESOURCE,umDBA,umSYSDBA,umCOLDUSER}
Private m_enUsermode                As UserMode

'time out for database in sec, 0 sec no timeout
Private m_lTimeOut                  As Long

'collection of connection objects
Private m_colConnections           As New Collection
'=====================================================================================


'=========================================================================================
'    property:      hWnd
'    description:   set window handle if application is using connections
'    arguments:     [in] handleWnd, window handle as long
'
'    return value:  /
'
'=========================================================================================
Public Property Let hWnd(handleWnd As Long)
    On Error GoTo errP_Let_hWnd

    Const FUNCTION_NAME = "P_LET hWnd"
    
    m_lWnd = handleWnd
    
    Exit Property
    
errP_Let_hWnd:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      hWnd
'    description:   return window handle if application is using connections
'    arguments:     /
'
'    return value:  window handle as long
'
'=========================================================================================
Public Property Get hWnd() As Long
    On Error GoTo errP_Get_hWnd

    Const FUNCTION_NAME = "P_GET hWnd"
    
    hWnd = m_lWnd
    
    Exit Property
    
errP_Get_hWnd:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      isNotExclusive
'    description:   return is user of connections is a not exclusive user
'    arguments:     /
'
'    return value:  true/false
'
'=========================================================================================
Public Property Get isNotExclusive() As Boolean
    On Error GoTo errP_Get_IsNotExclusive

    Const FUNCTION_NAME = "P_Get isNotExclusive"
    
    isNotExclusive = m_bNotExclusive
    
    Exit Property
    
errP_Get_IsNotExclusive:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      isResourceUser
'    description:   return is user of connections is a resource user
'    arguments:     /
'
'    return value:  true/false
'
'=========================================================================================
Public Property Get isResourceUser() As Boolean
    On Error GoTo errP_Get_isResourceUser

    Const FUNCTION_NAME = "P_Get isResourceUser"
    
    isResourceUser = m_bIsResourceUser
    
    Exit Property
    
errP_Get_isResourceUser:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      userMode
'    description:   return user mode of user which is connected
'    arguments:     /
'
'    return value:  UserMode as em type
'
'=========================================================================================
Public Property Get UserMode() As UserMode
    On Error GoTo errP_Get_UserMode

    Const FUNCTION_NAME = "P_GET userMode"
    
    UserMode = m_enUsermode
    
    Exit Property
    
errP_Get_UserMode:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      isConnected
'    description:   return if getconnection/initial connect succeeded
'    arguments:     /
'
'    return value:  true, if initial connect succeeded
'
'=========================================================================================
Public Property Get isConnected() As Boolean
    On Error GoTo errP_Get_IsConnected

    Const FUNCTION_NAME = "P_Get isConnected"
    
    isConnected = m_bIsConnected
    
    Exit Property
    
errP_Get_IsConnected:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      driver
'    description:   set driver name of connections object
'    arguments:     [in] newDriver name as string
'
'    return value:  /
'
'=========================================================================================
Public Property Let driver(newDriver As String)
    On Error GoTo errP_Let_Driver

    Const FUNCTION_NAME = "P_Let driver"
    
    m_sDriver = newDriver
    
    Exit Property
    
errP_Let_Driver:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      driver
'    description:   return driver name of connections object
'    arguments:     /
'
'    return value:  driver name as string
'
'=========================================================================================
Public Property Get driver() As String
    On Error GoTo errP_Get_Driver

    Const FUNCTION_NAME = "P_GET Driver"
    
    driver = m_sDriver
    
    Exit Property
    
errP_Get_Driver:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      uid
'    description:   set user ID of connections object
'    arguments:     [in] UserID ID as string
'
'    return value:  /
'
'=========================================================================================
Public Property Let uid(UserID As String)
    On Error GoTo errP_Let_uid

    Const FUNCTION_NAME = "P_LET uid"
    
    m_sUID = UserID
    
    Exit Property
    
errP_Let_uid:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      uid
'    description:   return user ID of connections object
'    arguments:     /
'
'    return value:  user ID as string
'
'=========================================================================================
Public Property Get uid() As String
    On Error GoTo errP_Get_uid

    Const FUNCTION_NAME = "P_GET uid"
    
    uid = m_sUID
    
    Exit Property
    
errP_Get_uid:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      userGroup
'    description:   return usergroup of connected user
'    arguments:     /
'
'    return value:  usergroup as string
'
'=========================================================================================
Public Property Get userGroup() As String
    On Error GoTo errP_Get_userGroup

    Const FUNCTION_NAME = "P_GET userGroup"
    
    userGroup = m_sUserGroup
    
    Exit Property
    
errP_Get_userGroup:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      pwd
'    description:   set password of connections object
'    arguments:     [in] sUserPWD, new password
'
'    return value:  /
'
'=========================================================================================
Public Property Let pwd(sUserPWD As String)
    On Error GoTo errP_Let_pwd

    Const FUNCTION_NAME = "P_Let pwd"
    
    m_sPWD = sUserPWD
    
    Exit Property
    
errP_Let_pwd:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      pwd
'    description:   return password of connections object
'    arguments:     /
'
'    return value:  password as string
'
'=========================================================================================
Public Property Get pwd() As String
    On Error GoTo errP_Get_pwd

    Const FUNCTION_NAME = "P_Get pwd"
    
    pwd = m_sPWD
    
    Exit Property
    
errP_Get_pwd:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      dataBase
'    description:   set server data base of connections object
'    arguments:     [in] snewServerDB, new server data base
'
'    return value:  /
'
'=========================================================================================
Public Property Let dataBase(snewdataBase As String)
    On Error GoTo errP_Let_dataBase

    Const FUNCTION_NAME = "P_Let dataBase"
    
    m_sDataBase = snewdataBase
    
    Exit Property
    
errP_Let_dataBase:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      dataBase
'    description:   return database of connections object
'    arguments:     /
'
'    return value:  dataBase as string
'
'=========================================================================================
Public Property Get dataBase() As String
    On Error GoTo errP_Get_dataBase

    Const FUNCTION_NAME = "P_GET dataBase"
    
    dataBase = m_sDataBase
    
    Exit Property
    
errP_Get_dataBase:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      dbServer
'    description:   set database server of connections object
'    arguments:     [in] snewServerNode, new database server
'
'    return value:  /
'
'=========================================================================================
Public Property Let dbServer(snewServerNode As String)
    On Error GoTo errP_Let_dbServer

    Const FUNCTION_NAME = "P_Let dbServer"
    
    m_sDBServer = snewServerNode
    
    Exit Property
    
errP_Let_dbServer:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      dbServer
'    description:   return database server of connections object
'    arguments:     /
'
'    return value:  database server as string
'
'=========================================================================================
Public Property Get dbServer() As String
    On Error GoTo errP_Get_dbServer

    Const FUNCTION_NAME = "P_GET dbServer"
    
    dbServer = m_sDBServer
    
    Exit Property
    
errP_Get_dbServer:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      driverIsoLevel
'    description:   return default iso level of the odbc driver
'    arguments:     /
'
'    return value:  isoLevel as em
'
'=========================================================================================
Public Property Get driverIsoLevel() As isoLevel
    On Error GoTo errP_Get_driverIsoLevel

    Const FUNCTION_NAME = "P_GET driverIsoLevel"
    
    driverIsoLevel = m_emDriverIsoLevel
    
    Exit Property
    
errP_Get_driverIsoLevel:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      driverIsoLevel
'    description:   set default iso level of the odbc driver
'    arguments:     newIsoLevel as em
'
'    return value:  /
'
'=========================================================================================
Public Property Let driverIsoLevel(newIsoLevel As isoLevel)
    On Error GoTo errP_Let_driverIsoLevel

    Const FUNCTION_NAME = "P_Let driverIsoLevel"
    
    m_emDriverIsoLevel = newIsoLevel
    
    Exit Property
    
errP_Let_driverIsoLevel:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      driverSQLMode
'    description:   return default sql mode of the odbc driver
'    arguments:     /
'
'    return value:  sqlMode as em
'
'=========================================================================================
Public Property Get driverSQLMode() As sqlMode
    On Error GoTo errP_Get_driverSQLMode

    Const FUNCTION_NAME = "P_GET driverSQLMode"
    
    driverSQLMode = m_emDriverSQLMode
    
    Exit Property
    
errP_Get_driverSQLMode:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      driverSQLMode
'    description:   set default sql mode of the odbc driver
'    arguments:     newSQLMode as em
'
'    return value:  /
'
'=========================================================================================
Public Property Let driverSQLMode(NewSQLMode As sqlMode)
    On Error GoTo errP_Let_driverSQLMode

    Const FUNCTION_NAME = "P_Let driverSQLMode"
    
    m_emDriverSQLMode = NewSQLMode
    
    Exit Property
    
errP_Let_driverSQLMode:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      isNIConnection
'    description:   return if connections object is running in NI Mode
'    arguments:     /
'
'    return value:  true is connections object is running in NI Mode
'
'=========================================================================================
Public Property Get isNIConnection() As Boolean
    On Error GoTo errP_Get_isNIConnection
    
    Const FUNCTION_NAME = "P_Get isNIConnection"
    
    'rc
    isNIConnection = m_bNI
    
    Exit Property
    
errP_Get_isNIConnection:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      timeout
'    description:   set timeout for database connection
'    arguments:     newTimeout, timeout in s
'
'    return value:  /
'
'=========================================================================================
Public Property Let timeout(newTimeOut As Long)
    On Error GoTo errP_Let_TimeOut

    Const FUNCTION_NAME = "P_Let timeOut"
    
    m_lTimeOut = newTimeOut
    
    Exit Property
    
errP_Let_TimeOut:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      poolingMode
'    description:   Get the (connection) pooling mode of the connections object
'    arguments:     [in] newPoolingMode, new poolingmode
'
'      WA_DBC_EXCL: The returned connection is
'             currently unused And won 't be returned
'             for another Connect until Disconnect is called.
'        WA_DBC_EXCL_autoCommit_OFF: Only autoCommit
'             off connections are exclusiv. autoCommit on
'             connections are shared.
'      WA_DBC_SHARED: The connection may be returned
'             by several Connects. Only if
'             #Connects-#Disconntecs is 0 the connection can be
'           used exclusive.
'
'    return value:  /
'
'=========================================================================================
Public Property Let poolingMode(newPoolingMode As poolingMode)
    On Error GoTo errP_Let_poolingMode

    Const FUNCTION_NAME = "P_Let poolingMode"
    
    m_lPoolingMode = newPoolingMode
    
    If m_lPool = 0 Then _
       m_lPool = sapdbwa_CreateDBCPool(-1)
       
    Exit Property
    
errP_Let_poolingMode:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      poolingMode
'    description:   Get the (connection) pooling mode of the connections object
'    arguments:     /
'
'    return value:  pooling mode of connections object
'
'=========================================================================================
Public Property Get poolingMode() As poolingMode
    On Error GoTo errP_Get_poolingMode

    Const FUNCTION_NAME = "P_Get poolingMode"
    
    poolingMode = m_lPoolingMode
        
    Exit Property
    
errP_Get_poolingMode:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    function:      getPoolConnection
'    description:   gets a connection object from the webagent pool
'    arguments:     [in] emIsoLevel, isolation level of database connection
'                   [in] emSqlMode, sqlMode mode of database connection
'                   [in] emAutoCommit, auto commit mode of database connection
'                   [in] bInitialConnect, is it the first connect
'                   [out] oConnection, a new connection object
'
'
'    return value:  long ,   0 for failure
'                            1 for success
'                           -1 for cancel connect box
'
'===========================================================================
Private Function getPoolConnection(emIsoLevel As isoLevel, _
                                   emSQLMode As sqlMode, _
                                   emAutoCommit As autoCommit, _
                                   bInitialConnect As Boolean, _
                                   ByRef oConnection As connection) As Long
    On Error GoTo errGetPoolConnection
    
    Const FUNCTION_NAME = "getPoolConnection"
    
    '=varibles============================================================================
    'handle for dbc class
    Dim lWaDBCHandle        As Long
    Dim hDbc                As Long
    Dim hEnv                As Long
    Dim lRc                 As Long
    Dim lError              As Long
    Dim lOut                As Long
    Dim bConnectError       As Boolean
    Dim sByteBuffer()       As Byte
    Dim ConnectStringIn     As String
    
    Dim bStartUtf8DS() As Byte
    Dim bStartUtf8Driver() As Byte
    Dim bStartUtf8DBServer() As Byte
    Dim bStartUtf8DB() As Byte
    Dim bStartUtf8UID() As Byte
    Dim bStartUtf8PWD() As Byte
    Dim bStartUtf8TF() As Byte

    '======================================================================================
    
    'rc
    getPoolConnection = 0
        
    'init
    bConnectError = False
    ReDim sByteBuffer(MX_DIAG_FIELD_OUT)
    
    If m_lPool = 0 Then
    
        MsgBox LoadResString(111)
        'set rc
        getPoolConnection = -1
        Exit Function
    End If
    
    'create a dbc
    lWaDBCHandle = sapdbwa_CreateDBC()
    If lWaDBCHandle = 0 Then
        
        MsgBox LoadResString(112)
        'set rc
        getPoolConnection = -1
        Exit Function
    End If
    
    If sapdbwa_DBCSetAttr(lWaDBCHandle, SQL_AUTOCOMMIT, emAutoCommit) = False Then

        showWAError lWaDBCHandle
        sapdbwa_DestroyDBC lWaDBCHandle
        'set rc
        getPoolConnection = -1
        Exit Function
    End If
    
    If m_lTimeOut > -1 Then
    
        If sapdbwa_DBCSetAttr(lWaDBCHandle, SQL_ATTR_SESSION_TIMEOUT, m_lTimeOut) = False Then
        
            showWAError lWaDBCHandle
            sapdbwa_DestroyDBC lWaDBCHandle
            'set rc
            getPoolConnection = -1
            Exit Function
        End If
    End If
    'if initial connect:
    'don't set connect attributes sqlmode and isolevel to get attributes from driver
    If Not bInitialConnect Then
    
        If sapdbwa_DBCSetAttr(lWaDBCHandle, SQL_SQLMODE, emSQLMode) = False Then
    
            showWAError lWaDBCHandle
            sapdbwa_DestroyDBC lWaDBCHandle
            Exit Function
        End If
    
        If sapdbwa_DBCSetAttr(lWaDBCHandle, SQL_TXN_ISOLATION, emIsoLevel) = False Then
    
            showWAError lWaDBCHandle
            sapdbwa_DestroyDBC lWaDBCHandle
            'set rc
            getPoolConnection = -1
            Exit Function
        End If
    Else
        'user forces sql mode in first connect
        If emSQLMode = Oracle Then
        
            If sapdbwa_DBCSetAttr(lWaDBCHandle, SQL_SQLMODE, emSQLMode) = False Then
        
                showWAError lWaDBCHandle
                sapdbwa_DestroyDBC lWaDBCHandle
                Exit Function
            End If
        End If
    End If
    
    '=convert from ucs2 to utf8==============================================
    bStartUtf8DS() = m_oComFunc.getUtf8FromString("")
    bStartUtf8Driver() = m_oComFunc.getUtf8FromString(m_sDriver)
    bStartUtf8DBServer() = m_oComFunc.getUtf8FromString(m_sDBServer)
    bStartUtf8DB() = m_oComFunc.getUtf8FromString(m_sDataBase)
    bStartUtf8UID() = m_oComFunc.getUtf8FromString(m_sUID)
    bStartUtf8PWD() = m_oComFunc.getUtf8FromString(m_sPWD)
    bStartUtf8TF() = m_oComFunc.getUtf8FromString(m_sTraceFile)
    '=========================================================================
    
    If sapdbwa_DBCConnect(m_lPool, _
                          lWaDBCHandle, _
                          bStartUtf8DS(0), _
                          bStartUtf8Driver(0), _
                          bStartUtf8DBServer(0), _
                          bStartUtf8DB(0), _
                          bStartUtf8UID(0), _
                          bStartUtf8PWD(0), _
                          bStartUtf8TF(0), _
                          SQL_DRIVER_COMPLETE, _
                          m_lWnd, _
                          m_lPoolingMode) = 1 Then

        hDbc = sapdbwa_DBCHandle(lWaDBCHandle)
        If hDbc = SQL_NULL_HDBC Then
        
            showWAError lWaDBCHandle
            'set rc
            getPoolConnection = -1
            sapdbwa_DestroyDBC lWaDBCHandle
            Exit Function
        End If
        
        hEnv = sapdbwa_DBCEnv(lWaDBCHandle)
        If hEnv = SQL_NULL_HENV Then
        
            showWAError lWaDBCHandle
            sapdbwa_DestroyDBC lWaDBCHandle
            'set rc
            getPoolConnection = -1
            Exit Function
        End If
    Else
        lError = sapdbwa_DBCGetErr(lWaDBCHandle)
        If sapdbwa_GetErrId(lError) <> WA_ERR_NO_ERROR Then bConnectError = True
    End If
        
    If bConnectError Then
                
        lRc = sapdbwa_GetNativeOdbcErr(lError)
        If lRc <> 0 Then
        
            showWAError lWaDBCHandle
            'gather some info
            handleConnectError sapdbwa_DBCHandle(lWaDBCHandle)
            'set rc

            'if it's the first connect set rc for loop
            If bInitialConnect Then
            
                'set rc
                getPoolConnection = 0
            Else
                getPoolConnection = -1
            End If
        Else
            lRc = sapdbwa_GetErrId(lError)
            If lRc <> 0 Then
            
                'showWAError lWaDBCHandle
            End If
            getPoolConnection = -1
        End If
            
        sapdbwa_DestroyDBC lWaDBCHandle
        Exit Function
    Else
        If hDbc = SQL_NULL_HDBC Then hDbc = sapdbwa_DBCHandle(lWaDBCHandle)
        If hDbc = SQL_NULL_HDBC Then
        
            showWAError lWaDBCHandle
            sapdbwa_DestroyDBC lWaDBCHandle
            Exit Function
        End If

        If hEnv = SQL_NULL_HENV Then hEnv = sapdbwa_DBCEnv(lWaDBCHandle)
        If hEnv = SQL_NULL_HENV Then
        
            showWAError lWaDBCHandle
            sapdbwa_DestroyDBC lWaDBCHandle
            Exit Function
        End If
        
        oConnection.waDBCPool = m_lPool
        'ser wa dbc hanlde for one connection object
        oConnection.waDBC = lWaDBCHandle
                
        'Scan uid
        sapdbwa_DBCGetUser lWaDBCHandle, sByteBuffer(0), UBound(sByteBuffer())
        m_sUID = m_oComFunc.getStringFromUtf8(sByteBuffer())
        
        'Scan pwd
        sapdbwa_DBCGetPassword lWaDBCHandle, sByteBuffer(0), UBound(sByteBuffer())
        m_sPWD = m_oComFunc.getStringFromUtf8(sByteBuffer())
                        
        'get server DB
        sapdbwa_DBCGetServerDb lWaDBCHandle, sByteBuffer(0), UBound(sByteBuffer())
        m_sDataBase = m_oComFunc.getStringFromUtf8(sByteBuffer())
        
        'get server node
        sapdbwa_DBCGetServerNode lWaDBCHandle, sByteBuffer(0), UBound(sByteBuffer())
        m_sDBServer = m_oComFunc.getStringFromUtf8(sByteBuffer())
        
        If sapdbwa_DBCGetAttr(lWaDBCHandle, SQL_TXN_ISOLATION, m_emDriverIsoLevel) = False Then
    
            showWAError lWaDBCHandle
            sapdbwa_DestroyDBC lWaDBCHandle
            Exit Function
        End If
        
        If sapdbwa_DBCGetAttr(lWaDBCHandle, SQL_SQLMODE, m_emDriverSQLMode) = False Then
    
            showWAError lWaDBCHandle
            sapdbwa_DestroyDBC lWaDBCHandle
            Exit Function
        End If
        
        'check if it is a ni connection
        lRc = SQLGetDiagFieldW(SQL_HANDLE_DBC, _
                               hDbc, _
                               0, _
                               SQL_DIAG_IS_NI_CONNECTION, _
                               sByteBuffer(0), _
                               MX_DIAG_FIELD_OUT, _
                               lOut)
        If lRc = SQL_SUCCESS Then
        
            ConnectStringIn = left$(sByteBuffer, lOut \ 2)
            If ConnectStringIn = "Y" Then
        
                m_bNI = True
            Else
                m_bNI = False
            End If
            oConnection.isNIConnection = m_bNI
        End If
        
        If bInitialConnect Then
        
            'before userdata get unicode info
            getUnicodeInfo oConnection
            
            'get user information
            If getUserInfo(oConnection) Then
            
                oConnection.userGroup = userGroup
            Else
                oConnection.userGroup = userGroup
            End If
        End If
        
        'set some properties
        oConnection.isoLevel = m_emDriverIsoLevel
        oConnection.sqlMode = m_emDriverSQLMode
        oConnection.autoCommit = emAutoCommit
        oConnection.userGroup = m_sUserGroup
        oConnection.unicodeDatabase = m_bUnicodeDatabase
    End If
    
    'rc
    getPoolConnection = 1
    
    Exit Function
    
errGetPoolConnection:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getStringFromZeroTermination
'    description:   return the a string out of a string with \0 termination
'    arguments:     [in] sIn, string to be examined
'
'    return value:  string with \0 termination
'
'=========================================================================================
Private Function getStringFromZeroTermination(ByRef sIn As String) As String
    On Error GoTo errGetStringFromZeroTermination
    
    Const FUNCTION_NAME = "getStringFromZeroTermination"

    '=varibles============================================================================
    Dim lPos As Long
    '======================================================================================
    
    'rc
    getStringFromZeroTermination = ""
    
    lPos = InStr(sIn, vbNullChar)
    If lPos > 0 Then getStringFromZeroTermination = left(sIn, lPos - 1)
    
    Exit Function
    
errGetStringFromZeroTermination:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getConnection
'    description:   gets a connection object
'    arguments:     [in] emIsoLevel, isolation level of database connection
'                   [in] emSqlMode, sqlMode mode of database connection
'                   [in] emAutoCommit, auto commit mode of database connection
'                   [out] oConnection, a new connection object
'
'    return value:  true, if success
'
'=========================================================================================
Public Function getConnection(emIsoLevel As isoLevel, _
                              emSQLMode As sqlMode, _
                              emAutoCommit As autoCommit, _
                              oConnection As connection) As Boolean
    On Error GoTo errGetConnection
    
    Const FUNCTION_NAME = "getConnection"
    
    '=varibles============================================================================
    Dim lReturn             As Long
    '======================================================================================
            
    'rc
    getConnection = False
    
    'init
    Set oConnection = New connection
    
    
    'if it's not the first connect for pool and there is no password cancel
    If m_bIsConnected And Len(m_sPWD) < 1 Then Exit Function
    'loop the connect box
    Do
        lReturn = getPoolConnection(emIsoLevel, emSQLMode, emAutoCommit, Not m_bIsConnected, oConnection)
        
    Loop While lReturn = 0 And (Not m_bNI)
    
    If lReturn = 1 Then
    
        'add new connection to collection
        m_colConnections.Add oConnection
         If m_bIsConnected = False Then m_bIsConnected = True
                            
        getConnection = True
    Else
        If lReturn = -1 Then
        
            'set the error property, no error, cancel button was pressed
            oConnection.error.isError = False
            oConnection.error.text = LoadResString(110)
        End If
        
        getConnection = False
    End If
        
    Exit Function

errGetConnection:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      handleConnectError
'    description:   called in getpoolconnection, if there is a error connection with wa pool
'    arguments:     handle for the dbc
'
'    return value:  /
'
'=========================================================================================
Private Function handleConnectError(hDbc As Long)
    On Error GoTo errHandleConnectError

    Const FUNCTION_NAME = "handleConnectError"
    
    '=varibles============================================================================
    Dim lOut             As Long
    Dim lRc              As Long
    Dim sByteBuffer()    As Byte
    Dim ConnectStringIn  As String
    '======================================================================================
           
    
    'init buffer
    ReDim sByteBuffer(MX_DIAG_FIELD_OUT)
    
    'check if it is a ni connection
    'Function SQLGetDiagFieldW is abused to get info if connection runs ni mode
    lRc = SQLGetDiagFieldW(SQL_HANDLE_DBC, _
                           hDbc, _
                           0, _
                           SQL_DIAG_IS_NI_CONNECTION, _
                           sByteBuffer(0), _
                           MX_DIAG_FIELD_OUT - 1, _
                           lOut)
    If lRc = SQL_SUCCESS Then

        ConnectStringIn = left$(sByteBuffer, lOut \ 2)
        If ConnectStringIn = "Y" Then

            m_bNI = True
        Else
            m_bNI = False
        End If
    End If
    
    'if connections is not in ni mode, get some connecgt params for redisplaying connect box
    If Not m_bNI Then
        
        m_sPWD = ""
        lRc = SQLGetDiagFieldW(SQL_HANDLE_DBC, _
                                hDbc, _
                                0, _
                                SQL_DIAG_CONNECT_STRING, _
                                sByteBuffer(0), _
                                MX_DIAG_FIELD_OUT, _
                                lOut)
        If lRc = SQL_SUCCESS Then
            ConnectStringIn = left$(sByteBuffer, lOut \ 2)
            'Scan uid
            getUID ConnectStringIn, m_sUID
            'get server DB
            getServerDB ConnectStringIn, m_sDataBase
            'get server node
            getServerNode ConnectStringIn, m_sDBServer
        End If
    End If
    
    Exit Function
    
errHandleConnectError:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      showWAError
'    description:   shows an error of the webagent dbc class
'    arguments:     [in] lWaDBCHandle, handle from wa dbc class
'
'    return value:  /
'
'=========================================================================================
Private Function showWAError(lWaDBCHandle As Long)
    On Error GoTo errShowWAError

    Const FUNCTION_NAME = "showWAError"

    '=varibles============================================================================
    Dim lError              As Long
    Dim sByteBuffer(2048)   As Byte
    '======================================================================================
    
    lError = sapdbwa_DBCGetErr(lWaDBCHandle)
    If sapdbwa_GetErrId(lError) <> WA_ERR_NO_ERROR Then

        sapdbwa_CopyErrMsg lError, sByteBuffer(0), UBound(sByteBuffer())
        MsgBox m_oComFunc.getStringFromUtf8(sByteBuffer()), vbCritical
        Exit Function
    End If
    
    Exit Function
    
errShowWAError:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getUserInfo
'    description:   get groupname, connectmode,usermode from user connected to database
'    arguments:     [in] oConnection, connection object to database
'
'    return value:  true if success
'
'=========================================================================================
Private Function getUserInfo(oConnection As connection) As Boolean
    On Error GoTo errGetUserInfo
    
    Const FUNCTION_NAME = "getUserInfo"
   
    '=varibles============================================================================
    Dim sUserMode        As String
    Dim sConnectMode     As String
    Dim oResult          As New Result
    '=====================================================================================
        
    'rc
    getUserInfo = False
    
    'reinit
    m_bIsResourceUser = False
    m_bNotExclusive = True
    m_enUsermode = umUnkown
    m_sUserGroup = m_sUID
    
    Set oResult.connection = oConnection
    
    'Open oResult
    If Not oResult.openResult("SELECT DOMAIN.USERS.GROUPNAME,DOMAIN.USERS.CONNECTMODE,DOMAIN.USERS.USERMODE FROM DOMAIN.USERS WHERE DOMAIN.USERS.USERNAME = USER", Internal) _
       Then Exit Function
        
    'If there is an error
    If oResult.noRows Then
        
        getUserInfo = True  'rc
        Exit Function
    End If
    
        'If there is an error
    If oResult.noRows Then
        
        getUserInfo = False
        Exit Function
    End If
    
    If Not oResult.fetch() Then
    
        getUserInfo = False
        Exit Function
    End If
    
    If Not oResult.getData() Then
    
        getUserInfo = False
        Exit Function
    End If
    
    m_sUserGroup = oResult.columns.Item(1).colValue
    sConnectMode = oResult.columns.Item(2).colValue
    sUserMode = oResult.columns.Item(3).colValue
    
    'if user group is null set uid
    If m_sUserGroup = cnstQUERY_NULL Or _
       Len(m_sUserGroup) < 1 Then m_sUserGroup = m_sUID
    
    If right$(m_sUserGroup, 1) = Chr(34) Then
    
        m_sUserGroup = left$(m_sUserGroup, Len(m_sUserGroup) - 1)
        If left(m_sUserGroup, 1) = Chr(34) Then _
           m_sUserGroup = right(m_sUserGroup, Len(m_sUserGroup) - 1)
    End If
        
    'Get connect Mode
    If sConnectMode <> "MULTIPLE" Then m_bNotExclusive = False
    
    'get usermode
    Select Case sUserMode
    
        Case "STANDARD"
            m_bIsResourceUser = False
            m_enUsermode = umSTANDARD
        Case "RESOURCE"
            m_bIsResourceUser = True
            m_enUsermode = umRESOURCE
        Case "DBA"
            m_bIsResourceUser = True
            m_enUsermode = umDBA
        Case "SYSDBA"
            m_bIsResourceUser = True
            m_enUsermode = umSYSDBA
        Case "COLDUSER"
            m_bIsResourceUser = False
            m_enUsermode = umCOLDUSER
    End Select
    
    Set oResult = Nothing
    
    getUserInfo = True
    
    Exit Function

errGetUserInfo:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      Disconnect
'    description:   disconnect all connection objects from database
'                   if pooling mode <> WA_NONE destroy connection pool of webagent
'    arguments:     /
'
'    return value:  true if success
'
'=========================================================================================
Public Function disconnect() As Boolean
    On Error GoTo ErrDisconnect
    
    Const FUNCTION_NAME = "Disconnect"
    
    Dim oConnection   As connection
    
    While m_colConnections.Count > 0
        
        Set oConnection = m_colConnections.Item(1)
        oConnection.disconnect
        m_colConnections.Remove 1
    Wend
    
    If m_lPool <> 0 Then

        sapdbwa_DestroyDBCPool m_lPool
        m_lPool = 0
        disconnect = True
    End If
   
    'set property
    m_bIsConnected = False
    
    'rc
    disconnect = True
    
    
    Exit Function
    
ErrDisconnect:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      class_Initialize
'    description:   Occurs when an application creates an instance of a Form, MDIForm, User control, Property Page, Webclass, DHTML Page Designer, or class.
'
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Private Sub Class_Initialize()
    On Error GoTo ErrClass_Initialize

    Const FUNCTION_NAME = "Class_Initialize"
    
    m_bNotExclusive = True
    m_bIsResourceUser = False
    m_enUsermode = umUnkown
    
    initConnectParams
    
    m_bNI = False
    m_lPool = 0
    m_lPoolingMode = WA_DBC_EXCL
    
    m_lTimeOut = -1
    
    m_bUnicodeDatabase = False
    
    m_sTraceFile = ""

    Exit Sub
    
ErrClass_Initialize:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      initConnectParams
'    description:   initialize some members concerning the connect parameters
'
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Private Function initConnectParams()
    On Error GoTo ErrInitConnectParams

    Const FUNCTION_NAME = "initConnectParams"
    
    m_sDriver = ""
    m_sUID = ""
    m_sPWD = ""
    m_sDataBase = ""
    m_sDBServer = ""
    
    Exit Function
    
ErrInitConnectParams:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function

'=========================================================================================
'    function:      getUnicodeInfo
'    description:   is db a unicode db
'    arguments:     [in] oConnection, connection object to database
'
'    return value:  true if success
'
'=========================================================================================
Private Function getUnicodeInfo(oConnection As connection) As Boolean
    On Error GoTo errGetUnicodeInfo
    
    Const FUNCTION_NAME = "getUnicodeInfo"
   
    '=local varibles============================================================================
    Dim oResult          As New Result
    '=====================================================================================
        
    'rc
    getUnicodeInfo = False
    
    Set oResult.connection = oConnection
    
    'init
    m_bUnicodeDatabase = False
    
    'Open oResult
    If Not oResult.openResult("SELECT SYSDBA.DBPARAMETERS.VALUE FROM SYSDBA.DBPARAMETERS WHERE SYSDBA.DBPARAMETERS.DESCRIPTION = '_UNICODE'", Internal) Then _
       Exit Function
       
    'If there is an error
    If oResult.noRows Then
        
        getUnicodeInfo = False
        Exit Function
    End If
    
    If Not oResult.fetch() Then
    
            getUnicodeInfo = False
            Exit Function
    End If
    
    If Not oResult.getData() Then
    
        getUnicodeInfo = False
        Exit Function
    End If
    
    If oResult.columns.Item(1).colValue = "YES" Then m_bUnicodeDatabase = True
    
    Set oResult = Nothing
    
    getUnicodeInfo = True
    
    Exit Function

errGetUnicodeInfo:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    property:      unicodeDatabase
'    description:   return if connected database is unicode
'    arguments:     /
'
'    return value:  true is connections object is running in NI Mode
'
'=========================================================================================
Public Property Get unicodeDatabase() As Boolean
    On Error GoTo errP_Get_UnicodeDatabase

    Const FUNCTION_NAME = "P_GET unicodeDatabase"

    unicodeDatabase = m_bUnicodeDatabase

    Exit Property

errP_Get_UnicodeDatabase:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      traceFile
'    description:   set TraceFile of connections object
'    arguments:     [in] snewTraceFile, new TraceFile
'
'    return value:  /
'
'=========================================================================================
Public Property Let traceFile(sNewTraceFile As String)
    On Error GoTo errP_Let_TraceFile

    Const FUNCTION_NAME = "P_Let traceFile"
    
    m_sTraceFile = sNewTraceFile
    
    Exit Property
    
errP_Let_TraceFile:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      traceFile
'    description:   return TraceFile of connections object
'    arguments:     /
'
'    return value:  TraceFile as string
'
'=========================================================================================
Public Property Get traceFile() As String
    On Error GoTo errP_Get_TraceFile

    Const FUNCTION_NAME = "P_Get traceFile"
    
    traceFile = m_sTraceFile
    
    Exit Property
    
errP_Get_TraceFile:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    function:      reconnect
'    description:   try to reconnect all valid connection objects
'    arguments:     /
'
'    return value:  true if success
'
'=========================================================================================
Public Function reconnect() As Boolean
    On Error GoTo errReconnect
    
    Const FUNCTION_NAME = "Reconnect"
    
    '=varibles============================================================================
    Dim iRun                 As Integer
    Dim oConnection          As connection
    '=====================================================================================
    
    'rc
    reconnect = True
    
    For iRun = 1 To m_colConnections.Count
        
        Set oConnection = m_colConnections.Item(iRun)
        oConnection.disconnect
    Next
    
    'create a new pool
    If m_lPool <> 0 Then

        sapdbwa_DestroyDBCPool m_lPool
        m_lPool = sapdbwa_CreateDBCPool(-1)
    End If
    
    For iRun = 1 To m_colConnections.Count
                
        Set oConnection = m_colConnections.Item(iRun)
        If getPoolConnection(oConnection.isoLevel, _
                        oConnection.sqlMode, _
                        oConnection.autoCommit, _
                        False, _
                        oConnection) = -1 _
        Then
            reconnect = False
        End If
    Next
        
    Exit Function

errReconnect:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    property:      poolHandle
'    description:   get pool handle
'    arguments:     /
'
'    return value:  pool handle as long
'
'=========================================================================================
Public Property Get poolHandle() As Long
    On Error GoTo errP_Get_PoolHandle

    Const FUNCTION_NAME = "P_Get poolHandle"
    
    poolHandle = m_lPool
    
    Exit Property
    
errP_Get_PoolHandle:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
