VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Functions"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' Module:      SQL Studio DB Layer
'
'-----------------------------------------------------------------------
'
' responsible:  WolfgangA
'
' special area: SQL Studio DB Layer
'
' description:  class for odbc function calls
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option Explicit

'module name
Const MODULE_NAME = "Functions"


Private m_bIsNIConnection     As Boolean

'private property variables
Private m_bAsyncExecution As Boolean
Private m_bStopExecute    As Boolean

Private m_lCallingWnd           As Long
Private m_colParamDefinitions   As New Collection

'object for error handling
'Private m_oError              As error
Private m_oConnection     As SQLStudioDBLayer.connection
'=========================================================================================
'    property:      connection
'    description:   set a sapdb connection object
'    arguments:     newConnection, connection object
'
'    return value:  /
'
'=========================================================================================
Public Property Set connection(newConnection As SQLStudioDBLayer.connection)
    On Error GoTo errP_SET_Connection

    Const FUNCTION_NAME = "P_SET Connection"
    
    Set m_oConnection = newConnection
    
    Exit Property
    
errP_SET_Connection:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      connection
'    description:   get a sapdb connection object
'    arguments:     /
'
'    return value:  sapdb connection object
'
'=========================================================================================
Public Property Get connection() As SQLStudioDBLayer.connection
    On Error GoTo errP_Get_Connection

    Const FUNCTION_NAME = "P_Get connection"
    
    Set connection = m_oConnection
    
    Exit Property
    
errP_Get_Connection:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      asyncExecution
'    description:   set if sql statements are executed asynchron
'    arguments:     [in] bNewAsyncExecution,flag to set asynchron execution
'
'    return value:  /
'
'=========================================================================================
Public Property Let asyncExecution(bNewAsyncExecution As Boolean)
    On Error GoTo errP_Let_AsyncExecution

    Const FUNCTION_NAME = "P_LET asyncExecution"
    
    m_bAsyncExecution = bNewAsyncExecution
    
    Exit Property
    
errP_Let_AsyncExecution:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      callingWindow
'    description:   handle of window using functions object
'    arguments:     [in] hCallingWnd, windows handle
'
'    return value:  /
'
'=========================================================================================
Public Property Let callingWindow(hCallingWnd As Long)
    On Error GoTo errP_Let_CallingWindow

    Const FUNCTION_NAME = "P_LET CallingWindow"
    
    m_lCallingWnd = hCallingWnd
    
    Exit Property
    
errP_Let_CallingWindow:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    function:      allocStmt
'    description:   allocate a statement handle
'    arguments:     [out] hStmt, statement handle
'
'    return value:  true if success
'
'=========================================================================================
Public Function allocStmt(hStmt As Long) As Boolean
    On Error GoTo errAllocStmt

    Const FUNCTION_NAME = "allocStmt"
    
    Dim lRc                As Integer
    'rc
    allocStmt = False
    'init
    hStmt = SQL_NULL_HSTMT
    
    If m_oConnection.hEnv = SQL_NULL_HENV Or _
       m_oConnection.hDbc = SQL_NULL_HDBC Then Exit Function
   
    ' Allocate storage for the statement
    lRc = SQLAllocStmt(m_oConnection.hDbc, hStmt)
    If lRc <> SQL_SUCCESS Then
    
        If Not m_oConnection.error.checkRC(lRc, hStmt) Then Exit Function
    End If
    
    'rc
    allocStmt = True

    Exit Function

errAllocStmt:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      cancelStmt
'    description:   cancel a running statement
'    arguments:     [in] hStmt, statement handle
'
'    return value:  true, if success
'
'=========================================================================================
Private Function cancelStmt(hStmt As Long) As Boolean
    On Error GoTo errCancelStmt

    Const FUNCTION_NAME = "cancelStmt"
    
    '=varibles============================================================================
    Dim lRc                As Integer
    Dim sOpt               As String
    Dim frmNi              As New frmNIString
    '=====================================================================================
    
    'rc
    cancelStmt = False
    
    If m_oConnection.hEnv = SQL_NULL_HENV Or _
       m_oConnection.hDbc = SQL_NULL_HDBC Then Exit Function
    
    If m_bIsNIConnection Then
    
        frmNi.Refresh
        frmNi.Show vbModal
        
        If frmNi.Ok Then
        
            sOpt = frmNi.niString
            'call a get option to set a value to receive the odbc driver manager
            lRc = pa81GetStrConnectOption(m_oConnection.hDbc, SQL_CANCEL_ROUTERSTRING, sOpt)
        Else
            Exit Function
        End If
    End If
        
    lRc = SQLCancel(hStmt)
    If lRc <> SQL_SUCCESS Then
    
        If Not m_oConnection.error.checkRC(lRc, hStmt) Then Exit Function
    End If
    
    Set frmNi = Nothing
    
    cancelStmt = True

    Exit Function

errCancelStmt:
    'sqlodext.dll if does not exist then it's not an NI LZU, so continue
    If err.Number = 53 Then Resume Next
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function

'=========================================================================================
'    function:      ExecDirectStmt
'    description:   execute a sql statement
'    arguments:     [in] hStmt, statement handle
'                   [in] sStatement, sql statement to execute
'
'    return value:  true if success
'
'=========================================================================================
Public Function execDirectStmt(ByVal hStmt As Long, ByVal sStatement As String) As Boolean
    On Error GoTo ErrExecDirectStmt

    Const FUNCTION_NAME = "ExecDirectStmt"
    
    '=local variables===================
    Dim bSQLCancel     As Boolean
    Dim bSQLCanceled   As Boolean
    Dim bCancelPermit  As Boolean
    Dim vKey           As Long
    Dim lMask          As Integer
    Dim lKeyState      As Integer
    Dim lRc            As Integer
    Dim bErrorHandled  As Boolean
    Dim bStatement()   As Byte
    Dim lStringLength  As Long
    '===================================
    
    'rc
    execDirectStmt = False
    
    If m_oConnection.hEnv = SQL_NULL_HENV Or _
       m_oConnection.hDbc = SQL_NULL_HDBC Then Exit Function
    
    'init
    lStringLength = 0
    lMask = &H8000
    bSQLCancel = False
    bSQLCanceled = False
    bCancelPermit = False
    bErrorHandled = False

    'execute command direct
    bStatement = sStatement + vbNullChar
    m_oConnection.error.statement = sStatement
    
    If m_lCallingWnd <> 0 Then InvalidateRect m_lCallingWnd, ByVal 0&, True
        
    If m_bAsyncExecution Then
        
        'lRc = SQLSetStmtOption(hStmt, SQL_ATTR_ASYNC_ENABLE, SQL_ASYNC_ENABLE_ON)
        lRc = SQLSetStmtAttrLong(hStmt, SQL_ATTR_ASYNC_ENABLE, SQL_ASYNC_ENABLE_ON, lStringLength)
        If lRc <> SQL_SUCCESS Then
        
            If Not m_oConnection.error.checkRC(lRc, hStmt) Then Exit Function
        End If
                  
        lRc = SQL_STILL_EXECUTING
        While lRc = SQL_STILL_EXECUTING
            
            'unicode functions that always return or take strings or
            'length arguments are passed as count-of-characters
            lRc = SQLExecDirectW(hStmt, bStatement(0), Len(sStatement))
            
            'is there a real db error
            If lRc <> SQL_SUCCESS And _
               lRc <> SQL_STILL_EXECUTING And _
               lRc <> SQL_NEED_DATA _
            Then
        
                If Not m_oConnection.error.checkRC(lRc, hStmt) Then
                
                    If frmSQLCancel.Visible Then frmSQLCancel.Hide
                    Exit Function
                End If
                bErrorHandled = True
            End If
            
            'give control to system
            Sleep (1)
                        
            If m_lCallingWnd <> 0 Then UpdateWindow m_lCallingWnd
            If bSQLCancel = False Then
            
                lKeyState = GetAsyncKeyState(VK_ESCAPE)
               
                If frmSQLCancel.Visible Then frmSQLCancel.Refresh
                If (lKeyState And lMask) <> 0 Then
                
                    If m_lCallingWnd <> 0 Then
                    
                        If m_lCallingWnd = GetForegroundWindow() Then bCancelPermit = True
                    Else
                        bCancelPermit = True
                    End If
                
                    If bCancelPermit Then
                        
                        bSQLCanceled = cancelStmt(hStmt)
                        If bSQLCanceled Then
                        
                            frmSQLCancel.setMessageText (LoadResString(104) + "  " + LoadResString(105))
                            frmSQLCancel.Show
                            frmSQLCancel.Refresh
                            bSQLCancel = True
                        End If
                    End If
                End If
            End If
        Wend
        
        bSQLCancel = False
        If frmSQLCancel.Visible Then frmSQLCancel.Hide
        
        If bErrorHandled = False Then
        
            If Not m_oConnection.error.checkRC(lRc, hStmt) Then Exit Function
        End If
    Else
        'unicode functions that always return or take strings or
        'length arguments are passed as count-of-characters
        lRc = SQLExecDirectW(hStmt, bStatement(0), Len(sStatement))
        'lRC = SQLExecDirectA(hStmt, sStatement, Len(sStatement))
        If lRc <> SQL_SUCCESS Then
        
            If Not m_oConnection.error.checkRC(lRc, hStmt) Then Exit Function
        End If
    End If

    execDirectStmt = Not bSQLCanceled

    Exit Function

ErrExecDirectStmt:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      PrepareStmt
'    description:   prepare a sql statement
'    arguments:     [in] hStmt, statement handle
'                   [in] sStatement, sql statement to prepare
'                   [in] emSQLMode, sql mode for prepare
'
'    return value:  true if success
'
'=========================================================================================
Public Function prepareStmt(ByVal hStmt As Long, _
                            ByVal sStatement As String, _
                            emSQLMode As sqlMode) As Boolean
    On Error GoTo errPrepareStmt

    Const FUNCTION_NAME = "prepareStmt"
       
    '=varibles============================================================================
    Dim lRc            As Integer
    Dim bStatement()   As Byte
    Dim lStringLength As Long
    '======================================================================================
    
    'rc
    prepareStmt = False
    
    If m_oConnection.hEnv = SQL_NULL_HENV Or _
       m_oConnection.hDbc = SQL_NULL_HDBC Then Exit Function
        
    'init
    lStringLength = 0
    bStatement = sStatement + vbNullChar
    m_oConnection.error.statement = sStatement
    
    'lRc = SQLSetStmtOption(hStmt, SQL_SQLMODE, emSQLMode)
    lRc = SQLSetStmtAttrLong(hStmt, SQL_SQLMODE, emSQLMode, lStringLength)
    If lRc <> SQL_SUCCESS Then
    
        If Not m_oConnection.error.checkRC(lRc, hStmt) Then
        
            Call commonFreeStmt(hStmt)
            Exit Function
        End If
    End If
    
    'unicode functions that always return or take strings or
    'length arguments are passed as count-of-characters
    lRc = SQLPrepareW(hStmt, bStatement(0), Len(sStatement))
    If lRc <> SQL_SUCCESS Then
    
        If Not m_oConnection.error.checkRC(lRc, hStmt) Then Exit Function
    End If
    
    prepareStmt = True

    Exit Function

errPrepareStmt:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      executeStmt
'    description:   execute a sql statement
'    arguments:     [in] hStmt, statement handle
'
'    return value:  true if success
'
'=========================================================================================
Public Function executeStmt(ByVal hStmt As Long) As Boolean
    On Error GoTo errExecuteStmt

    Const FUNCTION_NAME = "executeStmt"
    
    '=varibles============================================================================
    Dim bSQLCancel      As Boolean
    Dim bSQLCanceled    As Boolean
    Dim bCancelPermit   As Boolean
    Dim vKey            As Long
    Dim lMask           As Integer
    Dim lKeyState       As Integer
    Dim lRc             As Integer
    Dim bErrorHandled   As Boolean
    Dim lStringLength   As Long
    '=====================================================================================
              
    'rc
    executeStmt = False
    
    If m_oConnection.hEnv = SQL_NULL_HENV Or m_oConnection.hDbc = SQL_NULL_HDBC Then Exit Function
    
    'init
    lStringLength = 0
    lMask = &H8000
    bSQLCancel = False
    bSQLCanceled = False
    bCancelPermit = False
    bErrorHandled = False
    
    If m_lCallingWnd <> 0 Then InvalidateRect m_lCallingWnd, ByVal 0&, True
    
    If m_bAsyncExecution Then
        
        'lRc = SQLSetStmtOption(hStmt, SQL_ATTR_ASYNC_ENABLE, SQL_ASYNC_ENABLE_ON)
        lRc = SQLSetStmtAttrLong(hStmt, SQL_ATTR_ASYNC_ENABLE, SQL_ASYNC_ENABLE_ON, lStringLength)
        If lRc <> SQL_SUCCESS Then
        
            If Not m_oConnection.error.checkRC(lRc, hStmt) Then Exit Function
        End If
        
        lRc = SQL_STILL_EXECUTING
        While lRc = SQL_STILL_EXECUTING
        
            lRc = SQLExecute(hStmt)
            'is there a real db error
            If lRc <> SQL_SUCCESS And _
               lRc <> SQL_STILL_EXECUTING And _
               lRc <> SQL_NEED_DATA _
            Then
                If Not m_oConnection.error.checkRC(lRc, hStmt) Then
                
                    If frmSQLCancel.Visible Then frmSQLCancel.Hide
                    Exit Function
                End If
                bErrorHandled = True
            End If
            
            'give control to system
            Sleep (1)
            
            If m_lCallingWnd <> 0 Then UpdateWindow m_lCallingWnd
            
            If bSQLCancel = False Then
            
                lKeyState = GetAsyncKeyState(VK_ESCAPE)
                If frmSQLCancel.Visible Then frmSQLCancel.Refresh
                
                If (lKeyState And lMask) <> 0 Then
                
                    If m_lCallingWnd <> 0 Then
                    
                        If m_lCallingWnd = GetForegroundWindow() Then bCancelPermit = True
                    Else
                        bCancelPermit = True
                    End If
                
                    If bCancelPermit Then
                    
                        bSQLCanceled = cancelStmt(hStmt)
                        If bSQLCanceled Then
                            frmSQLCancel.setMessageText (LoadResString(104) + "  " + LoadResString(105))
                            frmSQLCancel.Show
                            frmSQLCancel.Refresh
                            bSQLCancel = True
                        End If
                    End If
                End If
           End If
        Wend
    
        bSQLCancel = False
        If frmSQLCancel.Visible Then frmSQLCancel.Hide
        
        If bErrorHandled = False Then
            
            If Not m_oConnection.error.checkRC(lRc, hStmt) Then
            
                If m_oConnection.error.rc <> SQL_NEED_DATA Then Exit Function
            End If
        End If
    Else
        lRc = SQLExecute(hStmt)
        If lRc <> SQL_SUCCESS Then
        
            If Not m_oConnection.error.checkRC(lRc, hStmt) Then
            
                If m_oConnection.error.rc <> SQL_NEED_DATA Then Exit Function
            End If
        End If
    End If

    executeStmt = Not bSQLCanceled

    Exit Function

errExecuteStmt:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      paramData
'    description:   get number of parameter
'    arguments:     [in] hStmt, statement handle
'                   [out] ParamNo,number of parameter
'
'
'    return value:  true if success
'
'=========================================================================================
Public Function paramData(ByVal hStmt As Long, ParamNo As Long) As Boolean
    On Error GoTo errParamData

    Const FUNCTION_NAME = "paramData"
    
    Dim lRc                As Integer

    'rc
    paramData = False
    
    If m_oConnection.hEnv = SQL_NULL_HENV Or _
       m_oConnection.hDbc = SQL_NULL_HDBC Then Exit Function

    'StatementHandle
    '[Input]
    'Statement handle.
    'ValuePtrPtr
    '[Output]
    'Pointer to a buffer in which to return
    'the address of the ParameterValuePtr buffer specified in SQLBindParameter (for parameter data) or the address of the TargetValuePtr buffer specified in SQLBindCol (for column data), as contained in the SQL_DESC_DATA_PTR descriptor record field.
    lRc = SQLParamData(hStmt, ParamNo)
    If lRc <> SQL_SUCCESS Then
    
        If Not m_oConnection.error.checkRC(lRc, hStmt) Then
        
            If m_oConnection.error.rc <> SQL_NEED_DATA Then Exit Function
        End If
    End If
    
    paramData = True

    Exit Function

errParamData:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      putBinaryData
'    description:   put binary data to columnn
'
'    arguments:     [in] hStmt, statement handle
'                   [in] DataToPut, data
'                   [in] lDatatLen, length of data
'
'    return value:  true if success
'
'=========================================================================================
Public Function putBinaryData(ByVal hStmt As Long, sDataToPut As String, lDataLen As Long) As Boolean
    On Error GoTo errPutBinaryData

    Const FUNCTION_NAME = "putBinaryData"
    
    '=local varibles=====================
    Dim lRc                As Integer
    Dim sByteBuffer()      As Byte
    '====================================

    'rc
    putBinaryData = False
    
    'sByteBuffer = sDataToPut
    
    If m_oConnection.hEnv = SQL_NULL_HENV Or _
       m_oConnection.hDbc = SQL_NULL_HDBC Then Exit Function

    'execute command
    lRc = SQLPutData(hStmt, sDataToPut, lDataLen)
    
    'call check routine to clear error object if everything was fine
    If Not m_oConnection.error.checkRC(lRc, hStmt) Then Exit Function
        
    putBinaryData = True

    Exit Function

errPutBinaryData:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      bindBlob
'    description:   bind a parameter for a long var binary object
'
'    arguments:     [in] hStmt, statement handle
'                   [in] lParamNum, number of param to bind
'                   [in] lParamTag, tag for param
'                   [in] lObjectLen, len of object to bind
'
'    return value:  true if success
'
'=========================================================================================
Public Function bindBlob(ByVal hStmt As Long, _
                                        lParamNum As Long, _
                                        lParamTag As Long, _
                                        lObjectLen As Long) As Boolean
    On Error GoTo errBindBlob

    Const FUNCTION_NAME = "bindBlob"
    
    Dim lRc                As Integer
    
    Dim TypeBinary          As Long
    Dim TypeLongVarBinary   As Long
    
    'init
    TypeBinary = -2
    TypeLongVarBinary = -4
    
    'rc
    bindBlob = False
    
    If m_oConnection.hEnv = SQL_NULL_HENV Or _
       m_oConnection.hDbc = SQL_NULL_HDBC Then Exit Function
    
    
    lRc = SQLBindParameter(hStmt, _
                           lParamNum, _
                           SQL_PARAM_INPUT, _
                           TypeBinary, _
                           TypeLongVarBinary, _
                           0, _
                           0, _
                           ByVal lParamTag, _
                           0, _
                           lObjectLen)

    If lRc <> SQL_SUCCESS Then
    
        If Not m_oConnection.error.checkRC(lRc, hStmt) Then Exit Function
    End If

    bindBlob = True

    Exit Function

errBindBlob:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      bindLongObjects
'    description:   bind a parameter for a long object
'
'    arguments:     [in] hStmt, statement handle
'                   [in] lParamNum, number of param to bind
'                   [in] lParamTag, tag for param
'                   [in] lObjectLen(), len of object to bind
'
'    return value:  true if success
'
'=========================================================================================
Public Function bindLongObjects(ByVal hStmt As Long, _
                                lParamNum As Long, _
                                lParamTag As Long, _
                                lObjectLen() As Long) As Boolean
    On Error GoTo errBindLongObjects

    Const FUNCTION_NAME = "bindLongObjects"
    
    Dim lRc                As Integer
    
    Dim TypeBinary          As Long
    Dim TypeLongVarBinary   As Long
    
    'init
    TypeBinary = -2
    TypeLongVarBinary = -4
    
    'rc
    bindLongObjects = False
    
    If (UBound(lObjectLen)) < lParamNum Then Exit Function
    lRc = SQLBindParameter(hStmt, _
                           lParamNum, _
                           1, _
                           TypeBinary, _
                           TypeLongVarBinary, _
                           0, _
                           0, _
                           ByVal lParamTag, _
                           0, _
                           lObjectLen(lParamNum))

    If lRc <> SQL_SUCCESS Then
    
        If Not m_oConnection.error.checkRC(lRc, hStmt) Then Exit Function
    End If

    bindLongObjects = True

    Exit Function

errBindLongObjects:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      bindLongObjects
'    description:   bind a parameter for a long object
'
'    arguments:     [in] hStmt, statement handle
'                   [in] lParamNum, number of param to bind
'                   [in] lParamTag, tag for param
'                   [in] lCDataType,the C data type of the parameter
'                   [in] SQLDataType,the SQL data type of the parameter
'                   [in] lObjectLen(), len of object to bind
'
'    return value:  true if success
'
'=========================================================================================
Public Function bindLongColumns(ByVal hStmt As Long, _
                            lParamNum As Long, _
                            lParamTag As Long, _
                            lSQLDataType As Long, _
                            lObjectLen() As Long) As Boolean
    On Error GoTo errBindLongObjects

    Const FUNCTION_NAME = "bindLongObjects"
    
    '=variables===================================================
    Dim lRc                As Integer
    Dim lCDataType         As Long
    '====================================================
        
    'rc
    bindLongColumns = False
    
    
    'determine a valid c_data_type from the sql data type
    'see article "Converting Data from C to SQL Data Types" in MSDN
    'default conversion are user
    'core column types
    Select Case lSQLDataType
        Case odbcColTypeChar, odbcColTypeVarChar, odbcColTypeLongVarChar
            lCDataType = odbcColTypeChar
        Case odbcColTypeWChar, odbcColTypeWVarChar, odbcColTypeWLongVarChar
            lCDataType = odbcColTypeWChar
        Case odbcColTypeBinary, odbcColTypeVarBinary, odbcColTypeLongVarBinary
            lCDataType = odbcColTypeBinary
        Case Else
            Exit Function
    End Select
    
    If (UBound(lObjectLen)) < lParamNum Then Exit Function
    lRc = SQLBindParameter(hStmt, _
                           lParamNum, _
                           1, _
                           lCDataType, _
                           lSQLDataType, _
                           0, _
                           0, _
                           ByVal lParamTag, _
                           0, _
                           lObjectLen(lParamNum))

    If lRc <> SQL_SUCCESS Then
    
        If Not m_oConnection.error.checkRC(lRc, hStmt) Then Exit Function
    End If

    bindLongColumns = True

    Exit Function

errBindLongObjects:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      setPosition
'    description:   set the cursor position in a rowset
'
'    arguments:     [in] hStmt, statement handle
'                   [in] lPosition, position in rowset
'
'    return value:  true if success
'
'=========================================================================================
Public Function setPosition(ByVal hStmt As Long, lPosition As Long) As Boolean
    On Error GoTo errSetPosition
    
    Const FUNCTION_NAME = "setPosition"
    
    Dim lRc                As Integer
    
    'rc
    setPosition = False
    
    If m_oConnection.hEnv = SQL_NULL_HENV Or m_oConnection.hDbc = SQL_NULL_HDBC Then Exit Function
    
   'SQLSetPos sets the cursor position in a rowset and allows an
   'application to refresh data in the rowset or to update or delete
   'data in the result set.
    lRc = SQLSetPos(hStmt, lPosition, SQL_POSITION, SQL_LOCK_NO_CHANGE)
    If lRc <> SQL_SUCCESS Then
        If Not m_oConnection.error.checkRC(lRc, hStmt) Then
        
           m_oConnection.error.showError
           Exit Function
        End If
    End If
    
    setPosition = True

    Exit Function

errSetPosition:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      fullExecDirectStmt
'    description:   execute a sql statement
'    arguments:     [in] sStatement, sql statement to execute
'                   [in] emSQLMode, sql mode for execute
'
'    return value:  true if success
'
'=========================================================================================
Function fullExecDirectStmt(ByVal statement As String, _
                            emSQLMode As sqlMode) As Boolean
    On Error GoTo errFullExecDirectStmt

    Const FUNCTION_NAME = "fullExecDirectStmt"

    Dim hStmt           As Long
    Dim lRc             As Integer
    Dim lStringLength   As Long

    'rc
    fullExecDirectStmt = False
    
    If m_oConnection.hEnv = SQL_NULL_HENV Or m_oConnection.hDbc = SQL_NULL_HDBC Then Exit Function
    
    'init
    lStringLength = 0
    
    'allocate handle for statement
    If Not allocStmt(hStmt) Then Exit Function
        
    'lRc = SQLSetStmtOption(hStmt, SQL_SQLMODE, emSQLMode)
    lRc = SQLSetStmtAttrLong(hStmt, SQL_SQLMODE, emSQLMode, lStringLength)
    If lRc <> SQL_SUCCESS Then
    
        If Not m_oConnection.error.checkRC(lRc, hStmt) Then
        
            Call commonFreeStmt(hStmt)
            Exit Function
        End If
    End If
    
    ' Execute command direct
    m_oConnection.error.statement = statement
    If Not execDirectStmt(hStmt, statement) Then
    
        Call commonFreeStmt(hStmt)
        Exit Function
    End If

    'free statement
    Call commonFreeStmt(hStmt)

    'rc
    fullExecDirectStmt = True

    Exit Function

errFullExecDirectStmt:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      fullExecuteStmt
'    description:   execute a sql statement
'    arguments:     [in] sStatement, sql statement to execute
'                   [in] emSQLMode, sql mode for execute
'
'    return value:  true if success
'
'=========================================================================================
Public Function fullExecuteStmt(ByVal statement As String, emSQLMode As sqlMode) As Boolean
    On Error GoTo errFullExecuteStmt

    Const FUNCTION_NAME = "fullExecuteStmt"
    
    Dim hStmt           As Long
    Dim lRc                As Integer
    
    'rc
    fullExecuteStmt = False
    
    If m_oConnection.hEnv = SQL_NULL_HENV Or _
       m_oConnection.hDbc = SQL_NULL_HDBC Then Exit Function
    
    'allocate handle for statement
    If Not allocStmt(hStmt) Then Exit Function
    
    'prepare statement
    m_oConnection.error.statement = statement
    If Not prepareStmt(hStmt, statement, emSQLMode) Then
    
        Call commonFreeStmt(hStmt)
        Exit Function
    End If

    'execute statement
    If Not executeStmt(hStmt) Then
    
        Call commonFreeStmt(hStmt)
        Exit Function
    End If

    'free statement
    Call commonFreeStmt(hStmt)
    
    'rc
    fullExecuteStmt = True

    Exit Function

errFullExecuteStmt:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      fullParameterExecute
'    description:   execute a sql statement
'    arguments:     [in] sStatement, sql statement to execute
'                   [in] emSQLMode, sql mode for execute
'
'    return value:  true if success
'
'=========================================================================================
Public Function fullParameterExecute(ByVal statement As String, emSQLMode As sqlMode) As Boolean
    On Error GoTo errFullParameterExecute
    
    Const FUNCTION_NAME = "fullParameterExecute"
    
    Dim hStmt           As Long
    Dim lRc             As Integer
    Dim lStringLength   As Long
    
    'rc
    fullParameterExecute = False
    
    If m_oConnection.hEnv = SQL_NULL_HENV Or _
       m_oConnection.hDbc = SQL_NULL_HDBC Then Exit Function
    
    'init
    lStringLength = 0
    
    'allocate handle for statement
    If Not allocStmt(hStmt) Then Exit Function
        
    'lRc = SQLSetStmtOption(hStmt, SQL_SQLMODE, emSQLMode)
    lRc = SQLSetStmtAttrLong(hStmt, SQL_SQLMODE, emSQLMode, lStringLength)
    If lRc <> SQL_SUCCESS Then
    
        If Not m_oConnection.error.checkRC(lRc, hStmt) Then
        
            Call commonFreeStmt(hStmt)
            Exit Function
        End If
    End If
    
    'prepare statement
    m_oConnection.error.statement = statement
    If Not prepareForParamaters(hStmt, statement) Then
    
        Call commonFreeStmt(hStmt)
        Exit Function
    End If

    'execute statement
    If Not executeStmt(hStmt) Then
    
        Call commonFreeStmt(hStmt)
        Exit Function
    End If

    'free statement
    Call commonFreeStmt(hStmt)
    
    'rc
    fullParameterExecute = True

    Exit Function

errFullParameterExecute:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      class_Initialize
'    description:   Occurs when an application creates an instance of a Form, MDIForm, User control, Property Page, Webclass, DHTML Page Designer, or class.
'
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Private Sub Class_Initialize()
    On Error GoTo ErrClass_Initialize

    Const FUNCTION_NAME = "Class_Initialize"

    m_bAsyncExecution = True
    m_bStopExecute = False
    m_lCallingWnd = 0
    
    m_bIsNIConnection = False
    
    Exit Sub

ErrClass_Initialize:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      PrepareForParamaters
'    description:   prepare a sql statement with parameters
'    arguments:     [in] hStmt, statement handle
'                   [in] sStatement, sql statement to prepare
'                   [in] BoundParameter, bound parameter
'
'    return value:  true if success
'
'=========================================================================================
Private Function prepareForParamaters(ByVal hStmt As Long, _
                                      ByVal sStatement As String, _
                                      Optional ByVal boundParameter As Boolean = True) As Boolean
    On Error GoTo errPrepareForParamaters

    Const FUNCTION_NAME = "prepareForParamaters"
    
    '=varibles============================================================================
    Dim lRc            As Integer
    Dim bStatement()   As Byte
    Dim lStringLength  As Long
    '======================================================================================

    'rc
    prepareForParamaters = False
        
    If hStmt = SQL_NULL_HSTMT Then Exit Function
    If m_oConnection.hDbc = SQL_NULL_HDBC Then Exit Function
    
    'init
    lStringLength = 0
    
    'prepare statement
    'lRc = SQLSetStmtOption(hStmt, SQL_PASS_THROUGH, 1)
    lRc = SQLSetStmtAttrLong(hStmt, SQL_PASS_THROUGH, 1, lStringLength)
    If lRc <> SQL_SUCCESS Then
    
        m_oConnection.error.checkRC lRc, hStmt
        Exit Function
    End If
    
    bStatement = sStatement + vbNullChar
    'unicode functions that always return or take strings or
    'length arguments are passed as count-of-characters
    lRc = SQLPrepareW(hStmt, bStatement(0), Len(sStatement))
    If lRc <> SQL_SUCCESS Then
    
        m_oConnection.error.checkRC lRc, hStmt
        Exit Function
    End If
    
    'get parameter description
    If Not getParameterDescription(hStmt, boundParameter) Then
    
         commonFreeStmt hStmt
        Exit Function
    End If
    
    'bind parameters
    If Not bindParameters() Then
    
         commonFreeStmt hStmt
        Exit Function
    End If
    
    'rc
    prepareForParamaters = True

    Exit Function

errPrepareForParamaters:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getParameterDescription
'    description:   Get the parameter description of a specific parameter
'                   and add it to the parameter collection m_colParameterDefinitions
'
'    arguments:     [in] hStmt, statement handle
'                   [in] BoundParameter, bound parameter
'
'    return value:  true if success
'
'=========================================================================================
Private Function getParameterDescription(hStmt As Long, _
                                         ByVal boundParameter As Boolean) As Boolean
    On Error GoTo errGetParameterDescription
    
    Const FUNCTION_NAME = "getParameterDescription"
    
    '=varibles============================================================================
    Dim lRc                     As Long
    Dim newParamDefinitions     As New Collection
    Dim ParameterCount          As Integer
    Dim i                       As Integer
    Dim ColType                 As Integer
    Dim colLength               As Long
    Dim colDecimal              As Integer
    Dim colNullable             As Integer
    Dim AbsColType              As Long
    '====================================================================================
    
    'rc
    getParameterDescription = False
    
    'clean
    While newParamDefinitions.Count > 0
    
        newParamDefinitions.Remove 1
    Wend
    
    If hStmt = SQL_NULL_HSTMT Then Exit Function
    If m_oConnection.hDbc = SQL_NULL_HDBC Then Exit Function
    
    lRc = SQLNumParams(hStmt, ParameterCount)
    If lRc <> SQL_SUCCESS Then
        m_oConnection.error.checkRC lRc, hStmt
        Exit Function
    End If
    
    For i = 1 To ParameterCount
        lRc = SQLDescribeParam(hStmt, i, ColType, colLength, colDecimal, colNullable)
        If lRc <> SQL_SUCCESS Then
            m_oConnection.error.checkRC lRc, hStmt
            Exit Function
        End If
        
        Dim newParamDefinition As New Parameter
        
        newParamDefinition.statementHandle = hStmt
        newParamDefinition.boundParameter = boundParameter
        newParamDefinition.parameterNo = i
        newParamDefinition.valueLength = SQL_DATA_AT_EXEC
        
        ' -99 <= ColType >= +99 then parameter type is input
        ' -9900 <= ColType >= -100 Or 100 <= ColType >= 9900 then parameter type is output
        ' -10099 <= ColType >= -10001 Or 10001 <= ColType >= 10099 then parameter type is input/output
        newParamDefinition.parameterType = ptUnknown
        AbsColType = Abs(ColType)
        If AbsColType > 99 Then
          If AbsColType > 9900 Then
            newParamDefinition.parameterType = ptInOut
            If ColType < 0 Then
              newParamDefinition.columnType = ColType + 10000
            Else
              newParamDefinition.columnType = ColType - 10000
            End If
          Else
          newParamDefinition.parameterType = ptOut
            newParamDefinition.columnType = ColType / 100
          End If
        Else
          newParamDefinition.parameterType = ptIn
          newParamDefinition.columnType = ColType
        End If
                        
        newParamDefinitions.Add newParamDefinition
        Set newParamDefinition = Nothing
    Next
    
    'rc
    Set m_colParamDefinitions = newParamDefinitions
    
    getParameterDescription = True
    
    Exit Function

errGetParameterDescription:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      bindParameters
'    description:   bind all parameters of parameter collection
'
'    arguments:     /
'
'    return value:  true if success
'
'=========================================================================================
Private Function bindParameters() As Boolean
    On Error GoTo ErrBindParameters
    
    Const FUNCTION_NAME = "bindParameters"
    
    Dim CurrentParam  As Parameter
    
    'inits
    bindParameters = False
    
    For Each CurrentParam In m_colParamDefinitions
        If Not CurrentParam.bindParameter Then Exit Function
    Next
    
    'rc
    bindParameters = True
    
    Exit Function

ErrBindParameters:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      parameterDefinitions
'    description:   get parameter collection
'
'    arguments:     /
'
'    return value:  parameter collection
'
'=========================================================================================
Public Property Get parameterDefinitions() As Collection
    On Error GoTo err_PGET_ParameterDefinitions
    
    Const FUNCTION_NAME = "Property Get parameterDefinitions"
    
    Set parameterDefinitions = m_colParamDefinitions
    
    Exit Property

err_PGET_ParameterDefinitions:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    function:      numberOfCols
'    description:   number of columns, if a db result exists
'
'    arguments:     [in] hStmt, statement handle
'
'    return value:  see description
'
'=========================================================================================
Public Function numberOfCols(ByVal hStmt As Long) As Integer
    On Error GoTo errNumberOfCols
    
    Const FUNCTION_NAME = "numberOfCols"
    
    Dim rc        As Long
    Dim nColCount As Integer
    
    numberOfCols = -1
    
    rc = SQLNumResultCols(hStmt, nColCount)
    
    If rc <> SQL_SUCCESS Then
        If Not m_oConnection.error.checkRC(rc, hStmt) Then Exit Function
    End If
    
    numberOfCols = nColCount
    
    Exit Function

errNumberOfCols:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getColumnDescription
'    description:   columns description of a column
'    arguments:     [in] hStmt, statement handle
'                   [in] index, column index
'                   [out] sColName, name of column
'                   [out] nLen, total number of bytes (excluding the null-termination byte) of colname
'                   [out] sType, data type of the column
'                   [out] nLength, the size of the column on the data source.
'                   [out] nDecimal,return the number of decimal digits of the column on the data source
'                   [out] bNotNull, indicates whether the column allows NULL values
'
'    return value:  true if success
'
'=========================================================================================
Public Function getColumnDescription(ByVal hStmt As Long, _
                                     index As Integer, _
                                     sColName As String, _
                                     nLen As Integer, _
                                     sType As String, _
                                     nLength As Long, _
                                     nDecimal As Integer, _
                                     bNotNull As Boolean) As Boolean
    On Error GoTo errGetColumnDescription
    
    Const FUNCTION_NAME = "getColumnDescription"
    
    '=local variables======================
    Dim rc        As Long
    Dim nColType  As Integer
    Dim nNotNull  As Integer
    Dim bColName() As Byte
    '=======================================
    
    'rc
    getColumnDescription = False
    
    'init
    ReDim bColName(MX_COLUMN_NAME * 2)
    rc = SQLDescribeColW(hStmt, _
                        index, _
                        bColName(0), _
                        MX_COLUMN_NAME, _
                        nLen, _
                        nColType, _
                        nLength, _
                        nDecimal, _
                        nNotNull)
    If rc <> SQL_SUCCESS Then
        If Not m_oConnection.error.checkRC(rc, hStmt) Then Exit Function
    End If
    
    sColName = left$(bColName, nLen)
    
    Select Case nColType
        Case 1: sType = "CHAR"
        Case 2: sType = "DECIMAL"
        Case 3: sType = "FIXED"
        Case 4: sType = "INTEGER"
        Case 5: sType = "SMALLINT"
        Case 6: sType = "FLOAT"
        Case 7: sType = "REAL"
        Case 8: sType = "DOUBLE PRECISION"
        Case 9: sType = "DATE"
        Case 10: sType = "TIME"
        Case 11: sType = "TIMESTAMP"
        Case 12: sType = "VARCHAR"
        Case -1: sType = "LONG"
        Case -2: sType = "CHAR() BYTE"
        Case -3: sType = "VARCHAR() BYTE"
        Case -4: sType = "LONG BYTE"
        Case -7: sType = "BOOLEAN"
        Case -8: sType = "CHAR UNICODE"
        Case -9: sType = "VARCHAR() UNICODE"
        Case -10: sType = "LONG UNICODE"
    End Select
    
    If nNotNull > 0 Then
    
        bNotNull = False
    Else
        bNotNull = True
    End If
    
    getColumnDescription = True
    
    Exit Function

errGetColumnDescription:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function       freeStmt
'    description:   free statement handle
'    arguments:     [in] hStmt, statement handle
'
'    return value:  true if success
'
'=========================================================================================
Public Function freeStmt(hStmt As Long)
    On Error GoTo errFreeStmt

    Const FUNCTION_NAME = "freeStmt"
    
     commonFreeStmt (hStmt)

    Exit Function

errFreeStmt:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
