VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Parameter"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' Class:      parameter
'
'
'-----------------------------------------------------------------------
'
' responsible:  WolfgangA
'
' special area: SQL Studio DB Layer
'
' description:  class for parameter handling in sql statements
'
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option Explicit

'module name
Const MODULE_NAME = "Parameter"

Private Const MX_BYTEVALUE = 32000

' emerations
Public Enum parameterType
  ptUnknown = 0
  ptIn = 1
  ptInOut = 2
  ptResultCol = 3
  ptOut = 4
  ptReturnValue = 5
End Enum

'object for error handling
Private m_oError              As New error

' Properties
Private m_lStatementHandle          As Long
Private m_bBoundParameter           As Boolean
Private m_lParameterNo              As Long
Private m_enParameterType           As parameterType
Private m_nColumnType               As Integer
Private m_sColumnName               As String
Private m_sValue                    As String
Private m_byByteValue(MX_BYTEVALUE) As Byte
Private m_lValueLength              As Long

Private m_oConnection                As SQLStudioDBLayer.connection
'=========================================================================================
'    property:      statementHandle
'    description:   set a statement handle
'    arguments:     NewStatementHandle, statement handle
'
'    return value:  /
'
'=========================================================================================
Friend Property Let statementHandle(NewStatementHandle As Long)
    On Error GoTo errP_Let_StatementHandle

    Const FUNCTION_NAME = "Property Let statementHandle"
    
    m_lStatementHandle = NewStatementHandle
    
    Exit Property
    
errP_Let_StatementHandle:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      poundParameter
'    description:   set bound parameter
'    arguments:     newBoundParameter, bound parameter
'
'    return value:  /
'
'=========================================================================================
Friend Property Let boundParameter(newBoundParameter As Boolean)
    On Error GoTo errP_Let_BoundParameter

    Const FUNCTION_NAME = "Property Let boundParameter"
    
    m_bBoundParameter = newBoundParameter
    
    Exit Property
    
errP_Let_BoundParameter:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      boundParameter
'    description:   get bound parameter
'    arguments:     bound parameter
'
'    return value:  /
'
'=========================================================================================
Friend Property Get boundParameter() As Boolean
    On Error GoTo errP_Get_BoundParameter

    Const FUNCTION_NAME = "Property Get boundParameter"
    
    boundParameter = m_bBoundParameter
    
    Exit Property
    
errP_Get_BoundParameter:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      parameterNo
'    description:   let number of parameter
'    arguments:     [in] newParameterNo, number of parameter
'
'    return value:  /
'
'=========================================================================================
Friend Property Let parameterNo(newParameterNo As Long)
    On Error GoTo errP_Let_ParameterNo

    Const FUNCTION_NAME = "Property Let parameterNo"
    
    m_lParameterNo = newParameterNo
    
    Exit Property
    
errP_Let_ParameterNo:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      parameterNo
'    description:   get number of parameter
'    arguments:     /
'
'    return value:  number of parameter
'
'=========================================================================================
Public Property Get parameterNo() As Long
    On Error GoTo errP_Get_ParameterNo

    Const FUNCTION_NAME = "Property Get parameterNo"
    
    parameterNo = m_lParameterNo
    
    Exit Property
    
errP_Get_ParameterNo:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      parameterType
'    description:   Let type of parameter
'    arguments:     [in] newParameterType, type of parameter
'
'    return value:  /
'
'=========================================================================================
Friend Property Let parameterType(newParameterType As parameterType)
    On Error GoTo errP_Let_ParameterType

    Const FUNCTION_NAME = "Property Let parameterType"
    
    m_enParameterType = newParameterType
    
    Exit Property
    
errP_Let_ParameterType:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      parameterType
'    description:   Get type of parameter
'    arguments:     /
'
'    return value:  type of parameter
'
'=========================================================================================
Public Property Get parameterType() As parameterType
    On Error GoTo errP_Get_ParameterType

    Const FUNCTION_NAME = "Property Get parameterType"
    
    parameterType = m_enParameterType
    
    Exit Property
    
errP_Get_ParameterType:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      columnType
'    description:   Let type of column
'    arguments:     [in] newColumnType, type of column
'
'    return value:  /
'
'=========================================================================================
Friend Property Let columnType(newColumnType As Integer)
    On Error GoTo errP_Let_ColumnType

    Const FUNCTION_NAME = "Property Let ColumnType"
    
    m_nColumnType = newColumnType
    
    Exit Property
    
errP_Let_ColumnType:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      columnType
'    description:   Get type of column
'    arguments:     /
'
'    return value:  type of column
'
'=========================================================================================
Public Property Get columnType() As Integer
    On Error GoTo errP_Get_ColumnType

    Const FUNCTION_NAME = "Property Get columnType"
    
    columnType = m_nColumnType
    
    Exit Property
    
errP_Get_ColumnType:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      value
'    description:   let value of parameter
'    arguments:     [in] newValue, value of parameter
'
'    return value:  /
'
'=========================================================================================
Public Property Let Value(ByVal newValue As String)
    On Error GoTo errP_Let_Value

    Const FUNCTION_NAME = "Property Let value"
    
    Dim i As Long
    
    m_lValueLength = Len(newValue)
    m_sValue = newValue
    
    i = 0
    While i < m_lValueLength And i < MX_BYTEVALUE
        m_byByteValue(i) = Asc(Mid(newValue, i + 1, 1))
        i = i + 1
    Wend
    
    m_byByteValue(m_lValueLength) = 0
    
    Exit Property
    
errP_Let_Value:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      value
'    description:   get value of parameter
'    arguments:     /
'
'    return value:  value of parameter
'
'=========================================================================================
Public Property Get Value() As String
  On Error GoTo errP_Get_Value

  Const FUNCTION_NAME = "Property Get value"

  Dim i As Long

  i = 0
  m_sValue = ""

  Select Case m_nColumnType
    Case odbcColTypeBinary, odbcColTypeLongVarBinary, odbcColTypeVarBinary:
      While i < m_lValueLength And i < UBound(m_byByteValue)
        m_sValue = m_sValue + Chr$(m_byByteValue(i))
        i = i + 1
      Wend

    Case Else:
      While m_byByteValue(i) <> 0 And i < UBound(m_byByteValue)
        m_sValue = m_sValue + Chr$(m_byByteValue(i))
        i = i + 1
      Wend
  End Select

  Value = m_sValue

  Exit Property

errP_Get_Value:
  internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      valueLength
'    description:   let value length of parameter
'    arguments:     [in] newValueLength, value length of parameter
'
'    return value:  /
'
'=========================================================================================
Friend Property Let valueLength(newValueLength As Long)
    On Error GoTo errP_Let_ValueLength

    Const FUNCTION_NAME = "Property Let valueLength"
    
    m_lValueLength = newValueLength
    
    Exit Property
    
errP_Let_ValueLength:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      valueLength
'    description:   get value length of parameter
'    arguments:     /
'
'    return value:  value length of parameter
'
'=========================================================================================
Public Property Get valueLength() As Long
    On Error GoTo errP_Get_ValueLength

    Const FUNCTION_NAME = "Property Get valueLength"
    
    valueLength = m_lValueLength
    
    Exit Property
    
errP_Get_ValueLength:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      columnName
'    description:   let column name parameter
'    arguments:     [in] newColumnName, column name of parameter
'
'    return value:  /
'
'=========================================================================================
Friend Property Let columnName(newColumnName As String)
    On Error GoTo errP_Let_ColumnName

    Const FUNCTION_NAME = "Property Let ColumnName"
    
    m_sColumnName = newColumnName
    
    Exit Property
    
errP_Let_ColumnName:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      columnName
'    description:   Get column name parameter
'    arguments:     /
'
'    return value:  column name of parameter
'
'=========================================================================================
Friend Property Get columnName() As String
    On Error GoTo errP_Get_ColumnName

    Const FUNCTION_NAME = "Property Get columnName"
    
    columnName = m_sColumnName
    
    Exit Property
    
errP_Get_ColumnName:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      bindParameter
'    description:   bind parameter
'    arguments:     /
'
'    return value:  true if success
'
'=========================================================================================
Friend Function bindParameter() As Boolean
    On Error GoTo errBindParameter

    Const FUNCTION_NAME = "bindParameter"

    Dim lRc          As Long

    'rc
    bindParameter = False

    If m_lStatementHandle = SQL_NULL_HSTMT Then Exit Function
    
    If m_bBoundParameter Or _
       m_enParameterType = ptInOut Or _
       m_enParameterType = ptOut Then
      Select Case m_nColumnType
          Case odbcColTypeBinary, odbcColTypeLongVarBinary, odbcColTypeVarBinary:
              lRc = SQLBindParameter(m_lStatementHandle, m_lParameterNo, m_enParameterType, _
                                      odbcColTypeBinary, m_nColumnType, 0, 0, m_byByteValue(0), MX_BYTEVALUE, m_lValueLength)
          Case Else:
              m_lValueLength = SQL_NTS
              lRc = SQLBindParameter(m_lStatementHandle, m_lParameterNo, m_enParameterType, _
                                      odbcColTypeChar, m_nColumnType, 0, 0, m_byByteValue(0), MX_BYTEVALUE, m_lValueLength)
      End Select
    Else
      Select Case m_nColumnType
        Case odbcColTypeBinary, odbcColTypeLongVarBinary, odbcColTypeVarBinary:
          lRc = SQLBindParameter(m_lStatementHandle, m_lParameterNo, SQL_PARAM_INPUT, _
                                  odbcColTypeBinary, m_nColumnType, 0, 0, ByVal m_lParameterNo, 0, m_lValueLength)
        Case Else:
          lRc = SQLBindParameter(m_lStatementHandle, m_lParameterNo, SQL_PARAM_INPUT, _
                                  odbcColTypeChar, m_nColumnType, 0, 0, ByVal m_lParameterNo, 0, m_lValueLength)
      End Select
    End If
   
    If lRc <> SQL_SUCCESS Then
        If Not m_oConnection.error.checkRC(lRc, m_lStatementHandle) Then Exit Function
    End If

    'rc
    bindParameter = True

    Exit Function

errBindParameter:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      class_Initialize
'    description:   Occurs when an application creates an instance of a Form, MDIForm, User control, Property Page, Webclass, DHTML Page Designer, or class.
'
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Private Sub Class_Initialize()
    On Error GoTo ErrClass_Initialize

    Const FUNCTION_NAME = "Class_Initialize"
    
    ' Initialize
    m_bBoundParameter = True
    m_lParameterNo = 0
    m_enParameterType = 0
    m_lValueLength = 0
    m_sValue = ""
    m_byByteValue(0) = 0
    
    Exit Sub
    
ErrClass_Initialize:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    property:      connection
'    description:   set a sapdb connection object
'    arguments:     newConnection, connection object
'
'    return value:  /
'
'=========================================================================================
Public Property Set connection(newConnection As SQLStudioDBLayer.connection)
    On Error GoTo errP_SET_Connection

    Const FUNCTION_NAME = "P_SET Connection"
    
    Set m_oConnection = newConnection
    
    Exit Property
    
errP_SET_Connection:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Property

