/* --- GPL ---

 Copyright (C) 2000-2001 SAP AG

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 --- GPL ---
*/

// SQLStudioDSQLInput.h : Declaration of the CSQLStudioInput

#ifndef __SQLSTUDIODSQLINPUT_H_
#define __SQLSTUDIODSQLINPUT_H_

#include "resource.h"       // main symbols
#include <atlctl.h>


#include "richedit.h"
#include "richole.h"
#include "tom.h"			// text object model


#include <COMMCTRL.H>       // for toolbar, statusbar and find/replace dialogs
#include <COMDEF.H>       


// STL includes
#include <map>				// STL map class
#include <vector>			// vector
#include <xstring>			// basic_string, string, wstring
#include <algorithm> 
#include <fstream>    
#include "SQLStudioDSQLCP.h"

// IDs of contained windows
#define ID_RICHEDIT		1100
#define ID_STATUS_BAR	1101

// Text Object Model 
typedef CComQIPtr<ITextDocument,	&__uuidof(ITextDocument)>		ITextDocumentPtr;
typedef CComQIPtr<ITextStoryRanges, &__uuidof(ITextStoryRanges)>	ITextStoryRangesPtr;
typedef CComQIPtr<ITextSelection,	&__uuidof(ITextSelection)>		ITextSelectionPtr;
typedef CComQIPtr<ITextRange,		&__uuidof(ITextRange)>			ITextRangePtr;
typedef CComQIPtr<ITextFont,		&__uuidof(ITextFont)>			ITextFontPtr;
typedef CComQIPtr<ITextPara,		&__uuidof(ITextPara)>			ITextParaPtr;

using namespace std;

// use STL string class in map of display strings.
// The advantage over CAutoString is that basic_string
// is reference counted.
// The disadvantage is that the string cannot be
// manipulated directy using the original C-Pointer.
// So the CAutoString class is also used
#ifdef UNICODE
	typedef std::wstring CBasicString;
#else
	typedef std::string CBasicString;
#endif



/////////////////////////////////////////////////////////////////////////////
// CSQLStudioInput
class ATL_NO_VTABLE CSQLStudioInput : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CStockPropImpl<CSQLStudioInput, ISQLStudioDSQLInput, &IID_ISQLStudioDSQLInput, &LIBID_SQLSTUDIODSQLLib>,	
	public CComCoClass<CSQLStudioInput, &CLSID_SQLStudioDSQLInput>,
	public CComControl<CSQLStudioInput>,	
	public IProvideClassInfo2Impl<&CLSID_SQLStudioDSQLInput, &DIID__ISQLStudioDSQLInputEvents, &LIBID_SQLSTUDIODSQLLib>,	
	public IOleControlImpl<CSQLStudioInput>,
	public IOleObjectImpl<CSQLStudioInput>,
	public IOleInPlaceActiveObjectImpl<CSQLStudioInput>,
	public IViewObjectExImpl<CSQLStudioInput>,
	public IOleInPlaceObjectWindowlessImpl<CSQLStudioInput>,
	public IDataObjectImpl<CSQLStudioInput>,
	// events and property change notifications
	public CProxy_ISQLStudioDSQLInputEvents<CSQLStudioInput>,
	public IPropertyNotifySinkCP<CSQLStudioInput>,
	public IConnectionPointContainerImpl<CSQLStudioInput>,
	// Persistance
	public IPersistPropertyBagImpl<CSQLStudioInput>,			// Saving properties
	public IPersistStreamInitImpl<CSQLStudioInput>,
	public IPersistStorageImpl<CSQLStudioInput>,	
	public IQuickActivateImpl<CSQLStudioInput>,
	// Property page implementation
	public IPerPropertyBrowsingImpl<CSQLStudioInput>,
	public ISpecifyPropertyPagesImpl<CSQLStudioInput>,
	// Property categories for Visual Basic Property View
    public ICategorizeProperties,
	public IRichEditOleCallback
{


public:
	

DECLARE_REGISTRY_RESOURCEID(IDR_SQLSTUDIODSQL)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CSQLStudioInput)
	COM_INTERFACE_ENTRY(ISQLStudioDSQLInput)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObject2)
	COM_INTERFACE_ENTRY(IViewObject)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObject)
	COM_INTERFACE_ENTRY2(IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY2(IPersist, IPersistStreamInit)
	COM_INTERFACE_ENTRY(IQuickActivate)
	COM_INTERFACE_ENTRY(IPersistStorage)
	COM_INTERFACE_ENTRY(IDataObject)
	COM_INTERFACE_ENTRY(IProvideClassInfo)
	COM_INTERFACE_ENTRY(IProvideClassInfo2)
	COM_INTERFACE_ENTRY(IPersistPropertyBag)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY_IID(IID_IRichEditOleCallback, IRichEditOleCallback)	
	COM_INTERFACE_ENTRY_IMPL(IPerPropertyBrowsing)
	COM_INTERFACE_ENTRY_IMPL(ISpecifyPropertyPages)
	COM_INTERFACE_ENTRY_IMPL(IConnectionPointContainer)
	// Property categories for Visual Basic Property View
	COM_INTERFACE_ENTRY(ICategorizeProperties)    	
END_COM_MAP()

BEGIN_PROP_MAP(CSQLStudioInput)
	PROP_DATA_ENTRY("_cx", m_sizeExtent.cx, VT_UI4)
	PROP_DATA_ENTRY("_cy", m_sizeExtent.cy, VT_UI4)	
	PROP_ENTRY("Text",							DISPID_TEXT,				CLSID_NULL)
	PROP_ENTRY("Readonly",						DISPID_READONLY,			CLSID_NULL)
	PROP_ENTRY("StatusBarVisible",				DISPID_STATUSBARVISIBLE,	CLSID_NULL)
	PROP_ENTRY("ToolBarVisible",				DISPID_TOOLBARVISIBLE,		CLSID_NULL)
	PROP_ENTRY("HighlightKeyAndCom",			DISPID_HIGHLIGHTKEYANDCOM,	CLSID_NULL)
	PROP_ENTRY("CommentPrefixAtStartOfLine",	DISPID_CPREFIXATSTARTOFLINE,CLSID_NULL)	
	PROP_ENTRY("KeyWordsCaseSensitive",			DISPID_CASESENSITIVE,		CLSID_NULL)
	PROP_ENTRY("WordWrap",						DISPID_BWORDWRAP,				CLSID_NULL)		
	PROP_ENTRY("ConvertWordWrapToLineBreak",	DISPID_CONVERTWORDWRAPTOLINEBREAK,	CLSID_NULL)
	PROP_ENTRY("AutoIndent",					DISPID_AUTOINDENT,					CLSID_NULL)	
	PROP_ENTRY("LocalContextMenuVisible",		DISPID_LOCALCONTEXTMENUVISIBLE,		CLSID_NULL)
	PROP_ENTRY("NavigateOnDblclick",			DISPID_NAVIGATEONDBLCLICK,			CLSID_NULL)
	PROP_ENTRY("SpacesOnIndent",				DISPID_SPACESONINDENT,				CLSID_NULL)
	PROP_ENTRY("ChangeCaseMode",				DISPID_CHANGECASE,					CLSID_NULL)
	PROP_ENTRY("ColorTextHighlight",			DISPID_CTHIGHLIGHT,					CLSID_NULL)
	PROP_ENTRY("ColorTextEditable",				DISPID_CTEDITABLE,					CLSID_NULL)
	PROP_ENTRY("OLEDropMode",					DISPID_OLEDROPMODE,				CLSID_NULL)	
	PROP_ENTRY("Font",							DISPID_FONT, CLSID_StockFontPage)
END_PROP_MAP()

BEGIN_MSG_MAP(CSQLStudioInput)

	DEFAULT_REFLECTION_HANDLER()
	MESSAGE_HANDLER(WM_CREATE,			OnCreate)
	MESSAGE_HANDLER(WM_DESTROY,			OnDestroy)
	MESSAGE_HANDLER(WM_ERASEBKGND,		OnEraseBkGnd)
	MESSAGE_HANDLER(WM_SETCURSOR,		OnSetCursor)
	MESSAGE_HANDLER(WM_SETFOCUS,		OnControlSetFocus)
	MESSAGE_HANDLER(WM_SIZE,			OnSize)
	MESSAGE_HANDLER(WM_TIMER,			OnTimer)
	MESSAGE_HANDLER(WM_KILLFOCUS,		OnKillFocus)
	MESSAGE_HANDLER(WM_MOUSEACTIVATE,	OnMouseActivate)
	

	// registered messages
	MESSAGE_HANDLER(RegisterWindowMessage(FINDMSGSTRING),	OnFindReplaceDlg)
	
	// Command handlers of toolbar buttons
	COMMAND_HANDLER(ID_CUT,				0,		OnCut)
	COMMAND_HANDLER(ID_COPY,			0,		OnCopy)
	COMMAND_HANDLER(ID_PASTE,			0,		OnPaste)
	COMMAND_HANDLER(ID_UNDO,			0,		OnUndo)
	COMMAND_HANDLER(ID_REDO,			0,		OnRedo)
	COMMAND_HANDLER(ID_COMMENT,			0,		OnComment)
	COMMAND_HANDLER(ID_UNCOMMENT,		0,		OnUnComment)
	COMMAND_HANDLER(ID_IMPORT,			0,		OnImport)
	COMMAND_HANDLER(ID_EXPORT,			0,		OnExport)
	COMMAND_HANDLER(ID_FIND_REPLACE,	0,		OnFindText)
	COMMAND_HANDLER(ID_FIND_NEXT,		0,		OnFindTextNext)
	COMMAND_HANDLER(ID_CHOOSEFONT,		0,		OnChooseFont)
	COMMAND_HANDLER(ID_FORTEXTEDIT,		0,		OnForeTextEditColor)
	COMMAND_HANDLER(ID_FORHIGHLIGHT,		0,		OnHighLightTextColor)
	
	// Notify handlers for richedit
	NOTIFY_HANDLER(ID_RICHEDIT,		EN_DROPFILES,	OnDropFiles)
	NOTIFY_HANDLER(ID_RICHEDIT,		EN_MSGFILTER,	OnMsgFilter)
	NOTIFY_HANDLER(ID_RICHEDIT,		EN_SELCHANGE,	OnSelChange)
	COMMAND_HANDLER(ID_RICHEDIT,	EN_SETFOCUS,	OnEditSetFocus)
	COMMAND_HANDLER(ID_RICHEDIT,	EN_KILLFOCUS,	OnEditKillFocus)

	// Command handlers for richedit
	COMMAND_HANDLER(ID_RICHEDIT,	EN_CHANGE,		OnChange)
	COMMAND_HANDLER(ID_RICHEDIT,	EN_VSCROLL,		OnVScroll)

	// Notify handlers for toolbar
	NOTIFY_CODE_HANDLER(TTN_NEEDTEXT,		OnNeedText)
	

	CHAIN_MSG_MAP(CComControl<CSQLStudioInput>)
	
ALT_MSG_MAP(1)
	//MESSAGE_HANDLER(WM_SETFOCUS, OnChildSetFocus)
	//MESSAGE_HANDLER(WM_KILLFOCUS, OnChildKillFocus)  
END_MSG_MAP()

	

	// Message handlers for control window
	LRESULT OnCreate			(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT OnDestroy			(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT OnEraseBkGnd		(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT OnSetCursor			(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT OnControlSetFocus	(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT OnSize				(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT OnTimer				(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);

	LRESULT  OnKillFocus	(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);	  
	LRESULT  OnMouseActivate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);	  
	

	// registered messages
	LRESULT OnFindReplaceDlg	(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);

	
	// Message handlers for richedit window
	LRESULT OnCharEdit			(WPARAM wParam, LPARAM lParam);
	LRESULT OnKeyDownEdit		(WPARAM wParam, LPARAM lParam);
	LRESULT OnLButtonDblClkEdit	(WPARAM wParam, LPARAM lParam);
	LRESULT OnVScrollEdit		(WPARAM wParam, LPARAM lParam);
	LRESULT OnMouseMove			(WPARAM wParam, LPARAM lParam);	  
	LRESULT OnMouseUp			(WPARAM wParam, LPARAM lParam);	  
	LRESULT OnMouseDown			(WPARAM wParam, LPARAM lParam);	  

	
	LRESULT OnEditSetFocus		(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnEditKillFocus		(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	
	  

	// Command handlers of toolbar
	LRESULT OnCut				(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnCopy				(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnPaste				(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnUndo				(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnRedo				(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnComment			(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnUnComment 		(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnImport			(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnExport			(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnFindText			(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnFindTextNext		(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnChooseFont		(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);

	LRESULT OnForeTextEditColor		(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnHighLightTextColor	(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	
	// Notify handlers for richedit
	LRESULT OnDropFiles			(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);
	LRESULT OnMsgFilter			(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);
	LRESULT OnSelChange			(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);

	// Command handlers for richedit
	LRESULT OnChange			(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnVScroll			(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	
	// Notify handlers for toolbar
	LRESULT OnNeedText			(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);


	STDMETHOD(TranslateAccelerator)(LPMSG lpmsg);


// IViewObjectEx
	DECLARE_VIEW_STATUS(VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE)

// ISQLStudioDSQLInput
public:
	HRESULT OnDraw(ATL_DRAWINFO& di);

public:
	STDMETHOD(SetClientSite)(LPOLECLIENTSITE pClientSite); 
	STDMETHOD(EmptyUndoBuffer)();
	STDMETHOD(get_Text)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(put_Text)(/*[in]*/ BSTR newVal);
	STDMETHOD(get_ToolBarVisible)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_ToolBarVisible)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(get_StatusBarVisible)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_StatusBarVisible)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(get_ReadOnly)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_ReadOnly)(/*[in]*/ VARIANT_BOOL newVal);

	// IRichEditOleCallback
	STDMETHOD(GetNewStorage)						(LPSTORAGE FAR * lplpstg);
    STDMETHOD(GetInPlaceContext)					(LPOLEINPLACEFRAME FAR * lplpFrame,
														LPOLEINPLACEUIWINDOW FAR * lplpDoc,
														LPOLEINPLACEFRAMEINFO lpFrameInfo);
	STDMETHOD(ShowContainerUI)						(BOOL fShow);
	STDMETHOD(QueryInsertObject)					(LPCLSID lpclsid, LPSTORAGE lpstg,	LONG cp);
	STDMETHOD(DeleteObject)							(LPOLEOBJECT lpoleobj);
	STDMETHOD(QueryAcceptData)						(LPDATAOBJECT lpdataobj,
														CLIPFORMAT FAR * lpcfFormat, DWORD reco,
														BOOL fReally, HGLOBAL hMetaPict);
	STDMETHOD(ContextSensitiveHelp)					(BOOL fEnterMode);
	STDMETHOD(GetClipboardData)						(CHARRANGE FAR * lpchrg, DWORD reco,
														LPDATAOBJECT FAR * lplpdataobj);
	STDMETHOD(GetDragDropEffect)					(BOOL fDrag, DWORD grfKeyState, LPDWORD pdwEffect);

	
	STDMETHOD (put_Font)(IFontDisp* pFont);
	STDMETHOD (putref_Font)(IFontDisp* pFont);
	STDMETHOD (get_Font)(IFontDisp** ppFont);

	


	/////////////////////////////////////////////////////////////////////////
	CSQLStudioInput();
	~CSQLStudioInput();
	/////////////////////////////////////////////////////////////////////////

	/////////////////////////////////////////////////////////////////////////
	// Stock property 'Text'
	/////////////////////////////////////////////////////////////////////////
	CComBSTR		m_bstrText;

	//////////////////////////////////////////////////////////////////////////
	// Font
	//////////////////////////////////////////////////////////////////////////
	CComPtr<IFontDisp> m_pFont;


private:
	/////////////////////////////////////////////////////////////////////////
	// Helper functions
	/////////////////////////////////////////////////////////////////////////
	void			CreateSQLStudioDSQLFont			();
	void			InternalFileOpen			(LPCTSTR lpszFileName);
	void			InternalFileSaveAs			(LPCTSTR lpszFileName);
	long			InternalSetText				(const CComBSTR &bstrText, long nTextLength=-1, long nFlags = 0);
	long			InternalGetText				(CComBSTR &bstrText, long nFlags = 0);
	long			InternalGetText				(vector<CBasicString> &vecString, bool bGatherOutputInFirstElelemt, bool bIncludeLineBreaks, long nFlags = 0);
	long			CheckLineFromTo				(long &nLineFrom, long &nLineTo);
	void			CheckIndexFromTo			(long &nIndexFrom,long &nIndexTo);
	long			CheckLinePos				(long nLine, long &nCol);
	void			CheckLinePosFromTo			(long &nLineFrom, long &nColFrom, long &nLineTo, long &nColTo);
	void			CharIndexToLinePos			(long nCharIndex, long &nLine, long &nCol);
	long			LinePosToCharIndex			(long nLine, long nCol);
	void			UpdateStatusBar				(void);
	void			ArrangeWindows				(void);
	long			GetSelectedLineCount		(void);
	long			GetSelectedWordCount		(void);
	bool			InternalFind				(long nCount, long nFlags, ITextRangePtr &pITextRange);
	bool			FindNext					(ITextRangePtr &pITextRange, bool bRestartAtTop);
	bool			findNextWithBeep			(ITextRangePtr &pITextRange);

	bool			findNextWithBeep			();
	void			ShowFindReplaceDialog		(void);
	void			ShowChooseFontDialog		(void);
	void			ShowColorRefDialog			(COLORREF &Color);
	void			UpdateFindReplaceRange		(bool bUpdateSearchString);
	ITextRangePtr	GetWord						(const ITextRangePtr &pITextRangeIn);
	bool			replace						(ITextRangePtr &pITextRange);
	bool			ReplaceAndFindNextWithBeep	(ITextRangePtr &pITextRange);
	long			InternalReplaceAll			(ITextRangePtr &pITextRange, bool bRestartAtTop);
	bool			AdoptFindReplaceRange		(ITextRangePtr &pITextRange);
	void			EnableDisableToolBarButtons	(void);
	long			GetNumberOfSelectedCharacters(void);
	bool			IsCommentLine				(long nLine);
	bool			IsCommentLine				(long nLine, ITextRangePtr &pITextRange);
	bool			IsComment					(const ITextRangePtr &pITextRange);
	void			HighlightAllComments		(void);
	void			HighlightVisibleComments	(void);
	long			GetFirstAndLastVisibleLine	(long &nLineFirst, long &nLineLast);
	CComBSTR		InternalGetLineText			(long nLine, bool nAlwaysDeterminWithLinebreak = false);
	LPCTSTR			GetDisplayText				(long nTextID);
	void			FillDisplayMapFromResource	(void);
	void			FillDisplayMapFromResource	(long nTextID);
	void			CreateRichEditWindow		(void);
	void			InternalSelectLines			(long &nLineFrom, long &nLineTo);
	void			StartTimerHighlightKeyAndCom	(void);
	void			StartTimerUpdateToolbar		(void);
	void			InternalSetSelectionPos		(long nLineFrom, long nColFrom, long nLineTo, long nColTo);
	void			InternalGetSelectedLines	(long * pnLineFrom, long * pnLineTo);
    void			InternalSetSelectionIndexes (long nIndexFrom, long nIndexTo);
	void			InternalGetSelectionPos		(long * pnLineFrom, long * pnColFrom, long * pnLineTo, long * pnColTo);
	void			InternalGetSelectionIndexes	(long * pnIndexFrom, long * pnIndexTo);
	void			InternalHighlight			(const ITextRangePtr &pITextRange, VARIANT_BOOL bHighlight);	
	void			InternalCheckRangeColoring	(const ITextRangePtr &pITextRange);
	bool			InternalIndentOnEnter		(void);
	void			InternalIndent				(long nLineFrom, long nColFrom, long nLineTo, long nColTo, int nMode = -1);
	void			InternalUnIndent			(long nLineFrom, long nLineTo);
	void			InternalComment				(long nLineFrom, long nLineTo);
	void			InternalUnComment			(long nLineFrom, long nLineTo);
	void			InternalSwitchInsertOverwrite	    (void);
	bool			InternalGetModifyStatus		(void);
	bool			InternalSetModifyStatus		(bool bNewStatus);
	void			InternalGetPoint			(long * px, long * py);
	void			CreateStatusBar				(void);
	void			CreateToolBar				(void);
	
	/////////////////////////////////////////////////////////////////////////
	// TOM specific helper methods
	/////////////////////////////////////////////////////////////////////////
	void			InitITextSelection			(ITextSelectionPtr &pITextSelection);
	void			InitITextStoryRanges		(ITextStoryRangesPtr &ITextStoryRanges);
	void			InitITextRange				(ITextRangePtr &pITextRange, long cpStart, long cpEnd);
	void			InitITextRange				(ITextRangePtr &pITextRange, long nLine, bool nExtendMode=true);
	void			InitITextPara				(const ITextRangePtr &pITextRange, ITextParaPtr &pITextPara);
	void			GetSelectionRangeDuplicate	(ITextRangePtr &pITextRangeDuplicate);
	void			GetDuplicate				(const ITextRangePtr &pITextRange, ITextRangePtr &pITextRangeDuplicate);

	HFONT			CreatePointFont(int nPointSize, LPCTSTR lpszFaceName);


	/////////////////////////////////////////////////////////////////////////
	// Keyword coloring specific helper methods
	/////////////////////////////////////////////////////////////////////////
	int				IsKeyword(LPCTSTR lpszSymbol);
	int				IsConstant(LPCTSTR lpszSymbol);

		
	/////////////////////////////////////////////////////////////////////////
	// static callback functions
	/////////////////////////////////////////////////////////////////////////
	static  bool			 IsStandardWordDelimiter	(const long & nChar);

	////////////////////////////////////////////////////////////////////////
	// Thread function for find/replace dialog
	// The separate thread for the modeless dialog is needed, to
	// call 'IsDialogMessage' in the thread's message loop.
	// Without calling this function the TAB key for navigating
	// the focus in the dialog will not work.
	////////////////////////////////////////////////////////////////////////
	static	DWORD			WINAPI ThreadFuncSearchReplace		(LPVOID pData);

	/////////////////////////////////////////////////////////////////////////
	// Internal classes
	/////////////////////////////////////////////////////////////////////////

	class CMyComBSTR : public CComBSTR
	{
		public:
			/*!----------------------------------------------------------------------
			   function: CMyComBSTR
			   description: constructor
                
			   arguments:    [in] src, constant CComBSTR ref
                 
			   return value: /
			-------------------------------------------------------------------------*/
			CMyComBSTR ( const CComBSTR& src )
				:CComBSTR(src)
			{
			}

			CMyComBSTR ()
				:CComBSTR()
			{
			}

			int Find(const CMyComBSTR& str) 
			{
			    
#ifndef _UNICODE
				USES_CONVERSION;	 
#endif
				const TCHAR* tmp =   OLE2T(this->m_str) ;
				TCHAR* pdest = _tcsstr(tmp, OLE2T(str.m_str) );   

			   if(pdest == NULL)
				   return -1;
			   else
				   return pdest - tmp;
				
			}

			void TrimLeft()
			{
#ifndef _UNICODE
			  USES_CONVERSION;	 
#endif
			  const TCHAR* tmp =   OLE2T(this->m_str) ;
  			  TCHAR* trimstr = _tcsspnp( tmp, _T(" \t") );			

			  if(trimstr != NULL)
			  {
				  this->Detach();
				  this->Append(trimstr);
			  }

			  return;
			};

			
			bool operator==(const CMyComBSTR& str) const
			{
				if(!this == false && (!str == true)) return false;
				if(!this == true && (!str == false)) return false;
				if(!this == true && (!str == true)) return true;
			
				if(this->m_str !=str.m_str) return false;			

				return true;
			}

			bool operator!=(const CMyComBSTR& str) const
			{
				if(!this == false && (!str == true)) return true;
				if(!this == true && (!str == false)) return true;
				if(!this == true && (!str == true)) return false;

				if(this->m_str != str.m_str) return true;			

				return false;
			}			

	};

	/////////////////////////////////////////////////////////////////////////
	// Helper class to save the scroll positions
	/////////////////////////////////////////////////////////////////////////
	class CSaveScrollPosition
	{
	public:
		CSaveScrollPosition(CSQLStudioInput& rTextEditCtrl):
			m_rTextEditCtrl(rTextEditCtrl)
		{			
			// Get position of horizontal scrollbar
			::GetScrollInfo(m_rTextEditCtrl.m_RichEditCtrl.m_hWnd, SB_HORZ, &m_rTextEditCtrl.m_oSCROLLINFOHorz);

			// Get position of vertical scrollbar
			::GetScrollInfo(m_rTextEditCtrl.m_RichEditCtrl.m_hWnd, SB_VERT, &m_rTextEditCtrl.m_oSCROLLINFOVert);
		}

		~CSaveScrollPosition(void)
		{			

			// Set position of horizontal scrollbar
			::SetScrollInfo(m_rTextEditCtrl.m_RichEditCtrl.m_hWnd, SB_HORZ, &m_rTextEditCtrl.m_oSCROLLINFOHorz, false);

			// Set position of vertical scrollbar
			::SetScrollInfo(m_rTextEditCtrl.m_RichEditCtrl.m_hWnd, SB_VERT, &m_rTextEditCtrl.m_oSCROLLINFOVert, false);

			// force repaint
			m_rTextEditCtrl.m_RichEditCtrl.SendMessage(WM_NCPAINT, 1, 0);

			m_rTextEditCtrl.UpdateStatusBar();
		}

		CSQLStudioInput&	m_rTextEditCtrl;
	};
	friend class		CSaveScrollPosition;
	SCROLLINFO			m_oSCROLLINFOHorz;
	SCROLLINFO			m_oSCROLLINFOVert;

	/////////////////////////////////////////////////////////////////////////
	// Helper class for keeping a toolbar button pressed
	/////////////////////////////////////////////////////////////////////////
	class CPressToolBarButton
	{
	public:
		CPressToolBarButton(CWindow& rToolBar, long nBtnID):
			m_rToolBar(rToolBar),
			m_nBtnID(nBtnID)
		{
			m_rToolBar.SendMessage(TB_PRESSBUTTON, m_nBtnID, MAKELONG(true, 0));
		}

		~CPressToolBarButton()
		{
			m_rToolBar.SendMessage(TB_PRESSBUTTON, m_nBtnID, MAKELONG(false, 0));
		}

	private:
		long	m_nBtnID;
		CWindow	&m_rToolBar;
	};

	/////////////////////////////////////////////////////////////////////////
	// Helper class to display a wait cursor
	// This class is needed because of a bug in richedit.
	// Even if the ENM_SCROLLEVENTS is not set the richedit
	// sends scroll notifications. So we need a class to set
	// a flag when to ignore these events
	/////////////////////////////////////////////////////////////////////////
	class CIgnoreScrollEvents
	{
	public:
		CIgnoreScrollEvents(CSQLStudioInput& rTextEditCtrl):
			m_rTextEditCtrl(rTextEditCtrl)
		{
			m_rTextEditCtrl.m_bIgnoreScrollEvents++;
		}

		~CIgnoreScrollEvents(void)
		{
			m_rTextEditCtrl.m_bIgnoreScrollEvents--;
		}

		CSQLStudioInput& m_rTextEditCtrl;
	};
	friend class		CIgnoreScrollEvents;
	// Flag when to process the scroll events
	// If this flag is 0 (FALSE) the scroll events are processed.
	// If this flag is not 0 the scroll events 
	long				m_bIgnoreScrollEvents;

	/////////////////////////////////////////////////////////////////////////
	// Helper class to update the find replace range properly
	// This class is needed for the handling in OnSelChange()
	// during find and replace situations
	/////////////////////////////////////////////////////////////////////////
	class CNoUpdateOfFindReplaceRange
	{
	public:
		CNoUpdateOfFindReplaceRange(CSQLStudioInput& rTextEditCtrl):
			m_rTextEditCtrl(rTextEditCtrl)
		{
				m_rTextEditCtrl.m_bUpdateOfFindReplaceRange = false;
		}

		CSQLStudioInput& m_rTextEditCtrl;
	};
	friend class		CNoUpdateOfFindReplaceRange;
	// Flag when to process the selection change events
	// If this flag is 0 (FALSE) the selection change events are processed.
	// If this flag is not 0 the selection change events are ignored.
	bool				m_bUpdateOfFindReplaceRange;
	long				m_nUpdateOfFindReplaceRangeCount;


	/////////////////////////////////////////////////////////////////////////
	// Helper class to set the value of m_bInsideHighlightKeyAndCom
	/////////////////////////////////////////////////////////////////////////
	class CInsideHighlightKeyAndCom
	{
	public:
		CInsideHighlightKeyAndCom(CSQLStudioInput& rTextEditCtrl):
			m_rTextEditCtrl(rTextEditCtrl)
		{
			m_rTextEditCtrl.m_bInsideHighlightKeyAndCom = true;
		}

		~CInsideHighlightKeyAndCom(void)
		{
			m_rTextEditCtrl.m_bInsideHighlightKeyAndCom = false;
		}

		CSQLStudioInput&	m_rTextEditCtrl;
	};
	friend class		CInsideHighlightKeyAndCom;
	// This flag is used to prevent recursions when comments are highlighted.
	// It is also used to disable statusbar updates during highlighting.
	bool				m_bInsideHighlightKeyAndCom;

	/////////////////////////////////////////////////////////////////////////
	// Helper class to set the value of m_bUpdateStatusBar
	/////////////////////////////////////////////////////////////////////////
	class CDisableUpdateStatusBar
	{
	public:
		CDisableUpdateStatusBar(CSQLStudioInput& rTextEditCtrl):
			m_rTextEditCtrl(rTextEditCtrl)
		{
			m_rTextEditCtrl.m_nUpdateCount++;
			if (m_rTextEditCtrl.m_nUpdateCount == 1)
			{
				m_rTextEditCtrl.m_bUpdateStatusBar = false;
			}
		}

		~CDisableUpdateStatusBar(void)
		{
			m_rTextEditCtrl.m_nUpdateCount--;
			if (m_rTextEditCtrl.m_nUpdateCount == 0)
			{
				m_rTextEditCtrl.m_bUpdateStatusBar = true;
				m_rTextEditCtrl.UpdateStatusBar();
			}

		}

		CSQLStudioInput&	m_rTextEditCtrl;
	};
	friend class		CDisableUpdateStatusBar;
	long				m_nUpdateCount;
	// This flag is used to prevent updates of the statusbar
	bool				m_bUpdateStatusBar;


	/////////////////////////////////////////////////////////////////////////
	// Helper class to display a wait cursor
	/////////////////////////////////////////////////////////////////////////
	class CDisplayWaitCursor
	{
	public:
		CDisplayWaitCursor(CSQLStudioInput& rTextEditCtrl):
			m_rTextEditCtrl(rTextEditCtrl)
		{
			m_hCursorPrev = NULL;
			m_rTextEditCtrl.m_nWaitCursorCount++;
			if (m_rTextEditCtrl.m_nWaitCursorCount == 1){

				m_hCursorPrev = ::SetCursor(m_rTextEditCtrl.m_hCursorWait);
			}
		}

		~CDisplayWaitCursor(void)
		{
			m_rTextEditCtrl.m_nWaitCursorCount--;

			if(m_rTextEditCtrl.m_nWaitCursorCount < 0) 
				m_rTextEditCtrl.m_nWaitCursorCount = 0;
			// There is no need to set the cursor back to arrow.
			// The SetCursor function in the constructor sets the 
			// cursor only temporary. After every mouse move or repaint
			// in the window, the normal cursor is displayed again.
			// To display the wait cursor even in these cases the
			// WM_SETCURSOR message is handled (See OnSetCursor).


			// But not in VB e.g Setting a Menu outside of the window doesn't reset cursor
			if (m_hCursorPrev != NULL) 
				::SetCursor(m_hCursorPrev);			
			else
				::SetCursor(m_rTextEditCtrl.m_hCursorBeam);			
		}

		CSQLStudioInput& m_rTextEditCtrl;
	private:
		HCURSOR				m_hCursorPrev;
	};
	friend class		CDisplayWaitCursor;
	long				m_nWaitCursorCount;
	HCURSOR				m_hCursorWait;
	HCURSOR				m_hCursorBeam;
	
	

	/////////////////////////////////////////////////////////////////////////
	// Helper class for saving the flag for modified text
	// This is necessary, because the modify flag is also
	// set when only format aspects, e.g. comment highlighting,
	// protection of text areas, ... is performed.
	// This class temporarly saves the modified status when instantiated
	// and restores the status when the destructor is called.
	/////////////////////////////////////////////////////////////////////////
	class CSaveModifyStatus
	{
	public:
		CSaveModifyStatus(CSQLStudioInput& rTextEditCtrl):
			m_rTextEditCtrl(rTextEditCtrl)
		{
			// get modify status
			m_bModifyStatus = m_rTextEditCtrl.m_RichEditCtrl.SendMessage(EM_GETMODIFY);
		}

		~CSaveModifyStatus(void)
		{
			// reset modify status
			m_rTextEditCtrl.m_RichEditCtrl.SendMessage(EM_SETMODIFY, m_bModifyStatus);
		}

		CSQLStudioInput& m_rTextEditCtrl;
		long	m_bModifyStatus;
	};
	friend class CSaveModifyStatus;

	/////////////////////////////////////////////////////////////////////////
	// Helper class for temporarly suspending undo  into the undo buffer
	// This class temporarly disables actions being submitted into the undo buffer
	// and restores the status when the destructor is called.
	/////////////////////////////////////////////////////////////////////////
	class CSuspendUndo
	{
	public:
		CSuspendUndo(CSQLStudioInput& rTextEditCtrl):
			m_rTextEditCtrl(rTextEditCtrl)
		{
            m_rTextEditCtrl.m_nSuspendUndoCount++;
			if (m_rTextEditCtrl.m_nSuspendUndoCount == 1)
			{
				if (m_rTextEditCtrl.m_pITextDocument)
					// suspend undo behavior
					m_rTextEditCtrl.m_pITextDocument->Undo(tomSuspend,NULL);
			}
		}

		~CSuspendUndo(void)
		{
			m_rTextEditCtrl.m_nSuspendUndoCount--;
			if (m_rTextEditCtrl.m_nSuspendUndoCount == 0)
			{
				if (m_rTextEditCtrl.m_pITextDocument)
					// resume Undo behavior
					m_rTextEditCtrl.m_pITextDocument->Undo(tomResume,NULL);
			}
		}

		CSQLStudioInput& m_rTextEditCtrl;
	};
	friend class		CSuspendUndo;
	long				m_nSuspendUndoCount;

	
	/////////////////////////////////////////////////////////////////////////
	// Helper class for temporarly collecting calls in the undo buffer in one action
	// This class temporarly enables the collection of several calls into one undoable action
	// and restores the status when the destructor is called.
	/////////////////////////////////////////////////////////////////////////
	class CCollectUndoCalls
	{
	public:
		CCollectUndoCalls(CSQLStudioInput& rTextEditCtrl):
			m_rTextEditCtrl(rTextEditCtrl)
		{
            m_rTextEditCtrl.m_nCollectUndoCallsCount++;
			if (m_rTextEditCtrl.m_nCollectUndoCallsCount == 1)
			{
				if (m_rTextEditCtrl.m_pITextDocument)
					// start collecting Undo calls
					m_rTextEditCtrl.m_pITextDocument->BeginEditCollection();
			}

		}

		~CCollectUndoCalls(void)
		{
			m_rTextEditCtrl.m_nCollectUndoCallsCount--;
			if (m_rTextEditCtrl.m_nCollectUndoCallsCount == 0)
			{
				if (m_rTextEditCtrl.m_pITextDocument)
					// stop collecting Undo calls
					m_rTextEditCtrl.m_pITextDocument->EndEditCollection();
			}

		}

		CSQLStudioInput& m_rTextEditCtrl;
	};
	friend class		CCollectUndoCalls;
	long				m_nCollectUndoCallsCount;

	// type definition to facilitate STL definitions within the following class
	typedef std::pair<ITextRangePtr,long> PAIR_RANGE_LONG;
	typedef std::vector<PAIR_RANGE_LONG> PAIR_RANGE_LONG_VECTOR;

	// function object to compare ranges on equality: needed within class CITextRanges
	class CIsEqualRange
	{
	public:
		CIsEqualRange(PAIR_RANGE_LONG &pair): m_pair(pair){}
		bool operator ()(const PAIR_RANGE_LONG &pair)
		{
			long nCompare;
			m_pair.first->IsEqual(pair.first,&nCompare);
			if (nCompare == tomTrue)
				return true;
			return false;
		}
	private:
		PAIR_RANGE_LONG m_pair;
	};

	// function object to sort ranges based on lower start index: needed within class CITextRanges
	class CLowerStartIndex
	{
	public:
		bool operator ()(const PAIR_RANGE_LONG &p1, const PAIR_RANGE_LONG &p2)
		{
			long nStart1;
			long nStart2;
			long nEnd1;
			long nEnd2;
			HRESULT hr;
			hr = p1.first->GetStart(&nStart1);
			if (SUCCEEDED(hr))
			{
				hr = p2.first->GetStart(&nStart2);
				if (SUCCEEDED(hr))
				{
					if ( nStart1 > nStart2)
						return false;
					if ( nStart1 < nStart2)
						return true;
					if (nStart1 == nStart2)
					{
						p1.first->GetEnd(&nEnd1);
						p2.first->GetEnd(&nEnd2);
						if ( nEnd1 <= nEnd2)
							return true;
						else
							return false;
					}
				}
			}
			return false;
		}
	};

	// type definition to facilitate STL definitions within the following class
	typedef std::vector<ITextRangePtr> ITextRangeVector;
	// helper class to manage ranges of equal formats (breakpoints, ...)
	class CITextRanges
	{
	friend CSuspendUndo;
	friend CLowerStartIndex;
	public:
		CITextRanges(CSQLStudioInput& rTextEditCtrl): 
			m_rTextEditCtrl(rTextEditCtrl), m_bSorted(true)
		{
	
		}
		~CITextRanges(void) 
		{ 		
			m_vecRanges.clear();
		}
		bool IsEmpty(void)
		{ 				
			return m_vecRanges.empty(); 
		}
		void AddRange(ITextRangePtr &pITextRange);
		int RemoveRange(ITextRangePtr &pITextRange);
		int RemoveEqualRanges(const ITextRangePtr &pITextRange);
		int RemoveAll(void);
		int FormatAll(void);
		void Sort(void);
		void UpdateFormat(const ITextRangePtr &pITextRangeOuter);
		void UpdatePairInfo();
	private:
		bool FormatRange(const ITextRangePtr &pITextRange, bool bFormat = true);		
		void RemoveOverlap(void);
		// only for test purposes
		void Print(void);
		// data
		bool					  m_bSorted;
		PAIR_RANGE_LONG_VECTOR    m_vecRanges;
		CSQLStudioInput&			      m_rTextEditCtrl;
	};
	friend class		CITextRanges;
	
	

	/////////////////////////////////////////////////////////////////////////
    // Properties
	/////////////////////////////////////////////////////////////////////////
	VARIANT_BOOL	m_bReadonly;
	VARIANT_BOOL	m_bStatusBarVisible;
	VARIANT_BOOL	m_bToolBarVisible;
	VARIANT_BOOL	m_bLocalContextMenuVisible;
	VARIANT_BOOL	m_bNavigateOnDblclick;
	VARIANT_BOOL	m_bHighlightKeyAndCom;
	CComBSTR		m_strCommentPrefix;
	CComBSTR		m_strCommentPrefix2;
	VARIANT_BOOL	m_bWordWrap;
	VARIANT_BOOL	m_bConvertWordWrapToLineBreak;
	VARIANT_BOOL	m_bAutoIndent;
	
	/////////////////////////////////////////////////////////////////////////
	//KeyWord Coloring
	/////////////////////////////////////////////////////////////////////////
	VARIANT_BOOL	m_bCaseSensitive;
	KeyWordChangeCase m_KeyWordChangeCase;
	CMyComBSTR		m_strConstants;
	CMyComBSTR		m_strKeywords;
	CMyComBSTR		m_strKeywordsLower;
	CMyComBSTR		m_strConstantsLower;	


	// Spaces to be inserted / delelted on indenting / unindenting respectively
	long			m_nSpacesOnIndent;

	OLEDropMode			m_OLEDropMode;

	
	/////////////////////////////////////////////////////////////////////////
	//Windows
	/////////////////////////////////////////////////////////////////////////
	CContainedWindow	m_RichEditCtrl;
	CWindow				m_StatusBar;
	CWindow				m_ToolBar;
	CWindow				m_WndFindReplace;
	
	HIMAGELIST			m_TBImageList;


	// to use TOM - Text Object Model
	ITextDocumentPtr	m_pITextDocument;
	ITextSelectionPtr   m_pITextSelection;

	
	// find/replace struct and thread handle
	FINDREPLACE			m_oFINDREPLACE;
	DWORD				m_dwThreadID;
	HANDLE				m_hThread;
	ITextRangePtr		m_pITextRangeFindReplace;
	bool				m_bFindReplaceInSelection;

	
	// Display context for EM_SETTARGETDEVICE
	HDC					m_hDCRichEdit;

	// Text limit
	long				m_nLimitText;

	// Edit notification mask
	ULONG				m_nENMFlags;

	
	// Colors for 'Editable' mode
	COLORREF			m_crTextEditable;
	COLORREF			m_crBackgroundEditable;
	
	// Colors for 'Readonly' mode
	// the fore color for readonly text is the same as for editable text
	COLORREF			m_crBackgroundReadonly;
	
	// Highlight text color for both 'Editable' and 'Readonly'
	COLORREF			m_crTextHighlight;

	
	// Data for find/replace dialog
	CComBSTR			m_bstrFindWhat;
	CComBSTR			m_bstrReplaceWith;
	TCHAR				m_strFindWhat[256];
	TCHAR				m_strReplaceWith[256];

	// Status bar pane IDs
	long				m_nStatusBarPartExternal;
	long				m_nStatusBarPartModified;
	long				m_nStatusBarPartInsOvr;
	long				m_nStatusBarPartSelectionPos;
	long				m_nStatusBarPartVisibleLines;

	// bool flag to display insert/overwrite modus within status bar
	bool				m_bInsertModus;
	
	// Map of display strings
	std::map<long, CBasicString>	m_mapDisplayText;	
	
	
	// flag for modified handling
	bool				m_bTextModifiedStatus;
	
	
	// members to assign virtual key codes for '<' and '>'
	short				m_sVirtualKeyCodeBigger;
	short				m_sVirtualKeyCodeLess;


	//Library Handle
	HINSTANCE m_hLibHandle;
	
public :
	STDMETHOD(get_CommentPrefix2)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(put_CommentPrefix2)(/*[in]*/ BSTR newVal);
	STDMETHOD(RefreshFont)();
	STDMETHOD(get_OLEDropMode)(/*[out, retval]*/ OLEDropMode *pVal);
	STDMETHOD(put_OLEDropMode)(/*[in]*/ OLEDropMode newVal);
	// ICategorizeProperties
    STDMETHODIMP MapPropertyToCategory(/*[in]*/ DISPID dispid, /*[out]*/ PROPCAT* ppropcat);
	STDMETHODIMP GetCategoryName(/*[in]*/ PROPCAT propcat, /*[in]*/ LCID lcid, /*[out]*/ BSTR* pbstrName);
    // IPerPropertyBrowsing
	STDMETHODIMP GetDisplayString(/*[in]*/ DISPID dispid, /*[out]*/ BSTR *pBstr);
    STDMETHODIMP GetPredefinedStrings(/*[in]*/ DISPID dispID, /*[out]*/ CALPOLESTR *pcaStringsOut, /*[out]*/ CADWORD *pcaCookiesOut);
    STDMETHODIMP GetPredefinedValue(/*[in]*/ DISPID dispID, /*[in]*/DWORD dwCookie, /*[out]*/VARIANT* pVarOut);
    STDMETHODIMP MapPropertyToPage(/*[in]*/ DISPID dispID, /*[out*/ CLSID *pClsid);
	//Control Properties
	STDMETHOD(get_ChangeCaseMode)(/*[out, retval]*/ KeyWordChangeCase *pVal);
	STDMETHOD(put_ChangeCaseMode)(/*[in]*/ KeyWordChangeCase newVal);
	STDMETHOD(SetContextMenu)(LPDISPATCH pMenu);
	STDMETHOD(get_TextAreaHeight)(/*[out, retval]*/ long *pVal);
	STDMETHOD(GetPoint)(long *px,long *py);
	STDMETHOD(get_ColorTextHighlight)(/*[out, retval]*/ OLE_COLOR *pVal);
	STDMETHOD(put_ColorTextHighlight)(/*[in]*/ OLE_COLOR newVal);
	STDMETHOD(get_ColorTextEditable)(/*[out, retval]*/ OLE_COLOR *pVal);
	STDMETHOD(put_ColorTextEditable)(/*[in]*/ OLE_COLOR newVal);
	STDMETHOD(UnIndentSelection)();
	STDMETHOD(UnIndentLines)(long nLineFrom, long nLineTo);
	STDMETHOD(UnCommentSelection)();
	STDMETHOD(UnCommentLines)(long nLineFrom, long nLineTo);
	STDMETHOD(SetSelectionPosInLine)(long nLine, long nCol);
	STDMETHOD(SetSelectionPos)(long nLineFrom, long nColFrom, long nLineTo, long nColTo);
	STDMETHOD(SetSelectionIndexes)(long nIndexFrom, long nIndexTo);
	STDMETHOD(ReplaceAll)(BSTR strTextFind, BSTR strTextReplace, VARIANT_BOOL bCaseSensitive, VARIANT_BOOL bWholeWord, LONG * pnReplaceCount);
	STDMETHOD(put_SelectedText)(/*[in]*/ BSTR newVal);
	STDMETHOD(get_SelectedText)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(MakeSelectionVisible)();
	STDMETHOD(IndentSelection)();
	STDMETHOD(IndentLines)(long nLineFrom, long nLineTo);
	STDMETHOD(HighlightSelection)(VARIANT_BOOL bHighlight);
	STDMETHOD(HighlightLines)(long nLineFrom, long nLineTo, VARIANT_BOOL bHighlight);
	STDMETHOD(GetSelectionPosStartLine)(long * pnLine);
	STDMETHOD(GetSelectionPosStartCol)(long * pnCol);
	STDMETHOD(GetSelectionPosEndLine)(long * pnLine);
	STDMETHOD(GetSelectionPosEndCol)(long * pnCol);
	STDMETHOD(GetSelectionPos)(long * pnLineFrom, long * pnColFrom, long * pnLineTo, long * pnColTo);
	STDMETHOD(GetSelectionIndexStart)(long *pnIndex);
	STDMETHOD(GetSelectionIndexes)(long *pnIndexFrom, long *pnIndexTo);
	STDMETHOD(GetSelectionIndexEnd)(long *pnIndex);
	STDMETHOD(GetLineText)(long nLine, BSTR * pstrText);
	STDMETHOD(get_TextModified)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_TextModified)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(get_SpacesOnIndent)(/*[out, retval]*/ long *pVal);
	STDMETHOD(put_SpacesOnIndent)(/*[in]*/ long newVal);
	STDMETHOD(get_SelPosStartLine)(/*[out, retval]*/ long *pVal);
	STDMETHOD(get_SelPosStartCol)(/*[out, retval]*/ long *pVal);
	STDMETHOD(get_SelPosEndLine)(/*[out, retval]*/ long *pVal);
	STDMETHOD(get_SelPosEndCol)(/*[out, retval]*/ long *pVal);
	STDMETHOD(get_SelIndexStart)(/*[out, retval]*/ long *pVal);
	STDMETHOD(get_SelIndexEnd)(/*[out, retval]*/ long *pVal);
	STDMETHOD(get_NavigateOnDblclick)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_NavigateOnDblclick)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(get_LocalContextMenuVisible)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_LocalContextMenuVisible)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(get_FirstVisibleLine)(/*[out, retval]*/ long *pVal);
	STDMETHOD(put_FirstVisibleLine)(/*[in]*/ long newVal);
	STDMETHOD(FindAndSelectText)(BSTR strText, VARIANT_BOOL bCaseSensitive, VARIANT_BOOL bWholeWord, VARIANT_BOOL * pbFound);
	STDMETHOD(FindAndReplace)(BSTR strTextFind, BSTR strTextReplace, VARIANT_BOOL bCaseSensitive, VARIANT_BOOL bWholeWord, VARIANT_BOOL * pbFound);
	STDMETHOD(CommentSelection)();
	STDMETHOD(CommentLines)(long nLineFrom, long nLineTo);
	STDMETHOD(get_AutoIndent)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_AutoIndent)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(get_ConvertWordWrapToLineBreak)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_ConvertWordWrapToLineBreak)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(OnFontChanged)();
	STDMETHOD(get_WordWrap)(/*[out, retval]*/ VARIANT_BOOL  *pVal);
	STDMETHOD(put_WordWrap)(/*[in]*/ VARIANT_BOOL  newVal);
	STDMETHOD(get_KeyWordsCaseSensitive)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_KeyWordsCaseSensitive)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(AddConstants)(BSTR newConstants);
	STDMETHOD(AddKeywords)(BSTR newKeyWords);
	STDMETHOD(get_CommentPrefix)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(put_CommentPrefix)(/*[in]*/ BSTR newVal);
	STDMETHOD(get_HighlightKeyAndCom)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(put_HighlightKeyAndCom)(/*[in]*/ VARIANT_BOOL newVal);
	STDMETHOD(GotoLine)(long nLine);
	STDMETHOD(SelectLines)(long nLineFrom, long nLineTo);
	STDMETHOD(LimitText)(long nLimit);
	STDMETHOD(FileSaveAs)(BSTR strFileName);
	STDMETHOD(FileOpen)(BSTR strFileName);
	STDMETHOD(GetContextMenu)(WORD seltyp, LPOLEOBJECT lpoleobj, CHARRANGE FAR * lpchrg, HMENU FAR * lphmenu);
	STDMETHOD(SetTextSymbol)(BSTR strKey, BSTR strText);
	STDMETHOD(SetStatusText)(BSTR strText);

BEGIN_CONNECTION_POINT_MAP(CSQLStudioInput)	
	CONNECTION_POINT_ENTRY(DIID__ISQLStudioDSQLInputEvents)
	CONNECTION_POINT_ENTRY(IID_IPropertyNotifySink)
END_CONNECTION_POINT_MAP()

};

#endif 

	