VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CmdLine"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---


''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' Form:       SQLStudio.CmdLine
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option Explicit

'--------------------------------------------------------
' Module Name
'--------------------------------------------------------
Const MODULE_NAME = "SQLStudio.CmdLine"


Private Const CML_PREFIX_SLASH = "/"
Private Const CML_PREFIX_MINUS = "-"
Private Const CML_PREFIX_PLUS = "+"

Public Enum CmdLinePrefixConstants
    cmlPrefixSlash
    cmlPrefixMinus
    cmlPrefixPlus
End Enum

Private Const QUOTE1 = """"
Private Const QUOTE2 = "'"

Private sCmdLine As String

Private oItems As New Collection

Public Property Get Text() As String
    On Error GoTo ErrGetText
    
    Const FUNCTION_NAME = "Get Text"
    
    Text = sCmdLine

    Exit Property
  
ErrGetText:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
Public Property Let Text(ByVal NewVal As String)
    On Error GoTo errLetText
    
    Const FUNCTION_NAME = "Let Text"
    
    sCmdLine = Trim(NewVal)
    
    GetCmdLineOpts
    
    Exit Property
    
errLetText:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property

Public Property Get count() As Long
    On Error GoTo ErrGetCount
    
    Const FUNCTION_NAME = "Get count"
    
    count = oItems.count

    Exit Property
    
ErrGetCount:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property

Public Property Get item(ByVal Index As Variant) As CmdLineOpt
Attribute item.VB_UserMemId = 0
    On Error GoTo ErrGetItem
    
    Const FUNCTION_NAME = "Get item"
    
    Set item = oItems(Index)

    Exit Property
ErrGetItem:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property

Public Property Get NewEnum()
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    On Error GoTo ErrGetNewEnum
    
    Const FUNCTION_NAME = "NewEnum"
    
    Set NewEnum = oItems.[_NewEnum]

    Exit Property
    
ErrGetNewEnum:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'
'
'
Public Function add(ByVal Prefix As CmdLinePrefixConstants, _
               ByVal Name As String, _
               ByVal isOptional As Boolean, _
               ByVal MatchCase As Boolean, _
               ByVal ValueDesc As String, _
               ByVal description As String)
    On Error GoTo ErrAdd
    
    Const FUNCTION_NAME = "Add"

    Dim oCmdLineOpt As New CmdLineOpt
    
    With oCmdLineOpt
        .Prefix = Prefix
        .Name = Name
        .isOptional = isOptional
        .ValueDesc = ValueDesc
        .description = description
        .inCmdLine = False
    End With
    
    oItems.add oCmdLineOpt, oCmdLineOpt.Name

    Exit Function
    
ErrAdd:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

Public Function exists(ByVal Index As Variant) As Boolean

    Dim oCmdLineOpt As CmdLineOpt
    
    On Error Resume Next
    Set oCmdLineOpt = oItems(Index)
    exists = (Err.Number = 0)
    
End Function

Public Function Remove(ByVal Index As Variant)
    On Error GoTo errRemove
    
    Const FUNCTION_NAME = "Remove"
        
    oItems.Remove Index

    Exit Function
    
errRemove:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

Public Function clear()
    On Error GoTo errClear
    
    Const FUNCTION_NAME = "clear"
    
    Dim l As Long
    For l = 1 To oItems.count
        oItems.Remove 1
    Next
    

    Exit Function
    
errClear:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'
' Private Sub/Function
'
Private Sub GetCmdLineOpts()
    On Error GoTo ErrGetCmdLineOpts
    
    Const FUNCTION_NAME = "GetCmdLineOpts"
    
    Dim oCmdLineOpt As CmdLineOpt
    Dim lLen As Long
    Dim lPos As Long
    Dim lPrefix As CmdLinePrefixConstants
    Dim sName As String
    Dim lNoNameNum As Long
    
    lLen = Len(sCmdLine)
    
    If lLen = 0 Then Exit Sub

    lNoNameNum = 1
    
    lPos = 1
    Do While lPos < lLen
        If IsPrefix(lPos, lPrefix) Then
            If lPos <= lLen Then
                sName = getName(lPos, lLen)
                If exists(sName) Then
                    Set oCmdLineOpt = oItems(sName)
                Else
                    Set oCmdLineOpt = New CmdLineOpt
                    oItems.add oCmdLineOpt, sName
                End If
                
                With oCmdLineOpt
                    .Prefix = lPrefix
                    .Name = sName
                    If lPos <= lLen Then
                        .Value = getValue(lPos, lLen)
                    End If
                    .inCmdLine = True
                End With
                
                Set oCmdLineOpt = Nothing
                
            End If
        Else
            If Mid$(sCmdLine, lPos, 1) <> " " Then
                sName = "O" + CStr(lNoNameNum)
                Set oCmdLineOpt = New CmdLineOpt
                oItems.add oCmdLineOpt, sName
                
                lNoNameNum = lNoNameNum + 1
                
                With oCmdLineOpt
                    .Name = sName
                    If lPos <= lLen Then
                        .Value = getValue(lPos, lLen, True)
                    End If
                    .inCmdLine = True
                End With
                
                Set oCmdLineOpt = Nothing
            Else
                lPos = lPos + 1
            End If
        End If
    Loop
    

    Exit Sub
    
ErrGetCmdLineOpts:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub

Private Function IsPrefix(lPos As Long, lPrefix As CmdLinePrefixConstants) As Boolean
    On Error GoTo ErrIsPrefix
    
    Const FUNCTION_NAME = "IsPrefix"
    
    Dim bPrefix As Boolean
    
    bPrefix = False
    
    If lPos > 1 Then
        If Mid$(sCmdLine, lPos - 1, 1) = " " Then
            If lPos < Len(sCmdLine) Then
                If Mid$(sCmdLine, lPos, 1) = CML_PREFIX_MINUS Then
                    lPrefix = cmlPrefixMinus
                    bPrefix = True
                End If
            End If
        End If
    Else
        If Mid$(sCmdLine, lPos, 1) = CML_PREFIX_MINUS Then
            lPrefix = cmlPrefixMinus
            bPrefix = True
        End If
    End If
    
    If bPrefix Then lPos = lPos + 1
    
    
    IsPrefix = bPrefix
    

    Exit Function
    
ErrIsPrefix:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

Private Function IsQuote(lPos As Long) As Boolean
    On Error GoTo ErrIsQuote
    
    Const FUNCTION_NAME = "IsQuote"
    
    If Mid(sCmdLine, lPos, 1) = QUOTE1 Or Mid(sCmdLine, lPos, 1) = QUOTE2 Then
        lPos = lPos + 1
        IsQuote = True
    Else
        IsQuote = False
    End If
    

    Exit Function
    
ErrIsQuote:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

Private Function getName(lPos As Long, lLen As Long) As String
    On Error GoTo ErrGetName
    
    Const FUNCTION_NAME = "GetName"
    
    Dim lPos1 As Long
    Dim lPos2 As Long
    
    blanks lPos, lLen

    lPos1 = lPos
    Do While lPos <= lLen
        If Mid(sCmdLine, lPos, 1) <> " " Then
            lPos = lPos + 1
        Else
            Exit Do
        End If
    Loop
    
    lPos2 = lPos
    
    getName = Mid(sCmdLine, lPos1, lPos2 - lPos1)

    Exit Function
ErrGetName:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

Private Function getValue(lPos As Long, lLen As Long, Optional ByVal bBlankSep As Boolean = False) As String
    On Error GoTo ErrGetValue
    
    Const FUNCTION_NAME = "GetValue"
    
    Dim sValue As String
    Dim bInQuote As Boolean
    Dim lPos1 As Long
    Dim lPos2 As Long
    Dim lPrefix As CmdLinePrefixConstants
    
    sValue = ""
    bInQuote = False
    
    blanks lPos, lLen
    
    lPos1 = lPos
    
    Do While lPos <= lLen
        If IsQuote(lPos) Then
            bInQuote = Not bInQuote
        End If
        
        If IsPrefix(lPos, lPrefix) Then
            lPos = lPos - 1
            Exit Do
        End If
        
        If Mid$(sCmdLine, lPos, 1) <> " " Then
            lPos = lPos + 1
        Else
            If Not bBlankSep Then
                If bInQuote Then
                    lPos = lPos + 1
                Else
                    blanks lPos, lLen
                End If
            Else
                If bInQuote Then
                    lPos = lPos + 1
                Else
                    Exit Do
                End If
            End If
        End If
    Loop
    
    lPos2 = lPos
    
    sValue = Trim(Mid(sCmdLine, lPos1, lPos2 - lPos1))
    
    If Left(sValue, 1) = QUOTE1 Or Left(sValue, 1) = QUOTE2 Then sValue = Mid(sValue, 2)
    If Right(sValue, 1) = QUOTE1 Or Right(sValue, 1) = QUOTE2 Then sValue = Left(sValue, Len(sValue) - 1)
    
    getValue = sValue
    

    Exit Function
    
ErrGetValue:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

Private Function blanks(lPos As Long, lLen As Long)
    On Error GoTo ErrBlanks
    
    Const FUNCTION_NAME = "Blanks"
    
    Do While lPos <= lLen
        If Mid(sCmdLine, lPos, 1) = " " Then
            lPos = lPos + 1
        Else
            Exit Do
        End If
    Loop

    Exit Function
    
ErrBlanks:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

