Attribute VB_Name = "Common"
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' Module:      Common - Common Module
'
'-----------------------------------------------------------------------
'
' responsible:  WolfgangA
'
' special area: SQLStudio
'
' description:  module for global varibles, functions ...
'
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option Explicit

'--------------------------------------------------------
' Module Name
'--------------------------------------------------------
Const MODULE_NAME = "SQLStudio.Common"

'=global public variables================================================

'global object for registry functions
Public m_oRegFunc                  As New SQLStudioUtil.Registry

'global object for saving/reading LRU connections in the registry
Public m_oLRUConnections           As New LRUConnections

'global object for the suppose list used in DSQL Dialog
Public m_oAutoSuppose              As New SQLStudioCCon.ASObject

'global object for
Public m_oStatementList            As SQLStudioUtil.History

'global object for communication between the application and some controls for sql studio
Public m_oGlobalInfo                As SQLStudioMan.globalInfo

'global object for a database connection
Public m_oAdminCon                  As SQLStudioDBLayer.connection

'global variable for the driver name, to be registered as odbc driver
Public m_sDriverName               As String

'global variable for maximum numbers of forems, which can be opened
Public m_csMaxForm                 As Long

'excel application object
Public m_oExcelApp                As Object

'with a commandline option, the initial connect ,ode can be set to aracle
'remember with this variable
Public m_emInitConMode          As sqlMode

'=========================================================================

'=global private variables================================================

'global object for common functions
Public m_oComFunc                 As New SQLStudioUtil.Functions

'global object for command line interface
Private m_oCML As New cmdLine

'===========================================================================


'=constants=================================================================
Const CML_OPT_HELP = "?"
Const CML_OPT_NODE = "n"
Const CML_OPT_DB = "d"
Const CML_OPT_USER = "u"
Const CML_OPT_NI = "O1"
Const CML_OPT_CM = "c"
Const CML_OPT_VER = "version"
Const CML_OPT_TRACE = "t"

Const cnstORACLE = "oracle"


Public Const LOCALFOLDERIND = "LocalFolderInd"

'registry key names
Public Const m_cnstKEYNAMEDRIVER = "DRIVER"
Public Const m_cnstKEYNAMEUID = "UID"
Public Const m_cnstKEYNAMEPWD = "PWD"
Public Const m_cnstKEYDATABASE = "SERVERDB"
Public Const m_cnstKEYSERVERNODE = "SERVERNODE"
'========================================================================================

'=========================================================================================
'    function:      internalError
'    description:   shows message box with error
'    arguments:     [in] CurrentError, vb error object
'                   [in] Modulname, name of source modul
'                   [in] FunctionName, name of used function
'    return value:  /
'
'=========================================================================================
Public Function internalError(ByVal CurrentError As ErrObject, ModulName As String, FunctionName As String)
    MsgBox "Module:" + Chr$(9) + ModulName + Chr$(13) + Chr$(10) + "Function:" + Chr$(9) + FunctionName + Chr$(13) + Chr$(10) + "Error No:" + Chr$(9) + Trim$(Str$(CurrentError.Number)) + Chr$(13) + Chr$(10) + "Error:" + Chr$(9) + CurrentError.description, vbExclamation, "Unexpected Error:"
End Function

'=========================================================================================
'    function:      Main
'    description:   called when starting application
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Sub Main()
    On Error GoTo ErrMain
    
    Const FUNCTION_NAME = "Main"
    
    '=varibles===================================================================
    'buffer for command line string
    Dim sCommandLine As String
    
    'instance of about form
    Dim oAbout       As New SQLStudioCCon.About
    '============================================================================
    
    copyRegValuesFromVersion73010
        
    'number of max forms allowed
    m_csMaxForm = 33
        
    'set inital connect mode
    m_emInitConMode = Internal
                
    'get command line
    sCommandLine = Command
    
    'init command line object
    cmdLineOptsInit
    
    'feature of ms ole the first use sets CommandLine to "-Embedding"
    If Len(sCommandLine) > 0 And sCommandLine <> "-Embedding" Then
    
        'analyze command line
        If checkCommandLine(sCommandLine) = True Then
        
            'connect to database
            Call connect
        Else
            Exit Sub
        End If
    Else
        oAbout.Files.add "SQLSTO.exe", App.Path
        'show version from main programm
        If (Len(oAbout.Files.item(1).versionString) > 3) Then
        
            oAbout.version = Left(oAbout.Files.item(1).versionString, 3)
        Else
            oAbout.version = oAbout.Files.item(1).versionString
        End If
        
        oAbout.showSplashForm 1500, True, True
    End If
            
    'show main window
    frmMain.Show
    
    'trigger resize
    frmMain.ctlCMContainer.Width = frmMain.Width / 4
               
    Exit Sub

ErrMain:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      checkCommandLine
'    description:   analyzes a string for command line options like -d, -n ...
'    arguments:     [in] sCommandline, string to analyze
'
'    return value:  true if command line triggers a db connect
'
'=========================================================================================
Function checkCommandLine(sCommandLine As String) As Boolean
    On Error GoTo errCheckCommandLine
    
    Const FUNCTION_NAME = "checkCommandLine"
    
    '=varibles===================================================================
    Dim lPos    As Long
    Dim sBuffer As String
    '============================================================================
    
    'rc
    checkCommandLine = False
    
     'inits
    If m_oGlobalInfo Is Nothing Then Set m_oGlobalInfo = New globalInfo
    
    'check
    If m_oGlobalInfo Is Nothing Then
    
        MsgBox LoadResString(350), vbCritical
        End
    Else
        Set m_oGlobalInfo.excelApplication = m_oExcelApp
    End If
        
    
    m_oCML.Text = sCommandLine
    
    If m_oCML(CML_OPT_HELP).inCmdLine Then
        Set frmCmdLineOpt.cmdLine = m_oCML
        frmCmdLineOpt.Show vbModal
        Exit Function
    End If
    
    If m_oCML(CML_OPT_VER).inCmdLine Then
        showAboutForm
        Exit Function
    End If
    
    'if sql should run in oracle mode then put value to userprofile
    If m_oCML(CML_OPT_CM).Value = cnstORACLE Then
    
        m_emInitConMode = Oracle
    Else
        m_emInitConMode = Internal
    End If
    
    'set trace file
    m_oGlobalInfo.ConMan.traceFile = m_oCML(CML_OPT_TRACE).Value
            
    If m_oCML.exists(CML_OPT_NI) Then
    
        m_oGlobalInfo.ConMan.dbServer = m_oCML(CML_OPT_NI).Value
        m_oGlobalInfo.ConMan.database = ""
        m_oGlobalInfo.ConMan.UID = ""
    Else
        If m_oCML(CML_OPT_NODE).inCmdLine Or _
           m_oCML(CML_OPT_DB).inCmdLine Or _
           m_oCML(CML_OPT_USER).inCmdLine _
        Then
            m_oGlobalInfo.ConMan.dbServer = m_oCML(CML_OPT_NODE).Value
            m_oGlobalInfo.ConMan.database = m_oCML(CML_OPT_DB).Value
            
            sBuffer = m_oCML(CML_OPT_USER).Value
            lPos = InStr(sBuffer, Chr(34) + "," + Chr(34))
            'quoated uid,pwd
            If lPos > 0 Then
            
                m_oGlobalInfo.ConMan.UID = Left(sBuffer, lPos - 1)
                m_oGlobalInfo.ConMan.PWD = Right(sBuffer, Len(sBuffer) - lPos - 2)
            Else
                lPos = InStr(sBuffer, ",")
                'quoated uid,pwd
                If lPos > 0 Then
                
                    m_oGlobalInfo.ConMan.UID = UCase(Left(sBuffer, lPos - 1))
                    m_oGlobalInfo.ConMan.PWD = UCase(Right(sBuffer, Len(sBuffer) - lPos))
                Else
                    m_oGlobalInfo.ConMan.UID = sBuffer
                End If
            End If
        End If
    End If
    
    'rc
    checkCommandLine = True
            
    Exit Function

errCheckCommandLine:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      instDriver
'    description:   installs a odbc driver for sql studio
'    arguments:     /
'
'    return value:  name of the installed odbc driver
'
'=========================================================================================
Function instDriver() As String
    On Error GoTo errInstDriver
    
    Const FUNCTION_NAME = "instDriver"

    '=varibles===================================================================
    Dim lOut            As Long
    Dim lOut2           As Long
    Dim lRet            As Long
    Dim lUsageCount     As Long
    Dim sDriverName     As String
    Dim sAppPath        As String
    Dim bExists         As Boolean
    Dim sFullName1      As String
    Dim sFullName2      As String
    Dim sMessage        As String
    Dim bByteMessage()     As Byte
    Dim bByteDriverName()  As Byte
    Dim bByteAppPath()     As Byte
    Dim lRun               As Long
    '============================================================================
        
    'ret
    instDriver = ""
    
    sFullName1 = App.Path + "\" + "sqlod32.dll"
    sFullName2 = App.Path + "\" + "sqlsp32.dll"
    bExists = (Not (Dir(sFullName1) = "")) And Not ((Dir(sFullName2) = ""))
    
    If bExists = False Then
    
        MsgBox LoadResString(360)
        Exit Function
    End If
       
    sAppPath = App.Path
    sFullName2 = App.FileDescription + "_" + App.Path
    
    lOut = InStr(sFullName2, ":")
    If lOut > 0 Then sFullName1 = Left(sFullName2, lOut - 1) + Right(sFullName2, Len(sFullName2) - lOut)
        
    sFullName2 = sFullName1
    
    Do
        lOut = InStr(sFullName2, "\")
        If lOut > 0 Then
        
            sFullName1 = Left(sFullName2, lOut - 1) + Right(sFullName2, Len(sFullName2) - lOut)
            sFullName2 = sFullName1
        End If
    
    Loop Until lOut < 1
    
    Do
        lOut = InStr(sFullName2, " ")
        If lOut > 0 Then
        
            sFullName1 = Left(sFullName2, lOut - 1) + Right(sFullName2, Len(sFullName2) - lOut)
            sFullName2 = sFullName1
        End If
    Loop Until lOut < 1
    
    sDriverName = sFullName2 + vbNullChar + _
                 "Driver=sqlod32.dll" + vbNullChar + _
                 "Setup=sqlsp32.dll" + vbNullChar + _
                 "CPTimeout=0" + vbNullChar + vbNullChar
       
    'init unicode buffers
    bByteDriverName = sDriverName + vbNullChar
    bByteAppPath = sAppPath + vbNullChar
    ReDim bByteMessage((SQL_MAX_MESSAGE_LENGTH + 1) * 2)
    
    
    lRet = SQLInstallDriverExW(bByteDriverName(0), _
                               bByteAppPath(0), _
                               bByteMessage(0), _
                               SQL_MAX_MESSAGE_LENGTH, _
                               lOut, _
                               ODBC_INSTALL_COMPLETE, _
                               lUsageCount)
   
    If lRet = 0 Then
        Do
            lRun = lRun + 1
            'reinit unicode buffers
            ReDim bByteMessage((SQL_MAX_MESSAGE_LENGTH + 1) * 2)
        
            lRet = SQLInstallerErrorW(lRun, lOut2, bByteMessage(0), SQL_MAX_MESSAGE_LENGTH, lOut)
            sMessage = Left$(bByteMessage, lOut)
            If lRet = SQL_SUCCESS Or lRet = SQL_SUCCESS_WITH_INFO Then
                
                MsgBox sMessage
                'Exit Function
            Else
                MsgBox LoadResString(360)
                Exit Function
            End If
        Loop Until lRet <> SQL_NO_DATA
    End If
    
    'lRun <1 no error occured
    If lRun < 1 Then instDriver = sFullName2

    Exit Function

errInstDriver:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      removeDriver
'    description:   removes a odbc driver which was installed sql studio
'    arguments:     [in] sDriverName, name od driver
'
'    return value:  empty string if successfully
'
'=========================================================================================
Function removeDriver(sDriverName As String) As String
    On Error GoTo errRemoveDriver
    
    Const FUNCTION_NAME = "removeDriver"
    
    '=varibles===================================================================
    Dim lUsage          As Long
    Dim lRet            As Long
    Dim lOut            As Long
    Dim lOut2           As Long
    Dim sMessage        As String
    
    Dim bByteDriverName() As Byte
    Dim bByteMessage()  As Byte
    Dim lRun            As Long
    Dim iCnt            As Integer
    Dim sMyPath          As String
    '============================================================================
    
    'init unicode buffers
    bByteDriverName = sDriverName + vbNullChar
        
    lRet = 1
    
    sMyPath = App.Path + "\" + App.EXEName + ".exe"
    
    iCnt = m_oComFunc.getInstancecount(sMyPath)
    If iCnt <= 1 Then
        Do
            lRet = SQLRemoveDriverW(bByteDriverName(0), 1, lUsage)
        Loop Until lUsage <= 0
    End If
    
    
    If lRet = 0 Then
            
        Do
            lRun = lRun + 1
            'reinit unicode buffers
            ReDim bByteMessage((SQL_MAX_MESSAGE_LENGTH + 1) * 2)
            
            lRet = SQLInstallerErrorW(lRun, lOut2, bByteMessage(0), SQL_MAX_MESSAGE_LENGTH, lOut)
            sMessage = Left$(bByteMessage, lOut)
            If lRet = SQL_SUCCESS Or lRet = SQL_SUCCESS_WITH_INFO Then
            
                MsgBox sMessage
            Else
                Exit Function
            End If
        Loop Until lRet <> SQL_NO_DATA
    End If
    
    removeDriver = ""

    Exit Function

errRemoveDriver:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      connect
'    description:   connect to database
'    arguments:     /
'
'    return value:  true if successfully
'
'=========================================================================================
Function connect() As Boolean
    On Error GoTo errConnect
    
    Const FUNCTION_NAME = "connect"
       
    '=varibles===================================================================
    Dim bConnectSuccess As Boolean
    Dim sDriverName     As String
    Dim sLocalRoot      As String
    Dim lRun            As Long
    Dim oConnection     As SQLStudioDBLayer.connection
    Dim emSQLMode       As sqlMode
    '============================================================================
            
    'rc
    connect = False
    
    'init
    bConnectSuccess = False
    Set m_oAdminCon = Nothing
    
    'ADIS ErrorId Begin: 1000796
    If m_oGlobalInfo.ConMan.hwnd <> frmMain.hwnd Then m_oGlobalInfo.ConMan.hwnd = frmMain.hwnd
    'ADIS ErrorId End: 1000796
    
    'create history list for statements in dsql dialog
    Set m_oStatementList = m_oComFunc.createHistory(100)
    m_oStatementList.checkDoubleEntry = deStringGeneral
            
    'if no ODBC Driver is installed exit function
    If Len(m_sDriverName) < 1 Then
    
        MsgBox LoadResString(360)
        Exit Function
    End If
    
'====================================================================
'set pooling mode for conection manager
'
'     WA_DBC_EXCL: The returned connection is
'         currently unused And won 't be returned
'         for another Connect until Disconnect is called.
'     WA_DBC_EXCL_AUTOCOMMIT_OFF: Only autocommit
'         off connections are exclusiv. Autocommit on
'         connections are shared.
'     WA_DBC_SHARED: The connection may be returned
'         by several Connects. Only if
'         #Connects-#Disconntecs is 0 the connection can be
'         used exclusive.
'====================================================================

        
    
    m_oGlobalInfo.ConMan.poolingMode = WA_DBC_EXCL_AUTOCOMMIT_OFF
    
    'set driver name
    m_oGlobalInfo.ConMan.driver = m_sDriverName
    
    'check for NO timeout value
    If frmMain.chkNoTimeOut.Value = vbChecked Then _
       m_oGlobalInfo.ConMan.timeout = 0
    
    'check the timeout value for ni connection
    'the value has to be determined before connect
    'if the connection params are like this
    'we can asume its a ni call
    If m_oGlobalInfo.ConMan.dbServer <> "" And _
        m_oGlobalInfo.ConMan.database = "" And _
        m_oGlobalInfo.ConMan.UID = "" And _
        m_oGlobalInfo.ConMan.PWD = "" Then
        
        frmMain.chkNoTimeOut.Value = vbChecked
        m_oGlobalInfo.ConMan.timeout = 0
    End If
    
    'turn hour glass on
    m_oGlobalInfo.Hourglass.State = qpHourglassOn
    
   '"Connecting ..."
    setState LoadResString(101), False
    
    'overwrite value from userprofile with value set by command line
    If m_emInitConMode = Oracle Then
    
        m_oGlobalInfo.ConMan.DriverSQLMode = m_emInitConMode
    End If
    
    'get the first connection
    bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                           m_oGlobalInfo.ConMan.DriverIsoLevel, _
                           m_oGlobalInfo.ConMan.DriverSQLMode, _
                           AutoCommitOn, _
                           oConnection)
                           
    If Not bConnectSuccess Then
    
        'end application if ni mode is active
        If m_oGlobalInfo.ConMan.isNIConnection Then End
    
        '"Connection failed ..."
        setState LoadResString(100), True
        
        'turn hour glass off
        m_oGlobalInfo.Hourglass.State = qpHourglassOff
        
        'if there is no connection exit function
        If oConnection Is Nothing Then Exit Function
        
        'if there is a error
        If Len(oConnection.Error.Text) > 0 Then
        
            'show error
            oConnection.Error.showError
            'end application if ni mode is active
            If Not oConnection Is Nothing Then
                If oConnection.isNIConnection Then End
            End If
            
            'react on this sql state
            If oConnection.Error.sqlState = "[ODBC] [IM002] [ODBC Driver Manager] Data source name not found and no default driver specified" Then
            
                m_oGlobalInfo.ConMan.UID = ""
                m_oGlobalInfo.ConMan.PWD = ""
                m_oGlobalInfo.ConMan.database = ""
                m_oGlobalInfo.ConMan.dbServer = ""
                                
                bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                                      m_oGlobalInfo.ConMan.DriverIsoLevel, _
                                      m_oGlobalInfo.ConMan.DriverSQLMode, _
                                      AutoCommitOn, _
                                      oConnection)
                If Not bConnectSuccess Then
                
                     If Len(oConnection.Error.Text) > 0 Then oConnection.Error.showError
                        Exit Function
                End If
            End If
        Else
            If Len(oConnection.Error.sqlState) > 0 Then
            
                MsgBox oConnection.Error.sqlState
             Else
                showError142 oConnection
            End If
        End If
        
        'end application if ni mode is active
        If Not oConnection Is Nothing Then
            If oConnection.isNIConnection Then End
        End If
            
        Exit Function
    Else
        If Len(oConnection.userGroup) < 1 Then
        'there are no systemtables, open auto off connection
        
            MsgBox LoadResString(116), vbCritical
                        
            '===========================================================
            'first disconnect autocommit on
            'release connection, very important for pooling mode <> WA_NONE
            
            m_oGlobalInfo.ConMan.disconnect
            m_oGlobalInfo.ConMan.poolingMode = WA_DBC_EXCL_AUTOCOMMIT_OFF
            '===========================================================
            
            m_oGlobalInfo.ConMan.PWD = ""
            
            'reinit pointer
            Set oConnection = Nothing
            
            'get auto off connection
            bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                                     m_oGlobalInfo.ConMan.DriverIsoLevel, _
                                     m_oGlobalInfo.ConMan.DriverSQLMode, _
                                     AutoCommitOff, _
                                     oConnection)
                     
            If Not bConnectSuccess Then
            
                m_oGlobalInfo.Hourglass.State = qpHourglassOff
                
                If Len(oConnection.Error.Text) > 0 Then oConnection.Error.showError
                Exit Function
            Else
               'open one DirectSQLWindow
               m_oGlobalInfo.systemtablesLoaded = False
               
               'set userprofile properties
               m_oGlobalInfo.userProfile.confirmation = False
               m_oGlobalInfo.userProfile.storePassword = False
               
               MsgBox LoadResString(143) + _
                        " (" + _
                        LoadResString(146) + sqlModeAsString(oConnection.sqlMode) + _
                        ", " + _
                        LoadResString(145) + isoLevelAsString(oConnection.isoLevel) + _
                        ", " + _
                        LoadResString(144) + ")", _
                        vbOKOnly
               
               showDirectSQLAutoCommitOff oConnection
            End If
        Else
            m_oGlobalInfo.systemtablesLoaded = True
        End If
    End If
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOff
        
    '==================================================================
    'now there is a valid connection, remember it, use it as an admin connection
    '==================================================================
    Set m_oAdminCon = oConnection
        
    'get user profile
    If m_oGlobalInfo.systemtablesLoaded Then _
       m_oGlobalInfo.userProfile.loadUserProfile m_oAdminCon
       
    'overwrite value from userprofile with value set by command line
    If m_emInitConMode = Oracle Then
    
        m_oGlobalInfo.userProfile.AppManagment.sqlMode = Oracle
    End If
           
    'check local root
    sLocalRoot = m_oGlobalInfo.userProfile.localRoot
    Call checkLocalRoot(sLocalRoot)
    m_oGlobalInfo.userProfile.localRoot = sLocalRoot
       
    'check for NI
    If m_oAdminCon.isNIConnection Then frmMain.chkTinyMode.Value = vbChecked
        
    'set a connection object
    Set frmMain.ctlCM.connection = m_oAdminCon
    'set a global com object
    Set frmMain.ctlCM.globalInfo = m_oGlobalInfo
    
    'enable some buttons from the toolbar
    frmMain.tbrMain.Buttons.item("DirectSQL").Enabled = m_oGlobalInfo.systemtablesLoaded
    frmMain.tbrMain.Buttons.item("DBError").Enabled = True
    frmMain.tbrMain.Buttons.item("Connect").Image = "Disconnect"
    frmMain.tbrMain.Buttons.item("Connect").ToolTipText = "Disconnect"
           
    'if WAN Mode is not activated
    If frmMain.chkTinyMode.Value = vbUnchecked Then
                
        'enable some menus if necessary systemtables are loaded
        frmMain.mnuViewDirectSQLAC.Enabled = m_oGlobalInfo.systemtablesLoaded
        frmMain.mnuViewNewQueryByExample.Enabled = m_oGlobalInfo.systemtablesLoaded
        frmMain.mnuViewNewVisualQuery.Enabled = m_oGlobalInfo.systemtablesLoaded
        frmMain.mnuSettings.Enabled = m_oGlobalInfo.systemtablesLoaded
        
        'enable some buttons from the toolbar
        frmMain.tbrMain.Buttons.item("DirectSQLAutoCommitOff").Enabled = m_oGlobalInfo.ConMan.isNotExclusive And m_oGlobalInfo.systemtablesLoaded
        frmMain.tbrMain.Buttons.item("VisualQuery").Enabled = m_oGlobalInfo.systemtablesLoaded
        frmMain.tbrMain.Buttons.item("QueryByExample").Enabled = m_oGlobalInfo.systemtablesLoaded
        frmMain.tbrMain.Buttons.item("CatalogManagement").Enabled = m_oGlobalInfo.systemtablesLoaded
        
        frmMain.mnuView.Enabled = True
        frmMain.mnuWindow.Enabled = True
        frmMain.lstTableFilter.Enabled = True
        
        'load tables filter list
        For lRun = 1 To 10
         
            If Len(m_oGlobalInfo.userProfile.getTableFilterList(lRun)) > 0 _
                And _
               m_oGlobalInfo.userProfile.getTableFilterList(lRun) <> "*" Then
                frmMain.lstTableFilter.AddItem m_oGlobalInfo.userProfile.getTableFilterList(lRun)
            End If
        Next
        'add one empty item
        frmMain.lstTableFilter.AddItem ""
        
        If m_oGlobalInfo.userProfile.tableFilter <> "*" Then
            frmMain.lstTableFilter.Text = m_oGlobalInfo.userProfile.tableFilter
        End If
        
        If m_oGlobalInfo.systemtablesLoaded Then
            
            'load supposelist
            m_oAutoSuppose.supposeTypeSQL = True
            m_oAutoSuppose.supposeTypeInfo = True
            m_oAutoSuppose.supposeTypeStatement = True
            m_oAutoSuppose.supposeTypeUser = True
            m_oAutoSuppose.loadSupposeLists oConnection
            
            'enable all features
            frmMain.ctlCM.tinyMode = False
            frmMain.ctlCM.useLocalFolderInd = m_oGlobalInfo.userProfile.localFolderIndependent
            'load all items
            
            frmMain.ctlCM.loadCatalog _
                                m_oGlobalInfo.ConMan.UID, _
                                frmMain.lstTableFilter.Text, _
                                itemAll
                        
            'set open nodes ...
            frmMain.ctlCM.setNodeInfo m_oGlobalInfo.userProfile.expandedNodes, m_oGlobalInfo.userProfile.restoreCatalogState
        End If
        
        'show tree just if the systemtables are loaded
        frmMain.ctlCMContainer.Visible = m_oGlobalInfo.systemtablesLoaded
        frmMain.ctlCM.Visible = m_oGlobalInfo.systemtablesLoaded
    Else
        'WAN Mode
        'disable all features, open the tree with stored (local) folders
        frmMain.ctlCM.tinyMode = True
        frmMain.ctlCM.useLocalFolderInd = m_oGlobalInfo.userProfile.localFolderIndependent
        frmMain.ctlCM.loadCatalog _
                            m_oGlobalInfo.ConMan.UID, _
                            frmMain.lstTableFilter.Text, _
                            itemStoredObjects
                                
        frmMain.mnuViewNewQueryByExample.Enabled = False
        frmMain.mnuViewNewVisualQuery.Enabled = False
        frmMain.mnuViewDirectSQLAC.Enabled = False
        frmMain.mnuSettings.Enabled = False
        frmMain.tbrMain.Buttons.item("DirectSQLAutoCommitOff").Enabled = False
        
        
        'show tree, because there is the local storage folder
        frmMain.ctlCMContainer.Visible = True
        frmMain.ctlCM.Visible = True
    End If
    
    'if catalog is visible set focus
    If frmMain.ctlCM.Visible = True Then
    
        frmMain.ctlCM.SetFocus
        frmMain.ctlCMContainer.Width = frmMain.Width / 4
    End If
        
    'check menu
    frmMain.mnuViewDirectSQL.Enabled = m_oGlobalInfo.systemtablesLoaded
    frmMain.mnuFileDisconnect.Enabled = True
    
    'disable connect params
    frmMain.chkTinyMode.Enabled = False
    frmMain.chkNoTimeOut.Enabled = False
    
    If frmMain.mnuFileClearLRUList.Visible Then _
       frmMain.mnuFileClearLRUList.Visible = False
    
    'check for NI Connetion
    If Not oConnection.isNIConnection Then
    
        'add connection to LRU List
        m_oLRUConnections.addLRU m_oGlobalInfo.ConMan.driver, _
                                 m_oGlobalInfo.ConMan.UID, _
                                 m_oGlobalInfo.ConMan.PWD, _
                                 m_oGlobalInfo.ConMan.database, _
                                 m_oGlobalInfo.ConMan.dbServer
    End If
    '=================================================================
    'first load LRU Connection, then the function OpenStoredObjects
    'in the form of StoredObjects can load the new LRU Connection order
    '=================================================================
    setLRUList
    
    If m_oGlobalInfo.systemtablesLoaded Then
    
        If oConnection.isNIConnection Then
        
            openNewDirectSQL AutoCommitOn
        Else
            openNewDirectSQL m_oAdminCon.AutoCommit
        End If
    End If
    
    'Show connection state
    setCompleteState "", _
                     m_oGlobalInfo.ConMan.UID, _
                     m_oGlobalInfo.ConMan.database, _
                     m_oGlobalInfo.ConMan.dbServer
                     
    If oConnection.isNIConnection Then setTaskIDToMainWnd
        
    connect = True
    
    Exit Function

errConnect:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      disconnect
'    description:   disconnects sql studio from the database
'    arguments:     [out] Cancel
'
'    return value:
'
'=========================================================================================
Sub disconnect(nCancel As Integer)
    On Error GoTo errDisconnect
    
    Const FUNCTION_NAME = "disconnect"
    
    '=varibles===================================================================
    Dim lRun            As Long
    '============================================================================
        
    'inits
    m_oGlobalInfo.disconnectCanceled = False
              
    'unload all database connected forms
    For lRun = Forms.count - 1 To 0 Step -1
    
        If Forms(lRun).Caption <> frmMain.Caption Then
        
            Unload Forms(lRun)
            If m_oGlobalInfo.disconnectCanceled Then    'Disconnect canceled?
            
                nCancel = 1
                Exit Sub
            End If
        End If
    Next
    
    'disconnect canceled, the globalcom property can be set the user by a dialog
    'which pops up when a form is unloading
    'and the data of the form has changed
    If m_oGlobalInfo.disconnectCanceled Then
    
        nCancel = 1
        Exit Sub
    End If
    
    If m_oGlobalInfo.systemtablesLoaded And frmMain.chkTinyMode.Value = vbUnchecked Then
    
        'clear open results before MDI unloading => if not crash !!!
        frmMain.ctlCM.clearResults
        frmMain.ctlCM.stopLoading = True
        
        If m_oGlobalInfo.ConMan.isConnected Then
            
               m_oGlobalInfo.userProfile.clearFilterList
               m_oGlobalInfo.userProfile.tableFilter = frmMain.lstTableFilter.Text
               
               For lRun = 0 To frmMain.lstTableFilter.ListCount - 1
               
                   m_oGlobalInfo.userProfile.addTableFilterList _
                                   frmMain.lstTableFilter.List(lRun)
               Next
               
               m_oGlobalInfo.userProfile.expandedNodes = frmMain.ctlCM.getNodeInfo
               
               If Not m_oAdminCon Is Nothing Then
               
                   m_oGlobalInfo.userProfile.saveUserProfile m_oAdminCon
               End If
        End If
    End If
    
    'ADIS Error 1106639
    If m_oGlobalInfo.systemtablesLoaded Then
    
        'if system tables are loaded
        m_oLRUConnections.savePassWord = m_oGlobalInfo.userProfile.storePassword
    Else
        'if no system tables are loaded, don't save password,
        ' because UserProfilestoredFileContent.StorePassword is not known
        m_oLRUConnections.savePassWord = False
    End If
    
    'just call ApplyPasswordOption once, if there was a open session and not in Query_unload form
    If m_oGlobalInfo.ConMan.isConnected Then m_oLRUConnections.applyPasswordOption
    
    'Disconnect
    If m_oGlobalInfo.ConMan.disconnect Then
    
        'Connection disabling
        frmMain.tbrMain.Buttons.item("Execute").Enabled = False
        frmMain.tbrMain.Buttons.item("DirectSQL").Enabled = False
        frmMain.tbrMain.Buttons.item("DirectSQLAutoCommitOff").Enabled = False
        frmMain.tbrMain.Buttons.item("VisualQuery").Enabled = False
        frmMain.tbrMain.Buttons.item("QueryByExample").Enabled = False
        frmMain.tbrMain.Buttons.item("CatalogManagement").Enabled = False
        frmMain.tbrMain.Buttons.item("DBError").Enabled = False
        
        frmMain.tbrMain.Buttons.item("Connect").Image = "Connect"
        frmMain.tbrMain.Buttons.item("Connect").ToolTipText = "Connect"
        
        
        If Not frmMain.mnuFileClearLRUList.Visible Then frmMain.mnuFileClearLRUList.Visible = True
        
        frmMain.mnuFileDisconnect.Enabled = False
        frmMain.mnuView.Enabled = False
        frmMain.mnuWindow.Enabled = False
        
        frmMain.lstTableFilter.Text = ""
        For lRun = 0 To frmMain.lstTableFilter.ListCount - 1
        
            frmMain.lstTableFilter.RemoveItem 0
        Next
                
        frmMain.ctlCM.clear

        frmMain.lstTableFilter.Enabled = False
        frmMain.chkTinyMode.Enabled = True
        frmMain.chkNoTimeOut.Enabled = True
        
        frmMain.ctlCM.Visible = False
        frmMain.ctlCMContainer.Visible = False
        
        Set m_oAutoSuppose = Nothing
    End If
    
    'Set disconnected state
    setCompleteState LoadResString(113), "", "", ""

    Exit Sub

errDisconnect:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openNewDirectSQL
'    description:   opens a direct sql window
'    arguments:     [in] Autocommit, flag to indicates autocommit mode for sql window
'
'    return value:   /
'
'=========================================================================================
Sub openNewDirectSQL(AutoCommit As AutoCommit)
    On Error GoTo errOpenNewDirectSQL
    
    Const FUNCTION_NAME = "openNewDirectSQL"
    
    '=varibles===================================================================
    Dim bConnectSuccess         As Boolean
    Dim oConnection             As SQLStudioDBLayer.connection
    Dim NewForm                 As New frmDirectSQL
    '=========================================================================
    
    'init
    bConnectSuccess = False
    
    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If
    
    'only a not exclusive user can open another database connection
    If AutoCommit = AutoCommitOff Then
    
        If m_oGlobalInfo.ConMan.isNotExclusive = False Then
        
            MsgBox LoadResString(114)
            Exit Sub
        End If
    End If
    
    'connect
    setState LoadResString(101), False
    m_oGlobalInfo.Hourglass.State = qpHourglassOn
    
    If m_oGlobalInfo.ConMan.isNotExclusive And Not m_oGlobalInfo.ConMan.isNIConnection Then
        
        bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                                m_oGlobalInfo.userProfile.AppManagment.isoLevel, _
                                m_oGlobalInfo.userProfile.AppManagment.sqlMode, _
                                AutoCommit, _
                                oConnection)
    Else
        bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                                m_oGlobalInfo.ConMan.DriverIsoLevel, _
                                m_oGlobalInfo.ConMan.DriverSQLMode, _
                                AutoCommit, _
                                oConnection)
    End If
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOff
    
    If Not bConnectSuccess Then
    
        showCriticalConMsg oConnection
        Exit Sub
    End If
    
    'load form
    Load NewForm
    
    'set properties
    NewForm.Caption = LoadResString(207)
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.statementList = m_oStatementList
    Set NewForm.connection = oConnection
    
    Set NewForm.StoredFileContent = Nothing
      
    'show form
    NewForm.Show
    If NewForm.Visible Then NewForm.SetFocus
    
    NewForm.refreshFont
    
    setState "", False
    
    Exit Sub
    
errOpenNewDirectSQL:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openNewTableDefinition
'    description:   opens a table definition window
'    arguments:     /
'
'    return value:   /
'
'=========================================================================================
Sub openNewTableDefinition()
    On Error GoTo errOpenNewTableDefinition
    
    Const FUNCTION_NAME = "openNewTableDefinition"

    '=varibles===================================================================
    Dim bConnectSuccess         As Boolean
    Dim oConnection             As SQLStudioDBLayer.connection
    Dim NewForm                 As New frmCreateTable
    '=========================================================================

    'init
    bConnectSuccess = False

    If Forms.count > m_csMaxForm Then

        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If

    'connect
    setState LoadResString(101), False
    m_oGlobalInfo.Hourglass.State = qpHourglassOn

    If m_oGlobalInfo.ConMan.isNotExclusive Then
        bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                                    m_oGlobalInfo.userProfile.AppManagment.isoLevel, _
                                    Internal, _
                                    AutoCommitOff, _
                                    oConnection)

        If Not bConnectSuccess Then
    
            showCriticalConMsg oConnection
            Exit Sub
        End If
    Else
        Exit Sub
    End If
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOff

    'load form
    Load NewForm
    
    'set properties
    NewForm.Caption = LoadResString(215)
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.connection = oConnection
    Set NewForm.StoredFileContent = Nothing
    NewForm.mode = 0
    
    'show form
    NewForm.Show
    If NewForm.Visible Then NewForm.SetFocus
    
    setState "", False
    
    Exit Sub
    
errOpenNewTableDefinition:
    m_oGlobalInfo.Hourglass.State = qpHourglassOff
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openViewDefinition
'    description:   opens an existing view definition
'    arguments:     [in] sName  - Name of the view
'                   [in] sOwner - Owner of the view
'                   [in] sNew   - Flag if a new or existing view will be opened
'    return value:   /
'
'=========================================================================================
Sub openViewDefinition(sName As String, sOwner As String, bNew As Boolean)
    On Error GoTo errOpenViewDefinition
    
    Const FUNCTION_NAME = "openViewDefinition"

    '=varibles===================================================================
    Dim bConnectSuccess         As Boolean
    Dim oConnection             As SQLStudioDBLayer.connection
    Dim NewForm                 As New frmCreateView
    Dim sql                     As String
    '=========================================================================
    
    'init
    bConnectSuccess = False

    If Forms.count > m_csMaxForm Then
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If

    'connect
    setState LoadResString(101), False
    
    
    If bNew Then

        m_oGlobalInfo.Hourglass.State = qpHourglassOn

        bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                                m_oGlobalInfo.userProfile.AppManagment.isoLevel, _
                                Internal, _
                                AutoCommitOff, _
                                oConnection)

        m_oGlobalInfo.Hourglass.State = qpHourglassOff

        If Not bConnectSuccess Then

            showCriticalConMsg oConnection
            Exit Sub
        End If
    Else
        'if view def is shown, open a standard commit on connection
        bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                                                m_oGlobalInfo.ConMan.DriverIsoLevel, _
                                                m_oGlobalInfo.ConMan.DriverSQLMode, _
                                                AutoCommitOn, _
                                                oConnection)
        If Not bConnectSuccess Then
    
            showCriticalConMsg oConnection
            Exit Sub
        End If
    End If
        
    If Not bNew Then
    
        'check if required data is available
        sql = "SELECT definition FROM viewdefs WHERE viewname = '" + sName + "' AND owner = '" + sOwner + "'"
        If checkResult(sql) = False Then
            MsgBox LoadResString(226), vbCritical, LoadResString(230)
            Exit Sub
        End If
    End If
    
    'load form
    Load NewForm
    
    'set properties
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.connection = oConnection
    Set NewForm.StoredFileContent = Nothing
    NewForm.appVersion = CLng(SQLSTOBJECTSVERSION)
    
    If bNew = True Then
        NewForm.Caption = LoadResString(218)
    Else
        NewForm.Caption = LoadResString(222)
        NewForm.loadViewDefinition sName, sOwner
    End If
    
    'show form
    NewForm.Show
    If NewForm.Visible Then NewForm.SetFocus
    
    setState "", False
    
    Exit Sub
    
errOpenViewDefinition:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openIndexDefinition
'    description:   opens an existing index definition
'    arguments:     [in] sName  - Name of the index
'                   [in] sOwner - Owner of the index
'                   [in] bNew   - Flag if a new or existing index will be opened
'    return value:   /
'
'=========================================================================================
Sub openIndexDefinition(sName As String, _
                        sOwner As String, _
                        sTable As String, _
                        bNew As Boolean)
    On Error GoTo errOpenIndexDefinition
    
    Const FUNCTION_NAME = "openIndexDefinition"

    '=varibles===================================================================
    Dim bConnectSuccess         As Boolean
    Dim oConnection             As SQLStudioDBLayer.connection
    Dim NewForm                 As New frmCreateIndex
    Dim sql                     As String
    '=========================================================================

    'init
    bConnectSuccess = False

    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If

    'connect
    setState LoadResString(101), False

    If bNew Then
    
        m_oGlobalInfo.Hourglass.State = qpHourglassOn
        
        bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                                m_oGlobalInfo.userProfile.AppManagment.isoLevel, _
                                Internal, _
                                AutoCommitOff, _
                                oConnection)

        m_oGlobalInfo.Hourglass.State = qpHourglassOff
        
        If Not bConnectSuccess Then
    
            showCriticalConMsg oConnection
            Exit Sub
        End If
    Else
        'if view def is shown, open a standard commit on connection
        bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                                                m_oGlobalInfo.ConMan.DriverIsoLevel, _
                                                m_oGlobalInfo.ConMan.DriverSQLMode, _
                                                AutoCommitOn, _
                                                oConnection)
        If Not bConnectSuccess Then
    
            showCriticalConMsg oConnection
            Exit Sub
        End If
    End If
        

    If Not bNew Then
    
        'check if required data is available
        sql = "SELECT columnname, sort FROM indexcolumns WHERE indexname = '" + sName + "' AND owner = '" + sOwner + "'" + " AND TABLENAME ='" + sTable + "'"
        If checkResult(sql) = False Then
            MsgBox LoadResString(227), vbCritical, LoadResString(230)
            Exit Sub
        End If
    End If
    
    'load form
    Load NewForm
    
    'set properties
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.connection = oConnection
    NewForm.appVersion = CLng(SQLSTOBJECTSVERSION)
    
    If bNew = True Then
        NewForm.Caption = LoadResString(219)
        NewForm.loadColumnDefinition sName
    Else
        NewForm.Caption = LoadResString(223)
        NewForm.loadIndexDefinition sName, sOwner, sTable
    End If
    
    'show form
    NewForm.Show
    If NewForm.Visible Then NewForm.SetFocus
    
    setState "", False
    
    Exit Sub
    
errOpenIndexDefinition:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openSynonymDefinition
'    description:   opens an existing synonym definition
'    arguments:     [in] sName  - Name of the synonym
'                   [in] sOwner - Owner of the synonym
'                   [in] sNew   - Flag if a new or existing synonym will be opened
'    return value:   /
'
'=========================================================================================
Sub openSynonymDefinition(sName As String, sOwner As String, bNew As Boolean)
    On Error GoTo errOpenSynonymDefinition
    
    Const FUNCTION_NAME = "openSynonymDefinition"

    '=varibles===================================================================
    Dim bConnectSuccess         As Boolean
    Dim oConnection             As SQLStudioDBLayer.connection
    Dim NewForm                 As New frmCreateSynonym
    Dim sql                     As String
    '=========================================================================

    'init
    bConnectSuccess = False

    If Forms.count > m_csMaxForm Then
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If

    'connect
    setState LoadResString(101), False

    If bNew Then
    
        m_oGlobalInfo.Hourglass.State = qpHourglassOn
        bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                                    m_oGlobalInfo.userProfile.AppManagment.isoLevel, _
                                    Internal, _
                                    AutoCommitOff, _
                                    oConnection)

        m_oGlobalInfo.Hourglass.State = qpHourglassOff
        
        If Not bConnectSuccess Then
    
            showCriticalConMsg oConnection
            Exit Sub
        End If
    Else
        'if view def is shown, open a standard commit on connection
        bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                                                m_oGlobalInfo.ConMan.DriverIsoLevel, _
                                                m_oGlobalInfo.ConMan.DriverSQLMode, _
                                                AutoCommitOn, _
                                                oConnection)
        If Not bConnectSuccess Then
    
            showCriticalConMsg oConnection
            Exit Sub
        End If
    End If
        
    If Not bNew Then
    
        'check if required data is available
        sql = "SELECT tablename, public, comment FROM synonyms WHERE synonymname = '" + sName + "' AND owner = '" + sOwner + "'"
        If checkResult(sql) = False Then
            MsgBox LoadResString(228), vbCritical, LoadResString(230)
            Exit Sub
        End If
    End If
    
    'load form
    Load NewForm
    
    'set properties
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.connection = oConnection
    NewForm.appVersion = CLng(SQLSTOBJECTSVERSION)
    
    If bNew = True Then
        NewForm.Caption = LoadResString(220)
        NewForm.dragDropActions sName
    Else
        NewForm.Caption = LoadResString(224)
        NewForm.loadSynonymDefinition sName, sOwner
    End If
    
    'show form
    NewForm.Show
    If NewForm.Visible Then NewForm.SetFocus
    
    setState "", False
    
    Exit Sub
    
errOpenSynonymDefinition:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openSequenceDefinition
'    description:   opens an existing sequence definition
'    arguments:     [in] sName  - Name of the sequence
'                   [in] sOwner - Owner of the sequence
'                   [in] sNew   - Flag if a new or existing sequence will be opened
'    return value:   /
'
'=========================================================================================
Sub openSequenceDefinition(sName As String, sOwner As String, bNew As Boolean)
    On Error GoTo errOpenSequenceDefinition
    
    Const FUNCTION_NAME = "openSequenceDefinition"

    '=varibles===================================================================
    Dim bConnectSuccess         As Boolean
    Dim oConnection             As SQLStudioDBLayer.connection
    Dim NewForm                 As New frmCreateSequence
    Dim sql                     As String
    '=========================================================================

    'init
    bConnectSuccess = False

    If Forms.count > m_csMaxForm Then
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If

    'connect
    setState LoadResString(101), False

    If bNew Then
    
        m_oGlobalInfo.Hourglass.State = qpHourglassOn
        
        bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                                m_oGlobalInfo.userProfile.AppManagment.isoLevel, _
                                Internal, _
                                AutoCommitOff, _
                                oConnection)

        m_oGlobalInfo.Hourglass.State = qpHourglassOff
        If Not bConnectSuccess Then
    
            showCriticalConMsg oConnection
            Exit Sub
        End If
    Else
        'if view def is shown, open a standard commit on connection
        bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                                                m_oGlobalInfo.ConMan.DriverIsoLevel, _
                                                m_oGlobalInfo.ConMan.DriverSQLMode, _
                                                AutoCommitOn, _
                                                oConnection)
        If Not bConnectSuccess Then
    
            showCriticalConMsg oConnection
            Exit Sub
        End If
    End If
    
    If Not bNew Then
    
        'check if required data is available
        sql = "SELECT min_value, max_value, increment_by, cycle_flag, order_flag, cache_size, comment FROM sequences WHERE sequence_name = '" + sName + "' AND owner = '" + sOwner + "'"
        If checkResult(sql) = False Then
            MsgBox LoadResString(229), vbCritical, LoadResString(230)
            Exit Sub
        End If
    End If
    
    'load form
    Load NewForm
    
    'set properties
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.connection = oConnection
    NewForm.appVersion = CLng(SQLSTOBJECTSVERSION)
    If bNew = True Then
        NewForm.Caption = LoadResString(221)
    Else
        NewForm.Caption = LoadResString(225)
        NewForm.loadSequenceDefinition sName, sOwner
    End If
    
    'show form
    NewForm.Show
    If NewForm.Visible Then NewForm.SetFocus
    
    setState "", False
    
    Exit Sub
    
errOpenSequenceDefinition:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      showDirectSQLAutoCommitOff
'    description:   opens a dsql dialog window, in autocommit off mode
'    arguments:     oConnection, connection object
'
'    return value:   /
'
'=========================================================================================
Sub showDirectSQLAutoCommitOff(oConnection As SQLStudioDBLayer.connection)
    On Error GoTo errShowDirectSQLAutoCommitOff
    
    Const FUNCTION_NAME = "showDirectSQLAutoCommitOff"
        
    '=varibles===================================================================
    Dim NewForm                 As New frmDirectSQL
    '=========================================================================
    
    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If
    
    'load form
    Load NewForm
    
    'set properties
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.connection = oConnection
    Set NewForm.StoredFileContent = Nothing
    
    NewForm.mnuViewDirectSQL.Enabled = m_oGlobalInfo.systemtablesLoaded
    NewForm.mnuViewDirectSQLAC.Enabled = m_oGlobalInfo.systemtablesLoaded
    NewForm.mnuViewNewQueryByExample.Enabled = m_oGlobalInfo.systemtablesLoaded
    NewForm.mnuViewNewVisualQuery.Enabled = m_oGlobalInfo.systemtablesLoaded
      
    'show form
    NewForm.Show
    
    If m_oGlobalInfo.systemtablesLoaded = False Then removeCloseButton NewForm
        
    If NewForm.Visible Then NewForm.SetFocus
        
    setState "", False
    
    Exit Sub
    
errShowDirectSQLAutoCommitOff:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openNewVisualQuery
'    description:   shows visual query dialog
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function openNewVisualQuery()
    On Error GoTo errOpenNewVisualQuery
    
    Const FUNCTION_NAME = "openNewVisualQuery"
    
    '=varibles===================================================================
    Dim bConnectSuccess         As Boolean
    Dim oConnection             As SQLStudioDBLayer.connection
    Dim NewForm                 As New frmVisualQuery
    '=========================================================================
    
    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Function
    End If
    
    bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                            m_oGlobalInfo.userProfile.AppManagment.isoLevel, _
                            Internal, _
                            AutoCommitOn, oConnection)

    If Not bConnectSuccess Then
    
        setState LoadResString(100), True
        Exit Function
    End If
    
    'load form
    Load NewForm
    
    'set properties
    Set NewForm.globalInfo = m_oGlobalInfo
    
    'set new object description
    NewForm.Caption = LoadResString(213)
    Set NewForm.connection = oConnection
    Set NewForm.StoredFileContent = Nothing
          
    'show form
    NewForm.Show
    
    Exit Function
    
errOpenNewVisualQuery:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      openNewQueryByExample
'    description:   shows query by example dialog window
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Sub openNewQueryByExample()
    On Error GoTo ErrOpenNewQueryByExample
    
    Const FUNCTION_NAME = "openNewQueryByExample"

    '=variables========================================================================
    Dim bConnectSuccess         As Boolean
    Dim oConnection             As SQLStudioDBLayer.connection
    Dim NewForm                  As Object
    '=========================================================================
    
    Set NewForm = New frmQuickQBE
    
    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If
    
    bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                        m_oGlobalInfo.userProfile.AppManagment.isoLevel, _
                        Internal, _
                        AutoCommitOn, oConnection)
                        
    If Not bConnectSuccess Then
    
        setState LoadResString(100), True
        Exit Sub
    End If
    
    'Load form
    Load NewForm
    
    NewForm.Caption = LoadResString(214)
    
    'Set properties
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.connection = oConnection
    Set NewForm.StoredFileContent = Nothing
    
    NewForm.Show 'Show form
    
    Exit Sub
    
ErrOpenNewQueryByExample:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openDirectSQL
'    description:   shows direct sql dialog window
'    arguments:     [in]  oSFC, sql studio storedFileContent object
'
'    return value:  /
'
'=========================================================================================
Sub openDirectSQL(oSFC As StoredFileContent)
    On Error GoTo ErrOpenDirectSQL
    
    Const FUNCTION_NAME = "openDirectSQL"
    
    '=varibles===================================================================
    Dim bConnectSuccess      As Boolean
    Dim oConnection          As SQLStudioDBLayer.connection
    Dim NewForm              As New frmDirectSQL
    '=========================================================================
    
    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If
    
    If oSFC.AutoCommit = AutoCommitOff Then
    
        If m_oGlobalInfo.ConMan.isNotExclusive = False Then _
           MsgBox LoadResString(114)
    End If
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOn
    bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                                            oSFC.isoLevel, _
                                            oSFC.sqlMode, _
                                            oSFC.AutoCommit, _
                                            oConnection)
                                             
    m_oGlobalInfo.Hourglass.State = qpHourglassOff
    
    If Not bConnectSuccess Then
    
        showCriticalConMsg oConnection
        Exit Sub
    End If
      
    'Load form
    Load NewForm
    
    'Set properties
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.statementList = m_oStatementList
    Set NewForm.connection = oConnection
    '----------------------------------------------------
    'init the Connection of the storedFileContent connection too
    '----------------------------------------------------
    Set NewForm.StoredFileContent = oSFC
    
    'Show form
    NewForm.Show
    
    NewForm.refreshFont
    
    Exit Sub
    
ErrOpenDirectSQL:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openVisualQuery
'    description:   shows viusal query dialog window
'    arguments:     [in]  oSFC, sql studio storedFileContent object
'
'    return value:  /
'
'=========================================================================================
Sub openVisualQuery(oSFC As StoredFileContent)
    On Error GoTo ErrOpenVisualQuery
    
    Const FUNCTION_NAME = "openVisualQuery"
    
    '=varibles===================================================================
    Dim bConnectSuccess      As Boolean
    Dim oConnection          As SQLStudioDBLayer.connection
    Dim NewForm              As New frmVisualQuery
    '=========================================================================
    
    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOn
    
    
    bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                      oSFC.isoLevel, _
                      oSFC.sqlMode, _
                      AutoCommitOn, oConnection)
                      
    m_oGlobalInfo.Hourglass.State = qpHourglassOff
    
    If Not bConnectSuccess Then
    
        showCriticalConMsg oConnection
        Exit Sub
    End If
      
    'Load form
    Load NewForm
    
    'Set properties
    Set NewForm.connection = oConnection
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.StoredFileContent = oSFC
      
    'Show form
    NewForm.Show
       
    Exit Sub
    
ErrOpenVisualQuery:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openQueryByExample
'    description:   shows query by example dialog window
'    arguments:     [in]  oSFC, sql studio storedFileContent object
'
'    return value:  /
'
'=========================================================================================
Sub openQueryByExample(oSFC As StoredFileContent)
    On Error GoTo ErrOpenQueryByExample
    
    Const FUNCTION_NAME = "openQueryByExample"
       
    '=varibles===================================================================
    Dim bConnectSuccess      As Boolean
    Dim oConnection          As SQLStudioDBLayer.connection
    Dim NewForm              As Object
    '=========================================================================
    
    
    Set NewForm = New frmQuickQBE
    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOn
    
    bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                      oSFC.isoLevel, _
                      oSFC.sqlMode, _
                      AutoCommitOn, oConnection)
                  
    m_oGlobalInfo.Hourglass.State = qpHourglassOff
    
    If Not bConnectSuccess Then
    
        showCriticalConMsg oConnection
        Exit Sub
    End If
    
    'Load form
    Load NewForm
    
    'Set properties
    Set NewForm.connection = oConnection
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.StoredFileContent = oSFC
      
    'Show form
    NewForm.Show
    
    Exit Sub
    
ErrOpenQueryByExample:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openStoredObjects
'    description:   shows dialog with stored sql studio objects
'
'    return value:  /
'
'=========================================================================================
Sub openStoredObjects()
    On Error GoTo errOpenStoredObjects
    
    Const FUNCTION_NAME = "openStoredObjects"
    
    Set frmShowStoredObjects.globalInfo = m_oGlobalInfo
    
    Select Case m_oGlobalInfo.userProfile.AppManagment.LastType

        Case AppStatement:
            frmShowStoredObjects.showStoredObjects fileTypeStatement

        Case AppVisualQuery:
        
            'don 't load because its not visible instead load SQL Statements
            If frmMain.chkTinyMode.Value = vbUnchecked Then
            
                 frmShowStoredObjects.showStoredObjects fileTypeVisualQuery
            Else
                frmShowStoredObjects.showStoredObjects fileTypeStatement
            End If

        Case AppQueryByExample:
        
            'don't load because its not visible instead load SQL Statements
            If frmMain.chkTinyMode.Value = vbUnchecked Then
            
                frmShowStoredObjects.showStoredObjects fileTypeQueryByExample
            Else
                frmShowStoredObjects.showStoredObjects fileTypeStatement
            End If

        Case AppDrafts:
            'don 't load because its not visible instead load SQL Statements
            If frmMain.chkTinyMode.Value = vbUnchecked Then
            
                frmShowStoredObjects.showStoredObjects fileTypeDraftTable
            Else
                frmShowStoredObjects.showStoredObjects fileTypeStatement
            End If
        End Select
    
    frmShowStoredObjects.Show
        
    Exit Sub

errOpenStoredObjects:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      openTableDefinition
'    description:   shows table definition dialog window
'    arguments:     [in]  oSFC, sql studio storedFileContent object
'                   [in]  bReadOnly, information is just display in dialog
'                   [in]  tablename, tablename if dialog is used for altering a table
'                   [in]  tableowner, tableowner if dialog is used for altering a table
'
'    return value:  /
'
'=========================================================================================
Sub openTableDefinition(oSFC As StoredFileContent, _
                        Optional tablename As String, _
                        Optional tableowner As String, _
                        Optional itemType As cvItemType)
    On Error GoTo errOpenTableDefinition
    
    Const FUNCTION_NAME = "openTableDefinition"
    
    '=varibles===================================================================
    Dim bConnectSuccess    As Boolean
    Dim oConnection        As SQLStudioDBLayer.connection
    Dim NewForm            As New frmCreateTable
    Dim isoLevel           As SQLStudioDBLayer.isoLevel
    Dim sqlMode            As SQLStudioDBLayer.sqlMode
    '=============================================================================
    
    If Forms.count > m_csMaxForm Then
    
        MsgBox LoadResString(115), vbCritical
        Exit Sub
    End If

    m_oGlobalInfo.Hourglass.State = qpHourglassOn
    
    If oSFC Is Nothing Then
    
        isoLevel = Committed
        sqlMode = Internal
    Else
        isoLevel = oSFC.isoLevel
        sqlMode = oSFC.sqlMode
    End If
    
    If IsMissing(itemType) Or itemType = cmUnknown Then
    
        bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                                                isoLevel, _
                                                sqlMode, _
                                                AutoCommitOff, _
                                                oConnection)
        If Not bConnectSuccess Then
    
            showCriticalConMsg oConnection
            Exit Sub
        End If
    Else
        'if table def is just shown, open a standard commit on connection
        bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                                                m_oGlobalInfo.ConMan.DriverIsoLevel, _
                                                m_oGlobalInfo.ConMan.DriverSQLMode, _
                                                AutoCommitOn, _
                                                oConnection)
        If Not bConnectSuccess Then
    
            showCriticalConMsg oConnection
            Exit Sub
        End If
    End If
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOff
      
    'Load form
    Load NewForm
    
    If oSFC Is Nothing Then
        NewForm.mode = 3                       ' Alter Table
        NewForm.Caption = LoadResString(216)
    Else
        NewForm.mode = 4                       ' Open Draft
        'don't set caption of form
        'the form's caption will be set in
        'Set NewForm.storedFileContent = oSFC
        'to the oSFC name
    End If
    
    Set NewForm.connection = oConnection
    
    'Set properties
    If Not (IsMissing(itemType) Or itemType = cmUnknown) Then
    
        'show definition
        NewForm.Caption = LoadResString(231)
    End If
    
    Set NewForm.globalInfo = m_oGlobalInfo
    Set NewForm.StoredFileContent = oSFC
      
    If oSFC Is Nothing Then
    
        If IsMissing(itemType) Or itemType = cmUnknown Then
        
            NewForm.InitializeAlterTable tablename, tableowner
        Else
            NewForm.InitializeAlterTable tablename, tableowner, itemType
        End If
    End If
    
    'Show form
    NewForm.Show
       
    Exit Sub
    
errOpenTableDefinition:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      setCompleteState
'    description:   set status bar and main frame caption
'    arguments:     [in] State, connection state
'                   [in] DSN, data source of odbc driver
'                   [in] UID, user name
'                   [in] Database, database name
'                   [in] dbServer, server node name
'    return value:  /
'
'=========================================================================================
Public Function setCompleteState(ByVal State As String, _
                            ByVal UID As String, _
                            ByVal database As String, _
                            ByVal dbServer As String)
    On Error GoTo errSetCompleteState
    
    Const FUNCTION_NAME = "setCompleteState"
    
    frmMain.lblState.Panels("xState").Text = State
    frmMain.lblState.Panels("xUID").Text = UID
    frmMain.lblState.Panels("xDataBase").Text = database
    
    If Not m_oGlobalInfo.ConMan.isNIConnection Then
    
        If Len(dbServer) < 1 Then
            
            If Len(UID) > 0 Then
            
                dbServer = LoadResString(320)
            Else
                dbServer = ""
            End If
        End If
    Else
        dbServer = ""
    End If
    
    frmMain.lblState.Panels("xDBServer").Text = dbServer
    
    frmMain.Caption = LoadResString(102) + " ["
    frmMain.Caption = frmMain.Caption + UID + "," + database
    
    'check for NI Connetion
    If Not m_oGlobalInfo.ConMan.isNIConnection Then _
       frmMain.Caption = frmMain.Caption + "," + dbServer
    
    frmMain.Caption = frmMain.Caption + "]"
    
    'check for empty caption
    If frmMain.Caption = LoadResString(102) + " [,,,]" Then frmMain.Caption = LoadResString(102)
    If frmMain.Caption = LoadResString(102) + " [,,]" Then frmMain.Caption = LoadResString(102)
    If frmMain.Caption = LoadResString(102) + " [,]" Then frmMain.Caption = LoadResString(102)
    
    Exit Function
    
errSetCompleteState:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      setState
'    description:   set status bar text
'    arguments:     [in] sState,
'                   [in] bWithTimer, delete message after a period
'    return value:  /
'
'=========================================================================================
Public Function setState(ByVal sState As String, ByVal bWithTimer As Boolean)
    On Error GoTo errSetState
    
    Const FUNCTION_NAME = "setState"
    
    frmMain.lblState.Panels("xState").Text = sState
    
    If bWithTimer Then frmMain.tmrState.Enabled = True
    
    Exit Function
    
errSetState:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      showHelp
'    description:   set status bar text
'    arguments:     [in] sHelpString, path for help (.chm) file
'
'    return value:  /
'
'=========================================================================================
Public Function showHelp(ByVal sHelpString As String)
    On Error GoTo errShowHelp
    
    Const FUNCTION_NAME = "showHelp"
    
    '=local variables==============
    Dim bOperation()    As Byte
    Dim bFile()         As Byte
    Dim bHelpString()   As Byte
    Dim bAppPath()      As Byte
    '==============================
    
    bOperation = "open" + vbNullChar
    bFile = "hh" + vbNullChar
    bHelpString = sHelpString + vbNullChar
    bAppPath = App.Path + vbNullChar
        
    ShellExecuteA frmMain.hwnd, _
                  "open", _
                  "hh", _
                  sHelpString, _
                  App.Path, _
                  SW_SHOWNORMAL
'    ShellExecuteW frmMain.hwnd, _
'                  bOperation(0), _
'                  bFile(0), _
'                  bHelpString(0), _
'                  bAppPath(0), _
'                  SW_SHOWNORMAL
    
    Exit Function
    
errShowHelp:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      loadAndSetLRUList
'    description:   load LRU list from registry and sets list to frmmain menu
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function loadAndSetLRUList()
    On Error GoTo errLoadAndSetLRUList

    Const FUNCTION_NAME = "loadAndSetLRUList"
    
    m_oLRUConnections.loadConnections
    
    setLRUList
    
    Exit Function

errLoadAndSetLRUList:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      setLRUList
'    description:   sets LRU list to frmmain menu
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function setLRUList()
    On Error GoTo errSetLRUList
    
    Const FUNCTION_NAME = "setLRUList"

    '=varibles===================================================================
    Dim i        As Integer
    Dim bReturn  As Boolean
    Dim sCaption As String
    Dim sPWD     As String
    Dim sText    As String
    '=============================================================================
            
    For i = 0 To 9
    
          bReturn = m_oLRUConnections.getCaption(i, sCaption)
          If bReturn Then
          
             m_oLRUConnections.getPWD i, sPWD
          
             frmMain.mnuFileSep1.Visible = True
             frmMain.mnuLRUConnect(i).Caption = sCaption
             frmMain.mnuLRUConnect(i).Visible = True
          Else
            frmMain.mnuLRUConnect(i).Caption = ""
            frmMain.mnuLRUConnect(i).Visible = False
          End If
    Next i
    
    If m_oLRUConnections.count = 0 Then frmMain.mnuFileSep1.Visible = False
         
    Exit Function
    
errSetLRUList:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      showResults
'    description:   shows a result dialog window
'    arguments:     [in] Results, collection of result objects
'                   [in] sCaption, caption for result window
'                   [in] emResultMode, mode for result objects
'                   [in] sTimeStart, beginning of action
'                   [in] sTimeEnd, end of action
'
'    return value:  true, if successfully
'
'=========================================================================================
Public Function showResults(results As Collection, _
                            sCaption As String, _
                            emResultMode As SQLStudioDBLayer.sapdbResultMode, _
                            sTimeStart As String, _
                            sTimeEnd As String _
                            ) As Boolean
    On Error GoTo errShowResults
    
    Const FUNCTION_NAME = "showResults"
    
    '=varibles===================================================================
    Dim lRun       As Long
    Dim NewForm    As frmShowResult
    '=============================================================================
  
    'rc
    showResults = False
    
    'Is there an result?
    If results Is Nothing Then
    
        showResults = True
        Exit Function
    End If
    
    If results.count > 0 Then

        Set NewForm = New frmShowResult
        Set NewForm.globalInfo = m_oGlobalInfo
        
        NewForm.resultMode = emResultMode
        Set NewForm.results = results
        
        NewForm.Caption = sCaption
        NewForm.showResult sTimeStart, sTimeEnd
    End If
    
    'Set rc
    showResults = True

    Exit Function

errShowResults:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      showContent
'    description:   show the content of the object
'    arguments:     [in]  sOwner, owner of the object
'                   [in]  sName, name of the object
'
'    return value:  /
'
'=========================================================================================
Public Function showContent(sOwner As String, sName As String) As Boolean
    On Error GoTo errShowContent

    Const FUNCTION_NAME = "showContent"
   
    showContent = False
    
    executeStatement "SELECT * FROM " + Chr$(34) + sOwner + Chr$(34) + "." + Chr$(34) + sName + Chr$(34)

    showContent = True

    Exit Function

errShowContent:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
Public Function showUserGroup(sOwner As String, sUserGroup As String) As Boolean
    On Error GoTo errShowUserGroup

    Const FUNCTION_NAME = "showUserGroup"
   
    'Init
    showUserGroup = False
    
    executeStatement "SELECT * FROM USERS WHERE OWNER ='" + sOwner + "' AND GROUPNAME ='" + sUserGroup + "' AND USERNAME = ''"
                    
    showUserGroup = True

    Exit Function

errShowUserGroup:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

Public Function showUser(sOwner As String, sUser As String) As Boolean
    On Error GoTo errShowUser

    Const FUNCTION_NAME = "showUser"
   
    'rc
    showUser = False
    
    executeStatement "SELECT * FROM USERS WHERE OWNER ='" + sOwner + "' AND USERNAME ='" + sUser + "'"
                    

    showUser = True

    Exit Function

errShowUser:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
Public Function executeStatement(ByVal sStatement As String)
    On Error GoTo errExecuteStatement
    
    Const FUNCTION_NAME = "executeStatement"
        
    '=varibles===================================================================
    Dim results                 As New Collection
    Dim oStatements             As New BreakStatement
    '=============================================================================
    
    'rc
    executeStatement = False
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOn

    Set oStatements.connection = m_oAdminCon
    oStatements.resultMode = sapdbVirtualMode
    oStatements.statement = sStatement
    If oStatements.executeStatements(results, Internal) = False Then

        m_oGlobalInfo.Hourglass.State = qpHourglassOff
        Exit Function
    End If

    m_oGlobalInfo.Hourglass.State = qpHourglassOff

    If Not results Is Nothing Then

        If results.count > 0 Then _
           showResults results, sStatement, sapdbVirtualMode, "", ""
    End If
    
    'rc
    executeStatement = True
    
    Exit Function
    
errExecuteStatement:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
Public Function removeCloseButton(oForm As Form)
    On Error GoTo errRemoveCloseButton
    
    Const FUNCTION_NAME = "removeCloseButton"
    
    '=varibles===================================================================
    Dim hSysMenu As Long
    Dim nCnt     As Long
    '=============================================================================
    
    ' Get handle to our form's system menu (Restore, Maximize, Move, close etc.)
    hSysMenu = GetSystemMenu(oForm.hwnd, False)

    If hSysMenu Then
        ' Get system menu's menu count
        nCnt = GetMenuItemCount(hSysMenu)

        If nCnt Then
            ' Menu count is based on 0 (0, 1, 2, 3...)
            RemoveMenu hSysMenu, nCnt - 2, MF_BYPOSITION Or MF_REMOVE
            RemoveMenu hSysMenu, nCnt - 3, MF_BYPOSITION Or MF_REMOVE ' Remove the seperator

            DrawMenuBar oForm.hwnd  ' Force caption bar's refresh. Disabling X button
        End If
    End If
    
    Exit Function
    
errRemoveCloseButton:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getWindowSettings
'    description:   reads the window settings like top,left ... from the registry
'    arguments:     [in] oForm, form for which the properties will be set
'
'    return value:  /
'
'=========================================================================================
Public Function getWindowSettings(oForm As Form)
    On Error GoTo errGetWindowSettings
    
    Const FUNCTION_NAME = "getWindowSettings"
    
    '=varibles===================================================================
    Dim bReturn      As Boolean
    Dim lKey         As Long
    Dim lDisPos      As Long
    Dim lValue       As Long
    Dim sKey         As String
    '=============================================================================
    
    sKey = m_oComFunc.getRegBasicKey + WINDOWPLACEMENT + "\" + NOVERSIONDEP
        
    m_oRegFunc.showErrors = False
    bReturn = m_oRegFunc.openKey(HKEY_CURRENT_USER, sKey, KEY_ALL_ACCESS, lKey)
    
    If bReturn = True Then
    
        bReturn = m_oRegFunc.queryValueLong(lKey, "WindowState", lValue)
        If bReturn Then oForm.WindowState = lValue
        
        If oForm.WindowState = vbNormal Then
        
            bReturn = m_oRegFunc.queryValueLong(lKey, "Left", lValue)
            If bReturn Then oForm.Left = lValue
            
            bReturn = m_oRegFunc.queryValueLong(lKey, "Top", lValue)
            If bReturn Then oForm.Top = lValue
            
            bReturn = m_oRegFunc.queryValueLong(lKey, "Height", lValue)
            If bReturn Then oForm.Height = lValue
            
            bReturn = m_oRegFunc.queryValueLong(lKey, "Width", lValue)
            If bReturn Then oForm.Width = lValue
        End If
        
        m_oRegFunc.closeKey sKey, lKey
    Else
        oForm.Left = Screen.Height / 5
        oForm.Top = Screen.Width / 5
        oForm.Height = Screen.Height * 2 / 3
        oForm.Width = Screen.Width * 2 / 3
    End If
    
    Exit Function
    
errGetWindowSettings:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      setWindowSettings
'    description:   writes the window settings like top,left ... from the registry
'    arguments:     [in] oForm, form which  properties will be written
'                   [in] lLOffset, offset for new main windows
'                   [in] lTOffset, offset for new main windows
'                   [in] lWOffset, offset for new main windows
'                   [in] lHOffset, offset for new main windows
'
'    return value:  /
'
'=========================================================================================
Public Function setWindowSettings(oForm As Form, lLOffset As Long, lTOffset As Long, lWOffset As Long, lHOffset As Long)
    On Error GoTo errSetWindowSettings
    
    Const FUNCTION_NAME = "setWindowSettings"
    
    '=varibles===================================================================
    Dim bReturn      As Boolean
    Dim lKey         As Long
    Dim lValue       As Long
    Dim lDisPos      As Long
    Dim sKey         As String
    Dim oKey         As SQLStudioUtil.Key
    Dim oVal         As SQLStudioUtil.Value
    '=============================================================================
    
    sKey = m_oComFunc.getRegBasicKey + WINDOWPLACEMENT + "\" + NOVERSIONDEP
        
    m_oRegFunc.showErrors = False
    bReturn = m_oRegFunc.openKey(HKEY_CURRENT_USER, sKey, KEY_ALL_ACCESS, lKey)
        
    If bReturn = False Then _
       bReturn = m_oRegFunc.createKey(HKEY_CURRENT_USER, sKey, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, lKey, lDisPos)
    
    If bReturn = True Then
    
        lValue = oForm.WindowState
        
        bReturn = m_oRegFunc.setValueLong(lKey, "WindowState", lValue)
        If lValue <> vbNormal Then

            lValue = Screen.Height / 5
            bReturn = m_oRegFunc.setValueLong(lKey, "Left", lValue)

            lValue = Screen.Width / 5
            bReturn = m_oRegFunc.setValueLong(lKey, "Top", lValue)

            lValue = Screen.Height * 2 / 3
            bReturn = m_oRegFunc.setValueLong(lKey, "Height", lValue)

            lValue = Screen.Width * 2 / 3
            bReturn = m_oRegFunc.setValueLong(lKey, "Width", lValue)
        Else
            lValue = oForm.Left + lLOffset
            bReturn = m_oRegFunc.setValueLong(lKey, "Left", lValue)
            lValue = oForm.Top + lTOffset
            bReturn = m_oRegFunc.setValueLong(lKey, "Top", lValue)
            lValue = oForm.Height + lHOffset
            bReturn = m_oRegFunc.setValueLong(lKey, "Height", lValue)
            lValue = oForm.Width + lWOffset
            bReturn = m_oRegFunc.setValueLong(lKey, "Width", lValue)
        End If
        
    End If
    
    m_oRegFunc.closeKey sKey, lKey

    Exit Function
    
errSetWindowSettings:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getNoTimeOut
'    description:   reads the value of timeout option from the registry to the timeout checkbox in the main frame window
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function getNoTimeOut()
    On Error GoTo errGetNoTimeOut
    
    Const FUNCTION_NAME = "getNoTimeOut"
    
    '=varibles===================================================================
    Dim bReturn      As Boolean
    Dim lKey         As Long
    Dim lDisPos      As Long
    Dim sValue       As String
    Dim sKey         As String
    '============================================================================
    
    sValue = Space$(64)
    sKey = m_oComFunc.getRegBasicKey + NOTIMEOUT + "\" + NOVERSIONDEP
        
    m_oRegFunc.showErrors = False
    bReturn = m_oRegFunc.openKey(HKEY_CURRENT_USER, sKey, KEY_ALL_ACCESS, lKey)
    
    If bReturn = True Then
    
        bReturn = m_oRegFunc.QueryValueString(lKey, NOTIMEOUT, sValue, Len(sValue))
        If bReturn Then
            If Trim(sValue) = SAPDB_TRUE Then frmMain.chkNoTimeOut.Value = vbChecked
        End If
    End If
    
    m_oRegFunc.closeKey sKey, lKey
    
    Exit Function
    
errGetNoTimeOut:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      setNoTimeOut
'    description:   writes the value of the timeout checkbox in the main frame window to the registry
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function setNoTimeOut()
    On Error GoTo errSetNoTimeOut
    
    Const FUNCTION_NAME = "setNoTimeOut"
    
    '=varibles===================================================================
    Dim bNoTimeOut   As Boolean
    Dim bReturn      As Boolean
    Dim lKey         As Long
    Dim lDisPos      As Long
    Dim sKey         As String
    Dim oKey         As SQLStudioUtil.Key
    Dim oVal         As SQLStudioUtil.Value
    '=============================================================================
        
    If frmMain.chkNoTimeOut.Value = vbChecked Then
    
        bNoTimeOut = True
    Else
        bNoTimeOut = False
    End If
    
    sKey = m_oComFunc.getRegBasicKey + NOTIMEOUT + "\" + NOVERSIONDEP
        
    m_oRegFunc.showErrors = False
    bReturn = m_oRegFunc.openKey(HKEY_CURRENT_USER, sKey, KEY_ALL_ACCESS, lKey)
        
    If bReturn = False Then _
       bReturn = m_oRegFunc.createKey(HKEY_CURRENT_USER, sKey, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, lKey, lDisPos)
    
    If bReturn = True Then _
       bReturn = m_oRegFunc.setValueString(lKey, NOTIMEOUT, Trim(Str(bNoTimeOut)))
    
    m_oRegFunc.closeKey sKey, lKey

    Exit Function
    
errSetNoTimeOut:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      isoLevelAsString
'    description:   returns isolevel as a string
'    arguments:     [in] IsoLevel, em representing isolevel
'
'
'    return value:   isolation level as string
'
'=========================================================================================
Public Function isoLevelAsString(isoLevel As SQLStudioDBLayer.isoLevel) As String
    On Error GoTo errIsoLevelAsString

    Const FUNCTION_NAME = "isoLevelAsString"
    
    Select Case isoLevel
    
        Case UnCommitted
            isoLevelAsString = LoadResString(301)
        Case Committed
            isoLevelAsString = LoadResString(302)
        Case Repeatable
            isoLevelAsString = LoadResString(303)
        Case Serializable
            isoLevelAsString = LoadResString(304)
    End Select
    
    Exit Function
    
errIsoLevelAsString:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

'=========================================================================================
'    function:      sqlModeAsString
'    description:   returns sqlmode as a string
'    arguments:     [in] SQLMode, em representing sqlmode
'
'    return value:  sqlmode as string
'
'=========================================================================================
Public Function sqlModeAsString(sqlMode As SQLStudioDBLayer.sqlMode) As String
    On Error GoTo errSQLModeAsString

    Const FUNCTION_NAME = "sqlModeAsString"
    
    Select Case sqlMode
    
        Case Internal
            sqlModeAsString = LoadResString(310)
        Case DB2
           sqlModeAsString = LoadResString(311)
        Case Ansi
            sqlModeAsString = LoadResString(312)
        Case Oracle
            sqlModeAsString = LoadResString(313)
    End Select
    
    Exit Function
    
errSQLModeAsString:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      timeDiff
'    description:   calculating the difference between two time values in different possible formats
'    arguments:     [in] sTimeFormat, string representig the time format to work with
'                   [in] sStart, start time
'                   [in] sEnd, end time
'    return value:  time difference as string
'
'=========================================================================================
Public Function timeDiff(sTimeFormat As String, sStart As String, sEnd As String) As String
    On Error GoTo errTimeDiff
    
    Const FUNCTION_NAME = "timeDiff"

    '=varibles===================================================================
    Dim iMilDiff   As Integer
    Dim iSecDiff   As Integer
    Dim iMinDiff   As Integer
    Dim iHourDiff  As Integer
    Dim iSecOffset As Integer
    Dim iMinOffset As Integer
    Dim sUnit      As String
    '=============================================================================
    
    iHourDiff = Mid(sEnd, 1, 2) - Mid(sStart, 1, 2)
    iMinDiff = Mid(sEnd, 4, 2) - Mid(sStart, 4, 2)
    iSecDiff = Mid(sEnd, 7, 2) - Mid(sStart, 7, 2)
    iMilDiff = Mid(sEnd, 10, 3) - Mid(sStart, 10, 3)
        
    If sTimeFormat = "m" Then
        timeDiff = Format$(iMinDiff + 60 * iHourDiff) + " min"
        Exit Function
    End If

    If sTimeFormat = "s" Then
        timeDiff = Format$(iSecDiff + 60 * iMinDiff + 3600 * iHourDiff) + " sec"
        Exit Function
    End If

    If sTimeFormat = "hh:mm:ss.xxx" Then
        If iMilDiff < 0 Then
            iSecDiff = iSecDiff - 1
            iMilDiff = iMilDiff + 1000
        End If
    
        If iSecDiff < 0 Then
            iMinDiff = iMinDiff - 1
            iSecDiff = iSecDiff + 60
        End If
    
        If iMinDiff < 0 Then
            iHourDiff = iHourDiff - 1
            iMinDiff = iMinDiff + 60
        End If
        
        If iHourDiff > 0 Then
            timeDiff = Format$(iHourDiff, "00") + ":" + _
                       Format$(iMinDiff, "00") + ":" + _
                       Format$(iSecDiff, "00") + "." + _
                       Format$(iMilDiff, "000") + " h"
            Exit Function
        End If
            
        If iMinDiff > 0 Then
            timeDiff = Format$(iMinDiff, "00") + ":" + _
                       Format$(iSecDiff, "00") + "." + _
                       Format$(iMilDiff, "000") + " min"
            Exit Function
        End If
                   
        timeDiff = Format$(iSecDiff, "00") + "." + _
                   Format$(iMilDiff, "000") + " sec"
        
        Exit Function
    End If

Exit Function

errTimeDiff:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      cmdLineOptsInit
'    description:   initis the global command line object
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function cmdLineOptsInit()
    On Error GoTo errCmdLineOptsInit
    
    Const FUNCTION_NAME = "cmdLineOptsInit"
    
    '=varibles===================================================================
    Dim lPrefix As CmdLinePrefixConstants
    '=============================================================================
    
    'init
    lPrefix = cmlPrefixMinus
    
    m_oCML.add lPrefix, CML_OPT_HELP, True, True, "", LoadResString(401)
    m_oCML.add lPrefix, CML_OPT_NODE, True, True, LoadResString(406), LoadResString(402)
    m_oCML.add lPrefix, CML_OPT_DB, True, True, LoadResString(407), LoadResString(403)
    m_oCML.add lPrefix, CML_OPT_USER, True, True, LoadResString(408), LoadResString(404)
    m_oCML.add lPrefix, CML_OPT_VER, True, True, "", LoadResString(405)
    m_oCML.add lPrefix, CML_OPT_CM, True, True, LoadResString(409), LoadResString(410)
    m_oCML.add lPrefix, CML_OPT_TRACE, True, True, LoadResString(411), LoadResString(412)

    Exit Function
    
errCmdLineOptsInit:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

'=========================================================================================
'    function:      showCriticalConMsg
'    description:   shows the error message
'    arguments:     oConnection: connection object
'
'    return value:  /
'
'=========================================================================================
Public Function showCriticalConMsg(oConnection As connection)
    On Error GoTo errShowCriticalConMsg
    
    Const FUNCTION_NAME = "showCriticalConMsg"
    
    '"Connection failed ..."
    setState LoadResString(100), True
    
    If Len(oConnection.Error.Text) > 0 Then
        
        oConnection.Error.showError
    Else
        If Len(oConnection.Error.sqlState) > 0 Then
        
            MsgBox oConnection.Error.sqlState
       Else
            showError142 oConnection
        End If
    End If

    Exit Function
    
errShowCriticalConMsg:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      showSettings
'    description:   shows dialog for user settings
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function showSettings()
    On Error GoTo errShowSettings
    
    Const FUNCTION_NAME = "showSettings"
        
    '=varibles===================================================================
    Dim frmUP     As New frmUserProperties
    '============================================================================
        
    'set a connection
    Set frmUP.connection = m_oAdminCon
    'set the user profile object
    Set frmUP.userProfile = m_oGlobalInfo.userProfile
    
        
    'if system tables are loaded and user is not exclusive set sqlmode, isolevel from user profile object
    If m_oGlobalInfo.ConMan.isNotExclusive And m_oGlobalInfo.systemtablesLoaded Then
    
        frmUP.sqlMode = m_oGlobalInfo.userProfile.AppManagment.sqlMode
        frmUP.isoLevel = m_oGlobalInfo.userProfile.AppManagment.isoLevel
    Else
        'set driver sqlmode, isolevel
        frmUP.sqlMode = m_oGlobalInfo.ConMan.DriverSQLMode
        frmUP.isoLevel = m_oGlobalInfo.ConMan.DriverIsoLevel
    End If
    
    'if system tables are loaded and user is not exclusive enable some properties
    frmUP.propertiesEnabled = (m_oGlobalInfo.ConMan.isNotExclusive And m_oGlobalInfo.systemtablesLoaded)
    
    'load the user profile from the database
    frmUP.loadUserProfile
    'show dialog
    frmUP.Show vbModal, frmMain
    
    If frmUP.ok Then
        
       If frmMain.chkTinyMode.Value = vbUnchecked Then
       
            'enable all features
            frmMain.ctlCM.tinyMode = False
            frmMain.ctlCM.useLocalFolderInd = m_oGlobalInfo.userProfile.localFolderIndependent
            
            'save node info of catalogtree
            m_oGlobalInfo.userProfile.expandedNodes = frmMain.ctlCM.getNodeInfo

            'load all catalog objects
            frmMain.ctlCM.loadCatalog _
                                m_oGlobalInfo.ConMan.UID, _
                                frmMain.lstTableFilter.Text, _
                                itemAll
                                
            frmMain.ctlCM.setNodeInfo m_oGlobalInfo.userProfile.expandedNodes, True
            
       Else
            'limitate sql studio objects for local folder
            frmMain.ctlCM.tinyMode = True
            frmMain.ctlCM.useLocalFolderInd = m_oGlobalInfo.userProfile.localFolderIndependent
            'if WAN Mode is active, just load stored objects
            frmMain.ctlCM.loadCatalog _
                            m_oGlobalInfo.ConMan.UID, _
                            frmMain.lstTableFilter.Text, _
                            itemStoredObjects
       End If
       
       'set the expanded nodes
       frmMain.ctlCM.setNodeInfo m_oGlobalInfo.userProfile.expandedNodes, m_oGlobalInfo.userProfile.restoreCatalogState
       
       If m_oGlobalInfo.ConMan.isNotExclusive And m_oGlobalInfo.systemtablesLoaded Then
    
            'set changed values
            m_oGlobalInfo.userProfile.AppManagment.sqlMode = frmUP.sqlMode
            m_oGlobalInfo.userProfile.AppManagment.isoLevel = frmUP.isoLevel
       End If
    End If
        
    Set frmUP = Nothing
    
    Exit Function
    
errShowSettings:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      changeBackSlashToSlash
'    description:   changes all backslashes in a string to slashes
'    arguments:     [in] sInput, string to be processed
'                   [out] sOutPut, new string
'    return value:  /
'
'=========================================================================================
Public Function changeBackSlashToSlash(ByVal sInput As String, sOutPut As String)
    On Error GoTo errChangeBackSlashToSlash

    Const FUNCTION_NAME = "changeBackSlashToSlash"
    
    'init output string
    sOutPut = ""

    Dim lPos As Long
        
    Do
        lPos = InStr(sInput, "\")
        If lPos > 0 Then

            sOutPut = sOutPut + Left(sInput, lPos - 1)
            sOutPut = sOutPut + "/"

            sInput = Right(sInput, Len(sInput) - lPos)
        End If
    Loop While lPos > 0 And Len(sInput) > 0

    sOutPut = sOutPut + sInput

    Exit Function

errChangeBackSlashToSlash:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      showError142
'    description:   shows the error with the resource id 142
'    arguments:     oConnection: connection object
'
'    return value:  /
'
'=========================================================================================
Function showError142(oConnection As connection)
    On Error GoTo errShowError142
    
    Const FUNCTION_NAME = "showError142"

    '"-23062 Couldn't open connection !"
    MsgBox LoadResString(142) + _
            " (" + _
            sqlModeAsString(oConnection.sqlMode) + _
            ", " + _
            isoLevelAsString(oConnection.isoLevel) + ")", _
            , vbCritical
    
    Exit Function
    
errShowError142:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      checkLocalRoot
'    description:   check for a valid local root
'    arguments:     [in/out] sLocalRoot to check
'
'
'    return value:  /
'
'=========================================================================================
Public Function checkLocalRoot(ByRef sLocalRoot As String)
    On Error GoTo errCheckLocalRoot

    Const FUNCTION_NAME = "checkLocalRoot"

    '=local varibles==========================
    Dim sTemp  As String
    '==========================================
        
    'if there is no local root in the the user profile, set system temp path
    If Len(sLocalRoot) < 1 Then
    
        'get temp path of system
        sTemp = App.Path + "/" + App.EXEName
        changeBackSlashToSlash sTemp, sLocalRoot
    Else
        If Not m_oGlobalInfo Is Nothing Then
            'remove name of database which was added to local root in previous verions
            If m_oGlobalInfo.ConMan.isConnected Then
                If Right(sLocalRoot, _
                         1 + Len(m_oGlobalInfo.ConMan.database)) = "/" + m_oGlobalInfo.ConMan.database Then
                sLocalRoot = Left(sLocalRoot, Len(sLocalRoot) - Len(m_oGlobalInfo.ConMan.database))
            End If
            End If
        End If
    End If
    
    Exit Function
    
errCheckLocalRoot:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      checkResult
'    description:   check if the is a valid result for the given SQL statement
'    arguments:     [in] statement - SQL statement to check
'    return value:  Boolean
'
'=========================================================================================
Public Function checkResult(statement As String) As Boolean
    On Error GoTo errCheckResult

    Const FUNCTION_NAME = "checkResult"
    
    Dim oResult   As New SQLStudioDBLayer.Result
    
    checkResult = False
    
    Set oResult.connection = m_oAdminCon
    If Not oResult.OpenResult(statement, Internal) Then Exit Function
    If Not oResult.Fetch() Then Exit Function
    If oResult.NoRows() Then Exit Function
    
    checkResult = True
    
    Exit Function
    
errCheckResult:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      procIDFromWnd
'    description:   check if the is a valid result for the given SQL statement
'    arguments:     [in] statement - SQL statement to check
'    return value:  proc id
'
'=========================================================================================
Function procIDFromWnd(ByVal hwnd As Long) As Long
    On Error GoTo errProcIDFromWnd

    Const FUNCTION_NAME = "procIDFromWnd"
    
    '=local variables=========
    Dim idProc As Long
    '=========================
   
    If hwnd = 0 Then Exit Function
   
    'get PID for this HWnd
    GetWindowThreadProcessId hwnd, idProc
   
    'Return PID
    procIDFromWnd = idProc
   
    Exit Function
    
errProcIDFromWnd:
    internalError Err, MODULE_NAME, FUNCTION_NAME
    End Function
'=========================================================================================
'    function:      setTaskIDToMainWnd
'    description:   set database task ID in frame window
'    arguments:     [in] statement - SQL statement to check
'    return value:  proc id
'
'=========================================================================================
Private Function setTaskIDToMainWnd()
    On Error GoTo errSetTaskIDToMainWnd

    Const FUNCTION_NAME = "setTaskIDToMainWnd"
    
    '=local variables=========
    Dim idProc As Long
    '=========================
    
    If frmMain.hwnd = 0 Then Exit Function
    
    idProc = procIDFromWnd(frmMain.hwnd)
    
    getDBTaskID idProc
    
    
    Exit Function
    
errSetTaskIDToMainWnd:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getDBTaskID
'    description:   check if the the necessary systemtables are installed
'    arguments:     [in] sOwner, owner of systemtable
'                   [in] sName, name  of systemtable
'
'    return value:  true if successfully
'
'=========================================================================================
Private Function getDBTaskID(lPID As Long) As Boolean
    On Error GoTo errGetDBTaskID
    
    Const FUNCTION_NAME = "getDBTaskID"
    
    '=local varibles==============================
    Dim sTask      As String
    Dim sStatement As String
    Dim oResult    As New SQLStudioDBLayer.Result
    '=============================================
    
    'rc
    getDBTaskID = False
    
    sStatement = "select PROCESS from SYSDBA.TRANSACTIONS where APPLPROCESS = " + Trim(Str(lPID))
                
    Set oResult.connection = m_oAdminCon
    oResult.LimitForLong = -1
    If Not oResult.OpenResult(sStatement, Internal) Then Exit Function
    If Not oResult.Fetch Then Exit Function
    Do While (Not oResult.NoRows)
    
        oResult.GetData
        sTask = sTask + "T" + oResult.Columns(1).ColValue + ","
        If Not oResult.Fetch Then Exit Do
    Loop
        
    If Len(sTask) > 2 Then sTask = Left(sTask, Len(sTask) - 1)
    
    Set oResult = Nothing
    
    frmMain.Caption = frmMain.Caption + " - " + LoadResString(625) + " " + sTask
    
    getDBTaskID = True
    
    Exit Function

errGetDBTaskID:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      openDBProcInfo
'    description:   show some info of a db procedure
'    arguments:     [in] sOwner, owner of db proc
'                   [in] sItem, name of db proc
'
'    return value:  /
'
'=========================================================================================
Public Function openDBProcInfo(ByVal sOwner As String, _
                               ByVal sItem As String)
    On Error GoTo errOpenDBProcInfo
    
    Const FUNCTION_NAME = "openDBProcInfo"
    
    '=local varibles==========================
    Dim oConnection     As SQLStudioDBLayer.connection
    Dim bConnectSuccess As Boolean
    Dim frmInfo         As New frmDBProcInfo
    '=========================================
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOn
    
    bConnectSuccess = m_oGlobalInfo.ConMan.GetConnection( _
                          m_oGlobalInfo.ConMan.DriverIsoLevel, _
                          m_oGlobalInfo.ConMan.DriverSQLMode, _
                          AutoCommitOn, _
                          oConnection)
    
    If Not bConnectSuccess Then
    
        m_oGlobalInfo.Hourglass.State = qpHourglassOff
        setState LoadResString(100), True
        Exit Function
    End If
    
    
    
    Load frmInfo
    frmInfo.procOwner = sOwner
    frmInfo.procName = sItem
    Set frmInfo.connection = oConnection
    Set frmInfo.globalInfo = m_oGlobalInfo
    Set frmInfo.statementList = m_oStatementList
    
    m_oGlobalInfo.Hourglass.State = qpHourglassOff
    
    If frmInfo.init() = False Then Exit Function
    frmInfo.Visible = True
    
    
        
    Exit Function

errOpenDBProcInfo:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

'=========================================================================================
'    function:      removeFailedConnectAttempts
'    description:   read the value if if failed connects should be removed from the registry
'    arguments:     /
'
'    return value:  see description
'
'=========================================================================================
Public Function removeFailedConnectAttempts() As Boolean
    On Error GoTo errRemoveFailedConnectAttempts
    
    Const FUNCTION_NAME = "removeFailedConnectAttempts"
    
    '=varibles===================================================================
    Dim bReturn      As Boolean
    Dim lKey         As Long
    Dim lDisPos      As Long
    Dim sValue       As String
    Dim sKey         As String
    '============================================================================
    
    'rc
    removeFailedConnectAttempts = False
    
    sValue = Space$(64)
    sKey = m_oComFunc.getRegBasicKey + REMOVEFAILEDCON + "\" + NOVERSIONDEP
        
    m_oRegFunc.showErrors = False
    bReturn = m_oRegFunc.openKey(HKEY_CURRENT_USER, sKey, KEY_ALL_ACCESS, lKey)
    
    If bReturn = True Then
    
        bReturn = m_oRegFunc.QueryValueString(lKey, REMOVEFAILEDCON, sValue, Len(sValue))
        If bReturn Then
            If Trim(sValue) = SAPDB_TRUE _
               Then removeFailedConnectAttempts = True
        End If
    Else
        '!!!!! if there is no key, the value for removing the LRU Connection is true!!!!!!!!
        removeFailedConnectAttempts = True
    End If
    
    m_oRegFunc.closeKey sKey, lKey
    
    Exit Function
    
errRemoveFailedConnectAttempts:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function


