VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "LRUConnections"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' Class:      LRUConnections
'
'
'-----------------------------------------------------------------------
'
' responsible:  sapdb-general@sap.com
'
' special area: SQLStudio
'
' description:  class for handling LastRecentlyUsed connections
'
' see also:
'
'-----------------------------------------------------------------------
'
'    Copyright (C) 2000 SAP AG
'
' This library is free software; you can redistribute it and/or
' modify it under the terms of the GNU Lesser General Public
' License as published by the Free Software Foundation; either
' version 2.1 of the License, or (at your option) any later version.
'
' This library is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
' Lesser General Public License for more details.
'
' You should have received a copy of the GNU Lesser General Public
' License along with this library; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''


Option Explicit


 'module name
Const MODULE_NAME = "SQLLStudio.LRUConnections"

'=private variables================================================================
'flag if valid registry key exits
Private m_bInitKeys As Boolean

'flag, for saving password in registry
Private m_bSavePassword As Boolean

' Wird beim Speichern/Verndern der Liste gestzt
Private m_bDirty As Boolean

'number of connections
Private m_iConnectionCount As Integer

'array (10 members) of LRU objects
Private m_oLRUConectionArray(9) As LRUConnection

Private m_sAppVersion               As String

'object for registry functions
Private m_oRegFunc                  As SQLStudioUtil.Registry

Const iMaxArrayMembers = 10

'=========================================================================================
'    function:      Class_Initialize
'    description:   Occurs when an application creates an instance of a Form, MDIForm, User control, Property Page, Webclass, DHTML Page Designer, or class.
'
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Private Sub Class_Initialize()
    On Error GoTo errClass_Initialize

    Const FUNCTION_NAME = "Class_Initialize"
    
    'init
    m_bSavePassword = False
    m_iConnectionCount = 0
    m_bDirty = False
    
    Exit Sub
    
errClass_Initialize:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
Private Function init() As Boolean
    On Error GoTo errInit
       
    Dim bReturn             As Boolean
    Dim iConnectionIndex    As Integer
    Dim lKeyNumber          As Long
    Dim lDisposition        As Long
    Dim strConnectionKey    As String
    
    Const FUNCTION_NAME = "Init"
    
              
    While (iConnectionIndex < iMaxArrayMembers)
 
        Set m_oLRUConectionArray(iConnectionIndex) = New LRUConnection
        
        Set m_oLRUConectionArray(iConnectionIndex).regFunctions = m_oRegFunc
     
        strConnectionKey = m_oComFunc.getRegBasicKey + "LRUConnections"
        If Len(m_sAppVersion) > 0 Then _
            strConnectionKey = strConnectionKey + "\" + m_sAppVersion
            
        strConnectionKey = strConnectionKey + "\Connection" + Trim(Str(iConnectionIndex))
        
        bReturn = m_oRegFunc.openKey(HKEY_CURRENT_USER, strConnectionKey, KEY_ALL_ACCESS, lKeyNumber)
        
        If bReturn = False Then
            bReturn = m_oRegFunc.createKey(HKEY_CURRENT_USER, strConnectionKey, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, lKeyNumber, lDisposition)
            If bReturn = False Then
                init = False
                Exit Function
            Else
                bReturn = m_oRegFunc.setValueString(lKeyNumber, m_cnstKEYNAMEUID, "")
                bReturn = m_oRegFunc.setValueString(lKeyNumber, m_cnstKEYNAMEPWD, "")
                bReturn = m_oRegFunc.setValueString(lKeyNumber, m_cnstKEYDATABASE, "")
                bReturn = m_oRegFunc.setValueString(lKeyNumber, m_cnstKEYSERVERNODE, "")
            End If
        Else
            bReturn = m_oRegFunc.DeleteValue(lKeyNumber, m_cnstKEYNAMEDRIVER)
        End If
        
        bReturn = m_oRegFunc.closeKey(strConnectionKey, lKeyNumber)
        iConnectionIndex = iConnectionIndex + 1
               
 Wend
    
 m_bInitKeys = True
 
 Exit Function
    
errInit:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

' Ein lterer Eintrag aus der LRU wird auf Platz 1 gesetzt
Private Function ReSortLRUList(iLRUIndex As Integer) As Boolean
    On Error GoTo errReSortLRUList
    
    Dim i As Integer
    Dim clsNewConnection As LRUConnection
    
    Const FUNCTION_NAME = "Resort LRUList"
    
    ' selektierte LRU merken
    Set clsNewConnection = m_oLRUConectionArray(iLRUIndex)
    
    ' Es mu ab der Indexgrenze umsortiert werden
    For i = iLRUIndex To 1 Step -1
       Set m_oLRUConectionArray(i) = m_oLRUConectionArray(i - 1)
    Next i
    
    Set m_oLRUConectionArray(0) = clsNewConnection
    
    Exit Function
    
errReSortLRUList:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
' Ein lterer Eintrag aus der LRU wird auf Platz 1 gesetzt
Public Function RemoveLRU(iLRUIndex As Integer) As Boolean
    On Error GoTo errRemoveLRU
    
    Dim i As Integer
    Dim oNewConnection As LRUConnection
    
    Const FUNCTION_NAME = "RemoveLRU"
    
    If iLRUIndex > m_iConnectionCount Then Exit Function
    
    
    'Eintrag lschen
    'Set m_oLRUConectionArray(iLRUIndex) = Nothing
    
    ' Es mu bis zur  Indexgrenze umsortiert werden
    For i = iLRUIndex To m_iConnectionCount - 2 Step 1
       Set m_oLRUConectionArray(i) = m_oLRUConectionArray(i + 1)
    Next i
    
    'letzten Eintrag lschem
    m_iConnectionCount = m_iConnectionCount - 1
    m_oLRUConectionArray(m_iConnectionCount).PWD = ""
    m_oLRUConectionArray(m_iConnectionCount).UID = ""
    'm_oLRUConectionArray(m_iConnectionCount).Driver = ""
    m_oLRUConectionArray(m_iConnectionCount).database = ""
    m_oLRUConectionArray(m_iConnectionCount).dbServer = ""
    
    Exit Function
    
errRemoveLRU:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

' Es wird ein neuer LRU auf Platz 1 gesetzt
Private Function MoveLRUList(clsNewConnection As LRUConnection) As Boolean
    On Error GoTo ErrMoveLRUList
    
    Dim i As Integer
    
    Const FUNCTION_NAME = "Move LRUList"
    
    For i = 9 To 1 Step -1
       Set m_oLRUConectionArray(i) = m_oLRUConectionArray(i - 1)
    Next i
    
    Set m_oLRUConectionArray(0) = clsNewConnection
    
    Exit Function

ErrMoveLRUList:
    internalError Err, MODULE_NAME, FUNCTION_NAME

End Function
'**********************************************************************
' Puclic Properties
'**********************************************************************
Public Property Get count() As Integer
    On Error GoTo ErrGetCount
    
    Const FUNCTION_NAME = "Get count"
    count = m_iConnectionCount
    
    Exit Property
    
ErrGetCount:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
Public Property Get savePassWord() As Boolean
    On Error GoTo errSavePassword
    
    Const FUNCTION_NAME = "Get save Password"
    
    savePassWord = m_bSavePassword
    
    Exit Property
    
errSavePassword:
    internalError Err, MODULE_NAME, FUNCTION_NAME
        
End Property

Public Property Let savePassWord(bnewSavePassword As Boolean)
    On Error GoTo errSavePassword
    
    Const FUNCTION_NAME = "Let save Password"
    
    m_bSavePassword = bnewSavePassword
    
    Exit Property
    
errSavePassword:
        internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
Public Function applyPasswordOption() As Boolean
    On Error GoTo ErrApplyPasswordOption
    
    Const FUNCTION_NAME = "Apply Passwordoption"
    
    If Not m_bSavePassword Then m_oLRUConectionArray(0).PWD = ""
            
    applyPasswordOption = True
    
    Exit Function
    
ErrApplyPasswordOption:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
Public Function addLRU(sDriver As String, sUID As String, _
                       sPWD As String, sDatabase As String, _
                       sdbServer As String) As Boolean
                      
    On Error GoTo ErrAddLRU
    
    Const FUNCTION_NAME = "AddLRU"

    Dim bNewInsert         As Boolean
    Dim iConnectionIndex   As Integer
    Dim iRun               As Integer
    Dim sEmptyDriver       As String
    Dim sUIDCopy           As String
    Dim sPWDCopy           As String
    Dim oCompareConnection As New LRUConnection
    
    
    If m_bInitKeys = False Then
    
        addLRU = False
        Exit Function
    End If
    
    bNewInsert = True
    
    m_bDirty = True
    
    sUIDCopy = sUID
    sPWDCopy = sPWD
    
    If Len(sUIDCopy) > 2 Then
    
        If InStr(sUIDCopy, Chr$(34)) = 1 Then
        
            sUIDCopy = Left(sUIDCopy, Len(sUIDCopy) - 1)
            sUIDCopy = Right(sUIDCopy, Len(sUIDCopy) - 1)
        End If
    End If
    
    If Len(sPWDCopy) > 2 Then
    
        If InStr(sPWDCopy, Chr$(34)) = 1 Then
        
            sPWDCopy = Left(sPWDCopy, Len(sPWDCopy) - 1)
            sPWDCopy = Right(sPWDCopy, Len(sPWDCopy) - 1)
        End If
    End If
    
    'oCompareConnection.Driver = sDriver
    oCompareConnection.UID = sUIDCopy
    oCompareConnection.PWD = sPWDCopy
    oCompareConnection.database = sDatabase
    oCompareConnection.dbServer = sdbServer
    
    For iRun = 0 To m_iConnectionCount - 1
        If oCompareConnection.IsEqual(m_oLRUConectionArray(iRun)) Then
            bNewInsert = False
            ReSortLRUList iRun
            addLRU = True
            Exit Function
        End If
    Next iRun
    
    
    If bNewInsert = True Then
        'Neue Connection ist nicht in LRU Liste enthalten, auf Platz null einfgen
        MoveLRUList oCompareConnection
        
        ' Eintrge in Liste erneut zhlen
        Do While (iConnectionIndex < iMaxArrayMembers)
             
             'sEmptyDriver = m_oLRUConectionArray(iConnectionIndex).Driver
            
             'If Len(sEmptyDriver) < 1 Then
             
                'Exit Do
             'Else
                iConnectionIndex = iConnectionIndex + 1
             'End If
         Loop
         
         m_iConnectionCount = iConnectionIndex
    End If
    
    addLRU = True
        
    Exit Function

ErrAddLRU:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
Public Function getLRU(ByVal iIndex As Integer, sDriver As String, sUID As String, sPWD As String, _
                        sDatabase As String, sdbServer As String) As Boolean
    On Error GoTo ErrGetLRU


Const FUNCTION_NAME = "Get LRU"

'rc
getLRU = False

If m_bInitKeys = False Then Exit Function

If iIndex > 10 Then Exit Function


'sDriver = m_oLRUConectionArray(iIndex).Driver
sUID = m_oLRUConectionArray(iIndex).UID
sPWD = m_oLRUConectionArray(iIndex).PWD
sDatabase = m_oLRUConectionArray(iIndex).database
sdbServer = m_oLRUConectionArray(iIndex).dbServer

If sDriver = "" Then Exit Function

'rc
getLRU = True

Exit Function

ErrGetLRU:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
Public Function getCaption(ByVal iIndex As Integer, sCaption As String) As Boolean
    On Error GoTo ErrGetCaption
    
    Dim sDriver As String
    Dim sUID As String
    Dim sdbServer As String
    Dim sDatabase As String
    
    
    Const FUNCTION_NAME = "Get caption"
    
    sCaption = "" ' String mit null initialisieren
    
    If m_bInitKeys = False Then
        getCaption = False
        Exit Function
    End If
    
    If iIndex > 10 Then
        getCaption = False
        Exit Function
      
    End If
    
    sUID = m_oLRUConectionArray(iIndex).UID
    sdbServer = m_oLRUConectionArray(iIndex).dbServer
    sDatabase = m_oLRUConectionArray(iIndex).database
    
    If sUID = "" Then
        getCaption = False
        Exit Function
    End If
    
    sCaption = "User=" + sUID + ";DB=" + sDatabase + ";Node=" + sdbServer
                
    getCaption = True
    Exit Function

ErrGetCaption:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
Public Function getPWD(ByVal iIndex As Integer, sPWD As String) As Boolean
    On Error GoTo ErrGetPWD

    Dim sDriver As String
    Dim sUID As String
    
    Const FUNCTION_NAME = "Get Password"
    
    'rc
    getPWD = False
    
    'init pwd string empty
    sPWD = ""
    
    If m_bInitKeys = False Then Exit Function
    If iIndex > 10 Then Exit Function
        
    sPWD = m_oLRUConectionArray(iIndex).PWD
    
    'rc
    getPWD = True
    
    Exit Function
    
ErrGetPWD:
        internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
Public Function loadConnections() As Boolean
    On Error GoTo errLoadConnections

    Const FUNCTION_NAME = "Load Connections"
    
    Dim bRead            As Boolean
    Dim bReturn          As Boolean
    Dim iConnectionIndex As Integer
    Dim lKeyNumber       As Long
    Dim lDisposition     As Long
    Dim strConnectionKey As String
    
    'rc
    loadConnections = False
    
    'Init
    init
    
    bRead = True
    iConnectionIndex = 0
    
    If m_bInitKeys = False Then Exit Function
    
    While (bRead And (iConnectionIndex < iMaxArrayMembers))
    
        strConnectionKey = m_oComFunc.getRegBasicKey + "LRUConnections"
        If Len(m_sAppVersion) > 0 Then _
            strConnectionKey = strConnectionKey + "\" + m_sAppVersion
            
        strConnectionKey = strConnectionKey + "\Connection" + Trim(Str(iConnectionIndex))
         bReturn = m_oRegFunc.openKey(HKEY_CURRENT_USER, strConnectionKey, KEY_QUERY_VALUE, lKeyNumber)
         
         If bReturn = False Then Exit Function
         bRead = m_oLRUConectionArray(iConnectionIndex).ReadConnection(lKeyNumber)
         
         bReturn = m_oRegFunc.closeKey(strConnectionKey, lKeyNumber)
         If bRead Then iConnectionIndex = iConnectionIndex + 1
    Wend
     
    m_iConnectionCount = iConnectionIndex
    
    loadConnections = True
    
        
    Exit Function
 
errLoadConnections:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
Public Function saveConnections() As Boolean
    On Error GoTo errSaveConnections

    Const FUNCTION_NAME = "Save Connections"

    Dim bRead As Boolean
    
    ' Laufvaribale fr ConnectionArray
    Dim iConnectionIndex As Integer
    Dim bReturn As Boolean
    Dim lKeyNumber As Long
    Dim lDisposition As Long
    Dim strConnectionKey As String
    
    'Inits
    bRead = True
    iConnectionIndex = 0
    
    If m_bInitKeys = False Then
    
        saveConnections = False
        Exit Function
    End If
    
    If m_bDirty = False Then
    
        saveConnections = True
        Exit Function
    End If
    
    While (bRead And (iConnectionIndex < iMaxArrayMembers))
    
        strConnectionKey = m_oComFunc.getRegBasicKey + "LRUConnections"
        If Len(m_sAppVersion) > 0 Then _
            strConnectionKey = strConnectionKey + "\" + m_sAppVersion
            
        strConnectionKey = strConnectionKey + "\Connection" + Trim(Str(iConnectionIndex))
        bReturn = m_oRegFunc.openKey(HKEY_CURRENT_USER, strConnectionKey, KEY_SET_VALUE, lKeyNumber)
        If bReturn = False Then
            saveConnections = False
            Exit Function
        End If
    
        Set m_oLRUConectionArray(iConnectionIndex).regFunctions = m_oRegFunc
        bRead = m_oLRUConectionArray(iConnectionIndex).WriteConnection(lKeyNumber)
        bReturn = m_oRegFunc.closeKey(strConnectionKey, lKeyNumber)
         
        If bRead Then iConnectionIndex = iConnectionIndex + 1
    Wend
     
    m_iConnectionCount = iConnectionIndex
    
    m_bDirty = False
    
    Exit Function

errSaveConnections:
    internalError Err, MODULE_NAME, FUNCTION_NAME
    
End Function
Public Function clearLRUList() As Boolean
    On Error GoTo errClearLRUList
    
    Const FUNCTION_NAME = "Clear LRU List"
           
    Dim bReturn          As Boolean
    Dim iConnectionIndex As Integer ' Laufvaribale fr ConnectionArray
    Dim lKeyNumber       As Long
    Dim lDisposition     As Long
    Dim strConnectionKey As String
      
        
    'rc
    clearLRUList = False
    
    While (iConnectionIndex < iMaxArrayMembers)
 
        Set m_oLRUConectionArray(iConnectionIndex) = New LRUConnection
     
        strConnectionKey = m_oComFunc.getRegBasicKey + "LRUConnections"
        If Len(m_sAppVersion) > 0 Then _
            strConnectionKey = strConnectionKey + "\" + m_sAppVersion
            
        strConnectionKey = strConnectionKey + "\Connection" + Trim(Str(iConnectionIndex))
        
        bReturn = m_oRegFunc.openKey(HKEY_CURRENT_USER, strConnectionKey, KEY_ALL_ACCESS, lKeyNumber)
        
        'if lru list is cleared, make new keys
        If bReturn = False Then
        
            bReturn = m_oRegFunc.createKey(HKEY_CURRENT_USER, strConnectionKey, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, lKeyNumber, lDisposition)
            
            If bReturn = False Then Exit Function
        End If
        
        bReturn = m_oRegFunc.DeleteValue(lKeyNumber, m_cnstKEYNAMEDRIVER)
        bReturn = m_oRegFunc.setValueString(lKeyNumber, m_cnstKEYNAMEUID, "")
        bReturn = m_oRegFunc.setValueString(lKeyNumber, m_cnstKEYNAMEPWD, "")
        bReturn = m_oRegFunc.setValueString(lKeyNumber, m_cnstKEYDATABASE, "")
        bReturn = m_oRegFunc.setValueString(lKeyNumber, m_cnstKEYSERVERNODE, "")
        
        bReturn = m_oRegFunc.closeKey(strConnectionKey, lKeyNumber)
        iConnectionIndex = iConnectionIndex + 1
               
 Wend
    
 m_bInitKeys = True
 
 clearLRUList = True
 
 Exit Function
 
errClearLRUList:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

Public Property Let AppVersion(NewAppVersion As String)
    On Error GoTo ErrAppVersion
    
    Const FUNCTION_NAME = "LET AppVersion"
    
    m_sAppVersion = NewAppVersion
    
    Exit Property
    
ErrAppVersion:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property

Public Property Get AppVersion() As String
    On Error GoTo ErrAppVersion
    
    Const FUNCTION_NAME = "GET AppVersion"
    
    AppVersion = m_sAppVersion
    
    Exit Property
    
ErrAppVersion:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    public property:      regFunctions
'    description:          sets a registry lib object for doing registry operations
'    arguments:            oNewRegFunctions, new registry object
'
'    return value:  /
'
'=========================================================================================
Public Property Set regFunctions(oNewRegFunctions As SQLStudioUtil.Registry)
    On Error GoTo errRegFunctions
    
    Const FUNCTION_NAME = "regFunctions"

    Set m_oRegFunc = oNewRegFunctions
        
    Exit Property
    
errRegFunctions:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    public property:      dirty
'    description:          set dirty flag of lru connections
'    arguments:            bNewDirty
'
'    return value:  /
'
'=========================================================================================
Friend Property Let dirty(bNewDirty As Boolean)
 On Error GoTo errLetDirty
    
    Const FUNCTION_NAME = "errLetDirty"
    
    m_bDirty = bNewDirty
    
    Exit Property
    
errLetDirty:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
