VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "BreakStatement"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''/////////
'
' Module:      BreakStatement
'
'
'-----------------------------------------------------------------------
'
' responsible:  WolfgangA
'
' special area: SQLStudio
'
' description:  module for handling a stream of sql statements
'
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''/////////

Option Explicit

'module name
Const MODULE_NAME = "BreakStatement"

Public Enum statementShowType

    statementShowUnknown = 0
    statementShowSelect = 1
    statementShowShow = 2
    statementShowExplain = 3
    statementShowCommand = 4
    statementCallProcedure = 5
End Enum



'  Privat property variables
Private m_oConnection             As SQLStudioDBLayer.connection
'statements to be processed
Private m_sStatement              As String
'curent statement in stream
Private m_sCurrentStatement       As String
'number of current statement
Private m_lCurrentStatementNo     As Long
'EOS = EndOfStream
Private m_bEOS                    As Boolean
'result mode of object
Private m_enumResultMode          As SAPDBResultMode
'handle of calling window
Private m_lWnd                    As Long
'rest of statement stream which was not processed
Private m_sStatementTail          As String
'collection of comments tags
Private m_oAddComment     As New Collection
Public Function addComment(sComment As String)
    On Error GoTo errAddComment

    Const FUNCTION_NAME = "addComment"
    
    m_oAddComment.add sComment
    
    Exit Function
    
errAddComment:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    property:      callingWindow
'    description:   set windows handle of calling window
'    arguments:     [in] hWnd, handle of calling window
'
'    return value:  /
'
'=========================================================================================
Public Property Let callingWindow(hWnd As Long)
    On Error GoTo errP_Let_CallingWindow

    Const FUNCTION_NAME = "P_LET CallingWindow"
    
    m_lWnd = hWnd
    
    Exit Property
    
errP_Let_CallingWindow:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      Connection
'    description:   sets a sapdb connection object
'    arguments:     NewConnection, Connection object
'
'    return value:  /
'
'=========================================================================================
Public Property Set connection(NewConnection As SQLStudioDBLayer.connection)
    On Error GoTo errP_SET_Connection

    Const FUNCTION_NAME = "P_SET connection"
    
    Set m_oConnection = NewConnection
    
    Exit Property
    
errP_SET_Connection:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      Statement
'    description:   set the statement(s) to be processed
'    arguments:     snewStatement
'
'    return value:  /
'
'=========================================================================================
Public Property Let statement(sNewStatement As String)
    On Error GoTo errP_Let_Statement

    Const FUNCTION_NAME = "P_LET Statement"
    
    m_sStatement = sNewStatement
    
    'init processing of statement
    initnewStatement
    
    Exit Property
    
errP_Let_Statement:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      statement
'    description:   Get the statement(s) to be processed
'    arguments:     /
'
'    return value:  statement(s) of object
'
'=========================================================================================
Public Property Get statement() As String
    On Error GoTo errP_Get_Statement

    Const FUNCTION_NAME = "P_Get statement"
    
    statement = m_sStatement
    
    Exit Property
    
errP_Get_Statement:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      currentStatement
'    description:   Get the current statement of the statement stream
'    arguments:     /
'
'    return value:  current statement of object
'
'=========================================================================================
Public Property Get currentStatement() As String
    On Error GoTo errP_Get_CurrentStatement

    Const FUNCTION_NAME = "P_Get currentStatement"
    
    currentStatement = m_sCurrentStatement
    
    Exit Property
    
errP_Get_CurrentStatement:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      currentStatementNo
'    description:   Get the number of current statement of the statement stream
'    arguments:     /
'
'    return value:  number of current statement
'
'=========================================================================================
Public Property Get currentStatementNo() As Long
    On Error GoTo errP_Get_CurrentStatementNo

    Const FUNCTION_NAME = "P_Get currentStatementNo"
    
    currentStatementNo = m_lCurrentStatementNo
    
    Exit Property
    
errP_Get_CurrentStatementNo:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      eos
'    description:   flag if end of stream is reached
'    arguments:     /
'
'    return value:  true if end of stream is reached
'
'=========================================================================================
Public Property Get eos() As Boolean
    On Error GoTo errP_Get_EOS

    Const FUNCTION_NAME = "P_Get eOS"
    
    eos = m_bEOS
    
    Exit Property
    
errP_Get_EOS:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      StatementType
'    description:   Get statement type of current statement
'    arguments:     /
'
'    return value:  statement type of current statement
'
'=========================================================================================
Public Property Get statementType() As statementShowType
    On Error GoTo errP_Get_StatementType

    Const FUNCTION_NAME = "P_Get statementType"
    
    Dim currentStatement        As String
    
    'skip leading new lines, tabs, blanks
    currentStatement = m_sCurrentStatement
    
    ' Get statement type
    m_oComFunc.ReplaceNonChars currentStatement
    currentStatement = Trim$(currentStatement)
        
    If StrComp(left$(Trim$(currentStatement), 6), "SELECT", 1) = 0 Then
    
        statementType = statementShowSelect
    ElseIf StrComp(left$(currentStatement, 7), "DECLARE", 1) = 0 Then
    
        If InStr(7, currentStatement, "CURSOR", 1) Then
            statementType = statementShowSelect
        End If
    ElseIf StrComp(left$(currentStatement, 7), "EXPLAIN", 1) = 0 Then
    
        statementType = statementShowExplain
    ElseIf StrComp(left$(currentStatement, 4), "CALL", 1) = 0 Then
    
        statementType = statementCallProcedure
    ElseIf StrComp(left$(currentStatement, 4), "SHOW", 1) = 0 Then
    
        statementType = statementShowShow
    ElseIf currentStatement = "" Then
    
        statementType = statementShowUnknown
    Else
        statementType = statementShowCommand
    End If
    
    Exit Property
    
errP_Get_StatementType:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    function:      execute
'    description:   execute one sql statement
'
'    arguments:     [in] outResult, a db result object
'                   [in] emSQLMode, sql mode for execute
'
'    return value:  true if success
'
'=========================================================================================
Public Function execute(outResult As SQLStudioDBLayer.result, emSQLMode As sqlMode) As Boolean
    On Error GoTo errExecute
    
    Const FUNCTION_NAME = "execute"
    
    Dim oDBFunctions           As New SQLStudioDBLayer.Functions
    Dim oResult                As New SQLStudioDBLayer.result
    Dim sStatement             As String
    
    'rc
    execute = False
    
    'inits
    Set outResult = Nothing
    
    sStatement = m_sCurrentStatement
    getParameters sStatement
    
    oDBFunctions.callingWindow = m_lWnd
    Set oDBFunctions.connection = m_oConnection
    
    oResult.callingWindow = m_lWnd
    oResult.resultMode = m_enumResultMode
    Set oResult.connection = m_oConnection
    
    'switch statement type
    Select Case statementType
        'SELECT
        Case statementShowSelect, statementShowShow, statementShowExplain:
            'execute pre SQL call?
            If statementType = statementShowShow Or statementType = statementShowExplain Then
                
                If oDBFunctions.FullExecDirectStmt(sStatement, emSQLMode) Then
                                        
                    If Not oResult.OpenResult("SELECT * FROM SHOW", emSQLMode) Then
                    
                     If m_oConnection.Error.NativeError = "-4004" Then
                        
                        m_oConnection.Error.Reset
                     Else
                        Exit Function
                     End If
                    End If
                Else
                    m_oConnection.Error.showError
                End If
                
                oResult.AdditionalStatement = sStatement
            Else
                oResult.AdditionalStatement = ""
                If Not oResult.OpenResult(sStatement, emSQLMode) Then Exit Function
            End If

            'no result?
            If oResult.NoRows Then
            
                Set oResult = Nothing
            Else
                Set outResult = oResult
            End If
            
        Case statementCallProcedure
        
            oResult.AdditionalStatement = ""
            If Not oResult.OpenParamResult(sStatement, emSQLMode) Then Exit Function
            
            'no result or Execute return 100 Row not found in dbproc !
            If oResult.NoRows Or m_oConnection.Error.RC = "100" Then
            
                Set oResult = Nothing
            Else
                Set outResult = oResult
            End If
            
        ' sql statement to excecute
        Case Else
            If Not oDBFunctions.FullExecDirectStmt(sStatement, emSQLMode) Then Exit Function
            
    End Select

    execute = True
    
    Exit Function

errExecute:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      buildOneStatement
'    description:   get a statement out of a statemen t stream
'    arguments:     [in] sStatementTail, rest of the statement to handle
'
'    return value:  one valid statement
'
'=========================================================================================
Friend Function buildOneStatement(sStatementTail As String) As String
    On Error GoTo errBuildOneStatement
    
    Const FUNCTION_NAME = "buildOneStatement"

    Dim sLine         As String
    Dim sBuffer       As String
    Dim sOneStatement As String
    
    If Len(sStatementTail) < 1 Then Exit Function
    
    'find first not comment line
    getLine sStatementTail, sLine
    Do While (isLineComment(sLine))
    
        If getLine(sStatementTail, sLine) = False Then Exit Do
    Loop
    
    sOneStatement = sOneStatement + sLine
    
    If Len(sStatementTail) > 0 Then
    
        'find next comment line
        getLine sStatementTail, sLine
        Do While (Not isLineComment(sLine))
         
             sOneStatement = sOneStatement + sLine
             If getLine(sStatementTail, sLine) = False Then Exit Do
        Loop
    
    End If
    'if statement is just \n \r \t or spaces make a recursive call
    sBuffer = sOneStatement
    m_oComFunc.ReplaceNonChars sBuffer
    If Len(Trim(sBuffer)) < 1 Then _
        sOneStatement = buildOneStatement(sStatementTail)
        
    buildOneStatement = Trim$(sOneStatement)
    
    Exit Function
    
errBuildOneStatement:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      isLineComment
'    description:   check if line is comment
'    arguments:     [in] sLine, line to be examined
'
'    return value:  true, if line is comment line
'
'=========================================================================================
Private Function isLineComment(ByVal sLine As String) As Boolean
    On Error GoTo errIsLineComment
    
    Const FUNCTION_NAME = "isLineComment"
                             
    Dim lRun        As Long
    Dim sStrComment As String
    
    m_oComFunc.ReplaceNonChars sLine
    sLine = Trim$(sLine)
    
    For lRun = 1 To m_oAddComment.count
    
        sStrComment = m_oAddComment.item(lRun)
        If left$(sLine, 2) = sStrComment Then
        
            isLineComment = True
            Exit Function
        End If
    Next
    
    isLineComment = False
    
    Exit Function

errIsLineComment:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getLine
'    description:   get next line
'    arguments:     [in/out] sStatementTail, rest of statement, will be reduced
'                   [out] sLine, line to be examined
'
'    return value:  true, if success
'
'=========================================================================================
Private Function getLine(sStatementTail As String, _
                         sLine As String) As Boolean
    On Error GoTo errGetLine
    
    Const FUNCTION_NAME = "getLine"
                                  
    Dim FirstPos As Long
    
    'rc
    getLine = False
    
    'init
    sLine = ""
    
    If Len(sStatementTail) < 1 Then Exit Function
    
    FirstPos = InStr(sStatementTail, vbCrLf)
    If FirstPos < 1 Then
        FirstPos = InStr(sStatementTail, vbCr)
        If FirstPos < 1 Then
            FirstPos = InStr(sStatementTail, vbLf)
        End If
    Else
        'FirstPos = FirstPos+1
    End If
    
    If FirstPos > 0 Then
        sLine = left(sStatementTail, FirstPos)
        sStatementTail = Right(sStatementTail, Len(sStatementTail) - FirstPos)
    Else
        sLine = sStatementTail
        sStatementTail = ""
    End If
    
    getLine = True
    
    Exit Function

errGetLine:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      nextStatement
'    description:   get next statement
'    arguments:     /
'
'    return value:  return next statement of statement stream
'
'=========================================================================================
Public Function nextStatement() As String
    On Error GoTo errNextStatement

    Const FUNCTION_NAME = "nextStatement"

    'initialization
    nextStatement = ""
    m_bEOS = True
    
    m_lCurrentStatementNo = m_lCurrentStatementNo + 1
    m_sCurrentStatement = buildOneStatement(m_sStatementTail)
    If Len(m_sCurrentStatement) < 1 Then
        m_bEOS = True
    Else
        m_bEOS = False
    End If

    nextStatement = m_sCurrentStatement

    Exit Function

errNextStatement:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      initnewStatement
'    description:   init break statements object
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Private Sub initnewStatement()
    On Error GoTo errInitnewStatement

    Const FUNCTION_NAME = "initnewStatement"

    'inits
    m_bEOS = True
    m_sCurrentStatement = ""
    m_lCurrentStatementNo = 0
    
    m_sStatementTail = m_sStatement
    
    nextStatement

    Exit Sub

errInitnewStatement:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    function:      getParameters
'    description:   search for parameters in query e.g. select * from [table:]
'    arguments:     [in] sStatement, statement to be searched
'
'    return value:  true if success
'
'=========================================================================================
Private Function getParameters(sStatement As String) As Boolean
    On Error GoTo errGetParameters

    Const FUNCTION_NAME = "getParameters"
    
    getParameters = commonGetParameters(sStatement)

    Exit Function

errGetParameters:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      GetSQLParams
'    description:   get parameters of sql statement
'    arguments:     [in] SQLStatement
'
'    return value:  /
'
'=========================================================================================
Sub getSQLParams(SQLStatement As String)
    On Error GoTo errGetSQLParams

    Const FUNCTION_NAME = "getSQLParams"

    Dim StrPos      As Integer
    Dim StartPos    As Integer
    Dim EndPos      As Integer
    Dim SQLLen      As Integer
    Dim newSQL      As String
    
    StartPos = InStr(SQLStatement, "[")
    If StartPos = 0 Then Exit Sub
        
    SQLLen = Len(SQLStatement)
    StrPos = 1

    While StrPos <= SQLLen
    
        StartPos = InStr(StrPos, SQLStatement, "[")
        If StartPos > 0 Then
            EndPos = InStr(StartPos, SQLStatement, "]")
            If EndPos > 0 Then
            
                newSQL = newSQL + Mid$(SQLStatement, StrPos, StartPos - StrPos)
                newSQL = newSQL + InputBox(Mid$(SQLStatement, StartPos + 1, EndPos - StartPos - 1), "Input Parameter")
                StrPos = EndPos + 1
            Else
                newSQL = newSQL + Right$(SQLStatement, SQLLen - StrPos + 1)
                StrPos = SQLLen + 1
            End If
        Else
            newSQL = newSQL + Right$(SQLStatement, SQLLen - StrPos + 1)
            StrPos = SQLLen + 1
        End If
    Wend

    SQLStatement = newSQL

    Exit Sub

errGetSQLParams:
    Call internalError(Err, MODULE_NAME, FUNCTION_NAME)
End Sub
'=========================================================================================
'    function:      ..._Initialize
'    description:   Occurs when an application creates an instance of a
'                   Form, MDIForm, User control, Property Page, DHTML Page Designer, or class.
'
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Private Sub Class_Initialize()
    On Error GoTo ErrClass_Initialize

    Const FUNCTION_NAME = "Class_Initialize"
    
    'set hanlde of calling window to zero
    m_lWnd = 0
    
    Exit Sub

ErrClass_Initialize:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    property:      resultMode
'    description:   value of sapdb result mode
'    arguments:     newResultMode
'
'    return value:  /
'
'=========================================================================================
Public Property Let resultMode(newResultMode As SAPDBResultMode)
    On Error GoTo errP_Let_ResultMode

    Const FUNCTION_NAME = "P_LET ResultMode"
    
    m_enumResultMode = newResultMode
    
    Exit Property
    
errP_Let_ResultMode:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      resultMode
'    description:   get value of sapdb result mode
'    arguments:
'
'    return value:  sap db result mode of object
'
'=========================================================================================
Public Property Get resultMode() As SAPDBResultMode
    On Error GoTo errP_Get_ResultMode

    Const FUNCTION_NAME = "P_Get resultMode"
    
    resultMode = m_enumResultMode
    
    Exit Property
    
errP_Get_ResultMode:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    function:      ExecuteStatements
'    description:   execute a bundle of statements
'
'    arguments:     [in] Results, a collection of results
'                   [in] emSQLMode, sql mode for execute
'
'    return value:  true if success
'
'=========================================================================================
Public Function executeStatements(Results As Collection, emSQLMode As sqlMode) As Boolean
    On Error GoTo ErrExecuteStatements

    Const FUNCTION_NAME = "executeStatements"

    Dim lRun            As Long
    Dim bOk             As Boolean
    Dim Answer          As Integer
    Dim NewCollection   As New Collection
    Dim oResult         As SQLStudioDBLayer.result
    
    'inits
    executeStatements = False
        
    If m_oConnection Is Nothing Then
    
        MsgBox LoadResString(114)
        Exit Function
    End If
    
    'test now because if m_oConnection Is Nothing m_oConnection.hDbc can't be accessed
    If m_oConnection.hDbc = 0 Then
    
        MsgBox LoadResString(114)
        Exit Function
    End If
    
    'statement to execute?
    While Not m_bEOS And _
          Len(Trim$(m_sCurrentStatement)) > 0 And _
          Trim$(m_sCurrentStatement) <> vbCrLf
          
        bOk = execute(oResult, emSQLMode)
        If bOk Then
        
            If Not oResult Is Nothing Then
                
                NewCollection.add oResult 'add result to result collection
                Set oResult = Nothing
            End If
        Else
            m_oConnection.Error.showError       'show error
        End If
                
        nextStatement       'get next statement
        
        '  If the last statement generated an error
        If Not bOk Then
            
            If Not m_bEOS Then 'Anyway next statement?
            
                Answer = MsgBox(LoadResString(209), vbYesNo + vbExclamation, LoadResString(102))
                If Answer = vbNo Then '  No of Yes/No
                
                    If NewCollection.count > 0 Then Set Results = NewCollection
                    Exit Function
                End If
            Else
                Exit Function
            End If
        End If
    Wend

    'set rc
    Set Results = NewCollection
    
    executeStatements = True

    Exit Function

ErrExecuteStatements:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function


