VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "PredicateOrderCols"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
'  Class:      PredicateOrderCols - Class for OrderColumns Predicates
'
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option Explicit

'--------------------------------------------------------
'module name
'--------------------------------------------------------
Const MODULE_NAME = "PredicateOrderCols"

'properties
Private m_colOrderColumns    As New Collection
Private m_emPredicateTyp   As predicateTyp


Private Property Let predicateTyp(NewPredicateTyp As predicateTyp)
    On Error GoTo errP_Let_PredicateTyp

    Const FUNCTION_NAME = "P_LET PredicateTyp"
    
    m_emPredicateTyp = NewPredicateTyp
    
    Exit Property
    
errP_Let_PredicateTyp:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
Public Property Get predicateTyp() As predicateTyp
    On Error GoTo errP_Get_PredicateTyp

    Const FUNCTION_NAME = "P_GET PredicateTyp"
    
    predicateTyp = m_emPredicateTyp
    
    Exit Property
    
errP_Get_PredicateTyp:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
Public Property Get OrderColumns() As Collection
    On Error GoTo errP_Get_OrderColumns

    Const FUNCTION_NAME = "P_GET OrderColumns"
    
    Set OrderColumns = m_colOrderColumns
    
    Exit Property
    
errP_Get_OrderColumns:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property

Public Sub AddOrderColumn(OrderColumn As PredicateOrderCol)
    On Error GoTo errAddOrderColumn

    Const FUNCTION_NAME = "AddOrderColumn"
    
    m_colOrderColumns.add OrderColumn, OrderColumn.columnName
    
    Exit Sub
    
errAddOrderColumn:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub

Public Sub RemoveOrderColumn(OrderColumnName As String)
    On Error GoTo errRemoveOrderColumn

    Const FUNCTION_NAME = "RemoveOrderColumn"
    
    m_colOrderColumns.remove OrderColumnName
    
    Exit Sub
    
errRemoveOrderColumn:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub

Public Sub FreeOrderColumns()
    On Error GoTo ErrFreeOrderColumns

    Const FUNCTION_NAME = "FreeOrderColumns"
    
    Dim i   As Integer
    
    For i = 1 To m_colOrderColumns.count
    
        OrderColumns.remove 1
    Next
    
    Exit Sub
    
ErrFreeOrderColumns:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub

Private Sub Class_Initialize()
    On Error GoTo ErrClass_Initialize

    Const FUNCTION_NAME = "Class_Initialize"
    
    'inits
    m_emPredicateTyp = PredicateOrderCols
        
    Exit Sub
    
ErrClass_Initialize:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub

Public Function getDescription(Description As String) As Boolean
    On Error GoTo errGetDescription

    Const FUNCTION_NAME = "GetDescription"
    
    Dim NewDescription          As String
    Dim OrderColumn             As PredicateOrderCol
    Dim OrderColumnDescription  As String

    Dim SeparatorValue As Integer
        
    '  Init
    getDescription = False
    SeparatorValue = 3
    
    'check separator value
    If SeparatorValue > 6 Then
        'separator value + 2 can be a printable character
        Exit Function
    End If
    
    'get property PredicateTyp
    NewDescription = Trim$(Str$(m_emPredicateTyp)) + Chr$(SeparatorValue)
    
    'get property OrderColumns
    For Each OrderColumn In m_colOrderColumns
        If Not OrderColumn.getDescription(OrderColumnDescription) Then
            Exit Function
        Else
            NewDescription = NewDescription + OrderColumnDescription + Chr$(SeparatorValue + 1)
        End If
    Next
    NewDescription = NewDescription + Chr$(SeparatorValue)
    
    'rc
    Description = NewDescription
    
    getDescription = True
    
    Exit Function

errGetDescription:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

Public Function setDescription(Description As String) As Boolean
    On Error GoTo errSetDescription

    Const FUNCTION_NAME = "SetDescription"
    
    Dim NextToken               As String
    Dim OrderColumnsDescription As String
    Dim OrderColumnDescription As String
    
    '  Init
    setDescription = False
    
    'check description
    If Len(Description) < 1 Then
    
        setDescription = True
        Exit Function
    End If
    
    'set property PredicateTyp
    m_oComFunc.GetNextToken Description, NextToken, m_oComFunc.GetTokenSeperator(qplObjectLevel4)
    m_emPredicateTyp = Val(NextToken)
    
    'set property OrderColumns
    m_oComFunc.GetNextToken Description, OrderColumnsDescription, m_oComFunc.GetTokenSeperator(qplObjectLevel4)
    While Len(OrderColumnsDescription) > 0
        Dim NewPredicateOrderCol  As New PredicateOrderCol
        
        m_oComFunc.GetNextToken OrderColumnsDescription, OrderColumnDescription, m_oComFunc.GetTokenSeperator(qplObjectLevel5)
        If Not NewPredicateOrderCol.setDescription(OrderColumnDescription) Then
            Exit Function
        End If
        
        AddOrderColumn NewPredicateOrderCol
        Set NewPredicateOrderCol = Nothing
    Wend
    
    'rc
    setDescription = True
    
    Exit Function

errSetDescription:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
Public Function getPredicateAsStatement(bFormated As Boolean) As String
    On Error GoTo errGetPredicateAsStatement

    Const FUNCTION_NAME = "GetPredicateAsStatement"

    Dim bFirst                    As Boolean
    Dim CurrentColumnClause       As String
    Dim oColumn                  As SQLStudioDBLayer.column
    Dim OrderColumn               As PredicateOrderCol
    

    'rc
    getPredicateAsStatement = ""
    
    'inits
    CurrentColumnClause = ""
    
    bFirst = True
    
    For Each OrderColumn In m_colOrderColumns
        
        If bFirst Then
            bFirst = False
            CurrentColumnClause = " ORDER BY "
        Else
            CurrentColumnClause = CurrentColumnClause + ", "
        End If
        
        If OrderColumn.direction = OrderAscending Then
            CurrentColumnClause = CurrentColumnClause + OrderColumn.columnName + " ASC"
        Else
            CurrentColumnClause = CurrentColumnClause + OrderColumn.columnName + " DESC"
        End If
    Next
    
    'rc
    getPredicateAsStatement = CurrentColumnClause
   
    Exit Function
    
errGetPredicateAsStatement:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function






