VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "ResultLevels"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' Module:      ResultLevels
'
'-----------------------------------------------------------------------
'
' responsible: WolfgangA
'
' special area: SQLStudio
'
' description:   object for dealing a collection of result levels
'
'
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''


Option Explicit

'module name
Const MODULE_NAME = "ResultLevels"


Private m_oItems                 As New Collection
Private m_sWhereClause           As String
Private m_sFromList              As String

'=========================================================================================
'    property:      whereClause
'    description:   Get where clause of sql statment(database result)
'                   which is described by result levels objet
'    arguments:     /
'
'    return value:  where clause as string
'
'=========================================================================================
Public Property Get whereClause() As String
    On Error GoTo errP_Get_WhereClause

    Const FUNCTION_NAME = "P_Get whereClause"
    
    whereClause = m_sWhereClause
    
    Exit Property
    
errP_Get_WhereClause:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      whereClause
'    description:   let where clause of sql statment(database result)
'                   which is described by result levels objet
'    arguments:     [in] NewWhereClause, new where clause as string
'
'    return value:  /
'
'=========================================================================================
Public Property Let whereClause(NewWhereClause As String)
    On Error GoTo errP_Let_WhereClause

    Const FUNCTION_NAME = "P_LET WhereClause"
    
    m_sWhereClause = NewWhereClause
    
    Exit Property
    
errP_Let_WhereClause:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      fromList
'    description:   let the from part of sql statment(database result)
'                   which is described by result levels objet
'    arguments:     /
'
'    return value:  new from part as string
'
'=========================================================================================
Public Property Get fromList() As String
    On Error GoTo errP_Get_FromList

    Const FUNCTION_NAME = "P_Get fromList"
    
    fromList = m_sFromList
    
    Exit Property
    
errP_Get_FromList:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      fromList
'    description:   let the from part of sql statment(database result)
'                   which is described by result levels objet
'    arguments:     [in] NewFromList, new from part as string
'
'    return value:  /
'
'=========================================================================================
Public Property Let fromList(NewFromList As String)
    On Error GoTo errP_Let_FromList

    Const FUNCTION_NAME = "P_LET FromList"
    
    m_sFromList = NewFromList
    
    Exit Property
    
errP_Let_FromList:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      count
'    description:   get number of items in resultlevel objects
'
'    arguments:     /
'
'    return value:  number of items in resultlevel objects
'
'=========================================================================================
Public Property Get count() As Integer
    On Error GoTo errP_Get_Count

    Const FUNCTION_NAME = "P_Get count"
    
    count = m_oItems.count
    
    Exit Property
    
errP_Get_Count:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      item
'    description:   get in resultlevel object
'
'    arguments:     Index, index of resultlevel object
'
'    return value:  resultlevel object
'
'=========================================================================================
Public Property Get item(ByVal index As Variant) As ResultLevel
    On Error GoTo errP_Get_Item

    Const FUNCTION_NAME = "P_GET Item"
    
    'rc
    Set item = Nothing
    If m_oItems.count > 0 Then Set item = m_oItems.item(index)
    
    Exit Property
    
errP_Get_Item:
    If Err.Number = 5 Then  ''' Key,Item doesn't exists
        
        Set item = Nothing
        Exit Property
    End If
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    function:      add
'    description:   add a ResultLevel object to collection
'
'    arguments:     [in] oItem, a ResultLevel object
'
'    return value:  true, if success
'
'=========================================================================================
Public Function add(oItem As ResultLevel) As Boolean
    On Error GoTo errAdd

    Const FUNCTION_NAME = "add"
    
    Dim sKey      As Variant
    Dim TestItem  As ResultLevel
    
    'rc
    add = False
    
    If oItem Is Nothing Then Exit Function
    
    sKey = Chr$(34) + oItem.tableOwner + Chr$(34) + _
           m_oComFunc.GetTokenSeperator(qplObjectLevel4) + _
           Chr$(34) + oItem.tableName + Chr$(34) + _
           m_oComFunc.GetTokenSeperator(qplObjectLevel4)
    'set synonym name
    If Len(oItem.synonym) < 1 Then
    
        sKey = sKey + Chr$(34) + oItem.columnName + Chr$(34)
    Else
        sKey = sKey + Chr$(34) + oItem.synonym + Chr$(34)
    End If
        
    Set TestItem = item(sKey)
    If TestItem Is Nothing Then
    
        m_oItems.add oItem, sKey
        oItem.index = m_oItems.count
    End If
    
    add = True
    
    Exit Function
    
errAdd:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      remove
'    description:   remove a result level from collection
'
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function remove(ByVal index As Variant)
    On Error GoTo errRemove

    Const FUNCTION_NAME = "Remove"
    
    Dim lIndex As Long
    m_oItems.remove index
    'reinit index values of items
    For lIndex = 1 To m_oItems.count
    
        m_oItems.item(lIndex).index = lIndex
    Next
    
    Exit Function
    
errRemove:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      clear
'    description:   clear result levels
'
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function clear()
    On Error GoTo errClear

    Const FUNCTION_NAME = "clear"
    
    Dim lIndex As Long
    
    For lIndex = 1 To m_oItems.count
    
        m_oItems.remove 1
    Next
    
    Exit Function
    
errClear:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
