VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "StoredFileContent"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' class:      StoredFileContent
'
'
'-----------------------------------------------------------------------
'
' responsible:  WolfgangA
'
' special area: CatalogView
'
' description:  class for loading,saving, ... stored objects of sql studio in the
'               database file system
'
' see also:
'
'-----------------------------------------------------------------------
'
'  copyright:                Copyright by SAP AG, 2000
'
'
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option Explicit

'--------------------------------------------------------
'module name
'--------------------------------------------------------
Const MODULE_NAME = "StoredFileContent"


Public Enum fileType

    fileTypeUnknown = 0
    fileTypeStatement = 1
    fileTypeVisualQuery = 2
    fileTypeQueryByExample = 5 ' don't change this value, because it was stored in Objects
    fileTypeDraftTable = 6
    fileTypeDraftView = 7
    fileTypeDraftIndex = 8
    fileTypeQuickQuery = 9
End Enum

Public Enum fileSaveState

    Saved = 0
    NotSave = 1
    SaveCanceled = 2
    SaveError = 3
    SaveBegin = 4
    DeleteBegin = 5
    DeleteEnd = 6
End Enum


'local property copies
Private m_sSQLData          As String
Private m_sTableData        As String
Private m_sJoinData         As String
Private m_sViewData         As String
Private m_sLinkData         As String
Private m_lVersion          As Long
Private m_lAppVersion       As Long


Private m_sFileName         As String

Private m_emAutoCommit      As SQLStudioDBLayer.autoCommit
Private m_emIsoLevel        As SQLStudioDBLayer.isoLevel
Private m_emSQLMode         As SQLStudioDBLayer.sqlMode

Private m_emfileType           As fileType
Private m_emStoredIcon         As cvItemType
'=========================================================================================
'    property:      fileType
'    description:   set which sql studio object type is represented
'    arguments:     newfileType
'
'    return value:  /
'
'=========================================================================================
Public Property Let fileType(NewfileType As fileType)
    On Error GoTo errP_Let_fileType

    Const FUNCTION_NAME = "P_LET fileType"
    
    m_emfileType = NewfileType
    
    Exit Property
    
errP_Let_fileType:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      fileType
'    description:   Get which sql studio object type is represented
'    arguments:     /
'
'    return value:  fileType
'
'=========================================================================================
Public Property Get fileType() As fileType
    On Error GoTo errP_Get_fileType

    Const FUNCTION_NAME = "P_Get fileType"
    
    fileType = m_emfileType
    
    Exit Property
    
errP_Get_fileType:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      sqlData
'    description:   set the sql data to be stored
'    arguments:     newSQLData
'
'    return value:  /
'
'=========================================================================================
Public Property Let sqlData(newSQLData As String)
    On Error GoTo errP_Let_SQLData

    Const FUNCTION_NAME = "P_LET SQLData"
    
    m_sSQLData = newSQLData
    
    Exit Property
    
errP_Let_SQLData:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      sqlData
'    description:   Get the sql data of the object
'    arguments:     /
'
'    return value:  sql data as string
'
'=========================================================================================
Public Property Get sqlData() As String
    On Error GoTo errP_Get_SQLData

    Const FUNCTION_NAME = "P_Get sQLData"
    
    sqlData = m_sSQLData
    
    Exit Property
    
errP_Get_SQLData:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      tableData
'    description:   set the table data of the object
'    arguments:     newTableData
'
'    return value:  /
'
'=========================================================================================
Public Property Let tableData(newTableData As String)
    On Error GoTo errP_Let_TableData

    Const FUNCTION_NAME = "P_LET TableData"
    
    m_sTableData = newTableData
    
    Exit Property
    
errP_Let_TableData:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      tableData
'    description:   Get the table data of the object
'    arguments:     /
'
'    return value:  table data as string
'
'=========================================================================================
Public Property Get tableData() As String
    On Error GoTo errP_Get_TableData

    Const FUNCTION_NAME = "P_Get tableData"
    
    tableData = m_sTableData
    
    Exit Property
    
errP_Get_TableData:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      joinData
'    description:   set the join data of the object
'    arguments:     newJoinData
'
'    return value:  /
'
'=========================================================================================
Public Property Let joinData(newJoinData As String)
    On Error GoTo errP_Let_JoinData

    Const FUNCTION_NAME = "P_LET JoinData"
    
    m_sJoinData = newJoinData
    
    Exit Property
    
errP_Let_JoinData:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      joinData
'    description:   Get the join data of the object
'    arguments:     /
'
'    return value:  join data as string
'
'=========================================================================================
Public Property Get joinData() As String
    On Error GoTo errP_Get_JoinData

    Const FUNCTION_NAME = "P_GET JoinData"
    
    joinData = m_sJoinData
    
    Exit Property
    
errP_Get_JoinData:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      viewData
'    description:   set the view data of the object
'    arguments:     newViewData
'
'    return value:  /
'
'=========================================================================================
Public Property Let viewData(newViewData As String)
    On Error GoTo errP_Let_ViewData

    Const FUNCTION_NAME = "P_Let viewData"
    
    m_sViewData = newViewData
    
    Exit Property
    
errP_Let_ViewData:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      viewData
'    description:   Get the view data of the object
'    arguments:     /
'
'    return value:  view data as string
'
'=========================================================================================
Public Property Get viewData() As String
    On Error GoTo errP_Get_ViewData

    Const FUNCTION_NAME = "P_GET ViewData"
    
    viewData = m_sViewData
    
    Exit Property
    
errP_Get_ViewData:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      linkData
'    description:   set the link data of the object
'    arguments:     newLinkData
'
'    return value:  /
'
'=========================================================================================
Public Property Let linkData(newLinkData As String)
    On Error GoTo errP_Let_LinkData

    Const FUNCTION_NAME = "P_LET LinkData"
    
    m_sLinkData = newLinkData
    
    Exit Property
    
errP_Let_LinkData:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      linkData
'    description:   Get the link data of the object
'    arguments:     /
'
'    return value:  link data as string
'
'=========================================================================================
Public Property Get linkData() As String
    On Error GoTo errP_Get_LinkData

    Const FUNCTION_NAME = "P_Get linkData"
    
    linkData = m_sLinkData
    
    Exit Property
    
errP_Get_LinkData:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      autoCommit
'    description:   Get the auto commit on/off flag of the object
'    arguments:     /
'
'    return value:  SQLStudioDBLayer.AutoCommit
'
'=========================================================================================
Public Property Get autoCommit() As SQLStudioDBLayer.autoCommit
    On Error GoTo errP_Get_AutoCommit

    Const FUNCTION_NAME = "P_Get autoCommit"
    
    autoCommit = m_emAutoCommit
    
    Exit Property
    
errP_Get_AutoCommit:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      autoCommit
'    description:   set the auto commit on/off flag of the object
'    arguments:     newAutoCommit
'
'    return value:  /
'
'=========================================================================================
Public Property Let autoCommit(newAutoCommit As SQLStudioDBLayer.autoCommit)
    On Error GoTo errP_Let_AutoCommit

    Const FUNCTION_NAME = "P_Let autoCommit"
    
    m_emAutoCommit = newAutoCommit
    
    Exit Property
    
errP_Let_AutoCommit:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      isoLevel
'    description:   Get the value of the iso level of the object
'    arguments:     /
'
'    return value:  SQLStudioDBLayer.isoLevel
'
'=========================================================================================
Public Property Get isoLevel() As SQLStudioDBLayer.isoLevel
    On Error GoTo errP_Get_IsoLevel

    Const FUNCTION_NAME = "P_GET IsoLevel"
    
    isoLevel = m_emIsoLevel
    
    Exit Property
    
errP_Get_IsoLevel:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      isoLevel
'    description:   set the value of the iso level of the object
'    arguments:     newIsoLevel
'
'    return value:  /
'
'=========================================================================================
Public Property Let isoLevel(NewIsoLevel As SQLStudioDBLayer.isoLevel)
    On Error GoTo errP_Let_IsoLevel

    Const FUNCTION_NAME = "P_Let isoLevel"
    
    m_emIsoLevel = NewIsoLevel
    
    Exit Property
    
errP_Let_IsoLevel:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      sqlMode
'    description:   Get the value of the sql mode of the object
'    arguments:     /
'
'    return value:  SQLStudioDBLayer.sqlMode
'
'=========================================================================================
Public Property Get sqlMode() As SQLStudioDBLayer.sqlMode
    On Error GoTo errP_Get_SQLMode

    Const FUNCTION_NAME = "P_Get sqlMode"
    
    sqlMode = m_emSQLMode
    
    Exit Property
    
errP_Get_SQLMode:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      sqlMode
'    description:   let the value of the sql mode of the object
'    arguments:     newSQLMode
'
'    return value:  /
'
'=========================================================================================
Public Property Let sqlMode(newSQLMode As SQLStudioDBLayer.sqlMode)
    On Error GoTo errP_Let_SQLMode

    Const FUNCTION_NAME = "P_LET SQLMode"
    
    m_emSQLMode = newSQLMode
    
    Exit Property
    
errP_Let_SQLMode:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    function:      setObjectFromBuffer
'    description:   set stored object from a buffer
'
'    arguments:     [in] sData, string that contains stored object data
'
'    return value:  true if successfully
'
'=========================================================================================
Public Function setObjectFromBuffer(sData As String) As Boolean
    On Error GoTo errSetObjectFromBuffer

    Const FUNCTION_NAME = "setObjectFromBuffer"
    
    Dim sNextToken As String
    Dim sMessage   As String
        
    'inits
    setObjectFromBuffer = False
    
    'set icon type
    sNextToken = ""
    m_oComFunc.GetNextToken sData, sNextToken, m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    If Len(sNextToken) < 1 Then Exit Function
    If IsNumeric(sNextToken) Then
        m_emStoredIcon = CLng(sNextToken)
    Else
        Exit Function
    End If
       
    'set object type
    sNextToken = ""
    m_oComFunc.GetNextToken sData, sNextToken, m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    If Len(sNextToken) < 1 Then Exit Function
    If IsNumeric(sNextToken) Then
        m_emfileType = CLng(sNextToken)
    Else
        Exit Function
    End If
    
    sNextToken = ""
    m_oComFunc.GetNextToken sData, sNextToken, m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    m_sSQLData = sNextToken
    
    m_oComFunc.GetNextToken sData, sNextToken, m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    m_sTableData = sNextToken
    
    m_oComFunc.GetNextToken sData, sNextToken, m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    m_sJoinData = sNextToken
    
    m_oComFunc.GetNextToken sData, sNextToken, m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    m_sViewData = sNextToken
    
    m_oComFunc.GetNextToken sData, sNextToken, m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    m_sLinkData = sNextToken
    
    m_oComFunc.GetNextToken sData, sNextToken, m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    If Len(sNextToken) < 1 Then Exit Function
    If IsNumeric(sNextToken) Then
        m_emAutoCommit = CLng(sNextToken)
    Else
        Exit Function
    End If
    
    m_oComFunc.GetNextToken sData, sNextToken, m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    If Len(sNextToken) < 1 Then Exit Function
    If IsNumeric(sNextToken) Then
        m_emSQLMode = CLng(sNextToken)
    Else
        Exit Function
    End If
    
    m_oComFunc.GetNextToken sData, sNextToken, m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    If Len(sNextToken) < 1 Then Exit Function
    If IsNumeric(sNextToken) Then
        m_emIsoLevel = CLng(sNextToken)
    Else
        Exit Function
    End If
    
    m_oComFunc.GetNextToken sData, sNextToken, m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    If Len(sNextToken) < 1 Then Exit Function
    If IsNumeric(sNextToken) Then
        m_lVersion = CLng(sNextToken)
    Else
        Exit Function
    End If
                                   
    'check it
    If m_lVersion > m_lAppVersion Then
    
        MsgBox LoadResString(210) + " (" + Trim$(Str$(m_lVersion)) + ") " + LoadResString(99), vbCritical
        Exit Function
    End If
                 
    'set rc
    setObjectFromBuffer = True
    
    Exit Function
    
errSetObjectFromBuffer:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getObjectToBuffer
'    description:   ghet stored object to a buffer
'
'    arguments:     /
'
'    return value:  buffer as string
'
'=========================================================================================
Public Function getObjectToBuffer() As String
    On Error GoTo errGetObjectToBuffer
    
    Const FUNCTION_NAME = "getObjectToBuffer"
    
    getObjectToBuffer = Trim(Str(m_emStoredIcon)) + m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    getObjectToBuffer = getObjectToBuffer + Trim$(Str$(m_emfileType)) + m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    getObjectToBuffer = getObjectToBuffer + m_sSQLData + m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    getObjectToBuffer = getObjectToBuffer + m_sTableData + m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    getObjectToBuffer = getObjectToBuffer + m_sJoinData + m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    getObjectToBuffer = getObjectToBuffer + m_sViewData + m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    getObjectToBuffer = getObjectToBuffer + m_sLinkData + m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    
    getObjectToBuffer = getObjectToBuffer + Trim(Str(m_emAutoCommit)) + m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    getObjectToBuffer = getObjectToBuffer + Trim(Str(m_emSQLMode)) + m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    getObjectToBuffer = getObjectToBuffer + Trim(Str(m_emIsoLevel)) + m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    getObjectToBuffer = getObjectToBuffer + Trim(Str(m_lVersion)) + m_oComFunc.GetTokenSeperator(qplSaveObjectLevel1)
    
    
    Exit Function
        
errGetObjectToBuffer:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      ..._Initialize
'    description:   Occurs when an application creates an instance of a
'                   Form, MDIForm, User control, Property Page, DHTML Page Designer, or class.
'
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Private Sub Class_Initialize()
    On Error GoTo ErrClass_Initialize

    Const FUNCTION_NAME = "Class_Initialize"
    
    m_emAutoCommit = AutoCommitOn
    m_emIsoLevel = Committed
    m_emSQLMode = Internal
    
    m_lAppVersion = 72200
    m_lVersion = m_lAppVersion
    
    Exit Sub
    
ErrClass_Initialize:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Sub
'=========================================================================================
'    property:      Version
'    description:   Let version of stored object
'    arguments:     [in] NewVersion, version of stored object
'
'    return value:  /
'
'=========================================================================================
Public Property Let version(newVersion As Long)
    On Error GoTo errP_Let_Version

    Const FUNCTION_NAME = "P_Let version"
    
    m_lVersion = newVersion
    
    Exit Property
    
errP_Let_Version:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      Version
'    description:   get version of stored object
'    arguments:     /
'
'    return value:  version of stored object
'
'=========================================================================================
Public Property Get version() As Long
    On Error GoTo errP_Get_Version

    Const FUNCTION_NAME = "P_GET Version"
    
   version = m_lVersion
    
    Exit Property
    
errP_Get_Version:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      AppVersion
'    description:   Let version of sql studio application which uses the class
'    arguments:     [in] NewAppVersion, version of sql studio application
'
'    return value:  /
'
'=========================================================================================
Public Property Let appVersion(newAppVersion As Long)
    On Error GoTo errP_Let_AppVersion

    Const FUNCTION_NAME = "P_Let appVersion"
    
    m_lAppVersion = newAppVersion
    
    Exit Property
    
errP_Let_AppVersion:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      AppVersion
'    description:   get version of sql studio application which uses the class
'    arguments:     /
'
'    return value:  version of sql studio application
'
'=========================================================================================
Public Property Get appVersion() As Long
    On Error GoTo errP_Get_AppVersion

    Const FUNCTION_NAME = "P_Get appVersion"
    
   appVersion = m_lAppVersion
    
    Exit Property
    
errP_Get_AppVersion:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      storedIcon
'    description:   set which item icon represents file data
'    arguments:     newStoredIcon
'
'    return value:  /
'
'=========================================================================================
Public Property Let storedIcon(newStoredIcon As cvItemType)
    On Error GoTo errP_Let_storedIcon

    Const FUNCTION_NAME = "P_LET storedIcon"
    
    m_emStoredIcon = newStoredIcon
    
    Exit Property
    
errP_Let_storedIcon:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      storedIcon
'    description:   Get which item icon represents file data
'    arguments:     /
'
'    return value:  cvItemType
'
'=========================================================================================
Public Property Get storedIcon() As cvItemType
    On Error GoTo errP_Get_StoredIcon

    Const FUNCTION_NAME = "P_Get storedIcon"
    
    storedIcon = m_emStoredIcon
    
    Exit Property
    
errP_Get_StoredIcon:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      fileName
'    description:   set name of studio object
'    arguments:     [in] newFileName,name of object as string
'
'    return value:  /
'
'=========================================================================================
Public Property Let fileName(newFileName As String)
    On Error GoTo errP_Let_fileName

    Const FUNCTION_NAME = "P_LET fileName"
    
    m_sFileName = newFileName
    
    Exit Property
    
errP_Let_fileName:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      fileName
'    description:   get name of studio object stored file
'    arguments:     /
'
'    return value:  see description
'
'=========================================================================================
Public Property Get fileName() As String
    On Error GoTo errP_Get_fileName

    Const FUNCTION_NAME = "P_Get fileName"
    
    fileName = m_sFileName
    
    Exit Property
    
errP_Get_fileName:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
