VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "History"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' Module:      SQLStudioUtil - History Module
'
'-----------------------------------------------------------------------
'
' responsible:  WolfgangA
'
' special area: SQLStudio
'
' description:  module for history of sql statements
'
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option Explicit

' Module name
Const MODULE_NAME = "SQLStudioUtil.History"

Public Enum doubleEntryTypes

    deNoCheck = 1
    deStringCaseSensitiv = 2
    deStringGeneral = 3
End Enum

'=Private copy of properties============================
Private pDimension  As Long
Private pCount      As Long
'=======================================================

'=Other local variables============================
Private pArray()        As Variant
Private pGetIndex       As Long
Private pSetIndex       As Long

Private pCheckDoubleEntry   As doubleEntryTypes
'==================================================


'=========================================================================================
'    property:      Dimension
'    description:   Get the dimension of the history list
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Property Get dimension() As Long
    On Error GoTo Err_GET_Dimension
    
    Const FUNCTION_NAME = "P_Get dimension"
    
    dimension = pDimension
    
    Exit Property
    
Err_GET_Dimension:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      count
'    description:   Get the number of entries in the history list
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Property Get count() As Long
    On Error GoTo Err_GET_Count
    
    Const FUNCTION_NAME = "P_get count"
    
    count = pCount
    
    Exit Property
    
Err_GET_Count:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      checkDoubleEntry
'    description:   set if history class checks for double entries
'    arguments:     [in] NewCheckDoubleEntry, enum to be set
'
'                   deNoCheck = 1
'                   deStringCaseSensitiv = 2
'                   deStringGeneral = 3
'
'    return value:  /
'
'=========================================================================================
Public Property Let checkDoubleEntry(NewCheckDoubleEntry As doubleEntryTypes)
    On Error GoTo Err_LET_CheckDoubleEntry
    
    Const FUNCTION_NAME = "P_LET CheckDoubleEntry"
    
    pCheckDoubleEntry = NewCheckDoubleEntry
    
    Exit Property
    
Err_LET_CheckDoubleEntry:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    property:      checkDoubleEntry
'    description:   get if history class checks for double entries
'    arguments:
'
'    return value:  /
'
'=========================================================================================
Public Property Get checkDoubleEntry() As doubleEntryTypes
    On Error GoTo Err_GET_CheckDoubleEntry
    
    Const FUNCTION_NAME = "P_get checkDoubleEntry"
    
    checkDoubleEntry = pCheckDoubleEntry
    
    Exit Property
    
Err_GET_CheckDoubleEntry:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Property
'=========================================================================================
'    function:      Add
'    description:   add an entry to the history class
'    arguments:     [in] NewEntry
'
'    return value:  /
'
'=========================================================================================
Public Function add(NewEntry As Variant)
    On Error GoTo ErrAdd
    
    Const FUNCTION_NAME = "Add"
    
    '=local variables==============
    Dim lRun As Long
    '==============================
    
    If pCheckDoubleEntry <> deNoCheck Then
        
        For lRun = 0 To pSetIndex - 1
            
            If pCheckDoubleEntry = deStringCaseSensitiv Then
            
                If pArray(lRun) = NewEntry Then Exit Function
            End If
            If pCheckDoubleEntry = deStringGeneral Then
                
                If UCase(Trim$(pArray(lRun))) = UCase(Trim$(NewEntry)) Then Exit Function
            End If
        Next
    End If
    
    ' Add entry
    pArray(pSetIndex) = NewEntry
    
    ' Set indices
    pGetIndex = pSetIndex
    pSetIndex = pSetIndex + 1
    If pSetIndex > pDimension - 1 Then pSetIndex = 0
    
    ' Set count
    If pCount < dimension Then pCount = pCount + 1
    
    Exit Function
    
ErrAdd:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getPrevious
'    description:   get previous entry of the history class
'    arguments:
'
'    return value:  previous entry
'
'=========================================================================================
Public Function getPrevious() As Variant
    On Error GoTo errGetPrevious
    
    Const FUNCTION_NAME = "GetPrevious"
    
    ' Check index
    pGetIndex = pGetIndex - 1
    If pGetIndex < 0 Then
        pGetIndex = pCount - 1
    End If
    
    ' Get previous
    getPrevious = pArray(pGetIndex)
    
    Exit Function
    
errGetPrevious:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getNext
'    description:   get next entry of the history class
'    arguments:
'
'    return value:  next entry
'
'=========================================================================================
Public Function getNext() As Variant
    On Error GoTo errGetNext
    
    Const FUNCTION_NAME = "GetNext"
    
    ' Set and check index
    pGetIndex = pGetIndex + 1
    If pGetIndex > pCount - 1 Then pGetIndex = 0
    
    ' get next
    getNext = pArray(pGetIndex)
    
    Exit Function
    
errGetNext:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      createHistory
'    description:   create the history Object
'    arguments:     [in] Dimension,number of entries in the history class
'
'    return value:  true if success
'
'=========================================================================================
Friend Function createHistory(ByVal dimension As Long) As Boolean
    On Error GoTo errCreateHistory
    
    Const FUNCTION_NAME = "CreateHistory"
    
    ' Inits
    createHistory = False
    
    ' Check
    If dimension < 0 Then Exit Function
        
    ' Set properties
    pDimension = dimension
    pCount = 0
    pSetIndex = 0
    pGetIndex = 0
    
    pCheckDoubleEntry = deNoCheck
    
    ' Create array
    ReDim pArray(dimension)
    
    ' Set rc
    createHistory = True
    
    Exit Function
    
errCreateHistory:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function



