VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Key"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---

Option Explicit

Private oRegistry As registry
Private oParent As Object
Private bRoot As Boolean

Private lhKey As RegistryHKeyConstants
Private sName As String

Private oKeys As keys
Private oValues As values

Private lKey As Long

Private oDefault As Value
'
' Public Properties
'
Friend Property Get registry() As registry
    Set registry = oRegistry
End Property
Friend Property Set registry(ByVal newVal As registry)
    Set oRegistry = newVal
End Property

Friend Property Get parent() As Object
    Set parent = oParent
End Property
Friend Property Set parent(ByVal newVal As Object)
    Set oParent = newVal
End Property

Friend Property Get root() As Boolean
    root = bRoot
End Property
Friend Property Let root(ByVal newVal As Boolean)
    bRoot = newVal
End Property

Public Property Get hKey() As RegistryHKeyConstants
    hKey = lhKey
End Property
Friend Property Let hKey(ByVal newVal As RegistryHKeyConstants)
    lhKey = newVal
End Property

Public Property Get name() As String
    name = sName
End Property
Public Property Let name(ByVal newVal As String)
    sName = newVal
End Property

Public Property Get Path() As String
    Dim sParentPath As String
    If bRoot Then
        Path = sName
    Else
        sParentPath = oParent.Path
        If sParentPath <> "" Then
            Path = oParent.Path & "\" & sName
        Else
            Path = sName
        End If
    End If
End Property

Public Property Get keys() As keys
    If oKeys Is Nothing Then
        Set oKeys = New keys
        With oKeys
            Set .registry = oRegistry
            Set .parent = Me
            .root = False
        End With
    End If
    Set keys = oKeys
End Property

Public Property Get values() As values
    If oValues Is Nothing Then
        Set oValues = New values
        With oValues
            Set .registry = oRegistry
        End With
    End If
    Set values = oValues
End Property

Public Property Get Key() As Long
    Key = lKey
End Property

Public Property Get default() As Value
    Set default = Nothing
    
    If oDefault Is Nothing Then
        Set oDefault = New Value
        Set oDefault.registry = oRegistry
        oDefault.name = "" ' (Default)
    End If
    
    If openKey() Then
        If oDefault.queryValue(lKey) Then
            Set default = oDefault
        End If
        closeKey
    End If
End Property
'
' Public Sub/Function
'
Public Function openKey() As Boolean
    openKey = oRegistry.openKey(hKey, Path, KEY_ALL_ACCESS, lKey)
End Function

Public Function closeKey() As Boolean
    closeKey = oRegistry.closeKey(Path, lKey)
End Function

Public Function check(ByVal WithSubKeys As Boolean, _
                      ByVal WithValues As Boolean) As Boolean
                      
    Dim bShowError As Boolean
    Dim bRaiseError As Boolean
    
    Dim oKey As Key
    Dim oValue As Value
        
    check = False
    
    bShowError = oRegistry.showErrors
    bRaiseError = oRegistry.raiseErrors
    oRegistry.showErrors = False
    oRegistry.raiseErrors = False
    
    If openKey() Then
        If WithValues Then
            For Each oValue In values
                If Not oValue.checkValue(lKey) Then
                    closeKey
                    GoTo Check_Exit
                End If
            Next
        End If

        If Not closeKey() Then Exit Function

        If WithSubKeys Then
            For Each oKey In keys
                If Not oKey.check(WithSubKeys, WithValues) Then
                    GoTo Check_Exit
                End If
            Next
        End If
    Else
        GoTo Check_Exit
    End If
    
    check = True
    
Check_Exit:
    oRegistry.showErrors = bShowError
    oRegistry.raiseErrors = bRaiseError
End Function

Public Function create(ByVal WithSubKeys As Boolean, _
                       ByVal WithValues As Boolean) As Boolean
    
    Dim lDisposition As Long
    Dim oKey As Key
    Dim oValue As Value
    
    create = False
    
    If oRegistry.createKey(hKey, Path, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, lKey, lDisposition) Then
        If WithValues Then
            For Each oValue In values
                If Not oValue.createValue(lKey) Then
                    If Not closeKey() Then Exit Function
                    Exit Function
                End If
            Next
        End If
        
        If Not closeKey() Then Exit Function
        
        If WithSubKeys Then
            For Each oKey In keys
                If Not oKey.create(WithSubKeys, WithValues) Then
                    Exit Function
                End If
            Next
        End If
    Else
        Exit Function
    End If

    create = True
End Function

Public Function queryValues(ByVal WithSubKeys As Boolean) As Boolean
                       
    Dim oKey As Key
    Dim oValue As Value
    
    queryValues = False
    
    If openKey() Then
        For Each oValue In values
            If Not oValue.queryValue(lKey) Then
                Exit Function
            End If
        Next
        
        If Not closeKey() Then Exit Function
    
        If WithSubKeys Then
            For Each oKey In keys
                If Not oKey.queryValues(WithSubKeys) Then
                    Exit Function
                End If
            Next
        End If
    Else
        Exit Function
    End If
    
    queryValues = True
End Function

Public Function setValues(ByVal WithSubKeys As Boolean) As Boolean
                       
    Dim oKey As Key
    Dim oValue As Value
    
    setValues = False

    If openKey() Then
        For Each oValue In values
            If Not oValue.setValue(lKey) Then
                Exit Function
            End If
        Next
        
        If Not closeKey() Then Exit Function
        
        If WithSubKeys Then
            For Each oKey In keys
                If Not oKey.setValues(WithSubKeys) Then
                    Exit Function
                End If
            Next
        End If
    Else
        Exit Function
    End If
    
    setValues = True
End Function

Public Function deleteValues(ByVal WithSubKeys As Boolean) As Boolean
    Dim oKey As Key
    Dim oValue As Value
    
    deleteValues = False

    If openKey() Then
        For Each oValue In values
            If Not oValue.deleteValue(lKey) Then
                Exit Function
            End If
        Next
        
        If Not closeKey() Then Exit Function
        
        If WithSubKeys Then
            For Each oKey In keys
                If Not oKey.deleteValues(WithSubKeys) Then
                    Exit Function
                End If
            Next
        End If
    Else
        Exit Function
    End If
    
    deleteValues = True
End Function

Public Function enumKeys(ByVal WithSubKeys As Boolean, _
                         ByVal WithValues As Boolean) As Boolean
                         
    Dim lRC As Long
    Dim lIndex As Long
    Dim sName As String
    Dim lName As Long
    Dim ftKey As FILETIME
    Dim oKey As Key
    
    Dim bName()  As Byte
    Dim bClass() As Byte
        
    enumKeys = False
    
    If WithValues Then
        If Not enumValues() Then
            Exit Function
        End If
    End If

    If openKey() Then
        lIndex = 0
        Do
            ReDim bName(256 * 2)
            bClass = "" + vbNullChar
            lName = 256
            
            lRC = regEnumKeyExW(lKey, lIndex, bName(0), lName, 0&, bClass(0), 0&, ftKey)
            sName = Left$(bName, lName)
            Select Case lRC
                Case ERROR_MORE_DATA:
                    
                    If Not keys.exists(sName) Then
                        Set oKey = keys.add(hKey, sName)
                    Else
                        Set oKey = keys.item(sName)
                    End If
                    
                    If WithSubKeys Then
                        If Not oKey.enumKeys(WithSubKeys, WithValues) Then
                            Exit Function
                        End If
                    End If
                    
                    lIndex = lIndex + 1
                Case ERROR_NO_MORE_ITEMS:
                    Exit Do
                Case Else:
                    If Not closeKey() Then Exit Function
                    Exit Function
            End Select
        Loop
        
        If Not closeKey() Then Exit Function
    
    Else
        Exit Function
    End If
    
    enumKeys = True
End Function

Public Function enumValues() As Boolean

    Dim lRC As Long
    Dim lIndex As Long
    Dim sName As String
    Dim lName As Long
    Dim lType As Long
    Dim bValue As ByteValue
    Dim lValue As Long
    Dim sValue As String
    Dim lSize As Long
    Dim oValue As Value
    
    Dim bName()  As Byte
    
    enumValues = False

    If openKey() Then ' Access = KEY_QUERY_VALUE
        lIndex = 0
        Do
            ReDim bName(256 * 2)
            lName = 256
            
            lSize = Len(bValue)
            lRC = regEnumValueW(lKey, lIndex, bName(0), lName, 0&, lType, bValue.b(0), lSize)
            sName = Left$(bName, lName)
            Select Case lRC
                Case ERROR_SUCCESS:
                    Select Case lType
                        Case REG_SZ:
                            sValue = bytesToString(bValue)
                            If Not values.exists(sName) Then
                                Set oValue = values.add(sName, sValue, REG_SZ)
                            Else
                                Set oValue = values.item(sName)
                                With oValue
                                    .Value = sValue
                                    .ValueType = REG_SZ
                                End With
                            End If
                        Case REG_DWORD:
                            lValue = bytesToLong(bValue)
                            If Not values.exists(sName) Then
                                Set oValue = values.add(sName, lValue, REG_DWORD)
                            Else
                                Set oValue = values.item(sName)
                                With oValue
                                    .Value = lValue
                                    .ValueType = REG_DWORD
                                End With
                            End If
                    End Select
                    lIndex = lIndex + 1
                Case ERROR_NO_MORE_ITEMS:
                    Exit Do
                Case Else:
                    If Not closeKey() Then Exit Function
                    Exit Function
            End Select
        Loop
        
        If Not closeKey() Then Exit Function
    
    Else
        Exit Function
    End If
    
    enumValues = True
End Function

Public Function rename(ByVal NewName As String) As Boolean
    
    rename = False
    
    If delete() Then
        name = NewName
        create True, True
    Else
        Exit Function
    End If

    rename = True
End Function

Public Function delete() As Boolean

    Dim oKey As Key
    
    delete = False
    
    For Each oKey In keys
        If Not oKey.delete() Then
            Exit Function
        End If
    Next
    
    If Not oRegistry.deleteKey(hKey, Path) Then
        Exit Function
    End If

    delete = True
End Function

Public Function findKeyByPath(ByVal WithSubKeys As Boolean, _
                              ByVal FindPath As String) As Key
    Dim oKey As Key
    
    Set findKeyByPath = Nothing
    
    If FindPath = Path Then
        Set findKeyByPath = Me
        Exit Function
    End If
    
    For Each oKey In keys
        If FindPath = oKey.Path Then
            Set findKeyByPath = oKey
            Exit Function
        End If
    Next

    For Each oKey In keys
        If WithSubKeys Then
            Set findKeyByPath = oKey.findKeyByPath(WithSubKeys, FindPath)
            If Not findKeyByPath Is Nothing Then
                Exit Function
            End If
        End If
    Next
End Function
'
' Private Sub/Function
'
Private Sub Class_Initialize()
    'Debug.Print "INIT Key"
    
    Set oRegistry = Nothing
    Set oParent = Nothing
    bRoot = False
    
    lhKey = HKEY_CLASSES_ROOT
    sName = ""
    
    Set oKeys = Nothing
    Set oValues = Nothing
    
    lKey = 0
    
    Set oDefault = Nothing
End Sub



