VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Registry"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Public Enum registryHKeyConstants

    HKEY_CLASSES_ROOT = &H80000000
    HKEY_CURRENT_USER = &H80000001
    HKEY_LOCAL_MACHINE = &H80000002
    HKEY_USERS = &H80000003
    HKEY_PERFORMANCE_DATA = &H80000004
    HKEY_CURRENT_CONFIG = &H80000005
    HKEY_DYN_DATA = &H80000006
End Enum

Public Enum registryTypeConstants
    REG_NONE = (0)                         'No value type
    REG_SZ = (1)                           'Unicode nul terminated string
'    REG_EXPAND_SZ = (2)                    'Unicode nul terminated string w/enviornment var
'    REG_BINARY = (3)                       'Free form binary
    REG_DWORD = (4)                        '32-bit number
'    REG_DWORD_LITTLE_ENDIAN = (4)          '32-bit number (same as REG_DWORD)
'    REG_DWORD_BIG_ENDIAN = (5)             '32-bit number
'    REG_LINK = (6)                         'Symbolic Link (unicode)
'    REG_MULTI_SZ = (7)                     'Multiple Unicode strings
'    REG_RESOURCE_LIST = (8)                'Resource list in the resource map
'    REG_FULL_RESOURCE_DESCRIPTOR = (9)     'Resource list in the hardware description
'    REG_RESOURCE_REQUIREMENTS_LIST = (10)
End Enum

Public Enum registryAccessConstants
    KEY_QUERY_VALUE = &H1
    KEY_SET_VALUE = &H2
    KEY_CREATE_SUB_KEY = &H4
    KEY_ENUMERATE_SUB_KEYS = &H8
    KEY_NOTIFY = &H10
    KEY_CREATE_LINK = &H20
    KEY_ALL_ACCESS = &H3F
End Enum

Public Enum registryErrorConstants
    ERROR_SUCCESS = 0
    ERROR_BADKEY = 2
    ERROR_OUTOFMEMORY = 6
    ERROR_MORE_DATA = 234
    ERROR_NO_MORE_ITEMS = 259
End Enum

Public Enum registryVolatileConstants
    REG_OPTION_NON_VOLATILE = 0&
    REG_OPTION_VOLATILE = &H1
End Enum

Public Enum registryDispositionConstants
    REG_CREATED_NEW_KEY = &H1
    REG_OPENED_EXISTING_KEY = &H2
End Enum

Private oKeys As keys

Private bShowErrors As Boolean
Private bRaiseErrors As Boolean

Private m_oComFunc                 As New SQLStudioUtil.Functions


'for windows 95 and 98 use A Functions
Private m_bUseAFunctions           As Boolean
'
' Public Properties
'
Public Property Get keys() As keys
    If oKeys Is Nothing Then
        Set oKeys = New keys
        With oKeys
            Set .registry = Me
            Set .parent = Me
            .root = True
        End With
    End If
    Set keys = oKeys
End Property

Public Property Get showErrors() As Boolean
    showErrors = bShowErrors
End Property
Public Property Let showErrors(ByVal newVal As Boolean)
    bShowErrors = newVal
End Property

Public Property Get raiseErrors() As Boolean
    raiseErrors = bRaiseErrors
End Property
Public Property Let raiseErrors(ByVal newVal As Boolean)
    bRaiseErrors = newVal
End Property
'
' Public Sub/Function
'
' Base Functions
'
Public Function openKey(ByVal hKey As registryHKeyConstants, _
                        ByVal Path As String, _
                        ByVal Access As registryAccessConstants, _
                        Key As Long) As Boolean
                        
    Dim lRC     As Long
    Dim bPath() As Byte
    
    openKey = False
    
    If m_bUseAFunctions Then
        
        lRC = RegOpenKeyExA(hKey, Path, 0&, Access, Key)
    Else
        bPath = Path + vbNullChar
        lRC = RegOpenKeyExW(hKey, bPath(0), 0&, Access, Key)
    End If
    
    If lRC = ERROR_SUCCESS Then
        openKey = True
    Else
        handleError lRC, Path
    End If
    
End Function

Public Function createKey(ByVal hKey As registryHKeyConstants, _
                          ByVal Path As String, _
                          ByVal Volatile As registryVolatileConstants, _
                          ByVal Access As registryAccessConstants, _
                          Key As Long, _
                          Disposition As Long) As Boolean

    Dim lRC As Long
    Dim saKey As SECURITY_ATTRIBUTES
    Dim bPath() As Byte
    Dim bClass() As Byte
    
    createKey = False
    
    If m_bUseAFunctions Then
    
        lRC = RegCreateKeyExA(hKey, Path, 0, "", Volatile, Access, saKey, Key, Disposition)
    Else
    
        bPath = Path + vbNullChar
        bClass = "" + vbNullChar
        
        lRC = RegCreateKeyExW(hKey, bPath(0), 0, bClass(0), Volatile, Access, saKey, Key, Disposition)
    End If
    
    If lRC = ERROR_SUCCESS Then
        createKey = True
    Else
        handleError lRC, Path
    End If
    
End Function

Public Function deleteKey(ByVal hKey As registryHKeyConstants, _
                          ByVal Path As String) As Boolean

    Dim lRC     As Long
    Dim bPath() As Byte
        
    deleteKey = False
    
    If m_bUseAFunctions Then
    
        lRC = RegDeleteKeyA(hKey, Path)
    Else
        bPath = Path + vbNullChar
        lRC = RegDeleteKeyW(hKey, bPath(0))
    End If
    
    If lRC = ERROR_SUCCESS Then
        deleteKey = True
    Else
        handleError lRC, Path
    End If
    
End Function

Public Function closeKey(ByVal Path, Key As Long) As Boolean

    Dim lRC As Long
        
    closeKey = False
    
    lRC = RegCloseKey(Key)
    If lRC = ERROR_SUCCESS Then
        Key = 0
        closeKey = True
    Else
        handleError lRC, Path
    End If
    
End Function
Public Function queryValueNull(ByVal hKey As Long, _
                               ByVal name As String, _
                               ValueType As registryTypeConstants, _
                               ValueLen As Long) As Boolean
    Dim lRC          As Long
    Dim bName()      As Byte
    
    'rc
    queryValueNull = False
    
    If m_bUseAFunctions Then
    
        lRC = RegQueryValueExNullA(hKey, name, 0&, ValueType, 0&, ValueLen)
    Else
        bName = name + vbNullChar
        lRC = RegQueryValueExNullW(hKey, bName(0), 0&, ValueType, 0&, ValueLen)
    End If
    
    If lRC = ERROR_SUCCESS Then
        queryValueNull = True
    Else
        handleError lRC, name
    End If
    
End Function

Public Function queryValueString(ByVal hKey As Long, _
                                 ByVal name As String, _
                                 Value As String, _
                                 ValueLen As Long) As Boolean
    
    Dim lRC          As Long
    Dim bName()      As Byte
    Dim bValue()     As Byte

    queryValueString = False
    
    If m_bUseAFunctions Then
    
        Value = String(ValueLen, 0)
        lRC = RegQueryValueExStringA(hKey, name, 0&, REG_SZ, Value, ValueLen)
    Else
        bName = name + vbNullChar
        ReDim bValue(ValueLen * 2)
        
        lRC = RegQueryValueExStringW(hKey, bName(0), 0&, REG_SZ, bValue(0), ValueLen)
    End If
    
    If lRC = ERROR_SUCCESS Then
    
        If m_bUseAFunctions Then
        
            If ValueLen > 1 Then Value = Left(Value, ValueLen - 1)
        Else
            If ValueLen > 1 Then Value = Left(bValue, (ValueLen \ 2) - 1)
        End If
        queryValueString = True
    Else
        handleError lRC, name
    End If
    
End Function

Public Function queryValueLong(ByVal hKey As Long, _
                               ByVal name As String, _
                               Value As Long) As Boolean
    
    Dim lRC       As Long
    Dim bName()   As Byte
    Dim lValueLen As Long
    
    'rc
    queryValueLong = False
    
    'init
    Value = 0
    If m_bUseAFunctions Then
    
        lRC = RegQueryValueExLongA(hKey, name, 0&, REG_DWORD, Value, 4)
    Else
        bName = name + vbNullChar
        lRC = RegQueryValueExLongW(hKey, bName(0), 0&, REG_DWORD, Value, 4)
    End If
    
    If lRC = ERROR_SUCCESS Then
        queryValueLong = True
    Else
        handleError lRC, name
    End If
End Function

Public Function setValueString(ByVal hKey As Long, _
                               ByVal name As String, _
                               ByVal Value As String) As Boolean

    Dim lRC       As Long
    Dim bName()   As Byte
    Dim bValue()  As Byte
    
    setValueString = False
    
    'cbData [in] Specifies the size,
    'in bytes, of the information pointed to by the lpData parameter.
    'If the data is of type REG_SZ, REG_EXPAND_SZ, or REG_MULTI_SZ,
    'cbData must include the size of the terminating null character.
    
    If m_bUseAFunctions Then
    
        Value = Value & Chr(0)
        lRC = RegSetValueExStringA(hKey, name, 0&, REG_SZ, Value, Len(Value))
    Else
        bValue = Value + vbNullChar
        bName = name + vbNullChar
        lRC = RegSetValueExStringW(hKey, bName(0), 0&, REG_SZ, bValue(0), 2 * Len(Value) + 2)
    End If
    
    If lRC = ERROR_SUCCESS Then
        setValueString = True
    Else
        handleError lRC, name
    End If
End Function

Public Function setValueLong(ByVal hKey As Long, _
                             ByVal name As String, _
                             ByVal Value As Long) As Boolean

    Dim lRC       As Long
    Dim bName()   As Byte
            
    setValueLong = False
    
    If m_bUseAFunctions Then
    
        lRC = RegSetValueExLongA(hKey, name, 0&, REG_DWORD, Value, 4)
    Else
        bName = name + vbNullChar
        lRC = RegSetValueExLongW(hKey, bName(0), 0&, REG_DWORD, Value, 4)
    End If
    
    If lRC = ERROR_SUCCESS Then
        setValueLong = True
    Else
        handleError lRC, name
    End If
    
End Function

Public Function deleteValue(ByVal hKey As Long, _
                            ByVal name As String) As Boolean

    Dim lRC       As Long
    Dim bName()   As Byte

    deleteValue = False
    
    If m_bUseAFunctions Then
    
        lRC = RegDeleteValueA(hKey, name)
    Else
        bName = name + vbNullChar
        lRC = RegDeleteValueW(hKey, bName(0))
    End If
    
    If lRC = ERROR_SUCCESS Then
        deleteValue = True
    Else
        handleError lRC, name
    End If
    
End Function
'
'
'
Public Function check(ByVal WithSubKeys As Boolean, _
                      ByVal WithValues As Boolean) As Boolean
                      
    Dim oKey As Key
    
    check = False
    
    For Each oKey In keys
        If Not oKey.check(WithSubKeys, WithValues) Then
            Exit Function
        End If
    Next
        
    check = True
End Function

Public Function create(ByVal WithSubKeys As Boolean, _
                       ByVal WithValues As Boolean) As Boolean

    Dim oKey As Key
    
    create = False
    
    For Each oKey In keys
        If Not oKey.create(WithSubKeys, WithValues) Then
            Exit Function
        End If
    Next
        
    create = True
End Function

Public Function queryValues(ByVal WithSubKeys As Boolean) As Boolean
                       
    Dim oKey As Key
    
    queryValues = False
    
    For Each oKey In keys
        If Not oKey.queryValues(WithSubKeys) Then
            Exit Function
        End If
    Next
        
    queryValues = True
End Function

Public Function setValues(ByVal WithSubKeys As Boolean) As Boolean
                       
    Dim oKey As Key
    
    setValues = False
    
    For Each oKey In keys
        If Not oKey.setValues(WithSubKeys) Then
            Exit Function
        End If
    Next
        
    setValues = True
End Function

Public Function enumKeys(ByVal WithSubKeys As Boolean, _
                         ByVal WithValues As Boolean) As Boolean

    Dim oKey As Key
    
    enumKeys = False
    
    For Each oKey In keys
        If Not oKey.enumKeys(WithSubKeys, WithValues) Then
            Exit Function
        End If
    Next
        
    enumKeys = True
End Function

Public Function findKeyByPath(ByVal WithSubKeys As Boolean, _
                              ByVal FindPath As String) As Key
    Dim oKey As Key
    
    Set findKeyByPath = Nothing
    
    For Each oKey In keys
        If FindPath = oKey.Path Then
            Set findKeyByPath = oKey
            Exit Function
        End If
        If WithSubKeys Then
            Set findKeyByPath = oKey.findKeyByPath(WithSubKeys, FindPath)
        End If
    Next
End Function

Friend Function handleError(ByVal RC As Long, ByVal Text As String)
    Dim sMsg As String
    
    If bShowErrors Then
        sMsg = "Error: " & errorText(RC) & ". " & Text
        MsgBox sMsg, vbExclamation
    End If
End Function
'
' Private Sub/Function
'
Private Sub Class_Initialize()
    'Debug.Print "INIT Registry"
    Set oKeys = Nothing
    bShowErrors = True
    bRaiseErrors = False
    
    'use a functions for win95/98
    If m_oComFunc.getPlatformID() = Win95_98 Then
    
        m_bUseAFunctions = True
    Else
        m_bUseAFunctions = False
    End If
End Sub
Private Function errorText(ByVal lRC As Long) As String
    Dim s As String
    Select Case lRC
        Case ERROR_BADKEY:          s = "Bad key"
        Case ERROR_MORE_DATA:       s = "More data"
        Case ERROR_OUTOFMEMORY:     s = "Out of memory"
        Case ERROR_NO_MORE_ITEMS:   s = "No more items"
        Case Else:                  s = "RC=" & CStr(lRC)
    End Select
    errorText = s
End Function

