/* --- GPL ---

 Copyright (C) 2000-2001 SAP AG

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 --- GPL ---
*/
/*!**********************************************************************

  module: SQLStudioInput.cpp

  -----------------------------------------------------------------------

  responsible:  WolfgangA

  special area: SQLStudioDSQL

  description:  implements CSQLStudioInput

  see also:

  -----------------------------------------------------------------------

  copyright:                Copyright by SAP AG, 2000

************************************************************************/

#include "stdafx.h"
#include "SQLStudioDSQL.h"
#include "SQLStudioInput.h"

#include <limits.h>
#include <mbstring.h>
#include <process.h>
#include <windowsx.h>		
#include <shlobj.h>


// #include <stdlib.h> would be need for variable MB_CUR_MAX giving the number of bytes per character of the current codepage
// However this works not properly as found out on a japanese PC.
// Thus the current windows system setting(two bytes for multibyte) is used.
#define BYTES_MBS   2   // MB_CUR_MAX

// needed for InternalIndent() and InternalUnindent()
#define DEFAULT_INDENT_WIDTH 4

// Timer IDs + defines for time to ellapse
// Although the WIN32 documentation says the ID can be any
// nonzero values an ID 1 does not work. So ID 1000 is used
const int ID_TIMER_HIGHLIGHT_COMMENTS	= 1000;
#define      TIME_HIGHLIGHT_COMMENTS 200
const int ID_TIMER_ON_CREATE            = 1001;
#define      TIME_ON_CREATE          1000
const int ID_TIMER_UPDATE_TOOLBAR       = 1002;
#define      TIME_UPDATE_TOOLBAR     150
const int ID_TIMER_SORT_RANGES          = 1003;
#define      TIME_SORT_RANGES        2000


// Other constants
const long SET_FOCUS_TIME	= 400;


#define DIM(a)(sizeof(a)/sizeof(a[0]))


// to store the ShellIDList ID from RegisterClipboardFormat().
typedef enum tagKEYMODIFIERS
  {
     KEYMOD_NONE    = 0X00000000,
     KEYMOD_SHIFT   = 0x00000001,
     KEYMOD_CONTROL = 0x00000002,
     KEYMOD_ALT     = 0x00000004
} KEYMODIFIERS;

static const LPCOLESTR   rszKeyWordChangeCase[] = {
							 OLESTR("None"),
							 OLESTR("LowerCase"),
							 OLESTR("UpperCase"),
							 OLESTR("TitleCase"),
							 OLESTR("SentenceCase"),
							 OLESTR("ToggleCase")
							};

static const DWORD        rdwKeyWordChangeCase [] = { 0, 1,2,3,4,5};





static const LPCOLESTR   rszOLEDropMode[] = {
							 OLESTR("OLEDropNone"),
							 OLESTR("OLEDropManual"),
							 OLESTR("OLEDropAutomatic")
							};

static const DWORD        rdwOLEDropMode [] = { 0,1,2};


#define vbShiftMask 1 //SHIFT key is pressed. 
#define vbCtrlMask  2 //CTRL key is pressed. 
#define vbAltMask   4 //ALT key is pressed. 

#define	vbLeftButton   1  //Left button is pressed. 
#define vbRightButton  2  //Right button is pressed. 
#define vbMiddleButton 4  //


#define GetPIDLFolder(pida)(LPCITEMIDLIST)(((LPBYTE)pida)+(pida)->aoffset[0])
#define GetPIDLItem(pida, i)(LPCITEMIDLIST)(((LPBYTE)pida)+(pida)->aoffset[i+1])

static const FONTDESC _fontdesc =
  { sizeof(FONTDESC), OLESTR("Courier New"), FONTSIZE(10), FW_NORMAL, 
     ANSI_CHARSET,FALSE, FALSE, FALSE };


////////////////////////////////////////////////////////////////////////////////////////////
// additional methods dealing with CComBstr
////////////////////////////////////////////////////////////////////////////////////////////

bool HasLineWhiteSpacesOnly(const CComBSTR & bstr)
{
	OLECHAR * strLocal = bstr;
	size_t nNumberWhiteChars = 0;

	while(iswspace(*strLocal))
	{
		strLocal++;
		nNumberWhiteChars++;
	}
	
	if(nNumberWhiteChars == bstr.Length())
		return true;
	else
		return false;
}

long DeleteLeadingWhiteSpaces(CComBSTR & bstr, long nMaxNumberToDelete = -1, bool bBreakOnCRLF = false)
{
	OLECHAR * strLocal = bstr;
	size_t nLength = bstr.Length();
	size_t nDeletedSpaces = 0;

	while(	iswspace(*strLocal)) 
	{
			if(bBreakOnCRLF == true)
			{
				if(*strLocal == L'\r')
				{
					OLECHAR * strOriginal = strLocal;
					strLocal++;
					if(*strLocal == L'\n')
					{
						break;
					}
					else
					{
						// reset position: decrement
						strLocal = strOriginal;
					}
				}
			}

			if((nDeletedSpaces <(size_t) nMaxNumberToDelete) ||(nDeletedSpaces == -1))
			{
				strLocal++;
				nDeletedSpaces++;
			}
			else
				break;			
	}
	
	memmove(bstr.m_str, strLocal,(nLength-nDeletedSpaces+1)*sizeof(OLECHAR));
			
	return nDeletedSpaces;
}

long DeleteLeadingCharacters(CComBSTR & bstr, long nToDelete)
{
	OLECHAR * strLocal = bstr;
	size_t nLength = bstr.Length();
	long nDeleted = 0;

	while(*strLocal != NULL && nDeleted < nToDelete) 
	{
		strLocal++;
		nDeleted++;
	}
	
	memmove(bstr.m_str, strLocal,(nLength-nDeleted+1)*sizeof(OLECHAR));
			
	return nDeleted;
}

long NumberLeadingWhiteSpaces(const CComBSTR & bstr)
{
	OLECHAR * strLocal = bstr;
	size_t nLeadingSpaces = 0;

	while(	iswspace(*strLocal)) 
	{
		strLocal++;
		nLeadingSpaces ++;
	}
			
	return nLeadingSpaces;
}

long NumberTrailingWhiteSpaces(const CComBSTR & bstr)
{
	OLECHAR * strLocal = bstr;
	size_t nTrailingSpaces = 0;

	strLocal += bstr.Length();
	while(	iswspace(*strLocal)){

		strLocal--;
		nTrailingSpaces ++;
	}
			
	return nTrailingSpaces;
}

long RemoveWhiteSpaces(ITextRangePtr pITextRange) 
{

	long lStart = 0;
	long lEnd   = 0;
	long lNewStart = 0;
	long lOldEnd = 0;

	BSTR bstrBuffer;
	pITextRange->GetText(&bstrBuffer);
	CComBSTR  bstrText = bstrBuffer;

	pITextRange->GetStart(&lNewStart);
	pITextRange->GetEnd(&lOldEnd);

	for(long lRun = 0;bstrText.Length()-1;lRun++){

		if(iswspace(bstrText.m_str[lRun])) {
			lStart ++;
		}
		else
			break;		
	}

	for(lRun = bstrText.Length()-1;lRun > 1;lRun--){

		if(iswspace(bstrText.m_str[lRun])) {
			lEnd ++;
		}
		else
			break;		
	}
	
	if(lStart > 0)
			pITextRange->SetStart(lNewStart+lStart);	

	if(lEnd > 0)
			pITextRange->SetEnd(lOldEnd-lEnd);
	

	pITextRange->GetStart(&lNewStart);
	pITextRange->GetEnd(&lOldEnd);


	return(lOldEnd - lNewStart);

}

/**************/
/* SetStrings */
/**************/

// Copy array of strings to counted array of strings
HRESULT SetStrings(/*[in*/ ULONG cElems, /*[in]*/ const LPCOLESTR* rgpsz, /*[out]*/ CALPOLESTR* pStrings)
{
    ATLASSERT(0 != cElems);
    ATLASSERT(NULL != rgpsz);
    ATLASSERT(NULL != pStrings);

    pStrings->cElems = 0;

    // Allocate array of string pointers
    ULONG len = cElems * sizeof(LPOLESTR);
    pStrings->pElems =(LPOLESTR*) CoTaskMemAlloc(len);
    if(NULL == pStrings->pElems) {
        return E_OUTOFMEMORY;
    }

#ifdef _DEBUG
    // Initialize the array to zero
    ::ZeroMemory(pStrings->pElems, len);
#endif

    // Allocate each string
    for(ULONG i = 0; i < cElems; i++) {
        // Allocate memory for the string and terminating NUL character
        ULONG len =(ocslen(rgpsz[i]) + 1) * sizeof(OLECHAR);
        pStrings->pElems[i] =(LPOLESTR) CoTaskMemAlloc(len);

        if(NULL == pStrings->pElems[i]) return E_OUTOFMEMORY;
        ocscpy(pStrings->pElems[i], rgpsz[i]);
        pStrings->cElems++;
    }
    return S_OK;
}

/**************/
/* SetCookies */
/**************/

// Set counted array of DWORDs to provided DWORD cookies
HRESULT SetCookies(/*[in*/ ULONG cElems, /*[in*/ const DWORD* pdwCookies, /*[out]*/ CADWORD* pcaCookies)
{
    ATLASSERT(0 != cElems);
    ATLASSERT(NULL != pcaCookies);

    pcaCookies->cElems = 0;

    // Allocate array of cookies
    ULONG len = cElems * sizeof(DWORD);
    pcaCookies->pElems =(LPDWORD) CoTaskMemAlloc(len);
    if(NULL == pcaCookies->pElems) {
        return E_OUTOFMEMORY;
    }

    pcaCookies->cElems = cElems;
    ::CopyMemory(pcaCookies->pElems, pdwCookies, cElems * sizeof(DWORD));

    return S_OK;
}



/////////////////////////////////////////////////////////////////////////////
// CSQLStudioInput
/////////////////////////////////////////////////////////////////////////////
CSQLStudioInput::CSQLStudioInput():
	m_RichEditCtrl					(RICHEDIT_CLASS, this, 1),
	m_StatusBar						(NULL),
	m_ToolBar						(NULL),
	m_WndFindReplace				(NULL),
	// find/replace struct and thread handle
	m_dwThreadID					(0),
	m_hThread						(NULL),
	m_bFindReplaceInSelection      (false),
	// richedit specific
	m_nLimitText					(0x1000000),
	m_nENMFlags						(ENM_UPDATE|ENM_CHANGE|ENM_DROPFILES|ENM_KEYEVENTS|ENM_MOUSEEVENTS|ENM_SCROLL|ENM_SCROLLEVENTS|ENM_SELCHANGE),
	m_hDCRichEdit					(NULL),
	// colors
	m_crTextEditable				(0),
	m_crBackgroundEditable			(RGB(255,255,255)),
	m_crBackgroundReadonly			(14737632),
	m_crTextHighlight				(RGB(0,0,255)),
	m_nStatusBarPartExternal		(0),
	m_nStatusBarPartModified		(1),
	m_nStatusBarPartInsOvr			(2),
	m_nStatusBarPartSelectionPos	(3),
	m_nStatusBarPartVisibleLines	(4),
	// public
	m_bReadonly						(VARIANT_FALSE),	
	m_bStatusBarVisible				(VARIANT_TRUE),
	m_bToolBarVisible				(VARIANT_TRUE),
	m_bLocalContextMenuVisible		(VARIANT_TRUE),	
	m_bNavigateOnDblclick			(VARIANT_FALSE),
	m_bHighlightKeyAndCom			(VARIANT_FALSE),
	m_strCommentPrefix("//"),
	m_strCommentPrefix2("--"),
	m_bWordWrap						(VARIANT_TRUE),
	m_bConvertWordWrapToLineBreak	(VARIANT_FALSE),
	m_bAutoIndent                  (VARIANT_FALSE),
	m_nSpacesOnIndent				(2),
	m_OLEDropMode				   (OLEDropAutomatic),
	// helper classes
	m_nUpdateCount					(0),
	m_nSuspendUndoCount				(0),
	m_nCollectUndoCallsCount		(0),
	m_bUpdateOfFindReplaceRange    (true),
	m_nUpdateOfFindReplaceRangeCount(0),
	m_bIgnoreScrollEvents			(0),
	m_nWaitCursorCount				(0),
	// others
	m_bInsertModus					(true),
	m_bTextModifiedStatus			(false),
	m_sVirtualKeyCodeBigger			(LOBYTE(::VkKeyScanEx(_T('>'), ::GetKeyboardLayout(0)))),
	m_sVirtualKeyCodeLess			(LOBYTE(::VkKeyScanEx(_T('<'), ::GetKeyboardLayout(0)))),
	m_bInsideHighlightKeyAndCom		(false),
	m_bUpdateStatusBar             (true),  // initially needed for InternalIndent to avoid update of statusbar in UpdateStatusBar()
	m_hCursorWait					(::LoadCursor(NULL, IDC_WAIT)),	
	m_hCursorBeam					(::LoadCursor(NULL, IDC_IBEAM)),	
	m_TBImageList					(NULL),
	m_bCaseSensitive				(VARIANT_FALSE),
	m_bstrText						(NULL),
	m_hLibHandle					(NULL),
	m_KeyWordChangeCase				(None)
{
	
	// Because of a bug in richedit 2.0 OleUninitialize should not
	// be called. If this is done, the program hangs when it is
	// ended.
	::OleInitialize(NULL);

	// Initialize texts
	m_bstrFindWhat.Empty();
	m_bstrReplaceWith.Empty();
	m_strFindWhat[0] = 0;
	m_strReplaceWith[0] = 0;
	FillDisplayMapFromResource();


	//Empty Text Container
	m_bstrText.Empty();

	// Initialize structs
	ZeroMemory(&m_oFINDREPLACE, sizeof(FINDREPLACE));

	// The control always has a window, even if the container
	// supports windowless controls.
	// This is necessary, because the controls does not just
	// painting, but also creates windows(RichEdit, toolbar, statusbar) itself.
	m_bWindowOnly = TRUE; 

	// Load the RichEdit library version 2.0	
	m_hLibHandle = ::LoadLibrary(_T("RICHED20.DLL"));

	// Initialize the common controls DLL(toolbar, statusbar)
	::InitCommonControls();
	
	
	// Initialize scrollinfo objects
	ZeroMemory(&m_oSCROLLINFOHorz, sizeof(SCROLLINFO));
	m_oSCROLLINFOHorz.cbSize = sizeof(SCROLLINFO);
	m_oSCROLLINFOHorz.fMask = SIF_ALL;

	ZeroMemory(&m_oSCROLLINFOVert, sizeof(SCROLLINFO));
	m_oSCROLLINFOVert.cbSize = sizeof(SCROLLINFO);
	m_oSCROLLINFOVert.fMask = SIF_ALL;
	
}


CSQLStudioInput::~CSQLStudioInput()
{

	if(m_hLibHandle != NULL){

		BOOL bRet = ::FreeLibrary(m_hLibHandle);
		m_hLibHandle = NULL;
	}	
}



LRESULT CSQLStudioInput::OnSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{		
	ArrangeWindows();
	SetTimer(ID_TIMER_HIGHLIGHT_COMMENTS, TIME_HIGHLIGHT_COMMENTS);

	return 0;
}

LRESULT CSQLStudioInput::OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	// Create statusbar
	CreateStatusBar();

	// Create toolbar
	CreateToolBar();

	// Create RichEdit control
	CreateRichEditWindow();
	
	// put the properties again because now the window is created
	put_ReadOnly(m_bReadonly);

	put_StatusBarVisible(m_bStatusBarVisible);
	put_ToolBarVisible(m_bToolBarVisible);
	
	put_ColorTextEditable(m_crTextEditable);
	put_ColorTextHighlight(m_crTextHighlight);
	put_OLEDropMode(m_OLEDropMode);
	put_AutoIndent(m_bAutoIndent);
	put_ChangeCaseMode(m_KeyWordChangeCase);
	
	// Display selection position in status bar
	UpdateStatusBar();

	// Upate toolbar
	EnableDisableToolBarButtons();	

	return 0;
}

HRESULT CSQLStudioInput::OnDraw(ATL_DRAWINFO& di)
{
	if(m_StatusBar.m_hWnd) m_StatusBar.Invalidate();	
	return S_OK;
}

void CSQLStudioInput::EnableDisableToolBarButtons(void)
{

	bool bCharactersSelected =(GetNumberOfSelectedCharacters() > 0);
	
	m_ToolBar.SendMessage(TB_ENABLEBUTTON, ID_CUT, MAKELONG(bCharactersSelected &&(!m_bReadonly), 0));
	m_ToolBar.SendMessage(TB_ENABLEBUTTON, ID_COPY, MAKELONG(bCharactersSelected, 0));
	m_ToolBar.SendMessage(TB_ENABLEBUTTON, ID_PASTE, MAKELONG(::IsClipboardFormatAvailable(CF_TEXT) &&(!m_bReadonly), 0));
	m_ToolBar.SendMessage(TB_ENABLEBUTTON, ID_UNDO, MAKELONG(m_RichEditCtrl.SendMessage(EM_CANUNDO) &&(!m_bReadonly), 0));
	m_ToolBar.SendMessage(TB_ENABLEBUTTON, ID_REDO, MAKELONG(m_RichEditCtrl.SendMessage(EM_CANREDO) &&(!m_bReadonly), 0));
	m_ToolBar.SendMessage(TB_ENABLEBUTTON, ID_FIND_NEXT, MAKELONG(m_bstrFindWhat!=NULL, 0));
	m_ToolBar.SendMessage(TB_ENABLEBUTTON, ID_IMPORT, MAKELONG(!m_bReadonly, 0));
}

long CSQLStudioInput::GetNumberOfSelectedCharacters(void)
{

	long nStart, nEnd;

	if(m_pITextSelection != NULL){		

		m_pITextSelection->GetStart(&nStart);
		m_pITextSelection->GetEnd(&nEnd);
	}
	return nEnd-nStart;
}


void CSQLStudioInput::HighlightAllComments(void)
{
	if(m_bInsideHighlightKeyAndCom) return;
	if(m_RichEditCtrl.m_hWnd == NULL) return;	
	if((!m_strCommentPrefix) &&(!m_strCommentPrefix2)) return;	
		
	CInsideHighlightKeyAndCom			oInsideHighlightKeyAndCom(*this);
	CDisplayWaitCursor					oDisplayWaitCursor(*this);

	ITextRangePtr pITextRange;
	long nNumberOfLines = m_RichEditCtrl.SendMessage(EM_GETLINECOUNT);
	
	for(long nLineIndex = 1; nLineIndex <= nNumberOfLines; nLineIndex++){

		if(IsCommentLine(nLineIndex, pITextRange)){

			if(pITextRange) InternalHighlight(pITextRange, m_bHighlightKeyAndCom);			
		}
	}

	// reformat areas
	pITextRange->SetStart(0);
	pITextRange->Expand(tomStory,NULL);
	
}

LRESULT CSQLStudioInput::OnSelChange(int, LPNMHDR, BOOL&)
{	

	TCHAR s [100];
	::GetWindowText(m_RichEditCtrl.m_hWnd,(LPTSTR) s,99);

	// while currently highligthing nothing should happen
	if(m_bInsideHighlightKeyAndCom)
		return 0;

	// update search find/replace range only flag is not set(the search methods set the flag to a value != 0)
	if(m_WndFindReplace != NULL && m_bUpdateOfFindReplaceRange == true){

		UpdateFindReplaceRange(false);
	}
	else
	{   // reset flag
		m_bUpdateOfFindReplaceRange = true;
	}

	// Show the caret. By default the caret is not visible when
	// text is selected.
	m_RichEditCtrl.ShowCaret();
	
	// Display selection position in status bar
	UpdateStatusBar();

	// Update toolbar
	StartTimerUpdateToolbar();

	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// Command handlers 
/////////////////////////////////////////////////////////////////////////////
LRESULT CSQLStudioInput::OnCut(WORD, WORD, HWND, BOOL&)
{
	m_RichEditCtrl.SendMessage(WM_CUT);
	return 0;
}


LRESULT CSQLStudioInput::OnCopy(WORD, WORD, HWND, BOOL&)
{
	m_RichEditCtrl.SendMessage(WM_COPY);
	return 0;
}


LRESULT CSQLStudioInput::OnPaste(WORD, WORD, HWND, BOOL&)
{
	// The handling concernig the allowed formats is done in 'QueryAcceptData'
	m_RichEditCtrl.SendMessage(WM_PASTE);
	return 0;
}


LRESULT CSQLStudioInput::OnUndo(WORD, WORD, HWND, BOOL&)
{
	m_RichEditCtrl.SendMessage(WM_UNDO);
	m_RichEditCtrl.SendMessage(EM_SETSEL,-1,0);

	StartTimerHighlightKeyAndCom();
	EnableDisableToolBarButtons();

	return 0;
}

LRESULT CSQLStudioInput::OnRedo(WORD, WORD, HWND, BOOL&)
{
	m_RichEditCtrl.SendMessage(EM_REDO);

	StartTimerHighlightKeyAndCom();
	EnableDisableToolBarButtons();

	return 0;
}

LRESULT CSQLStudioInput::OnComment(WORD, WORD, HWND, BOOL&)
{	
	long nLineFrom,
		 nLineTo;

	InternalGetSelectedLines(&nLineFrom, &nLineTo);
	InternalComment(nLineFrom, nLineTo);

	return 0;
}

LRESULT CSQLStudioInput::OnUnComment(WORD, WORD, HWND, BOOL&)
{	
	long nLineFrom,
		 nLineTo;

	InternalGetSelectedLines(&nLineFrom, &nLineTo);
	InternalUnComment(nLineFrom, nLineTo);

	return 0;
}

LRESULT CSQLStudioInput::OnImport(WORD, WORD wID, HWND, BOOL&)
{
	CPressToolBarButton obj(m_ToolBar, wID);
	FileOpen(NULL);
	return 0;
}

LRESULT CSQLStudioInput::OnExport(WORD, WORD wID, HWND, BOOL&)
{	
	CPressToolBarButton obj(m_ToolBar, wID);
	FileSaveAs(NULL);
	return 0;
}


LRESULT CSQLStudioInput::OnFindText(WORD, WORD, HWND, BOOL&)
{	
	ShowFindReplaceDialog();
	return 0;
}


LRESULT CSQLStudioInput::OnForeTextEditColor(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{	
	ShowColorRefDialog(m_crTextEditable);
	return 0;
}

LRESULT CSQLStudioInput::OnHighLightTextColor(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	ShowColorRefDialog(m_crTextHighlight);

	return 0;
}


LRESULT CSQLStudioInput::OnChooseFont(WORD, WORD, HWND, BOOL&)
{
	ShowChooseFontDialog();
	return 0;
}



LRESULT CSQLStudioInput::OnFindTextNext(WORD, WORD, HWND, BOOL&)
{	
	// find the text
	findNextWithBeep();
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// Message handlers for registered messages
/////////////////////////////////////////////////////////////////////////////
LRESULT CSQLStudioInput::OnFindReplaceDlg(UINT, WPARAM, LPARAM, BOOL&)
{	
	// no update of find and replace range
	CNoUpdateOfFindReplaceRange oNoUpdateOfFindReplaceRange(*this);

	// The dialog box was closed
	if(m_oFINDREPLACE.Flags & FR_DIALOGTERM)
	{
		::PostThreadMessage(m_dwThreadID, WM_QUIT, 0, 0);
		m_dwThreadID = 0;
		m_WndFindReplace = NULL;
		m_bFindReplaceInSelection = false;

		// Set the focus to the richedit
		m_RichEditCtrl.SetFocus();
	}

	if(m_strFindWhat[0] != 0)
	{
		USES_CONVERSION;
		m_bstrFindWhat = T2OLE(m_strFindWhat);
	}
	if(m_strReplaceWith[0] != 0)
	{
		USES_CONVERSION;
		m_bstrReplaceWith = T2OLE(m_strReplaceWith);
	}

	// initialize range: either the findreplace range or the current selection
	ITextRangePtr pITextRange;

	// The 'Find' button was pressed
	if(m_oFINDREPLACE.Flags & FR_FINDNEXT){

		// internally this method either selects the range found or beeps
		if(findNextWithBeep(pITextRange) == true)	{

			// if search was successfull: adopt search/find range
			AdoptFindReplaceRange(pITextRange);
		}
		else
		{
			// reset flag needed for propper handling in OnSelChange:
			// If the something is found the found text will be selected:
			//  this leads to a SelChange being fired afterwards.
			// But the update of the FindReplaceRange should place in case
			// of nothing beeing found
			m_bUpdateOfFindReplaceRange = true;
		}
	}

	// The 'Replace' button was pressed
	if(m_oFINDREPLACE.Flags & FR_REPLACE)
	{
		// internally this method either selects the range found or beeps
		if(ReplaceAndFindNextWithBeep(pITextRange) == true)
		{
			// if a next occurance was found: adopt search/find range
			AdoptFindReplaceRange(pITextRange);
		}
		else
		{
			// reset flag needed for propper handling in OnSelChange:
			// If the something is found the found text will be selected:
			//  this leads to a SelChange being fired afterwards.
			// But the update of the FindReplaceRange should place in case
			// of nothing beeing found
			m_bUpdateOfFindReplaceRange = true;
		}

	}

	// The 'Replace All' button was pressed
	if(m_oFINDREPLACE.Flags & FR_REPLACEALL)
	{		
		// if at least one replace took place
		// the method InternalReplaceAll internally updates the find/replace range(m_pITextRangeFindReplace)
		if(InternalReplaceAll(pITextRange, true) > 0)
		{
			// update selection and scroll it into view
			pITextRange->Select();
			pITextRange->ScrollIntoView(tomStart);
		}
		else
		{
			// reset flag needed for propper handling in OnSelChange:
			// If the something is found the found text will be selected:
			//  this leads to a SelChange being fired afterwards.
			// But the update of the FindReplaceRange should place in case
			// of nothing beeing found
			m_bUpdateOfFindReplaceRange = true;
		}

	}

	// Update toolbar
	EnableDisableToolBarButtons();

	return 0;
}

LRESULT CSQLStudioInput::OnTimer(UINT, WPARAM wParam, LPARAM, BOOL&)
{	
	
	switch(wParam)
	{
		case ID_TIMER_HIGHLIGHT_COMMENTS:
			KillTimer(ID_TIMER_HIGHLIGHT_COMMENTS);
			HighlightVisibleComments();	
			m_RichEditCtrl.Invalidate();
			break;
		case ID_TIMER_UPDATE_TOOLBAR:
			KillTimer(ID_TIMER_UPDATE_TOOLBAR);
			EnableDisableToolBarButtons();			
			break;
		case ID_TIMER_ON_CREATE:
			KillTimer(ID_TIMER_ON_CREATE);
			// Trick/hack to solve lost focus problem, see also method ::CreateRichEditWindow
			m_RichEditCtrl.PostMessage(WM_KEYDOWN, VK_RIGHT, 0);
			m_RichEditCtrl.PostMessage(WM_KEYDOWN, VK_LEFT, 0);
			break;		
	}
	return 0;
}

LRESULT CSQLStudioInput::OnSetCursor(UINT, WPARAM, LPARAM, BOOL& bHandled)
{
	if(m_nWaitCursorCount > 0)
		::SetCursor(m_hCursorWait);
	else
		::SetCursor(m_hCursorBeam);		
	
	return 0;
}
LRESULT CSQLStudioInput::OnEraseBkGnd(UINT, WPARAM, LPARAM, BOOL&)
{
	// To reduce flickering do not process this message
	return 0;
}


LRESULT CSQLStudioInput::OnDestroy(UINT, WPARAM, LPARAM, BOOL&)
{
	// Delete the font
	if(m_RichEditCtrl.GetFont() != NULL)
		::DeleteObject(m_RichEditCtrl.GetFont());	

	// Release display DC
	m_RichEditCtrl.ReleaseDC(m_hDCRichEdit);

	// if search and replace thread is still running: kill thread
	if(m_hThread != NULL)
	{
		DWORD dwExitCode;
		::GetExitCodeThread(m_hThread,(LPDWORD) &dwExitCode);
		
		if(dwExitCode & STILL_ACTIVE)
		{
			::PostThreadMessage(m_dwThreadID, WM_QUIT, 0, 0);
			::CloseHandle(m_hThread);
			m_dwThreadID = 0;
			m_WndFindReplace = NULL;		
		}		
	}

	
	m_mapDisplayText[ID_CUT].erase();
	m_mapDisplayText[ID_COPY].erase();
	m_mapDisplayText[ID_PASTE].erase();
	m_mapDisplayText[ID_UNDO].erase();
	m_mapDisplayText[ID_REDO].erase();
	m_mapDisplayText[ID_COMMENT].erase();
	m_mapDisplayText[ID_UNCOMMENT].erase();
	m_mapDisplayText[ID_FIND_REPLACE].erase();
	m_mapDisplayText[ID_FIND_NEXT].erase();
	m_mapDisplayText[ID_IMPORT].erase();
	m_mapDisplayText[ID_EXPORT].erase();
	m_mapDisplayText[ID_LINE_COL].erase();
	m_mapDisplayText[ID_FROM_LINE_COL_TO_LINE_COL].erase();
	m_mapDisplayText[ID_LINE_FROM_TO_OF].erase();
	m_mapDisplayText[ID_MODIFIED_TRUE].erase();
	m_mapDisplayText[ID_MODIFIED_FALSE].erase();
	m_mapDisplayText[ID_OVERWRITE].erase();
	m_mapDisplayText[ID_INSERT].erase();
	m_mapDisplayText[ID_PROPERTIES].erase();
	m_mapDisplayText[ID_CHOOSEFONT].erase();	
	m_mapDisplayText[ID_FORTEXTEDIT].erase();	
	m_mapDisplayText[ID_FORHIGHLIGHT].erase();

	m_mapDisplayText.clear();

	// Delete header bitmap
	if(m_TBImageList){

		ImageList_Destroy(m_TBImageList);
		m_TBImageList = NULL;
	}

	if(m_pITextSelection.p != NULL && m_pITextSelection != NULL){

		m_pITextSelection.Release();
		m_pITextSelection.p = NULL;
	}

	// Finally free global QI pointers to avoid memory leaks.
	if(m_pITextDocument.p != NULL && m_pITextDocument != NULL){

		m_pITextDocument.Release();
		m_pITextDocument.p = NULL;
	}

	// Especially the ones retrieved from the richedit which will be destroyed right afterwards
	// and thus are probably not anymore valid within the destructor ::~CSQLStudioInput()!!
	if(m_pITextRangeFindReplace){

		m_pITextRangeFindReplace.Release();		
	}	
	

	return 0;
}


LRESULT CSQLStudioInput::OnLButtonDblClkEdit(WPARAM wParam, LPARAM)
{	
	if(m_bNavigateOnDblclick != VARIANT_FALSE)
	{
		long	nChar;
		m_pITextSelection->GetChar(&nChar);

		if(wParam & MK_CONTROL 
			 || iswspace(static_cast<unsigned short>(nChar)) 
			 || iswcntrl(static_cast<unsigned short>(nChar)))
		{
			m_pITextSelection->ScrollIntoView(tomStart);
			return 1;
		}
	}

	// Fire the event
	Fire_DblClick();

	return 0;
	
}


LRESULT CSQLStudioInput::OnKeyDownEdit(WPARAM wParam, LPARAM lParam)
{	
	
	switch(wParam)
	{

		// to register insert/overwrite modus for displaying status within statusbar
		case VK_INSERT:
			{
			    DWORD dwExtendedKey = lParam;
				// feature!!: richedit only changes insert/overwrite mode in edit mode 
				if((dwExtendedKey >> 23) && m_bReadonly == false)
				{
					InternalSwitchInsertOverwrite();
					UpdateStatusBar();
				}
			}
			return 0;

		
		// Ctrl + Tab -> do nothing
	    case VK_TAB:
		    if(GetKeyState(VK_CONTROL)>>15)

				if(GetKeyState(VK_SHIFT)>>15){

					UnIndentSelection();
				}
				else{
				
					IndentSelection();
				}	
				return 1;
			break;

		// Ctrl + 'F' -> Find Dialog
		case 'F':
			if(GetKeyState(VK_CONTROL)>>15){

				ShowFindReplaceDialog();
				return 1;
			}
			break;

		// Ctrl + 'G' -> FindNext
		case 'G':
			if(GetKeyState(VK_CONTROL)>>15){

				// find the text
				findNextWithBeep();
				return 1;
			}
			break;

		// F1 -> Fire F1 Event
		case VK_F1:
			if(GetKeyState(VK_CONTROL)>>15)
				return 0;
			if(GetKeyState(VK_SHIFT)>>15)
				return 0;
			if(GetKeyState(VK_MENU)>>15)
				return 0;
			// fire event F1 only when no other virtual keys are pressed
			Fire_F1();
			return 1;

		// F2 -> Fire F2 Event
		case VK_F2:
			if(GetKeyState(VK_CONTROL)>>15)
				return 0;
			if(GetKeyState(VK_SHIFT)>>15)
				return 0;
			if(GetKeyState(VK_MENU)>>15)
				return 0;
			// fire event F2 only when no other virtual keys are pressed
			Fire_F2();
			return 1;

		// Enter -> Indent
		case VK_RETURN:
			if(m_bAutoIndent != VARIANT_FALSE)
			{
				if(InternalIndentOnEnter())
				{	// The message was not handled by InternalIndentOnEnter()
					return 1;
				}
				else{	
					// The message was not handled by InternalIndentOnEnter()
					// Thus let the control process the message
					return 0;
				}
			}
			else							
				return 0; // Let the control process the message
			
			break;
	}

	if(m_sVirtualKeyCodeBigger != m_sVirtualKeyCodeLess)
	{
	    //  '>' and '<' are located on different keys, e.g. US keyboards

	// Shift + Crtl + '>' -> UnComment	
		if(wParam ==(unsigned) m_sVirtualKeyCodeBigger)
		{
			if(GetKeyState(VK_CONTROL)>>15)
			{
				long nLineFrom,
					 nLineTo;

				InternalGetSelectedLines(&nLineFrom, &nLineTo);
				InternalUnComment(nLineFrom, nLineTo);

				return 1;
			}
		}
		
		// Ctrl + '<' -> Comment
		if(wParam ==(unsigned) m_sVirtualKeyCodeLess)
		{
			if(GetKeyState(VK_CONTROL)>>15)
			{
		
				long nLineFrom,
					 nLineTo;

				InternalGetSelectedLines(&nLineFrom, &nLineTo);
				InternalComment(nLineFrom, nLineTo);

				return 1;
			}
		}
	}
	else
	{
		//  '>' and '<' are located on the same key, e.g. german keyboards		
		if(wParam ==(unsigned) m_sVirtualKeyCodeBigger)
		{
			if(GetKeyState(VK_CONTROL)>>15)
			{
				if(GetKeyState(VK_SHIFT)>>15)
				{
					// Shift + Crtl + '<' -> UnComment	
					long nLineFrom,
						 nLineTo;

					InternalGetSelectedLines(&nLineFrom, &nLineTo);
					InternalUnComment(nLineFrom, nLineTo);

					return 1;
				}
				else
				{
					// Crtl + '<' -> Comment	
					long nLineFrom,
						 nLineTo;

					InternalGetSelectedLines(&nLineFrom, &nLineTo);
					InternalComment(nLineFrom, nLineTo);

					return 1;
				}
			}		
		}
	}

	BOOL bShift =(GetKeyState(VK_SHIFT) < 0);
    BOOL bCtrl =(GetKeyState(VK_CONTROL) < 0);
    BOOL bAlt =(GetKeyState(VK_MENU) < 0);

    short nKeyState =(short)(bShift +(bCtrl << 1) +(bAlt << 2));


	Fire_KeyDown((short)wParam,nKeyState);
	Fire_Change();

	return 0; // Let the control process the message
}


LRESULT CSQLStudioInput::OnCharEdit(WPARAM wParam, LPARAM lParam)
{
	
	switch(wParam)
	{
		// Tabs are supported for indenting
		case VK_TAB:

			if(GetKeyState(VK_CONTROL)>>15)
				return 1;

			// Shift + Tab
			if(GetKeyState(VK_SHIFT)>>15)
			{
				long nLineFrom,
					 nLineTo;

				InternalGetSelectedLines(&nLineFrom, &nLineTo);
				InternalUnIndent(nLineFrom, nLineTo);
				
				return 1;
			}

			long nLineFrom, nColFrom, nLineTo, nColTo;
			InternalGetSelectionPos(&nLineFrom, &nColFrom, &nLineTo, &nColTo);

			InternalIndent(nLineFrom, nColFrom, nLineTo, nColTo, -1);

			return 1;
				
		// Ctrl + 'A' -> update toolbar
		case 'A':
			if(GetKeyState(VK_CONTROL)>>15)
			{
				StartTimerUpdateToolbar();
				return 0;
			}
	};

	return 0; // Let the control process the message
}

LRESULT CSQLStudioInput::OnVScrollEdit(WPARAM wParam, LPARAM)
{
	// Because of a bug in richedit EN_VSCROLL is not sent when
	// the user drags the scrollbar to a position. So we have
	// to get the WM_VSCROLL message directly from the control.
	
	if(m_bIgnoreScrollEvents) return 0;

	if(m_bInsideHighlightKeyAndCom == false && LOWORD(wParam) == SB_THUMBTRACK){

		StartTimerHighlightKeyAndCom();
		UpdateStatusBar();
	}

	m_RichEditCtrl.Invalidate();

	return 0; // Let the control process the message
}


STDMETHODIMP CSQLStudioInput::TranslateAccelerator(LPMSG lpmsg)
{
     CComQIPtr<IOleControlSite,&IID_IOleControlSite>spCtrlSite(m_spClientSite);
     if(spCtrlSite)
     {

		if((lpmsg->message == WM_KEYDOWN) &&(
			(LOWORD(lpmsg->wParam) == VK_RETURN) ||
           (LOWORD(lpmsg->wParam) == VK_END)	 ||	
			(LOWORD(lpmsg->wParam) == VK_HOME)	 ||	
			(LOWORD(lpmsg->wParam) == VK_LEFT)	 ||	
			(LOWORD(lpmsg->wParam) == VK_RIGHT)	 ||	
			(LOWORD(lpmsg->wParam) == VK_DOWN)	 ||
			(LOWORD(lpmsg->wParam) == VK_UP)			
			))
		{	
			DWORD keymods = KEYMOD_NONE;
			keymods +=(GetKeyState(VK_CONTROL) & 0x1000) ? KEYMOD_CONTROL : KEYMOD_NONE;
			keymods +=(GetKeyState(VK_SHIFT) & 0x1000) ? KEYMOD_SHIFT: KEYMOD_NONE;			   
			keymods +=(GetKeyState(VK_MENU) & 0x1000) ? KEYMOD_ALT: KEYMOD_NONE;
			
			//VK_SHIFT is down the Riched32 receives a double Keystroke	
			if((GetKeyState(VK_SHIFT) & 0x1000) == false	&& (GetKeyState(VK_CONTROL) & 0x1000) == false){
			   		   
				m_RichEditCtrl.SendMessage(lpmsg->message, lpmsg->wParam, lpmsg->lParam);
			}
		   
            return spCtrlSite->TranslateAccelerator(lpmsg,keymods);
        }
     }

     return S_FALSE;
}


LRESULT CSQLStudioInput::OnControlSetFocus(UINT, WPARAM, LPARAM, BOOL& bHandled)
{
	
	// the following three lines fix a problem within Visual Basic when hitting the "Enter" key
	CComQIPtr <IOleControlSite, &IID_IOleControlSite> spSite(m_spClientSite);
	if(m_bInPlaceActive && spSite)
		spSite->OnFocus(TRUE);
		
    
	m_RichEditCtrl.SetFocus();

	bHandled = TRUE;    

	// Update toolbar
	EnableDisableToolBarButtons();

	return 0;
}

LRESULT CSQLStudioInput::OnEditSetFocus	(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
    CComQIPtr <IOleControlSite, &IID_IOleControlSite>spSite(m_spClientSite);
    if(m_bInPlaceActive && spSite){

       spSite->OnFocus(TRUE);
	}

    return 0;
}

LRESULT CSQLStudioInput::OnKillFocus(UINT ,WPARAM ,LPARAM ,BOOL&)
{
    return 0;
}


LRESULT CSQLStudioInput::OnEditKillFocus(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{

    CComQIPtr <IOleControlSite, &IID_IOleControlSite>spSite(m_spClientSite);
	if(m_bInPlaceActive && spSite){
		spSite->OnFocus(FALSE);
	}
	return 0;
}

// implemented to fix problems when running within Visual Basic
// see MSDN-Articles: "ATL Controls Do More with Less" 
//               and  "PRB: Focus and Tab Issues with ATL Subclassed Edit Control"
LRESULT CSQLStudioInput::OnMouseActivate(UINT,WPARAM,LPARAM,BOOL&)
{

	InPlaceActivate(OLEIVERB_UIACTIVATE);
	return 0;

} 

LRESULT  CSQLStudioInput::OnMouseMove(WPARAM wParam, LPARAM lParam)
{

	short xPos = GET_X_LPARAM(lParam); 
	short yPos = GET_Y_LPARAM(lParam); 

	short lKey = 0;
	short lMouseButton = 0;

	RECT x;
	m_ToolBar.GetWindowRect (&x);


	/*
	vbShiftMask 1 SHIFT key is pressed. 
	vbCtrlMask 2 CTRL key is pressed. 
	vbAltMask 4 ALT key is pressed. 

	vbLeftButton 1 Left button is pressed. 
	vbRightButton 2 Right button is pressed. 
	vbMiddleButton 4 Not supported. 

	*/

	if(wParam & MK_SHIFT) lKey = vbShiftMask;
	if(wParam & MK_CONTROL) lKey |= vbCtrlMask;
	if(wParam & MK_ALT) lKey |= vbAltMask;
	if(lParam & MK_LBUTTON) lMouseButton = vbLeftButton;
	if(lParam & MK_RBUTTON) lMouseButton = vbRightButton;	
	if(lParam & MK_MBUTTON) lMouseButton = vbMiddleButton;
	
	Fire_MouseMove(lMouseButton,lKey,yPos,xPos);

	return 0;
}
/*!----------------------------------------------------------------------
   function: OnMouseUp
   description: when the user releases the mouse button 
                
   arguments:    [in] wParam,Indicates whether various virtual keys are down. 
					  This parameter can be one or more of the following values
					    MK_CONTROL   The CTRL key is down. 
						MK_LBUTTON   The left mouse button is down. 
						MK_MBUTTON   The middle mouse button is down. 
						MK_RBUTTON   The right mouse button is down. 
						MK_SHIFT     The SHIFT key is down. 
						MK_XBUTTON1  Windows 2000: The first X button is down. 
						MK_XBUTTON2  Windows 2000: The second X button is down. 					  

				 [in] lParam, Specifies a POINTS structure that contains the x- and y-coordinates of
					  the cursor. The coordinates are relative to the upper-left corner of the screen. 


                 
   return value: / 
-------------------------------------------------------------------------*/
LRESULT  CSQLStudioInput::OnMouseUp(WPARAM wParam, LPARAM lParam)
{

	short xPos = GET_X_LPARAM(lParam); 
	short yPos = GET_Y_LPARAM(lParam); 

	short lKey = 0;
	short lMouseButton = 0;

	/*
	vbShiftMask 1 SHIFT key is pressed. 
	vbCtrlMask 2 CTRL key is pressed. 
	vbAltMask 4 ALT key is pressed. 

	vbLeftButton 1 Left button is pressed. 
	vbRightButton 2 Right button is pressed. 
	vbMiddleButton 4 Not supported. 

	*/

	if(wParam & MK_SHIFT) lKey = vbShiftMask;	
	if(wParam & MK_CONTROL) lKey |= vbCtrlMask;
	if(wParam & MK_ALT) lKey |= vbAltMask;
	if(lParam & MK_LBUTTON) lMouseButton = vbLeftButton;
	if(lParam & MK_RBUTTON) lMouseButton = vbRightButton;	
	if(lParam & MK_MBUTTON) lMouseButton = vbMiddleButton;
	
	Fire_MouseUp(lMouseButton,lKey,yPos,xPos);

	return 0;
}

/*!----------------------------------------------------------------------
   function: OnMouseDown
   description: when the user when the user presses the mouse button while the cursor is in the client area of a window
                
   arguments:    [in] wParam,Indicates whether various virtual keys are down. 
					  This parameter can be one or more of the following values
					    MK_CONTROL   The CTRL key is down. 
						MK_LBUTTON   The left mouse button is down. 
						MK_MBUTTON   The middle mouse button is down. 
						MK_RBUTTON   The right mouse button is down. 
						MK_SHIFT     The SHIFT key is down. 
						MK_XBUTTON1  Windows 2000: The first X button is down. 
						MK_XBUTTON2  Windows 2000: The second X button is down. 					  

				 [in] lParam, Specifies a POINTS structure that contains the x- and y-coordinates of
					  the cursor. The coordinates are relative to the upper-left corner of the screen. 


                 
   return value: / 
-------------------------------------------------------------------------*/
LRESULT  CSQLStudioInput::OnMouseDown(WPARAM wParam, LPARAM lParam)
{

	short xPos = GET_X_LPARAM(lParam); 
	short yPos = GET_Y_LPARAM(lParam); 

	short lKey = 0;
	short lMouseButton = 0;

	/*
	vbShiftMask 1 SHIFT key is pressed. 
	vbCtrlMask 2 CTRL key is pressed. 
	vbAltMask 4 ALT key is pressed. 

	vbLeftButton 1 Left button is pressed. 
	vbRightButton 2 Right button is pressed. 
	vbMiddleButton 4 Not supported. 

	*/

	if(wParam & MK_SHIFT) lKey = vbShiftMask;	
	if(wParam & MK_CONTROL) lKey |= vbCtrlMask;
	if(wParam & MK_ALT) lKey |= vbAltMask;
	if(lParam & MK_LBUTTON) lMouseButton = vbLeftButton;
	if(lParam & MK_RBUTTON) lMouseButton = vbRightButton;	
	if(lParam & MK_MBUTTON) lMouseButton = vbMiddleButton;
	
	Fire_MouseDown(lMouseButton,lKey,yPos,xPos);

	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// Notify handlers for richedit
/////////////////////////////////////////////////////////////////////////////
LRESULT CSQLStudioInput::OnDropFiles(int, LPNMHDR pnmh, BOOL&)
{
	
	if(m_OLEDropMode == OLEDropNone) return 0;
	
	long nDroppedFiles = 0;
    TCHAR lpszFile[_MAX_PATH];

    // Get the handle to the drop object.
    HDROP hDrop = reinterpret_cast<HDROP>(reinterpret_cast<ENDROPFILES*>(pnmh)->hDrop);

    // Determine how many objects have been dropped.
    nDroppedFiles = DragQueryFile(hDrop, 0xFFFFFFFF, NULL, 0);

	if(nDroppedFiles > 1) return 0;
	
	
	// Get the name of the file dropped.
	DragQueryFile(hDrop, 0, lpszFile, sizeof(lpszFile));

	// Open the file
	InternalFileOpen(lpszFile);

	// Close the drop handle
	DragFinish(hDrop); 

	return 1;

}

LRESULT CSQLStudioInput::OnMsgFilter(int, LPNMHDR pnmh, BOOL&)
{

	MSGFILTER *pMSGFILTER = reinterpret_cast<MSGFILTER*>(pnmh);
	
	switch(pMSGFILTER->msg)
	{
		case WM_CHAR:
			return OnCharEdit(pMSGFILTER->wParam, pMSGFILTER->lParam);

		case WM_SYSKEYDOWN:
		case WM_KEYDOWN:
			return OnKeyDownEdit(pMSGFILTER->wParam, pMSGFILTER->lParam);

		case WM_LBUTTONDBLCLK:
			return OnLButtonDblClkEdit(pMSGFILTER->wParam, pMSGFILTER->lParam);

		case WM_VSCROLL:
			return OnVScrollEdit(pMSGFILTER->wParam, pMSGFILTER->lParam);
		case WM_MOUSEMOVE:
			return OnMouseMove(pMSGFILTER->wParam, pMSGFILTER->lParam);
		case WM_LBUTTONDOWN:
		case WM_RBUTTONDOWN:
		case WM_MBUTTONDOWN:
			return OnMouseDown(pMSGFILTER->wParam, pMSGFILTER->lParam);		
		case WM_LBUTTONUP:
		case WM_RBUTTONUP:
		case WM_MBUTTONUP:
			return OnMouseUp(pMSGFILTER->wParam, pMSGFILTER->lParam);		
	}


	return 0; // By default let the control process the event
}

/////////////////////////////////////////////////////////////////////////////
// Command handlers for richedit
/////////////////////////////////////////////////////////////////////////////
LRESULT CSQLStudioInput::OnChange(WORD, WORD, HWND, BOOL&)
{	
	if(m_bInsideHighlightKeyAndCom == false)
		StartTimerHighlightKeyAndCom();	

	return 0;
}

LRESULT CSQLStudioInput::OnVScroll(WORD wNotifyCode, WORD wID, HWND, BOOL& bHandled)
{
	
	if(m_bIgnoreScrollEvents)
	{
		bHandled = false;
		return 0;
	}

	CIgnoreScrollEvents oIgnoreScrollEvents(*this);

	m_RichEditCtrl.SendMessage(WM_VSCROLL, MAKELONG(wNotifyCode, wID), 0);

	if(m_bInsideHighlightKeyAndCom == false)
	{
	    StartTimerHighlightKeyAndCom();
		UpdateStatusBar();
	}

	m_RichEditCtrl.Invalidate();
	
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// Notify handlers for toolbar
/////////////////////////////////////////////////////////////////////////////
LRESULT CSQLStudioInput::OnNeedText(int, LPNMHDR pnmh, BOOL&)
{	

	TOOLTIPTEXT *pToolTipText = reinterpret_cast<TOOLTIPTEXT*>(pnmh);
	pToolTipText->lpszText = const_cast<LPTSTR>(GetDisplayText(pToolTipText->hdr.idFrom));

    return 0;
}

void CSQLStudioInput::UpdateStatusBar(void)
{	

	if(m_bInsideHighlightKeyAndCom ||(!m_bUpdateStatusBar))
		return;

	TCHAR szBuffer[256];

	if(InternalGetModifyStatus()){

		//_stprintf(szBuffer, GetDisplayText(ID_MODIFIED_TRUE));

		//!!! don't show modified true text, not supported now !
		_stprintf(szBuffer, GetDisplayText(ID_MODIFIED_FALSE));
	}
	else
		_stprintf(szBuffer, GetDisplayText(ID_MODIFIED_FALSE));
	m_StatusBar.SendMessage(SB_SETTEXT, m_nStatusBarPartModified, reinterpret_cast<LPARAM>(szBuffer));

	if(m_bInsertModus == true)
		_stprintf(szBuffer, GetDisplayText(ID_INSERT));
	else
		_stprintf(szBuffer, GetDisplayText(ID_OVERWRITE));

	m_StatusBar.SendMessage(SB_SETTEXT, m_nStatusBarPartInsOvr, reinterpret_cast<LPARAM>(szBuffer));


	// Display caret position / selection position
	CHARRANGE oCHARRANGE;
	m_RichEditCtrl.SendMessage(EM_EXGETSEL, 0, reinterpret_cast<LPARAM>(&oCHARRANGE));

	LONG nLine;
	LONG nCol;
	CharIndexToLinePos(oCHARRANGE.cpMin, nLine, nCol);

	if(oCHARRANGE.cpMin == oCHARRANGE.cpMax)

		_stprintf(szBuffer, GetDisplayText(ID_LINE_COL), nLine, nCol);	
	else{

		LONG nLineTo;
		LONG nColTo;
		CharIndexToLinePos(oCHARRANGE.cpMax, nLineTo, nColTo);
		_stprintf(szBuffer, GetDisplayText(ID_FROM_LINE_COL_TO_LINE_COL), nLine, nCol, nLineTo, nColTo);
	}

	m_StatusBar.SendMessage(SB_SETTEXT, m_nStatusBarPartSelectionPos, reinterpret_cast<LPARAM>(szBuffer));

	// Display first and last visible line and total number of lines
	LONG nFirstLine;
	LONG nLastLine;
	LONG nLineCount = GetFirstAndLastVisibleLine(nFirstLine, nLastLine);

	_stprintf(szBuffer, GetDisplayText(ID_LINE_FROM_TO_OF), nFirstLine, nLastLine, nLineCount);
	m_StatusBar.SendMessage(SB_SETTEXT, m_nStatusBarPartVisibleLines, reinterpret_cast<LPARAM>(szBuffer));
}



void CSQLStudioInput::CreateRichEditWindow(void)
{	

	if(m_hWnd == NULL)
		return;	

	if(m_RichEditCtrl.m_hWnd)
	{
		m_RichEditCtrl.ReleaseDC(m_hDCRichEdit);
		m_RichEditCtrl.DestroyWindow();
	}

	// Default styles for all modes
	// The ES_DISABLENOSCROLL style is set because of a bug in RICHEDIT 2.0
	//(not in 1.0). If an empty richedit is created and text is inserted
	// which is so big that both scrollbars should appear only one
	// scrollbar appears. With this style both scrollbars are always visible
	// so the problem does not occur.
	DWORD dwStyle =	WS_BORDER|WS_CHILD|WS_VISIBLE|WS_VSCROLL|WS_CLIPSIBLINGS|
					ES_AUTOVSCROLL|ES_MULTILINE|ES_NOHIDESEL|ES_WANTRETURN;

	if(m_bWordWrap == VARIANT_FALSE){

		dwStyle |= WS_HSCROLL|ES_AUTOHSCROLL;
	}

	// Create the richedit window
	RECT rc = {0,0,0,0};

	
	// appropriate creation for subclassed RicheditWindow
	m_RichEditCtrl.Create(m_hWnd, rc, _T(""), dwStyle, 0, ID_RICHEDIT);

	// Due to a bug in SAPGUI the ES_SELECTIONBAR style cannot be set
	// during Create. So the style is set after creation
	m_RichEditCtrl.SendMessage(EM_SETOPTIONS, ECOOP_OR, ECO_SELECTIONBAR);

	m_hDCRichEdit = m_RichEditCtrl.GetDC();

	
	CreateSQLStudioDSQLFont();
		
	// Set EventMasks
	// Important: The EN_PROTECTED notification is not only sent when the
	// user tries to alter the text, but also when the text is changed by the program.
	m_RichEditCtrl.SendMessage(EM_SETEVENTMASK, 0,	m_nENMFlags);

	// Limit the text size(Default is 1 MB)
	m_RichEditCtrl.SendMessage(EM_EXLIMITTEXT, 0, m_nLimitText);

	// Set the OLE callback interface
	CComQIPtr<IRichEditOleCallback, &IID_IRichEditOleCallback> pCallback(this);
	m_RichEditCtrl.SendMessage(EM_SETOLECALLBACK, 0, reinterpret_cast<LPARAM>(pCallback.p));

	// to use tom!!! used especially to suspend and resume Undo-buffering(e.g. in method HighlightVisibleComments)
	CComPtr<IRichEditOle> pRich;
	m_RichEditCtrl.SendMessage(EM_GETOLEINTERFACE, 0, reinterpret_cast<LPARAM>(&(pRich.p)));
	if(pRich)
	{
		if(m_pITextDocument.p)
			m_pITextDocument.Release();

		m_pITextDocument = pRich;
		InitITextSelection(m_pITextSelection);
	}
		
	// Accept dropped files
	m_RichEditCtrl.DragAcceptFiles();

	
	// Rearrange windows
	ArrangeWindows();
	
	//  The next two lines represent a trick to overcome the lost focus problem appeared
	//  with riched20.dll version 5.30.08.0701.
	//  We insert the space character and via a timer a keydown combination is triggered.
	//  Became obsolete with version 5.30.09.2701 thus the following two lines were changed into comments!
    // InternalSetText(" ");
	// SetTimer(ID_TIMER_ON_CREATE,TIME_ON_CREATE);

}

HFONT CSQLStudioInput::CreatePointFont(int nPointSize, LPCTSTR lpszFaceName)
{	

	//-----------------------------------------------------------------------------
	// This code was copied from the MFC implementation of CFont::CreatePointFont
	//-----------------------------------------------------------------------------
	LOGFONT logFont;
	memset(&logFont, 0, sizeof(LOGFONT));
	logFont.lfCharSet = DEFAULT_CHARSET;
	logFont.lfHeight = nPointSize;
	lstrcpyn(logFont.lfFaceName, lpszFaceName, LF_FACESIZE);

	HDC hDC = ::GetDC(NULL);

	// convert nPointSize to logical units based on pDC
	POINT pt;
	pt.y = ::GetDeviceCaps(hDC, LOGPIXELSY) * logFont.lfHeight;
	pt.y /= 720;    // 72 points/inch, 10 decipoints/point
	::DPtoLP(hDC, &pt, 1);
	POINT ptOrg = { 0, 0 };
	::DPtoLP(hDC, &ptOrg, 1);
	logFont.lfHeight = -abs(pt.y - ptOrg.y);

	::ReleaseDC(NULL, hDC);

	return CreateFontIndirect(&logFont);
}

void CSQLStudioInput::CreateSQLStudioDSQLFont()
{

	HFONT	hFont = NULL;
	HRESULT hr = E_FAIL;

	/*Create Font*/
	CComQIPtr<IFont,&IID_IFont> pFont(m_pFont);
	if(pFont != NULL){

		hr = pFont->get_hFont(&hFont);
		// the get_hFont method returns a temporary font
		// using the font requires making a copy
		if(SUCCEEDED(hr))
		{
			LOGFONT oLogFont;
			::GetObject(hFont, sizeof(LOGFONT), &oLogFont);
			hFont = ::CreateFontIndirect(&oLogFont);
		}	
	}
	else{
		// If no font is found use 'Courier New',if no font size is found use 10pt
#ifdef UNICODE

			CMyComBSTR strFont("Courier New");
			hFont = CreatePointFont(100,strFont);
#else
			hFont = CreatePointFont(100,"Courier New");
#endif
	}

	if(m_RichEditCtrl.m_hWnd != NULL  && hFont != NULL)
	{
		m_RichEditCtrl.SetFont(hFont);
		m_StatusBar.SetFont(hFont);
	}
}

STDMETHODIMP CSQLStudioInput::get_ReadOnly(VARIANT_BOOL *pVal)
{
	*pVal = m_bReadonly;	
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::put_ReadOnly(VARIANT_BOOL newVal)
{
	COLORREF	crBackground;

	m_bReadonly = newVal;

	if(m_bReadonly == VARIANT_FALSE)
		crBackground = m_crBackgroundEditable;	
	else
		crBackground = m_crBackgroundReadonly;


	if(m_RichEditCtrl.m_hWnd)
	{
		// Readonly mode
		m_RichEditCtrl.SendMessage(EM_SETREADONLY, m_bReadonly);

		// Background color
		m_RichEditCtrl.SendMessage(EM_SETBKGNDCOLOR, false, crBackground);

		if(m_bReadonly) EmptyUndoBuffer();

		EnableDisableToolBarButtons();		
	}	

	return S_OK;
}

void CSQLStudioInput::CreateStatusBar(void)
{	
	// Create statusbar
	m_StatusBar.Attach(CreateStatusWindow(WS_VISIBLE|WS_CHILD|WS_CLIPSIBLINGS, _T(""), m_hWnd, ID_STATUS_BAR));

}

void CSQLStudioInput::CreateToolBar(void)
{
		
		m_TBImageList = ImageList_LoadImage(_Module.GetResourceInstance(), 
											MAKEINTRESOURCE(IDB_TOOLBAR), 
											17, 3, RGB(192,192,192), 
											IMAGE_BITMAP, 
											LR_DEFAULTCOLOR);
	
	
		// Create the toolbar control.
		long nWidth = 17;
		long nHeight = 17;

		m_ToolBar.Attach(CreateToolbarEx(
					m_hWnd,					// parent
					WS_CHILD|WS_VISIBLE|CCS_NODIVIDER|TBSTYLE_TOOLTIPS|TBSTYLE_FLAT, // window style
					0,						// toolbar ID
					0,						// number of bitmaps
					_Module.GetResourceInstance(), // mod instance
					0,						// resource ID for bitmap
					NULL,					// address of buttons
					0,						// number of buttons
					nWidth, nHeight,		// width & height of buttons
					nWidth, nHeight,		// width & height of bitmaps
					sizeof(TBBUTTON)));		// structure size
		
		SendMessage(m_ToolBar, TB_SETIMAGELIST, 0,(LPARAM)m_TBImageList);
		SendMessage(m_ToolBar, TB_SETEXTENDEDSTYLE, 0, TBSTYLE_EX_DRAWDDARROWS);

		TBBUTTON tpButton;
		
		// Cut
		tpButton.iBitmap = 1;	
		tpButton.idCommand = ID_CUT;	
		tpButton.fsState = TBSTATE_ENABLED;
		tpButton.fsStyle = TBSTYLE_BUTTON;
		tpButton.dwData = 0;
		tpButton.iString = 0;
		SendMessage(m_ToolBar, TB_ADDBUTTONS, 1,(LPARAM)&tpButton);

		// Copy
		tpButton.iBitmap = 2;
		tpButton.idCommand = ID_COPY;	
		tpButton.fsState = TBSTATE_ENABLED;
		tpButton.fsStyle = TBSTYLE_BUTTON;
		tpButton.dwData = 0;
		tpButton.iString = 0;
		SendMessage(m_ToolBar, TB_ADDBUTTONS, 1,(LPARAM)&tpButton);

		// Paste
		tpButton.iBitmap = 3;
		tpButton.idCommand = ID_PASTE;	
		tpButton.fsState = TBSTATE_ENABLED;
		tpButton.fsStyle = TBSTYLE_BUTTON;
		tpButton.dwData = 0;
		tpButton.iString = 0;
		SendMessage(m_ToolBar, TB_ADDBUTTONS, 1,(LPARAM)&tpButton);

		// Seperator
		tpButton.iBitmap = 0;
		tpButton.idCommand = 0;	
		tpButton.fsState = TBSTATE_ENABLED;
		tpButton.fsStyle = TBSTYLE_SEP;
		tpButton.dwData = 0;
		tpButton.iString = 0;
		SendMessage(m_ToolBar, TB_ADDBUTTONS, 1,(LPARAM)&tpButton);

		// Undo
		tpButton.iBitmap = 4;
		tpButton.idCommand = ID_UNDO;	
		tpButton.fsState = TBSTATE_ENABLED;
		tpButton.fsStyle = TBSTYLE_BUTTON;
		tpButton.dwData = 0;
		tpButton.iString = 0;
		SendMessage(m_ToolBar, TB_ADDBUTTONS, 1,(LPARAM)&tpButton);

		// Redo
		tpButton.iBitmap = 5;
		tpButton.idCommand = ID_REDO;	
		tpButton.fsState = TBSTATE_ENABLED;
		tpButton.fsStyle = TBSTYLE_BUTTON;
		tpButton.dwData = 0;
		tpButton.iString = 0;
		SendMessage(m_ToolBar, TB_ADDBUTTONS, 1,(LPARAM)&tpButton);

		// Seperator: this seperator as an idCommand to have the possibility to hide him
		tpButton.iBitmap = 0;
		tpButton.idCommand = ID_SEPUNDO;	
		tpButton.fsState = TBSTATE_ENABLED;
		tpButton.fsStyle = TBSTYLE_SEP;
		tpButton.dwData = 0;
		tpButton.iString = 0;
		SendMessage(m_ToolBar, TB_ADDBUTTONS, 1,(LPARAM)&tpButton);

		// Find
		tpButton.iBitmap = 6;
		tpButton.idCommand = ID_FIND_REPLACE;	
		tpButton.fsState = TBSTATE_ENABLED;
		tpButton.fsStyle = TBSTYLE_BUTTON;
		tpButton.dwData = 0;
		tpButton.iString = 0;
		SendMessage(m_ToolBar, TB_ADDBUTTONS, 1,(LPARAM)&tpButton);

		// FindNext
		tpButton.iBitmap = 7;
		tpButton.idCommand = ID_FIND_NEXT;	
		tpButton.fsState = TBSTATE_ENABLED;
		tpButton.fsStyle = TBSTYLE_BUTTON;
		tpButton.dwData = 0;
		tpButton.iString = 0;
		SendMessage(m_ToolBar, TB_ADDBUTTONS, 1,(LPARAM)&tpButton);

		// Seperator
		tpButton.iBitmap = 0;
		tpButton.idCommand = 0;	
		tpButton.fsState = TBSTATE_ENABLED;
		tpButton.fsStyle = TBSTYLE_SEP;
		tpButton.dwData = 0;
		tpButton.iString = 0;
		SendMessage(m_ToolBar, TB_ADDBUTTONS, 1,(LPARAM)&tpButton);

		// Import
		tpButton.iBitmap = 8;
		tpButton.idCommand = ID_IMPORT;	
		tpButton.fsState = TBSTATE_ENABLED;
		tpButton.fsStyle = TBSTYLE_BUTTON;
		tpButton.dwData = 0;
		tpButton.iString = 0;
		SendMessage(m_ToolBar, TB_ADDBUTTONS, 1,(LPARAM)&tpButton);

		// Export
		tpButton.iBitmap = 9;
		tpButton.idCommand = ID_EXPORT;	
		tpButton.fsState = TBSTATE_ENABLED;
		tpButton.fsStyle = TBSTYLE_BUTTON;
		tpButton.dwData = 0;
		tpButton.iString = 0;
		SendMessage(m_ToolBar, TB_ADDBUTTONS, 1,(LPARAM)&tpButton);



}


STDMETHODIMP CSQLStudioInput::EmptyUndoBuffer()
{
	
	m_RichEditCtrl.SendMessage(EM_EMPTYUNDOBUFFER);
	return S_OK;
	
}

STDMETHODIMP CSQLStudioInput::get_StatusBarVisible(VARIANT_BOOL *pVal)
{
	
	*pVal = m_bStatusBarVisible;

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::put_StatusBarVisible(VARIANT_BOOL newVal)
{
	
	m_bStatusBarVisible = newVal;

	if(m_StatusBar.m_hWnd)
	{
		if(m_bStatusBarVisible == VARIANT_FALSE)
			m_StatusBar.ShowWindow(SW_HIDE);
		else
			m_StatusBar.ShowWindow(SW_SHOW);

		ArrangeWindows();
	}


	return S_OK;
}

STDMETHODIMP CSQLStudioInput::get_ToolBarVisible(VARIANT_BOOL *pVal)
{
	
	*pVal = m_bToolBarVisible;

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::put_ToolBarVisible(VARIANT_BOOL newVal)
{
	
	m_bToolBarVisible = newVal;

	if(m_ToolBar.m_hWnd)
	{
		if(m_bToolBarVisible == VARIANT_FALSE)
			m_ToolBar.ShowWindow(SW_HIDE);
		else
			m_ToolBar.ShowWindow(SW_SHOW);
		ArrangeWindows();
	}

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::get_Text(BSTR *pVal)
{
	
	if(m_RichEditCtrl.m_hWnd == NULL)
	{
		// if the richedit is destroyed, the saved text in the m_bstrText
		// member is used
		*pVal = m_bstrText.Detach();
	}
	else
	{
		CComBSTR	bstrText;
		InternalGetText(bstrText);
		
		*pVal = bstrText.Detach();
	}

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::put_Text(BSTR newVal)
{
	if(m_RichEditCtrl.m_hWnd == NULL)
		// Save for use in OnCreate
		m_bstrText = newVal;
	else
		InternalSetText(newVal);	

	//put text modified false
	InternalSetModifyStatus(false);

	return S_OK;
}





////////////////////////////////////////////////////////////////////////////////////////////
// Internal methods to initialize TOM interface pointers.
// The methods are designed to avoid not released interface pointers.
// Therefore their your usage is encouraged,
// especially avoid to use Range(), GetDuplicate(), GetSelection() and GetStoryRanges()!!!
////////////////////////////////////////////////////////////////////////////////////////////

void CSQLStudioInput::InitITextSelection(ITextSelectionPtr &pITextSelection)
{
	
	if(m_pITextDocument && m_pITextDocument.p)
	{
		if(pITextSelection.p)
			pITextSelection.Release();

		m_pITextDocument->GetSelection(&pITextSelection);
	}
}

void CSQLStudioInput::InitITextStoryRanges(ITextStoryRangesPtr &pITextStoryRanges)
{
	
	if(m_pITextDocument && m_pITextDocument.p)
	{
		if(pITextStoryRanges.p)
			pITextStoryRanges.Release();

		m_pITextDocument->GetStoryRanges(&pITextStoryRanges);
	}
}


void CSQLStudioInput::InitITextRange(ITextRangePtr &pITextRange, long cpStart, long cpEnd)
{
	
	if(m_pITextDocument && m_pITextDocument.p)
	{
		if(pITextRange.p)
			pITextRange->SetRange(cpStart, cpEnd);
		else
			m_pITextDocument->Range(cpStart, cpEnd,  &pITextRange);
	}
}

void CSQLStudioInput::InitITextRange(ITextRangePtr &pITextRange, long nLine, bool nExtendMode)
{
	
	if(m_pITextDocument && m_pITextDocument.p)
	{	
		if(pITextRange.p == NULL)
			m_pITextDocument->Range(0, 0,  &pITextRange);

		pITextRange->SetIndex(tomLine, nLine, nExtendMode);
	}	
}

// If NULL is passed as first parameter the ITextPara of the selection is returned
void CSQLStudioInput::InitITextPara(const ITextRangePtr &pITextRange, ITextParaPtr &pITextPara)
{
	
	if(m_pITextDocument && m_pITextDocument.p)
	{
		if(pITextPara.p)
			pITextPara.Release();

		if(pITextRange==NULL || pITextRange.p == NULL)
		{
			if(m_pITextSelection && m_pITextSelection.p)
			m_pITextSelection->GetPara(&pITextPara);
		}
		else
			pITextRange->GetPara(&pITextPara);
	}
}

// If NULL is passed as first parameter a range representing the selection is returned
void CSQLStudioInput::GetDuplicate(const ITextRangePtr &pITextRange, ITextRangePtr &pITextRangeDuplicate)
{	
	if(m_pITextDocument && m_pITextDocument.p){

		if(pITextRangeDuplicate.p)
			pITextRangeDuplicate.Release();

		if(pITextRange && pITextRange.p){

			pITextRange->GetDuplicate(&pITextRangeDuplicate);
		}
		else{

			m_pITextSelection->GetDuplicate(&pITextRangeDuplicate);
		}
	}
}

void CSQLStudioInput::GetSelectionRangeDuplicate(ITextRangePtr &pITextRangeDuplicate)
{
	
    if(m_pITextSelection && m_pITextSelection.p)
	{	
		if(pITextRangeDuplicate.p)
			pITextRangeDuplicate.Release();

		m_pITextSelection->GetDuplicate(&pITextRangeDuplicate);
	}
}


void CSQLStudioInput::InternalFileOpen(LPCTSTR lpszFileName)
{
	
	
	HRESULT hRes = S_OK;
	
	if(lpszFileName){

		// Open the file
		CComVariant var(lpszFileName);

		//Select all text and replace
		m_RichEditCtrl.SendMessage(EM_SETSEL, 0, -1);

		//	select all and use tomPasteFile because of tom - doku:
		//	If the tomPasteFile flag is not set in the Flags parameter, the method first closes the current document after saving any unsaved changes
		
		if(m_pITextDocument->Open(&var, tomPasteFile + tomOpenExisting + tomReadOnly ,0) == S_OK) 
		{

			EmptyUndoBuffer();

			// scroll to upper left corner
			m_RichEditCtrl.SendMessage(EM_SETSEL, 0, 0);
		}
	}
}


void CSQLStudioInput::InternalFileSaveAs(LPCTSTR lpszFileName)
{	

	if(m_bConvertWordWrapToLineBreak == VARIANT_FALSE)
	{
		if(lpszFileName)
		{
			CComVariant var(lpszFileName);
			m_pITextDocument->Save(&var, tomText+tomCreateAlways, 0);
		}
	}
	else
	{
		// get a file handle 
		HANDLE hFile = ::CreateFile(lpszFileName, 
								GENERIC_WRITE, 
								FILE_SHARE_READ,
								NULL, 
								CREATE_ALWAYS,
								FILE_ATTRIBUTE_NORMAL,
								NULL);
		if(hFile != INVALID_HANDLE_VALUE)
		{
			DWORD dwNumberOfBytesWritten;
			vector<CBasicString> vecText;
			vector<CBasicString>::iterator itFirst;

			// get text in first element of the vector
			InternalGetText(vecText, true, true);
			itFirst = vecText.begin();

			::WriteFile(hFile, itFirst->c_str(), itFirst->length()*sizeof(TCHAR), &dwNumberOfBytesWritten, NULL);

			// Close the file
	        ::CloseHandle(hFile);
		}
	}
}

// returns the length of the text inserted into the control
long CSQLStudioInput::InternalSetText(const CComBSTR &bstrText, long nTextLength, long nFlags)
{
	
	CComBSTR bstrLocal;
	HRESULT hr;
	ITextRangePtr pITextRange;

	if(!bstrText)
		nTextLength = 0;

	if(nTextLength == -1)
		nTextLength = bstrText.Length();
	else 
	{
		if(nTextLength != static_cast<long>(bstrText.Length()))
		{
			bstrLocal.Append(bstrText.m_str, nTextLength);
		}
	}

	// if the whole text is replaced
	if(!(nFlags & SFF_SELECTION))
	{
		// check wether the amount of text is smaller then the set limit
		if(nTextLength > m_nLimitText)
		{
			::MessageBeep(0xFFFFFFFF);
			return 0;
		}		

		InitITextRange(pITextRange, 1);
		pITextRange->Expand(tomStory, NULL);

		if(bstrLocal)
			hr = pITextRange->SetText(bstrLocal);
		else
			hr = pITextRange->SetText(bstrText);

		ITextFontPtr		pITextFont;

		if(SUCCEEDED(pITextRange->GetFont(&pITextFont)))
		{
			pITextFont->SetForeColor(tomAutoColor);
			pITextFont->SetBackColor(tomAutoColor);
		}

		// empty undo buffer
		EmptyUndoBuffer();

		// set modify status to false
		m_RichEditCtrl.SendMessage(EM_SETMODIFY, false);

		// scroll to upper left corner
		m_RichEditCtrl.SendMessage(EM_SETSEL, 0, 0);

		pITextRange->Collapse(tomFalse);
		pITextRange->Select();

	}
	else // status will be set to modified internally
	{ 
		// check wether the amount of text is smaller 
		// then the set limit minus the selected range(which will be replaced)
		CHARRANGE oCHARRANGE_Original;
		long nTotalChars;
		
		GetSelectionRangeDuplicate(pITextRange);

		// get current selection
		m_RichEditCtrl.SendMessage(EM_EXGETSEL, 0, reinterpret_cast<LPARAM>(&oCHARRANGE_Original));
		pITextRange->GetStoryLength(&nTotalChars);

		if(nTextLength >(m_nLimitText				// the maximal allowed text
							 - nTotalChars	// minus the already available text plus the selected text
							 - oCHARRANGE_Original.cpMin + oCHARRANGE_Original.cpMax))
		{
			::MessageBeep(0xFFFFFFFF);
			return 0;
		}

		if(bstrLocal)
			hr = m_pITextSelection->SetText(bstrLocal);
		else
			hr = m_pITextSelection->SetText(bstrText);
	}

	UpdateStatusBar();
	
	if(SUCCEEDED(hr))
		return nTextLength;
	else 
		return 0;
}


long CSQLStudioInput::InternalGetText(CComBSTR &bstrText, long nFlags)
{

	CDisplayWaitCursor oDisplayWaitCursor(*this);

	ITextRangePtr pITextRange;

	if(!(nFlags & SFF_SELECTION))
	{   // if the whole text is retrieved
		// initialize range to contain extend over the first line
		InitITextRange(pITextRange, 1);
		// set range to contain the whole story
		pITextRange->Expand(tomStory, NULL);

	}
	else
	{   // selection only
		GetSelectionRangeDuplicate(pITextRange);
	}

	// get the text
	pITextRange->GetText(&bstrText);


	return bstrText.Length();
}

long CSQLStudioInput::InternalGetText(vector<CBasicString> &vecString,
									bool bGatherOutputInFirstElelemt, 
									bool bIncludeLineBreaks, 
									long nFlags)
{	

	CDisplayWaitCursor oDisplayWaitCursor(*this);

	vector<CBasicString>::iterator itFirst;
	ITextRangePtr pITextRange;
	ITextRangePtr pITextRangeLastLine; 
	CComBSTR bstrLineText,
			 bstrLoc;
	OLECHAR *pText;
	long nFirstLine,
		 nLastLine,
	     nTotalChars = 0,
		 nLineLength;

	USES_CONVERSION;

	// clear vecString if not empty
	if(!vecString.empty())
		vecString.clear();

	// initialize pITextRange / pITextRangeLastLine and nFirstLine / nLastLine
	if(!(nFlags & SFF_SELECTION))
	{   // if the whole text is retrieved
		// initialize range to contain extend over the first line
		nFirstLine = 1;
		nLastLine = m_RichEditCtrl.SendMessage(EM_GETLINECOUNT);
		InitITextRange(pITextRange, 1);
		InitITextRange(pITextRangeLastLine, nLastLine);
	}
	else
	{   // selection only
		GetSelectionRangeDuplicate(pITextRange);
		GetSelectionRangeDuplicate(pITextRangeLastLine);
		
		// retrieve line numbers of last and first line for further processing
		// first line
		pITextRange->Collapse(tomTrue);
		pITextRange->GetIndex(tomLine, &nFirstLine);		
		
		// last line
		pITextRangeLastLine->Collapse(tomFalse);
		pITextRangeLastLine->GetIndex(tomLine, &nLastLine);

		// intialize ranges
		if(nFirstLine == nLastLine)
		{
			GetSelectionRangeDuplicate(pITextRange);
		}
		else
		{
			pITextRange->MoveEnd(tomLine, 1, NULL);
			pITextRangeLastLine->MoveStart(tomLine, 1, NULL);
		}
	
	}

	// Special handling for first line: 
	// necessary for the treatement of selected text,
	// only that part of the line starting at position of selection start has to be considered
	pITextRange->GetText(&bstrLineText);
	
	pText = bstrLineText;
	nLineLength = bstrLineText.Length();
	if(nLineLength > 0)
		pText +=(nLineLength - 1);

	if(bIncludeLineBreaks)
	{
		if(m_bConvertWordWrapToLineBreak != VARIANT_FALSE)
		{
			// Check hard line break(->'\r')
			if(nLineLength > 0 && *pText == L'\r')
			{
				bstrLineText.Append("\n");
			}
			else
			{
				bstrLineText.Append("\r\n");
			}
		}
	}
	else
	{   // if no linebreaks(\r) should be included
		// Check hard line break(->'\r')
		if(nLineLength > 0 && *pText == L'\r')
		{
			bstrLoc.Empty();
			bstrLoc.Append(bstrLineText,bstrLineText.Length()-1);
			bstrLineText = bstrLoc;
		}
	}
	
	// finally append line text to bstrText
	if(nFirstLine == nLastLine)
	{   // already done!

		// if not selected text is treatet exclude final word break \r\n
		if(!(nFlags & SFF_SELECTION))
		{ 
			if(bIncludeLineBreaks)
			{
				nLineLength = bstrLineText.Length();

				if(m_bConvertWordWrapToLineBreak != VARIANT_FALSE)
				{
					// special handling for last line to exclude final \r\n
					bstrLoc.Empty();
					bstrLoc.Append(bstrLineText,nLineLength - 2);
					bstrLineText = bstrLoc;
				}
				else
				{
					pText = bstrLineText;

					if(nLineLength > 0)
						pText +=(nLineLength - 1);

					if(nLineLength > 0 && *pText == L'\r')
					{
						// special handling for last line to exclude final \r
						bstrLoc.Empty();
						bstrLoc.Append(bstrLineText,nLineLength - 1);
						bstrLineText = bstrLoc;
					}
				}
			}
		}

		if(bstrLineText.Length() != 0)
			vecString.push_back(OLE2T(bstrLineText));
		else
			vecString.push_back(_T(""));

		return bstrLineText.Length();
	}
	else
	{
		if(bstrLineText.Length() != 0)
			vecString.push_back(OLE2T(bstrLineText));
		else
			vecString.push_back(_T(""));

		itFirst = vecString.begin();
		nTotalChars = bstrLineText.Length();
	}

	for(long nLine=nFirstLine+1; nLine<nLastLine; nLine++)
	{
		pITextRange->SetIndex(tomLine, nLine, tomExtend);
		pITextRange->GetText(&bstrLineText);

		pText = bstrLineText;
		nLineLength = bstrLineText.Length();
		if(nLineLength > 0)
			pText +=(nLineLength - 1);

		if(bIncludeLineBreaks)
		{
			if(m_bConvertWordWrapToLineBreak != VARIANT_FALSE)
			{
				// Check hard line break(->'\r')
				if(nLineLength > 0 && *pText == L'\r')
				{
					bstrLineText.Append("\n");
				}
				else
				{
					bstrLineText.Append("\r\n");
				}
			}
		}
		else
		{   // if no linebreaks(\r) should be included
			// Check hard line break(->'\r')
			if(nLineLength > 0 && *pText == L'\r')
			{
				bstrLoc.Empty();
				bstrLoc.Append(bstrLineText,nLineLength-1);
				bstrLineText = bstrLoc;
			}
		}
		
		if(bGatherOutputInFirstElelemt)
		{
			if(bstrLineText.Length() > 0)
			{
				itFirst->append(OLE2T(bstrLineText));
			}
		}
		else
		{
			if(bstrLineText.Length() > 0)
				vecString.push_back(OLE2T(bstrLineText));
			else
				vecString.push_back(_T(""));
			
			nTotalChars += bstrLineText.Length();
		}
	}

	// Special handling for last line
	// necessary for the treatement of selected text,
	// only that part of the line up to position of selection end has to be considered
	pITextRangeLastLine->GetText(&bstrLineText);
	
	// if not selected text is treatet exclude final \r
	if(!(nFlags & SFF_SELECTION))
	{ 
		pText = bstrLineText;
		nLineLength = bstrLineText.Length();

		if(nLineLength > 0)
			pText +=(nLineLength - 1);

		if(nLineLength > 0 && *pText == L'\r')
		{
			// special handling for last line to exclude final \r
			bstrLoc.Empty();
			bstrLoc.Append(bstrLineText,nLineLength - 1);
			bstrLineText = bstrLoc;
		}
	}

	if(bGatherOutputInFirstElelemt)
	{
		if(bstrLineText.Length() != 0)
		{
			itFirst->append(OLE2T(bstrLineText));
			nTotalChars = itFirst->length();
		}
	}
	else
	{
		if(bstrLineText.Length() != 0)
			vecString.push_back(OLE2T(bstrLineText));
		else
			vecString.push_back(_T(""));

		nTotalChars += bstrLineText.Length();
	}
	

	return nTotalChars;
}

bool CSQLStudioInput::InternalGetModifyStatus(void)
{	

	if(!m_RichEditCtrl.SendMessage(EM_GETMODIFY))
	{   // text has the status: not modifed
		if(m_bTextModifiedStatus)
		{   // status has changed
			m_bTextModifiedStatus = false;

			// fire change event
			Fire_ModifiedStatusChanged(VARIANT_FALSE);
		}
		return false;
	}
	else
	{   // text has the status: modifed
		if(!m_bTextModifiedStatus)
		{   // status has changed
			m_bTextModifiedStatus = true;

			// fire change event
			Fire_ModifiedStatusChanged(VARIANT_TRUE);

		}
		return true;
	}
}

bool CSQLStudioInput::InternalSetModifyStatus(bool bNewStatus)
{	

	if(bNewStatus != m_bTextModifiedStatus)
	{
		m_bTextModifiedStatus = bNewStatus;

		// set modify status
		m_RichEditCtrl.SendMessage(EM_SETMODIFY, m_bTextModifiedStatus);

		// fire change event
		Fire_ModifiedStatusChanged(m_bTextModifiedStatus);

		return true;
	}

	return false;
}

long CSQLStudioInput::CheckLineFromTo(long &nLineFrom, long &nLineTo)
{	

	int nLineCount = m_RichEditCtrl.SendMessage(EM_GETLINECOUNT);

	if(nLineFrom > nLineTo)
		std::swap(nLineFrom, nLineTo);

	if(nLineFrom < 1)
		nLineFrom = 1;

	if(nLineTo < 1)
		nLineTo = 1;

	if(nLineFrom > nLineCount)
		nLineFrom = nLineCount;

	if(nLineTo > nLineCount)
		nLineTo = nLineCount;

	// return total number of lines
	return nLineCount;
}

long CSQLStudioInput::CheckLinePos(long nLine, long &nCol)
{	

	// Check parameters
	int nLineIndex = m_RichEditCtrl.SendMessage(EM_LINEINDEX, nLine-1);
	int nLineLength = m_RichEditCtrl.SendMessage(EM_LINELENGTH, nLineIndex);

	if(nCol < 1)
		nCol = 1;

	if(nCol > nLineLength+1)
		nCol = nLineLength+1;

	// return number of characters in that line
	return nLineIndex;
}

void CSQLStudioInput::CheckLinePosFromTo(long &nLineFrom, long &nColFrom, long &nLineTo, long &nColTo)
{	

	const long nOriginalLineFrom = nLineFrom;
	const long nOriginalLineTo = nLineTo;
	long nLastLine;
	long nCharsInLine;

	// get number of last line and check values of lines
	nLastLine = CheckLineFromTo(nLineFrom, nLineTo);

	if((nOriginalLineFrom == nLineFrom) ||(nLineFrom != 1))
		// if checking the "from"-line resulted in no change check 
		// or the new line is not the first line
		// additionally check the "from" position 
	{
		CheckLinePos(nLineFrom, nColFrom);
	}
	else
		// changed line is the first line, 
		// we assume the user wants to get the first position of the whole text
	{
		 nColFrom = 1;
	}
		
	
	nCharsInLine = 	CheckLinePos(nLineTo, nColTo);
	if((nOriginalLineTo != nLineTo) &&(nLineTo == nLastLine))
	{
		// if check lead to change of "to"-line and is now the last line
		// we assume that the user wants to get the last position of the whole text
		nColTo = nCharsInLine;
	}

	if((nLineFrom == nLineTo) &&(nColFrom>nColTo))
		std::swap(nColFrom, nColTo);
}

void CSQLStudioInput::CheckIndexFromTo(long &nIndexFrom,long &nIndexTo)
{	

	// has a special meaning(whole text) for several routines
	if(nIndexFrom == 0 && nIndexTo == -1)  return;

	if(nIndexFrom > nIndexTo) std::swap(nIndexFrom, nIndexTo);

	if(nIndexFrom < 0) nIndexFrom = 0;
	if(nIndexTo < 0) nIndexTo = 0;

}

// TOM adoption would be slower
void CSQLStudioInput::CharIndexToLinePos(long nCharIndex, long &nLine, long &nCol)
{	

	nLine = m_RichEditCtrl.SendMessage(EM_EXLINEFROMCHAR, 0, nCharIndex);

	LONG nLineIndex = m_RichEditCtrl.SendMessage(EM_LINEINDEX, nLine);

	nCol = nCharIndex - nLineIndex + 1; // one based index

	nLine++; // one based index
}

// TOM adoption would be slower
long CSQLStudioInput::LinePosToCharIndex(long nLine, long nCol)
{

	// richedit uses a zero based lineindex we deal with an one based
	LONG nLineIndex = m_RichEditCtrl.SendMessage(EM_LINEINDEX, nLine-1);

	return nLineIndex+nCol-1;
}


void CSQLStudioInput::ArrangeWindows(void)
{

	// The toolbar and status bar resize automatically to the top/bottom 
	// of the window when they receive WM_SIZE messages
	m_ToolBar.SendMessage(WM_SIZE);
	m_StatusBar.SendMessage(WM_SIZE);

	RECT rcStatusBar;
	m_StatusBar.GetWindowRect(&rcStatusBar);

	RECT rcToolBar;
	m_ToolBar.GetWindowRect(&rcToolBar);

	RECT rcRichEdit;
	GetClientRect(&rcRichEdit);

	rcRichEdit.left = 0;

	if(m_bToolBarVisible != VARIANT_FALSE)
		rcRichEdit.top +=(rcToolBar.bottom - rcToolBar.top);

	if(m_bStatusBarVisible != VARIANT_FALSE)
	{
		rcRichEdit.bottom -=(rcStatusBar.bottom - rcStatusBar.top);

		// Set the parts of the status bar
		int nWidthStatusBar = rcStatusBar.right-rcStatusBar.left;
		int aStatusBarPartsPos[] = {nWidthStatusBar-445,nWidthStatusBar-430,nWidthStatusBar-400, nWidthStatusBar-200, nWidthStatusBar};
		m_StatusBar.SendMessage(SB_SETPARTS, sizeof(aStatusBarPartsPos)/sizeof(aStatusBarPartsPos[0]), reinterpret_cast<LPARAM>(aStatusBarPartsPos));
	}

    m_RichEditCtrl.MoveWindow(&rcRichEdit);

	UpdateStatusBar();
}


long CSQLStudioInput::GetSelectedLineCount(void)
{

	CHARRANGE oCHARRANGE;
	m_RichEditCtrl.SendMessage(EM_EXGETSEL, 0, reinterpret_cast<LPARAM>(&oCHARRANGE));

	LONG nLineFrom = m_RichEditCtrl.SendMessage(EM_EXLINEFROMCHAR, 0, oCHARRANGE.cpMin);
	LONG nLineTo = m_RichEditCtrl.SendMessage(EM_EXLINEFROMCHAR, 0, oCHARRANGE.cpMax);

	return(nLineTo-nLineFrom+1);
}

long CSQLStudioInput::GetSelectedWordCount(void)
{	

	ITextRangePtr pITextRangeSelStart;
	GetSelectionRangeDuplicate(pITextRangeSelStart);
	ITextRangePtr pITextRangeSelEnd;
	GetSelectionRangeDuplicate(pITextRangeSelEnd); 
	long nWordIndexStart, nWordIndexEnd;

	// retrieve word index at start of selection
	pITextRangeSelStart->GetIndex(tomWord, &nWordIndexStart);

	// retrieve word index at end of selection
	pITextRangeSelEnd->Collapse(tomFalse);
	pITextRangeSelEnd->GetIndex(tomWord, &nWordIndexEnd);

	return(nWordIndexEnd-nWordIndexStart+1);
}


bool CSQLStudioInput::InternalFind(long nCount, long nFlags, ITextRangePtr &pITextRange)
{
	
	long nTextLength;

	pITextRange->FindText(m_bstrFindWhat, nCount, nFlags, &nTextLength);

	if(nTextLength != 0)
		return true;

	return false;
}


bool CSQLStudioInput::FindNext(ITextRangePtr &pITextRange, bool bRestartAtTop)
{	

	bool bFound = false;
	long nFlags = 0;
	
	if(m_oFINDREPLACE.Flags & FR_WHOLEWORD)
		nFlags += tomMatchWord;
	if(m_oFINDREPLACE.Flags & FR_MATCHCASE)
		nFlags += tomMatchCase;
	
	// Check if search should not take place in selection
	if(m_bFindReplaceInSelection == false)
	{
		if(pITextRange == NULL)
		{
			// just to get a starting point(start of selction range) for the search
			GetSelectionRangeDuplicate(pITextRange);

			long nSelType;
			m_pITextSelection->GetType(&nSelType);
			// to guarantee that search will loop over the whole text
			// even for the case that the replace string contents the search string
			if(nSelType == tomSelectionNormal){

				pITextRange->MoveStart(tomCharacter, 1, NULL);
			}
	
			// extend range to the end of the document
			pITextRange->MoveEnd(tomStory, 1, NULL);
		}

		// search from the insertion point to the end of the document
		bFound = InternalFind(0, nFlags, pITextRange);
		
		if(bFound == false && bRestartAtTop == true)
		{
			// restart search from the beginning of the document
			// especially extend range in order to find strings being right 
			// at the original caret position
			long nOldStart;
			pITextRange->GetStart(&nOldStart);
			pITextRange->SetRange(0, nOldStart + m_bstrFindWhat.Length() - 1);

			// finally search within that range
			bFound = InternalFind(0, nFlags, pITextRange);
		}
	}
	else // a selection exists: search only within selected area
	{
		if( pITextRange == NULL)
			GetDuplicate(m_pITextRangeFindReplace, pITextRange);

		bFound = InternalFind(0, nFlags, pITextRange);

		// InternalFind doesn't work as expected if the range itself matches the search string.
		// Therefore it has to be done by hand
		if(!bFound)
		{
			long nStart, nEnd;
			pITextRange->GetStart(&nStart);
			pITextRange->GetEnd(&nEnd);
			if(nEnd-nStart == static_cast<long>(m_bstrFindWhat.Length()))
			{
				CComBSTR bstrText;
				pITextRange->GetText(&bstrText);
				if(_wcsicmp(bstrText, m_bstrFindWhat) == 0)
				{
					if(m_oFINDREPLACE.Flags & FR_MATCHCASE)
						bFound =(wcscmp(bstrText, m_bstrFindWhat) == 0);
					else 
						bFound = true;

					if(bFound &&(m_oFINDREPLACE.Flags & FR_WHOLEWORD))
					{
						ITextRangePtr pITextRangeWord = GetWord(pITextRange);
						if(pITextRangeWord)
						{
							long nIsEqual;
							pITextRangeWord->IsEqual(pITextRange,&nIsEqual);
							if(nIsEqual != tomTrue)
							  bFound = false;
						}
						else // GetWord found no word but a delimiter
							bFound = false;
					}
				}
			}
		}
	}

	return bFound;
}

/*!-------------------------------------------------------------------------
   findNextWithBeep
 --------------------------------------------------------------------------*/
bool CSQLStudioInput::findNextWithBeep(ITextRangePtr &pITextRange)
{	

	// find the text
	if(FindNext(pITextRange, true)){

		if(pITextRange){

			// if anything is found: select the string found and scroll it into the view
			pITextRange->Select();
			pITextRange->ScrollIntoView(tomStart);
			return true;
		}
	}
	else
		::MessageBeep(0xFFFFFFFF);

	return false;
}

/*!-------------------------------------------------------------------------
     findNextWithBeep
--------------------------------------------------------------------------*/
bool CSQLStudioInput::findNextWithBeep()
{	
	
	ITextRangePtr pITextRange;

	return findNextWithBeep(pITextRange);
}

/*!-------------------------------------------------------------------------
    replace    
--------------------------------------------------------------------------*/
bool CSQLStudioInput::replace(ITextRangePtr &pITextRange)
{

	// get the selected text
	CComBSTR bstrText;
	m_pITextSelection->GetText(&bstrText);
	if(!bstrText){

		if(FindNext(pITextRange, true) && pITextRange){

			pITextRange->Select();
			pITextRange->ScrollIntoView(tomStart);
			return true;
		}
		else
			return false;
	}

	bool bFound;

	// for propper "match case": perform the respective comparision
	if(m_oFINDREPLACE.Flags & FR_MATCHCASE)
		bFound =(wcscmp(bstrText, m_bstrFindWhat) == 0);
	else
		bFound =(_wcsicmp(bstrText, m_bstrFindWhat) == 0);

	if(bFound)
	{
		// replace the selection
		m_pITextSelection->SetText(m_bstrReplaceWith);
		m_pITextSelection->Collapse(tomFalse);

		if(FindNext(pITextRange, true) && pITextRange){

			pITextRange->Select();
			pITextRange->ScrollIntoView(tomStart);
			return true;
		}
		return true;
	}

	return false;
}

bool CSQLStudioInput::ReplaceAndFindNextWithBeep(ITextRangePtr &pITextRange)
{

	if(!replace(pITextRange)) ::MessageBeep(0xFFFFFFFF);
	
	return false;
}

void CSQLStudioInput::ShowColorRefDialog(COLORREF &Color)
{
		
	CHOOSECOLOR pcc = {0};   // initialization data

	pcc.lStructSize = sizeof(CHOOSECOLOR);
	pcc.hwndOwner = m_RichEditCtrl.m_hWnd;
	pcc.hInstance = NULL;
	pcc.Flags = CC_RGBINIT;
	pcc.rgbResult = Color;
	pcc.lpCustColors = &Color;

	if(!ChooseColor(&pcc))	return;
	
	Color = pcc.rgbResult;

	StartTimerHighlightKeyAndCom();

	Fire_ColorChange();

}

/*!----------------------------------------------------------------------
   ShowChooseFontDialog   
-------------------------------------------------------------------------*/
void CSQLStudioInput::ShowChooseFontDialog(void)
{

	LOGFONT oLF;
	CHOOSEFONT oCSF = {0};
	CHARFORMAT2 oCF;
	HRESULT hr = E_FAIL;
	HFONT	hFont;
	
	CComBSTR	bstrFontName;
	int	bFontBold;
	int	bFontItalic;
	int	bFontStrikeThrough;
	int	bFontUnderline;
	CY	lFontSize;

	CComQIPtr<IFont,&IID_IFont> pFont(m_pFont);

	if(pFont == NULL) return;

	//init	
	oCSF.lStructSize = sizeof(CHOOSEFONT);
	oCF.cbSize = sizeof(CHARFORMAT2);	
	
	(void) m_RichEditCtrl.SendMessage(EM_GETCHARFORMAT,(WPARAM) VARIANT_TRUE,(LPARAM) &oCF);		
	
	
	hr = pFont->get_hFont(&hFont);	
	pFont->get_Size(&lFontSize);

	// the get_hFont method returns a temporary font
	if(SUCCEEDED(hr)) ::GetObject(hFont, sizeof(LOGFONT), &oLF);
	

	oCSF.lpLogFont = &oLF;
	oCSF.hwndOwner = m_RichEditCtrl.m_hWnd;
	oCSF.hDC = 0;	
	oCSF.Flags =  CF_SCREENFONTS | CF_INITTOLOGFONTSTRUCT | CF_LIMITSIZE | CF_EFFECTS | CF_ENABLETEMPLATE;
	oCSF.lpTemplateName = MAKEINTRESOURCE(FORMATDLGORD31);
	oCSF.hInstance = _Module.GetResourceInstance();	
	oCSF.nSizeMin = 1;
	oCSF.nSizeMax = yHeightCharPtsMost;
	oCSF.rgbColors = m_crTextEditable;
	oCSF.lpszStyle = NULL;
	oCSF.nFontType = REGULAR_FONTTYPE | SCREEN_FONTTYPE;
	
	USES_CONVERSION;
	if(!ChooseFont(&oCSF))  return;
	

	// Fill out the CHARFORMAT2 structure to set character effects.
	// don't change read-only bit
	oCF.dwMask =  CFM_SIZE | CFM_FACE | CFM_CHARSET | CFM_BOLD | CFM_ITALIC | CFM_UNDERLINE | 
					 CFM_STRIKEOUT | CFM_COLOR ;
	oCF.yHeight =(LONG) oCSF.iPointSize * 2;
	
	oCF.dwEffects = 0;
	if(oLF.lfWeight == FW_BOLD){

		oCF.dwEffects |= CFE_BOLD;
		bFontBold = TRUE;
	}
	else
		bFontBold = FALSE;
	
	if(oLF.lfItalic){

		oCF.dwEffects |= CFE_ITALIC;
		bFontItalic= TRUE;
	}
	else
		bFontItalic = FALSE;
	
	if(oLF.lfUnderline){

		oCF.dwEffects |= CFE_UNDERLINE;
		bFontUnderline = TRUE;
	}
	else
		bFontUnderline = FALSE;
	
	if(oLF.lfStrikeOut){

		oCF.dwEffects |= CFE_STRIKEOUT;
		bFontStrikeThrough = TRUE;
	}
	else
		bFontStrikeThrough = FALSE;
	
	
	oCF.bCharSet = oLF.lfCharSet;
	oCF.bPitchAndFamily = oLF.lfPitchAndFamily;
	_tcscpy(oCF.szFaceName, oLF.lfFaceName);

	bstrFontName = oLF.lfFaceName;

	lFontSize.Lo = oCF.yHeight*500;
	lFontSize.Hi = 0;

	
	FONTDESC fd;
	fd.cbSizeofstruct = sizeof(FONTDESC);
	fd.lpstrName = bstrFontName;
	fd.cySize = lFontSize;
	fd.sWeight =(short)oLF.lfWeight;
	fd.sCharset = oLF.lfCharSet;
	fd.fItalic = oLF.lfItalic ;
	fd.fUnderline = oLF.lfUnderline;
	fd.fStrikethrough= oLF.lfStrikeOut;

	pFont.Release();
	m_pFont.Release();

	OleCreateFontIndirect(&fd,IID_IFontDisp,(void**)&m_pFont);
	
	
	if(m_RichEditCtrl.m_hWnd != NULL){

		InternalSetSelectionIndexes(0,0);
		m_RichEditCtrl.SendMessage(EM_SETCHARFORMAT, SCF_ALL,(LPARAM) &oCF);
	}

	Fire_FontChange();
}

void CSQLStudioInput::ShowFindReplaceDialog(void)
{

	if(m_WndFindReplace == NULL)
	{
		// Initialize the struct only the first time
		if(m_oFINDREPLACE.lStructSize == 0)
		{
			USES_CONVERSION;
			if(!(m_bstrFindWhat) || m_bstrFindWhat[0] == 13)
				m_strFindWhat[0] = 0;
			else
				_tcscpy(m_strFindWhat, OLE2T(m_bstrFindWhat));

			if(!m_bstrReplaceWith)
				m_strReplaceWith[0] = 0;
			else
				_tcscpy(m_strReplaceWith, OLE2T(m_bstrReplaceWith));

			m_oFINDREPLACE.lStructSize = sizeof(FINDREPLACE);
			m_oFINDREPLACE.hwndOwner = m_hWnd;
			m_oFINDREPLACE.lpstrFindWhat = m_strFindWhat;
			m_oFINDREPLACE.lpstrFindWhat[0] = 0;
			m_oFINDREPLACE.lpstrReplaceWith = m_strReplaceWith;
			m_oFINDREPLACE.lpstrReplaceWith[0] = 0;
			m_oFINDREPLACE.wFindWhatLen = sizeof(m_strFindWhat);
			m_oFINDREPLACE.wReplaceWithLen = sizeof(m_strReplaceWith);
		}

		// Delete all flags except FR_MATCHCASE and FR_WHOLEWORD
		m_oFINDREPLACE.Flags &=(FR_MATCHCASE|FR_WHOLEWORD);

		UpdateFindReplaceRange(true);

		// Start the thread which shows the modeless dialog
		// as discussed by MS System Journal July 1999: 
		// multithreaded applications should use _beginthreadex() instead of CreateThread
		// therefore the latter call is replaced
		// m_hThread = ::CreateThread(NULL, 0, ThreadFuncSearchReplace, this, 0, &m_dwThreadID);
		m_hThread = reinterpret_cast<HANDLE>(::_beginthreadex(NULL, 0, reinterpret_cast<unsigned int(__stdcall *)(void *)>(ThreadFuncSearchReplace), this, 0, reinterpret_cast<unsigned int*>(&m_dwThreadID)));

	}
	else
	{
		::BringWindowToTop(m_WndFindReplace);
	}
}

void CSQLStudioInput::UpdateFindReplaceRange(bool bUpdateSearchString)
{	

	bool bUpdateSearchStringInternal = bUpdateSearchString;
	long nSelType;
	m_pITextSelection->GetType(&nSelType);
	// to guarantee that search will loop over the whole text
	// even for the case that the replace string contents the search string

	if(nSelType == tomSelectionNormal){

		// update flag
		m_bFindReplaceInSelection = true;
		// update search range
		GetSelectionRangeDuplicate(m_pITextRangeFindReplace);
		
		// change search string only if selection is in one line 
		// and the number of selected characters is smaller than the possible length of the search string
		if(bUpdateSearchString && GetSelectedLineCount() == 1 && GetNumberOfSelectedCharacters() < sizeof(m_strFindWhat)){

			// get whole selected text as search string
			m_pITextSelection->GetText(&m_bstrFindWhat);
		}
	}
	else{
		// update flag
		m_bFindReplaceInSelection = false;

		if(bUpdateSearchStringInternal){

			ITextRangePtr pITextRange = GetWord(static_cast<IUnknown*>(NULL));

			// get the string
			if(pITextRange)				
				pITextRange->GetText(&m_bstrFindWhat);			
			else
				bUpdateSearchStringInternal = false;			
		}
	}

	if(bUpdateSearchStringInternal == true && m_bstrFindWhat)
	{
		//don't look for LF
		if((m_bstrFindWhat.m_str[0] != 13)){// &&(m_bstrFindWhat.m_str[1] != 0)){

			// finally update search string used in find/replace dialog
			USES_CONVERSION;
			_tcscpy(m_strFindWhat, OLE2T(m_bstrFindWhat));
		}
	}
}

ITextRangePtr CSQLStudioInput::GetWord(const ITextRangePtr &pITextRangeIn)
{
	
	ITextRangePtr pITextRange;
	// initialize pITextRange	
	if(pITextRangeIn)
		GetDuplicate(pITextRangeIn, pITextRange);
	else
		GetSelectionRangeDuplicate(pITextRange);

	// get word close to start of range
	pITextRange->Collapse(tomTrue);
	pITextRange->Expand(tomWord, NULL);  // expand includes trailing delimiters

	long nStart;
	long nEnd;
	pITextRange->GetStart(&nStart);
	pITextRange->GetEnd(&nEnd);
	long nLength = nEnd-nStart;
	
	if(nLength)
	{   // to get rid of the trailing delimiters
		long nChar;
		ITextRangePtr pITextRangeLocal;
		// intialize pITextRangeLocal
		GetDuplicate(pITextRange, pITextRangeLocal);

		pITextRangeLocal->Collapse(tomFalse);
		pITextRangeLocal->MoveEnd(tomCharacter, -1, NULL);

		for(pITextRangeLocal->GetChar(&nChar); 
			 nLength >= 0 && IsStandardWordDelimiter(nChar) ; 
			 --nLength, pITextRangeLocal->GetChar(&nChar))
		{
			pITextRange->MoveEnd(tomCharacter, -1, NULL);
			pITextRangeLocal->MoveEnd(tomCharacter, -1, NULL);
		}
		
	}

	return pITextRange;
}


long CSQLStudioInput::InternalReplaceAll(ITextRangePtr &pITextRangeLoop, bool bRestartAtTop)
{

	// If the control is in readonly mode do not change text
	if(m_bReadonly != VARIANT_FALSE)
		return 0;

	// Check if find and replace text are equal
	if(m_bstrReplaceWith.Length() > 0){

		if(m_oFINDREPLACE.Flags & FR_MATCHCASE){

			if(wcscmp(m_bstrFindWhat, m_bstrReplaceWith) == 0)
				return 0;
		}
		else{

			if(_wcsicmp(m_bstrFindWhat, m_bstrReplaceWith) == 0)
				return 0;
		}
	}


	CDisplayWaitCursor	oDisplayWaitCursor(*this);
	long				nReplaceCount = 0; 
	bool				bInitialRangeLoop = false;
	ITextRangePtr       pITextRangeLatestFound;
	ITextRangePtr		pITextRangeRestartAtTop;
	ITextRangePtr		pITextRangeSearch;
	// intialization ranged to be searched through(pITextRangeSearch) 
	// and if need the range for the restart at the top(pITextRangeRestartAtTop)
	if(pITextRangeLoop){

		bInitialRangeLoop = true;
		GetDuplicate(pITextRangeLoop, pITextRangeSearch);
		if(bRestartAtTop)
			GetDuplicate(pITextRangeLoop, pITextRangeRestartAtTop);
	}
	else{

		GetSelectionRangeDuplicate(pITextRangeSearch);
		pITextRangeSearch->Collapse(tomTrue);
		if(bRestartAtTop)
			GetDuplicate(pITextRangeSearch, pITextRangeRestartAtTop);
	}
	if(bRestartAtTop){

		pITextRangeRestartAtTop->Collapse(tomTrue);
		pITextRangeRestartAtTop->MoveEnd(tomCharacter, m_bstrFindWhat.Length() - 1, NULL);
		pITextRangeRestartAtTop->MoveStart(tomStory, -1, NULL);
	}

	while(FindNext(pITextRangeLoop, false)){

		if(pITextRangeLoop){

			pITextRangeLoop->SetText(m_bstrReplaceWith);
			// update the previous range(that only marks the currently found and replaced range)
			GetDuplicate(pITextRangeLoop, pITextRangeLatestFound);
			// set range start right behind the replaced string
			pITextRangeLoop->MoveStart(tomCharacter, m_bstrReplaceWith.Length(), NULL);
		}
		
		// adjust the search range: start the new start range right after the last position found
		if(m_bFindReplaceInSelection == true){

			if(AdoptFindReplaceRange(pITextRangeLoop) == false){

				// if the whole find/replace range was searched through

				// adjust the range to the one previously found
				if(pITextRangeLatestFound)
					GetDuplicate(pITextRangeLatestFound, pITextRangeLoop);				

				break;
			}	// when search and replace should be restricted to selection	
		}
		else
		{
			if(pITextRangeSearch)
			{		
				long nIsEqual;				
				pITextRangeSearch->IsEqual(pITextRangeLoop, &nIsEqual);

				if(nIsEqual == tomTrue)
					break;				
				else
				{
					// extend new search range to the end of the document
					if(bInitialRangeLoop == true)
					{
						long nEnd;
  						pITextRangeLoop->GetEnd(&nEnd);
  						pITextRangeSearch->SetStart(nEnd);
  						GetDuplicate(pITextRangeSearch, pITextRangeLoop);
					}
					else
					{
						pITextRangeLoop->MoveEnd(tomStory, 1, NULL);
					}
					// adopt the new range still to be searched through
					GetDuplicate(pITextRangeLoop, pITextRangeSearch);
				}
			}
		}
	}

	// if search and replace should restart from top
	if(pITextRangeRestartAtTop && m_bFindReplaceInSelection == false)
	{
		long nReplacedFromTop = InternalReplaceAll(pITextRangeRestartAtTop, false);
		// if text parts were replaced: update the range for the latest found range
		if(nReplacedFromTop > 0){

			GetDuplicate(pITextRangeRestartAtTop, pITextRangeLatestFound);
			nReplaceCount += nReplacedFromTop;
		}
	}

	// restore the latest found range as output parameter
	if(pITextRangeLatestFound) 
		GetDuplicate(pITextRangeLatestFound, pITextRangeLoop);

	return nReplaceCount;
}

bool CSQLStudioInput::AdoptFindReplaceRange(ITextRangePtr &pITextRange)
{	

	if(m_bFindReplaceInSelection == true && pITextRange)
	{
		long nEnd, nEndSelection;

		// adjust the search ranges 
		pITextRange->GetEnd(&nEnd);
		m_pITextRangeFindReplace->GetEnd(&nEndSelection);
		if(nEndSelection > nEnd){

			m_pITextRangeFindReplace->SetStart(nEnd);
			GetDuplicate(m_pITextRangeFindReplace, pITextRange);
			return true;
		}
	}
	return false;
}


void CSQLStudioInput::HighlightVisibleComments(void)
{
	if(m_bHighlightKeyAndCom == VARIANT_FALSE) return;
	if((!m_strCommentPrefix) &&(!m_strCommentPrefix2))  return;
	if(m_bInsideHighlightKeyAndCom == true) return;
	if(m_RichEditCtrl.m_hWnd == NULL) return;		
	
	
	CSaveScrollPosition             oSaveScrollPosition(*this);
 	CInsideHighlightKeyAndCom		oInsideHighlightKeyAndCom(*this);
	
	long nLineFirst;
	long nLineLast;
	long nNumberOfLines = GetFirstAndLastVisibleLine(nLineFirst, nLineLast);
	long nNumberOfVisibleLines = nLineLast - nLineFirst + 2; // Add 2 to reach partly visilble lines

	// Parse the page before
	nLineFirst -= nNumberOfVisibleLines;
	if(nLineFirst < 1) nLineFirst = 1;

	// Parse the page after
	nLineLast += nNumberOfVisibleLines;
	if(nLineLast > nNumberOfLines) nLineLast = nNumberOfLines;

	// initialize pITextRange
	ITextRangePtr pITextRange;
	InitITextRange(pITextRange, nLineFirst);
	
	for(long nLine = nLineFirst; nLine <= nLineLast; nLine++){

		// select the whole line
		pITextRange->SetIndex(tomLine, nLine, tomExtend);		
		
		CComBSTR bstrLineText;	
		pITextRange->GetText(&bstrLineText);		
		
		if(IsComment(pITextRange))
			InternalHighlight(pITextRange,(!VARIANT_FALSE));
		else
			InternalHighlight(pITextRange, VARIANT_FALSE);					
		
	}
	//reformat areas 
	pITextRange->SetIndex(tomLine, nLineFirst, tomExtend);
	if(nLineFirst != nLineLast)
		pITextRange->MoveEnd(tomLine, nLineLast-nLineFirst, NULL);

		
}



long CSQLStudioInput::GetFirstAndLastVisibleLine(long &nLineFirst, long &nLineLast)
{

	nLineFirst = m_RichEditCtrl.SendMessage(EM_GETFIRSTVISIBLELINE) + 1;
	
	ITextRangePtr pITextRange;
	InitITextRange(pITextRange, nLineFirst);

	pITextRange->EndOf(tomWindow, tomMove, NULL);
	pITextRange->GetIndex(tomLine, &nLineLast);

	long nLineCount = m_RichEditCtrl.SendMessage(EM_GETLINECOUNT);	

	return nLineCount;
}

CComBSTR CSQLStudioInput::InternalGetLineText(long nLine, bool nAlwaysDeterminWithLinebreak)
{
	
	CComBSTR bstrLineText;
	ITextRangePtr pITextRange;
	InitITextRange(pITextRange, nLine);

	pITextRange->GetText(&bstrLineText);

	// Remove CR/LF
	OLECHAR *pText = bstrLineText;
	if(bstrLineText.Length() > 0)
		pText +=(bstrLineText.Length() - 1);

	if(*pText == L'\r')
	{
		CComBSTR bstrLocal(bstrLineText.Length() - 1, bstrLineText);
		bstrLineText = bstrLocal;
	}

	if(nAlwaysDeterminWithLinebreak) // include linebreaks
	{
		bstrLineText.Append("\r\n");
	}

	return bstrLineText;
}

LPCTSTR CSQLStudioInput::GetDisplayText(long nTextID)
{
	return m_mapDisplayText[nTextID].c_str();
}

void CSQLStudioInput::FillDisplayMapFromResource(void)
{	

	FillDisplayMapFromResource(ID_CUT);
	FillDisplayMapFromResource(ID_COPY);
	FillDisplayMapFromResource(ID_PASTE);
	FillDisplayMapFromResource(ID_UNDO);
	FillDisplayMapFromResource(ID_REDO);
	FillDisplayMapFromResource(ID_COMMENT);
	FillDisplayMapFromResource(ID_UNCOMMENT);
	FillDisplayMapFromResource(ID_FIND_REPLACE);
	FillDisplayMapFromResource(ID_FIND_NEXT);
	FillDisplayMapFromResource(ID_IMPORT);
	FillDisplayMapFromResource(ID_EXPORT);
	FillDisplayMapFromResource(ID_LINE_COL);
	FillDisplayMapFromResource(ID_FROM_LINE_COL_TO_LINE_COL);
	FillDisplayMapFromResource(ID_LINE_FROM_TO_OF);
	FillDisplayMapFromResource(ID_MODIFIED_TRUE);
	FillDisplayMapFromResource(ID_MODIFIED_FALSE);
	FillDisplayMapFromResource(ID_OVERWRITE);
	FillDisplayMapFromResource(ID_INSERT);
	FillDisplayMapFromResource(ID_CHOOSEFONT);
	FillDisplayMapFromResource(ID_PROPERTIES);
	FillDisplayMapFromResource(ID_FORTEXTEDIT);	
	FillDisplayMapFromResource(ID_FORHIGHLIGHT);
	
}

void CSQLStudioInput::FillDisplayMapFromResource(long nTextID)
{	
	CComBSTR strDisplayText;

	//_Module.GetResourceInstance(),
	if(strDisplayText.LoadString(nTextID) == true){

		USES_CONVERSION;
		m_mapDisplayText[nTextID] = OLE2T(strDisplayText.Detach());
		strDisplayText.Empty();
	}
}


void CSQLStudioInput::InternalSelectLines(long &nLineFrom, long &nLineTo)
{

	CheckLineFromTo(nLineFrom, nLineTo);
	m_pITextSelection->SetIndex(tomLine, nLineFrom, tomExtend);
	if(nLineFrom != nLineTo)
		m_pITextSelection->MoveEnd(tomLine, nLineTo-nLineFrom, NULL);
}

void CSQLStudioInput::StartTimerHighlightKeyAndCom(void)
{

	KillTimer(ID_TIMER_HIGHLIGHT_COMMENTS);
	SetTimer(ID_TIMER_HIGHLIGHT_COMMENTS, TIME_HIGHLIGHT_COMMENTS);
}

void CSQLStudioInput::StartTimerUpdateToolbar(void)
{
	KillTimer(ID_TIMER_UPDATE_TOOLBAR);
	SetTimer(ID_TIMER_UPDATE_TOOLBAR, TIME_UPDATE_TOOLBAR);
}

void CSQLStudioInput::InternalSetSelectionPos(long nLineFrom, long nColFrom, long nLineTo, long nColTo)
{
	
	CheckLinePosFromTo(nLineFrom, nColFrom, nLineTo, nColTo);
	InternalSetSelectionIndexes(LinePosToCharIndex(nLineFrom, nColFrom), 
								LinePosToCharIndex(nLineTo, nColTo));
}

void CSQLStudioInput::InternalSetSelectionIndexes(long nIndexFrom, long nIndexTo)
{

	CheckIndexFromTo(nIndexFrom, nIndexTo);

	m_pITextSelection->SetRange(nIndexFrom, nIndexTo);
}

void CSQLStudioInput::InternalHighlight(const ITextRangePtr &pITextRange, VARIANT_BOOL bHighlight)
{

	CSaveModifyStatus  oSaveModifyStatus(*this);
	CSuspendUndo	   oSuspendUndo(*this);
	
	ITextFontPtr	pITextFont;

	// initialize font interface pointer
	if(pITextRange == NULL)
		m_pITextSelection->GetFont(&pITextFont);
	else    
		pITextRange->GetFont(&pITextFont);

	if(bHighlight != VARIANT_FALSE)

		pITextFont->SetForeColor(m_crTextHighlight);	
	else
		InternalCheckRangeColoring(pITextRange);		
	
}

/*!----------------------------------------------------------------------
   InternalCheckRangeColoring   
-------------------------------------------------------------------------*/
void CSQLStudioInput::InternalCheckRangeColoring(const ITextRangePtr &pITextRange)
{

	if((m_strCommentPrefix.Length() < 1) &&(m_strCommentPrefix2.Length() < 1)) return;

	BSTR pLine = NULL;
	
	pITextRange->GetText(&pLine);
	if(pLine == NULL) return;
	
	int nPos		= 0;
	long nTextStart = 0;

	bool dQuoteClose = true;
	bool sQuoteClose = true;
	bool bIsComment = false;

	
	ITextFontPtr	pITextFont;
	ITextRangePtr	pIKeyWordRange;		
		
	pITextRange->GetDuplicate(&pIKeyWordRange);

	// initial set klone to m_crTextEditable 
	pIKeyWordRange->GetFont(&pITextFont);			
	if(pITextFont){
		
		pITextFont->SetForeColor(m_crTextEditable);
		if(pITextFont)	 pITextFont.Release();		
	}

	pIKeyWordRange->GetStart(&nTextStart);		
	
	USES_CONVERSION;
	TCHAR* pLineStart = NULL;	
	pLineStart = OLE2T(pLine);

	CMyComBSTR strComment = OLE2T(m_strCommentPrefix);
	CMyComBSTR strComment2 = OLE2T(m_strCommentPrefix2);

	TCHAR ch = 0;
	TCHAR *pNewStart;
	TCHAR *pPtr;
	pNewStart = pPtr = pLineStart;

	TCHAR* pSymbolStart = NULL;

	long nLineFirst = 0;
	get_FirstVisibleLine(&nLineFirst);

	// create vector for ranges
	CITextRanges *pvecColor = new CITextRanges(*this);
	
	ch = *pPtr;		
	if(ch == strComment[0] || ch == strComment2[0]) {

		for(unsigned int i = 1;i< strComment.Length();i++){

			ch = *(pPtr+i);				
				
			if(strComment[i]  == ch)
				bIsComment  = true;			
			else{					
				bIsComment = false;
				break;
			}			
		}

		if(bIsComment){

			pSymbolStart = pPtr;
			pPtr = NULL;			
		}			
		else{
			for(unsigned int i = 1;i< strComment2.Length();i++){

				ch = *(pPtr+i);				
					
				if(strComment2[i]  == ch)
					bIsComment  = true;
				else{					
					bIsComment = false;
					break;
				}			
			}
			if(bIsComment){

				pSymbolStart = pPtr;
				pPtr = NULL;			
			}			
		}
	}		
		

	ch = *pPtr;		
	if(ch == '"') {

		dQuoteClose = !dQuoteClose;
		ch = *(++pPtr);
	}
	if(ch == '\'') {

		if(dQuoteClose){

			sQuoteClose = !sQuoteClose;
			ch = *(++pPtr);
		}
	}

	
	while(*pPtr != 0) {
		
		ch = *pPtr;		

		//ch == '_' || ch == '.' ||
		//select" * from "select" where 'x select y'		
		if(	((!(ch == ' '  ||
					 ch == '\n' ||
					 ch == '\r' ||
					 ch == '('  ||
					 ch == ')'  ||
					 ch == ','  ||
					 ch == '\t')
					)        || 				  				  
				  ch == '"'  || 
				  ch == '\''
				) &&
			 sQuoteClose && 
			 dQuoteClose && 
			(ch != 0))
		{ 

			pSymbolStart = pPtr;

			if(ch == '"')  dQuoteClose = !dQuoteClose;
			
			if(ch == '\'') {

				if(dQuoteClose) sQuoteClose = !sQuoteClose;				
			}			

			do {

				ch = *(++pPtr);
				if(ch == '"')  dQuoteClose = !dQuoteClose;				
				if(ch == '\'') {

					if(dQuoteClose) sQuoteClose = !sQuoteClose;					
				}

			} while((	(!(ch == ' '  ||						    
						    ch == '\r' || 
						    ch == '\n' ||
							ch == '('  ||
							ch == ')'  ||
							ch == ','  ||
						    ch == '\t')
						  )		  || 
						   ch == '"'  ||						   
						   ch == '\'') 
					 && sQuoteClose 
					 && dQuoteClose
					 &&(ch != 0)
					);
			
			//limit pPtr and pSymbolStart
			*pPtr = 0;
			nPos = -1;
			if((!iswcntrl(*pSymbolStart)) && 
				(*pSymbolStart != '"') && 
				(*pSymbolStart != '.') && 
				(*pSymbolStart != ',') && 
				(*pSymbolStart != '=') && 
				(*pSymbolStart != ' ')){
		
				if(pSymbolStart) nPos = IsKeyword(pSymbolStart);				
			}			
			if(nPos < 1) {				

				pSymbolStart = NULL;				
				/*nPos = IsConstant(pSymbolStart);
				if(nPos >= 0) {

				}
				else {
					
				}*/			
			}
			*pPtr = ch;
		} 
		else {

			if(ch == '"')  dQuoteClose = !dQuoteClose;			
			if(ch == '\'') {

				if(dQuoteClose) sQuoteClose = !sQuoteClose;						
			}
			pPtr++;
		}

		if(pSymbolStart != NULL) {

			pIKeyWordRange->SetRange(nTextStart + pSymbolStart - pLineStart,
									 nTextStart + pPtr - pLineStart);		
			pvecColor->AddRange(pIKeyWordRange);
			pNewStart = pPtr;						
		}

		//reinit
		pSymbolStart = NULL;	
	}	

	if(pITextFont)	 pITextFont.Release();
	
	pvecColor->RemoveAll();
	delete pvecColor;

	pIKeyWordRange.Release();	
}




int CSQLStudioInput::IsConstant(LPCTSTR lpszSymbol)
{
	int iPos;
	CMyComBSTR strSymbol;

	strSymbol.Append(" ");
	strSymbol.Append(lpszSymbol);
	strSymbol.Append(" ");

	if(!m_bCaseSensitive) strSymbol.ToLower();

	iPos = m_strConstantsLower.Find(strSymbol);
				
	return iPos;
}

int CSQLStudioInput::IsKeyword(LPCTSTR lpszSymbol)
{

	if (m_strKeywordsLower.Length() < 1 ) return  -1;

	int iPos;
	CMyComBSTR strSymbol((TCHAR*)lpszSymbol);
	CMyComBSTR strSearch(" ");

	strSearch.Append(strSymbol);
	strSearch.Append(" ");
	if(!m_bCaseSensitive) strSearch.ToLower();	
				
	iPos = m_strKeywordsLower.Find(strSearch);
	
	return iPos;
}



void CSQLStudioInput::InternalGetSelectionPos(long * pnLineFrom, long * pnColFrom, long * pnLineTo, long * pnColTo)
{	
	long nIndexFrom, 
		 nIndexTo;

	// retrieve character Indexes of selection
	InternalGetSelectionIndexes(&nIndexFrom, &nIndexTo);

	// translate character indexes to line information
	CharIndexToLinePos(nIndexFrom, *pnLineFrom, *pnColFrom);
	CharIndexToLinePos(nIndexTo, *pnLineTo,	*pnColTo);
}

// the method check for the special selection behavior of the richedit
void CSQLStudioInput::InternalGetSelectedLines(long * pnLineFrom, long * pnLineTo)
{	
	long nColFrom,
		 nColTo; 
	
	InternalGetSelectionPos(pnLineFrom, &nColFrom, pnLineTo, &nColTo);

	// special treatment necessary for selection due to a strange behavior of richedit:
	//     when selecting a complete line(n), richedit extends the selection 
	//     on the beginning(position 1) of the next line(n+1).
	//     Since the indenting acts on the total range of sected lines this would lead
	//     to a not intended indention of line n+1.
	//   and here is the coding
	if(nColTo == 1 &&(*pnLineFrom != *pnLineTo))
	{
		*pnLineTo --;  // the previous line
					 // since always a complete line is indented nColTo has not to be treated 
	}

}

void CSQLStudioInput::InternalGetPoint(long * px, long * py)
{
	m_pITextSelection->GetPoint(tomStart| TA_TOP | TA_LEFT,px,py);	
}


void CSQLStudioInput::InternalGetSelectionIndexes(long * pnIndexFrom, long * pnIndexTo)
{
	m_pITextSelection->GetStart(pnIndexFrom);
	m_pITextSelection->GetEnd(pnIndexTo);
}


bool CSQLStudioInput::InternalIndentOnEnter(void)  
{
	
	long  nLineFrom;
	long  nColFrom;
	long  nLineTo;
	long  nColTo;
	long  nLeadingWhiteSpaces=0;
	long  nSpacesToDelete=0;

	CComBSTR bstrLineText;
	CComBSTR bstrTextToInsert;

	// retrieve selection positions before starting to indent
	InternalGetSelectionPos(&nLineFrom, &nColFrom, &nLineTo, &nColTo);

	ITextRangePtr pITextRange;
	GetSelectionRangeDuplicate(pITextRange);

	// get text of the line with number LineFrom to be processed
	bstrLineText = InternalGetLineText(nLineFrom);
	// determin trailing white spaces
	nLeadingWhiteSpaces = NumberLeadingWhiteSpaces(bstrLineText);

	// mount string to be inserted 
	bstrTextToInsert.Append("\r");
	for(long  nCount = 1; nCount <= nLeadingWhiteSpaces; nCount++)
		bstrTextToInsert.Append(" ");

		
	if(nLeadingWhiteSpaces == 0)
		return false; // let the control process the enter itself

	// to get propper indention LineTo has to be checked for leading whitespaces too
	// get text of the line with number LineTo to be processed
	bstrLineText = InternalGetLineText(nLineTo);
	// get the string starting at position nColTo
	DeleteLeadingCharacters(bstrLineText, nColTo-1);
	// get the number of leading white spaces
	nSpacesToDelete = NumberLeadingWhiteSpaces(bstrLineText);

	if(nSpacesToDelete > 0)
		pITextRange->MoveEnd(tomCharacter, nSpacesToDelete, NULL); // extend selection
	

	// insert/replace string
	pITextRange->SetText(bstrTextToInsert);
	pITextRange->Collapse(tomFalse);
	pITextRange->Select();

	StartTimerHighlightKeyAndCom();

	return true;

}


//parameter nMode==-1 is  designed to perform a special behavior:
//on TAB key pressed without a selected text area acts like
//a normal tabulator. Namely insert spaces at the position of the caret.
//Used in method CSQLStudioInput::OnCharEdit
void CSQLStudioInput::InternalIndent(long nLineFrom, long nColFrom, long nLineTo, long nColTo, int nMode)  
{
	
    long  nLastLineToIndent;
	long  nNewLines = 0;
	CComBSTR bstrLineText;
	CComBSTR bstrTextToIndent;
	ITextRangePtr pITextRange;

	//generate text with spaces for indention
    for(long  nCount = 1; nCount <= m_nSpacesOnIndent; nCount++)
		bstrTextToIndent.Append(" ");

	
	// retrieve total number of lines
	long nTotalNumberOfLines = m_RichEditCtrl.SendMessage(EM_GETLINECOUNT);

	// if nMode==-1 and no selection exists at all insert only spaces
	//    this simulates a ordinary TAB
	if(nMode == -1 && nLineFrom == nLineTo && nColFrom == nColTo){
		
         m_pITextSelection->TypeText(bstrTextToIndent);  // no selection
		 return;
	}

	CMyComBSTR bstrSelText;
	//if in one line selection is not line, delete selection
	if(nLineFrom == nLineTo){
		
		m_pITextSelection->GetText(&bstrSelText);
		if(bstrSelText){
		
			CMyComBSTR bstrLineText;
			bstrLineText = InternalGetLineText(nLineFrom);

			if(bstrLineText != bstrSelText){

				m_pITextSelection->SetText(NULL);
				m_pITextSelection->Collapse(tomFalse);
				m_pITextSelection->TypeText(bstrTextToIndent);  // no selection
				return;				
			}
		}
	}

	//	   special treatment necessary for selection due to a strange behavior of richedit:
	//     when selecting a complete line(n), richedit extends the selection 
	//     on the beginning(position 1) of the next line(n+1).
	//     Since the indenting acts on the total range of sected lines this would lead
	//     to a not intended indention of line n+1.
	if(nColTo == 1 &&(nLineFrom != nLineTo))
		nLineTo --;  // the previous line
					 // since always a complete line is indented nColTo has not to be treated 	

	// initialization
	nLastLineToIndent = nLineTo;

	// now the real indention
	for(long nLine = nLineFrom; nLine <= nLastLineToIndent; nLine++)  // loop over selected lines and indent them
	{
		InitITextRange(pITextRange, nLine);
		
		//if(bstrSelText.Length () > 0) pITextRange->MoveStart(tomCharacter, bstrSelText.Length (), NULL);
		//pITextRange->Collapse(tomFalse);
		pITextRange->Collapse(tomTrue);
		pITextRange->SetText(bstrTextToIndent);						
	
		nNewLines = m_RichEditCtrl.SendMessage(EM_GETLINECOUNT) - nTotalNumberOfLines;
		// did indention lead to new lines?
		if(nNewLines != 0)
		{  // if so, check whether the new lines consist only of white spaces

			CComBSTR bstrNewLineText;
			
			// for every new line
			for(long nCount = nNewLines - 1; nCount >= 0 ; nCount--){	

				// retrieve line with linebreak(\r\n) - always
				bstrNewLineText = InternalGetLineText(nLine + nCount, true);
				pITextRange->SetIndex(tomLine, nLine+nCount, tomExtend);
				
				// determine whether the new line consists only of empty spaces
				if(HasLineWhiteSpacesOnly(bstrNewLineText)){   
					// if so delete the line
					pITextRange->SetText(NULL);
					nNewLines--;
				}
				else{	
					// insert line with linebreak - thus softbreaks are always elimated
					pITextRange->SetText(bstrNewLineText);
				}
			}
			
			// If indention lead to new lines consisting only of whitespaces,
			// these lines have been deleted above.
			// However no changes should occur when after the deletion no new line was created at all.
			// Thus restore the original line(otherwise there might be whitespaces missing!).
			if(nNewLines != 0){

				// update loop parameters
				nLine += nNewLines;
				nLastLineToIndent += nNewLines;
				nTotalNumberOfLines += nNewLines;
			}
		} //(nNewLines != 0)
	} // loop over selected lines and indent them
	

	
	// extend the selection over the whole block of indented lines	
	pITextRange->SetIndex(tomLine, nLineFrom, tomExtend);
	pITextRange->MoveEnd(tomLine, nLastLineToIndent-nLineFrom, NULL);
	pITextRange->MoveEnd(tomCharacter, -1, NULL);
	pITextRange->Select();
	
	
} // void CSQLStudioInput::InternalIndent(void)


void CSQLStudioInput::InternalUnIndent(long nLineFrom, long nLineTo)  
{
	
	long  nSpacesDeleted;
    long  nLastLineToUnIndent;
	long  nUnIndentLength;
	long  nCanEdit = tomTrue;
	ITextRangePtr pITextRange;
	CComBSTR bstrLineText;

	nUnIndentLength = DEFAULT_INDENT_WIDTH;	
	// initialization
	nLastLineToUnIndent = nLineTo;

	
	// the real unindention
	for(long nLine = nLineFrom; nLine <= nLastLineToUnIndent; nLine++)  // loop over selected lines and indent them
	{
		if(m_bHighlightKeyAndCom != VARIANT_FALSE
			&&
			IsCommentLine(nLine, pITextRange) == true
		)
		{	
			// uindenting a comment is a little bit more complex

			// get text of the line to be processed
			pITextRange->GetText(&bstrLineText);

			// set insertion point behind the comment
			pITextRange->Collapse(tomTrue);
			pITextRange->Move(tomCharacter, nUnIndentLength, NULL);

			// get rid of comment
			DeleteLeadingCharacters(bstrLineText, nUnIndentLength);
			nSpacesDeleted = DeleteLeadingWhiteSpaces(bstrLineText, m_nSpacesOnIndent, true);

			// if some white spaces were deleted
			if(nSpacesDeleted != 0) {

				// select that part of line starting right after the comment and replace it
				pITextRange->MoveEnd(tomCharacter, nSpacesDeleted, NULL);
				pITextRange->SetText(NULL);
			}
		}
		else
		{
			// get line text
			pITextRange->GetText(&bstrLineText);

			// ignore empty lines
			if(*bstrLineText.m_str == L'\r' || *bstrLineText.m_str == L'\n')
				continue;

			nSpacesDeleted = DeleteLeadingWhiteSpaces(bstrLineText, m_nSpacesOnIndent, true);

			if(nSpacesDeleted != 0) // if some white spaces were deleted
			{
				// select that part of line starting right after the comment and replace it
				pITextRange->Collapse(tomTrue);
				pITextRange->MoveEnd(tomCharacter, nSpacesDeleted, NULL);
				pITextRange->SetText(NULL);
			}
		}
	}
		
	// extend the selection over the whole block of indented lines	
	InitITextRange(pITextRange, nLineFrom);
	pITextRange->MoveEnd(tomLine, nLastLineToUnIndent-nLineFrom, NULL);
	pITextRange->MoveEnd(tomCharacter, -1, NULL);
	pITextRange->Select();
	
} // void CSQLStudioInput::InternalUnIndent(void)


void CSQLStudioInput::InternalComment(long nLineFrom, long nLineTo)  
{
	// makes only sense if comment highlighting is enabled
	if(m_bHighlightKeyAndCom == VARIANT_FALSE)
		return;
	
    long  nLastLineToComment;
	long  nNewLines = 0;
	ITextRangePtr pITextRange;

	CComBSTR bstrNewLineText;

	// retrieve total number of lines
	long nTotalNumberOfLines = m_RichEditCtrl.SendMessage(EM_GETLINECOUNT);

	// initialization
	nLastLineToComment = nLineTo;
	for(long nLine = nLineFrom; nLine <= nLastLineToComment; nLine++)  // loop over selected lines and indent them
	{
		// get range as insertion point at the beginning of the line
		InitITextRange(pITextRange, nLine, false);

		// Set selection to first postion in line
		pITextRange->SetText(m_strCommentPrefix);

		nNewLines = m_RichEditCtrl.SendMessage(EM_GETLINECOUNT) - nTotalNumberOfLines;

		// has commenting lead to new lines?
		if(nNewLines != 0)
		{  // if so, check whether the new lines consist only of white spaces
			
			// for every new line
			for(long nCount = nNewLines - 1; nCount >= 0 ; nCount--)
			{	
				// retrieve line with linebreak(\r\n) - always
				bstrNewLineText = InternalGetLineText(nLine + nCount, true);
				InitITextRange(pITextRange, nLine+nCount);

				// set selection
				InternalSetSelectionPos(nLine + nCount, 0, nLine + nCount + 1, 0);
				
				// determine whether the new line consists only of empty spaces
				if(HasLineWhiteSpacesOnly(bstrNewLineText)){   

					// if so delete the line
					pITextRange->SetText(NULL);
					nNewLines--;
				}
				else{	
					// insert line with linebreak - thus softbreaks are always elimated
					pITextRange->SetText(bstrNewLineText);
				}
			}
			
			if(nNewLines != 0)
			{
				// update loop parameters
				nLine += nNewLines;
				nLastLineToComment += nNewLines;
				nTotalNumberOfLines += nNewLines;
			}

		} //(nNewLines != 0)

	} // loop over selected lines and change them to comments

	// extend the selection over the whole block of indented lines	
	InitITextRange(pITextRange, nLineFrom);
	pITextRange->MoveEnd(tomLine, nLastLineToComment-nLineFrom, NULL);
	pITextRange->MoveEnd(tomCharacter, -1, NULL);
	pITextRange->Select();

	StartTimerHighlightKeyAndCom();
	
} // void CSQLStudioInput::InternalComment(void)


void CSQLStudioInput::InternalUnComment(long nLineFrom, long nLineTo)  
{
	// makes only sense if comment highlighting is enabled
	if(m_bHighlightKeyAndCom == VARIANT_FALSE)
		return;
	
	long  nLastLineToUnComment;
	ITextRangePtr pITextRange;

	CComBSTR bstrLineText;
	
	// initialization
	nLastLineToUnComment = nLineTo;
	for(long nLine = nLineFrom; nLine <= nLastLineToUnComment; nLine++)  // loop over selected lines and indent them
	{
		if(!IsCommentLine(nLine, pITextRange))
		{
			// if line is no comment perform NO uncomment!!
			continue;
		}

		// get range including the comment and delete the text in this range
		pITextRange->Collapse(tomTrue);
		pITextRange->MoveEnd(tomCharacter, m_strCommentPrefix.Length(), NULL);
		pITextRange->SetText(NULL);		
	}
				
	// extend the selection over the whole block of indented lines	
	pITextRange->SetIndex(tomLine, nLineFrom, tomExtend);
	pITextRange->MoveEnd(tomLine, nLastLineToUnComment-nLineFrom, NULL);
	pITextRange->MoveEnd(tomCharacter, -1, NULL);
	pITextRange->Select();

	StartTimerHighlightKeyAndCom();
	
} // void CSQLStudioInput::InternalUnComment(void)




bool CSQLStudioInput::IsCommentLine(long nLine)
{	
	ITextRangePtr pITextRange;
	return IsCommentLine(nLine, pITextRange);
}

// the range returned extends over the whole line
bool CSQLStudioInput::IsCommentLine(long nLine, ITextRangePtr &pITextRangeOuter)
{
	ITextRangePtr pITextRange;
	InitITextRange(pITextRange, nLine);

	if(pITextRangeOuter)
		pITextRangeOuter.Release();
	
	pITextRange.CopyTo(&pITextRangeOuter);

	return IsComment(pITextRange);
}

bool CSQLStudioInput::IsComment(const ITextRangePtr &pITextRange)
{

	if((!m_strCommentPrefix) && (!m_strCommentPrefix2) || pITextRange == NULL)
		return false;

	CMyComBSTR bstrLineText;
	pITextRange->GetText(&bstrLineText);

	bstrLineText.TrimLeft();

	// is line a comment?
	if(0 == wcsncmp(bstrLineText, m_strCommentPrefix, m_strCommentPrefix.Length()))

		return true;	
	else{
		if(0 == wcsncmp(bstrLineText, m_strCommentPrefix2, m_strCommentPrefix2.Length()))
			return true;
		
		else
			return false;
		
	}	
}


void CSQLStudioInput::InternalSwitchInsertOverwrite()
{
	// switch bool member flag
	m_bInsertModus = (! m_bInsertModus);		
	return;
}


/////////////////////////////////////////////////////////////////////////////
// Static callback functions
/////////////////////////////////////////////////////////////////////////////
bool CSQLStudioInput::IsStandardWordDelimiter(const long &nChar)
{
	
	if(nChar == L' '
			|| nChar == L'\r'
			|| nChar == L'\n'
			|| nChar == L'\t'	
			|| nChar == L'.'
			|| nChar == L','
			|| nChar == L';'
			|| nChar == L':'
			|| nChar == L'?'
			|| nChar == L'+'
			|| nChar == L'-'
			|| nChar == L'*'
			|| nChar == L'/'
			|| nChar == L'!'
			|| nChar == L'"'
			|| nChar == L'\''
		)
		return TRUE;
	else
		return FALSE;
}


/////////////////////////////////////////////////////////////////////////////
// Thread functions
/////////////////////////////////////////////////////////////////////////////
DWORD WINAPI CSQLStudioInput::ThreadFuncSearchReplace(LPVOID pData)
{
	
	MSG msg;
	CSQLStudioInput *pThis = reinterpret_cast<CSQLStudioInput *>(pData);

	// Show the dialog
	pThis->m_WndFindReplace = ::ReplaceText(&pThis->m_oFINDREPLACE);

	// This is a workaround for a special bevaviour when the control
	// is used in SAPGUI. If the dialog is called
	// in PAI or PBO the dialog receives the focus for a moment, but
	// after the screen is displayed, the SAPGUI gets the focus again.
	// To get back the focus to the dialog a timer is started. When
	// the timer event is received the focus is again set to the control.
	// If this is not done, there seems to be the focus on the dialog
	// and on the R/3 screen at the same time.
	// This workaround will be obsolete after 4.0c because the the controls
	// will be integrated into the tab order of a R/3 screen and so the
	// focus will remain on a control after PBO.
	UINT nTimerID = ::SetTimer(NULL, 0, SET_FOCUS_TIME, NULL);

	// Enter message loop
    while(::GetMessage(&msg, NULL, 0, 0)) 
    { 
		if(!::IsDialogMessage(pThis->m_WndFindReplace, &msg))
		{
			// See big comment above !!!
			if((msg.message == WM_TIMER) &&(msg.hwnd == NULL) &&(msg.wParam == nTimerID))
			{
				::KillTimer(0, nTimerID);
				::SetFocus(pThis->m_WndFindReplace);
			}

			::TranslateMessage(&msg); 
			::DispatchMessage(&msg); 
		}
    } 

	return 0;
}




////////////////////////////////////////////////////////////////////////////////////////////
//  class implementation to handle the vector of TOM range pointers
//  intended to use for protected areas
////////////////////////////////////////////////////////////////////////////////////////////
	
// insertion in a way to get an ordered range where no overlap between the ranges is guaranteed(nCurrentEnd<=nNextStart)
void CSQLStudioInput::CITextRanges::AddRange(ITextRangePtr &pITextRange)
{
	
	long nRangeStart;
	long nRangeEnd;

	// initilization
	pITextRange->GetStart(&nRangeStart);
	pITextRange->GetEnd(&nRangeEnd);

	if(nRangeStart == nRangeEnd) // degenerate range: do nothing
		return;

	// insert range only if formatting was successful
	if(FormatRange(pITextRange))
	{
		PAIR_RANGE_LONG pair = make_pair(pITextRange,nRangeEnd-nRangeStart);
		m_vecRanges.push_back(pair);
		m_bSorted = false;
	}
}

// several cases have to be considered when pITextRange should be removed from the ordered vector:
//(the iterator iter loops over the elements and(*iter) corresponds to a range)
//                    pITextRange
//                   |-----------|
// 1.                               (*iter)
//                                  |-------|	=> no overlap if nIterRangeStart >= nRangeEnd
//													nothing do to
// 2.    |-------|								=> no overlap if nIterRangeEnd <= nRangeStart
//													nothing do to
// 3.           |-------|						=> overlapping:           bIterStartInRange = false, bIterEndInRange = true
//	 shrink to:	|----|								adjust nIterRangeEnd = nRangeStart
// 4.                  |-------|				=> iter contained:        bIterStartInRange = true,  bIterEndInRange = true
//   remove whole iter
// 5.                        |-------|          => overlapping:           bIterStartInRange = true,  bIterEndInRange = false
//   shrink to:					 |---|				adjust nIterRangeStart = nRangeEnd
// 6.             |--------------------|		=> pITextRange contained: bIterStartInRange = false, bIterEndInRange = false
//   divide into: |--|           |-----|			1. adjust nIterRangeEnd = nRangeStart
//													2. unformat range(nRangeStart,nIterRangeEnd)
//                                                  3. create new Range(nRangeEnd, nIterRangeEnd)
//														and addRange()
int CSQLStudioInput::CITextRanges::RemoveRange(ITextRangePtr &pITextRange)
{
	
	PAIR_RANGE_LONG_VECTOR::iterator	iter = m_vecRanges.begin();
	PAIR_RANGE_LONG_VECTOR::iterator	iterLast = m_vecRanges.end();
	PAIR_RANGE_LONG_VECTOR::iterator	iterPrev = NULL;
	long nManipulated = 0;
	long nRangeStart = 0;
	long nRangeEnd = 0;
	long nIterRangeStart = 0;
	long nIterRangeEnd = 0;
	bool bIterStartInRange = 0;
	bool bIterEndInRange = 0;

	// initilization
	pITextRange->GetStart(&nRangeStart);
	pITextRange->GetEnd(&nRangeEnd);
	if(nRangeStart == nRangeEnd) // degenerate range
		return 0;

	
	while(iter != m_vecRanges.end() && iter != iterLast){

		// is range still valid?
		if((*iter).first == NULL){

			// if range is invalid delete it and conitnue with next iter
			iter = m_vecRanges.erase(iter);
			continue;
		}
			
		// initialize nIterRangeStart
		(*iter).first->GetStart(&nIterRangeStart);
		(*iter).first->GetEnd(&nIterRangeEnd);

		// is range degenerate
		if(nIterRangeStart == nIterRangeEnd){
			
			// if range is degenerate delete it and conitnue with next iter
			iter = m_vecRanges.erase(iter);
			continue;
		}

		if(nIterRangeStart >= nRangeEnd){

			// no overlap: case 1.
			iter++;
			continue;
		}
		else{ 
			// initialize bool flag
			if(nIterRangeStart >= nRangeStart)
				bIterStartInRange = true;
			else
				bIterStartInRange = false;
		}

		if(nIterRangeEnd <= nRangeStart)
		{
			// no overlap: case 2.
			iter++;
			continue;
		}
		else
		{   // initialize bool flag
			if(nIterRangeEnd <= nRangeEnd)
				bIterEndInRange = true;
			else
				bIterEndInRange = false;
		}
	

		if(bIterStartInRange)
		{
			if(bIterEndInRange)
			{   // case 4. reset format for the whole range and remove it
				if(FormatRange((*iter).first, false))
				{
					iter = m_vecRanges.erase(iter);
					continue;
				}
			}
			else
			{   // case 5.
				ITextRangePtr pContainedRange;
				m_rTextEditCtrl.GetDuplicate((*iter).first, pContainedRange);
				pContainedRange->SetEnd(nRangeEnd);
				// delete format of the contained part
				if(FormatRange(pContainedRange, false))
				{
					// adjust iter range
					(*iter).first->SetStart(nRangeEnd);
					(*iter).second = nIterRangeEnd - nRangeEnd;
				}
				
			}
		}
		else
		{
			if(bIterEndInRange)
			{   // case 3.
				ITextRangePtr pContainedRange;
				m_rTextEditCtrl.GetDuplicate((*iter).first, pContainedRange);
				pContainedRange->SetStart(nRangeStart);
				// delete format of the contained part
				if(FormatRange(pContainedRange, false))
				{
					// adjust iter range
					(*iter).first->SetEnd(nRangeStart);
					(*iter).second = nRangeStart - nIterRangeStart;
				}
				
			}
			else
			{   // case 6.
				ITextRangePtr pNewRange, pRangeToUnFormat;

				m_rTextEditCtrl.GetDuplicate(pITextRange, pRangeToUnFormat);
				pRangeToUnFormat->SetEnd(nIterRangeEnd);

				m_rTextEditCtrl.GetDuplicate((*iter).first, pNewRange);
				pNewRange->SetStart(nRangeEnd);

				if(FormatRange(pRangeToUnFormat, false))
				{
					// adjust iter range
					(*iter).first->SetEnd(nRangeStart);
					(*iter).second = nRangeStart - nIterRangeStart;
					
					AddRange(pNewRange);
				}
			}
		}

		if(iter != m_vecRanges.begin())
			iterPrev = iter;
		else
			iterPrev = NULL;

		if(iter)
			++iter;
		else
			iter = m_vecRanges.begin();

		nManipulated ++;
	}

	m_bSorted = false;
	return nManipulated;
}

int CSQLStudioInput::CITextRanges::RemoveEqualRanges(const ITextRangePtr &pITextRange)
{
	
	PAIR_RANGE_LONG_VECTOR::iterator	iter;
	PAIR_RANGE_LONG_VECTOR::iterator	itLast = m_vecRanges.end();
	long nErased = 0;
	long nRangeEnd, nRangeStart;

	pITextRange->GetStart(&nRangeStart);
	pITextRange->GetEnd(&nRangeEnd);

	PAIR_RANGE_LONG pair = make_pair(pITextRange,nRangeEnd-nRangeStart);

	while((iter = find_if(m_vecRanges.begin(), itLast, CIsEqualRange(pair))) !=  itLast)
	{
		FormatRange((*iter).first, false);
		m_vecRanges.erase(iter);
		itLast = m_vecRanges.end();
		nErased++;
	}

	return nErased;
}



int CSQLStudioInput::CITextRanges::RemoveAll(void)
{
	
	PAIR_RANGE_LONG_VECTOR::iterator	iter = m_vecRanges.begin();
	PAIR_RANGE_LONG_VECTOR::iterator	itLast = m_vecRanges.end();
	long						nErased;

	nErased = itLast - iter;
	for(; iter != itLast; ++iter)
	{
		if((*iter).first)
			FormatRange((*iter).first, false);
	}

	m_vecRanges.clear();
	m_bSorted = true;

	return nErased;
}

int CSQLStudioInput::CITextRanges::FormatAll(void)
{
	
	PAIR_RANGE_LONG_VECTOR::iterator	iter = m_vecRanges.begin();
	PAIR_RANGE_LONG_VECTOR::iterator	itLast = m_vecRanges.end();
	long						nFormated;

	nFormated = itLast - iter;
	for(; iter != itLast; ++iter)
	{
		if((*iter).first)
			FormatRange((*iter).first, false);
	}
	
	return nFormated;
}

void CSQLStudioInput::CITextRanges::Sort(void)
{

	if(m_bSorted == true)
		return;

	sort(m_vecRanges.begin(), m_vecRanges.end(), CLowerStartIndex());

	RemoveOverlap();

	m_bSorted = true;
}

void CSQLStudioInput::CITextRanges::UpdateFormat(const ITextRangePtr &pITextRangeOuter)
{
	
	PAIR_RANGE_LONG_VECTOR::iterator	iter = m_vecRanges.begin();
	PAIR_RANGE_LONG_VECTOR::iterator	iterPrev = NULL;

	long						nOuterStart;
	long						nOuterEnd;
	long						nIterStart;
	long						nIterEnd;

	// intitalize start and end indexes of outer range
	pITextRangeOuter->GetStart(&nOuterStart);
	pITextRangeOuter->GetEnd(&nOuterEnd);

	while(iter != m_vecRanges.end())
	{
		// is range still valid?
		if((*iter).first == NULL)
		{   // if range is invalid delete it and conitnue with next iter
			iter = m_vecRanges.erase(iter);
			continue;
		}
		
		// initialize start and end indexes of range(*iter)
		(*iter).first->GetStart(&nIterStart);
		(*iter).first->GetEnd(&nIterEnd);

		// is range degenerate?
		if(nIterStart == nIterEnd)
		{   // if range is degenerate delete it and conitnue with next iter
			iter = m_vecRanges.erase(iter);
			continue;
		}

		// is(*iter) overlapping with pITextRangeOuter ?
		// then format(*iter)
		if(nIterStart < nOuterEnd && nIterEnd > nOuterStart){

			FormatRange((*iter).first);
			
		}

		if(iter != m_vecRanges.begin())
			iterPrev = iter;
		else
			iterPrev = NULL;

		if(iter)
			++iter;
		else
			iter = m_vecRanges.begin();
	}
}

void CSQLStudioInput::CITextRanges::UpdatePairInfo()
{
	PAIR_RANGE_LONG_VECTOR::iterator	iter = m_vecRanges.begin();
	PAIR_RANGE_LONG_VECTOR::iterator	itLast = m_vecRanges.end();
	long								nIterStart;
	long								nIterEnd;

	for(; iter != itLast; ++iter)
	{
		if(SUCCEEDED((*iter).first->GetStart(&nIterStart)))
		{
			if(SUCCEEDED((*iter).first->GetEnd(&nIterEnd)))
			{
				(*iter).second = nIterEnd - nIterStart;
			}
		}
	}
}

bool CSQLStudioInput::CITextRanges::FormatRange(const ITextRangePtr &pITextRange, bool bFormat)
{
	
	CSaveModifyStatus	oSaveModifyStatus(m_rTextEditCtrl);
	CSuspendUndo		oSuspendUndo(m_rTextEditCtrl);
	HRESULT				hr;
	ITextFontPtr		pITextFont;

	long						nIterStart = 0;
	long						nIterEnd = 0;

	hr = pITextRange->GetFont(&pITextFont);

	if(!SUCCEEDED(hr))
		return false;
			
	switch(m_rTextEditCtrl.m_KeyWordChangeCase){

		case LowerCase: 
			pITextRange->ChangeCase(tomLowerCase);
			break;
		case UpperCase: 
			pITextRange->ChangeCase(tomUpperCase);
			break;
		case TitleCase: 
			pITextRange->ChangeCase(tomTitleCase);
			break;
		case SentenceCase: 
			pITextRange->ChangeCase(tomSentenceCase);
			break;
		case ToggleCase:
			pITextRange->ChangeCase(tomToggleCase);
			break;
		default:
			break;
	};				
	
	hr = pITextFont->SetForeColor(m_rTextEditCtrl.m_crTextHighlight);
	if(!SUCCEEDED(hr))
		return false;


	pITextRange->GetStart(&nIterStart);
	pITextRange->GetEnd(&nIterEnd);

			

	return true;
}

// removes overlapping ranges by consolidating them into one range
// this method requires the array to be sorted
void CSQLStudioInput::CITextRanges::RemoveOverlap(void)
{
	
	PAIR_RANGE_LONG_VECTOR::iterator	iter = m_vecRanges.begin();
	PAIR_RANGE_LONG_VECTOR::iterator	itPrevious;
	long						nIterStart = 0;
	long						nIterEnd = 0;
	long						nPreviousStart;
	long						nPreviousEnd;

	if(iter != m_vecRanges.end())
	{
		// check first element
		// is a valid range assigned
		while(iter != m_vecRanges.end() &&(*iter).first == NULL)
		{
			iter = m_vecRanges.erase(iter);
		}

		// is range degenerate?
		for(; iter != m_vecRanges.end(); ++iter) 
		{
			(*iter).first->GetStart(&nIterStart);
			(*iter).first->GetEnd(&nIterEnd);
					// is range degenerate?
			if(nIterStart == nIterEnd)
			{   // if range is degenerate delete it and conitnue with next iter
				iter = m_vecRanges.erase(iter);
			}
			else
				break;
		}

		// normalizing removes overlapping ranges: if there are less than 2 nothing has to be done
		if(m_vecRanges.end() - iter < 2)
			return;

		// initialization for the following while-loop
		itPrevious = iter; 
		nPreviousStart = nIterStart;
		nPreviousEnd = nIterEnd;
		++iter;

		while(iter != m_vecRanges.end())
		{
			// is range still valid?
			if((*iter).first == NULL)
			{   // if range is invalid delete it and conitnue with next iter
				iter = m_vecRanges.erase(iter);
				continue;
			}

			// initialize start and end indexes of range(*iter)
			(*iter).first->GetStart(&nIterStart);
			(*iter).first->GetEnd(&nIterEnd);

			// is range degenerate?
			if(nIterStart == nIterEnd)
			{   // if range is degenerate delete it and conitnue with next iter
				iter = m_vecRanges.erase(iter);
				continue;
			}
			
			// are iter and previous overlapping(remember: the algorithm assumes the vector to be sorted)
			if(nIterStart < nPreviousEnd)
			{
				if(nIterEnd <= nPreviousEnd)
				{   // iter range is contained within previous range
					iter = m_vecRanges.erase(iter);
					continue;
				}
				else
				{  // iter starts within previous but extends further than previous
					(*itPrevious).first->SetEnd(nIterEnd);
					(*itPrevious).second = nIterEnd - nPreviousStart;
					iter = m_vecRanges.erase(iter);
					continue;
				}
			}

			// when iter is valid and doesn't overlap with previous
			itPrevious = iter,
			nPreviousStart = nIterStart;
			nPreviousEnd = nIterEnd; 
			++iter;
		}
	}
}

void CSQLStudioInput::CITextRanges::Print(void)
{
	
	ofstream outfile;
	
	PAIR_RANGE_LONG_VECTOR::iterator	iter = m_vecRanges.begin();
	PAIR_RANGE_LONG_VECTOR::iterator	itLast = m_vecRanges.end();
	long						nStart;
	long						nEnd;

	outfile.open("Ranges.txt", ios_base::out | ios_base::app);
	outfile << "\n\n\n" << itLast-iter << " of ";
	outfile << "Elements in Total:\n";

	for(; iter != itLast; ++iter)
	{
		(*iter).first->GetStart(&nStart);
		(*iter).first->GetEnd(&nEnd);
		outfile << " (" << nStart << ", " << nEnd << ", " <<(*iter).second  << ")\n";
	}
}
/////////////////////////////////////////////////////////////////////////////
// Interface IRichEditOleCallback
/////////////////////////////////////////////////////////////////////////////
STDMETHODIMP CSQLStudioInput::QueryAcceptData(LPDATAOBJECT lpdataobj,
										 CLIPFORMAT FAR * lpcfFormat,
										 DWORD reco,
										 BOOL fReally, HGLOBAL)
{
	
    if(m_OLEDropMode == OLEDropNone) return S_OK;	
    if(lpcfFormat)
  	{

  		if(fReally != TRUE)
  		{
  			// it's just a query whether data will be accepted

  			// We only accept data in CF_TEXT format. The richedit is able to
  			// accept formatted text with pictures inside, too.
  			// But in this case the control is used as an editor for plain text.
  			*lpcfFormat = CF_TEXT;
  		}
  		else // user wants to paste or drop data
  		{
  			STGMEDIUM stg;
  			FORMATETC fe;

#ifdef UNICODE
			fe.cfFormat = CF_UNICODETEXT;
#else
  			fe.cfFormat = CF_TEXT;
#endif			
  			fe.ptd = NULL;
  			fe.tymed = TYMED_HGLOBAL;
  			fe.dwAspect = DVASPECT_CONTENT;
  			fe.lindex = -1;

  			lpdataobj->GetData(&fe, &stg);

			//	get data. GlobalLock() and GlobalUnlock() are required 
  			//  because the handle by the data object is to moveable memory
#ifdef _UNICODE
  			TCHAR* strText =(TCHAR*) GlobalLock(stg.hGlobal);
#else
			char* strText =(char*) GlobalLock(stg.hGlobal);
#endif

			if(m_OLEDropMode == OLEDropManual){

					
#ifdef _UNICODE
					Fire_TCDropText(strText);
#else
					USES_CONVERSION;
					Fire_TCDropText(A2BSTR(strText));
#endif
					GlobalUnlock(stg.hGlobal);
			}
			else{
				// insert Text into Control
  				InternalSetText(strText, -1, SFF_SELECTION);
				m_pITextSelection->Collapse(tomFalse);
  				
  				GlobalUnlock(stg.hGlobal);
			}

  			// release storage medium and data object
  			ReleaseStgMedium(&stg);

  			return S_FALSE;
  		}
  	}	

	return S_OK;
}	


STDMETHODIMP CSQLStudioInput::GetNewStorage(LPSTORAGE FAR *)
{
ATLTRACENOTIMPL("CSQLStudioInput::GetNewStorage");
}

STDMETHODIMP CSQLStudioInput::GetInPlaceContext(LPOLEINPLACEFRAME FAR *,
  							  LPOLEINPLACEUIWINDOW FAR *,
  							  LPOLEINPLACEFRAMEINFO)
{
	ATLTRACENOTIMPL("CSQLStudioInput::GetInPlaceContext");
}

STDMETHODIMP CSQLStudioInput::ShowContainerUI(BOOL)
{
	ATLTRACENOTIMPL("CSQLStudioInput::ShowContainerUI");
}

STDMETHODIMP CSQLStudioInput::QueryInsertObject(LPCLSID, LPSTORAGE,	LONG)
{
	ATLTRACENOTIMPL("CSQLStudioInput::QueryInsertObject");
}

STDMETHODIMP CSQLStudioInput::DeleteObject(LPOLEOBJECT)
{
	ATLTRACENOTIMPL("CSQLStudioInput::DeleteObject");
}

STDMETHODIMP CSQLStudioInput::ContextSensitiveHelp(BOOL)
{
	ATLTRACENOTIMPL("CSQLStudioInput::ContextSensitiveHelp");
}

STDMETHODIMP CSQLStudioInput::GetClipboardData(CHARRANGE FAR *, DWORD,
									LPDATAOBJECT FAR *)
{
	ATLTRACENOTIMPL("CSQLStudioInput::GetClipboardData");
}

STDMETHODIMP CSQLStudioInput::SetStatusText(BSTR strText)
{
	
	USES_CONVERSION;
	m_StatusBar.SendMessage(SB_SETTEXT, m_nStatusBarPartExternal, reinterpret_cast<LPARAM>(OLE2A(strText)));

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::GetDragDropEffect(BOOL fDrag, DWORD grfKeyState, LPDWORD pdwEffect)
{
  	ATLTRACENOTIMPL("CSQLStudioInput::GetDragDropEffect");
}


STDMETHODIMP CSQLStudioInput::SetTextSymbol(BSTR strKey, BSTR strText)
{
	USES_CONVERSION;
	m_mapDisplayText[_wtol(strKey)] = OLE2T(strText);
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::GetContextMenu(WORD seltyp, LPOLEOBJECT lpoleobj, CHARRANGE *lpchrg, HMENU *lphmenu)
{	

	// only if context menu should be displayed at all
	if(m_bLocalContextMenuVisible != VARIANT_FALSE)
	{		

		HMENU hContextMenu = CreatePopupMenu();
		MENUITEMINFO oMENUITEMINFO;
		ZeroMemory(&oMENUITEMINFO, sizeof(MENUITEMINFO));

		oMENUITEMINFO.cbSize = sizeof(MENUITEMINFO);
		oMENUITEMINFO.fMask = MIIM_ID|MIIM_STATE|MIIM_TYPE;
		oMENUITEMINFO.fType = MFT_STRING;
		UINT uItemPosition = 0;
		UINT uSubItemPosition = 0;

		
		// Cut
		if(lpchrg->cpMin == lpchrg->cpMax || m_bReadonly)
			oMENUITEMINFO.fState = MFS_DISABLED|MFS_GRAYED;
		else
			oMENUITEMINFO.fState = MFS_ENABLED;

		oMENUITEMINFO.wID = ID_CUT;
		oMENUITEMINFO.dwTypeData = const_cast<LPTSTR>(GetDisplayText(ID_CUT));
		InsertMenuItem(hContextMenu, uItemPosition, TRUE, &oMENUITEMINFO);
		uItemPosition++;

		// Copy
		if(lpchrg->cpMin == lpchrg->cpMax)
			oMENUITEMINFO.fState = MFS_DISABLED|MFS_GRAYED;
		else
			oMENUITEMINFO.fState = MFS_ENABLED;

		oMENUITEMINFO.wID = ID_COPY;
		oMENUITEMINFO.dwTypeData = const_cast<LPTSTR>(GetDisplayText(ID_COPY));
		InsertMenuItem(hContextMenu, uItemPosition, TRUE, &oMENUITEMINFO);
		uItemPosition++;

		// Paste
		if(::IsClipboardFormatAvailable(CF_TEXT) &&(!m_bReadonly))
			oMENUITEMINFO.fState = MFS_ENABLED;
		else
			oMENUITEMINFO.fState = MFS_DISABLED|MFS_GRAYED;

		oMENUITEMINFO.wID = ID_PASTE;
		oMENUITEMINFO.dwTypeData = const_cast<LPTSTR>(GetDisplayText(ID_PASTE));
		InsertMenuItem(hContextMenu, uItemPosition, TRUE, &oMENUITEMINFO);
		uItemPosition++;

		// Insert separator
		oMENUITEMINFO.fMask = MIIM_TYPE;
		oMENUITEMINFO.fType = MFT_SEPARATOR;
		InsertMenuItem(hContextMenu, uItemPosition, TRUE, &oMENUITEMINFO);
		uItemPosition++;

		oMENUITEMINFO.fMask = MIIM_ID|MIIM_STATE|MIIM_TYPE;
		oMENUITEMINFO.fType = MFT_STRING;

		/* Undo*/
		if(m_RichEditCtrl.SendMessage(EM_CANUNDO) &&(!m_bReadonly))
			oMENUITEMINFO.fState = MFS_ENABLED;
		else
			oMENUITEMINFO.fState = MFS_DISABLED|MFS_GRAYED;

		oMENUITEMINFO.wID = ID_UNDO;
		oMENUITEMINFO.dwTypeData = const_cast<LPTSTR>(GetDisplayText(ID_UNDO));
		InsertMenuItem(hContextMenu, uItemPosition, TRUE, &oMENUITEMINFO);
		uItemPosition++;

		// Redo
		if(m_RichEditCtrl.SendMessage(EM_CANREDO) &&(!m_bReadonly))
			oMENUITEMINFO.fState = MFS_ENABLED;
		else
			oMENUITEMINFO.fState = MFS_DISABLED|MFS_GRAYED;

		oMENUITEMINFO.wID = ID_REDO;
		oMENUITEMINFO.dwTypeData = const_cast<LPTSTR>(GetDisplayText(ID_REDO));
		InsertMenuItem(hContextMenu, uItemPosition, TRUE, &oMENUITEMINFO);
		uItemPosition++;

		if(m_bHighlightKeyAndCom != VARIANT_FALSE){

			// Insert separator
			oMENUITEMINFO.fMask = MIIM_TYPE;
			oMENUITEMINFO.fType = MFT_SEPARATOR;
			InsertMenuItem(hContextMenu, uItemPosition, TRUE, &oMENUITEMINFO);
			uItemPosition++;

			oMENUITEMINFO.fMask = MIIM_ID|MIIM_STATE|MIIM_TYPE;
			oMENUITEMINFO.fType = MFT_STRING;


			// Comment
			if((!m_bReadonly))
				oMENUITEMINFO.fState = MFS_ENABLED;
			else
				oMENUITEMINFO.fState = MFS_DISABLED|MFS_GRAYED;

			oMENUITEMINFO.wID = ID_COMMENT;
			oMENUITEMINFO.dwTypeData = const_cast<LPTSTR>(GetDisplayText(ID_COMMENT));
			InsertMenuItem(hContextMenu, uItemPosition, TRUE, &oMENUITEMINFO);
			uItemPosition++;

			// UnComment
			if((!m_bReadonly))
				oMENUITEMINFO.fState = MFS_ENABLED;
			else
				oMENUITEMINFO.fState = MFS_DISABLED|MFS_GRAYED;

			oMENUITEMINFO.wID = ID_UNCOMMENT;
			oMENUITEMINFO.dwTypeData = const_cast<LPTSTR>(GetDisplayText(ID_UNCOMMENT));
			InsertMenuItem(hContextMenu, uItemPosition, TRUE, &oMENUITEMINFO);
			uItemPosition++;
		}


		// Insert separator
		oMENUITEMINFO.fMask = MIIM_TYPE;
		oMENUITEMINFO.fType = MFT_SEPARATOR;
		InsertMenuItem(hContextMenu, uItemPosition, TRUE, &oMENUITEMINFO);
		uItemPosition++;

		oMENUITEMINFO.fMask = MIIM_ID|MIIM_STATE|MIIM_TYPE;
		oMENUITEMINFO.fType = MFT_STRING;


		// Find/Replace
		oMENUITEMINFO.fState = MFS_ENABLED;

		oMENUITEMINFO.wID = ID_FIND_REPLACE;
		oMENUITEMINFO.dwTypeData = const_cast<LPTSTR>(GetDisplayText(ID_FIND_REPLACE));
		InsertMenuItem(hContextMenu, uItemPosition, TRUE, &oMENUITEMINFO);
		uItemPosition++;

		// Find next
		if(m_bstrFindWhat)
			oMENUITEMINFO.fState = MFS_ENABLED;
		else
			oMENUITEMINFO.fState = MFS_DISABLED|MFS_GRAYED;

		oMENUITEMINFO.wID = ID_FIND_NEXT;
		oMENUITEMINFO.dwTypeData = const_cast<LPTSTR>(GetDisplayText(ID_FIND_NEXT));
		InsertMenuItem(hContextMenu, uItemPosition, TRUE, &oMENUITEMINFO);
		uItemPosition++;

		// Insert separator
		oMENUITEMINFO.fMask = MIIM_TYPE;
		oMENUITEMINFO.fType = MFT_SEPARATOR;
		InsertMenuItem(hContextMenu, uItemPosition, TRUE, &oMENUITEMINFO);
		uItemPosition++;

		oMENUITEMINFO.fMask = MIIM_ID|MIIM_STATE|MIIM_TYPE;
		oMENUITEMINFO.fType = MFT_STRING;

		// Import
		if(m_bReadonly)
			oMENUITEMINFO.fState = MFS_DISABLED|MFS_GRAYED;
		else
			oMENUITEMINFO.fState = MFS_ENABLED;

		oMENUITEMINFO.wID = ID_IMPORT;
		oMENUITEMINFO.dwTypeData = const_cast<LPTSTR>(GetDisplayText(ID_IMPORT));
		InsertMenuItem(hContextMenu, uItemPosition, TRUE, &oMENUITEMINFO);
		uItemPosition++;

		// Export
		oMENUITEMINFO.fState = MFS_ENABLED;

		oMENUITEMINFO.wID = ID_EXPORT;
		oMENUITEMINFO.dwTypeData = const_cast<LPTSTR>(GetDisplayText(ID_EXPORT));
		InsertMenuItem(hContextMenu, uItemPosition, TRUE, &oMENUITEMINFO);
		uItemPosition++;


		//Popup Properties
		HMENU hContextPopMenu = CreatePopupMenu();		
		
		oMENUITEMINFO.wID = ID_CHOOSEFONT;		
		oMENUITEMINFO.dwTypeData = const_cast<LPTSTR>(GetDisplayText(ID_CHOOSEFONT));
		InsertMenuItem(hContextPopMenu, uSubItemPosition, TRUE, &oMENUITEMINFO);
		uSubItemPosition++;

		// Insert separator
		oMENUITEMINFO.fMask = MIIM_TYPE;
		oMENUITEMINFO.fType = MFT_SEPARATOR;
		InsertMenuItem(hContextPopMenu, uSubItemPosition, TRUE, &oMENUITEMINFO);
		uSubItemPosition++;


		oMENUITEMINFO.wID = ID_FORTEXTEDIT;		
		oMENUITEMINFO.fState = MFS_ENABLED;
		oMENUITEMINFO.fMask = MIIM_ID|MIIM_STATE|MIIM_TYPE;
		oMENUITEMINFO.fType = MFT_STRING;
		oMENUITEMINFO.dwTypeData = const_cast<LPTSTR>(GetDisplayText(ID_FORTEXTEDIT));
		InsertMenuItem(hContextPopMenu, uSubItemPosition, TRUE, &oMENUITEMINFO);
		uSubItemPosition++;

		oMENUITEMINFO.wID = ID_FORHIGHLIGHT;		
		oMENUITEMINFO.dwTypeData = const_cast<LPTSTR>(GetDisplayText(ID_FORHIGHLIGHT));
		InsertMenuItem(hContextPopMenu, uSubItemPosition, TRUE, &oMENUITEMINFO);
		uSubItemPosition++;

		
		oMENUITEMINFO.wID = ID_PROPERTIES;
		oMENUITEMINFO.fMask = MIIM_ID|MIIM_STATE|MIIM_TYPE |MIIM_SUBMENU;
		oMENUITEMINFO.fType = MFT_STRING;
		oMENUITEMINFO.hSubMenu = hContextPopMenu;
		oMENUITEMINFO.dwTypeData = const_cast<LPTSTR>(GetDisplayText(ID_PROPERTIES));
		InsertMenuItem(hContextMenu, uItemPosition, TRUE, &oMENUITEMINFO);
		uItemPosition++;

		// return menu handle
		*lphmenu = hContextMenu;		
	}
	else
		*lphmenu = NULL;	

	return S_OK;
}



STDMETHODIMP CSQLStudioInput::FileOpen(BSTR strFileName)
{
	USES_CONVERSION;
	LPTSTR pFileName = OLE2T(strFileName);
	
	if((pFileName == NULL) ||(pFileName[0] == 0))
	{	
		pFileName = new TCHAR[512];
		memset(pFileName,0,512);

		// Open the FileOpen dialog
		OPENFILENAME oOPENFILENAME;
		ZeroMemory(&oOPENFILENAME, sizeof(oOPENFILENAME));

		oOPENFILENAME.lStructSize		= sizeof(oOPENFILENAME);
		oOPENFILENAME.hwndOwner			= m_hWnd;
		oOPENFILENAME.lpstrFile			= (LPTSTR)pFileName;
		oOPENFILENAME.nMaxFile			= _MAX_PATH;
		oOPENFILENAME.Flags				= OFN_FILEMUSTEXIST|OFN_HIDEREADONLY|OFN_PATHMUSTEXIST;

		if(GetOpenFileName(&oOPENFILENAME)) pFileName = oOPENFILENAME.lpstrFile;

		InternalFileOpen(pFileName);

		delete[] pFileName;
	}
	else
		InternalFileOpen(pFileName);

	

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::FileSaveAs(BSTR strFileName)
{
	USES_CONVERSION;
	LPTSTR pFileName = OLE2T(strFileName);
	
	if((pFileName == NULL) ||(pFileName[0] == 0))
	{	

		pFileName = new TCHAR[512];
		memset(pFileName,0,512);
		
		// Open the Save dialog
		OPENFILENAME oOPENFILENAME;
		ZeroMemory(&oOPENFILENAME, sizeof(oOPENFILENAME));

		oOPENFILENAME.lStructSize		= sizeof(oOPENFILENAME);
		oOPENFILENAME.hwndOwner			= m_hWnd;
		oOPENFILENAME.lpstrFile			= (LPTSTR) pFileName;
		oOPENFILENAME.nMaxFile			= _MAX_PATH;
		oOPENFILENAME.Flags				= OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT|OFN_PATHMUSTEXIST;

		if(GetSaveFileName(&oOPENFILENAME)) pFileName = oOPENFILENAME.lpstrFile;

		InternalFileSaveAs(pFileName);

		delete[] pFileName;
	}
	else
		InternalFileSaveAs(pFileName);

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::LimitText(long nLimit)
{
	if((nLimit > 0) &&(m_RichEditCtrl.GetWindowTextLength() == 0)){
	
		m_nLimitText = nLimit;
	    m_RichEditCtrl.SendMessage(EM_EXLIMITTEXT, 0, nLimit);
	}

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::SelectLines(long nLineFrom, long nLineTo)
{
	LONG nLineFirst = m_RichEditCtrl.SendMessage(EM_GETFIRSTVISIBLELINE) + 1;

	// Parameter checking in InternalSelectLines
	InternalSelectLines(nLineFrom, nLineTo);

	if(nLineFirst > nLineTo)
		m_RichEditCtrl.SendMessage(WM_VSCROLL, MAKEWPARAM(SB_LINEUP,0), NULL);


	return S_OK;
}

STDMETHODIMP CSQLStudioInput::GotoLine(long nLine)
{
	// Parameter checking in InternalSelectLines
	InternalSelectLines(nLine, nLine);
	m_RichEditCtrl.SendMessage(EM_SCROLLCARET);

	return S_OK;
}



STDMETHODIMP CSQLStudioInput::get_HighlightKeyAndCom(VARIANT_BOOL *pVal)
{
	
	*pVal = m_bHighlightKeyAndCom;
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::put_HighlightKeyAndCom(VARIANT_BOOL newVal)
{
	m_bHighlightKeyAndCom = newVal;
	if(m_RichEditCtrl.m_hWnd != NULL){

		if(m_bHighlightKeyAndCom == VARIANT_FALSE)
			HighlightAllComments();
		else
			StartTimerHighlightKeyAndCom();
	}

	if(m_ToolBar.m_hWnd != NULL) EnableDisableToolBarButtons();	
	
	return S_OK;
}


STDMETHODIMP CSQLStudioInput::get_CommentPrefix(BSTR *pVal)
{
	CComBSTR strText(m_strCommentPrefix);
	*pVal = strText.Detach();

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::put_CommentPrefix(BSTR newVal)
{
	if((newVal != NULL))
	{
		if(newVal[0] != 0)

			m_strCommentPrefix = newVal;
		
		else
			m_strCommentPrefix.Detach();			
	}	

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::get_CommentPrefix2(BSTR *pVal)
{
	
	CComBSTR strText(m_strCommentPrefix2);
	*pVal = strText.Detach();

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::put_CommentPrefix2(BSTR newVal)
{
	
	if((newVal != NULL))
	{
		if(newVal[0] != 0)

			m_strCommentPrefix2 = newVal;		
		else
			m_strCommentPrefix2.Detach();			
	}	

	return S_OK;
}



STDMETHODIMP CSQLStudioInput::AddKeywords(BSTR newKeyWords)
{
	USES_CONVERSION;
	
	m_strKeywords += OLE2T(newKeyWords);
	m_strKeywordsLower = m_strKeywords;
	if(!m_bCaseSensitive)
		m_strKeywordsLower.ToLower();

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::AddConstants(BSTR newConstants)
{
	USES_CONVERSION;
	m_strConstants += OLE2T(newConstants);
	m_strConstantsLower = m_strConstants;
	if(!m_bCaseSensitive)
		m_strConstantsLower.ToLower();

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::get_KeyWordsCaseSensitive(VARIANT_BOOL *pVal)
{
	
	*pVal = m_bCaseSensitive;
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::put_KeyWordsCaseSensitive(VARIANT_BOOL newVal)
{
	
	m_bCaseSensitive = newVal;
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::get_WordWrap(VARIANT_BOOL  *pVal)
{
	 
	*pVal = m_bWordWrap;
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::put_WordWrap(VARIANT_BOOL newVal)
{
	
	if((newVal>= 0) &&(newVal<= 1))
	{
		if(m_bWordWrap != newVal)
		{
			if(
				(m_RichEditCtrl.m_hWnd == NULL) || 
		        // change is only possible when there is no text in richedit control
				(m_RichEditCtrl.GetWindowTextLength() == 0))
			{
				m_bWordWrap = newVal;

				if(m_bWordWrap == VARIANT_FALSE)
					m_bConvertWordWrapToLineBreak = VARIANT_FALSE;									

				CreateRichEditWindow();
			}
		}
	}

	return S_OK;
}


STDMETHODIMP CSQLStudioInput::RefreshFont()
{	

	OnFontChanged(); 
	return S_OK;
}


STDMETHODIMP CSQLStudioInput::OnFontChanged() 
{
	
	HFONT	hFont = NULL;
	CHARFORMAT2 oCF;
	CHOOSEFONT oCSF = {0};
	LOGFONT oLF;

	CY	lFontSize;
	ITextFontPtr	pITextFont;
	
	HRESULT hr = E_FAIL;

	CComBSTR	bstrFontName;
	TCHAR *pName = NULL;

	/*Create Font*/
	CComQIPtr<IFont,&IID_IFont> pFont(m_pFont);

	if(pFont == NULL) return S_FALSE;

	if(m_RichEditCtrl.m_hWnd == NULL) return S_FALSE;

	hr = pFont->get_hFont(&hFont);
	if(!SUCCEEDED(hr))	return S_FALSE;
	
	::GetObject(hFont, sizeof(LOGFONT), &oLF);
	hFont = ::CreateFontIndirect(&oLF);	
		
	oCF.cbSize = sizeof(CHARFORMAT2);
	(void) m_RichEditCtrl.SendMessage(EM_GETCHARFORMAT,(WPARAM) VARIANT_TRUE,(LPARAM) &oCF);	

	hr = pFont->get_hFont(&hFont);	
	pFont->get_Size(&lFontSize);		
	oCF.yHeight = lFontSize.Lo / 500;
	
	// Fill out the CHARFORMAT2 structure to set character effects.
	// don't change read-only bit
	oCF.dwMask =  CFM_SIZE | CFM_FACE | CFM_CHARSET | CFM_BOLD | CFM_ITALIC | CFM_UNDERLINE | 
					 CFM_STRIKEOUT | CFM_COLOR ;	
	
	oCF.dwEffects = 0;

	if(oLF.lfWeight == FW_BOLD)
		oCF.dwEffects |= CFE_BOLD;			
	
	if(oLF.lfItalic)
		oCF.dwEffects |= CFE_ITALIC;		
	
	if(oLF.lfUnderline)
		oCF.dwEffects |= CFE_UNDERLINE;			
	
	if(oLF.lfStrikeOut)
		oCF.dwEffects |= CFE_STRIKEOUT;		
	
	
	oCF.crTextColor = m_crTextEditable;

	oCF.bCharSet = oLF.lfCharSet;
	oCF.bPitchAndFamily = oLF.lfPitchAndFamily;

	USES_CONVERSION;
	bstrFontName = oLF.lfFaceName;
	pName = OLE2T(bstrFontName);
	if(pName  != NULL) _tcscpy(oCF.szFaceName, pName);
	
	if(m_RichEditCtrl.m_hWnd != NULL)		
		m_RichEditCtrl.SendMessage(EM_SETCHARFORMAT, SCF_ALL,(LPARAM) &oCF);	
	
	pFont.Release();

	return S_OK;

}


STDMETHODIMP CSQLStudioInput::get_ConvertWordWrapToLineBreak(VARIANT_BOOL *pVal)
{	
	*pVal = m_bConvertWordWrapToLineBreak;
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::put_ConvertWordWrapToLineBreak(VARIANT_BOOL newVal)
{	
	if(newVal == VARIANT_FALSE)
		m_bConvertWordWrapToLineBreak = VARIANT_FALSE;	
	else
	{
		if(m_bWordWrap	!= VARIANT_FALSE)
			m_bConvertWordWrapToLineBreak = VARIANT_TRUE;
		else
			m_bConvertWordWrapToLineBreak = VARIANT_FALSE;
	}

	return S_OK;
}


// Font
HRESULT STDMETHODCALLTYPE CSQLStudioInput::put_Font(IFontDisp* pFont)
{
	ATLTRACE2(atlTraceControls,2,_T("CSQLStudioInput::put_Font\n"));
	CSQLStudioInput* pT =(CSQLStudioInput*) this;
	if(pT->FireOnRequestEdit(DISPID_FONT) == S_FALSE)
		return S_FALSE;
	pT->m_pFont = 0;
	if(pFont)
	{
		CComQIPtr<IFont, &IID_IFont> p(pFont);
		if(p)
		{
			CComPtr<IFont> pFont;
			p->Clone(&pFont);
			if(pFont)
				pFont->QueryInterface(IID_IFontDisp,(void**) &pT->m_pFont);
		}
	}
	pT->m_bRequiresSave = TRUE;
	pT->FireOnChanged(DISPID_FONT);
	pT->FireViewChange();
	pT->SendOnDataChange(NULL);

	OnFontChanged();

	return S_OK;
}
HRESULT STDMETHODCALLTYPE CSQLStudioInput::putref_Font(IFontDisp* pFont)
{
	ATLTRACE2(atlTraceControls,2,_T("CSQLStudioInput::putref_Font\n"));
	CSQLStudioInput* pT =(CSQLStudioInput*) this;
	if(pT->FireOnRequestEdit(DISPID_FONT) == S_FALSE)
		return S_FALSE;
	pT->m_pFont = pFont;
	pT->m_bRequiresSave = TRUE;
	pT->FireOnChanged(DISPID_FONT);
	pT->FireViewChange();
	pT->SendOnDataChange(NULL);

	OnFontChanged();

	return S_OK;
}

HRESULT STDMETHODCALLTYPE CSQLStudioInput::get_Font(IFontDisp** ppFont)
{
	ATLTRACE2(atlTraceControls,2,_T("CSQLStudioInput::get_Font\n"));
	CSQLStudioInput* pT =(CSQLStudioInput*) this;
	*ppFont = pT->m_pFont;	

	if(*ppFont != NULL){

		(*ppFont)->AddRef();
	}
	return S_OK;
}


STDMETHODIMP CSQLStudioInput::get_AutoIndent(VARIANT_BOOL *pVal)
{	
	*pVal = m_bAutoIndent;
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::put_AutoIndent(VARIANT_BOOL newVal)
{
	m_bAutoIndent = newVal;
	return S_OK;
}


STDMETHODIMP CSQLStudioInput::CommentLines(long nLineFrom, long nLineTo)
{
	// Parameter checking
	CheckLineFromTo(nLineFrom, nLineTo);	
	InternalComment(nLineFrom, nLineTo);	

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::CommentSelection()
{	
	long nLineFrom, nLineTo;
	InternalGetSelectedLines(&nLineFrom, &nLineTo);
	
	InternalComment(nLineFrom, nLineTo);	

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::FindAndReplace(BSTR strTextFind, BSTR strTextReplace, VARIANT_BOOL bCaseSensitive, VARIANT_BOOL bWholeWord, VARIANT_BOOL *pbFound)
{
	// initialize global member for search and replace
	m_oFINDREPLACE.Flags = 0;
	m_bFindReplaceInSelection = false;


	if(bCaseSensitive != VARIANT_FALSE)
		m_oFINDREPLACE.Flags |= FR_MATCHCASE;

	if(bWholeWord != VARIANT_FALSE)
		m_oFINDREPLACE.Flags |= FR_WHOLEWORD;

	USES_CONVERSION;
	// Check if string parameters are NULL pointers
	if(strTextFind == NULL || 0 == SysStringLen(strTextFind))
		m_bstrFindWhat.Empty();	
	else
		m_bstrFindWhat = strTextFind;	

	if(strTextReplace == NULL || 0 == SysStringLen(strTextReplace))
		m_bstrReplaceWith.Empty();	
	else
		m_bstrReplaceWith = strTextReplace;	

	// If nothing is to find, display the find/replace dialog
	if(!m_bstrFindWhat){

		ShowFindReplaceDialog();
		*pbFound = VARIANT_FALSE;
	}
	else
	{
		ITextRangePtr pITextRange;
		if(FindNext(pITextRange, true) && pITextRange){

			pITextRange->SetText(m_bstrReplaceWith);
			pITextRange->Select();
			pITextRange->ScrollIntoView(tomStart);
			*pbFound = VARIANT_TRUE;

		}
		else{

			*pbFound = VARIANT_FALSE;
		}
	}

	// Update toolbar
	EnableDisableToolBarButtons();

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::FindAndSelectText(BSTR strText, VARIANT_BOOL bCaseSensitive, VARIANT_BOOL bWholeWord, VARIANT_BOOL *pbFound)
{
	// initializing global members for 
	m_oFINDREPLACE.Flags = 0;
	m_bFindReplaceInSelection = false;

	if(bCaseSensitive != VARIANT_FALSE)
		m_oFINDREPLACE.Flags |= FR_MATCHCASE;

	if(bWholeWord != VARIANT_FALSE)
		m_oFINDREPLACE.Flags |= FR_WHOLEWORD;

	USES_CONVERSION;
	if(NULL == strText || 0 == SysStringLen(strText))
		m_bstrFindWhat.Empty();
	
	else
		m_bstrFindWhat = strText;
	

	if(!m_bstrFindWhat){

		ShowFindReplaceDialog();
		*pbFound = VARIANT_FALSE;
	}
	else
	{
		ITextRangePtr pITextRange;		
		if(FindNext(pITextRange, true) && pITextRange){

			pITextRange->Select();
			pITextRange->ScrollIntoView(tomStart);
			*pbFound = VARIANT_TRUE;
		}
		else	
			*pbFound = VARIANT_FALSE;
		
	}

	// Update toolbar
	EnableDisableToolBarButtons();

	return S_OK;
}




STDMETHODIMP CSQLStudioInput::get_FirstVisibleLine(long *pVal)
{
	// Add one to the value because the user should have 
	// a one based index
    *pVal = m_RichEditCtrl.SendMessage(EM_GETFIRSTVISIBLELINE) + 1;

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::put_FirstVisibleLine(long newVal)
{
	// Check if the line is already the first line
	LONG nLineFirst = m_RichEditCtrl.SendMessage(EM_GETFIRSTVISIBLELINE) + 1;

	if(newVal == nLineFirst) return S_OK;

	// Check if the parameter value is too small
	if(newVal < 1) newVal = 1;

	// initialize range
	ITextRangePtr pITextRange;
	InitITextRange(pITextRange, newVal, false);

	pITextRange->ScrollIntoView(tomStart);
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::get_LocalContextMenuVisible(VARIANT_BOOL *pVal)
{
	*pVal = m_bLocalContextMenuVisible;
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::put_LocalContextMenuVisible(VARIANT_BOOL newVal)
{
	m_bLocalContextMenuVisible = newVal;
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::get_NavigateOnDblclick(VARIANT_BOOL *pVal)
{
	*pVal = m_bNavigateOnDblclick;
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::put_NavigateOnDblclick(VARIANT_BOOL newVal)
{
	m_bNavigateOnDblclick = newVal;
	return S_OK;
}



STDMETHODIMP CSQLStudioInput::get_SelIndexEnd(long *pVal)
{
	long nDummy;
	InternalGetSelectionIndexes(&nDummy, pVal);

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::get_SelIndexStart(long *pVal)
{
	long nDummy;
	InternalGetSelectionIndexes(pVal, &nDummy);


	return S_OK;
}

STDMETHODIMP CSQLStudioInput::get_SelPosEndCol(long *pVal)
{	
	long nDummy;
	InternalGetSelectionPos(&nDummy, &nDummy, &nDummy, pVal);

	return S_OK;
}



STDMETHODIMP CSQLStudioInput::get_SelPosEndLine(long *pVal)
{
	long nDummy;
	InternalGetSelectionPos(&nDummy, &nDummy, pVal, &nDummy);

	return S_OK;
}


STDMETHODIMP CSQLStudioInput::get_SelPosStartCol(long *pVal)
{
	long nDummy;
	InternalGetSelectionPos(&nDummy, pVal, &nDummy, &nDummy);

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::get_SelPosStartLine(long *pVal)
{
	long nDummy;
	InternalGetSelectionPos(pVal, &nDummy, &nDummy, &nDummy);
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::get_SpacesOnIndent(long *pVal)
{
	*pVal = m_nSpacesOnIndent;
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::put_SpacesOnIndent(long newVal)
{
	if(newVal >= 0)
		m_nSpacesOnIndent = newVal;

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::get_TextModified(VARIANT_BOOL *pVal)
{
	
	*pVal = VARIANT_TRUE;
	
	// get modify status
	if(!InternalGetModifyStatus())
		*pVal = VARIANT_FALSE;
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::put_TextModified(VARIANT_BOOL newVal)
{
	bool bNewStatus = true;

	if(newVal == VARIANT_FALSE)
		bNewStatus = false;

    if(InternalSetModifyStatus(bNewStatus)) UpdateStatusBar();	

	OnFontChanged(); 

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::GetLineText(long nLine, BSTR *pstrText)
{
	CComBSTR strText;

	if(nLine <= m_RichEditCtrl.SendMessage(EM_GETLINECOUNT))
		strText = InternalGetLineText(nLine);

	*pstrText = strText.Detach();


	return S_OK;
}

STDMETHODIMP CSQLStudioInput::GetSelectionIndexEnd(long *pnIndex)
{
	long nDummy;
	InternalGetSelectionIndexes(&nDummy, pnIndex);

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::GetSelectionIndexes(long *pnIndexFrom, long *pnIndexTo)
{
	InternalGetSelectionIndexes(pnIndexFrom, pnIndexTo);
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::GetSelectionIndexStart(long *pnIndex)
{
	long nDummy;
	InternalGetSelectionIndexes(pnIndex, &nDummy);

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::GetSelectionPos(long *pnLineFrom, long *pnColFrom, long *pnLineTo, long *pnColTo)
{	
	InternalGetSelectionPos(pnLineFrom, pnColFrom, pnLineTo, pnColTo);

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::GetSelectionPosEndCol(long *pnCol)
{
	long nDummy;
	InternalGetSelectionPos(&nDummy, &nDummy, &nDummy, pnCol);

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::GetSelectionPosEndLine(long *pnLine)
{
	long nDummy;
	InternalGetSelectionPos(&nDummy, &nDummy, pnLine, &nDummy);

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::GetSelectionPosStartCol(long *pnCol)
{
	long nDummy;
	InternalGetSelectionPos(&nDummy, pnCol, &nDummy, &nDummy);

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::GetSelectionPosStartLine(long *pnLine)
{
	long nDummy;
	InternalGetSelectionPos(pnLine, &nDummy, &nDummy, &nDummy);

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::HighlightLines(long nLineFrom, long nLineTo, VARIANT_BOOL bHighlight)
{
	// Highlighting only performend when there is no automatic highlighting of comments
	if 	(m_bHighlightKeyAndCom == VARIANT_FALSE)
	{		
		// initialize range
		ITextRangePtr pITextRange;
		InitITextRange(pITextRange, nLineFrom);
		
		if(nLineFrom != nLineTo)
			pITextRange->MoveEnd(tomLine, nLineTo-nLineFrom, NULL);

		InternalHighlight(pITextRange, bHighlight);
	}

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::HighlightSelection(VARIANT_BOOL bHighlight)
{
	if 	(m_bHighlightKeyAndCom == VARIANT_FALSE){

		InternalHighlight(static_cast<ITextRangePtr>(m_pITextSelection), bHighlight);
	}
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::IndentLines(long nLineFrom, long nLineTo)
{
	// Parameter checking
	CheckLineFromTo(nLineFrom, nLineTo);

	InternalIndent(nLineFrom, 0, nLineTo, 0);

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::IndentSelection()
{
	long nLineFrom, nColFrom, nLineTo, nColTo;
	InternalGetSelectionPos(&nLineFrom, &nColFrom, &nLineTo, &nColTo);	
	InternalIndent(nLineFrom, nColFrom, nLineTo, nColTo);	

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::MakeSelectionVisible()
{
	m_RichEditCtrl.SendMessage(EM_SCROLLCARET);

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::get_SelectedText(BSTR * pVal)
{
	CComBSTR bstrText;
	InternalGetText(bstrText, SFF_SELECTION);

	*pVal = bstrText.Detach();

	return S_OK;
}


STDMETHODIMP CSQLStudioInput::put_SelectedText(BSTR newVal)
{
	
	InternalSetText(newVal, -1, SFF_SELECTION);
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::ReplaceAll(BSTR strTextFind, BSTR strTextReplace, VARIANT_BOOL bCaseSensitive, VARIANT_BOOL bWholeWord, LONG *pnReplaceCount)
{
	// initialize global member data for search and replace
	m_oFINDREPLACE.Flags = 0;
	m_bFindReplaceInSelection = false;

	if(bCaseSensitive != VARIANT_FALSE) m_oFINDREPLACE.Flags |= FR_MATCHCASE;

	if(bWholeWord != VARIANT_FALSE) m_oFINDREPLACE.Flags |= FR_WHOLEWORD;

	USES_CONVERSION;
	// Check if string parameters are NULL pointers
	if(strTextFind == NULL || 0 == SysStringLen(strTextFind))
		m_bstrFindWhat.Empty();	
	else
		m_bstrFindWhat = strTextFind;
	

	if(strTextReplace == NULL || 0 == SysStringLen(strTextReplace))

		m_bstrReplaceWith.Empty();	
	else
		m_bstrReplaceWith = strTextReplace;
	

	if(!m_bstrFindWhat || !m_bstrReplaceWith){

		ShowFindReplaceDialog();
		*pnReplaceCount = 0;
	}
	else{

		ITextRangePtr pITextRange;
		GetSelectionRangeDuplicate(pITextRange);
		pITextRange->Collapse(tomTrue);
		pITextRange->MoveEnd(tomStory, 1, NULL);
		*pnReplaceCount = InternalReplaceAll(pITextRange, false);
		// if at least one replace took place
		// the method InternalReplaceAll internally updates the find/replace range(m_pITextRangeFindReplace)
		if(*pnReplaceCount > 0){

			// update selection and scroll it into view
			pITextRange->Select();
			pITextRange->ScrollIntoView(tomStart);
		}
	}

	// Update toolbar
	EnableDisableToolBarButtons();

	return S_OK;
}


STDMETHODIMP CSQLStudioInput::SetContextMenu(LPDISPATCH pMenu)
{
	// TODO: Add your implementation code here

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::SetSelectionIndexes(long nIndexFrom, long nIndexTo)
{
	// Parameter checking in InternalSetSelectionPos
	InternalSetSelectionIndexes(nIndexFrom, nIndexTo);

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::SetSelectionPos(long nLineFrom, long nColFrom, long nLineTo, long nColTo)
{
	InternalSetSelectionPos(nLineFrom, nColFrom, nLineTo, nColTo);
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::SetSelectionPosInLine(long nLine, long nCol)
{
	// Parameter checking in InternalSetSelectionPos
	InternalSetSelectionPos(nLine, nCol, nLine, nCol);

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::UnCommentLines(long nLineFrom, long nLineTo)
{	
	// Parameter checking
	CheckLineFromTo(nLineFrom, nLineTo);
	InternalUnComment(nLineFrom, nLineTo);	

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::UnCommentSelection()
{
	long nLineFrom, nLineTo;
	InternalGetSelectedLines(&nLineFrom, &nLineTo);

	InternalUnComment(nLineFrom, nLineTo);	

	return S_OK;
}


STDMETHODIMP CSQLStudioInput::UnIndentLines(long nLineFrom, long nLineTo)
{
	// Parameter checking
	CheckLineFromTo(nLineFrom, nLineTo);

	InternalUnIndent(nLineFrom, nLineTo);
	
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::UnIndentSelection()
{
	long nLineFrom, nLineTo;
	InternalGetSelectedLines(&nLineFrom, &nLineTo);

	InternalUnIndent(nLineFrom, nLineTo);
	
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::get_ColorTextEditable(OLE_COLOR *pVal)
{

	*pVal =(OLE_COLOR)m_crTextEditable;
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::put_ColorTextEditable(OLE_COLOR newVal)
{

	m_crTextEditable  =(long)newVal;
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::get_ColorTextHighlight(OLE_COLOR  *pVal)
{
	*pVal =(OLE_COLOR)m_crTextHighlight;
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::put_ColorTextHighlight(OLE_COLOR  newVal)
{
	m_crTextHighlight =(long)newVal;

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::SetClientSite(IOleClientSite *pClientSite)
{	
	// special handling to warn users that they are doing something that 
	//   isn't allowed and may produce wrong behavior
	if(!(pClientSite == NULL || m_spClientSite == NULL))
		::MessageBox(NULL, _T("Error while setting client site!\n No warranty for further processing!!"), NULL, MB_OK);


  if(!m_pFont && pClientSite)
  {
	  FONTDESC fd = _fontdesc;

	  CComPtr<IFont> pAF;
	  CComPtr<IFont> pClone;
	  if(SUCCEEDED(GetAmbientFont(&pAF)))
	  {
		 //clone the font
		 if(SUCCEEDED(pAF->Clone(&pClone)))
			 pClone->QueryInterface(IID_IFontDisp,(void**)&m_pFont);
	  }
	  else{

		OleCreateFontIndirect(&fd,IID_IFontDisp,(void**)&m_pFont);
	  }
  }
  return IOleObjectImpl<CSQLStudioInput>::SetClientSite(pClientSite);

}



STDMETHODIMP CSQLStudioInput::GetPoint(long *px, long *py)
{
	
	InternalGetPoint(px,py);

	RECT Rect;
	m_RichEditCtrl.GetWindowRect(&Rect);

	*px = *px -Rect.left;
	*py = *py - Rect.top;
	

	return S_OK;
}


STDMETHODIMP CSQLStudioInput::get_TextAreaHeight(long *pVal)
{
	
	long lHeight = m_sizeNatural.cy;//m_rcPos.bottom - m_rcPos.top;

	if(m_bToolBarVisible != VARIANT_FALSE){

		RECT rect;
		m_ToolBar.GetWindowRect(&rect);
		lHeight -=(rect.bottom-rect.top);
	}

	if(m_bStatusBarVisible != VARIANT_FALSE){

		RECT rect;
		m_StatusBar.GetWindowRect(&rect);
		lHeight -=(rect.bottom-rect.top);
	}

	if(lHeight < 0) {

		lHeight = 0;
	}

	*pVal = lHeight;

	return S_OK;
}

// Notice the calls to Lock and Unlock.
// These methods are provided by CcomObjectRootEx and provide basic thread 
// synchronization(basic in the sense that we are using them here.
// Support for the various threading models in very comprehensive in ATL
// and in the Win32 API).
// The Threading Model used in our project determines if actual locking
// and unlocking(using a critical section) takes place. You can change the 
// behavior of these methods by un/defining pre-processor symbols that are 
// approprate for your server's threading needs. Our project is compatible with
// the client's threading model, therefore, 
// I am calling the Lock and Unlock methods to ensure that the object's state 
// remains consistent.
STDMETHODIMP CSQLStudioInput::get_ChangeCaseMode(KeyWordChangeCase *pVal)
{
	
	*pVal = m_KeyWordChangeCase;
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::put_ChangeCaseMode(KeyWordChangeCase newVal)
{

	m_KeyWordChangeCase = newVal;

	ITextRangePtr	pITextRange;
	InitITextRange(pITextRange, 1);
	
	return S_OK;
}


STDMETHODIMP CSQLStudioInput::get_OLEDropMode(OLEDropMode *pVal)
{	
	*pVal = m_OLEDropMode;
	return S_OK;
}

STDMETHODIMP CSQLStudioInput::put_OLEDropMode(OLEDropMode newVal)
{
	
	m_OLEDropMode = newVal;
	return S_OK;
}



STDMETHODIMP CSQLStudioInput::GetDisplayString(DISPID dispid, BSTR *pBstr)
{

	
	USES_CONVERSION;			
	
	switch(dispid) {

		case DISPID_CHANGECASE:	

			// return the string representation of KeyWordChangeCase
			*pBstr =(unsigned short *)rszKeyWordChangeCase[m_KeyWordChangeCase];

			return *pBstr ? S_OK : E_OUTOFMEMORY;

		// Make Visual Basic apply default formatting for these properties
		// Otherwise it displays color values in decimal
		// and doesn't draw the color sample correctly
		
		case DISPID_OLEDROPMODE:

			*pBstr =(unsigned short *)rszOLEDropMode[m_OLEDropMode];

			return *pBstr ? S_OK : E_OUTOFMEMORY;

		case DISPID_CTHIGHLIGHT:
		case DISPID_CTEDITABLE:
		case DISPID_CASESENSITIVE:
		case DISPID_READONLY:
		case DISPID_STATUSBARVISIBLE:
		case DISPID_TOOLBARVISIBLE:
		case DISPID_HIGHLIGHTKEYANDCOM:
		case DISPID_BWORDWRAP:
		case DISPID_CONVERTWORDWRAPTOLINEBREAK:
		case DISPID_AUTOINDENT:
		case DISPID_LOCALCONTEXTMENUVISIBLE:
		case DISPID_NAVIGATEONDBLCLICK:
		case DISPID_SPACESONINDENT:		
		case DISPID_TEXTMODIFIED:
		case DISPID_TEXT:
		
				return S_FALSE;         // This is an undocumented return value that works...


	}
	
	return IPerPropertyBrowsingImpl<CSQLStudioInput>::GetDisplayString(dispid, pBstr);
	

	return S_OK;
}

STDMETHODIMP CSQLStudioInput::GetPredefinedStrings(DISPID dispID, CALPOLESTR *pCaStringsOut, CADWORD *pCaCookiesOut)
{
	
	if(NULL == pCaStringsOut || NULL == pCaCookiesOut) return E_POINTER;

	HRESULT hr = S_OK;

	switch(dispID)
	{
		case DISPID_CHANGECASE : 
		
			hr = SetStrings(DIM(rszKeyWordChangeCase), rszKeyWordChangeCase, pCaStringsOut);
			if(FAILED(hr)) return hr;
			return SetCookies(DIM(rdwKeyWordChangeCase), rdwKeyWordChangeCase, pCaCookiesOut);

			return S_OK;

		
		case DISPID_OLEDROPMODE:

			hr = SetStrings(DIM(rszOLEDropMode), rszOLEDropMode, pCaStringsOut);
			if(FAILED(hr)) return hr;
			return SetCookies(DIM(rdwOLEDropMode), rdwOLEDropMode, pCaCookiesOut);

			return S_OK;

	}

	return IPerPropertyBrowsingImpl<CSQLStudioInput>::GetPredefinedStrings(dispID, pCaStringsOut, pCaCookiesOut);


	return S_OK;
}

STDMETHODIMP CSQLStudioInput::GetPredefinedValue(DISPID dispID, DWORD dwCookie, VARIANT *pVarOut)
{
	
	if(NULL == pVarOut) return E_POINTER;

    switch(dispID) {

		case DISPID_CHANGECASE:
			
			if(dwCookie >= 0 && dwCookie < 6){

				VariantClear(pVarOut);
				V_VT(pVarOut) = VT_I2;
				V_I2(pVarOut) =(short)dwCookie;
				return S_OK;
			}
		
		case DISPID_OLEDROPMODE:

			if(dwCookie >= 0 && dwCookie < 3){

				VariantClear(pVarOut);
				V_VT(pVarOut) = VT_I2;
				V_I2(pVarOut) =(short)dwCookie;
			}
			return S_OK;
	}
	

	return IPerPropertyBrowsingImpl<CSQLStudioInput>::GetPredefinedValue(dispID, dwCookie, pVarOut);		
}


//Overwrite MapPropertyToPage to ATL Internals Page 535
STDMETHODIMP CSQLStudioInput::MapPropertyToPage(DISPID dispid, CLSID *pClsid)
{
  
  // Look up the property page CLSID in the PROP MAP
  HRESULT hr = IPerPropertyBrowsingImpl<CSQLStudioInput>::MapPropertyToPage(dispid, pClsid);

  // When there is no property page for a listed property, fail the call
  if(SUCCEEDED(hr) && CLSID_NULL == *pClsid) hr = E_INVALIDARG;

  return hr;
}

/////////////////////////////////////////////////////////////////////////////
// ICategorizeProperties interface implementation

/***********************************************************************************************/
/* MapPropertyToCategory */
/***********************************************************************************************/
STDMETHODIMP CSQLStudioInput::MapPropertyToCategory(/*[in]*/ DISPID dispid, /*[out]*/ PROPCAT* ppropcat)
{
    if(NULL == ppropcat) return E_POINTER;

    switch(dispid) {
    case DISPID_CTHIGHLIGHT:
    case DISPID_CTEDITABLE:
        *ppropcat = PROPCAT_Appearance;
        return S_OK;

   
    default:
        return E_FAIL;
    }
}

/***********************************************************************************************/
/* GetCategoryName                                  
/***********************************************************************************************/
STDMETHODIMP CSQLStudioInput::GetCategoryName(/*[in]*/ PROPCAT propcat, /*[in]*/ LCID lcid, /*[out]*/ BSTR* pbstrName)
{
    return E_FAIL;
}



// These warnings are issued when using warning level 4.
// They are produced by template classes used in this file(STL and ATL).
// Template functions are placed invisible at the end of each file
// by the compiler. Because we are not interested in these warnings
// which are not produced by the source itself, these warnings are disabled.
#pragma warning(disable:4018) // signed/unsigned mismatch
#pragma warning(disable:4100) // unreferenced formal parameter
#pragma warning(disable:4245) // conversion from 'const int' to 'const unsigned int', signed/unsigned mismatch
















